@@ -19,10 +19,10 @@ import (
 	"github.com/juju/loggo"
 	"github.com/juju/names"
 	"github.com/juju/utils"
+	"github.com/juju/utils/shell"
 
 	"github.com/juju/juju/api"
 	"github.com/juju/juju/apiserver/params"
-	"github.com/juju/juju/cloudinit"
 	"github.com/juju/juju/juju/paths"
 	"github.com/juju/juju/mongo"
 	"github.com/juju/juju/network"
@@ -32,20 +32,27 @@ import (
 
 var logger = loggo.GetLogger("juju.agent")
 
-// logDir returns a filesystem path to the location where juju
-// may create a folder containing its logs
-var logDir = paths.MustSucceed(paths.LogDir(version.Current.Series))
+// These are base values used for the corresponding defaults.
+var (
+	logDir  = paths.MustSucceed(paths.LogDir(version.Current.Series))
+	dataDir = paths.MustSucceed(paths.DataDir(version.Current.Series))
+	confDir = paths.MustSucceed(paths.ConfDir(version.Current.Series))
+)
 
-// dataDir returns the default data directory for this running system
-var dataDir = paths.MustSucceed(paths.DataDir(version.Current.Series))
+var (
+	// DefaultLogDir defines the default log directory for juju agents.
+	// It's defined as a variable so it could be overridden in tests.
+	DefaultLogDir = path.Join(logDir, "juju")
 
-// DefaultLogDir defines the default log directory for juju agents.
-// It's defined as a variable so it could be overridden in tests.
-var DefaultLogDir = path.Join(logDir, "juju")
+	// DefaultDataDir defines the default data directory for juju agents.
+	// It's defined as a variable so it could be overridden in tests.
+	DefaultDataDir = dataDir
 
-// DefaultDataDir defines the default data directory for juju agents.
-// It's defined as a variable so it could be overridden in tests.
-var DefaultDataDir = dataDir
+	// DefaultConfDir defines the default config file directory for
+	// Juju agents.
+	// It's defined as a variable so it could be overridden in tests.
+	DefaultConfDir = confDir
+)
 
 // SystemIdentity is the name of the file where the environment SSH key is kept.
 const SystemIdentity = "system-identity"
@@ -110,7 +117,7 @@ type Config interface {
 	// WriteCommands returns shell commands to write the agent configuration.
 	// It returns an error if the configuration does not have all the right
 	// elements.
-	WriteCommands(series string) ([]string, error)
+	WriteCommands(renderer shell.Renderer) ([]string, error)
 
 	// StateServingInfo returns the details needed to run
 	// a state server and reports whether those details
@@ -139,6 +146,10 @@ type Config interface {
 	// PreferIPv6 returns whether to prefer using IPv6 addresses (if
 	// available) when connecting to the state or API server.
 	PreferIPv6() bool
+
+	// Environment returns the tag for the environment that the agent belongs
+	// to.
+	Environment() names.EnvironTag
 }
 
 type ConfigSetterOnly interface {
@@ -187,6 +198,13 @@ type ConfigSetterOnly interface {
 	SetStateServingInfo(info params.StateServingInfo)
 }
 
+// LogFileName returns the filename for the Agent's log file.
+func LogFilename(c Config) string {
+	return filepath.Join(c.LogDir(), c.Tag().String()+".log")
+}
+
+type ConfigMutator func(ConfigSetter) error
+
 type ConfigWriter interface {
 	// Write writes the agent configuration.
 	Write() error
@@ -212,6 +230,7 @@ type MigrateParams struct {
 	Jobs         []multiwatcher.MachineJob
 	DeleteValues []string
 	Values       map[string]string
+	Environment  names.EnvironTag
 }
 
 // Ensure that the configInternal struct implements the Config interface.
@@ -237,6 +256,7 @@ type configInternal struct {
 	logDir            string
 	tag               names.Tag
 	nonce             string
+	environment       names.EnvironTag
 	jobs              []multiwatcher.MachineJob
 	upgradedToVersion version.Number
 	caCert            string
@@ -256,6 +276,7 @@ type AgentConfigParams struct {
 	Tag               names.Tag
 	Password          string
 	Nonce             string
+	Environment       names.EnvironTag
 	StateAddresses    []string
 	APIAddresses      []string
 	CACert            string
@@ -288,6 +309,11 @@ func NewAgentConfig(configParams AgentConfigParams) (ConfigSetterWriter, error)
 	if configParams.Password == "" {
 		return nil, errors.Trace(requiredError("password"))
 	}
+	if uuid := configParams.Environment.Id(); uuid == "" {
+		return nil, errors.Trace(requiredError("environment"))
+	} else if !names.IsValidEnvironment(uuid) {
+		return nil, errors.Errorf("%q is not a valid environment uuid", uuid)
+	}
 	if len(configParams.CACert) == 0 {
 		return nil, errors.Trace(requiredError("CA certificate"))
 	}
@@ -300,6 +326,7 @@ func NewAgentConfig(configParams AgentConfigParams) (ConfigSetterWriter, error)
 		upgradedToVersion: configParams.UpgradedToVersion,
 		tag:               configParams.Tag,
 		nonce:             configParams.Nonce,
+		environment:       configParams.Environment,
 		caCert:            configParams.CACert,
 		oldPassword:       configParams.Password,
 		values:            configParams.Values,
@@ -454,6 +481,9 @@ func (config *configInternal) Migrate(newParams MigrateParams) error {
 		}
 		config.values[key] = value
 	}
+	if newParams.Environment.Id() != "" {
+		config.environment = newParams.Environment
+	}
 	if err := config.check(); err != nil {
 		return fmt.Errorf("migrated agent config is invalid: %v", err)
 	}
@@ -582,6 +612,10 @@ func (c *configInternal) Tag() names.Tag {
 	return c.tag
 }
 
+func (c *configInternal) Environment() names.EnvironTag {
+	return c.environment
+}
+
 func (c *configInternal) Dir() string {
 	return Dir(c.dataDir, c.tag)
 }
@@ -628,17 +662,15 @@ func (c *configInternal) fileContents() ([]byte, error) {
 	return buf.Bytes(), nil
 }
 
-func (c *configInternal) WriteCommands(series string) ([]string, error) {
-	renderer, err := cloudinit.NewRenderer(series)
-	if err != nil {
-		return nil, err
-	}
+func (c *configInternal) WriteCommands(renderer shell.Renderer) ([]string, error) {
 	data, err := c.fileContents()
 	if err != nil {
-		return nil, err
+		return nil, errors.Trace(err)
 	}
-	commands := renderer.Mkdir(c.Dir())
-	commands = append(commands, renderer.WriteFile(c.File(agentConfigFilename), string(data), 0600)...)
+	commands := renderer.MkdirAll(c.Dir())
+	filename := c.File(agentConfigFilename)
+	commands = append(commands, renderer.WriteFile(filename, data)...)
+	commands = append(commands, renderer.Chmod(filename, 0600)...)
 	return commands, nil
 }
 
@@ -663,11 +695,12 @@ func (c *configInternal) APIInfo() *api.Info {
 		}
 	}
 	return &api.Info{
-		Addrs:    addrs,
-		Password: c.apiDetails.password,
-		CACert:   c.caCert,
-		Tag:      c.tag,
-		Nonce:    c.nonce,
+		Addrs:      addrs,
+		Password:   c.apiDetails.password,
+		CACert:     c.caCert,
+		Tag:        c.tag,
+		Nonce:      c.nonce,
+		EnvironTag: c.environment,
 	}
 }
 
