@@ -7,17 +7,15 @@ import (
 	"fmt"
 	"os"
 
-	"github.com/juju/juju/state/api/params"
+	"github.com/juju/juju/state/multiwatcher"
 )
 
 func Password(config Config) string {
 	c := config.(*configInternal)
 	if c.stateDetails == nil {
 		return c.apiDetails.password
-	} else {
-		return c.stateDetails.password
 	}
-	return ""
+	return c.stateDetails.password
 }
 
 func PatchConfig(config Config, fieldName string, value interface{}) error {
@@ -28,7 +26,7 @@ func PatchConfig(config Config, fieldName string, value interface{}) error {
 	case "LogDir":
 		conf.logDir = value.(string)
 	case "Jobs":
-		conf.jobs = value.([]params.MachineJob)[:]
+		conf.jobs = value.([]multiwatcher.MachineJob)[:]
 	case "DeleteValues":
 		for _, key := range value.([]string) {
 			delete(conf.values, key)
@@ -52,3 +50,8 @@ func ConfigFileExists(config Config) bool {
 	_, err := os.Lstat(conf.configFilePath)
 	return err == nil
 }
+
+var (
+	MachineJobFromParams = machineJobFromParams
+	IsLocalEnv           = &isLocalEnv
+)
