@@ -7,95 +7,37 @@ import (
 	"bytes"
 	"encoding/json"
 	"fmt"
-	"io"
 	"io/ioutil"
 	"net/http"
 	"net/url"
 	"os"
 	"path/filepath"
+	"runtime"
 
 	jc "github.com/juju/testing/checkers"
 	"github.com/juju/utils"
 	gc "gopkg.in/check.v1"
-	"gopkg.in/juju/charm.v4"
+	"gopkg.in/juju/charm.v5"
 
 	apihttp "github.com/juju/juju/apiserver/http"
 	"github.com/juju/juju/apiserver/params"
-	jujutesting "github.com/juju/juju/juju/testing"
 	"github.com/juju/juju/state"
 	"github.com/juju/juju/state/storage"
 	"github.com/juju/juju/testcharms"
-	"github.com/juju/juju/testing/factory"
 )
 
-type authHttpSuite struct {
-	jujutesting.JujuConnSuite
-	userTag            string
-	password           string
-	archiveContentType string
-}
-
-func (s *authHttpSuite) SetUpTest(c *gc.C) {
-	s.JujuConnSuite.SetUpTest(c)
-	s.password = "password"
-	user := s.Factory.MakeUser(c, &factory.UserParams{Password: s.password})
-	s.userTag = user.Tag().String()
-}
-
-func (s *authHttpSuite) sendRequest(c *gc.C, tag, password, method, uri, contentType string, body io.Reader) (*http.Response, error) {
-	req, err := http.NewRequest(method, uri, body)
-	c.Assert(err, jc.ErrorIsNil)
-	if tag != "" && password != "" {
-		req.SetBasicAuth(tag, password)
-	}
-	if contentType != "" {
-		req.Header.Set("Content-Type", contentType)
-	}
-	return utils.GetNonValidatingHTTPClient().Do(req)
-}
-
-func (s *authHttpSuite) baseURL(c *gc.C) *url.URL {
-	info := s.APIInfo(c)
-	return &url.URL{
-		Scheme: "https",
-		Host:   info.Addrs[0],
-		Path:   "",
-	}
-}
-
-func (s *authHttpSuite) authRequest(c *gc.C, method, uri, contentType string, body io.Reader) (*http.Response, error) {
-	return s.sendRequest(c, s.userTag, s.password, method, uri, contentType, body)
-}
-
-func (s *authHttpSuite) uploadRequest(c *gc.C, uri string, asZip bool, path string) (*http.Response, error) {
-	contentType := apihttp.CTypeRaw
-	if asZip {
-		contentType = s.archiveContentType
-	}
-
-	if path == "" {
-		return s.authRequest(c, "POST", uri, contentType, nil)
-	}
-
-	file, err := os.Open(path)
-	c.Assert(err, jc.ErrorIsNil)
-	defer file.Close()
-	return s.authRequest(c, "POST", uri, contentType, file)
-}
-
-func (s *authHttpSuite) assertErrorResponse(c *gc.C, resp *http.Response, expCode int, expError string) {
-	body := assertResponse(c, resp, expCode, apihttp.CTypeJSON)
-	c.Check(jsonResponse(c, body).Error, gc.Matches, expError)
-}
-
 type charmsSuite struct {
-	authHttpSuite
+	userAuthHttpSuite
 }
 
 var _ = gc.Suite(&charmsSuite{})
 
 func (s *charmsSuite) SetUpSuite(c *gc.C) {
-	s.authHttpSuite.SetUpSuite(c)
+	// TODO(bogdanteleaga): Fix this on windows
+	if runtime.GOOS == "windows" {
+		c.Skip("bug 1403084: Skipping this on windows for now")
+	}
+	s.userAuthHttpSuite.SetUpSuite(c)
 	s.archiveContentType = "application/zip"
 }
 
@@ -250,10 +192,20 @@ func (s *charmsSuite) TestUploadAllowsTopLevelPath(c *gc.C) {
 func (s *charmsSuite) TestUploadAllowsEnvUUIDPath(c *gc.C) {
 	// Check that we can upload charms to https://host:port/ENVUUID/charms
 	ch := testcharms.Repo.CharmArchive(c.MkDir(), "dummy")
-	environ, err := s.State.Environment()
-	c.Assert(err, jc.ErrorIsNil)
 	url := s.charmsURL(c, "series=quantal")
-	url.Path = fmt.Sprintf("/environment/%s/charms", environ.UUID())
+	url.Path = fmt.Sprintf("/environment/%s/charms", s.envUUID)
+	resp, err := s.uploadRequest(c, url.String(), true, ch.Path)
+	c.Assert(err, jc.ErrorIsNil)
+	expectedURL := charm.MustParseURL("local:quantal/dummy-1")
+	s.assertUploadResponse(c, resp, expectedURL.String())
+}
+
+func (s *charmsSuite) TestUploadAllowsOtherEnvUUIDPath(c *gc.C) {
+	envState := s.setupOtherEnvironment(c)
+	// Check that we can upload charms to https://host:port/ENVUUID/charms
+	ch := testcharms.Repo.CharmArchive(c.MkDir(), "dummy")
+	url := s.charmsURL(c, "series=quantal")
+	url.Path = fmt.Sprintf("/environment/%s/charms", envState.EnvironUUID())
 	resp, err := s.uploadRequest(c, url.String(), true, ch.Path)
 	c.Assert(err, jc.ErrorIsNil)
 	expectedURL := charm.MustParseURL("local:quantal/dummy-1")
@@ -428,12 +380,12 @@ func (s *charmsSuite) TestGetStarReturnsArchiveBytes(c *gc.C) {
 }
 
 func (s *charmsSuite) TestGetAllowsTopLevelPath(c *gc.C) {
+	// Backwards compatibility check, that we can GET from charms at
+	// https://host:port/charms
 	ch := testcharms.Repo.CharmArchive(c.MkDir(), "dummy")
 	_, err := s.uploadRequest(
 		c, s.charmsURI(c, "?series=quantal"), true, ch.Path)
 	c.Assert(err, jc.ErrorIsNil)
-	// Backwards compatibility check, that we can GET from charms at
-	// https://host:port/charms
 	url := s.charmsURL(c, "url=local:quantal/dummy-1&file=revision")
 	url.Path = "/charms"
 	resp, err := s.authRequest(c, "GET", url.String(), "", nil)
@@ -446,18 +398,28 @@ func (s *charmsSuite) TestGetAllowsEnvUUIDPath(c *gc.C) {
 	_, err := s.uploadRequest(
 		c, s.charmsURI(c, "?series=quantal"), true, ch.Path)
 	c.Assert(err, jc.ErrorIsNil)
-	// Check that we can GET from charms at https://host:port/ENVUUID/charms
-	environ, err := s.State.Environment()
+	url := s.charmsURL(c, "url=local:quantal/dummy-1&file=revision")
+	url.Path = fmt.Sprintf("/environment/%s/charms", s.envUUID)
+	resp, err := s.authRequest(c, "GET", url.String(), "", nil)
+	c.Assert(err, jc.ErrorIsNil)
+	s.assertGetFileResponse(c, resp, "1", "text/plain; charset=utf-8")
+}
+
+func (s *charmsSuite) TestGetAllowsOtherEnvironment(c *gc.C) {
+	envState := s.setupOtherEnvironment(c)
+
+	ch := testcharms.Repo.CharmArchive(c.MkDir(), "dummy")
+	_, err := s.uploadRequest(
+		c, s.charmsURI(c, "?series=quantal"), true, ch.Path)
 	c.Assert(err, jc.ErrorIsNil)
 	url := s.charmsURL(c, "url=local:quantal/dummy-1&file=revision")
-	url.Path = fmt.Sprintf("/environment/%s/charms", environ.UUID())
+	url.Path = fmt.Sprintf("/environment/%s/charms", envState.EnvironUUID())
 	resp, err := s.authRequest(c, "GET", url.String(), "", nil)
 	c.Assert(err, jc.ErrorIsNil)
 	s.assertGetFileResponse(c, resp, "1", "text/plain; charset=utf-8")
 }
 
 func (s *charmsSuite) TestGetRejectsWrongEnvUUIDPath(c *gc.C) {
-	// Check that we cannot upload charms to https://host:port/BADENVUUID/charms
 	url := s.charmsURL(c, "url=local:quantal/dummy-1&file=revision")
 	url.Path = "/environment/dead-beef-123456/charms"
 	resp, err := s.authRequest(c, "GET", url.String(), "", nil)
@@ -513,7 +475,11 @@ func (s *charmsSuite) TestGetUsesCache(c *gc.C) {
 
 func (s *charmsSuite) charmsURL(c *gc.C, query string) *url.URL {
 	uri := s.baseURL(c)
-	uri.Path += "/charms"
+	if s.envUUID == "" {
+		uri.Path = "/charms"
+	} else {
+		uri.Path = fmt.Sprintf("/environment/%s/charms", s.envUUID)
+	}
 	uri.RawQuery = query
 	return uri
 }
