@@ -10,9 +10,7 @@ import (
 
 	"github.com/juju/juju/constraints"
 	"github.com/juju/juju/instance"
-	"github.com/juju/juju/network"
 	"github.com/juju/juju/state/multiwatcher"
-	"github.com/juju/juju/storage"
 	"github.com/juju/juju/tools"
 	"github.com/juju/juju/version"
 )
@@ -59,64 +57,6 @@ type StringsResult struct {
 	Result []string
 }
 
-// PortsResults holds the bulk operation result of an API call
-// that returns a slice of network.Port.
-type PortsResults struct {
-	Results []PortsResult
-}
-
-// PortsResult holds the result of an API call that returns a slice
-// of network.Port or an error.
-type PortsResult struct {
-	Error *Error
-	// TODO(dimitern): Add explicit JSON serialization tags and use
-	// []string instead in order to break the dependency on the
-	// network package, as this potentially introduces hard to catch
-	// and debug wire-format changes in the protocol when the type
-	// changes!
-	Ports []network.Port
-}
-
-// MachinePorts holds a machine and network tags. It's used when
-// referring to opened ports on the machine for a network.
-type MachinePorts struct {
-	MachineTag string
-	NetworkTag string
-}
-
-// MachinePortRange holds a single port range open on a machine for
-// the given unit and relation tags.
-type MachinePortRange struct {
-	UnitTag     string
-	RelationTag string
-	// TODO(dimitern): Add explicit JSON serialization tags and use
-	// string instead in order to break the dependency on the network
-	// package, as this potentially introduces hard to catch and debug
-	// wire-format changes in the protocol when the type changes!
-	PortRange network.PortRange
-}
-
-// MachinePortsParams holds the arguments for making a
-// FirewallerAPIV1.GetMachinePorts() API call.
-type MachinePortsParams struct {
-	Params []MachinePorts
-}
-
-// MachinePortsResult holds a single result of the
-// FirewallerAPIV1.GetMachinePorts() and UniterAPI.AllMachinePorts()
-// API calls.
-type MachinePortsResult struct {
-	Error *Error
-	Ports []MachinePortRange
-}
-
-// MachinePortsResults holds all the results of the
-// FirewallerAPIV1.GetMachinePorts() and UniterAPI.AllMachinePorts()
-// API calls.
-type MachinePortsResults struct {
-	Results []MachinePortsResult
-}
-
 // StringsResults holds the bulk operation result of an API call
 // that returns a slice of strings or an error.
 type StringsResults struct {
@@ -143,6 +83,44 @@ type EnvironmentResult struct {
 	UUID  string
 }
 
+// EnvironmentSkeletonConfigArgs wraps the args for environmentmanager.SkeletonConfig.
+type EnvironmentSkeletonConfigArgs struct {
+	Provider string
+	Region   string
+}
+
+// EnvironmentCreateArgs holds the arguments that are necessary to create
+// and environment.
+type EnvironmentCreateArgs struct {
+	// OwnerTag represents the user that will own the new environment.
+	// The OwnerTag must be a valid user tag.  If the user tag represents
+	// a local user, that user must exist.
+	OwnerTag string
+
+	// Account holds the provider specific account details necessary to
+	// interact with the provider to create, list and destroy machines.
+	Account map[string]interface{}
+
+	// Config defines the environment config, which includes the name of the
+	// environment.  An environment UUID is allocated by the API server during
+	// the creation of the environment.
+	Config map[string]interface{}
+}
+
+// Environment holds the result of an API call returning a name and UUID
+// for an environment and the tag of the server in which it is running.
+type Environment struct {
+	Name       string
+	UUID       string
+	OwnerTag   string
+	ServerUUID string
+}
+
+// EnvironmentList holds information about a list of environments.
+type EnvironmentList struct {
+	Environments []Environment
+}
+
 // ResolvedModeResult holds a resolved mode or an error.
 type ResolvedModeResult struct {
 	Error *Error
@@ -181,19 +159,19 @@ type BoolResults struct {
 	Results []BoolResult
 }
 
-// RelationSettings holds relation settings names and values.
-type RelationSettings map[string]string
+// Settings holds relation settings names and values.
+type Settings map[string]string
 
-// RelationSettingsResult holds a relation settings map or an error.
-type RelationSettingsResult struct {
+// SettingsResult holds a relation settings map or an error.
+type SettingsResult struct {
 	Error    *Error
-	Settings RelationSettings
+	Settings Settings
 }
 
-// RelationSettingsResults holds the result of an API calls that
+// SettingsResults holds the result of an API calls that
 // returns settings for multiple relations.
-type RelationSettingsResults struct {
-	Results []RelationSettingsResult
+type SettingsResults struct {
+	Results []SettingsResult
 }
 
 // ConfigSettings holds unit, service or cham configuration settings
@@ -254,7 +232,7 @@ type RelationUnitPairs struct {
 type RelationUnitSettings struct {
 	Relation string
 	Unit     string
-	Settings RelationSettings
+	Settings Settings
 }
 
 // RelationUnitsSettings holds the arguments for making a EnterScope
@@ -279,33 +257,6 @@ type RelationResults struct {
 	Results []RelationResult
 }
 
-// EntityPort holds an entity's tag, a protocol and a port.
-type EntityPort struct {
-	Tag      string
-	Protocol string
-	Port     int
-}
-
-// EntitiesPorts holds the parameters for making an OpenPort or
-// ClosePort on some entities.
-type EntitiesPorts struct {
-	Entities []EntityPort
-}
-
-// EntityPortRange holds an entity's tag, a protocol and a port range.
-type EntityPortRange struct {
-	Tag      string
-	Protocol string
-	FromPort int
-	ToPort   int
-}
-
-// EntitiesPortRanges holds the parameters for making an OpenPorts or
-// ClosePorts on some entities.
-type EntitiesPortRanges struct {
-	Entities []EntityPortRange
-}
-
 // EntityCharmURL holds an entity's tag and a charm URL.
 type EntityCharmURL struct {
 	Tag      string
@@ -359,44 +310,6 @@ type SetProvisioned struct {
 	Machines []MachineSetProvisioned
 }
 
-// Network describes a single network available on an instance.
-type Network struct {
-	// Tag is the network's tag.
-	Tag string
-
-	// ProviderId is the provider-specific network id.
-	ProviderId network.Id
-
-	// CIDR of the network, in "123.45.67.89/12" format.
-	CIDR string
-
-	// VLANTag needs to be between 1 and 4094 for VLANs and 0 for
-	// normal networks. It's defined by IEEE 802.1Q standard.
-	VLANTag int
-}
-
-// NetworkInterface describes a single network interface available on
-// an instance.
-type NetworkInterface struct {
-	// MACAddress is the network interface's hardware MAC address
-	// (e.g. "aa:bb:cc:dd:ee:ff").
-	MACAddress string
-
-	// InterfaceName is the OS-specific network device name (e.g.
-	// "eth1", even for for a VLAN eth1.42 virtual interface).
-	InterfaceName string
-
-	// NetworkTag is this interface's network tag.
-	NetworkTag string
-
-	// IsVirtual is true when the interface is a virtual device, as
-	// opposed to a physical device.
-	IsVirtual bool
-
-	// Disabled returns whether the interface is disabled.
-	Disabled bool
-}
-
 // InstanceInfo holds a machine tag, provider-specific instance id, a
 // nonce, a list of networks and interfaces to set up.
 type InstanceInfo struct {
@@ -406,6 +319,10 @@ type InstanceInfo struct {
 	Characteristics *instance.HardwareCharacteristics
 	Networks        []Network
 	Interfaces      []NetworkInterface
+	Volumes         []Volume
+	// VolumeAttachments is a mapping from volume tag to
+	// volume attachment info.
+	VolumeAttachments map[string]VolumeAttachmentInfo
 }
 
 // InstancesInfo holds the parameters for making a SetInstanceInfo
@@ -414,33 +331,6 @@ type InstancesInfo struct {
 	Machines []InstanceInfo
 }
 
-// RequestedNetworkResult holds requested networks or an error.
-type RequestedNetworkResult struct {
-	Error    *Error
-	Networks []string
-}
-
-// RequestedNetworksResults holds multiple requested networks results.
-type RequestedNetworksResults struct {
-	Results []RequestedNetworkResult
-}
-
-// MachineNetworkInfoResult holds network info for a single machine.
-type MachineNetworkInfoResult struct {
-	Error *Error
-	// TODO(dimitern): Add explicit JSON serialization tags and use
-	// []NetworkInfo (locally defined) instead in order to break the
-	// dependency on the network package, as this potentially
-	// introduces hard to catch and debug wire-format changes in the
-	// protocol when the type changes!
-	Info []network.Info
-}
-
-// MachineNetworkInfoResults holds network info for multiple machines.
-type MachineNetworkInfoResults struct {
-	Results []MachineNetworkInfoResult
-}
-
 // EntityStatus holds an entity tag, status and extra info.
 type EntityStatus struct {
 	Tag    string
@@ -454,6 +344,21 @@ type SetStatus struct {
 	Entities []EntityStatus
 }
 
+type HistoryKind string
+
+const (
+	KindCombined HistoryKind = "combined"
+	KindAgent    HistoryKind = "agent"
+	KindWorkload HistoryKind = "workload"
+)
+
+// StatusHistory holds the parameters to filter a status history query.
+type StatusHistory struct {
+	Kind HistoryKind
+	Size int
+	Name string
+}
+
 // StatusResult holds an entity status, extra information, or an
 // error.
 type StatusResult struct {
@@ -463,6 +368,7 @@ type StatusResult struct {
 	Status Status
 	Info   string
 	Data   map[string]interface{}
+	Since  *time.Time
 }
 
 // StatusResults holds multiple status results.
@@ -470,15 +376,16 @@ type StatusResults struct {
 	Results []StatusResult
 }
 
-// MachineAddresses holds an machine tag and addresses.
-type MachineAddresses struct {
-	Tag string
-	// TODO(dimitern): Add explicit JSON serialization tags and use
-	// []string instead in order to break the dependency on the
-	// network package, as this potentially introduces hard to catch
-	// and debug wire-format changes in the protocol when the type
-	// changes!
-	Addresses []network.Address
+// ServiceStatusResult holds results for a service Full Status
+type ServiceStatusResult struct {
+	Service StatusResult
+	Units   map[string]StatusResult
+	Error   *Error
+}
+
+// ServiceStatusResults holds multiple StatusResult.
+type ServiceStatusResults struct {
+	Results []ServiceStatusResult
 }
 
 // SetMachinesAddresses holds the parameters for making a SetMachineAddresses call.
@@ -596,6 +503,20 @@ type RelationUnitsWatchResults struct {
 	Results []RelationUnitsWatchResult
 }
 
+// MachineStorageIdsWatchResult holds a MachineStorageIdsWatcher id,
+// changes and an error (if any).
+type MachineStorageIdsWatchResult struct {
+	MachineStorageIdsWatcherId string
+	Changes                    []MachineStorageId
+	Error                      *Error
+}
+
+// MachineStorageIdsWatchResults holds the results for any API call which ends
+// up returning a list of MachineStorageIdsWatchers.
+type MachineStorageIdsWatchResults struct {
+	Results []MachineStorageIdsWatchResult
+}
+
 // CharmsResponse is the server response to charm upload or GET requests.
 type CharmsResponse struct {
 	Error    string   `json:",omitempty"`
@@ -642,6 +563,8 @@ type ProvisioningInfo struct {
 	Placement   string
 	Networks    []string
 	Jobs        []multiwatcher.MachineJob
+	Volumes     []VolumeParams
+	Tags        map[string]string
 }
 
 // ProvisioningInfoResult holds machine provisioning info or an error.
@@ -673,6 +596,25 @@ type MetricsParams struct {
 	Metrics []MetricsParam
 }
 
+// MetricBatch is a list of metrics with metadata.
+type MetricBatch struct {
+	UUID     string
+	CharmURL string
+	Created  time.Time
+	Metrics  []Metric
+}
+
+// MetricBatchParam contains a single metric batch.
+type MetricBatchParam struct {
+	Tag   string
+	Batch MetricBatch
+}
+
+// MetricBatchParams contains multiple metric batches.
+type MetricBatchParams struct {
+	Batches []MetricBatchParam
+}
+
 // MeterStatusResult holds unit meter status or error.
 type MeterStatusResult struct {
 	Code  string
@@ -684,56 +626,3 @@ type MeterStatusResult struct {
 type MeterStatusResults struct {
 	Results []MeterStatusResult
 }
-
-// MachineBlockDevices holds a machine tag and the block devices present
-// on that machine.
-type MachineBlockDevices struct {
-	Machine      string
-	BlockDevices []storage.BlockDevice
-}
-
-// SetMachineBlockDevices holds the arguments for recording the block
-// devices present on a set of machines.
-type SetMachineBlockDevices struct {
-	MachineBlockDevices []MachineBlockDevices
-}
-
-// BlockDeviceResult holds the result of an API call to retrieve details
-// of a block device.
-type BlockDeviceResult struct {
-	Result storage.BlockDevice `json:"result"`
-	Error  *Error              `json:"error,omitempty"`
-}
-
-// BlockDeviceResults holds the result of an API call to retrieve details
-// of multiple block devices.
-type BlockDeviceResults struct {
-	Results []BlockDeviceResult `json:"results,omitempty"`
-}
-
-// BlockDevicesResult holds the result of an API call to retrieve details
-// of all block devices relating to some entity.
-type BlockDevicesResult struct {
-	Result []storage.BlockDevice `json:"result"`
-	Error  *Error                `json:"error,omitempty"`
-}
-
-// BlockDevicseResults holds the result of an API call to retrieve details
-// of all block devices relating to some entities.
-type BlockDevicesResults struct {
-	Results []BlockDevicesResult `json:"results,omitempty"`
-}
-
-// BlockDeviceFilesystem holds the parameters for recording information about
-// the filesystem corresponding to the specified block device.
-type BlockDeviceFilesystem struct {
-	DiskTag    string             `json:"disktag"`
-	Datastore  string             `json:"datastore"`
-	Filesystem storage.Filesystem `json:"filesystem"`
-}
-
-// SetBlockDeviceFilesystem holds the parameters for recording information about
-// the filesystems corresponding to the specified block devices.
-type SetBlockDeviceFilesystem struct {
-	Filesystems []BlockDeviceFilesystem `json:"filesystems"`
-}
