@@ -20,12 +20,18 @@ import (
 	apiservertesting "github.com/juju/juju/apiserver/testing"
 	"github.com/juju/juju/constraints"
 	"github.com/juju/juju/container"
+	"github.com/juju/juju/environs/tags"
+	"github.com/juju/juju/feature"
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/juju/testing"
 	"github.com/juju/juju/network"
+	"github.com/juju/juju/provider/dummy"
 	"github.com/juju/juju/state"
 	"github.com/juju/juju/state/multiwatcher"
 	statetesting "github.com/juju/juju/state/testing"
+	"github.com/juju/juju/storage/poolmanager"
+	storagedummy "github.com/juju/juju/storage/provider/dummy"
+	"github.com/juju/juju/storage/provider/registry"
 	coretesting "github.com/juju/juju/testing"
 )
 
@@ -51,6 +57,10 @@ func (s *provisionerSuite) SetUpTest(c *gc.C) {
 
 func (s *provisionerSuite) setUpTest(c *gc.C, withStateServer bool) {
 	s.JujuConnSuite.SetUpTest(c)
+	// We're testing with address allocation on by default. There are
+	// separate tests to check the behavior when the flag is not
+	// enabled.
+	s.SetFeatureFlags(feature.AddressAllocation)
 
 	// Reset previous machines (if any) and create 3 machines
 	// for the tests, plus an optional state server machine.
@@ -447,11 +457,11 @@ func (s *withoutStateServerSuite) assertLife(c *gc.C, index int, expectLife stat
 func (s *withoutStateServerSuite) assertStatus(c *gc.C, index int, expectStatus state.Status, expectInfo string,
 	expectData map[string]interface{}) {
 
-	status, info, data, err := s.machines[index].Status()
+	statusInfo, err := s.machines[index].Status()
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(status, gc.Equals, expectStatus)
-	c.Assert(info, gc.Equals, expectInfo)
-	c.Assert(data, gc.DeepEquals, expectData)
+	c.Assert(statusInfo.Status, gc.Equals, expectStatus)
+	c.Assert(statusInfo.Message, gc.Equals, expectInfo)
+	c.Assert(statusInfo.Data, gc.DeepEquals, expectData)
 }
 
 func (s *withoutStateServerSuite) TestWatchContainers(c *gc.C) {
@@ -558,6 +568,15 @@ func (s *withoutStateServerSuite) TestStatus(c *gc.C) {
 	}}
 	result, err := s.provisioner.Status(args)
 	c.Assert(err, jc.ErrorIsNil)
+	// Zero out the updated timestamps so we can easily check the results.
+	for i, statusResult := range result.Results {
+		r := statusResult
+		if r.Status != "" {
+			c.Assert(r.Since, gc.NotNil)
+		}
+		r.Since = nil
+		result.Results[i] = r
+	}
 	c.Assert(result, gc.DeepEquals, params.StatusResults{
 		Results: []params.StatusResult{
 			{Status: params.StatusStarted, Info: "blah", Data: map[string]interface{}{}},
@@ -724,6 +743,14 @@ func (s *withoutStateServerSuite) TestDistributionGroupMachineAgentAuth(c *gc.C)
 }
 
 func (s *withoutStateServerSuite) TestProvisioningInfo(c *gc.C) {
+	registry.RegisterProvider("static", &storagedummy.StorageProvider{IsDynamic: false})
+	defer registry.RegisterProvider("static", nil)
+	registry.RegisterEnvironStorageProviders("dummy", "static")
+
+	pm := poolmanager.New(state.NewStateSettings(s.State))
+	_, err := pm.Create("static-pool", "static", map[string]interface{}{"foo": "bar"})
+	c.Assert(err, jc.ErrorIsNil)
+
 	cons := constraints.MustParse("cpu-cores=123 mem=8G networks=^net3,^net4")
 	template := state.MachineTemplate{
 		Series:            "quantal",
@@ -731,6 +758,10 @@ func (s *withoutStateServerSuite) TestProvisioningInfo(c *gc.C) {
 		Constraints:       cons,
 		Placement:         "valid",
 		RequestedNetworks: []string{"net1", "net2"},
+		Volumes: []state.MachineVolumeParams{
+			{Volume: state.VolumeParams{Size: 1000, Pool: "static-pool"}},
+			{Volume: state.VolumeParams{Size: 2000, Pool: "static-pool"}},
+		},
 	}
 	placementMachine, err := s.State.AddOneMachine(template)
 	c.Assert(err, jc.ErrorIsNil)
@@ -744,12 +775,16 @@ func (s *withoutStateServerSuite) TestProvisioningInfo(c *gc.C) {
 	}}
 	result, err := s.provisioner.ProvisioningInfo(args)
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(result, gc.DeepEquals, params.ProvisioningInfoResults{
+
+	expected := params.ProvisioningInfoResults{
 		Results: []params.ProvisioningInfoResult{
 			{Result: &params.ProvisioningInfo{
 				Series:   "quantal",
 				Networks: []string{},
 				Jobs:     []multiwatcher.MachineJob{multiwatcher.JobHostUnits},
+				Tags: map[string]string{
+					tags.JujuEnv: coretesting.EnvironmentTag.Id(),
+				},
 			}},
 			{Result: &params.ProvisioningInfo{
 				Series:      "quantal",
@@ -757,11 +792,104 @@ func (s *withoutStateServerSuite) TestProvisioningInfo(c *gc.C) {
 				Placement:   template.Placement,
 				Networks:    template.RequestedNetworks,
 				Jobs:        []multiwatcher.MachineJob{multiwatcher.JobHostUnits},
+				Tags: map[string]string{
+					tags.JujuEnv: coretesting.EnvironmentTag.Id(),
+				},
+				Volumes: []params.VolumeParams{{
+					VolumeTag:  "volume-0",
+					Size:       1000,
+					Provider:   "static",
+					Attributes: map[string]interface{}{"foo": "bar"},
+					Tags: map[string]string{
+						tags.JujuEnv: coretesting.EnvironmentTag.Id(),
+					},
+					Attachment: &params.VolumeAttachmentParams{
+						MachineTag: placementMachine.Tag().String(),
+						VolumeTag:  "volume-0",
+						Provider:   "static",
+					},
+				}, {
+					VolumeTag:  "volume-1",
+					Size:       2000,
+					Provider:   "static",
+					Attributes: map[string]interface{}{"foo": "bar"},
+					Tags: map[string]string{
+						tags.JujuEnv: coretesting.EnvironmentTag.Id(),
+					},
+					Attachment: &params.VolumeAttachmentParams{
+						MachineTag: placementMachine.Tag().String(),
+						VolumeTag:  "volume-1",
+						Provider:   "static",
+					},
+				}},
 			}},
 			{Error: apiservertesting.NotFoundError("machine 42")},
 			{Error: apiservertesting.ErrUnauthorized},
 			{Error: apiservertesting.ErrUnauthorized},
 		},
+	}
+	// The order of volumes is not predictable, so we make sure we compare the right ones. This only
+	// applies to Results[1] since it is the only result to contain volumes.
+	if expected.Results[1].Result.Volumes[0].VolumeTag != result.Results[1].Result.Volumes[0].VolumeTag {
+		vols := expected.Results[1].Result.Volumes
+		vols[0], vols[1] = vols[1], vols[0]
+	}
+	c.Assert(result, jc.DeepEquals, expected)
+}
+
+func (s *withoutStateServerSuite) TestStorageProviderFallbackToType(c *gc.C) {
+	registry.RegisterProvider("dynamic", &storagedummy.StorageProvider{IsDynamic: true})
+	defer registry.RegisterProvider("dynamic", nil)
+	registry.RegisterProvider("static", &storagedummy.StorageProvider{IsDynamic: false})
+	defer registry.RegisterProvider("static", nil)
+	registry.RegisterEnvironStorageProviders("dummy", "dynamic", "static")
+
+	template := state.MachineTemplate{
+		Series:            "quantal",
+		Jobs:              []state.MachineJob{state.JobHostUnits},
+		Placement:         "valid",
+		RequestedNetworks: []string{"net1", "net2"},
+		Volumes: []state.MachineVolumeParams{
+			{Volume: state.VolumeParams{Size: 1000, Pool: "dynamic"}},
+			{Volume: state.VolumeParams{Size: 1000, Pool: "static"}},
+		},
+	}
+	placementMachine, err := s.State.AddOneMachine(template)
+	c.Assert(err, jc.ErrorIsNil)
+
+	args := params.Entities{Entities: []params.Entity{
+		{Tag: placementMachine.Tag().String()},
+	}}
+	result, err := s.provisioner.ProvisioningInfo(args)
+	c.Assert(err, jc.ErrorIsNil)
+
+	c.Assert(result, jc.DeepEquals, params.ProvisioningInfoResults{
+		Results: []params.ProvisioningInfoResult{
+			{Result: &params.ProvisioningInfo{
+				Series:      "quantal",
+				Constraints: template.Constraints,
+				Placement:   template.Placement,
+				Networks:    template.RequestedNetworks,
+				Jobs:        []multiwatcher.MachineJob{multiwatcher.JobHostUnits},
+				Tags: map[string]string{
+					tags.JujuEnv: coretesting.EnvironmentTag.Id(),
+				},
+				Volumes: []params.VolumeParams{{
+					VolumeTag:  "volume-1",
+					Size:       1000,
+					Provider:   "static",
+					Attributes: nil,
+					Tags: map[string]string{
+						tags.JujuEnv: coretesting.EnvironmentTag.Id(),
+					},
+					Attachment: &params.VolumeAttachmentParams{
+						MachineTag: placementMachine.Tag().String(),
+						VolumeTag:  "volume-1",
+						Provider:   "static",
+					},
+				}},
+			}},
+		},
 	})
 }
 
@@ -784,12 +912,15 @@ func (s *withoutStateServerSuite) TestProvisioningInfoPermissions(c *gc.C) {
 
 	// Only machine 0 and containers therein can be accessed.
 	results, err := aProvisioner.ProvisioningInfo(args)
-	c.Assert(results, gc.DeepEquals, params.ProvisioningInfoResults{
+	c.Assert(results, jc.DeepEquals, params.ProvisioningInfoResults{
 		Results: []params.ProvisioningInfoResult{
 			{Result: &params.ProvisioningInfo{
 				Series:   "quantal",
 				Networks: []string{},
 				Jobs:     []multiwatcher.MachineJob{multiwatcher.JobHostUnits},
+				Tags: map[string]string{
+					tags.JujuEnv: coretesting.EnvironmentTag.Id(),
+				},
 			}},
 			{Error: apiservertesting.NotFoundError("machine 0/lxc/0")},
 			{Error: apiservertesting.ErrUnauthorized},
@@ -917,9 +1048,30 @@ func (s *withoutStateServerSuite) TestSetProvisioned(c *gc.C) {
 }
 
 func (s *withoutStateServerSuite) TestSetInstanceInfo(c *gc.C) {
+	registry.RegisterProvider("static", &storagedummy.StorageProvider{IsDynamic: false})
+	defer registry.RegisterProvider("static", nil)
+	registry.RegisterEnvironStorageProviders("dummy", "static")
+
+	pm := poolmanager.New(state.NewStateSettings(s.State))
+	_, err := pm.Create("static-pool", "static", map[string]interface{}{"foo": "bar"})
+	c.Assert(err, jc.ErrorIsNil)
+	err = s.State.UpdateEnvironConfig(map[string]interface{}{
+		"storage-default-block-source": "static-pool",
+	}, nil, nil)
+	c.Assert(err, jc.ErrorIsNil)
+
 	// Provision machine 0 first.
 	hwChars := instance.MustParseHardware("arch=i386", "mem=4G")
-	err := s.machines[0].SetInstanceInfo("i-am", "fake_nonce", &hwChars, nil, nil)
+	err = s.machines[0].SetInstanceInfo("i-am", "fake_nonce", &hwChars, nil, nil, nil, nil)
+	c.Assert(err, jc.ErrorIsNil)
+
+	volumesMachine, err := s.State.AddOneMachine(state.MachineTemplate{
+		Series: "quantal",
+		Jobs:   []state.MachineJob{state.JobHostUnits},
+		Volumes: []state.MachineVolumeParams{{
+			Volume: state.VolumeParams{Size: 1000},
+		}},
+	})
 	c.Assert(err, jc.ErrorIsNil)
 
 	networks := []params.Network{{
@@ -993,6 +1145,22 @@ func (s *withoutStateServerSuite) TestSetInstanceInfo(c *gc.C) {
 		Characteristics: nil,
 		Networks:        networks,
 		Interfaces:      ifaces,
+	}, {
+		Tag:        volumesMachine.Tag().String(),
+		InstanceId: "i-am-also",
+		Nonce:      "fake",
+		Volumes: []params.Volume{{
+			VolumeTag: "volume-0",
+			Info: params.VolumeInfo{
+				VolumeId: "vol-0",
+				Size:     1234,
+			},
+		}},
+		VolumeAttachments: map[string]params.VolumeAttachmentInfo{
+			"volume-0": {
+				DeviceName: "sda",
+			},
+		},
 	},
 		{Tag: "machine-42"},
 		{Tag: "unit-foo-0"},
@@ -1003,10 +1171,11 @@ func (s *withoutStateServerSuite) TestSetInstanceInfo(c *gc.C) {
 	c.Assert(result, jc.DeepEquals, params.ErrorResults{
 		Results: []params.ErrorResult{
 			{&params.Error{
-				Message: `aborted instance "i-was": cannot set instance data for machine "0": already set`,
+				Message: `cannot record provisioning info for "i-was": cannot set instance data for machine "0": already set`,
 			}},
 			{nil},
 			{nil},
+			{nil},
 			{apiservertesting.NotFoundError("machine 42")},
 			{apiservertesting.ErrUnauthorized},
 			{apiservertesting.ErrUnauthorized},
@@ -1049,7 +1218,7 @@ func (s *withoutStateServerSuite) TestSetInstanceInfo(c *gc.C) {
 	c.Assert(ifacesMachine2[0].MACAddress(), gc.Equals, ifaces[5].MACAddress)
 	c.Assert(ifacesMachine2[0].NetworkTag().String(), gc.Equals, ifaces[5].NetworkTag)
 	c.Assert(ifacesMachine2[0].MachineId(), gc.Equals, s.machines[2].Id())
-	for i, _ := range networks {
+	for i := range networks {
 		if i == 3 {
 			// Last one was ignored, so don't check.
 			break
@@ -1057,14 +1226,34 @@ func (s *withoutStateServerSuite) TestSetInstanceInfo(c *gc.C) {
 		tag, err := names.ParseNetworkTag(networks[i].Tag)
 		c.Assert(err, jc.ErrorIsNil)
 		networkName := tag.Id()
-		network, err := s.State.Network(networkName)
+		nw, err := s.State.Network(networkName)
 		c.Assert(err, jc.ErrorIsNil)
-		c.Check(network.Name(), gc.Equals, networkName)
-		c.Check(network.ProviderId(), gc.Equals, networks[i].ProviderId)
-		c.Check(network.Tag().String(), gc.Equals, networks[i].Tag)
-		c.Check(network.VLANTag(), gc.Equals, networks[i].VLANTag)
-		c.Check(network.CIDR(), gc.Equals, networks[i].CIDR)
+		c.Check(nw.Name(), gc.Equals, networkName)
+		c.Check(nw.ProviderId(), gc.Equals, network.Id(networks[i].ProviderId))
+		c.Check(nw.Tag().String(), gc.Equals, networks[i].Tag)
+		c.Check(nw.VLANTag(), gc.Equals, networks[i].VLANTag)
+		c.Check(nw.CIDR(), gc.Equals, networks[i].CIDR)
 	}
+
+	// Verify the machine with requested volumes was provisioned, and the
+	// volume information recorded in state.
+	volumeAttachments, err := s.State.MachineVolumeAttachments(volumesMachine.MachineTag())
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(volumeAttachments, gc.HasLen, 1)
+	volumeAttachmentInfo, err := volumeAttachments[0].Info()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(volumeAttachmentInfo, gc.Equals, state.VolumeAttachmentInfo{DeviceName: "sda"})
+	volume, err := s.State.Volume(volumeAttachments[0].Volume())
+	c.Assert(err, jc.ErrorIsNil)
+	volumeInfo, err := volume.Info()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(volumeInfo, gc.Equals, state.VolumeInfo{VolumeId: "vol-0", Pool: "static-pool", Size: 1234})
+
+	// Verify the machine without requested volumes still has no volume
+	// attachments recorded in state.
+	volumeAttachments, err = s.State.MachineVolumeAttachments(s.machines[1].MachineTag())
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(volumeAttachments, gc.HasLen, 0)
 }
 
 func (s *withoutStateServerSuite) TestInstanceId(c *gc.C) {
@@ -1131,18 +1320,49 @@ func (s *withoutStateServerSuite) TestWatchEnvironMachines(c *gc.C) {
 	c.Assert(result, gc.DeepEquals, params.StringsWatchResult{})
 }
 
-func (s *withoutStateServerSuite) TestContainerManagerConfig(c *gc.C) {
-	args := params.ContainerManagerConfigParams{Type: instance.KVM}
+func (s *provisionerSuite) getManagerConfig(c *gc.C, typ instance.ContainerType) map[string]string {
+	args := params.ContainerManagerConfigParams{Type: typ}
 	results, err := s.provisioner.ContainerManagerConfig(args)
-	c.Check(err, jc.ErrorIsNil)
-	c.Assert(results.ManagerConfig, gc.DeepEquals, map[string]string{
+	c.Assert(err, jc.ErrorIsNil)
+	return results.ManagerConfig
+}
+
+func (s *withoutStateServerSuite) TestContainerManagerConfig(c *gc.C) {
+	cfg := s.getManagerConfig(c, instance.KVM)
+	c.Assert(cfg, jc.DeepEquals, map[string]string{
+		container.ConfigName: "juju",
+
+		// dummy provider supports both networking and address
+		// allocation by default, so IP forwarding should be enabled.
+		container.ConfigIPForwarding: "true",
+	})
+}
+
+func (s *withoutStateServerSuite) TestContainerManagerConfigNoFeatureFlagNoIPForwarding(c *gc.C) {
+	s.SetFeatureFlags() // clear the flags.
+
+	cfg := s.getManagerConfig(c, instance.KVM)
+	c.Assert(cfg, jc.DeepEquals, map[string]string{
+		container.ConfigName: "juju",
+		// ConfigIPForwarding should be missing.
+	})
+}
+
+func (s *withoutStateServerSuite) TestContainerManagerConfigNoIPForwarding(c *gc.C) {
+	// Break dummy provider's SupportsAddressAllocation method to
+	// ensure ConfigIPForwarding is not set below.
+	s.AssertConfigParameterUpdated(c, "broken", "SupportsAddressAllocation")
+
+	cfg := s.getManagerConfig(c, instance.KVM)
+	c.Assert(cfg, jc.DeepEquals, map[string]string{
 		container.ConfigName: "juju",
 	})
 }
 
 func (s *withoutStateServerSuite) TestContainerConfig(c *gc.C) {
 	attrs := map[string]interface{}{
-		"http-proxy": "http://proxy.example.com:9000",
+		"http-proxy":            "http://proxy.example.com:9000",
+		"allow-lxc-loop-mounts": true,
 	}
 	err := s.State.UpdateEnvironConfig(attrs, nil, nil)
 	c.Assert(err, jc.ErrorIsNil)
@@ -1159,21 +1379,17 @@ func (s *withoutStateServerSuite) TestContainerConfig(c *gc.C) {
 	c.Check(results.Proxy, gc.DeepEquals, expectedProxy)
 	c.Check(results.AptProxy, gc.DeepEquals, expectedProxy)
 	c.Check(results.PreferIPv6, jc.IsTrue)
+	c.Check(results.AllowLXCLoopMounts, jc.IsTrue)
 }
 
 func (s *withoutStateServerSuite) TestSetSupportedContainers(c *gc.C) {
-	args := params.MachineContainersParams{
-		Params: []params.MachineContainers{
-			{
-				MachineTag:     "machine-0",
-				ContainerTypes: []instance.ContainerType{instance.LXC},
-			},
-			{
-				MachineTag:     "machine-1",
-				ContainerTypes: []instance.ContainerType{instance.LXC, instance.KVM},
-			},
-		},
-	}
+	args := params.MachineContainersParams{Params: []params.MachineContainers{{
+		MachineTag:     "machine-0",
+		ContainerTypes: []instance.ContainerType{instance.LXC},
+	}, {
+		MachineTag:     "machine-1",
+		ContainerTypes: []instance.ContainerType{instance.LXC, instance.KVM},
+	}}}
 	results, err := s.provisioner.SetSupportedContainers(args)
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(results.Results, gc.HasLen, 2)
@@ -1255,11 +1471,9 @@ func (s *withStateServerSuite) SetUpTest(c *gc.C) {
 }
 
 func (s *withStateServerSuite) TestAPIAddresses(c *gc.C) {
-	hostPorts := [][]network.HostPort{{{
-		Address: network.NewAddress("0.1.2.3", network.ScopeUnknown),
-		Port:    1234,
-	}}}
-
+	hostPorts := [][]network.HostPort{
+		network.NewHostPorts(1234, "0.1.2.3"),
+	}
 	err := s.State.SetAPIHostPorts(hostPorts)
 	c.Assert(err, jc.ErrorIsNil)
 
@@ -1332,7 +1546,48 @@ func (s *withoutStateServerSuite) TestFindTools(c *gc.C) {
 	c.Assert(result.Error, gc.IsNil)
 	c.Assert(result.List, gc.Not(gc.HasLen), 0)
 	for _, tools := range result.List {
-		url := fmt.Sprintf("https://%s/environment/90168e4c-2f10-4e9c-83c2-feedfacee5a9/tools/%s", s.APIState.Addr(), tools.Version)
+		url := fmt.Sprintf("https://%s/environment/%s/tools/%s",
+			s.APIState.Addr(), coretesting.EnvironmentTag.Id(), tools.Version)
 		c.Assert(tools.URL, gc.Equals, url)
 	}
 }
+
+type lxcDefaultMTUSuite struct {
+	provisionerSuite
+}
+
+var _ = gc.Suite(&lxcDefaultMTUSuite{})
+
+func (s *lxcDefaultMTUSuite) SetUpTest(c *gc.C) {
+	// Because lxc-default-mtu is an immutable setting, we need to set
+	// it in the default config JujuConnSuite uses, before the
+	// environment is "created".
+	s.DummyConfig = dummy.SampleConfig()
+	s.DummyConfig["lxc-default-mtu"] = 9000
+	s.provisionerSuite.SetUpTest(c)
+
+	stateConfig, err := s.State.EnvironConfig()
+	c.Assert(err, jc.ErrorIsNil)
+	value, ok := stateConfig.LXCDefaultMTU()
+	c.Assert(ok, jc.IsTrue)
+	c.Assert(value, gc.Equals, 9000)
+	c.Logf("environ config lxc-default-mtu set to %v", value)
+}
+
+func (s *lxcDefaultMTUSuite) TestContainerManagerConfigLXCDefaultMTU(c *gc.C) {
+	managerConfig := s.getManagerConfig(c, instance.LXC)
+	c.Assert(managerConfig, jc.DeepEquals, map[string]string{
+		container.ConfigName:          "juju",
+		container.ConfigLXCDefaultMTU: "9000",
+
+		"use-aufs":                   "false",
+		container.ConfigIPForwarding: "true",
+	})
+
+	// KVM instances are not affected.
+	managerConfig = s.getManagerConfig(c, instance.KVM)
+	c.Assert(managerConfig, jc.DeepEquals, map[string]string{
+		container.ConfigName:         "juju",
+		container.ConfigIPForwarding: "true",
+	})
+}
