@@ -9,14 +9,15 @@ import (
 	"fmt"
 	"net/url"
 	"path"
-	"time"
 
 	"github.com/juju/errors"
 	"github.com/juju/names"
-	"gopkg.in/juju/charm.v4"
+	"gopkg.in/juju/charm.v5"
 
 	"github.com/juju/juju/apiserver/common"
+	leadershipapiserver "github.com/juju/juju/apiserver/leadership"
 	"github.com/juju/juju/apiserver/params"
+	"github.com/juju/juju/leadership"
 	"github.com/juju/juju/network"
 	"github.com/juju/juju/state"
 	"github.com/juju/juju/state/multiwatcher"
@@ -27,12 +28,13 @@ import (
 // and it's intended for embedding.
 type uniterBaseAPI struct {
 	*common.LifeGetter
-	*common.StatusSetter
+	*StatusAPI
 	*common.DeadEnsurer
 	*common.AgentEntityWatcher
 	*common.APIAddresser
 	*common.EnvironWatcher
 	*common.RebootRequester
+	*leadershipapiserver.LeadershipSettingsAccessor
 
 	st            *state.State
 	auth          common.Authorizer
@@ -93,13 +95,17 @@ func newUniterBaseAPI(st *state.State, resources *common.Resources, authorizer c
 
 	accessUnitOrService := common.AuthEither(accessUnit, accessService)
 	return &uniterBaseAPI{
-		LifeGetter:         common.NewLifeGetter(st, accessUnitOrService),
-		StatusSetter:       common.NewStatusSetter(st, accessUnit),
-		DeadEnsurer:        common.NewDeadEnsurer(st, accessUnit),
-		AgentEntityWatcher: common.NewAgentEntityWatcher(st, resources, accessUnitOrService),
-		APIAddresser:       common.NewAPIAddresser(st, resources),
-		EnvironWatcher:     common.NewEnvironWatcher(st, resources, authorizer),
-		RebootRequester:    common.NewRebootRequester(st, accessMachine),
+		LifeGetter:                 common.NewLifeGetter(st, accessUnitOrService),
+		DeadEnsurer:                common.NewDeadEnsurer(st, accessUnit),
+		AgentEntityWatcher:         common.NewAgentEntityWatcher(st, resources, accessUnitOrService),
+		APIAddresser:               common.NewAPIAddresser(st, resources),
+		EnvironWatcher:             common.NewEnvironWatcher(st, resources, authorizer),
+		RebootRequester:            common.NewRebootRequester(st, accessMachine),
+		LeadershipSettingsAccessor: leadershipSettingsAccessorFactory(st, resources, authorizer),
+
+		// TODO(fwereade): so *every* unit should be allowed to
+		// get/set its own status *and* its service's? FFS.
+		StatusAPI: NewStatusAPI(st, accessUnitOrService),
 
 		st:            st,
 		auth:          authorizer,
@@ -825,6 +831,34 @@ func (u *uniterBaseAPI) Actions(args params.Entities) (params.ActionsQueryResult
 	return results, nil
 }
 
+// BeginActions marks the actions represented by the passed in Tags as running.
+func (u *uniterBaseAPI) BeginActions(args params.Entities) (params.ErrorResults, error) {
+	nothing := params.ErrorResults{}
+
+	actionFn, err := u.authAndActionFromTagFn()
+	if err != nil {
+		return nothing, err
+	}
+
+	results := params.ErrorResults{Results: make([]params.ErrorResult, len(args.Entities))}
+
+	for i, arg := range args.Entities {
+		action, err := actionFn(arg.Tag)
+		if err != nil {
+			results.Results[i].Error = common.ServerError(err)
+			continue
+		}
+
+		_, err = action.Begin()
+		if err != nil {
+			results.Results[i].Error = common.ServerError(err)
+			continue
+		}
+	}
+
+	return results, nil
+}
+
 // FinishActions saves the result of a completed Action
 func (u *uniterBaseAPI) FinishActions(args params.ActionExecutionResults) (params.ErrorResults, error) {
 	nothing := params.ErrorResults{}
@@ -1027,13 +1061,13 @@ func (u *uniterBaseAPI) LeaveScope(args params.RelationUnits) (params.ErrorResul
 
 // ReadSettings returns the local settings of each given set of
 // relation/unit.
-func (u *uniterBaseAPI) ReadSettings(args params.RelationUnits) (params.RelationSettingsResults, error) {
-	result := params.RelationSettingsResults{
-		Results: make([]params.RelationSettingsResult, len(args.RelationUnits)),
+func (u *uniterBaseAPI) ReadSettings(args params.RelationUnits) (params.SettingsResults, error) {
+	result := params.SettingsResults{
+		Results: make([]params.SettingsResult, len(args.RelationUnits)),
 	}
 	canAccess, err := u.accessUnit()
 	if err != nil {
-		return params.RelationSettingsResults{}, err
+		return params.SettingsResults{}, err
 	}
 	for i, arg := range args.RelationUnits {
 		unit, err := names.ParseUnitTag(arg.Unit)
@@ -1056,13 +1090,13 @@ func (u *uniterBaseAPI) ReadSettings(args params.RelationUnits) (params.Relation
 
 // ReadRemoteSettings returns the remote settings of each given set of
 // relation/local unit/remote unit.
-func (u *uniterBaseAPI) ReadRemoteSettings(args params.RelationUnitPairs) (params.RelationSettingsResults, error) {
-	result := params.RelationSettingsResults{
-		Results: make([]params.RelationSettingsResult, len(args.RelationUnitPairs)),
+func (u *uniterBaseAPI) ReadRemoteSettings(args params.RelationUnitPairs) (params.SettingsResults, error) {
+	result := params.SettingsResults{
+		Results: make([]params.SettingsResult, len(args.RelationUnitPairs)),
 	}
 	canAccess, err := u.accessUnit()
 	if err != nil {
-		return params.RelationSettingsResults{}, err
+		return params.SettingsResults{}, err
 	}
 	for i, arg := range args.RelationUnitPairs {
 		unit, err := names.ParseUnitTag(arg.LocalUnit)
@@ -1178,43 +1212,6 @@ func (u *uniterBaseAPI) WatchUnitAddresses(args params.Entities) (params.NotifyW
 	return result, nil
 }
 
-// AddMetrics adds the metrics for the specified unit.
-func (u *uniterBaseAPI) AddMetrics(args params.MetricsParams) (params.ErrorResults, error) {
-	result := params.ErrorResults{
-		Results: make([]params.ErrorResult, len(args.Metrics)),
-	}
-	canAccess, err := u.accessUnit()
-	if err != nil {
-		return params.ErrorResults{}, common.ErrPerm
-	}
-	for i, unitMetrics := range args.Metrics {
-		tag, err := names.ParseUnitTag(unitMetrics.Tag)
-		if err != nil {
-			result.Results[i].Error = common.ServerError(common.ErrPerm)
-			continue
-		}
-		err = common.ErrPerm
-		if canAccess(tag) {
-			var unit *state.Unit
-			unit, err = u.getUnit(tag)
-			if err == nil {
-				metricBatch := make([]state.Metric, len(unitMetrics.Metrics))
-				for j, metric := range unitMetrics.Metrics {
-					// TODO (tasdomas) 2014-08-26: set credentials for metrics when available
-					metricBatch[j] = state.Metric{
-						Key:   metric.Key,
-						Value: metric.Value,
-						Time:  metric.Time,
-					}
-				}
-				_, err = unit.AddMetrics(time.Now(), metricBatch)
-			}
-		}
-		result.Results[i].Error = common.ServerError(err)
-	}
-	return result, nil
-}
-
 // GetMeterStatus returns meter status information for each unit.
 func (u *uniterBaseAPI) GetMeterStatus(args params.Entities) (params.MeterStatusResults, error) {
 	result := params.MeterStatusResults{
@@ -1231,17 +1228,16 @@ func (u *uniterBaseAPI) GetMeterStatus(args params.Entities) (params.MeterStatus
 			continue
 		}
 		err = common.ErrPerm
-		var code string
-		var info string
+		var status state.MeterStatus
 		if canAccess(unitTag) {
 			var unit *state.Unit
 			unit, err = u.getUnit(unitTag)
 			if err == nil {
-				code, info, err = unit.GetMeterStatus()
+				status, err = unit.GetMeterStatus()
 			}
+			result.Results[i].Code = status.Code.String()
+			result.Results[i].Info = status.Info
 		}
-		result.Results[i].Code = code
-		result.Results[i].Info = info
 		result.Results[i].Error = common.ServerError(err)
 	}
 	return result, nil
@@ -1473,7 +1469,10 @@ func (u *uniterBaseAPI) checkRemoteUnit(relUnit *state.RelationUnit, remoteUnitT
 		return "", common.ErrPerm
 	}
 	remoteUnitName := tag.Id()
-	remoteServiceName := names.UnitService(remoteUnitName)
+	remoteServiceName, err := names.UnitService(remoteUnitName)
+	if err != nil {
+		return "", common.ErrPerm
+	}
 	rel := relUnit.Relation()
 	_, err = rel.RelatedEndpoints(remoteServiceName)
 	if err != nil {
@@ -1519,8 +1518,8 @@ func (u *uniterBaseAPI) authAndActionFromTagFn() (func(string) (*state.Action, e
 	}, nil
 }
 
-func convertRelationSettings(settings map[string]interface{}) (params.RelationSettings, error) {
-	result := make(params.RelationSettings)
+func convertRelationSettings(settings map[string]interface{}) (params.Settings, error) {
+	result := make(params.Settings)
 	for k, v := range settings {
 		// All relation settings should be strings.
 		sval, ok := v.(string)
@@ -1543,3 +1542,42 @@ func relationsInScopeTags(unit *state.Unit) ([]string, error) {
 	}
 	return tags, nil
 }
+
+func leadershipSettingsAccessorFactory(
+	st *state.State,
+	resources *common.Resources,
+	auth common.Authorizer,
+) *leadershipapiserver.LeadershipSettingsAccessor {
+	registerWatcher := func(serviceId string) (string, error) {
+		service, err := st.Service(serviceId)
+		if err != nil {
+			return "", err
+		}
+		w := service.WatchLeaderSettings()
+		if _, ok := <-w.Changes(); ok {
+			return resources.Register(w), nil
+		}
+		return "", watcher.EnsureErr(w)
+	}
+	getSettings := func(serviceId string) (map[string]string, error) {
+		service, err := st.Service(serviceId)
+		if err != nil {
+			return nil, err
+		}
+		return service.LeaderSettings()
+	}
+	writeSettings := func(token leadership.Token, serviceId string, settings map[string]string) error {
+		service, err := st.Service(serviceId)
+		if err != nil {
+			return err
+		}
+		return service.UpdateLeaderSettings(token, settings)
+	}
+	return leadershipapiserver.NewLeadershipSettingsAccessor(
+		auth,
+		registerWatcher,
+		getSettings,
+		st.LeadershipChecker().LeadershipCheck,
+		writeSettings,
+	)
+}
