@@ -4,6 +4,7 @@
 package uniter_test
 
 import (
+	"fmt"
 	"time"
 
 	"github.com/juju/errors"
@@ -11,7 +12,7 @@ import (
 	jc "github.com/juju/testing/checkers"
 	"github.com/juju/utils"
 	gc "gopkg.in/check.v1"
-	"gopkg.in/juju/charm.v4"
+	"gopkg.in/juju/charm.v5"
 
 	"github.com/juju/juju/apiserver/common"
 	"github.com/juju/juju/apiserver/params"
@@ -22,6 +23,7 @@ import (
 	"github.com/juju/juju/state"
 	"github.com/juju/juju/state/multiwatcher"
 	statetesting "github.com/juju/juju/state/testing"
+	coretesting "github.com/juju/juju/testing"
 	jujuFactory "github.com/juju/juju/testing/factory"
 )
 
@@ -41,7 +43,10 @@ type uniterBaseSuite struct {
 	mysql         *state.Service
 	wordpressUnit *state.Unit
 	mysqlUnit     *state.Unit
-	meteredUnit   *state.Unit
+
+	meteredService *state.Service
+	meteredCharm   *state.Charm
+	meteredUnit    *state.Unit
 }
 
 func (s *uniterBaseSuite) setUpTest(c *gc.C) {
@@ -59,6 +64,7 @@ func (s *uniterBaseSuite) setUpTest(c *gc.C) {
 	})
 	s.wpCharm = factory.MakeCharm(c, &jujuFactory.CharmParams{
 		Name: "wordpress",
+		URL:  "cs:quantal/wordpress-3",
 	})
 	s.wordpress = factory.MakeService(c, &jujuFactory.ServiceParams{
 		Name:    "wordpress",
@@ -82,15 +88,15 @@ func (s *uniterBaseSuite) setUpTest(c *gc.C) {
 		Machine: s.machine1,
 	})
 
-	meteredCharm := s.Factory.MakeCharm(c, &jujuFactory.CharmParams{
+	s.meteredCharm = s.Factory.MakeCharm(c, &jujuFactory.CharmParams{
 		Name: "metered",
 		URL:  "cs:quantal/metered",
 	})
-	meteredService := s.Factory.MakeService(c, &jujuFactory.ServiceParams{
-		Charm: meteredCharm,
+	s.meteredService = s.Factory.MakeService(c, &jujuFactory.ServiceParams{
+		Charm: s.meteredCharm,
 	})
 	s.meteredUnit = s.Factory.MakeUnit(c, &jujuFactory.UnitParams{
-		Service:     meteredService,
+		Service:     s.meteredService,
 		SetCharmURL: true,
 	})
 
@@ -123,15 +129,15 @@ func (s *uniterBaseSuite) testSetStatus(
 		SetStatus(args params.SetStatus) (params.ErrorResults, error)
 	},
 ) {
-	err := s.wordpressUnit.SetStatus(state.StatusActive, "blah", nil)
+	err := s.wordpressUnit.SetAgentStatus(state.StatusExecuting, "blah", nil)
 	c.Assert(err, jc.ErrorIsNil)
-	err = s.mysqlUnit.SetStatus(state.StatusStopping, "foo", nil)
+	err = s.mysqlUnit.SetAgentStatus(state.StatusExecuting, "foo", nil)
 	c.Assert(err, jc.ErrorIsNil)
 
 	args := params.SetStatus{
 		Entities: []params.EntityStatus{
 			{Tag: "unit-mysql-0", Status: params.StatusError, Info: "not really"},
-			{Tag: "unit-wordpress-0", Status: params.StatusStopping, Info: "foobar"},
+			{Tag: "unit-wordpress-0", Status: params.StatusRebooting, Info: "foobar"},
 			{Tag: "unit-foo-42", Status: params.StatusActive, Info: "blah"},
 		}}
 	result, err := facade.SetStatus(args)
@@ -145,15 +151,93 @@ func (s *uniterBaseSuite) testSetStatus(
 	})
 
 	// Verify mysqlUnit - no change.
-	status, info, _, err := s.mysqlUnit.Status()
+	statusInfo, err := s.mysqlUnit.AgentStatus()
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(status, gc.Equals, state.StatusStopping)
-	c.Assert(info, gc.Equals, "foo")
+	c.Assert(statusInfo.Status, gc.Equals, state.StatusExecuting)
+	c.Assert(statusInfo.Message, gc.Equals, "foo")
 	// ...wordpressUnit is fine though.
-	status, info, _, err = s.wordpressUnit.Status()
+	statusInfo, err = s.wordpressUnit.AgentStatus()
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(status, gc.Equals, state.StatusStopping)
-	c.Assert(info, gc.Equals, "foobar")
+	c.Assert(statusInfo.Status, gc.Equals, state.StatusRebooting)
+	c.Assert(statusInfo.Message, gc.Equals, "foobar")
+}
+
+func (s *uniterBaseSuite) testSetAgentStatus(
+	c *gc.C,
+	facade interface {
+		SetAgentStatus(args params.SetStatus) (params.ErrorResults, error)
+	},
+) {
+	err := s.wordpressUnit.SetAgentStatus(state.StatusExecuting, "blah", nil)
+	c.Assert(err, jc.ErrorIsNil)
+	err = s.mysqlUnit.SetAgentStatus(state.StatusExecuting, "foo", nil)
+	c.Assert(err, jc.ErrorIsNil)
+
+	args := params.SetStatus{
+		Entities: []params.EntityStatus{
+			{Tag: "unit-mysql-0", Status: params.StatusError, Info: "not really"},
+			{Tag: "unit-wordpress-0", Status: params.StatusExecuting, Info: "foobar"},
+			{Tag: "unit-foo-42", Status: params.StatusRebooting, Info: "blah"},
+		}}
+	result, err := facade.SetAgentStatus(args)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(result, gc.DeepEquals, params.ErrorResults{
+		Results: []params.ErrorResult{
+			{apiservertesting.ErrUnauthorized},
+			{nil},
+			{apiservertesting.ErrUnauthorized},
+		},
+	})
+
+	// Verify mysqlUnit - no change.
+	statusInfo, err := s.mysqlUnit.AgentStatus()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(statusInfo.Status, gc.Equals, state.StatusExecuting)
+	c.Assert(statusInfo.Message, gc.Equals, "foo")
+	// ...wordpressUnit is fine though.
+	statusInfo, err = s.wordpressUnit.AgentStatus()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(statusInfo.Status, gc.Equals, state.StatusExecuting)
+	c.Assert(statusInfo.Message, gc.Equals, "foobar")
+}
+
+func (s *uniterBaseSuite) testSetUnitStatus(
+	c *gc.C,
+	facade interface {
+		SetUnitStatus(args params.SetStatus) (params.ErrorResults, error)
+	},
+) {
+	err := s.wordpressUnit.SetStatus(state.StatusActive, "blah", nil)
+	c.Assert(err, jc.ErrorIsNil)
+	err = s.mysqlUnit.SetStatus(state.StatusTerminated, "foo", nil)
+	c.Assert(err, jc.ErrorIsNil)
+
+	args := params.SetStatus{
+		Entities: []params.EntityStatus{
+			{Tag: "unit-mysql-0", Status: params.StatusError, Info: "not really"},
+			{Tag: "unit-wordpress-0", Status: params.StatusTerminated, Info: "foobar"},
+			{Tag: "unit-foo-42", Status: params.StatusActive, Info: "blah"},
+		}}
+	result, err := facade.SetUnitStatus(args)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(result, gc.DeepEquals, params.ErrorResults{
+		Results: []params.ErrorResult{
+			{apiservertesting.ErrUnauthorized},
+			{nil},
+			{apiservertesting.ErrUnauthorized},
+		},
+	})
+
+	// Verify mysqlUnit - no change.
+	statusInfo, err := s.mysqlUnit.Status()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(statusInfo.Status, gc.Equals, state.StatusTerminated)
+	c.Assert(statusInfo.Message, gc.Equals, "foo")
+	// ...wordpressUnit is fine though.
+	statusInfo, err = s.wordpressUnit.Status()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(statusInfo.Status, gc.Equals, state.StatusTerminated)
+	c.Assert(statusInfo.Message, gc.Equals, "foobar")
 }
 
 func (s *uniterBaseSuite) testLife(
@@ -354,7 +438,9 @@ func (s *uniterBaseSuite) testPublicAddress(
 	})
 
 	// Now set it an try again.
-	err = s.machine0.SetAddresses(network.NewAddress("1.2.3.4", network.ScopePublic))
+	err = s.machine0.SetProviderAddresses(
+		network.NewScopedAddress("1.2.3.4", network.ScopePublic),
+	)
 	c.Assert(err, jc.ErrorIsNil)
 	address, ok := s.wordpressUnit.PublicAddress()
 	c.Assert(address, gc.Equals, "1.2.3.4")
@@ -397,7 +483,9 @@ func (s *uniterBaseSuite) testPrivateAddress(
 	})
 
 	// Now set it and try again.
-	err = s.machine0.SetAddresses(network.NewAddress("1.2.3.4", network.ScopeCloudLocal))
+	err = s.machine0.SetProviderAddresses(
+		network.NewScopedAddress("1.2.3.4", network.ScopeCloudLocal),
+	)
 	c.Assert(err, jc.ErrorIsNil)
 	address, ok := s.wordpressUnit.PrivateAddress()
 	c.Assert(address, gc.Equals, "1.2.3.4")
@@ -1109,16 +1197,15 @@ func (s *uniterBaseSuite) testCharmArchiveURLs(
 ) {
 	dummyCharm := s.AddTestingCharm(c, "dummy")
 
-	hostPorts := [][]network.HostPort{{{
-		Address: network.NewAddress("1.2.3.4", network.ScopePublic),
-		Port:    1234,
-	}, {
-		Address: network.NewAddress("0.1.2.3", network.ScopeCloudLocal),
-		Port:    1234,
-	}}, {{
-		Address: network.NewAddress("1.2.3.5", network.ScopePublic),
-		Port:    1234,
-	}}}
+	hostPorts := [][]network.HostPort{
+		network.AddressesWithPort([]network.Address{
+			network.NewScopedAddress("1.2.3.4", network.ScopePublic),
+			network.NewScopedAddress("0.1.2.3", network.ScopeCloudLocal),
+		}, 1234),
+		network.AddressesWithPort([]network.Address{
+			network.NewScopedAddress("1.2.3.5", network.ScopePublic),
+		}, 1234),
+	}
 	err := s.State.SetAPIHostPorts(hostPorts)
 	c.Assert(err, jc.ErrorIsNil)
 
@@ -1131,15 +1218,15 @@ func (s *uniterBaseSuite) testCharmArchiveURLs(
 	c.Assert(err, jc.ErrorIsNil)
 
 	wordpressURLs := []string{
-		"https://0.1.2.3:1234/environment/90168e4c-2f10-4e9c-83c2-feedfacee5a9/charms?file=%2A&url=cs%3Aquantal%2Fwordpress-3",
-		"https://1.2.3.5:1234/environment/90168e4c-2f10-4e9c-83c2-feedfacee5a9/charms?file=%2A&url=cs%3Aquantal%2Fwordpress-3",
+		fmt.Sprintf("https://0.1.2.3:1234/environment/%s/charms?file=%%2A&url=cs%%3Aquantal%%2Fwordpress-3", coretesting.EnvironmentTag.Id()),
+		fmt.Sprintf("https://1.2.3.5:1234/environment/%s/charms?file=%%2A&url=cs%%3Aquantal%%2Fwordpress-3", coretesting.EnvironmentTag.Id()),
 	}
 	dummyURLs := []string{
-		"https://0.1.2.3:1234/environment/90168e4c-2f10-4e9c-83c2-feedfacee5a9/charms?file=%2A&url=local%3Aquantal%2Fdummy-1",
-		"https://1.2.3.5:1234/environment/90168e4c-2f10-4e9c-83c2-feedfacee5a9/charms?file=%2A&url=local%3Aquantal%2Fdummy-1",
+		fmt.Sprintf("https://0.1.2.3:1234/environment/%s/charms?file=%%2A&url=local%%3Aquantal%%2Fdummy-1", coretesting.EnvironmentTag.Id()),
+		fmt.Sprintf("https://1.2.3.5:1234/environment/%s/charms?file=%%2A&url=local%%3Aquantal%%2Fdummy-1", coretesting.EnvironmentTag.Id()),
 	}
 
-	c.Assert(result, gc.DeepEquals, params.StringsResults{
+	c.Assert(result, jc.DeepEquals, params.StringsResults{
 		Results: []params.StringsResult{
 			{Error: apiservertesting.ErrUnauthorized},
 			{Result: wordpressURLs},
@@ -1404,6 +1491,35 @@ func (s *uniterBaseSuite) testFinishActionsAuthAccess(c *gc.C, facade finishActi
 	}
 }
 
+type beginActions interface {
+	BeginActions(args params.Entities) (params.ErrorResults, error)
+}
+
+func (s *uniterBaseSuite) testBeginActions(c *gc.C, facade beginActions) {
+	ten_seconds_ago := time.Now().Add(-10 * time.Second)
+	good, err := s.wordpressUnit.AddAction("fakeaction", nil)
+	c.Assert(err, jc.ErrorIsNil)
+
+	running, err := s.wordpressUnit.RunningActions()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(len(running), gc.Equals, 0, gc.Commentf("expected no running actions, got %d", len(running)))
+
+	args := params.Entities{Entities: []params.Entity{{Tag: good.ActionTag().String()}}}
+	res, err := facade.BeginActions(args)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(len(res.Results), gc.Equals, 1)
+	c.Assert(res.Results[0].Error, gc.IsNil)
+
+	running, err = s.wordpressUnit.RunningActions()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(len(running), gc.Equals, 1, gc.Commentf("expected one running action, got %d", len(running)))
+	c.Assert(running[0].ActionTag(), gc.Equals, good.ActionTag())
+	enqueued, started := running[0].Enqueued(), running[0].Started()
+	c.Assert(ten_seconds_ago.Before(enqueued), jc.IsTrue, gc.Commentf("enqueued time should be after 10 seconds ago"))
+	c.Assert(ten_seconds_ago.Before(started), jc.IsTrue, gc.Commentf("started time should be after 10 seconds ago"))
+	c.Assert(started.After(enqueued) || started.Equal(enqueued), jc.IsTrue, gc.Commentf("started should be after or equal to enqueued time"))
+}
+
 func (s *uniterBaseSuite) testRelation(
 	c *gc.C,
 	facade interface {
@@ -1508,7 +1624,9 @@ func (s *uniterBaseSuite) testEnterScope(
 	},
 ) {
 	// Set wordpressUnit's private address first.
-	err := s.machine0.SetAddresses(network.NewAddress("1.2.3.4", network.ScopeCloudLocal))
+	err := s.machine0.SetProviderAddresses(
+		network.NewScopedAddress("1.2.3.4", network.ScopeCloudLocal),
+	)
 	c.Assert(err, jc.ErrorIsNil)
 
 	rel := s.addRelation(c, "wordpress", "mysql")
@@ -1654,7 +1772,7 @@ func (s *uniterBaseSuite) testJoinedRelations(
 }
 
 type readSettings interface {
-	ReadSettings(args params.RelationUnits) (params.RelationSettingsResults, error)
+	ReadSettings(args params.RelationUnits) (params.SettingsResults, error)
 }
 
 func (s *uniterBaseSuite) testReadSettings(c *gc.C, facade readSettings) {
@@ -1683,10 +1801,10 @@ func (s *uniterBaseSuite) testReadSettings(c *gc.C, facade readSettings) {
 	}}
 	result, err := facade.ReadSettings(args)
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(result, gc.DeepEquals, params.RelationSettingsResults{
-		Results: []params.RelationSettingsResult{
+	c.Assert(result, gc.DeepEquals, params.SettingsResults{
+		Results: []params.SettingsResult{
 			{Error: apiservertesting.ErrUnauthorized},
-			{Settings: params.RelationSettings{
+			{Settings: params.Settings{
 				"some": "settings",
 			}},
 			{Error: apiservertesting.ErrUnauthorized},
@@ -1720,15 +1838,15 @@ func (s *uniterBaseSuite) testReadSettingsWithNonStringValuesFails(c *gc.C, faca
 	expectErr := `unexpected relation setting "invalid-bool": expected string, got bool`
 	result, err := facade.ReadSettings(args)
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(result, gc.DeepEquals, params.RelationSettingsResults{
-		Results: []params.RelationSettingsResult{
+	c.Assert(result, gc.DeepEquals, params.SettingsResults{
+		Results: []params.SettingsResult{
 			{Error: &params.Error{Message: expectErr}},
 		},
 	})
 }
 
 type readRemoteSettings interface {
-	ReadRemoteSettings(args params.RelationUnitPairs) (params.RelationSettingsResults, error)
+	ReadRemoteSettings(args params.RelationUnitPairs) (params.SettingsResults, error)
 }
 
 func (s *uniterBaseSuite) testReadRemoteSettings(c *gc.C, facade readRemoteSettings) {
@@ -1762,8 +1880,8 @@ func (s *uniterBaseSuite) testReadRemoteSettings(c *gc.C, facade readRemoteSetti
 	// We don't set the remote unit settings on purpose to test the error.
 	expectErr := `cannot read settings for unit "mysql/0" in relation "wordpress:db mysql:server": settings`
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(result, jc.DeepEquals, params.RelationSettingsResults{
-		Results: []params.RelationSettingsResult{
+	c.Assert(result, jc.DeepEquals, params.SettingsResults{
+		Results: []params.SettingsResult{
 			{Error: apiservertesting.ErrUnauthorized},
 			{Error: apiservertesting.ErrUnauthorized},
 			{Error: apiservertesting.NotFoundError(expectErr)},
@@ -1797,9 +1915,9 @@ func (s *uniterBaseSuite) testReadRemoteSettings(c *gc.C, facade readRemoteSetti
 		LocalUnit:  "unit-wordpress-0",
 		RemoteUnit: "unit-mysql-0",
 	}}}
-	expect := params.RelationSettingsResults{
-		Results: []params.RelationSettingsResult{
-			{Settings: params.RelationSettings{
+	expect := params.SettingsResults{
+		Results: []params.SettingsResult{
+			{Settings: params.Settings{
 				"other": "things",
 			}},
 		},
@@ -1840,8 +1958,8 @@ func (s *uniterBaseSuite) testReadRemoteSettingsWithNonStringValuesFails(c *gc.C
 	expectErr := `unexpected relation setting "invalid-bool": expected string, got bool`
 	result, err := facade.ReadRemoteSettings(args)
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(result, gc.DeepEquals, params.RelationSettingsResults{
-		Results: []params.RelationSettingsResult{
+	c.Assert(result, gc.DeepEquals, params.SettingsResults{
+		Results: []params.SettingsResult{
 			{Error: &params.Error{Message: expectErr}},
 		},
 	})
@@ -1863,7 +1981,7 @@ func (s *uniterBaseSuite) testUpdateSettings(
 	err = relUnit.EnterScope(settings)
 	s.assertInScope(c, relUnit, true)
 
-	newSettings := params.RelationSettings{
+	newSettings := params.Settings{
 		"some":  "different",
 		"other": "",
 	}
@@ -1992,11 +2110,9 @@ func (s *uniterBaseSuite) testAPIAddresses(
 		APIAddresses() (params.StringsResult, error)
 	},
 ) {
-	hostPorts := [][]network.HostPort{{{
-		Address: network.NewAddress("0.1.2.3", network.ScopeUnknown),
-		Port:    1234,
-	}}}
-
+	hostPorts := [][]network.HostPort{
+		network.NewHostPorts(1234, "0.1.2.3"),
+	}
 	err := s.State.SetAPIHostPorts(hostPorts)
 	c.Assert(err, jc.ErrorIsNil)
 
@@ -2045,78 +2161,6 @@ func (s *uniterBaseSuite) testWatchUnitAddresses(
 	wc.AssertNoChange()
 }
 
-type addMetrics interface {
-	AddMetrics(args params.MetricsParams) (params.ErrorResults, error)
-}
-
-func (s *uniterBaseSuite) testAddMetrics(c *gc.C, facade addMetrics) {
-	now := time.Now()
-	sentMetrics := []params.Metric{{"pings", "5", now}, {"juju-unit-time", "0.71", now}}
-	args := params.MetricsParams{
-		Metrics: []params.MetricsParam{{
-			Tag:     s.meteredUnit.Tag().String(),
-			Metrics: sentMetrics,
-		}},
-	}
-	result, err := facade.AddMetrics(args)
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(result.Results, gc.HasLen, 1)
-	c.Assert(result.Results[0].Error, gc.IsNil)
-
-	metrics, err := s.State.MetricBatches()
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(metrics, gc.HasLen, 1)
-
-	unitMetrics := metrics[0].Metrics()
-	c.Assert(unitMetrics, gc.HasLen, 2)
-
-	for i, unitMetric := range unitMetrics {
-		c.Assert(unitMetric.Key, gc.Equals, sentMetrics[i].Key)
-		c.Assert(unitMetric.Value, gc.Equals, sentMetrics[i].Value)
-	}
-}
-
-func (s *uniterBaseSuite) testAddMetricsIncorrectTag(c *gc.C, facade addMetrics) {
-	now := time.Now()
-
-	tags := []string{"user-admin", "unit-nosuchunit", "thisisnotatag", "machine-0", "environment-blah"}
-
-	for _, tag := range tags {
-
-		args := params.MetricsParams{
-			Metrics: []params.MetricsParam{{
-				Tag:     tag,
-				Metrics: []params.Metric{{"A", "5", now}, {"B", "0.71", now}},
-			}},
-		}
-
-		result, err := facade.AddMetrics(args)
-		c.Assert(err, jc.ErrorIsNil)
-		c.Assert(result.Results, gc.HasLen, 1)
-		c.Assert(result.Results[0].Error, gc.ErrorMatches, "permission denied")
-		metrics, err := s.State.MetricBatches()
-		c.Assert(err, jc.ErrorIsNil)
-		c.Assert(metrics, gc.HasLen, 0)
-	}
-}
-
-func (s *uniterBaseSuite) testAddMetricsUnauthenticated(c *gc.C, facade addMetrics) {
-	now := time.Now()
-	args := params.MetricsParams{
-		Metrics: []params.MetricsParam{{
-			Tag:     s.mysqlUnit.Tag().String(),
-			Metrics: []params.Metric{{"A", "5", now}, {"B", "0.71", now}},
-		}},
-	}
-	result, err := facade.AddMetrics(args)
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(result.Results, gc.HasLen, 1)
-	c.Assert(result.Results[0].Error, gc.ErrorMatches, "permission denied")
-	metrics, err := s.State.MetricBatches()
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(metrics, gc.HasLen, 0)
-}
-
 type getMeterStatus interface {
 	GetMeterStatus(args params.Entities) (params.MeterStatusResults, error)
 }
