@@ -9,7 +9,7 @@ import (
 	"github.com/juju/cmd"
 
 	"github.com/juju/juju/cmd/envcmd"
-	"github.com/juju/juju/juju"
+	"github.com/juju/juju/cmd/juju/block"
 )
 
 // AddRelationCommand adds a relation between two service endpoints.
@@ -35,11 +35,11 @@ func (c *AddRelationCommand) Init(args []string) error {
 }
 
 func (c *AddRelationCommand) Run(_ *cmd.Context) error {
-	client, err := juju.NewAPIClientFromName(c.EnvName)
+	client, err := c.NewAPIClient()
 	if err != nil {
 		return err
 	}
 	defer client.Close()
 	_, err = client.AddRelation(c.Endpoints...)
-	return err
+	return block.ProcessBlockedError(err, block.BlockChange)
 }
