@@ -6,7 +6,9 @@ package main
 import (
 	"fmt"
 	"os"
+	"runtime"
 	"strings"
+	"time"
 
 	"github.com/juju/cmd"
 	"github.com/juju/errors"
@@ -15,7 +17,9 @@ import (
 	jc "github.com/juju/testing/checkers"
 	gc "gopkg.in/check.v1"
 
+	"github.com/juju/juju/apiserver/params"
 	"github.com/juju/juju/cmd/envcmd"
+	"github.com/juju/juju/cmd/juju/block"
 	cmdtesting "github.com/juju/juju/cmd/testing"
 	"github.com/juju/juju/constraints"
 	"github.com/juju/juju/environs"
@@ -32,6 +36,7 @@ import (
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/juju"
 	"github.com/juju/juju/juju/arch"
+	"github.com/juju/juju/juju/osenv"
 	"github.com/juju/juju/network"
 	"github.com/juju/juju/provider/dummy"
 	coretesting "github.com/juju/juju/testing"
@@ -43,6 +48,7 @@ type BootstrapSuite struct {
 	coretesting.FakeJujuHomeSuite
 	gitjujutesting.MgoSuite
 	envtesting.ToolsFixture
+	mockBlockClient *mockBlockClient
 }
 
 var _ = gc.Suite(&BootstrapSuite{})
@@ -67,6 +73,11 @@ func (s *BootstrapSuite) SetUpTest(c *gc.C) {
 	s.PatchValue(&envtools.DefaultBaseURL, sourceDir)
 
 	s.PatchValue(&envtools.BundleTools, toolstesting.GetMockBundleTools(c))
+
+	s.mockBlockClient = &mockBlockClient{}
+	s.PatchValue(&blockAPI, func(c *envcmd.EnvCommandBase) (block.BlockListAPI, error) {
+		return s.mockBlockClient, nil
+	})
 }
 
 func (s *BootstrapSuite) TearDownSuite(c *gc.C) {
@@ -81,6 +92,69 @@ func (s *BootstrapSuite) TearDownTest(c *gc.C) {
 	dummy.Reset()
 }
 
+type mockBlockClient struct {
+	retry_count int
+	num_retries int
+}
+
+func (c *mockBlockClient) List() ([]params.Block, error) {
+	c.retry_count += 1
+	if c.retry_count == 5 {
+		return nil, fmt.Errorf("upgrade in progress")
+	}
+	if c.num_retries < 0 {
+		return nil, fmt.Errorf("other error")
+	}
+	if c.retry_count < c.num_retries {
+		return nil, fmt.Errorf("upgrade in progress")
+	}
+	return []params.Block{}, nil
+}
+
+func (c *mockBlockClient) Close() error {
+	return nil
+}
+
+func (s *BootstrapSuite) TestBootstrapAPIReadyRetries(c *gc.C) {
+	s.PatchValue(&bootstrapReadyPollDelay, 1*time.Millisecond)
+	s.PatchValue(&bootstrapReadyPollCount, 5)
+	defaultSeriesVersion := version.Current
+	// Force a dev version by having a non zero build number.
+	// This is because we have not uploaded any tools and auto
+	// upload is only enabled for dev versions.
+	defaultSeriesVersion.Build = 1234
+	s.PatchValue(&version.Current, defaultSeriesVersion)
+	for _, t := range []struct {
+		num_retries int
+		err         string
+	}{
+		{0, ""},                    // agent ready immediately
+		{2, ""},                    // agent ready after 2 polls
+		{6, "upgrade in progress"}, // agent ready after 6 polls but that's too long
+		{-1, "other error"},        // another error is returned
+	} {
+		resetJujuHome(c, "devenv")
+
+		s.mockBlockClient.num_retries = t.num_retries
+		s.mockBlockClient.retry_count = 0
+		_, err := coretesting.RunCommand(c, envcmd.Wrap(&BootstrapCommand{}), "-e", "devenv")
+		if t.err == "" {
+			c.Check(err, jc.ErrorIsNil)
+		} else {
+			c.Check(err, gc.ErrorMatches, t.err)
+		}
+		expectedRetries := t.num_retries
+		if t.num_retries <= 0 {
+			expectedRetries = 1
+		}
+		// Only retry maximum of bootstrapReadyPollCount times.
+		if expectedRetries > 5 {
+			expectedRetries = 5
+		}
+		c.Check(s.mockBlockClient.retry_count, gc.Equals, expectedRetries)
+	}
+}
+
 func (s *BootstrapSuite) TestRunTests(c *gc.C) {
 	for i, test := range bootstrapTests {
 		c.Logf("\ntest %d: %s", i, test.info)
@@ -166,9 +240,9 @@ func (s *BootstrapSuite) run(c *gc.C, test bootstrapTest) (restore gitjujutestin
 
 	opFinalizeBootstrap := (<-opc).(dummy.OpFinalizeBootstrap)
 	c.Check(opFinalizeBootstrap.Env, gc.Equals, "peckham")
-	c.Check(opFinalizeBootstrap.MachineConfig.Tools, gc.NotNil)
+	c.Check(opFinalizeBootstrap.InstanceConfig.Tools, gc.NotNil)
 	if test.upload != "" {
-		c.Check(opFinalizeBootstrap.MachineConfig.Tools.Version.String(), gc.Equals, test.upload)
+		c.Check(opFinalizeBootstrap.InstanceConfig.Tools.Version.String(), gc.Equals, test.upload)
 	}
 
 	store, err := configstore.Default()
@@ -265,6 +339,46 @@ var bootstrapTests = []bootstrapTest{{
 	err:  `unrecognized args: \["anything" "else"\]`,
 }}
 
+func (s *BootstrapSuite) TestRunEnvNameMissing(c *gc.C) {
+	s.PatchValue(&getEnvName, func(*BootstrapCommand) string { return "" })
+
+	_, err := coretesting.RunCommand(c, envcmd.Wrap(&BootstrapCommand{}))
+
+	c.Check(err, gc.ErrorMatches, "the name of the environment must be specified")
+}
+
+const provisionalEnvs = `
+environments:
+    devenv:
+        type: dummy
+    cloudsigma:
+        type: cloudsigma
+    vsphere:
+        type: vsphere
+`
+
+func (s *BootstrapSuite) TestCheckProviderProvisional(c *gc.C) {
+	coretesting.WriteEnvironments(c, provisionalEnvs)
+
+	err := checkProviderType("devenv")
+	c.Assert(err, jc.ErrorIsNil)
+
+	for name, flag := range provisionalProviders {
+		// vsphere is disabled for gccgo. See lp:1440940.
+		if name == "vsphere" && runtime.Compiler == "gccgo" {
+			continue
+		}
+		c.Logf(" - trying %q -", name)
+		err := checkProviderType(name)
+		c.Check(err, gc.ErrorMatches, ".* provider is provisional .* set JUJU_DEV_FEATURE_FLAGS=.*")
+
+		err = os.Setenv(osenv.JujuFeatureFlagEnvKey, flag)
+		c.Assert(err, jc.ErrorIsNil)
+		err = checkProviderType(name)
+		c.Check(err, jc.ErrorIsNil)
+	}
+}
+
 func (s *BootstrapSuite) TestBootstrapTwice(c *gc.C) {
 	env := resetJujuHome(c, "devenv")
 	defaultSeriesVersion := version.Current
@@ -287,19 +401,19 @@ type mockBootstrapInstance struct {
 }
 
 func (*mockBootstrapInstance) Addresses() ([]network.Address, error) {
-	return []network.Address{network.Address{Value: "localhost"}}, nil
+	return []network.Address{{Value: "localhost"}}, nil
 }
 
 func (s *BootstrapSuite) TestSeriesDeprecation(c *gc.C) {
 	ctx := s.checkSeriesArg(c, "--series")
 	c.Check(coretesting.Stderr(ctx), gc.Equals,
-		"Use of --series is obsolete. --upload-tools now expands to all supported series of the same operating system.\n")
+		"Use of --series is obsolete. --upload-tools now expands to all supported series of the same operating system.\nBootstrap complete\n")
 }
 
 func (s *BootstrapSuite) TestUploadSeriesDeprecation(c *gc.C) {
 	ctx := s.checkSeriesArg(c, "--upload-series")
 	c.Check(coretesting.Stderr(ctx), gc.Equals,
-		"Use of --upload-series is obsolete. --upload-tools now expands to all supported series of the same operating system.\n")
+		"Use of --upload-series is obsolete. --upload-tools now expands to all supported series of the same operating system.\nBootstrap complete\n")
 }
 
 func (s *BootstrapSuite) checkSeriesArg(c *gc.C, argVariant string) *cmd.Context {
@@ -321,6 +435,10 @@ func (s *BootstrapSuite) checkSeriesArg(c *gc.C, argVariant string) *cmd.Context
 // In the case where we cannot examine an environment, we want the
 // error to propagate back up to the user.
 func (s *BootstrapSuite) TestBootstrapPropagatesEnvErrors(c *gc.C) {
+	//TODO(bogdanteleaga): fix this for windows once permissions are fixed
+	if runtime.GOOS == "windows" {
+		c.Skip("bug 1403084: this is very platform specific. When/if we will support windows state machine, this will probably be rewritten.")
+	}
 
 	const envName = "devenv"
 	env := resetJujuHome(c, envName)
@@ -331,6 +449,7 @@ func (s *BootstrapSuite) TestBootstrapPropagatesEnvErrors(c *gc.C) {
 	// upload is only enabled for dev versions.
 	defaultSeriesVersion.Build = 1234
 	s.PatchValue(&version.Current, defaultSeriesVersion)
+	s.PatchValue(&environType, func(string) (string, error) { return "", nil })
 
 	_, err := coretesting.RunCommand(c, envcmd.Wrap(&BootstrapCommand{}), "-e", envName)
 	c.Assert(err, jc.ErrorIsNil)
@@ -350,6 +469,7 @@ func (s *BootstrapSuite) TestBootstrapCleansUpIfEnvironPrepFails(c *gc.C) {
 
 	cleanupRan := false
 
+	s.PatchValue(&environType, func(string) (string, error) { return "", nil })
 	s.PatchValue(
 		&environFromName,
 		func(
@@ -422,6 +542,7 @@ func (s *BootstrapSuite) TestBootstrapFailToPrepareDiesGracefully(c *gc.C) {
 	// Simulation: prepare should fail and we should only clean up the
 	// jenv file. Any existing environment should not be destroyed.
 	s.PatchValue(&destroyPreparedEnviron, mockDestroyPreparedEnviron)
+	s.PatchValue(&environType, func(string) (string, error) { return "", nil })
 	s.PatchValue(&environFromName, mockEnvironFromName)
 	s.PatchValue(&environs.PrepareFromName, mockPrepare)
 	s.PatchValue(&destroyEnvInfo, mockDestroyEnvInfo)
@@ -552,9 +673,9 @@ func (s *BootstrapSuite) TestAutoUploadAfterFailedSync(c *gc.C) {
 	opc, errc := cmdtesting.RunCommand(cmdtesting.NullContext(c), envcmd.Wrap(new(BootstrapCommand)), "-e", "devenv")
 	c.Assert(<-errc, gc.IsNil)
 	c.Check((<-opc).(dummy.OpBootstrap).Env, gc.Equals, "devenv")
-	mcfg := (<-opc).(dummy.OpFinalizeBootstrap).MachineConfig
-	c.Assert(mcfg, gc.NotNil)
-	c.Assert(mcfg.Tools.Version.String(), gc.Equals, "1.7.3.1-raring-"+version.Current.Arch)
+	icfg := (<-opc).(dummy.OpFinalizeBootstrap).InstanceConfig
+	c.Assert(icfg, gc.NotNil)
+	c.Assert(icfg.Tools.Version.String(), gc.Equals, "1.7.3.1-raring-"+version.Current.Arch)
 }
 
 func (s *BootstrapSuite) TestAutoUploadOnlyForDev(c *gc.C) {
