@@ -11,6 +11,7 @@ import (
 	gc "gopkg.in/check.v1"
 
 	"github.com/juju/juju/cmd/envcmd"
+	"github.com/juju/juju/cmd/juju/service"
 	cmdtesting "github.com/juju/juju/cmd/testing"
 	"github.com/juju/juju/juju/osenv"
 	"github.com/juju/juju/juju/testing"
@@ -151,10 +152,10 @@ var deployTests = []struct {
 		&DeployCommand{BumpRevision: true},
 	}, {
 		[]string{"--num-units", "33", "charm-name"},
-		&DeployCommand{UnitCommandBase: UnitCommandBase{NumUnits: 33}},
+		&DeployCommand{UnitCommandBase: service.UnitCommandBase{NumUnits: 33}},
 	}, {
 		[]string{"-n", "104", "charm-name"},
-		&DeployCommand{UnitCommandBase: UnitCommandBase{NumUnits: 104}},
+		&DeployCommand{UnitCommandBase: service.UnitCommandBase{NumUnits: 104}},
 	},
 }
 
@@ -207,19 +208,6 @@ func (*CmdSuite) TestDeployCommandInit(c *gc.C) {
 	_, err = initDeployCommand()
 	c.Assert(err, gc.ErrorMatches, "no charm specified")
 
-	// environment tested elsewhere
-}
-
-func initAddUnitCommand(args ...string) (*AddUnitCommand, error) {
-	com := &AddUnitCommand{}
-	return com, coretesting.InitCommand(com, args)
-}
-
-func (*CmdSuite) TestAddUnitCommandInit(c *gc.C) {
-	// missing args
-	_, err := initAddUnitCommand()
-	c.Assert(err, gc.ErrorMatches, "no service specified")
-
 	// bad unit count
 	_, err = initDeployCommand("charm-name", "--num-units", "0")
 	c.Assert(err, gc.ErrorMatches, "--num-units must be a positive integer")
@@ -281,66 +269,6 @@ func (*CmdSuite) TestSCPCommandInit(c *gc.C) {
 	c.Assert(err, gc.ErrorMatches, "at least two arguments required")
 }
 
-func initGetCommand(args ...string) (*GetCommand, error) {
-	com := &GetCommand{}
-	return com, coretesting.InitCommand(com, args)
-}
-
-func (*CmdSuite) TestGetCommandInit(c *gc.C) {
-	// missing args
-	_, err := initGetCommand()
-	c.Assert(err, gc.ErrorMatches, "no service name specified")
-}
-
-func initSetCommand(args ...string) (*SetCommand, error) {
-	com := &SetCommand{}
-	return com, coretesting.InitCommand(com, args)
-}
-
-func (*CmdSuite) TestSetCommandInit(c *gc.C) {
-	// missing args
-	_, err := initSetCommand()
-	c.Assert(err, gc.ErrorMatches, "no service name specified")
-	// missing service name
-	_, err = initSetCommand("name=cow")
-	c.Assert(err, gc.ErrorMatches, "no service name specified")
-
-	// test --config path
-	expected := []byte("this: is some test data")
-	ctx := coretesting.Context(c)
-	path := ctx.AbsPath("testconfig.yaml")
-	file, err := os.Create(path)
-	c.Assert(err, jc.ErrorIsNil)
-	_, err = file.Write(expected)
-	c.Assert(err, jc.ErrorIsNil)
-	file.Close()
-	com, err := initSetCommand("--config", "testconfig.yaml", "service")
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(com.SettingsYAML.Path, gc.Equals, "testconfig.yaml")
-	actual, err := com.SettingsYAML.Read(ctx)
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(actual, gc.DeepEquals, expected)
-
-	// --config path, but no service
-	com, err = initSetCommand("--config", "testconfig")
-	c.Assert(err, gc.ErrorMatches, "no service name specified")
-
-	// --config and options specified
-	com, err = initSetCommand("service", "--config", "testconfig", "bees=")
-	c.Assert(err, gc.ErrorMatches, "cannot specify --config when using key=value arguments")
-}
-
-func initUnsetCommand(args ...string) (*UnsetCommand, error) {
-	com := &UnsetCommand{}
-	return com, coretesting.InitCommand(com, args)
-}
-
-func (*CmdSuite) TestUnsetCommandInit(c *gc.C) {
-	// missing args
-	_, err := initUnsetCommand()
-	c.Assert(err, gc.ErrorMatches, "no service name specified")
-}
-
 func initRemoveUnitCommand(args ...string) (*RemoveUnitCommand, error) {
 	com := &RemoveUnitCommand{}
 	return com, coretesting.InitCommand(com, args)
