@@ -19,10 +19,19 @@ import (
 	"github.com/juju/juju/juju/testing"
 	"github.com/juju/juju/provider/dummy"
 	coretesting "github.com/juju/juju/testing"
+	"github.com/juju/juju/testing/factory"
 )
 
 type destroyEnvSuite struct {
 	testing.JujuConnSuite
+	CmdBlockHelper
+}
+
+func (s *destroyEnvSuite) SetUpTest(c *gc.C) {
+	s.JujuConnSuite.SetUpTest(c)
+	s.CmdBlockHelper = NewCmdBlockHelper(s.APIState)
+	c.Assert(s.CmdBlockHelper, gc.NotNil)
+	s.AddCleanup(func(*gc.C) { s.CmdBlockHelper.Close() })
 }
 
 var _ = gc.Suite(&destroyEnvSuite{})
@@ -56,7 +65,9 @@ func (s *destroyEnvSuite) checkDestroyEnvironment(c *gc.C, blocked, force bool)
 	//Setup environment
 	envName := "dummyenv"
 	s.startEnvironment(c, envName)
-	s.AssertConfigParameterUpdated(c, "block-destroy-environment", blocked)
+	if blocked {
+		s.BlockDestroyEnvironment(c, "checkDestroyEnvironment")
+	}
 	opc := make(chan dummy.Operation)
 	errc := make(chan error)
 	if force {
@@ -123,14 +134,109 @@ func (s *destroyEnvSuite) TestDestroyEnvironmentCommandEFlag(c *gc.C) {
 }
 
 func (s *destroyEnvSuite) TestDestroyEnvironmentCommandEmptyJenv(c *gc.C) {
-	info := s.ConfigStore.CreateInfo("emptyenv")
-	err := info.Write()
+	oldinfo, err := s.ConfigStore.ReadInfo("dummyenv")
+	info := s.ConfigStore.CreateInfo("dummy-no-bootstrap")
+	info.SetAPICredentials(oldinfo.APICredentials())
+	info.SetAPIEndpoint(oldinfo.APIEndpoint())
+	err = info.Write()
 	c.Assert(err, jc.ErrorIsNil)
 
-	context, err := coretesting.RunCommand(c, new(DestroyEnvironmentCommand), "-e", "emptyenv")
+	opc, errc := cmdtesting.RunCommand(cmdtesting.NullContext(c), new(DestroyEnvironmentCommand), "dummy-no-bootstrap", "--yes")
+	c.Check(<-errc, gc.IsNil)
+	c.Check((<-opc).(dummy.OpDestroy).Env, gc.Equals, "dummyenv")
+
+	// Verify that the environment information has been removed.
+	_, err = s.ConfigStore.ReadInfo("dummyenv")
+	c.Assert(err, jc.Satisfies, errors.IsNotFound)
+}
+
+func (s *destroyEnvSuite) TestDestroyEnvironmentCommandNonStateServer(c *gc.C) {
+	s.setupHostedEnviron(c, "dummy-non-state-server")
+	opc, errc := cmdtesting.RunCommand(cmdtesting.NullContext(c), new(DestroyEnvironmentCommand), "dummy-non-state-server", "--yes")
+	c.Check(<-errc, gc.IsNil)
+	// Check that there are no operations on the provider, we do not want to call
+	// Destroy on it.
+	c.Check(<-opc, gc.IsNil)
+
+	_, err := s.ConfigStore.ReadInfo("dummy-non-state-server")
+	c.Assert(err, jc.Satisfies, errors.IsNotFound)
+}
+
+func (s *destroyEnvSuite) TestForceDestroyEnvironmentCommandOnNonStateServerFails(c *gc.C) {
+	s.setupHostedEnviron(c, "dummy-non-state-server")
+	opc, errc := cmdtesting.RunCommand(cmdtesting.NullContext(c), new(DestroyEnvironmentCommand), "dummy-non-state-server", "--yes", "--force")
+	c.Check(<-errc, gc.ErrorMatches, "cannot force destroy environment without bootstrap information")
+	c.Check(<-opc, gc.IsNil)
+
+	serverInfo, err := s.ConfigStore.ReadInfo("dummy-non-state-server")
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(serverInfo, gc.Not(gc.IsNil))
+}
+
+func (s *destroyEnvSuite) TestForceDestroyEnvironmentCommandOnNonStateServerNoConfimFails(c *gc.C) {
+	s.setupHostedEnviron(c, "dummy-non-state-server")
+	opc, errc := cmdtesting.RunCommand(cmdtesting.NullContext(c), new(DestroyEnvironmentCommand), "dummy-non-state-server", "--force")
+	c.Check(<-errc, gc.ErrorMatches, "cannot force destroy environment without bootstrap information")
+	c.Check(<-opc, gc.IsNil)
+
+	serverInfo, err := s.ConfigStore.ReadInfo("dummy-non-state-server")
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(serverInfo, gc.Not(gc.IsNil))
+}
+
+func (s *destroyEnvSuite) TestDestroyEnvironmentCommandTwiceOnNonStateServer(c *gc.C) {
+	s.setupHostedEnviron(c, "dummy-non-state-server")
+	oldInfo, err := s.ConfigStore.ReadInfo("dummy-non-state-server")
 	c.Assert(err, jc.ErrorIsNil)
 
-	c.Assert(coretesting.Stderr(context), gc.Equals, "removing empty environment file\n")
+	opc, errc := cmdtesting.RunCommand(cmdtesting.NullContext(c), new(DestroyEnvironmentCommand), "dummy-non-state-server", "--yes")
+	c.Check(<-errc, gc.IsNil)
+	c.Check(<-opc, gc.IsNil)
+
+	_, err = s.ConfigStore.ReadInfo("dummy-non-state-server")
+	c.Assert(err, jc.Satisfies, errors.IsNotFound)
+
+	// Simluate another client calling destroy on the same environment. This
+	// client will have a local cache of the environ info, so write it back out.
+	info := s.ConfigStore.CreateInfo("dummy-non-state-server")
+	info.SetAPIEndpoint(oldInfo.APIEndpoint())
+	info.SetAPICredentials(oldInfo.APICredentials())
+	err = info.Write()
+	c.Assert(err, jc.ErrorIsNil)
+
+	// Call destroy again.
+	context, err := coretesting.RunCommand(c, new(DestroyEnvironmentCommand), "dummy-non-state-server", "--yes")
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(coretesting.Stderr(context), gc.Equals, "environment not found, removing config file\n")
+
+	// Check that the client's cached info has been removed.
+	_, err = s.ConfigStore.ReadInfo("dummy-non-state-server")
+	c.Assert(err, jc.Satisfies, errors.IsNotFound)
+}
+
+func (s *destroyEnvSuite) setupHostedEnviron(c *gc.C, name string) {
+	st := s.Factory.MakeEnvironment(c, &factory.EnvParams{
+		Name:        name,
+		Prepare:     true,
+		ConfigAttrs: coretesting.Attrs{"state-server": false},
+	})
+	defer st.Close()
+
+	ports, err := st.APIHostPorts()
+	c.Assert(err, jc.ErrorIsNil)
+	info := s.ConfigStore.CreateInfo(name)
+	endpoint := configstore.APIEndpoint{
+		CACert:      st.CACert(),
+		EnvironUUID: st.EnvironUUID(),
+		Addresses:   []string{ports[0][0].String()},
+	}
+	info.SetAPIEndpoint(endpoint)
+
+	ssinfo, err := s.ConfigStore.ReadInfo("dummyenv")
+	c.Assert(err, jc.ErrorIsNil)
+	info.SetAPICredentials(ssinfo.APICredentials())
+	err = info.Write()
+	c.Assert(err, jc.ErrorIsNil)
 }
 
 func (s *destroyEnvSuite) TestDestroyEnvironmentCommandBroken(c *gc.C) {
@@ -158,8 +264,8 @@ func (s *destroyEnvSuite) TestDestroyEnvironmentCommandBroken(c *gc.C) {
 	opc, errc := cmdtesting.RunCommand(cmdtesting.NullContext(c), new(DestroyEnvironmentCommand), "dummyenv", "--yes")
 	op, ok := (<-opc).(dummy.OpDestroy)
 	c.Assert(ok, jc.IsTrue)
-	c.Assert(op.Error, gc.ErrorMatches, "dummy.Destroy is broken")
-	c.Check(<-errc, gc.Equals, op.Error)
+	c.Assert(op.Error, gc.ErrorMatches, ".*dummy.Destroy is broken")
+	c.Check(<-errc, gc.ErrorMatches, ".*dummy.Destroy is broken")
 	c.Check(<-opc, gc.IsNil)
 }
 
