@@ -6,11 +6,13 @@ package environment
 import (
 	"github.com/juju/cmd"
 	"github.com/juju/loggo"
+	"github.com/juju/utils/featureflag"
 
 	"github.com/juju/juju/cmd/envcmd"
+	"github.com/juju/juju/feature"
 )
 
-var logger = loggo.GetLogger("juju.cmd.juju.machine")
+var logger = loggo.GetLogger("juju.cmd.juju.environment")
 
 const commandDoc = `
 "juju environment" provides commands to interact with the Juju environment.
@@ -29,5 +31,15 @@ func NewSuperCommand() cmd.Command {
 	environmentCmd.Register(envcmd.Wrap(&SetCommand{}))
 	environmentCmd.Register(envcmd.Wrap(&UnsetCommand{}))
 	environmentCmd.Register(&JenvCommand{})
+	environmentCmd.Register(envcmd.Wrap(&RetryProvisioningCommand{}))
+	environmentCmd.Register(envcmd.Wrap(&EnvSetConstraintsCommand{}))
+	environmentCmd.Register(envcmd.Wrap(&EnvGetConstraintsCommand{}))
+
+	if featureflag.Enabled(feature.JES) {
+		environmentCmd.Register(envcmd.Wrap(&ShareCommand{}))
+		environmentCmd.Register(envcmd.Wrap(&UnshareCommand{}))
+		environmentCmd.Register(envcmd.Wrap(&CreateCommand{}))
+		environmentCmd.Register(envcmd.Wrap(&UsersCommand{}))
+	}
 	return environmentCmd
 }
