@@ -143,6 +143,17 @@ References:
   http://juju.ubuntu.com/docs/provider-configuration-openstack.html
   http://askubuntu.com/questions/132411/how-can-i-configure-juju-for-deployment-on-openstack
 
+Placement directives:
+
+  OpenStack environments support the following placement directives for use
+  with "juju bootstrap" and "juju add-machine":
+
+    zone=<availability-zone-name>
+      The "zone" placement directive instructs the OpenStack provider to
+      allocate a machine in the specified availability zone. If the zone
+      does not exist, or a machine cannot be allocated within it, then
+      the machine addition will fail.
+
 Other OpenStack Based Clouds:
 
 This answer is for generic OpenStack support, if you're using an OpenStack-based
@@ -175,6 +186,17 @@ key.
 
 And that's it, you're ready to go!
 
+Placement directives:
+
+  EC2 environments support the following placement directives for use with
+  "juju bootstrap" and "juju add-machine":
+
+    zone=<availability-zone-name>
+      The "zone" placement directive instructs the EC2 provider to
+      allocate a machine in the specified availability zone. If the zone
+      does not exist, or a machine cannot be allocated within it, then
+      the machine addition will fail.
+
 References:
 
   [1]: http://askubuntu.com/questions/730/how-do-i-set-environment-variables
@@ -227,10 +249,17 @@ A generic Windows Azure environment looks like this:
     # Most useful for developers.
     # force-image-name: b39f27a8b8c64d52b05eac6a62ebad85__Ubuntu-13_10-amd64-server-DEVELOPMENT-20130713-Juju_ALPHA-en-us-30GB
 
-    # Pick a simplestreams stream to select OS images from: daily or released
-    # images, or any other stream available on simplestreams.  Leave blank for
-    # released images.
-    # image-stream: ""
+    # image-stream chooses a simplestreams stream from which to select
+    # OS images, for example daily or released images (or any other stream
+    # available on simplestreams).
+    #
+    # image-stream: "released"
+
+    # agent-stream chooses a simplestreams stream from which to select tools,
+    # for example released or proposed tools (or any other stream available
+    # on simplestreams).
+    #
+    # agent-stream: "released"
 
 This is the environments.yaml configuration file needed to run on Windows Azure.
 You will need to set the management-subscription-id, management-certificate-
@@ -244,6 +273,37 @@ See the online help for more information:
   https://juju.ubuntu.com/docs/config-azure.html
 `
 
+const helpMAASProvider = `
+A generic MAAS environment looks like this:
+
+  sample_maas:
+    type: maas
+    maas-server: 'http://<my-maas-server>:80/MAAS'
+    maas-oauth: 'MAAS-API-KEY'
+
+The API key can be obtained from the preferences page in the MAAS web UI.
+
+Placement directives:
+
+  MAAS environments support the following placement directives for use with
+  "juju bootstrap" and "juju add-machine":
+
+    zone=<physical-zone-name>
+      The "zone" placement directive instructs the MAAS provider to
+      allocate a machine in the specified availability zone. If the zone
+      does not exist, or a machine cannot be allocated within it, then
+      the machine addition will fail.
+
+    <hostname>
+      If the placement directive does not contain an "=" symbol, then
+      it is assumed to be the hostname of a node in MAAS. MAAS will attempt
+      to acquire that node and will fail if it cannot.
+
+See the online help for more information:
+
+  https://juju.ubuntu.com/docs/config-maas.html
+`
+
 const helpConstraints = `
 Constraints constrain the possible instances that may be started by juju
 commands. They are usually passed as a flag to commands that provision a new
@@ -321,8 +381,9 @@ cpu-power
 
 tags
    Tags defines the list of tags that the machine must have applied to it.
-   Multiple tags must be delimited by a comma. Tags are currently only supported
-   by the MaaS environment.
+   Multiple tags must be delimited by a comma. Both positive and negative
+   tags constraints are supported, the latter have a "^" prefix. Tags are
+   currently only supported by the MaaS environment.
 
 networks
    Networks defines the list of networks to ensure are available or not on the
@@ -333,9 +394,15 @@ networks
    network. Positive network constraints do not imply the networks will be enabled,
    use the --networks argument for that, just that they could be enabled.
 
+instance-type
+   Instance-type is the provider-specific name of a type of machine to deploy,
+   for example m1.small on EC2 or A4 on Azure.  Specifying this constraint may
+   conflict with other constraints depending on the provider (since the instance
+   type my determine things like memory size etc.)
+
 Example:
 
-   juju add-machine --constraints "arch=amd64 mem=8G tags=foo,bar"
+   juju add-machine --constraints "arch=amd64 mem=8G tags=foo,^bar"
 
 See Also:
    juju help set-constraints
@@ -346,6 +413,28 @@ See Also:
    juju help bootstrap
 `
 
+const helpPlacement = `
+Placement directives provide users with a means of providing instruction
+to the cloud provider on how to allocate a machine. For example, the MAAS
+provider can be directed to acquire a particular node by specifying its
+hostname.
+
+See provider-specific documentation for details of placement directives
+supported by that provider.
+
+Examples:
+
+  # Bootstrap using an instance in the "us-east-1a" EC2 availability zone.
+  juju bootstrap --to zone=us-east-1a
+
+  # Acquire the node "host01.maas" and add it to Juju.
+  juju add-machine host01.maas
+
+See also:
+  juju help add-machine
+  juju help bootstrap
+`
+
 const helpGlossary = `
 Bootstrap
   To boostrap an environment means initializing it so that Services may be
@@ -399,6 +488,10 @@ Machine Agent
   able to handle the needs of deploying and managing Service Units in this
   machine.
 
+Placement Directive
+  A provider-specific string that directs the provisioner on how to allocate a
+  machine instance.
+
 Provisioning Agent
   Software responsible for automatically allocating and terminating machines in
   an Environment, as necessary for the requested configuration.
