@@ -17,15 +17,18 @@ import (
 	"github.com/juju/juju/cmd/juju/backups"
 	"github.com/juju/juju/cmd/juju/block"
 	"github.com/juju/juju/cmd/juju/cachedimages"
+	"github.com/juju/juju/cmd/juju/common"
 	"github.com/juju/juju/cmd/juju/environment"
 	"github.com/juju/juju/cmd/juju/machine"
+	"github.com/juju/juju/cmd/juju/service"
+	"github.com/juju/juju/cmd/juju/storage"
 	"github.com/juju/juju/cmd/juju/user"
 	"github.com/juju/juju/environs"
 	"github.com/juju/juju/juju"
 	"github.com/juju/juju/juju/osenv"
-	"github.com/juju/juju/version"
 	// Import the providers.
 	_ "github.com/juju/juju/provider/all"
+	"github.com/juju/juju/version"
 )
 
 func init() {
@@ -101,6 +104,7 @@ func NewJujuCommand(ctx *cmd.Context) cmd.Command {
 type commandRegistry interface {
 	Register(cmd.Command)
 	RegisterSuperAlias(name, super, forName string, check cmd.DeprecationCheck)
+	RegisterDeprecated(subcmd cmd.Command, check cmd.DeprecationCheck)
 }
 
 // registerCommands registers commands in the specified registry.
@@ -114,7 +118,6 @@ func registerCommands(r commandRegistry, ctx *cmd.Context) {
 	r.Register(wrapEnvCommand(&BootstrapCommand{}))
 	r.Register(wrapEnvCommand(&DeployCommand{}))
 	r.Register(wrapEnvCommand(&AddRelationCommand{}))
-	r.Register(wrapEnvCommand(&AddUnitCommand{}))
 
 	// Destruction commands.
 	r.Register(wrapEnvCommand(&RemoveRelationCommand{}))
@@ -127,6 +130,7 @@ func registerCommands(r commandRegistry, ctx *cmd.Context) {
 	r.Register(&SwitchCommand{})
 	r.Register(wrapEnvCommand(&EndpointCommand{}))
 	r.Register(wrapEnvCommand(&APIInfoCommand{}))
+	r.Register(wrapEnvCommand(&StatusHistoryCommand{}))
 
 	// Error resolution and debugging commands.
 	r.Register(wrapEnvCommand(&RunCommand{}))
@@ -135,15 +139,13 @@ func registerCommands(r commandRegistry, ctx *cmd.Context) {
 	r.Register(wrapEnvCommand(&ResolvedCommand{}))
 	r.Register(wrapEnvCommand(&DebugLogCommand{}))
 	r.Register(wrapEnvCommand(&DebugHooksCommand{}))
-	r.Register(wrapEnvCommand(&RetryProvisioningCommand{}))
 
 	// Configuration commands.
 	r.Register(&InitCommand{})
-	r.Register(wrapEnvCommand(&GetCommand{}))
-	r.Register(wrapEnvCommand(&SetCommand{}))
-	r.Register(wrapEnvCommand(&UnsetCommand{}))
-	r.Register(wrapEnvCommand(&GetConstraintsCommand{}))
-	r.Register(wrapEnvCommand(&SetConstraintsCommand{}))
+	r.RegisterDeprecated(wrapEnvCommand(&common.GetConstraintsCommand{}),
+		twoDotOhDeprecation("environment get-constraints or service get-constraints"))
+	r.RegisterDeprecated(wrapEnvCommand(&common.SetConstraintsCommand{}),
+		twoDotOhDeprecation("environment set-constraints or service set-constraints"))
 	r.Register(wrapEnvCommand(&ExposeCommand{}))
 	r.Register(wrapEnvCommand(&SyncToolsCommand{}))
 	r.Register(wrapEnvCommand(&UnexposeCommand{}))
@@ -183,18 +185,27 @@ func registerCommands(r commandRegistry, ctx *cmd.Context) {
 	r.RegisterSuperAlias("set-env", "environment", "set", twoDotOhDeprecation("environment set"))
 	r.RegisterSuperAlias("unset-environment", "environment", "unset", twoDotOhDeprecation("environment unset"))
 	r.RegisterSuperAlias("unset-env", "environment", "unset", twoDotOhDeprecation("environment unset"))
+	r.RegisterSuperAlias("retry-provisioning", "environment", "retry-provisioning", twoDotOhDeprecation("environment retry-provisioning"))
 
-	// Manage and control actions.
-	if featureflag.Enabled(action.FeatureFlag) {
-		r.Register(action.NewSuperCommand())
-	}
+	// Manage and control actions
+	r.Register(action.NewSuperCommand())
 
-	// Manage state server availability.
+	// Manage state server availability
 	r.Register(wrapEnvCommand(&EnsureAvailabilityCommand{}))
 
+	// Manage and control services
+	r.Register(service.NewSuperCommand())
+	r.RegisterSuperAlias("add-unit", "service", "add-unit", twoDotOhDeprecation("service add-unit"))
+	r.RegisterSuperAlias("get", "service", "get", twoDotOhDeprecation("service get"))
+	r.RegisterSuperAlias("set", "service", "set", twoDotOhDeprecation("service set"))
+	r.RegisterSuperAlias("unset", "service", "unset", twoDotOhDeprecation("service unset"))
+
 	// Operation protection commands
-	r.Register(wrapEnvCommand(&block.BlockCommand{}))
+	r.Register(block.NewSuperBlockCommand())
 	r.Register(wrapEnvCommand(&block.UnblockCommand{}))
+
+	// Manage storage
+	r.Register(storage.NewSuperCommand())
 }
 
 // envCmdWrapper is a struct that wraps an environment command and lets us handle
