@@ -1,19 +1,19 @@
 // Copyright 2014 Canonical Ltd.
 // Licensed under the AGPLv3, see LICENCE file for details.
 
+package main
+
 // TODO(dimitern): bug http://pad.lv/1425569
 // Disabled until we have time to fix these tests on i386 properly.
 //
 // +build !386
 
-package main
-
 import (
 	"flag"
 	stdtesting "testing"
 
 	cmdtesting "github.com/juju/juju/cmd/testing"
-	_ "github.com/juju/juju/provider/dummy" // XXX Why?
+	_ "github.com/juju/juju/provider/dummy"
 	"github.com/juju/juju/testing"
 )
 
