@@ -19,11 +19,22 @@ type RemoveUnitCommand struct {
 	UnitNames []string
 }
 
+const removeUnitDoc = `
+Remove service units from the environment.
+
+If this is the only unit running, the machine on which
+the unit is hosted will also be destroyed, if possible.
+The machine will be destroyed if:
+- it is not a state server
+- it is not hosting any Juju managed containers
+`
+
 func (c *RemoveUnitCommand) Info() *cmd.Info {
 	return &cmd.Info{
 		Name:    "remove-unit",
 		Args:    "<unit> [...]",
 		Purpose: "remove service units from the environment",
+		Doc:     removeUnitDoc,
 		Aliases: []string{"destroy-unit"},
 	}
 }
