@@ -52,7 +52,7 @@ func (c *SSHCommon) setProxyCommand(options *ssh.Options) error {
 	if err != nil {
 		return fmt.Errorf("failed to get juju executable path: %v", err)
 	}
-	options.SetProxyCommand(juju, "ssh", "--proxy=false", "--pty=false", apiServerHost, "nc", "-q0", "%h", "%p")
+	options.SetProxyCommand(juju, "ssh", "--proxy=false", "--pty=false", apiServerHost, "nc", "%h", "%p")
 	return nil
 }
 
