@@ -6,16 +6,18 @@ package main
 import (
 	"encoding/json"
 	"fmt"
+	"os"
+	"strconv"
 
 	"github.com/juju/cmd"
-	"launchpad.net/gnuflag"
-
 	"github.com/juju/errors"
+	"launchpad.net/gnuflag"
 
 	"github.com/juju/juju/api"
 	"github.com/juju/juju/apiserver/params"
 	"github.com/juju/juju/cmd/envcmd"
 	"github.com/juju/juju/instance"
+	"github.com/juju/juju/juju/osenv"
 	"github.com/juju/juju/network"
 	"github.com/juju/juju/state/multiwatcher"
 )
@@ -24,6 +26,7 @@ type StatusCommand struct {
 	envcmd.EnvCommandBase
 	out      cmd.Output
 	patterns []string
+	isoTime  bool
 }
 
 var statusDoc = `
@@ -68,7 +71,13 @@ func (c *StatusCommand) Info() *cmd.Info {
 }
 
 func (c *StatusCommand) SetFlags(f *gnuflag.FlagSet) {
-	c.out.AddFlags(f, "yaml", map[string]cmd.Formatter{
+	f.BoolVar(&c.isoTime, "utc", false, "display time as UTC in RFC3339 format")
+
+	defaultFormat := "yaml"
+	if c.CompatVersion() > 1 {
+		defaultFormat = "tabular"
+	}
+	c.out.AddFlags(f, defaultFormat, map[string]cmd.Formatter{
 		"yaml":    cmd.FormatYaml,
 		"json":    cmd.FormatJson,
 		"short":   FormatOneline,
@@ -81,6 +90,17 @@ func (c *StatusCommand) SetFlags(f *gnuflag.FlagSet) {
 
 func (c *StatusCommand) Init(args []string) error {
 	c.patterns = args
+	// If use of ISO time not specified on command line,
+	// check env var.
+	if !c.isoTime {
+		var err error
+		envVarValue := os.Getenv(osenv.JujuStatusIsoTimeEnvKey)
+		if envVarValue != "" {
+			if c.isoTime, err = strconv.ParseBool(envVarValue); err != nil {
+				return errors.Annotatef(err, "invalid %s env var, expected true|false", osenv.JujuStatusIsoTimeEnvKey)
+			}
+		}
+	}
 	return nil
 }
 
@@ -120,15 +140,16 @@ func (c *StatusCommand) Run(ctx *cmd.Context) error {
 		return errors.Errorf("unable to obtain the current status")
 	}
 
-	result := newStatusFormatter(status).format()
+	result := newStatusFormatter(status, c.CompatVersion(), c.isoTime).format()
 	return c.out.Write(ctx, result)
 }
 
 type formattedStatus struct {
-	Environment string                   `json:"environment"`
-	Machines    map[string]machineStatus `json:"machines"`
-	Services    map[string]serviceStatus `json:"services"`
-	Networks    map[string]networkStatus `json:"networks,omitempty" yaml:",omitempty"`
+	Environment      string                   `json:"environment"`
+	AvailableVersion string                   `json:"available-version,omitempty" yaml:"available-version,omitempty"`
+	Machines         map[string]machineStatus `json:"machines"`
+	Services         map[string]serviceStatus `json:"services"`
+	Networks         map[string]networkStatus `json:"networks,omitempty" yaml:",omitempty"`
 }
 
 type errorStatus struct {
@@ -168,7 +189,8 @@ func (s machineStatus) GetYAML() (tag string, value interface{}) {
 	}
 	// TODO(rog) rename mNoMethods to noMethods (and also in
 	// the other GetYAML methods) when people are using the non-buggy
-	// goyaml version.
+	// goyaml version. // TODO(jw4) however verify that gccgo does not
+	// complain about symbol already defined.
 	type mNoMethods machineStatus
 	return "", mNoMethods(s)
 }
@@ -179,6 +201,7 @@ type serviceStatus struct {
 	CanUpgradeTo  string                `json:"can-upgrade-to,omitempty" yaml:"can-upgrade-to,omitempty"`
 	Exposed       bool                  `json:"exposed" yaml:"exposed"`
 	Life          string                `json:"life,omitempty" yaml:"life,omitempty"`
+	StatusInfo    statusInfoContents    `json:"service-status,omitempty" yaml:"service-status,omitempty"`
 	Relations     map[string][]string   `json:"relations,omitempty" yaml:"relations,omitempty"`
 	Networks      map[string][]string   `json:"networks,omitempty" yaml:"networks,omitempty"`
 	SubordinateTo []string              `json:"subordinate-to,omitempty" yaml:"subordinate-to,omitempty"`
@@ -191,29 +214,60 @@ func (s serviceStatus) MarshalJSON() ([]byte, error) {
 	if s.Err != nil {
 		return json.Marshal(errorStatus{s.Err.Error()})
 	}
-	type sNoMethods serviceStatus
-	return json.Marshal(sNoMethods(s))
+	type ssNoMethods serviceStatus
+	return json.Marshal(ssNoMethods(s))
 }
 
 func (s serviceStatus) GetYAML() (tag string, value interface{}) {
 	if s.Err != nil {
 		return "", errorStatus{s.Err.Error()}
 	}
-	type sNoMethods serviceStatus
-	return "", sNoMethods(s)
+	type ssNoMethods serviceStatus
+	return "", ssNoMethods(s)
 }
 
 type unitStatus struct {
-	Err            error                 `json:"-" yaml:",omitempty"`
-	Charm          string                `json:"upgrading-from,omitempty" yaml:"upgrading-from,omitempty"`
-	AgentState     params.Status         `json:"agent-state,omitempty" yaml:"agent-state,omitempty"`
-	AgentStateInfo string                `json:"agent-state-info,omitempty" yaml:"agent-state-info,omitempty"`
-	AgentVersion   string                `json:"agent-version,omitempty" yaml:"agent-version,omitempty"`
-	Life           string                `json:"life,omitempty" yaml:"life,omitempty"`
-	Machine        string                `json:"machine,omitempty" yaml:"machine,omitempty"`
-	OpenedPorts    []string              `json:"open-ports,omitempty" yaml:"open-ports,omitempty"`
-	PublicAddress  string                `json:"public-address,omitempty" yaml:"public-address,omitempty"`
-	Subordinates   map[string]unitStatus `json:"subordinates,omitempty" yaml:"subordinates,omitempty"`
+	// New Juju Health Status fields.
+	WorkloadStatusInfo statusInfoContents `json:"workload-status,omitempty" yaml:"workload-status,omitempty"`
+	AgentStatusInfo    statusInfoContents `json:"agent-status,omitempty" yaml:"agent-status,omitempty"`
+
+	// Legacy status fields, to be removed in Juju 2.0
+	AgentState     params.Status `json:"agent-state,omitempty" yaml:"agent-state,omitempty"`
+	AgentStateInfo string        `json:"agent-state-info,omitempty" yaml:"agent-state-info,omitempty"`
+	Err            error         `json:"-" yaml:",omitempty"`
+	AgentVersion   string        `json:"agent-version,omitempty" yaml:"agent-version,omitempty"`
+	Life           string        `json:"life,omitempty" yaml:"life,omitempty"`
+
+	Charm         string                `json:"upgrading-from,omitempty" yaml:"upgrading-from,omitempty"`
+	Machine       string                `json:"machine,omitempty" yaml:"machine,omitempty"`
+	OpenedPorts   []string              `json:"open-ports,omitempty" yaml:"open-ports,omitempty"`
+	PublicAddress string                `json:"public-address,omitempty" yaml:"public-address,omitempty"`
+	Subordinates  map[string]unitStatus `json:"subordinates,omitempty" yaml:"subordinates,omitempty"`
+}
+
+type statusInfoContents struct {
+	Err     error         `json:"-" yaml:",omitempty"`
+	Current params.Status `json:"current,omitempty" yaml:"current,omitempty"`
+	Message string        `json:"message,omitempty" yaml:"message,omitempty"`
+	Since   string        `json:"since,omitempty" yaml:"since,omitempty"`
+	Version string        `json:"version,omitempty" yaml:"version,omitempty"`
+}
+
+type statusInfoContentsNoMarshal statusInfoContents
+
+func (s statusInfoContents) MarshalJSON() ([]byte, error) {
+	if s.Err != nil {
+		return json.Marshal(errorStatus{s.Err.Error()})
+	}
+	return json.Marshal(statusInfoContentsNoMarshal(s))
+}
+
+func (s statusInfoContents) GetYAML() (tag string, value interface{}) {
+	if s.Err != nil {
+		return "", errorStatus{s.Err.Error()}
+	}
+	type sicNoMethods statusInfoContents
+	return "", sicNoMethods(s)
 }
 
 type unitStatusNoMarshal unitStatus
@@ -229,8 +283,8 @@ func (s unitStatus) GetYAML() (tag string, value interface{}) {
 	if s.Err != nil {
 		return "", errorStatus{s.Err.Error()}
 	}
-	type uNoMethods unitStatus
-	return "", unitStatusNoMarshal(s)
+	type usNoMethods unitStatus
+	return "", usNoMethods(s)
 }
 
 type networkStatus struct {
@@ -259,14 +313,18 @@ func (n networkStatus) GetYAML() (tag string, value interface{}) {
 }
 
 type statusFormatter struct {
-	status    *api.Status
-	relations map[int]api.RelationStatus
+	status        *api.Status
+	relations     map[int]api.RelationStatus
+	isoTime       bool
+	compatVersion int
 }
 
-func newStatusFormatter(status *api.Status) *statusFormatter {
+func newStatusFormatter(status *api.Status, compatVersion int, isoTime bool) *statusFormatter {
 	sf := statusFormatter{
-		status:    status,
-		relations: make(map[int]api.RelationStatus),
+		status:        status,
+		relations:     make(map[int]api.RelationStatus),
+		compatVersion: compatVersion,
+		isoTime:       isoTime,
 	}
 	for _, relation := range status.Relations {
 		sf.relations[relation.Id] = relation
@@ -279,9 +337,10 @@ func (sf *statusFormatter) format() formattedStatus {
 		return formattedStatus{}
 	}
 	out := formattedStatus{
-		Environment: sf.status.EnvironmentName,
-		Machines:    make(map[string]machineStatus),
-		Services:    make(map[string]serviceStatus),
+		Environment:      sf.status.EnvironmentName,
+		AvailableVersion: sf.status.AvailableVersion,
+		Machines:         make(map[string]machineStatus),
+		Services:         make(map[string]serviceStatus),
 	}
 	for k, m := range sf.status.Machines {
 		out.Machines[k] = sf.formatMachine(m)
@@ -323,7 +382,7 @@ func (sf *statusFormatter) formatMachine(machine api.MachineStatus) machineStatu
 		agent := machine.Agent
 		out = machineStatus{
 			AgentState:     machine.AgentState,
-			AgentStateInfo: adjustInfoIfAgentDown(machine.AgentState, agent.Status, agent.Info),
+			AgentStateInfo: adjustInfoIfMachineAgentDown(machine.AgentState, agent.Status, agent.Info),
 			AgentVersion:   agent.Version,
 			Life:           agent.Life,
 			Err:            agent.Err,
@@ -361,6 +420,7 @@ func (sf *statusFormatter) formatService(name string, service api.ServiceStatus)
 		CanUpgradeTo:  service.CanUpgradeTo,
 		SubordinateTo: service.SubordinateTo,
 		Units:         make(map[string]unitStatus),
+		StatusInfo:    sf.getServiceStatusInfo(service),
 	}
 	if len(service.Networks.Enabled) > 0 {
 		out.Networks["enabled"] = service.Networks.Enabled
@@ -374,41 +434,91 @@ func (sf *statusFormatter) formatService(name string, service api.ServiceStatus)
 	return out
 }
 
-func (sf *statusFormatter) formatUnit(unit api.UnitStatus, serviceName string) unitStatus {
-	out := unitStatus{
-		Err:            unit.Err,
-		AgentState:     unit.AgentState,
-		AgentStateInfo: sf.getUnitStatusInfo(unit, serviceName),
-		AgentVersion:   unit.AgentVersion,
-		Life:           unit.Life,
-		Machine:        unit.Machine,
-		OpenedPorts:    unit.OpenedPorts,
-		PublicAddress:  unit.PublicAddress,
-		Charm:          unit.Charm,
-		Subordinates:   make(map[string]unitStatus),
+func (sf *statusFormatter) getServiceStatusInfo(service api.ServiceStatus) statusInfoContents {
+	info := statusInfoContents{
+		Err:     service.Status.Err,
+		Current: service.Status.Status,
+		Message: service.Status.Info,
+		Version: service.Status.Version,
 	}
+	if service.Status.Since != nil {
+		info.Since = formatStatusTime(service.Status.Since, sf.isoTime)
+	}
+	return info
+}
+
+func (sf *statusFormatter) formatUnit(unit api.UnitStatus, serviceName string) unitStatus {
+	// TODO(Wallyworld) - this should be server side but we still need to support older servers.
+	sf.updateUnitStatusInfo(&unit, serviceName)
+
+	out := unitStatus{
+		WorkloadStatusInfo: sf.getWorkloadStatusInfo(unit),
+		AgentStatusInfo:    sf.getAgentStatusInfo(unit),
+		Machine:            unit.Machine,
+		OpenedPorts:        unit.OpenedPorts,
+		PublicAddress:      unit.PublicAddress,
+		Charm:              unit.Charm,
+		Subordinates:       make(map[string]unitStatus),
+	}
+
+	// These legacy fields will be dropped for Juju 2.0.
+	if sf.compatVersion < 2 || out.AgentStatusInfo.Current == "" {
+		out.Err = unit.Err
+		out.AgentState = unit.AgentState
+		out.AgentStateInfo = unit.AgentStateInfo
+		out.Life = unit.Life
+		out.AgentVersion = unit.AgentVersion
+	}
+
 	for k, m := range unit.Subordinates {
 		out.Subordinates[k] = sf.formatUnit(m, serviceName)
 	}
 	return out
 }
 
-func (sf *statusFormatter) getUnitStatusInfo(unit api.UnitStatus, serviceName string) string {
-	if unit.Agent.Status == "" {
-		// Old server that doesn't support this field and others.
-		// Just return the info string as-is.
-		return unit.AgentStateInfo
+func (sf *statusFormatter) getWorkloadStatusInfo(unit api.UnitStatus) statusInfoContents {
+	info := statusInfoContents{
+		Err:     unit.Workload.Err,
+		Current: unit.Workload.Status,
+		Message: unit.Workload.Info,
+		Version: unit.Workload.Version,
 	}
-	statusInfo := unit.Agent.Info
-	if unit.Agent.Status == params.StatusError {
+	if unit.Workload.Since != nil {
+		info.Since = formatStatusTime(unit.Workload.Since, sf.isoTime)
+	}
+	return info
+}
+
+func (sf *statusFormatter) getAgentStatusInfo(unit api.UnitStatus) statusInfoContents {
+	info := statusInfoContents{
+		Err:     unit.UnitAgent.Err,
+		Current: unit.UnitAgent.Status,
+		Message: unit.UnitAgent.Info,
+		Version: unit.UnitAgent.Version,
+	}
+	if unit.UnitAgent.Since != nil {
+		info.Since = formatStatusTime(unit.UnitAgent.Since, sf.isoTime)
+	}
+	return info
+}
+
+func (sf *statusFormatter) updateUnitStatusInfo(unit *api.UnitStatus, serviceName string) {
+	// This logic has no business here but can't be moved until Juju 2.0.
+	statusInfo := unit.Workload.Info
+	if unit.Workload.Status == "" {
+		// Old server that doesn't support this field and others.
+		// Just use the info string as-is.
+		statusInfo = unit.AgentStateInfo
+	}
+	if unit.Workload.Status == params.StatusError {
 		if relation, ok := sf.relations[getRelationIdFromData(unit)]; ok {
 			// Append the details of the other endpoint on to the status info string.
 			if ep, ok := findOtherEndpoint(relation.Endpoints, serviceName); ok {
-				statusInfo = statusInfo + " for " + ep.String()
+				unit.Workload.Info = statusInfo + " for " + ep.String()
+				unit.AgentStateInfo = unit.Workload.Info
 			}
 		}
 	}
-	return adjustInfoIfAgentDown(unit.AgentState, unit.Agent.Status, statusInfo)
 }
 
 func (sf *statusFormatter) formatNetwork(network api.NetworkStatus) networkStatus {
@@ -435,8 +545,8 @@ func makeHAStatus(hasVote, wantsVote bool) string {
 	return s
 }
 
-func getRelationIdFromData(unit api.UnitStatus) int {
-	if relationId_, ok := unit.Agent.Data["relation-id"]; ok {
+func getRelationIdFromData(unit *api.UnitStatus) int {
+	if relationId_, ok := unit.Workload.Data["relation-id"]; ok {
 		if relationId, ok := relationId_.(float64); ok {
 			return int(relationId)
 		} else {
@@ -459,11 +569,11 @@ func findOtherEndpoint(endpoints []api.EndpointStatus, serviceName string) (api.
 	return api.EndpointStatus{}, false
 }
 
-// adjustInfoIfAgentDown modifies the agent status info string if the
+// adjustInfoIfMachineAgentDown modifies the agent status info string if the
 // agent is down. The original status and info is included in
 // parentheses.
-func adjustInfoIfAgentDown(status, origStatus params.Status, info string) string {
-	if status == params.StatusDown || status == params.StatusFailed {
+func adjustInfoIfMachineAgentDown(status, origStatus params.Status, info string) string {
+	if status == params.StatusDown {
 		if info == "" {
 			return fmt.Sprintf("(%s)", origStatus)
 		}
