@@ -9,7 +9,7 @@ import (
 	"github.com/juju/cmd"
 
 	"github.com/juju/juju/cmd/envcmd"
-	"github.com/juju/juju/juju"
+	"github.com/juju/juju/cmd/juju/block"
 )
 
 // UnexposeCommand is responsible exposing services.
@@ -37,10 +37,10 @@ func (c *UnexposeCommand) Init(args []string) error {
 // Run changes the juju-managed firewall to hide any
 // ports that were also explicitly marked by units as closed.
 func (c *UnexposeCommand) Run(_ *cmd.Context) error {
-	client, err := juju.NewAPIClientFromName(c.EnvName)
+	client, err := c.NewAPIClient()
 	if err != nil {
 		return err
 	}
 	defer client.Close()
-	return client.ServiceUnexpose(c.ServiceName)
+	return block.ProcessBlockedError(client.ServiceUnexpose(c.ServiceName), block.BlockChange)
 }
