@@ -5,12 +5,13 @@ package agent
 
 import (
 	"fmt"
+	"io"
 	"io/ioutil"
 	"os"
 	"path/filepath"
 	"reflect"
+	"runtime"
 	"strings"
-	"sync"
 	"testing"
 	"time"
 
@@ -23,13 +24,16 @@ import (
 	"github.com/juju/utils/set"
 	"github.com/juju/utils/symlink"
 	gc "gopkg.in/check.v1"
-	"gopkg.in/juju/charm.v4"
+	"gopkg.in/juju/charm.v5"
+	"gopkg.in/juju/charm.v5/charmrepo"
+	"gopkg.in/natefinch/lumberjack.v2"
 
 	"github.com/juju/juju/agent"
 	"github.com/juju/juju/api"
 	apideployer "github.com/juju/juju/api/deployer"
 	apienvironment "github.com/juju/juju/api/environment"
 	apifirewaller "github.com/juju/juju/api/firewaller"
+	apimachiner "github.com/juju/juju/api/machiner"
 	apimetricsmanager "github.com/juju/juju/api/metricsmanager"
 	apinetworker "github.com/juju/juju/api/networker"
 	apirsyslog "github.com/juju/juju/api/rsyslog"
@@ -41,6 +45,7 @@ import (
 	lxctesting "github.com/juju/juju/container/lxc/testing"
 	"github.com/juju/juju/environs/config"
 	envtesting "github.com/juju/juju/environs/testing"
+	"github.com/juju/juju/feature"
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/juju"
 	jujutesting "github.com/juju/juju/juju/testing"
@@ -50,21 +55,27 @@ import (
 	"github.com/juju/juju/service/upstart"
 	"github.com/juju/juju/state"
 	"github.com/juju/juju/state/watcher"
+	"github.com/juju/juju/storage"
 	coretesting "github.com/juju/juju/testing"
+	"github.com/juju/juju/testing/factory"
 	"github.com/juju/juju/tools"
 	"github.com/juju/juju/utils/ssh"
 	sshtesting "github.com/juju/juju/utils/ssh/testing"
 	"github.com/juju/juju/version"
 	"github.com/juju/juju/worker"
+	"github.com/juju/juju/worker/apiaddressupdater"
 	"github.com/juju/juju/worker/authenticationworker"
 	"github.com/juju/juju/worker/certupdater"
 	"github.com/juju/juju/worker/deployer"
+	"github.com/juju/juju/worker/diskmanager"
 	"github.com/juju/juju/worker/instancepoller"
+	"github.com/juju/juju/worker/machiner"
 	"github.com/juju/juju/worker/networker"
 	"github.com/juju/juju/worker/peergrouper"
 	"github.com/juju/juju/worker/proxyupdater"
 	"github.com/juju/juju/worker/rsyslog"
 	"github.com/juju/juju/worker/singular"
+	"github.com/juju/juju/worker/storageprovisioner"
 	"github.com/juju/juju/worker/upgrader"
 )
 
@@ -75,12 +86,6 @@ var (
 )
 
 func TestPackage(t *testing.T) {
-	// Change the default init dir in worker/deployer,
-	// so the deployer doesn't try to remove upstart
-	// jobs from tests.
-	restore := gitjujutesting.PatchValue(&deployer.InitDir, mkdtemp("juju-worker-deployer"))
-	defer restore()
-
 	// TODO(waigani) 2014-03-19 bug 1294458
 	// Refactor to use base suites
 
@@ -95,7 +100,7 @@ func TestPackage(t *testing.T) {
 type commonMachineSuite struct {
 	singularRecord *singularRunnerRecord
 	lxctesting.TestSuite
-	fakeEnsureMongo agenttesting.FakeEnsure
+	fakeEnsureMongo *agenttesting.FakeEnsureMongo
 	AgentSuite
 }
 
@@ -112,8 +117,8 @@ func (s *commonMachineSuite) TearDownSuite(c *gc.C) {
 func (s *commonMachineSuite) SetUpTest(c *gc.C) {
 	s.AgentSuite.SetUpTest(c)
 	s.TestSuite.SetUpTest(c)
-	s.AgentSuite.PatchValue(&charm.CacheDir, c.MkDir())
-	s.AgentSuite.PatchValue(&stateWorkerDialOpts, mongo.DialOpts{})
+	s.AgentSuite.PatchValue(&charmrepo.CacheDir, c.MkDir())
+	s.AgentSuite.PatchValue(&stateWorkerDialOpts, mongo.DefaultDialOpts())
 
 	os.Remove(JujuRun) // ignore error; may not exist
 	// Patch ssh user to avoid touching ~ubuntu/.ssh/authorized_keys.
@@ -127,15 +132,14 @@ func (s *commonMachineSuite) SetUpTest(c *gc.C) {
 
 	s.AgentSuite.PatchValue(&upstart.InitDir, c.MkDir())
 
-	s.singularRecord = &singularRunnerRecord{startedWorkers: make(set.Strings)}
+	s.singularRecord = newSingularRunnerRecord()
 	s.AgentSuite.PatchValue(&newSingularRunner, s.singularRecord.newSingularRunner)
 	s.AgentSuite.PatchValue(&peergrouperNew, func(st *state.State) (worker.Worker, error) {
 		return newDummyWorker(), nil
 	})
 
-	s.fakeEnsureMongo = agenttesting.FakeEnsure{}
-	s.AgentSuite.PatchValue(&cmdutil.EnsureMongoServer, s.fakeEnsureMongo.FakeEnsureMongo)
-	s.AgentSuite.PatchValue(&maybeInitiateMongoServer, s.fakeEnsureMongo.FakeInitiateMongo)
+	s.fakeEnsureMongo = agenttesting.InstallFakeEnsureMongo(s)
+	s.AgentSuite.PatchValue(&maybeInitiateMongoServer, s.fakeEnsureMongo.InitiateMongo)
 }
 
 func fakeCmd(path string) {
@@ -209,16 +213,20 @@ func (s *commonMachineSuite) configureMachine(c *gc.C, machineId string, vers ve
 
 // newAgent returns a new MachineAgent instance
 func (s *commonMachineSuite) newAgent(c *gc.C, m *state.Machine) *MachineAgent {
-	agentConf := AgentConf{DataDir: s.DataDir()}
+	confDir := filepath.Join(s.RootDir, "/etc/juju")
+	s.PatchValue(&agent.DefaultConfDir, confDir)
+
+	agentConf := agentConf{dataDir: s.DataDir()}
 	agentConf.ReadConfig(names.NewMachineTag(m.Id()).String())
 	machineAgentFactory := MachineAgentFactoryFn(&agentConf, &agentConf)
 	return machineAgentFactory(m.Id())
 }
 
 func (s *MachineSuite) TestParseSuccess(c *gc.C) {
-	create := func() (cmd.Command, *AgentConf) {
-		agentConf := AgentConf{DataDir: s.DataDir()}
+	create := func() (cmd.Command, AgentConf) {
+		agentConf := agentConf{dataDir: s.DataDir()}
 		a := NewMachineAgentCmd(
+			nil,
 			MachineAgentFactoryFn(&agentConf, &agentConf),
 			&agentConf,
 			&agentConf,
@@ -236,20 +244,28 @@ type MachineSuite struct {
 	metricAPI *mockMetricAPI
 }
 
-const initialMachinePassword = "machine-password-1234567890"
-
-func (s *MachineSuite) NewMockMetricAPI() *mockMetricAPI {
-	cleanup := make(chan struct{})
-	sender := make(chan struct{})
-	return &mockMetricAPI{cleanup, sender}
+var perEnvSingularWorkers = []string{
+	"cleaner",
+	"minunitsworker",
+	"addresserworker",
+	"environ-provisioner",
+	"charm-revision-updater",
+	"firewaller",
 }
 
+const initialMachinePassword = "machine-password-1234567890"
+
 func (s *MachineSuite) SetUpTest(c *gc.C) {
-	s.metricAPI = s.NewMockMetricAPI()
+	s.commonMachineSuite.SetUpTest(c)
+	s.metricAPI = newMockMetricAPI()
 	s.PatchValue(&getMetricAPI, func(_ *api.State) apimetricsmanager.MetricsManagerClient {
 		return s.metricAPI
 	})
-	s.commonMachineSuite.SetUpTest(c)
+	s.AddCleanup(func(*gc.C) { s.metricAPI.Stop() })
+	// Most of these tests normally finish sub-second on a fast machine.
+	// If any given test hits a minute, we have almost certainly become
+	// wedged, so dump the logs.
+	coretesting.DumpTestLogsAfter(time.Minute, c, s)
 }
 
 func (s *MachineSuite) TestParseNonsense(c *gc.C) {
@@ -257,14 +273,14 @@ func (s *MachineSuite) TestParseNonsense(c *gc.C) {
 		{},
 		{"--machine-id", "-4004"},
 	} {
-		var agentConf AgentConf
+		var agentConf agentConf
 		err := ParseAgentCommand(&machineAgentCmd{agentInitializer: &agentConf}, args)
 		c.Assert(err, gc.ErrorMatches, "--machine-id option must be set, and expects a non-negative integer")
 	}
 }
 
 func (s *MachineSuite) TestParseUnknown(c *gc.C) {
-	var agentConf AgentConf
+	var agentConf agentConf
 	a := &machineAgentCmd{agentInitializer: &agentConf}
 	err := ParseAgentCommand(a, []string{"--machine-id", "42", "blistering barnacles"})
 	c.Assert(err, gc.ErrorMatches, `unrecognized args: \["blistering barnacles"\]`)
@@ -277,6 +293,83 @@ func (s *MachineSuite) TestRunInvalidMachineId(c *gc.C) {
 	c.Assert(err, gc.ErrorMatches, "some error")
 }
 
+type FakeConfig struct {
+	agent.Config
+}
+
+func (FakeConfig) LogDir() string {
+	return filepath.FromSlash("/var/log/juju/")
+}
+
+func (FakeConfig) Tag() names.Tag {
+	return names.NewMachineTag("42")
+}
+
+type FakeAgentConfig struct {
+	AgentConfigWriter
+	apiaddressupdater.APIAddressSetter
+	AgentInitializer
+}
+
+func (FakeAgentConfig) ReadConfig(string) error { return nil }
+
+func (FakeAgentConfig) CurrentConfig() agent.Config {
+	return FakeConfig{}
+}
+
+func (FakeAgentConfig) CheckArgs([]string) error { return nil }
+
+func (s *MachineSuite) TestUseLumberjack(c *gc.C) {
+	ctx, err := cmd.DefaultContext()
+	c.Assert(err, gc.IsNil)
+
+	agentConf := FakeAgentConfig{}
+
+	a := NewMachineAgentCmd(
+		ctx,
+		MachineAgentFactoryFn(agentConf, agentConf),
+		agentConf,
+		agentConf,
+	)
+	// little hack to set the data that Init expects to already be set
+	a.(*machineAgentCmd).machineId = "42"
+
+	err = a.Init(nil)
+	c.Assert(err, gc.IsNil)
+
+	l, ok := ctx.Stderr.(*lumberjack.Logger)
+	c.Assert(ok, jc.IsTrue)
+	c.Check(l.MaxAge, gc.Equals, 0)
+	c.Check(l.MaxBackups, gc.Equals, 2)
+	c.Check(l.Filename, gc.Equals, filepath.FromSlash("/var/log/juju/machine-42.log"))
+	c.Check(l.MaxSize, gc.Equals, 300)
+}
+
+func (s *MachineSuite) TestDontUseLumberjack(c *gc.C) {
+	ctx, err := cmd.DefaultContext()
+	c.Assert(err, gc.IsNil)
+
+	agentConf := FakeAgentConfig{}
+
+	a := NewMachineAgentCmd(
+		ctx,
+		MachineAgentFactoryFn(agentConf, agentConf),
+		agentConf,
+		agentConf,
+	)
+	// little hack to set the data that Init expects to already be set
+	a.(*machineAgentCmd).machineId = "42"
+
+	// set the value that normally gets set by the flag parsing
+	a.(*machineAgentCmd).logToStdErr = true
+
+	err = a.Init(nil)
+	c.Assert(err, gc.IsNil)
+
+	_, ok := ctx.Stderr.(*lumberjack.Logger)
+	c.Assert(ok, jc.IsFalse)
+}
+
 func (s *MachineSuite) TestRunStop(c *gc.C) {
 	m, ac, _ := s.primeAgent(c, version.Current, state.JobHostUnits)
 	a := s.newAgent(c, m)
@@ -287,7 +380,7 @@ func (s *MachineSuite) TestRunStop(c *gc.C) {
 	err := a.Stop()
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(<-done, jc.ErrorIsNil)
-	c.Assert(charm.CacheDir, gc.Equals, filepath.Join(ac.DataDir(), "charmcache"))
+	c.Assert(charmrepo.CacheDir, gc.Equals, filepath.Join(ac.DataDir(), "charmcache"))
 }
 
 func (s *MachineSuite) TestWithDeadMachine(c *gc.C) {
@@ -361,7 +454,7 @@ func (s *MachineSuite) TestHostUnits(c *gc.C) {
 
 	// "start the agent" for u0 to prevent short-circuited remove-on-destroy;
 	// check that it's kept deployed despite being Dying.
-	err = u0.SetStatus(state.StatusActive, "", nil)
+	err = u0.SetAgentStatus(state.StatusIdle, "", nil)
 	c.Assert(err, jc.ErrorIsNil)
 	err = u0.Destroy()
 	c.Assert(err, jc.ErrorIsNil)
@@ -414,10 +507,8 @@ func patchDeployContext(c *gc.C, st *state.State) (*fakeContext, func()) {
 }
 
 func (s *commonMachineSuite) setFakeMachineAddresses(c *gc.C, machine *state.Machine) {
-	addrs := []network.Address{
-		network.NewAddress("0.1.2.3", network.ScopeUnknown),
-	}
-	err := machine.SetAddresses(addrs...)
+	addrs := network.NewAddresses("0.1.2.3")
+	err := machine.SetProviderAddresses(addrs...)
 	c.Assert(err, jc.ErrorIsNil)
 	// Set the addresses in the environ instance as well so that if the instance poller
 	// runs it won't overwrite them.
@@ -445,6 +536,15 @@ func (s *MachineSuite) TestManageEnviron(c *gc.C) {
 		done <- a.Run(nil)
 	}()
 
+	// See state server runners start
+	r0 := s.singularRecord.nextRunner(c)
+	r0.waitForWorker(c, "resumer")
+	r0.waitForWorker(c, "txnpruner")
+
+	r1 := s.singularRecord.nextRunner(c)
+	lastWorker := perEnvSingularWorkers[len(perEnvSingularWorkers)-1]
+	r1.waitForWorker(c, lastWorker)
+
 	// Check that the provisioner and firewaller are alive by doing
 	// a rudimentary check that it responds to state changes.
 
@@ -486,36 +586,37 @@ func (s *MachineSuite) TestManageEnviron(c *gc.C) {
 	case <-time.After(5 * time.Second):
 		c.Fatalf("timed out waiting for agent to terminate")
 	}
-
-	c.Assert(s.singularRecord.started(), jc.DeepEquals, []string{
-		"charm-revision-updater",
-		"cleaner",
-		"environ-provisioner",
-		"firewaller",
-		"minunitsworker",
-		"resumer",
-	})
 }
 
+const startWorkerWait = 250 * time.Millisecond
+
 func (s *MachineSuite) TestManageEnvironDoesNotRunFirewallerWhenModeIsNone(c *gc.C) {
-	started := make(chan struct{}, 1)
+	s.PatchValue(&getFirewallMode, func(*api.State) (string, error) {
+		return config.FwNone, nil
+	})
+	started := make(chan struct{})
 	s.AgentSuite.PatchValue(&newFirewaller, func(st *apifirewaller.State) (worker.Worker, error) {
-		select {
-		case started <- struct{}{}:
-		default:
-		}
+		close(started)
 		return newDummyWorker(), nil
 	})
+
 	m, _, _ := s.primeAgent(c, version.Current, state.JobManageEnviron)
 	a := s.newAgent(c, m)
 	defer a.Stop()
 	go func() {
 		c.Check(a.Run(nil), jc.ErrorIsNil)
 	}()
+
+	// Wait for the worker that starts before the firewaller to start.
+	_ = s.singularRecord.nextRunner(c)
+	r := s.singularRecord.nextRunner(c)
+	r.waitForWorker(c, "charm-revision-updater")
+
+	// Now make sure the firewaller doesn't start.
 	select {
 	case <-started:
 		c.Fatalf("firewaller worker unexpectedly started")
-	case <-time.After(coretesting.ShortWait):
+	case <-time.After(startWorkerWait):
 	}
 }
 
@@ -541,7 +642,7 @@ func (s *MachineSuite) TestManageEnvironRunsInstancePoller(c *gc.C) {
 	m, instId := s.waitProvisioned(c, units[0])
 	insts, err := s.Environ.Instances([]instance.Id{instId})
 	c.Assert(err, jc.ErrorIsNil)
-	addrs := []network.Address{network.NewAddress("1.2.3.4", network.ScopeUnknown)}
+	addrs := network.NewAddresses("1.2.3.4")
 	dummy.SetInstanceAddresses(insts[0], addrs)
 	dummy.SetInstanceStatus(insts[0], "running")
 
@@ -583,6 +684,41 @@ func (s *MachineSuite) TestManageEnvironRunsPeergrouper(c *gc.C) {
 	}
 }
 
+func (s *MachineSuite) TestManageEnvironRunsDbLogPrunerIfFeatureFlagEnabled(c *gc.C) {
+	s.SetFeatureFlags(feature.DbLog)
+
+	m, _, _ := s.primeAgent(c, version.Current, state.JobManageEnviron)
+	a := s.newAgent(c, m)
+	defer func() { c.Check(a.Stop(), jc.ErrorIsNil) }()
+	go func() { c.Check(a.Run(nil), jc.ErrorIsNil) }()
+
+	runner := s.singularRecord.nextRunner(c)
+	runner.waitForWorker(c, "dblogpruner")
+}
+
+func (s *MachineSuite) TestManageEnvironDoesntRunDbLogPrunerByDefault(c *gc.C) {
+	m, _, _ := s.primeAgent(c, version.Current, state.JobManageEnviron)
+	a := s.newAgent(c, m)
+	defer func() { c.Check(a.Stop(), jc.ErrorIsNil) }()
+	go func() { c.Check(a.Run(nil), jc.ErrorIsNil) }()
+
+	// Wait for the resumer to be started. This is started just after
+	// dblogpruner would be started.
+	runner := s.singularRecord.nextRunner(c)
+	started := set.NewStrings(runner.waitForWorker(c, "resumer")...)
+	c.Assert(started.Contains("dblogpruner"), jc.IsFalse)
+}
+
+func (s *MachineSuite) TestManageEnvironRunsStatusHistoryPruner(c *gc.C) {
+	m, _, _ := s.primeAgent(c, version.Current, state.JobManageEnviron)
+	a := s.newAgent(c, m)
+	defer func() { c.Check(a.Stop(), jc.ErrorIsNil) }()
+	go func() { c.Check(a.Run(nil), jc.ErrorIsNil) }()
+
+	runner := s.singularRecord.nextRunner(c)
+	runner.waitForWorker(c, "statushistorypruner")
+}
+
 func (s *MachineSuite) TestManageEnvironCallsUseMultipleCPUs(c *gc.C) {
 	// If it has been enabled, the JobManageEnviron agent should call utils.UseMultipleCPUs
 	usefulVersion := version.Current
@@ -671,6 +807,22 @@ func (s *MachineSuite) TestUpgradeRequest(c *gc.C) {
 	m, _, currentTools := s.primeAgent(c, version.Current, state.JobManageEnviron, state.JobHostUnits)
 	a := s.newAgent(c, m)
 	s.testUpgradeRequest(c, a, m.Tag().String(), currentTools)
+	c.Assert(a.isAgentUpgradePending(), jc.IsTrue)
+}
+
+func (s *MachineSuite) TestNoUpgradeRequired(c *gc.C) {
+	m, _, _ := s.primeAgent(c, version.Current, state.JobManageEnviron, state.JobHostUnits)
+	a := s.newAgent(c, m)
+	done := make(chan error)
+	go func() { done <- a.Run(nil) }()
+	select {
+	case <-a.initialAgentUpgradeCheckComplete:
+	case <-time.After(coretesting.LongWait):
+		c.Fatalf("timeout waiting for upgrade check")
+	}
+	defer a.Stop() // in case of failure
+	s.waitStopped(c, state.JobManageEnviron, a, done)
+	c.Assert(a.isAgentUpgradePending(), jc.IsFalse)
 }
 
 var fastDialOpts = api.DialOpts{
@@ -732,7 +884,7 @@ func (s *MachineSuite) assertJobWithState(
 // passed to the test function for further checking.
 func (s *MachineSuite) assertAgentOpensState(
 	c *gc.C,
-	reportOpened *func(interface{}),
+	reportOpened *func(io.Closer),
 	job state.MachineJob,
 	test func(agent.Config, interface{}),
 ) {
@@ -744,8 +896,8 @@ func (s *MachineSuite) assertAgentOpensState(
 	// All state jobs currently also run an APIWorker, so no
 	// need to check for that here, like in assertJobWithState.
 
-	agentAPIs := make(chan interface{}, 1)
-	s.AgentSuite.PatchValue(reportOpened, func(st interface{}) {
+	agentAPIs := make(chan io.Closer, 1)
+	s.AgentSuite.PatchValue(reportOpened, func(st io.Closer) {
 		select {
 		case agentAPIs <- st:
 		default:
@@ -887,6 +1039,10 @@ func (s *MachineSuite) TestJobManageEnvironRunsMinUnitsWorker(c *gc.C) {
 }
 
 func (s *MachineSuite) TestMachineAgentRunsAuthorisedKeysWorker(c *gc.C) {
+	//TODO(bogdanteleaga): Fix once we get authentication worker up on windows
+	if runtime.GOOS == "windows" {
+		c.Skip("bug 1403084: authentication worker not yet implemented on windows")
+	}
 	// Start the machine agent.
 	m, _, _ := s.primeAgent(c, version.Current, state.JobHostUnits)
 	a := s.newAgent(c, m)
@@ -963,6 +1119,11 @@ func (s *MachineSuite) TestMachineAgentSymlinkJujuRun(c *gc.C) {
 }
 
 func (s *MachineSuite) TestMachineAgentSymlinkJujuRunExists(c *gc.C) {
+	if runtime.GOOS == "windows" {
+		// Cannot make symlink to nonexistent file on windows or
+		// create a file point a symlink to it then remove it
+		c.Skip("Cannot test this on windows")
+	}
 	err := symlink.New("/nowhere/special", JujuRun)
 	c.Assert(err, jc.ErrorIsNil)
 	_, err = os.Stat(JujuRun)
@@ -1097,10 +1258,138 @@ func (s *MachineSuite) TestMachineAgentRunsAPIAddressUpdaterWorker(c *gc.C) {
 	c.Fatalf("timeout while waiting for agent config to change")
 }
 
+func (s *MachineSuite) TestMachineAgentRunsDiskManagerWorker(c *gc.C) {
+	// Start the machine agent.
+	m, _, _ := s.primeAgent(c, version.Current, state.JobHostUnits)
+	a := s.newAgent(c, m)
+	go func() { c.Check(a.Run(nil), jc.ErrorIsNil) }()
+	defer func() { c.Check(a.Stop(), jc.ErrorIsNil) }()
+
+	started := make(chan struct{})
+	newWorker := func(diskmanager.ListBlockDevicesFunc, diskmanager.BlockDeviceSetter) worker.Worker {
+		close(started)
+		return worker.NewNoOpWorker()
+	}
+	s.PatchValue(&newDiskManager, newWorker)
+
+	// Wait for worker to be started.
+	select {
+	case <-started:
+	case <-time.After(coretesting.LongWait):
+		c.Fatalf("timeout while waiting for diskmanager worker to start")
+	}
+}
+
+func (s *MachineSuite) TestDiskManagerWorkerUpdatesState(c *gc.C) {
+	expected := []storage.BlockDevice{{DeviceName: "whatever"}}
+	s.PatchValue(&diskmanager.DefaultListBlockDevices, func() ([]storage.BlockDevice, error) {
+		return expected, nil
+	})
+
+	// Start the machine agent.
+	m, _, _ := s.primeAgent(c, version.Current, state.JobHostUnits)
+	a := s.newAgent(c, m)
+	go func() { c.Check(a.Run(nil), jc.ErrorIsNil) }()
+	defer func() { c.Check(a.Stop(), jc.ErrorIsNil) }()
+
+	// Wait for state to be updated.
+	s.BackingState.StartSync()
+	for attempt := coretesting.LongAttempt.Start(); attempt.Next(); {
+		devices, err := s.BackingState.BlockDevices(m.MachineTag())
+		c.Assert(err, jc.ErrorIsNil)
+		if len(devices) > 0 {
+			c.Assert(devices, gc.HasLen, 1)
+			c.Assert(devices[0].DeviceName, gc.Equals, expected[0].DeviceName)
+			return
+		}
+	}
+	c.Fatalf("timeout while waiting for block devices to be recorded")
+}
+
+func (s *MachineSuite) TestMachineAgentRunsMachineStorageWorker(c *gc.C) {
+	// Start the machine agent.
+	m, _, _ := s.primeAgent(c, version.Current, state.JobHostUnits)
+	a := s.newAgent(c, m)
+	go func() { c.Check(a.Run(nil), jc.ErrorIsNil) }()
+	defer func() { c.Check(a.Stop(), jc.ErrorIsNil) }()
+
+	started := make(chan struct{})
+	newWorker := func(
+		scope names.Tag,
+		storageDir string,
+		_ storageprovisioner.VolumeAccessor,
+		_ storageprovisioner.FilesystemAccessor,
+		_ storageprovisioner.LifecycleManager,
+		_ storageprovisioner.EnvironAccessor,
+		_ storageprovisioner.MachineAccessor,
+	) worker.Worker {
+		c.Check(scope, gc.Equals, m.Tag())
+		// storageDir is not empty for machine scoped storage provisioners
+		c.Assert(storageDir, gc.Not(gc.Equals), "")
+		close(started)
+		return worker.NewNoOpWorker()
+	}
+	s.PatchValue(&newStorageWorker, newWorker)
+
+	// Wait for worker to be started.
+	select {
+	case <-started:
+	case <-time.After(coretesting.LongWait):
+		c.Fatalf("timeout while waiting for storage worker to start")
+	}
+}
+
+func (s *MachineSuite) TestMachineAgentRunsEnvironStorageWorker(c *gc.C) {
+	// Start the machine agent.
+	m, _, _ := s.primeAgent(c, version.Current, state.JobManageEnviron)
+	a := s.newAgent(c, m)
+	go func() { c.Check(a.Run(nil), jc.ErrorIsNil) }()
+	defer func() { c.Check(a.Stop(), jc.ErrorIsNil) }()
+
+	machineWorkerStarted := false
+	environWorkerStarted := false
+	numWorkers := 0
+	started := make(chan struct{})
+	newWorker := func(
+		scope names.Tag,
+		storageDir string,
+		_ storageprovisioner.VolumeAccessor,
+		_ storageprovisioner.FilesystemAccessor,
+		_ storageprovisioner.LifecycleManager,
+		_ storageprovisioner.EnvironAccessor,
+		_ storageprovisioner.MachineAccessor,
+	) worker.Worker {
+		// storageDir is empty for environ storage provisioners
+		if storageDir == "" {
+			c.Check(scope, gc.Equals, s.State.EnvironTag())
+			environWorkerStarted = true
+			numWorkers = numWorkers + 1
+		}
+		if storageDir != "" {
+			c.Check(scope, gc.Equals, m.Tag())
+			machineWorkerStarted = true
+			numWorkers = numWorkers + 1
+		}
+		if environWorkerStarted && machineWorkerStarted {
+			close(started)
+		}
+		return worker.NewNoOpWorker()
+	}
+	s.PatchValue(&newStorageWorker, newWorker)
+
+	// Wait for worker to be started.
+	select {
+	case <-started:
+		c.Assert(numWorkers, gc.Equals, 2)
+	case <-time.After(coretesting.LongWait):
+		c.Fatalf("timeout while waiting for storage worker to start")
+	}
+}
+
 func (s *MachineSuite) TestMachineAgentRunsCertificateUpdateWorkerForStateServer(c *gc.C) {
 	started := make(chan struct{})
 	newUpdater := func(certupdater.AddressWatcher, certupdater.StateServingInfoGetter, certupdater.EnvironConfigGetter,
-		certupdater.StateServingInfoSetter, chan params.StateServingInfo,
+		certupdater.APIHostPortsGetter, certupdater.StateServingInfoSetter,
 	) worker.Worker {
 		close(started)
 		return worker.NewNoOpWorker()
@@ -1124,7 +1413,7 @@ func (s *MachineSuite) TestMachineAgentRunsCertificateUpdateWorkerForStateServer
 func (s *MachineSuite) TestMachineAgentDoesNotRunsCertificateUpdateWorkerForNonStateServer(c *gc.C) {
 	started := make(chan struct{})
 	newUpdater := func(certupdater.AddressWatcher, certupdater.StateServingInfoGetter, certupdater.EnvironConfigGetter,
-		certupdater.StateServingInfoSetter, chan params.StateServingInfo,
+		certupdater.APIHostPortsGetter, certupdater.StateServingInfoSetter,
 	) worker.Worker {
 		close(started)
 		return worker.NewNoOpWorker()
@@ -1183,7 +1472,7 @@ func (s *MachineSuite) TestCertificateUpdateWorkerUpdatesCertificate(c *gc.C) {
 func (s *MachineSuite) TestCertificateDNSUpdated(c *gc.C) {
 	// Disable the certificate work so it doesn't update the certificate.
 	newUpdater := func(certupdater.AddressWatcher, certupdater.StateServingInfoGetter, certupdater.EnvironConfigGetter,
-		certupdater.StateServingInfoSetter, chan params.StateServingInfo,
+		certupdater.APIHostPortsGetter, certupdater.StateServingInfoSetter,
 	) worker.Worker {
 		return worker.NewNoOpWorker()
 	}
@@ -1258,7 +1547,7 @@ func (s *MachineSuite) TestMachineAgentNetworkerMode(c *gc.C) {
 
 		modeCh := make(chan bool, 1)
 		s.AgentSuite.PatchValue(&newNetworker, func(
-			st *apinetworker.State,
+			st apinetworker.State,
 			conf agent.Config,
 			intrusiveMode bool,
 			configBaseDir string,
@@ -1294,6 +1583,45 @@ func (s *MachineSuite) TestMachineAgentNetworkerMode(c *gc.C) {
 	}
 }
 
+func (s *MachineSuite) TestMachineAgentIgnoreAddresses(c *gc.C) {
+	for _, expectedIgnoreValue := range []bool{true, false} {
+		ignoreAddressCh := make(chan bool, 1)
+		s.AgentSuite.PatchValue(&newMachiner, func(
+			st *apimachiner.State,
+			conf agent.Config,
+			ignoreMachineAddresses bool,
+		) worker.Worker {
+			select {
+			case ignoreAddressCh <- ignoreMachineAddresses:
+			default:
+			}
+			return machiner.NewMachiner(st, conf, ignoreMachineAddresses)
+		})
+
+		attrs := coretesting.Attrs{"ignore-machine-addresses": expectedIgnoreValue}
+		err := s.BackingState.UpdateEnvironConfig(attrs, nil, nil)
+		c.Assert(err, jc.ErrorIsNil)
+
+		m, _, _ := s.primeAgent(c, version.Current, state.JobHostUnits)
+		a := s.newAgent(c, m)
+		defer a.Stop()
+		doneCh := make(chan error)
+		go func() {
+			doneCh <- a.Run(nil)
+		}()
+
+		select {
+		case ignoreMachineAddresses := <-ignoreAddressCh:
+			if ignoreMachineAddresses != expectedIgnoreValue {
+				c.Fatalf("expected ignore-machine-addresses = %v, got = %v", expectedIgnoreValue, ignoreMachineAddresses)
+			}
+		case <-time.After(coretesting.LongWait):
+			c.Fatalf("timed out waiting for the machiner to start")
+		}
+		s.waitStopped(c, state.JobHostUnits, a, doneCh)
+	}
+}
+
 func (s *MachineSuite) TestMachineAgentUpgradeMongo(c *gc.C) {
 	m, agentConfig, _ := s.primeAgent(c, version.Current, state.JobManageEnviron)
 	agentConfig.SetUpgradedToVersion(version.MustParse("1.18.0"))
@@ -1302,6 +1630,9 @@ func (s *MachineSuite) TestMachineAgentUpgradeMongo(c *gc.C) {
 	err = s.State.MongoSession().DB("admin").RemoveUser(m.Tag().String())
 	c.Assert(err, jc.ErrorIsNil)
 
+	s.fakeEnsureMongo.ServiceInstalled = true
+	s.fakeEnsureMongo.ReplicasetInitiated = false
+
 	s.AgentSuite.PatchValue(&ensureMongoAdminUser, func(p mongo.EnsureAdminUserParams) (bool, error) {
 		err := s.State.MongoSession().DB("admin").AddUser(p.User, p.Password, false)
 		c.Assert(err, jc.ErrorIsNil)
@@ -1309,7 +1640,7 @@ func (s *MachineSuite) TestMachineAgentUpgradeMongo(c *gc.C) {
 	})
 
 	stateOpened := make(chan interface{}, 1)
-	s.AgentSuite.PatchValue(&reportOpenedState, func(st interface{}) {
+	s.AgentSuite.PatchValue(&reportOpenedState, func(st io.Closer) {
 		select {
 		case stateOpened <- st:
 		default:
@@ -1378,6 +1709,172 @@ func (s *MachineSuite) TestMachineAgentRestoreRequiresPrepare(c *gc.C) {
 	c.Assert(err, gc.ErrorMatches, "not in restore mode, cannot begin restoration")
 	c.Assert(a.IsRestoreRunning(), jc.IsFalse)
 }
+func (s *MachineSuite) TestMachineAgentAPIWorkerErrorClosesAPI(c *gc.C) {
+	// Start the machine agent.
+	m, _, _ := s.primeAgent(c, version.Current, state.JobHostUnits)
+	a := s.newAgent(c, m)
+	a.apiStateUpgrader = &machineAgentUpgrader{}
+
+	closedAPI := make(chan io.Closer, 1)
+	s.AgentSuite.PatchValue(&reportClosedAPI, func(st io.Closer) {
+		select {
+		case closedAPI <- st:
+			close(closedAPI)
+		default:
+		}
+	})
+
+	worker, err := a.APIWorker()
+
+	select {
+	case closed := <-closedAPI:
+		c.Assert(closed, gc.NotNil)
+	case <-time.After(coretesting.LongWait):
+		c.Fatalf("API not opened")
+	}
+
+	c.Assert(worker, gc.IsNil)
+	c.Assert(err, gc.ErrorMatches, "cannot set machine agent version: test failure")
+	c.Assert(a.isAgentUpgradePending(), jc.IsTrue)
+}
+
+type machineAgentUpgrader struct{}
+
+func (m *machineAgentUpgrader) SetVersion(s string, v version.Binary) error {
+	return errors.New("test failure")
+}
+
+func (s *MachineSuite) TestNewEnvironmentStartsNewWorkers(c *gc.C) {
+	_, expectedWorkers, closer := s.setUpNewEnvironment(c)
+	defer closer()
+
+	r1 := s.singularRecord.nextRunner(c)
+	workers := r1.waitForWorker(c, "firewaller")
+	c.Assert(workers, jc.DeepEquals, expectedWorkers)
+}
+
+func (s *MachineSuite) TestNewStorageWorkerIsScopedToNewEnviron(c *gc.C) {
+	st, _, closer := s.setUpNewEnvironment(c)
+	defer closer()
+
+	// Check that newStorageWorker is called and the environ tag is scoped to
+	// that of the new environment tag.
+	started := make(chan struct{})
+	newWorker := func(
+		scope names.Tag,
+		storageDir string,
+		_ storageprovisioner.VolumeAccessor,
+		_ storageprovisioner.FilesystemAccessor,
+		_ storageprovisioner.LifecycleManager,
+		_ storageprovisioner.EnvironAccessor,
+		_ storageprovisioner.MachineAccessor,
+	) worker.Worker {
+		// storageDir is empty for environ storage provisioners
+		if storageDir == "" {
+			c.Check(scope, gc.Equals, st.EnvironTag())
+			close(started)
+		}
+		return worker.NewNoOpWorker()
+	}
+	s.PatchValue(&newStorageWorker, newWorker)
+
+	// Wait for newStorageWorker to be started.
+	select {
+	case <-started:
+	case <-time.After(coretesting.LongWait):
+		c.Fatalf("timeout while waiting for storage worker to start")
+	}
+}
+
+func (s *MachineSuite) setUpNewEnvironment(c *gc.C) (newSt *state.State, expectedWorkers []string, closer func()) {
+	expectedWorkers = make([]string, 0, len(perEnvSingularWorkers)+1)
+	for _, w := range perEnvSingularWorkers {
+		expectedWorkers = append(expectedWorkers, w)
+		if w == "environ-provisioner" {
+			expectedWorkers = append(expectedWorkers, "environ-storageprovisioner")
+		}
+	}
+	s.PatchValue(&watcher.Period, 100*time.Millisecond)
+
+	m, _, _ := s.primeAgent(c, version.Current, state.JobManageEnviron)
+	a := s.newAgent(c, m)
+	go func() { c.Check(a.Run(nil), jc.ErrorIsNil) }()
+
+	_ = s.singularRecord.nextRunner(c) // Don't care about this one for this test.
+
+	// Wait for the workers for the initial env to start. The
+	// firewaller is the last worker started for a new environment.
+	r0 := s.singularRecord.nextRunner(c)
+	workers := r0.waitForWorker(c, "firewaller")
+	c.Assert(workers, jc.DeepEquals, expectedWorkers)
+
+	// Create a new environment, tests can now watch if workers start for it.
+	newSt = s.Factory.MakeEnvironment(c, &factory.EnvParams{
+		ConfigAttrs: map[string]interface{}{
+			"state-server": false,
+		},
+		Prepare: true,
+	})
+	return newSt, expectedWorkers, func() {
+		newSt.Close()
+		c.Check(a.Stop(), jc.ErrorIsNil)
+	}
+}
+
+func (s *MachineSuite) TestReplicasetInitiation(c *gc.C) {
+	if runtime.GOOS == "windows" {
+		c.Skip("state servers on windows aren't supported")
+	}
+
+	s.fakeEnsureMongo.ReplicasetInitiated = false
+
+	m, _, _ := s.primeAgent(c, version.Current, state.JobManageEnviron)
+	a := s.newAgent(c, m)
+	agentConfig := a.CurrentConfig()
+
+	err := a.ensureMongoServer(agentConfig)
+	c.Assert(err, jc.ErrorIsNil)
+
+	c.Assert(s.fakeEnsureMongo.EnsureCount, gc.Equals, 1)
+	c.Assert(s.fakeEnsureMongo.InitiateCount, gc.Equals, 1)
+}
+
+func (s *MachineSuite) TestReplicasetAlreadyInitiated(c *gc.C) {
+	if runtime.GOOS == "windows" {
+		c.Skip("state servers on windows aren't supported")
+	}
+
+	s.fakeEnsureMongo.ReplicasetInitiated = true
+
+	m, _, _ := s.primeAgent(c, version.Current, state.JobManageEnviron)
+	a := s.newAgent(c, m)
+	agentConfig := a.CurrentConfig()
+
+	err := a.ensureMongoServer(agentConfig)
+	c.Assert(err, jc.ErrorIsNil)
+
+	c.Assert(s.fakeEnsureMongo.EnsureCount, gc.Equals, 1)
+	c.Assert(s.fakeEnsureMongo.InitiateCount, gc.Equals, 0)
+}
+
+func (s *MachineSuite) TestReplicasetInitForNewStateServer(c *gc.C) {
+	if runtime.GOOS == "windows" {
+		c.Skip("state servers on windows aren't supported")
+	}
+
+	s.fakeEnsureMongo.ServiceInstalled = false
+	s.fakeEnsureMongo.ReplicasetInitiated = true
+
+	m, _, _ := s.primeAgent(c, version.Current, state.JobManageEnviron)
+	a := s.newAgent(c, m)
+	agentConfig := a.CurrentConfig()
+
+	err := a.ensureMongoServer(agentConfig)
+	c.Assert(err, jc.ErrorIsNil)
+
+	c.Assert(s.fakeEnsureMongo.EnsureCount, gc.Equals, 1)
+	c.Assert(s.fakeEnsureMongo.InitiateCount, gc.Equals, 0)
+}
 
 // MachineWithCharmsSuite provides infrastructure for tests which need to
 // work with charms.
@@ -1554,8 +2051,13 @@ func (m *mockAgentConfig) Value(key string) string {
 }
 
 type singularRunnerRecord struct {
-	mu             sync.Mutex
-	startedWorkers set.Strings
+	runnerC chan *fakeSingularRunner
+}
+
+func newSingularRunnerRecord() *singularRunnerRecord {
+	return &singularRunnerRecord{
+		runnerC: make(chan *fakeSingularRunner, 5),
+	}
 }
 
 func (r *singularRunnerRecord) newSingularRunner(runner worker.Runner, conn singular.Conn) (worker.Runner, error) {
@@ -1563,29 +2065,54 @@ func (r *singularRunnerRecord) newSingularRunner(runner worker.Runner, conn sing
 	if err != nil {
 		return nil, err
 	}
-	return &fakeSingularRunner{
+	fakeRunner := &fakeSingularRunner{
 		Runner: sr,
-		record: r,
-	}, nil
+		startC: make(chan string, 64),
+	}
+	r.runnerC <- fakeRunner
+	return fakeRunner, nil
 }
 
-// started returns the names of all singular-started workers.
-func (r *singularRunnerRecord) started() []string {
-	return r.startedWorkers.SortedValues()
+// nextRunner blocks until a new singular runner is created.
+func (r *singularRunnerRecord) nextRunner(c *gc.C) *fakeSingularRunner {
+	for {
+		select {
+		case r := <-r.runnerC:
+			return r
+		case <-time.After(coretesting.LongWait):
+			c.Fatal("timed out waiting for singular runner to be created")
+		}
+	}
 }
 
 type fakeSingularRunner struct {
 	worker.Runner
-	record *singularRunnerRecord
+	startC chan string
 }
 
 func (r *fakeSingularRunner) StartWorker(name string, start func() (worker.Worker, error)) error {
-	r.record.mu.Lock()
-	defer r.record.mu.Unlock()
-	r.record.startedWorkers.Add(name)
+	r.startC <- name
 	return r.Runner.StartWorker(name, start)
 }
 
+// waitForWorker waits for a given worker to be started, returning all
+// workers started while waiting.
+func (r *fakeSingularRunner) waitForWorker(c *gc.C, target string) []string {
+	var seen []string
+	timeout := time.After(coretesting.LongWait)
+	for {
+		select {
+		case workerName := <-r.startC:
+			seen = append(seen, workerName)
+			if workerName == target {
+				return seen
+			}
+		case <-timeout:
+			c.Fatal("timed out waiting for " + target)
+		}
+	}
+}
+
 func newDummyWorker() worker.Worker {
 	return worker.NewSimpleWorker(func(stop <-chan struct{}) error {
 		<-stop
@@ -1594,19 +2121,37 @@ func newDummyWorker() worker.Worker {
 }
 
 type mockMetricAPI struct {
+	stop          chan struct{}
 	cleanUpCalled chan struct{}
 	sendCalled    chan struct{}
 }
 
+func newMockMetricAPI() *mockMetricAPI {
+	return &mockMetricAPI{
+		stop:          make(chan struct{}),
+		cleanUpCalled: make(chan struct{}),
+		sendCalled:    make(chan struct{}),
+	}
+}
+
 func (m *mockMetricAPI) CleanupOldMetrics() error {
 	go func() {
-		m.cleanUpCalled <- struct{}{}
+		select {
+		case m.cleanUpCalled <- struct{}{}:
+		case <-m.stop:
+			break
+		}
 	}()
 	return nil
 }
+
 func (m *mockMetricAPI) SendMetrics() error {
 	go func() {
-		m.sendCalled <- struct{}{}
+		select {
+		case m.sendCalled <- struct{}{}:
+		case <-m.stop:
+			break
+		}
 	}()
 	return nil
 }
@@ -1619,6 +2164,10 @@ func (m *mockMetricAPI) CleanupCalled() <-chan struct{} {
 	return m.cleanUpCalled
 }
 
+func (m *mockMetricAPI) Stop() {
+	close(m.stop)
+}
+
 func mkdtemp(prefix string) string {
 	d, err := ioutil.TempDir("", prefix)
 	if err != nil {
