@@ -35,6 +35,7 @@ import (
 	"github.com/juju/juju/state/multiwatcher"
 	"github.com/juju/juju/state/storage"
 	"github.com/juju/juju/state/toolstorage"
+	"github.com/juju/juju/storage/poolmanager"
 	"github.com/juju/juju/utils/ssh"
 	"github.com/juju/juju/version"
 	"github.com/juju/juju/worker/peergrouper"
@@ -60,6 +61,13 @@ type BootstrapCommand struct {
 	ImageMetadataDir string
 }
 
+// NewBootstrapCommand returns a new BootstrapCommand that has been initialized.
+func NewBootstrapCommand() *BootstrapCommand {
+	return &BootstrapCommand{
+		AgentConf: agentcmd.NewAgentConf(""),
+	}
+}
+
 // Info returns a decription of the command.
 func (c *BootstrapCommand) Info() *cmd.Info {
 	return &cmd.Info{
@@ -232,6 +240,11 @@ func (c *BootstrapCommand) Run(_ *cmd.Context) error {
 		}
 	}
 
+	// Populate the storage pools.
+	if err := c.populateDefaultStoragePools(st); err != nil {
+		return err
+	}
+
 	// bootstrap machine always gets the vote
 	return m.SetHasVote(true)
 }
@@ -284,6 +297,12 @@ func (c *BootstrapCommand) startMongo(addrs []network.Address, agentConfig agent
 	})
 }
 
+// populateDefaultStoragePools creates the default storage pools.
+func (c *BootstrapCommand) populateDefaultStoragePools(st *state.State) error {
+	settings := state.NewStateSettings(st)
+	return poolmanager.AddDefaultStoragePools(settings)
+}
+
 // populateTools stores uploaded tools in provider storage
 // and updates the tools metadata.
 func (c *BootstrapCommand) populateTools(st *state.State, env environs.Environ) error {
@@ -357,6 +376,7 @@ func (c *BootstrapCommand) storeCustomImageMetadata(stor storage.Storage) error
 			return err
 		}
 		defer f.Close()
+		relpath = filepath.ToSlash(relpath)
 		logger.Debugf("storing %q in environment storage (%d bytes)", relpath, info.Size())
 		return stor.Put(relpath, f, info.Size())
 	})
