@@ -7,12 +7,15 @@ import (
 	"fmt"
 	"os"
 	"path/filepath"
+	"runtime"
+	"strings"
 	"time"
 
 	"github.com/juju/cmd"
 	"github.com/juju/loggo"
 	"github.com/juju/names"
 	jc "github.com/juju/testing/checkers"
+	"github.com/juju/utils"
 	"github.com/juju/utils/exec"
 	"github.com/juju/utils/fslock"
 	gc "gopkg.in/check.v1"
@@ -106,7 +109,7 @@ func (*RunTestSuite) TestArgParsing(c *gc.C) {
 		forceRemoteUnit: true,
 	},
 	} {
-		c.Logf("\n%d: %s", i, test.title)
+		c.Logf("%d: %s", i, test.title)
 		runCommand := &RunCommand{}
 		err := testing.InitCommand(runCommand, test.args)
 		if test.errMatch == "" {
@@ -165,14 +168,14 @@ func (s *RunTestSuite) TestNoContext(c *gc.C) {
 	ctx, err := testing.RunCommand(c, &RunCommand{}, "--no-context", "echo done")
 	c.Assert(err, jc.Satisfies, cmd.IsRcPassthroughError)
 	c.Assert(err, gc.ErrorMatches, "subprocess encountered error code 0")
-	c.Assert(testing.Stdout(ctx), gc.Equals, "done\n")
+	c.Assert(strings.TrimRight(testing.Stdout(ctx), "\r\n"), gc.Equals, "done")
 }
 
 func (s *RunTestSuite) TestNoContextAsync(c *gc.C) {
 	channel := startRunAsync(c, []string{"--no-context", "echo done"})
 	ctx, err := waitForResult(channel, testing.LongWait)
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(testing.Stdout(ctx), gc.Equals, "done\n")
+	c.Assert(strings.TrimRight(testing.Stdout(ctx), "\r\n"), gc.Equals, "done")
 }
 
 func (s *RunTestSuite) TestNoContextWithLock(c *gc.C) {
@@ -191,16 +194,19 @@ func (s *RunTestSuite) TestNoContextWithLock(c *gc.C) {
 
 	ctx, err = waitForResult(channel, testing.LongWait)
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(testing.Stdout(ctx), gc.Equals, "done\n")
+	c.Assert(strings.TrimRight(testing.Stdout(ctx), "\r\n"), gc.Equals, "done")
 }
 
 func (s *RunTestSuite) TestMissingSocket(c *gc.C) {
+	if runtime.GOOS == "windows" {
+		c.Skip("Current implementation of named pipes loops if the socket is missing")
+	}
 	agentDir := filepath.Join(cmdutil.DataDir, "agents", "unit-foo-1")
 	err := os.MkdirAll(agentDir, 0755)
 	c.Assert(err, jc.ErrorIsNil)
 
 	_, err = testing.RunCommand(c, &RunCommand{}, "foo/1", "bar")
-	c.Assert(err, gc.ErrorMatches, `dial unix .*/run.socket: no such file or directory`)
+	c.Assert(err, gc.ErrorMatches, `dial unix .*/run.socket: `+utils.NoSuchFileErrRegexp)
 }
 
 func (s *RunTestSuite) TestRunning(c *gc.C) {
@@ -283,7 +289,7 @@ func (s *RunTestSuite) TestCheckRelationIdValid(c *gc.C) {
 			err:    true,
 		},
 	} {
-		c.Logf("\n%d: %s", i, test.title)
+		c.Logf("%d: %s", i, test.title)
 		relationId, err := checkRelationId(test.input)
 		c.Assert(relationId, gc.Equals, test.output)
 		if test.err {
