@@ -5,7 +5,7 @@ package main
 
 import (
 	"fmt"
-	"path/filepath"
+	"io"
 	"runtime"
 
 	"github.com/juju/cmd"
@@ -18,6 +18,8 @@ import (
 	"launchpad.net/tomb"
 
 	"github.com/juju/juju/agent"
+	"github.com/juju/juju/api"
+	"github.com/juju/juju/api/leadership"
 	agentcmd "github.com/juju/juju/cmd/jujud/agent"
 	cmdutil "github.com/juju/juju/cmd/jujud/util"
 	"github.com/juju/juju/network"
@@ -32,17 +34,43 @@ import (
 	"github.com/juju/juju/worker/upgrader"
 )
 
-var agentLogger = loggo.GetLogger("juju.jujud")
+var (
+	agentLogger     = loggo.GetLogger("juju.jujud")
+	reportClosedAPI = func(io.Closer) {}
+)
 
 // UnitAgent is a cmd.Command responsible for running a unit agent.
 type UnitAgent struct {
 	cmd.CommandBase
 	tomb tomb.Tomb
 	agentcmd.AgentConf
-	UnitName     string
-	runner       worker.Runner
-	setupLogging func(agent.Config) error
-	logToStdErr  bool
+	UnitName         string
+	runner           worker.Runner
+	setupLogging     func(agent.Config) error
+	logToStdErr      bool
+	ctx              *cmd.Context
+	apiStateUpgrader agentcmd.APIStateUpgrader
+
+	// Used to signal that the upgrade worker will not
+	// reboot the agent on startup because there are no
+	// longer any immediately pending agent upgrades.
+	// Channel used as a selectable bool (closed means true).
+	initialAgentUpgradeCheckComplete chan struct{}
+}
+
+// NewUnitAgent creates a new UnitAgent value properly initialized.
+func NewUnitAgent() *UnitAgent {
+	return &UnitAgent{
+		AgentConf:                        agentcmd.NewAgentConf(""),
+		initialAgentUpgradeCheckComplete: make(chan struct{}),
+	}
+}
+
+func (a *UnitAgent) getUpgrader(st *api.State) agentcmd.APIStateUpgrader {
+	if a.apiStateUpgrader != nil {
+		return a.apiStateUpgrader
+	}
+	return st.Upgrader()
 }
 
 // Info returns usage information for the command.
@@ -71,6 +99,22 @@ func (a *UnitAgent) Init(args []string) error {
 		return err
 	}
 	a.runner = worker.NewRunner(cmdutil.IsFatal, cmdutil.MoreImportant)
+
+	if !a.logToStdErr {
+		if err := a.ReadConfig(a.Tag().String()); err != nil {
+			return err
+		}
+		agentConfig := a.CurrentConfig()
+
+		// the writer in ctx.stderr gets set as the loggo writer in github.com/juju/cmd/logging.go
+		a.ctx.Stderr = &lumberjack.Logger{
+			Filename:   agent.LogFilename(agentConfig),
+			MaxSize:    300, // megabytes
+			MaxBackups: 2,
+		}
+
+	}
+
 	return nil
 }
 
@@ -88,19 +132,6 @@ func (a *UnitAgent) Run(ctx *cmd.Context) error {
 	}
 	agentConfig := a.CurrentConfig()
 
-	if !a.logToStdErr {
-		filename := filepath.Join(agentConfig.LogDir(), agentConfig.Tag().String()+".log")
-
-		log := &lumberjack.Logger{
-			Filename:   filename,
-			MaxSize:    300, // megabytes
-			MaxBackups: 2,
-		}
-
-		if err := cmdutil.SwitchProcessToRollingLogs(log); err != nil {
-			return err
-		}
-	}
 	agentLogger.Infof("unit agent %v start (%s [%s])", a.Tag().String(), version.Current, runtime.Compiler)
 	if flags := featureflag.String(); flags != "" {
 		logger.Warningf("developer feature flags enabled: %s", flags)
@@ -113,10 +144,10 @@ func (a *UnitAgent) Run(ctx *cmd.Context) error {
 	return err
 }
 
-func (a *UnitAgent) APIWorkers() (worker.Worker, error) {
+func (a *UnitAgent) APIWorkers() (_ worker.Worker, err error) {
 	agentConfig := a.CurrentConfig()
 	dataDir := agentConfig.DataDir()
-	hookLock, err := cmdutil.HookExecutionLock(dataDir)
+	machineLock, err := cmdutil.HookExecutionLock(dataDir)
 	if err != nil {
 		return nil, err
 	}
@@ -124,11 +155,41 @@ func (a *UnitAgent) APIWorkers() (worker.Worker, error) {
 	if err != nil {
 		return nil, err
 	}
+	unitTag, err := names.ParseUnitTag(entity.Tag())
+	if err != nil {
+		return nil, errors.Trace(err)
+	}
+	// Ensure that the environment uuid is stored in the agent config.
+	// Luckily the API has it recorded for us after we connect.
+	if agentConfig.Environment().Id() == "" {
+		err := a.ChangeConfig(func(setter agent.ConfigSetter) error {
+			environTag, err := st.EnvironTag()
+			if err != nil {
+				return errors.Annotate(err, "no environment uuid set on api")
+			}
+
+			return setter.Migrate(agent.MigrateParams{
+				Environment: environTag,
+			})
+		})
+		if err != nil {
+			logger.Warningf("unable to save environment uuid: %v", err)
+			// Not really fatal, just annoying.
+		}
+	}
+
+	defer func() {
+		if err != nil {
+			st.Close()
+			reportClosedAPI(st)
+		}
+	}()
 
 	// Before starting any workers, ensure we record the Juju version this unit
 	// agent is running.
 	currentTools := &tools.Tools{Version: version.Current}
-	if err := st.Upgrader().SetVersion(agentConfig.Tag().String(), currentTools.Version); err != nil {
+	apiStateUpgrader := a.getUpgrader(st)
+	if err := apiStateUpgrader.SetVersion(agentConfig.Tag().String(), currentTools.Version); err != nil {
 		return nil, errors.Annotate(err, "cannot set unit agent version")
 	}
 
@@ -138,11 +199,12 @@ func (a *UnitAgent) APIWorkers() (worker.Worker, error) {
 		return proxyupdater.New(st.Environment(), false), nil
 	})
 	runner.StartWorker("upgrader", func() (worker.Worker, error) {
-		return upgrader.NewUpgrader(
+		return upgrader.NewAgentUpgrader(
 			st.Upgrader(),
 			agentConfig,
 			agentConfig.UpgradedToVersion(),
 			func() bool { return false },
+			a.initialAgentUpgradeCheckComplete,
 		), nil
 	})
 	runner.StartWorker("logger", func() (worker.Worker, error) {
@@ -153,11 +215,7 @@ func (a *UnitAgent) APIWorkers() (worker.Worker, error) {
 		if err != nil {
 			return nil, errors.Trace(err)
 		}
-		unitTag, err := names.ParseUnitTag(entity.Tag())
-		if err != nil {
-			return nil, errors.Trace(err)
-		}
-		return uniter.NewUniter(uniterFacade, unitTag, dataDir, hookLock), nil
+		return uniter.NewUniter(uniterFacade, unitTag, leadership.NewClient(st), dataDir, machineLock, nil), nil
 	})
 
 	runner.StartWorker("apiaddressupdater", func() (worker.Worker, error) {
