@@ -8,8 +8,8 @@ import (
 	"strings"
 
 	"github.com/juju/cmd"
+	"gopkg.in/amz.v3/aws"
 	gc "gopkg.in/check.v1"
-	"launchpad.net/goamz/aws"
 
 	"github.com/juju/juju/cmd/envcmd"
 	"github.com/juju/juju/environs/filestorage"
