@@ -11,15 +11,22 @@ var (
 	ContainerConfigFilename = containerConfigFilename
 	ContainerDirFilesystem  = containerDirFilesystem
 	GenerateNetworkConfig   = generateNetworkConfig
-	DiscoverHostNIC         = &discoverHostNIC
+	ParseConfigLine         = parseConfigLine
+	UpdateContainerConfig   = updateContainerConfig
+	ReorderNetworkConfig    = reorderNetworkConfig
 	NetworkConfigTemplate   = networkConfigTemplate
 	RestartSymlink          = restartSymlink
 	ReleaseVersion          = &releaseVersion
 	PreferFastLXC           = preferFastLXC
 	InitProcessCgroupFile   = &initProcessCgroupFile
 	RuntimeGOOS             = &runtimeGOOS
+	WriteWgetTmpFile        = &writeWgetTmpFile
 )
 
 func GetCreateWithCloneValue(mgr container.Manager) bool {
 	return mgr.(*containerManager).createWithClone
 }
+
+func WgetEnvironment(caCert []byte) ([]string, func(), error) {
+	return wgetEnvironment(caCert)
+}
