@@ -10,7 +10,8 @@ import (
 	"strings"
 	"testing"
 
-	gc "launchpad.net/gocheck"
+	jc "github.com/juju/testing/checkers"
+	gc "gopkg.in/check.v1"
 )
 
 func Test(t *testing.T) {
@@ -23,14 +24,14 @@ var _ = gc.Suite(&dependenciesTest{})
 
 func projectRoot(c *gc.C) string {
 	p, err := build.Import("github.com/juju/juju", "", build.FindOnly)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	return p.Dir
 }
 
 func (*dependenciesTest) TestDependenciesTsvFormat(c *gc.C) {
 	filename := filepath.Join(projectRoot(c), "dependencies.tsv")
 	content, err := ioutil.ReadFile(filename)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 
 	for _, line := range strings.Split(string(content), "\n") {
 		if line == "" {
