@@ -12,6 +12,7 @@ import (
 	"github.com/juju/loggo"
 	"github.com/juju/utils"
 
+	"github.com/juju/juju/cloudconfig/instancecfg"
 	"github.com/juju/juju/constraints"
 	"github.com/juju/juju/environs"
 	"github.com/juju/juju/environs/imagemetadata"
@@ -91,8 +92,10 @@ func Bootstrap(ctx environs.BootstrapContext, environ environs.Environ, args Boo
 		return err
 	}
 
+	_, supportsNetworking := environs.SupportsNetworking(environ)
+
 	ctx.Infof("Bootstrapping environment %q", cfg.Name())
-	logger.Debugf("environment %q supports service/machine networks: %v", cfg.Name(), environ.SupportNetworks())
+	logger.Debugf("environment %q supports service/machine networks: %v", cfg.Name(), supportsNetworking)
 	disableNetworkManagement, _ := cfg.DisableNetworkManagement()
 	logger.Debugf("network management by juju enabled: %v", !disableNetworkManagement)
 	availableTools, err := findAvailableTools(environ, args.Constraints.Arch, args.UploadTools)
@@ -101,10 +104,13 @@ func Bootstrap(ctx environs.BootstrapContext, environ environs.Environ, args Boo
 	} else if err != nil {
 		return err
 	}
+	if lxcMTU, ok := cfg.LXCDefaultMTU(); ok {
+		logger.Debugf("using MTU %v for all created LXC containers' network interfaces", lxcMTU)
+	}
 
 	// If we're uploading, we must override agent-version;
 	// if we're not uploading, we want to ensure we have an
-	// agent-version set anyway, to appease FinishMachineConfig.
+	// agent-version set anyway, to appease FinishInstanceConfig.
 	// In the latter case, setBootstrapTools will later set
 	// agent-version to the correct thing.
 	if cfg, err = cfg.Apply(map[string]interface{}{
@@ -154,16 +160,16 @@ func Bootstrap(ctx environs.BootstrapContext, environ environs.Environ, args Boo
 	}
 
 	ctx.Infof("Installing Juju agent on bootstrap instance")
-	machineConfig, err := environs.NewBootstrapMachineConfig(args.Constraints, series)
+	instanceConfig, err := instancecfg.NewBootstrapInstanceConfig(args.Constraints, series)
 	if err != nil {
 		return err
 	}
-	machineConfig.Tools = selectedTools
-	machineConfig.CustomImageMetadata = imageMetadata
-	if err := finalizer(ctx, machineConfig); err != nil {
+	instanceConfig.Tools = selectedTools
+	instanceConfig.CustomImageMetadata = imageMetadata
+	if err := finalizer(ctx, instanceConfig); err != nil {
 		return err
 	}
-	ctx.Infof("Bootstrap complete")
+	ctx.Infof("Bootstrap agent installed")
 	return nil
 }
 
@@ -255,7 +261,7 @@ func setPrivateMetadataSources(env environs.Environ, metadataDir string) ([]*ima
 	}
 
 	// Add an image metadata datasource for constraint validation, etc.
-	environs.RegisterImageDataSourceFunc("bootstrap metadata", func(environs.Environ) (simplestreams.DataSource, error) {
+	environs.RegisterUserImageDataSourceFunc("bootstrap metadata", func(environs.Environ) (simplestreams.DataSource, error) {
 		return datasource, nil
 	})
 	logger.Infof("custom image metadata added to search path")
