@@ -52,6 +52,15 @@ func validateEnvironmentKind(rawEnviron map[string]interface{}) error {
 	return nil
 }
 
+// disallowedWithNew holds those attributes
+// that can not be set in an initial environment
+// config used to bootstrap (they must only be set
+// on a running environment where appropriate
+// validation can be performed).
+var disallowedWithBootstrap = []string{
+	config.StorageDefaultBlockSourceKey,
+}
+
 // Config returns the environment configuration for the environment
 // with the given name. If the configuration is not
 // found, an errors.NotFoundError is returned.
@@ -70,6 +79,13 @@ func (envs *Environs) Config(name string) (*config.Config, error) {
 		return nil, errors.Trace(err)
 	}
 
+	// Check that we don't have any disallowed fields in new configs used for bootstrap.
+	for _, attr := range disallowedWithBootstrap {
+		if _, ok := attrs[attr]; ok {
+			return nil, fmt.Errorf("attribute %q is not allowed in bootstrap configurations", attr)
+		}
+	}
+
 	// If deprecated config attributes are used, log warnings so the user can know
 	// that they need to be fixed.
 	// We also look up what any new values might be so we can tell the user.
@@ -92,6 +108,9 @@ func (envs *Environs) Config(name string) (*config.Config, error) {
 	// tools-stream has been renamed to agent-stream, so log warnings to the user
 	envs.logDeprecatedWarnings(attrs, newAttrs, config.ToolsStreamKey, config.AgentStreamKey)
 
+	// Block attributes only matter if they have been used
+	envs.logBlockDeprecationWarnings(attrs)
+
 	cfg, err := config.New(config.UseDefaults, attrs)
 	if err != nil {
 		return nil, err
@@ -99,6 +118,28 @@ func (envs *Environs) Config(name string) (*config.Config, error) {
 	return cfg, nil
 }
 
+func (envs *Environs) logBlockDeprecationWarnings(attrs map[string]interface{}) {
+	checkBlockVar := func(key string) {
+		if used, ok := attrs[key]; ok {
+			if used.(bool) {
+				envs.logBlockWarning(key, used)
+			}
+		}
+	}
+	checkBlockVar(config.PreventDestroyEnvironmentKey)
+	checkBlockVar(config.PreventRemoveObjectKey)
+	checkBlockVar(config.PreventAllChangesKey)
+}
+
+func (envs *Environs) logBlockWarning(key string, value interface{}) {
+	logger.Warningf(
+		"Config attribute %q (%v) is deprecated and will be ignored since \n"+
+			"upgrade process takes care of it. \n"+
+			"The attribute %q should be removed from your configuration.",
+		key, value, key,
+	)
+}
+
 // logDeprecatedWarnings constructs log warning messages for deprecated attributes names.
 // It checks if both old and new attribute names are provided.
 // When both are provided, the message warns to remove old attribute from configuration.
@@ -146,17 +187,26 @@ var providerAliases = make(map[string]string)
 // are registered more than once.
 func RegisterProvider(name string, p EnvironProvider, alias ...string) {
 	if providers[name] != nil || providerAliases[name] != "" {
-		panic(fmt.Errorf("juju: duplicate provider name %q", name))
+		panic(errors.Errorf("juju: duplicate provider name %q", name))
 	}
 	providers[name] = p
 	for _, alias := range alias {
 		if providers[alias] != nil || providerAliases[alias] != "" {
-			panic(fmt.Errorf("juju: duplicate provider alias %q", alias))
+			panic(errors.Errorf("juju: duplicate provider alias %q", alias))
 		}
 		providerAliases[alias] = name
 	}
 }
 
+// RegisteredProviders enumerate all the environ providers which have been registered.
+func RegisteredProviders() []string {
+	var p []string
+	for k := range providers {
+		p = append(p, k)
+	}
+	return p
+}
+
 // Provider returns the previously registered provider with the given type.
 func Provider(providerType string) (EnvironProvider, error) {
 	if alias, ok := providerAliases[providerType]; ok {
@@ -164,7 +214,7 @@ func Provider(providerType string) (EnvironProvider, error) {
 	}
 	p, ok := providers[providerType]
 	if !ok {
-		return nil, fmt.Errorf("no registered provider for %q", providerType)
+		return nil, errors.Errorf("no registered provider for %q", providerType)
 	}
 	return p, nil
 }
@@ -184,7 +234,7 @@ func ReadEnvironsBytes(data []byte) (*Environs, error) {
 	}
 
 	if raw.Default != "" && raw.Environments[raw.Default] == nil {
-		return nil, fmt.Errorf("default environment %q does not exist", raw.Default)
+		return nil, errors.Errorf("default environment %q does not exist", raw.Default)
 	}
 	if raw.Default == "" {
 		// If there's a single environment, then we get the default
