@@ -5,6 +5,7 @@ package config_test
 
 import (
 	"fmt"
+	"os"
 	"regexp"
 	"strings"
 	stdtesting "testing"
@@ -16,6 +17,7 @@ import (
 	jc "github.com/juju/testing/checkers"
 	"github.com/juju/utils/proxy"
 	gc "gopkg.in/check.v1"
+	"gopkg.in/juju/charm.v5/charmrepo"
 
 	"github.com/juju/juju/cert"
 	"github.com/juju/juju/environs/config"
@@ -68,6 +70,11 @@ type configTest struct {
 	err         string
 }
 
+var testResourceTags = []string{"a=b", "c=", "d=e"}
+var testResourceTagsMap = map[string]string{
+	"a": "b", "c": "", "d": "e",
+}
+
 var configTests = []configTest{
 	{
 		about:       "The minimum good configuration",
@@ -195,6 +202,60 @@ var configTests = []configTest{
 			"lxc-use-clone": false,
 			"lxc-clone":     true,
 		},
+	}, {
+		about:       "Allow LXC loop mounts true",
+		useDefaults: config.UseDefaults,
+		attrs: testing.Attrs{
+			"type":                  "my-type",
+			"name":                  "my-name",
+			"allow-lxc-loop-mounts": "true",
+		},
+	}, {
+		about:       "Allow LXC loop mounts default",
+		useDefaults: config.UseDefaults,
+		attrs: testing.Attrs{
+			"type":                  "my-type",
+			"name":                  "my-name",
+			"allow-lxc-loop-mounts": "false",
+		},
+		expected: testing.Attrs{
+			"type":                  "my-type",
+			"name":                  "my-name",
+			"allow-lxc-loop-mounts": false,
+		},
+	}, {
+		about:       "LXC default MTU not set",
+		useDefaults: config.UseDefaults,
+		attrs: testing.Attrs{
+			"type": "my-type",
+			"name": "my-name",
+		},
+	}, {
+		about:       "LXC default MTU set explicitly",
+		useDefaults: config.UseDefaults,
+		attrs: testing.Attrs{
+			"type":            "my-type",
+			"name":            "my-name",
+			"lxc-default-mtu": 9000,
+		},
+	}, {
+		about:       "LXC default MTU invalid (not a number)",
+		useDefaults: config.UseDefaults,
+		attrs: testing.Attrs{
+			"type":            "my-type",
+			"name":            "my-name",
+			"lxc-default-mtu": "foo",
+		},
+		err: `lxc-default-mtu: expected number, got string\("foo"\)`,
+	}, {
+		about:       "LXC default MTU invalid (negative)",
+		useDefaults: config.UseDefaults,
+		attrs: testing.Attrs{
+			"type":            "my-type",
+			"name":            "my-name",
+			"lxc-default-mtu": -42,
+		},
+		err: `lxc-default-mtu: expected positive integer, got -42`,
 	}, {
 		about:       "CA cert & key from path",
 		useDefaults: config.UseDefaults,
@@ -269,7 +330,7 @@ var configTests = []configTest{
 			"name":         "my-name",
 			"ca-cert-path": "no-such-file",
 		},
-		err: `open .*\.juju/no-such-file: .*`,
+		err: fmt.Sprintf(`open .*\.juju%sno-such-file: .*`, regexp.QuoteMeta(string(os.PathSeparator))),
 	}, {
 		about:       "CA key specified as non-existent file",
 		useDefaults: config.UseDefaults,
@@ -278,7 +339,7 @@ var configTests = []configTest{
 			"name":                "my-name",
 			"ca-private-key-path": "no-such-file",
 		},
-		err: `open .*\.juju/no-such-file: .*`,
+		err: fmt.Sprintf(`open .*\.juju%sno-such-file: .*`, regexp.QuoteMeta(string(os.PathSeparator))),
 	}, {
 		about:       "Specified agent version",
 		useDefaults: config.UseDefaults,
@@ -343,6 +404,31 @@ var configTests = []configTest{
 			"name": "my-name",
 			"disable-network-management": true,
 		},
+	}, {
+		about:       "Invalid ignore-machine-addresses flag",
+		useDefaults: config.UseDefaults,
+		attrs: testing.Attrs{
+			"type": "my-type",
+			"name": "my-name",
+			"ignore-machine-addresses": "invalid",
+		},
+		err: `ignore-machine-addresses: expected bool, got string\("invalid"\)`,
+	}, {
+		about:       "ignore-machine-addresses off",
+		useDefaults: config.UseDefaults,
+		attrs: testing.Attrs{
+			"type": "my-type",
+			"name": "my-name",
+			"ignore-machine-addresses": false,
+		},
+	}, {
+		about:       "ignore-machine-addresses on",
+		useDefaults: config.UseDefaults,
+		attrs: testing.Attrs{
+			"type": "my-type",
+			"name": "my-name",
+			"ignore-machine-addresses": true,
+		},
 	}, {
 		about:       "set-numa-control-policy on",
 		useDefaults: config.UseDefaults,
@@ -850,20 +936,6 @@ var configTests = []configTest{
 		},
 		err: "uuid: expected uuid, got string\\(\"\"\\)",
 	},
-	authTokenConfigTest("token=value, tokensecret=value", true),
-	authTokenConfigTest("token=value, ", true),
-	authTokenConfigTest("token=value, \ttokensecret=value", true),
-	authTokenConfigTest("", true),
-	authTokenConfigTest("token=value, tokensecret=value, \t", true),
-	authTokenConfigTest("=", false),
-	authTokenConfigTest("tokenvalue", false),
-	authTokenConfigTest("token=value, sometoken=", false),
-	authTokenConfigTest("token==value", false),
-	authTokenConfigTest(" token=value", false),
-	authTokenConfigTest("=value", false),
-	authTokenConfigTest("token=value, =z", false),
-	authTokenConfigTest("token=value =z", false),
-	authTokenConfigTest("\t", false),
 	missingAttributeNoDefault("firewall-mode"),
 	missingAttributeNoDefault("development"),
 	missingAttributeNoDefault("ssl-hostname-verification"),
@@ -890,28 +962,34 @@ var configTests = []configTest{
 			"apt-mirror": "http://my.archive.ubuntu.com",
 		},
 	},
-}
-
-// authTokenConfigTest returns a config test that checks
-// that a configuration with the given auth token
-// will pass or fail, depending on the value of ok.
-func authTokenConfigTest(token string, ok bool) configTest {
-	var testName string
-	var err string
-
-	if ok {
-		testName = fmt.Sprintf("Valid auth token test: %q", token)
-	} else {
-		testName = fmt.Sprintf("Invalid auth token test: %q", token)
-		err = fmt.Sprintf("charm store auth token needs to be a set of key-value pairs, not %q", token)
-	}
-
-	return configTest{
-		about:       testName,
+	{
+		about:       "Resource tags as space-separated string",
 		useDefaults: config.UseDefaults,
-		attrs:       sampleConfig.Merge(testing.Attrs{"charm-store-auth": token}),
-		err:         regexp.QuoteMeta(err),
-	}
+		attrs: testing.Attrs{
+			"type":          "my-type",
+			"name":          "my-name",
+			"resource-tags": strings.Join(testResourceTags, " "),
+		},
+	},
+	{
+		about:       "Resource tags as list of strings",
+		useDefaults: config.UseDefaults,
+		attrs: testing.Attrs{
+			"type":          "my-type",
+			"name":          "my-name",
+			"resource-tags": testResourceTags,
+		},
+	},
+	{
+		about:       "Resource tags contains non-keyvalues",
+		useDefaults: config.UseDefaults,
+		attrs: testing.Attrs{
+			"type":          "my-type",
+			"name":          "my-name",
+			"resource-tags": []string{"a"},
+		},
+		err: `validating resource tags: expected "key=value", got "a"`,
+	},
 }
 
 func missingAttributeNoDefault(attrName string) configTest {
@@ -988,7 +1066,7 @@ var emptyCertFilesTests = []configTest{
 			"authorized-keys": testing.FakeAuthKeys,
 			"ca-private-key":  caKey,
 		},
-		err: `file ".*/my-name-cert.pem" is empty`,
+		err: fmt.Sprintf(`file ".*%smy-name-cert.pem" is empty`, regexp.QuoteMeta(string(os.PathSeparator))),
 	}, {
 		about:       "Cert and key unspecified",
 		useDefaults: config.UseDefaults,
@@ -997,7 +1075,7 @@ var emptyCertFilesTests = []configTest{
 			"name":            "my-name",
 			"authorized-keys": testing.FakeAuthKeys,
 		},
-		err: `file ".*/my-name-cert.pem" is empty`,
+		err: fmt.Sprintf(`file ".*%smy-name-cert.pem" is empty`, regexp.QuoteMeta(string(os.PathSeparator))),
 	}, {
 		about:       "Cert specified, key unspecified",
 		useDefaults: config.UseDefaults,
@@ -1007,7 +1085,7 @@ var emptyCertFilesTests = []configTest{
 			"authorized-keys": testing.FakeAuthKeys,
 			"ca-cert":         caCert,
 		},
-		err: `file ".*/my-name-private-key.pem" is empty`,
+		err: fmt.Sprintf(`file ".*%smy-name-private-key.pem" is empty`, regexp.QuoteMeta(string(os.PathSeparator))),
 	}, /* {
 		about: "Cert and key specified as absent",
 		useDefaults: config.UseDefaults,
@@ -1044,6 +1122,21 @@ func (s *ConfigSuite) TestConfigEmptyCertFiles(c *gc.C) {
 	}
 }
 
+func (s *ConfigSuite) TestNoDefinedPrivateCert(c *gc.C) {
+	// Server-side there is no juju home.
+	osenv.SetJujuHome("")
+	attrs := testing.Attrs{
+		"type":            "my-type",
+		"name":            "my-name",
+		"authorized-keys": testing.FakeAuthKeys,
+		"ca-cert":         testing.CACert,
+		"ca-private-key":  "",
+	}
+
+	_, err := config.New(config.UseDefaults, attrs)
+	c.Assert(err, jc.ErrorIsNil)
+}
+
 func (s *ConfigSuite) TestSafeModeDeprecatesGracefully(c *gc.C) {
 
 	cfg, err := config.New(config.UseDefaults, testing.Attrs{
@@ -1279,6 +1372,14 @@ func (test configTest) check(c *gc.C, home *gitjujutesting.FakeHome) {
 	} else {
 		c.Assert(useLxcCloneAufs, jc.IsFalse)
 	}
+
+	resourceTags, cfgHasResourceTags := cfg.ResourceTags()
+	if _, ok := test.attrs["resource-tags"]; ok {
+		c.Assert(cfgHasResourceTags, jc.IsTrue)
+		c.Assert(resourceTags, jc.DeepEquals, testResourceTagsMap)
+	} else {
+		c.Assert(cfgHasResourceTags, jc.IsFalse)
+	}
 }
 
 func (test configTest) assertDuration(c *gc.C, name string, actual time.Duration, defaultInSeconds int) {
@@ -1311,7 +1412,6 @@ func (s *ConfigSuite) TestConfigAttrs(c *gc.C) {
 		"bootstrap-retry-delay":     30,
 		"bootstrap-addresses-delay": 10,
 		"default-series":            testing.FakeDefaultSeries,
-		"charm-store-auth":          "token=auth",
 		"test-mode":                 false,
 	}
 	cfg, err := config.New(config.NoDefaults, attrs)
@@ -1329,9 +1429,7 @@ func (s *ConfigSuite) TestConfigAttrs(c *gc.C) {
 	attrs["lxc-clone-aufs"] = false
 	attrs["prefer-ipv6"] = false
 	attrs["set-numa-control-policy"] = false
-	attrs["block-destroy-environment"] = false
-	attrs["block-remove-object"] = false
-	attrs["block-all-changes"] = false
+	attrs["allow-lxc-loop-mounts"] = false
 
 	// Default firewall mode is instance
 	attrs["firewall-mode"] = string(config.FwInstance)
@@ -1438,6 +1536,11 @@ var validationTests = []validationTest{{
 	old:   testing.Attrs{"lxc-clone-aufs": false},
 	new:   testing.Attrs{"lxc-clone-aufs": true},
 	err:   `cannot change lxc-clone-aufs from false to true`,
+}, {
+	about: "Cannot change lxc-default-mtu",
+	old:   testing.Attrs{"lxc-default-mtu": 9000},
+	new:   testing.Attrs{"lxc-default-mtu": 42},
+	err:   `cannot change lxc-default-mtu from 9000 to 42`,
 }, {
 	about: "Cannot change prefer-ipv6",
 	old:   testing.Attrs{"prefer-ipv6": false},
@@ -1771,6 +1874,50 @@ func (s *ConfigSuite) TestGenerateStateServerCertAndKey(c *gc.C) {
 	}
 }
 
+var specializeCharmRepoTests = []struct {
+	about    string
+	testMode bool
+	repo     charmrepo.Interface
+}{{
+	about: "test mode disabled, charm store",
+	repo:  &specializedCharmRepo{},
+}, {
+	about: "test mode disabled, local repo",
+	repo:  &charmrepo.LocalRepository{},
+}, {
+	about:    "test mode enabled, charm store",
+	testMode: true,
+	repo:     &specializedCharmRepo{},
+}, {
+	about:    "test mode enabled, local repo",
+	testMode: true,
+	repo:     &charmrepo.LocalRepository{},
+}}
+
+func (s *ConfigSuite) TestSpecializeCharmRepo(c *gc.C) {
+	for i, test := range specializeCharmRepoTests {
+		c.Logf("test %d: %s", i, test.about)
+		cfg := newTestConfig(c, testing.Attrs{"test-mode": test.testMode})
+		repo := config.SpecializeCharmRepo(test.repo, cfg)
+		if store, ok := repo.(*specializedCharmRepo); ok {
+			c.Assert(store.testMode, gc.Equals, test.testMode)
+			continue
+		}
+		// Just check that the original local repo has not been modified.
+		c.Assert(repo.(*charmrepo.LocalRepository), gc.Equals, test.repo)
+	}
+}
+
+type specializedCharmRepo struct {
+	*charmrepo.CharmStore
+	testMode bool
+}
+
+func (s *specializedCharmRepo) WithTestMode() charmrepo.Interface {
+	s.testMode = true
+	return s
+}
+
 func (s *ConfigSuite) TestLastestLtsSeriesFallback(c *gc.C) {
 	config.ResetCachedLtsSeries()
 	s.PatchValue(config.DistroLtsSeries, func() (string, error) {
