@@ -8,10 +8,13 @@ import (
 	"io/ioutil"
 	"os"
 	"path/filepath"
+	"runtime"
 	"strings"
 
 	"github.com/juju/errors"
+	"github.com/juju/loggo"
 	jc "github.com/juju/testing/checkers"
+	"github.com/juju/utils/fslock"
 	gc "gopkg.in/check.v1"
 
 	"github.com/juju/juju/environs/configstore"
@@ -114,7 +117,7 @@ func (*diskStoreSuite) TestRead(c *gc.C) {
 		Hostnames: []string{"example.com", "kremvax.ru"},
 		CACert:    "first line\nsecond line",
 	})
-	c.Assert(info.Location(), gc.Equals, fmt.Sprintf("file %q", dir+"/environments/someenv.jenv"))
+	c.Assert(info.Location(), gc.Equals, fmt.Sprintf("file %q", filepath.Join(dir, "environments", "someenv.jenv")))
 	c.Assert(info.BootstrapConfig(), gc.DeepEquals, map[string]interface{}{
 		"secret": "blah",
 		"arble":  "bletch",
@@ -141,8 +144,11 @@ func (*diskStoreSuite) TestWriteFails(c *gc.C) {
 	err = os.Chmod(storePath(dir, ""), 0555)
 	c.Assert(err, jc.ErrorIsNil)
 
-	err = info.Write()
-	c.Assert(err, gc.ErrorMatches, ".* permission denied")
+	// Cannot use permissions properly on windows for now
+	if runtime.GOOS != "windows" {
+		err = info.Write()
+		c.Assert(err, gc.ErrorMatches, ".* permission denied")
+	}
 
 	// Make the directory writable again so that gocheck can clean it up.
 	err = os.Chmod(storePath(dir, ""), 0777)
@@ -150,6 +156,9 @@ func (*diskStoreSuite) TestWriteFails(c *gc.C) {
 }
 
 func (*diskStoreSuite) TestRenameFails(c *gc.C) {
+	if runtime.GOOS == "windows" {
+		c.Skip("issue 1403084: the way the error is checked doesn't work on windows")
+	}
 	dir := c.MkDir()
 	store, err := configstore.NewDisk(dir)
 	c.Assert(err, jc.ErrorIsNil)
@@ -194,7 +203,7 @@ func (*diskStoreSuite) TestWriteSmallerFile(c *gc.C) {
 	endpoint := configstore.APIEndpoint{
 		Addresses:   []string{"this", "is", "never", "validated", "here"},
 		Hostnames:   []string{"neither", "is", "this"},
-		EnvironUUID: "90168e4c-2f10-4e9c-83c2-feedfacee5a9",
+		EnvironUUID: testing.EnvironmentTag.Id(),
 	}
 	info.SetAPIEndpoint(endpoint)
 	err = info.Write()
@@ -215,3 +224,29 @@ func (*diskStoreSuite) TestWriteSmallerFile(c *gc.C) {
 	c.Assert(yaInfo.APIEndpoint().Addresses, gc.DeepEquals, []string{"just one"})
 	c.Assert(yaInfo.APIEndpoint().Hostnames, gc.DeepEquals, []string{"just this"})
 }
+
+func (*diskStoreSuite) TestConcurrentAccess(c *gc.C) {
+	var tw loggo.TestWriter
+	c.Assert(loggo.RegisterWriter("test-log", &tw, loggo.DEBUG), gc.IsNil)
+
+	dir := c.MkDir()
+	store, err := configstore.NewDisk(dir)
+	c.Assert(err, jc.ErrorIsNil)
+
+	envDir := storePath(dir, "")
+	lock, err := configstore.AcquireEnvironmentLock(envDir, "blocking-op")
+	c.Assert(err, jc.ErrorIsNil)
+	defer lock.Unlock()
+
+	_, err = store.ReadInfo("someenv")
+	c.Assert(errors.Cause(err), gc.Equals, fslock.ErrTimeout)
+
+	// Using . between environments and env.lock so we don't have to care
+	// about forward vs. backwards slash separator.
+	messages := []jc.SimpleMessage{
+		{loggo.WARNING, `configstore lock held, lock dir: .*environments.env\.lock`},
+		{loggo.WARNING, `lock holder message: pid: \d+, operation: blocking-op`},
+	}
+
+	c.Check(tw.Log(), jc.LogMatches, messages)
+}
