@@ -5,7 +5,7 @@ package arch_test
 
 import (
 	jc "github.com/juju/testing/checkers"
-	gc "launchpad.net/gocheck"
+	gc "gopkg.in/check.v1"
 
 	"github.com/juju/juju/juju/arch"
 	"github.com/juju/juju/testing"
