@@ -9,12 +9,13 @@ import (
 
 	"github.com/juju/errors"
 	"github.com/juju/names"
-	"gopkg.in/juju/charm.v4"
+	"gopkg.in/juju/charm.v5"
 
 	"github.com/juju/juju/constraints"
 	"github.com/juju/juju/environs"
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/state"
+	"github.com/juju/juju/storage"
 )
 
 // DeployServiceParams contains the arguments required to deploy the referenced
@@ -33,6 +34,7 @@ type DeployServiceParams struct {
 	ToMachineSpec string
 	// Networks holds a list of networks to required to start on boot.
 	Networks []string
+	Storage  map[string]storage.Constraints
 }
 
 // DeployService takes a charm and various parameters and deploys it.
@@ -70,7 +72,7 @@ func DeployService(st *state.State, args DeployServiceParams) (*state.Service, e
 		if err != nil {
 			return nil, err
 		}
-		if !env.SupportNetworks() {
+		if _, ok := environs.SupportsNetworking(env); !ok {
 			return nil, fmt.Errorf("cannot deploy with networks: not suppored by the environment")
 		}
 	}
@@ -79,15 +81,12 @@ func DeployService(st *state.State, args DeployServiceParams) (*state.Service, e
 		args.ServiceOwner,
 		args.Charm,
 		args.Networks,
+		stateStorageConstraints(args.Storage),
+		settings,
 	)
 	if err != nil {
 		return nil, err
 	}
-	if len(settings) > 0 {
-		if err := service.UpdateConfigSettings(settings); err != nil {
-			return nil, err
-		}
-	}
 	if args.Charm.Meta().Subordinate {
 		return service, nil
 	}
@@ -180,3 +179,15 @@ func AddUnits(st *state.State, svc *state.Service, n int, machineIdSpec string)
 	}
 	return units, nil
 }
+
+func stateStorageConstraints(cons map[string]storage.Constraints) map[string]state.StorageConstraints {
+	result := make(map[string]state.StorageConstraints)
+	for name, cons := range cons {
+		result[name] = state.StorageConstraints{
+			Pool:  cons.Pool,
+			Size:  cons.Size,
+			Count: cons.Count,
+		}
+	}
+	return result
+}
