@@ -17,7 +17,8 @@ import (
 	jc "github.com/juju/testing/checkers"
 	"github.com/juju/utils"
 	gc "gopkg.in/check.v1"
-	"gopkg.in/juju/charm.v4"
+	"gopkg.in/juju/charm.v5"
+	"gopkg.in/juju/charm.v5/charmrepo"
 	goyaml "gopkg.in/yaml.v1"
 
 	"github.com/juju/juju/agent"
@@ -63,8 +64,8 @@ type JujuConnSuite struct {
 	// /var/lib/juju: the use cases are completely non-overlapping, and any tests that
 	// really do need both to exist ought to be embedding distinct fixtures for the
 	// distinct environments.
-	testing.FakeJujuHomeSuite
 	gitjujutesting.MgoSuite
+	testing.FakeJujuHomeSuite
 	envtesting.ToolsFixture
 
 	DefaultToolsStorageDir string
@@ -87,18 +88,18 @@ type JujuConnSuite struct {
 const AdminSecret = "dummy-secret"
 
 func (s *JujuConnSuite) SetUpSuite(c *gc.C) {
-	s.FakeJujuHomeSuite.SetUpSuite(c)
 	s.MgoSuite.SetUpSuite(c)
+	s.FakeJujuHomeSuite.SetUpSuite(c)
 }
 
 func (s *JujuConnSuite) TearDownSuite(c *gc.C) {
-	s.MgoSuite.TearDownSuite(c)
 	s.FakeJujuHomeSuite.TearDownSuite(c)
+	s.MgoSuite.TearDownSuite(c)
 }
 
 func (s *JujuConnSuite) SetUpTest(c *gc.C) {
-	s.FakeJujuHomeSuite.SetUpTest(c)
 	s.MgoSuite.SetUpTest(c)
+	s.FakeJujuHomeSuite.SetUpTest(c)
 	s.ToolsFixture.SetUpTest(c)
 	s.PatchValue(&configstore.DefaultAdminUsername, dummy.AdminUserTag().Name())
 	s.setUpConn(c)
@@ -108,8 +109,8 @@ func (s *JujuConnSuite) SetUpTest(c *gc.C) {
 func (s *JujuConnSuite) TearDownTest(c *gc.C) {
 	s.tearDownConn(c)
 	s.ToolsFixture.TearDownTest(c)
-	s.MgoSuite.TearDownTest(c)
 	s.FakeJujuHomeSuite.TearDownTest(c)
+	s.MgoSuite.TearDownTest(c)
 }
 
 // Reset returns environment state to that which existed at the start of
@@ -136,11 +137,7 @@ func (s *JujuConnSuite) APIInfo(c *gc.C) *api.Info {
 	c.Assert(err, jc.ErrorIsNil)
 	apiInfo.Tag = s.AdminUserTag(c)
 	apiInfo.Password = "dummy-secret"
-
-	env, err := s.State.Environment()
-	c.Assert(err, jc.ErrorIsNil)
-	apiInfo.EnvironTag = env.EnvironTag()
-
+	apiInfo.EnvironTag = s.State.EnvironTag()
 	return apiInfo
 }
 
@@ -267,6 +264,18 @@ func (s *JujuConnSuite) setUpConn(c *gc.C) {
 	err = s.State.SetAPIHostPorts(s.APIState.APIHostPorts())
 	c.Assert(err, jc.ErrorIsNil)
 
+	// Make sure the config store has the api endpoint address set
+	info, err := s.ConfigStore.ReadInfo("dummyenv")
+	c.Assert(err, jc.ErrorIsNil)
+	endpoint := info.APIEndpoint()
+	endpoint.Addresses = []string{s.APIState.APIHostPorts()[0][0].String()}
+	info.SetAPIEndpoint(endpoint)
+	err = info.Write()
+	c.Assert(err, jc.ErrorIsNil)
+
+	// Make sure the jenv file has the local host ports.
+	c.Logf("jenv host ports: %#v", s.APIState.APIHostPorts())
+
 	s.Environ = environ
 
 	// Insert expected values...
@@ -319,21 +328,27 @@ var redialStrategy = utils.AttemptStrategy{
 // newState returns a new State that uses the given environment.
 // The environment must have already been bootstrapped.
 func newState(environ environs.Environ, mongoInfo *mongo.MongoInfo) (*state.State, error) {
-	password := environ.Config().AdminSecret()
+	config := environ.Config()
+	password := config.AdminSecret()
 	if password == "" {
 		return nil, fmt.Errorf("cannot connect without admin-secret")
 	}
+	environUUID, ok := config.UUID()
+	if !ok {
+		return nil, fmt.Errorf("cannot connect without environment UUID")
+	}
+	environTag := names.NewEnvironTag(environUUID)
 
 	mongoInfo.Password = password
 	opts := mongo.DefaultDialOpts()
-	st, err := state.Open(mongoInfo, opts, environs.NewStatePolicy())
-	if errors.IsUnauthorized(err) {
+	st, err := state.Open(environTag, mongoInfo, opts, environs.NewStatePolicy())
+	if errors.IsUnauthorized(errors.Cause(err)) {
 		// We try for a while because we might succeed in
 		// connecting to mongo before the state has been
 		// initialized and the initial password set.
 		for a := redialStrategy.Start(); a.Next(); {
-			st, err = state.Open(mongoInfo, opts, environs.NewStatePolicy())
-			if !errors.IsUnauthorized(err) {
+			st, err = state.Open(environTag, mongoInfo, opts, environs.NewStatePolicy())
+			if !errors.IsUnauthorized(errors.Cause(err)) {
 				break
 			}
 		}
@@ -377,9 +392,9 @@ func updateSecrets(env environs.Environ, st *state.State) error {
 // the same URL already exists in the state.
 // If bumpRevision is true, the charm must be a local directory,
 // and the revision number will be incremented before pushing.
-func PutCharm(st *state.State, curl *charm.URL, repo charm.Repository, bumpRevision bool) (*state.Charm, error) {
+func PutCharm(st *state.State, curl *charm.URL, repo charmrepo.Interface, bumpRevision bool) (*state.Charm, error) {
 	if curl.Revision == -1 {
-		rev, err := charm.Latest(repo, curl)
+		rev, err := charmrepo.Latest(repo, curl)
 		if err != nil {
 			return nil, fmt.Errorf("cannot get latest charm revision: %v", err)
 		}
@@ -522,6 +537,13 @@ func (s *JujuConnSuite) DataDir() string {
 	return filepath.Join(s.RootDir, "/var/lib/juju")
 }
 
+func (s *JujuConnSuite) ConfDir() string {
+	if s.RootDir == "" {
+		panic("DataDir called out of test context")
+	}
+	return filepath.Join(s.RootDir, "/etc/juju")
+}
+
 // WriteConfig writes a juju config file to the "home" directory.
 func (s *JujuConnSuite) WriteConfig(configData string) {
 	if s.RootDir == "" {
@@ -538,7 +560,10 @@ func (s *JujuConnSuite) AddTestingCharm(c *gc.C, name string) *state.Charm {
 	ch := testcharms.Repo.CharmDir(name)
 	ident := fmt.Sprintf("%s-%d", ch.Meta().Name, ch.Revision())
 	curl := charm.MustParseURL("local:quantal/" + ident)
-	repo, err := charm.InferRepository(curl.Reference(), testcharms.Repo.Path())
+	repo, err := charmrepo.InferRepository(
+		curl.Reference(),
+		charmrepo.NewCharmStoreParams{},
+		testcharms.Repo.Path())
 	c.Assert(err, jc.ErrorIsNil)
 	sch, err := PutCharm(s.State, curl, repo, false)
 	c.Assert(err, jc.ErrorIsNil)
@@ -549,10 +574,17 @@ func (s *JujuConnSuite) AddTestingService(c *gc.C, name string, ch *state.Charm)
 	return s.AddTestingServiceWithNetworks(c, name, ch, nil)
 }
 
+func (s *JujuConnSuite) AddTestingServiceWithStorage(c *gc.C, name string, ch *state.Charm, storage map[string]state.StorageConstraints) *state.Service {
+	owner := s.AdminUserTag(c).String()
+	service, err := s.State.AddService(name, owner, ch, nil, storage, nil)
+	c.Assert(err, jc.ErrorIsNil)
+	return service
+}
+
 func (s *JujuConnSuite) AddTestingServiceWithNetworks(c *gc.C, name string, ch *state.Charm, networks []string) *state.Service {
 	c.Assert(s.State, gc.NotNil)
 	owner := s.AdminUserTag(c).String()
-	service, err := s.State.AddService(name, owner, ch, networks)
+	service, err := s.State.AddService(name, owner, ch, networks, nil, nil)
 	c.Assert(err, jc.ErrorIsNil)
 	return service
 }
@@ -570,6 +602,7 @@ func (s *JujuConnSuite) AgentConfigForTag(c *gc.C, tag names.Tag) agent.ConfigSe
 			StateAddresses:    s.MongoInfo(c).Addrs,
 			APIAddresses:      s.APIInfo(c).Addrs,
 			CACert:            testing.CACert,
+			Environment:       s.State.EnvironTag(),
 		})
 	c.Assert(err, jc.ErrorIsNil)
 	return config
