@@ -10,6 +10,7 @@ import (
 	gc "gopkg.in/check.v1"
 
 	"github.com/juju/juju/api"
+	"github.com/juju/juju/cloudconfig/instancecfg"
 	"github.com/juju/juju/constraints"
 	"github.com/juju/juju/environs"
 	"github.com/juju/juju/environs/config"
@@ -41,10 +42,11 @@ func FakeStateInfo(machineId string) *mongo.MongoInfo {
 // of the machine to be started.
 func FakeAPIInfo(machineId string) *api.Info {
 	return &api.Info{
-		Addrs:    []string{"0.1.2.3:1234"},
-		Tag:      names.NewMachineTag(machineId),
-		Password: "unimportant",
-		CACert:   testing.CACert,
+		Addrs:      []string{"0.1.2.3:1234"},
+		Tag:        names.NewMachineTag(machineId),
+		Password:   "unimportant",
+		CACert:     testing.CACert,
+		EnvironTag: testing.EnvironmentTag,
 	}
 }
 
@@ -83,7 +85,7 @@ func AssertStartInstance(
 func StartInstance(
 	env environs.Environ, machineId string,
 ) (
-	instance.Instance, *instance.HardwareCharacteristics, []network.Info, error,
+	instance.Instance, *instance.HardwareCharacteristics, []network.InterfaceInfo, error,
 ) {
 	return StartInstanceWithConstraints(env, machineId, constraints.Value{})
 }
@@ -107,7 +109,7 @@ func AssertStartInstanceWithConstraints(
 func StartInstanceWithConstraints(
 	env environs.Environ, machineId string, cons constraints.Value,
 ) (
-	instance.Instance, *instance.HardwareCharacteristics, []network.Info, error,
+	instance.Instance, *instance.HardwareCharacteristics, []network.InterfaceInfo, error,
 ) {
 	return StartInstanceWithConstraintsAndNetworks(env, machineId, cons, nil)
 }
@@ -134,7 +136,7 @@ func StartInstanceWithConstraintsAndNetworks(
 	env environs.Environ, machineId string, cons constraints.Value,
 	networks []string,
 ) (
-	instance.Instance, *instance.HardwareCharacteristics, []network.Info, error,
+	instance.Instance, *instance.HardwareCharacteristics, []network.InterfaceInfo, error,
 ) {
 	params := environs.StartInstanceParams{Constraints: cons}
 	result, err := StartInstanceWithParams(env, machineId, params, networks)
@@ -147,7 +149,7 @@ func StartInstanceWithConstraintsAndNetworks(
 // StartInstanceWithParams is a test helper function that starts an instance
 // with the given parameters, and a plausible but invalid configuration, and
 // returns the result of Environ.StartInstance. The provided params's
-// MachineConfig and Tools field values will be ignored.
+// InstanceConfig and Tools field values will be ignored.
 func StartInstanceWithParams(
 	env environs.Environ, machineId string,
 	params environs.StartInstanceParams,
@@ -174,7 +176,7 @@ func StartInstanceWithParams(
 	machineNonce := "fake_nonce"
 	stateInfo := FakeStateInfo(machineId)
 	apiInfo := FakeAPIInfo(machineId)
-	machineConfig, err := environs.NewMachineConfig(
+	instanceConfig, err := instancecfg.NewInstanceConfig(
 		machineId,
 		machineNonce,
 		imagemetadata.ReleasedStream,
@@ -188,6 +190,6 @@ func StartInstanceWithParams(
 		return nil, errors.Trace(err)
 	}
 	params.Tools = possibleTools
-	params.MachineConfig = machineConfig
+	params.InstanceConfig = instanceConfig
 	return env.StartInstance(params)
 }
