@@ -4,7 +4,6 @@
 package mongo
 
 import (
-	"bytes"
 	"crypto/rand"
 	"encoding/base64"
 	"fmt"
@@ -13,67 +12,31 @@ import (
 	"os/exec"
 	"path"
 	"path/filepath"
-	"strconv"
+	"strings"
 
 	"github.com/juju/errors"
 	"github.com/juju/loggo"
+	"github.com/juju/replicaset"
 	"github.com/juju/utils"
-	"github.com/juju/utils/apt"
+	"github.com/juju/utils/packaging/config"
+	"github.com/juju/utils/packaging/manager"
 	"gopkg.in/mgo.v2"
 
 	"github.com/juju/juju/network"
-	"github.com/juju/juju/replicaset"
-	"github.com/juju/juju/service/common"
-	"github.com/juju/juju/service/upstart"
+	"github.com/juju/juju/service"
 	"github.com/juju/juju/version"
 )
 
-const (
-	maxFiles = 65000
-	maxProcs = 20000
-
-	serviceName = "juju-db"
-
-	// SharedSecretFile is the name of the Mongo shared secret file
-	// located within the Juju data directory.
-	SharedSecretFile = "shared-secret"
-
-	// ReplicaSetName is the name of the replica set that juju uses for its
-	// state servers.
-	ReplicaSetName = "juju"
-)
-
 var (
 	logger          = loggo.GetLogger("juju.mongo")
 	mongoConfigPath = "/etc/default/mongodb"
 
-	// JujuMongodPath holds the default path to the juju-specific mongod.
+	// JujuMongodPath holds the default path to the juju-specific
+	// mongod.
 	JujuMongodPath = "/usr/lib/juju/bin/mongod"
 
-	upstartConfInstall          = (*upstart.Service).Install
-	upstartServiceExists        = (*upstart.Service).Exists
-	upstartServiceRunning       = (*upstart.Service).Running
-	upstartServiceStopAndRemove = (*upstart.Service).StopAndRemove
-	upstartServiceStop          = (*upstart.Service).Stop
-	upstartServiceStart         = (*upstart.Service).Start
-
 	// This is NUMACTL package name for apt-get
 	numaCtlPkg = "numactl"
-	// This is the name of the variable to use in ExtraScript
-	// fragment to substitu into upstart template.
-	multinodeVarName = "MULTI_NODE"
-	// This value will be used to wrap desired mongo cmd in numactl if wanted/needed
-	numaCtlWrap = "$%v"
-	// Extra shell script fragment for upstart template.
-	// This determines if we are dealing with multi-node environment
-	detectMultiNodeScript = `%v=""
-if [ $(find /sys/devices/system/node/ -maxdepth 1 -mindepth 1 -type d -name node\* | wc -l ) -gt 1 ]
-then
-    %v=" numactl --interleave=all "
-    # Ensure sysctl turns off zone_reclaim_mode if not already set
-    (grep -q vm.zone_reclaim_mode /etc/sysctl.conf || echo vm.zone_reclaim_mode = 0 >> /etc/sysctl.conf) && sysctl -p
-fi
-`
 )
 
 // WithAddresses represents an entity that has a set of
@@ -153,12 +116,6 @@ func Path() (string, error) {
 	return path, nil
 }
 
-// RemoveService removes the mongoDB upstart service from this machine.
-func RemoveService(namespace string) error {
-	svc := upstart.NewService(ServiceName(namespace), common.Conf{})
-	return upstartServiceStopAndRemove(svc)
-}
-
 // EnsureServerParams is a parameter struct for EnsureServer.
 type EnsureServerParams struct {
 	// APIPort is the port to connect to the api server.
@@ -200,10 +157,10 @@ type EnsureServerParams struct {
 	SetNumaControlPolicy bool
 }
 
-// EnsureServer ensures that the correct mongo upstart script is installed
-// and running.
+// EnsureServer ensures that the MongoDB server is installed,
+// configured, and ready to run.
 //
-// This method will remove old versions of the mongo upstart script as necessary
+// This method will remove old versions of the mongo init service as necessary
 // before installing the new version.
 //
 // The namespace is a unique identifier to prevent multiple instances of mongo
@@ -228,8 +185,13 @@ func EnsureServer(args EnsureServerParams) error {
 		}
 	}
 
-	if err := aptGetInstallMongod(args.SetNumaControlPolicy); err != nil {
-		return fmt.Errorf("cannot install mongod: %v", err)
+	if err := installMongod(args.SetNumaControlPolicy); err != nil {
+		// This isn't treated as fatal because the Juju MongoDB
+		// package is likely to be already installed anyway. There
+		// could just be a temporary issue with apt-get/yum/whatever
+		// and we don't want this to stop jujud from starting.
+		// (LP #1441904)
+		logger.Errorf("cannot install/upgrade mongod (will proceed anyway): %v", err)
 	}
 	mongoPath, err := Path()
 	if err != nil {
@@ -237,18 +199,6 @@ func EnsureServer(args EnsureServerParams) error {
 	}
 	logVersion(mongoPath)
 
-	svc, err := upstartService(args.Namespace, args.DataDir, dbDir, mongoPath, args.StatePort, oplogSizeMB, args.SetNumaControlPolicy)
-	if err != nil {
-		return err
-	}
-	if upstartServiceExists(svc) {
-		logger.Debugf("mongo exists as expected")
-		if !upstartServiceRunning(svc) {
-			return upstartServiceStart(svc)
-		}
-		return nil
-	}
-
 	if err := UpdateSSLKey(args.DataDir, args.Cert, args.PrivateKey); err != nil {
 		return err
 	}
@@ -272,8 +222,35 @@ func EnsureServer(args EnsureServerParams) error {
 		}
 	}
 
-	if err := upstartServiceStop(svc); err != nil {
-		return fmt.Errorf("failed to stop mongo: %v", err)
+	svcConf := newConf(args.DataDir, dbDir, mongoPath, args.StatePort, oplogSizeMB, args.SetNumaControlPolicy)
+	svc, err := newService(ServiceName(args.Namespace), svcConf)
+	if err != nil {
+		return err
+	}
+	installed, err := svc.Installed()
+	if err != nil {
+		return errors.Trace(err)
+	}
+	if installed {
+		exists, err := svc.Exists()
+		if err != nil {
+			return errors.Trace(err)
+		}
+		if exists {
+			logger.Debugf("mongo exists as expected")
+			running, err := svc.Running()
+			if err != nil {
+				return errors.Trace(err)
+			}
+			if !running {
+				return svc.Start()
+			}
+			return nil
+		}
+	}
+
+	if err := svc.Stop(); err != nil {
+		return errors.Annotatef(err, "failed to stop mongo")
 	}
 	if err := makeJournalDirs(dbDir); err != nil {
 		return fmt.Errorf("error creating journal directories: %v", err)
@@ -281,16 +258,10 @@ func EnsureServer(args EnsureServerParams) error {
 	if err := preallocOplog(dbDir, oplogSizeMB); err != nil {
 		return fmt.Errorf("error creating oplog files: %v", err)
 	}
-	return upstartConfInstall(svc)
-}
-
-// ServiceName returns the name of the upstart service config for mongo using
-// the given namespace.
-func ServiceName(namespace string) string {
-	if namespace != "" {
-		return fmt.Sprintf("%s-%s", serviceName, namespace)
+	if err := service.InstallAndStart(svc); err != nil {
+		return errors.Trace(err)
 	}
-	return serviceName
+	return nil
 }
 
 // UpdateSSLKey writes a new SSL key used by mongo to validate connections from Juju state server(s)
@@ -324,94 +295,91 @@ func logVersion(mongoPath string) {
 	logger.Debugf("using mongod: %s --version: %q", mongoPath, output)
 }
 
-func sslKeyPath(dataDir string) string {
-	return filepath.Join(dataDir, "server.pem")
+// getPackageManager is a helper function which returns the
+// package manager implementation for the current system.
+func getPackageManager() (manager.PackageManager, error) {
+	return manager.NewPackageManager(version.Current.Series)
 }
 
-func sharedSecretPath(dataDir string) string {
-	return filepath.Join(dataDir, SharedSecretFile)
+// getPackagingConfigurer is a helper function which returns the
+// packaging configuration manager for the current system.
+func getPackagingConfigurer() (config.PackagingConfigurer, error) {
+	return config.NewPackagingConfigurer(version.Current.Series)
 }
 
-// upstartService returns the upstart config for the mongo state service.
-// It also returns the path to the mongod executable that the upstart config
-// will be using.
-func upstartService(namespace, dataDir, dbDir, mongoPath string, port, oplogSizeMB int, wantNumaCtl bool) (*upstart.Service, error) {
-	mongoCmd := mongoPath + " --auth" +
-		" --dbpath=" + utils.ShQuote(dbDir) +
-		" --sslOnNormalPorts" +
-		" --sslPEMKeyFile " + utils.ShQuote(sslKeyPath(dataDir)) +
-		" --sslPEMKeyPassword ignored" +
-		" --port " + fmt.Sprint(port) +
-		" --noprealloc" +
-		" --syslog" +
-		" --smallfiles" +
-		" --journal" +
-		" --keyFile " + utils.ShQuote(sharedSecretPath(dataDir)) +
-		" --replSet " + ReplicaSetName +
-		" --ipv6 " +
-		" --oplogSize " + strconv.Itoa(oplogSizeMB)
-	extraScript := ""
-	if wantNumaCtl {
-		extraScript = fmt.Sprintf(detectMultiNodeScript, multinodeVarName, multinodeVarName)
-		mongoCmd = fmt.Sprintf(numaCtlWrap, multinodeVarName) + mongoCmd
+func installMongod(numaCtl bool) error {
+	series := version.Current.Series
+
+	pacconfer, err := getPackagingConfigurer()
+	if err != nil {
+		return err
+	}
+
+	pacman, err := getPackageManager()
+	if err != nil {
+		return err
 	}
-	conf := common.Conf{
-		Desc: "juju state database",
-		Limit: map[string]string{
-			"nofile": fmt.Sprintf("%d %d", maxFiles, maxFiles),
-			"nproc":  fmt.Sprintf("%d %d", maxProcs, maxProcs),
-		},
-		ExtraScript: extraScript,
-		Cmd:         mongoCmd,
-	}
-	svc := upstart.NewService(ServiceName(namespace), conf)
-	return svc, nil
-}
 
-func aptGetInstallMongod(numaCtl bool) error {
 	// Only Quantal requires the PPA.
-	if version.Current.Series == "quantal" {
-		if err := addAptRepository("ppa:juju/stable"); err != nil {
+	if series == "quantal" {
+		// install python-software-properties:
+		if err := pacman.InstallPrerequisite(); err != nil {
+			return err
+		}
+		if err := pacman.AddRepository("ppa:juju/stable"); err != nil {
+			return err
+		}
+	}
+	// CentOS requires "epel-release" for the epel repo mongodb-server is in.
+	if series == "centos7" {
+		// install epel-release
+		if err := pacman.Install("epel-release"); err != nil {
 			return err
 		}
 	}
-	mongoPkg := packageForSeries(version.Current.Series)
 
-	aptPkgs := []string{mongoPkg}
+	mongoPkg := packageForSeries(series)
+
+	pkgs := []string{mongoPkg}
 	if numaCtl {
-		aptPkgs = []string{mongoPkg, numaCtlPkg}
+		pkgs = []string{mongoPkg, numaCtlPkg}
 		logger.Infof("installing %s and %s", mongoPkg, numaCtlPkg)
 	} else {
 		logger.Infof("installing %s", mongoPkg)
 	}
-	cmds := apt.GetPreparePackages(aptPkgs, version.Current.Series)
-	for _, cmd := range cmds {
-		if err := apt.GetInstall(cmd...); err != nil {
-			return err
-		}
-	}
-	return nil
-}
 
-func addAptRepository(name string) error {
-	// add-apt-repository requires python-software-properties
-	cmds := apt.GetPreparePackages(
-		[]string{"python-software-properties"},
-		version.Current.Series,
-	)
-	logger.Infof("installing python-software-properties")
-	for _, cmd := range cmds {
-		if err := apt.GetInstall(cmd...); err != nil {
+	for i, _ := range pkgs {
+		// apply release targeting if needed.
+		if pacconfer.IsCloudArchivePackage(pkgs[i]) {
+			pkgs[i] = strings.Join(pacconfer.ApplyCloudArchiveTarget(pkgs[i]), " ")
+		}
+
+		if err := pacman.Install(pkgs[i]); err != nil {
 			return err
 		}
 	}
 
-	logger.Infof("adding apt repository %q", name)
-	cmd := exec.Command("add-apt-repository", "-y", name)
-	out, err := cmd.CombinedOutput()
-	if err != nil {
-		return fmt.Errorf("cannot add apt repository: %v (output %s)", err, bytes.TrimSpace(out))
+	// Work around SELinux on centos7
+	if series == "centos7" {
+		cmd := []string{"chcon", "-R", "-v", "-t", "mongod_var_lib_t", "/var/lib/juju/"}
+		logger.Infof("running %s %v", cmd[0], cmd[1:])
+		_, err = utils.RunCommand(cmd[0], cmd[1:]...)
+		if err != nil {
+			logger.Infof("chcon error %s", err)
+			logger.Infof("chcon error %s", err.Error())
+			return err
+		}
+
+		cmd = []string{"semanage", "port", "-a", "-t", "mongod_port_t", "-p", "tcp", "37017"}
+		logger.Infof("running %s %v", cmd[0], cmd[1:])
+		_, err = utils.RunCommand(cmd[0], cmd[1:]...)
+		if err != nil {
+			if !strings.Contains(err.Error(), "exit status 1") {
+				return err
+			}
+		}
 	}
+
 	return nil
 }
 
@@ -419,7 +387,7 @@ func addAptRepository(name string) error {
 // of the machine that it is going to be running on.
 func packageForSeries(series string) string {
 	switch series {
-	case "precise", "quantal", "raring", "saucy":
+	case "precise", "quantal", "raring", "saucy", "centos7":
 		return "mongodb-server"
 	default:
 		// trusty and onwards
