@@ -29,6 +29,13 @@ var (
 // bootstrap, agent startup, before any CLI command).
 var globalPreferIPv6 bool = false
 
+// ResetGobalPreferIPv6 resets the global variable back to the default,
+// and is called only from the isolation test suite to make sure we have
+// a clean environment.
+func ResetGobalPreferIPv6() {
+	globalPreferIPv6 = false
+}
+
 func mustParseCIDR(s string) *net.IPNet {
 	_, net, err := net.ParseCIDR(s)
 	if err != nil {
@@ -100,10 +107,37 @@ func (a Address) GoString() string {
 	return a.String()
 }
 
-// NewAddresses is a convenience function to create addresses from a string slice
+// NewAddress creates a new Address, deriving its type from the value
+// and using ScopeUnknown as scope. It's a shortcut to calling
+// NewScopedAddress(value, ScopeUnknown).
+func NewAddress(value string) Address {
+	return NewScopedAddress(value, ScopeUnknown)
+}
+
+// NewScopedAddress creates a new Address, deriving its type from the
+// value.
+//
+// If the specified scope is ScopeUnknown, then NewScopedAddress will
+// attempt derive the scope based on reserved IP address ranges.
+// Because passing ScopeUnknown is fairly common, NewAddress() above
+// does exactly that.
+func NewScopedAddress(value string, scope Scope) Address {
+	addr := Address{
+		Value: value,
+		Type:  DeriveAddressType(value),
+		Scope: scope,
+	}
+	if scope == ScopeUnknown {
+		addr.Scope = deriveScope(addr)
+	}
+	return addr
+}
+
+// NewAddresses is a convenience function to create addresses from a
+// string slice.
 func NewAddresses(inAddresses ...string) (outAddresses []Address) {
 	for _, address := range inAddresses {
-		outAddresses = append(outAddresses, NewAddress(address, ScopeUnknown))
+		outAddresses = append(outAddresses, NewAddress(address))
 	}
 	return outAddresses
 }
@@ -169,22 +203,6 @@ func deriveScope(addr Address) Scope {
 	return addr.Scope
 }
 
-// NewAddress creates a new Address, deriving its type from the value.
-//
-// If the specified scope is ScopeUnknown, then NewAddress will
-// attempt derive the scope based on reserved IP address ranges.
-func NewAddress(value string, scope Scope) Address {
-	addr := Address{
-		Value: value,
-		Type:  DeriveAddressType(value),
-		Scope: scope,
-	}
-	if scope == ScopeUnknown {
-		addr.Scope = deriveScope(addr)
-	}
-	return addr
-}
-
 // SelectPublicAddress picks one address from a slice that would be
 // appropriate to display as a publicly accessible endpoint. If there
 // are no suitable addresses, the empty string is returned.
