@@ -11,10 +11,9 @@ import (
 	jc "github.com/juju/testing/checkers"
 	gc "gopkg.in/check.v1"
 
-	"github.com/juju/juju/agent"
 	"github.com/juju/juju/api"
-	"github.com/juju/juju/environs"
-	"github.com/juju/juju/environs/cloudinit"
+	"github.com/juju/juju/cloudconfig/instancecfg"
+	"github.com/juju/juju/cloudconfig/providerinit"
 	"github.com/juju/juju/juju/paths"
 	"github.com/juju/juju/mongo"
 	"github.com/juju/juju/state/multiwatcher"
@@ -37,17 +36,18 @@ func must(s string, err error) string {
 }
 
 var logDir = must(paths.LogDir("precise"))
+var dataDir = must(paths.DataDir("precise"))
 var cloudInitOutputLog = path.Join(logDir, "cloud-init-output.log")
 
-// makeMachineConfig produces a valid cloudinit machine config.
-func makeMachineConfig(c *gc.C) *cloudinit.MachineConfig {
+// makeInstanceConfig produces a valid cloudinit machine config.
+func makeInstanceConfig(c *gc.C) *instancecfg.InstanceConfig {
 	machineId := "0"
 	machineTag := names.NewMachineTag(machineId)
-	return &cloudinit.MachineConfig{
+	return &instancecfg.InstanceConfig{
 		MachineId:    machineId,
 		MachineNonce: "gxshasqlnng",
-		DataDir:      environs.DataDir,
-		LogDir:       agent.DefaultLogDir,
+		DataDir:      dataDir,
+		LogDir:       logDir,
 		Jobs: []multiwatcher.MachineJob{
 			multiwatcher.JobManageEnviron,
 			multiwatcher.JobHostUnits,
@@ -68,36 +68,37 @@ func makeMachineConfig(c *gc.C) *cloudinit.MachineConfig {
 			Password: "password",
 		},
 		APIInfo: &api.Info{
-			CACert: testing.CACert,
-			Addrs:  []string{"127.0.0.1:123"},
-			Tag:    machineTag,
+			CACert:     testing.CACert,
+			Addrs:      []string{"127.0.0.1:123"},
+			Tag:        machineTag,
+			EnvironTag: testing.EnvironmentTag,
 		},
 		MachineAgentServiceName: "jujud-machine-0",
 	}
 }
 
-// makeBadMachineConfig produces a cloudinit machine config that cloudinit
+// makeBadInstanceConfig produces a cloudinit machine config that cloudinit
 // will reject as invalid.
-func makeBadMachineConfig() *cloudinit.MachineConfig {
+func makeBadInstanceConfig() *instancecfg.InstanceConfig {
 	// As it happens, a default-initialized config is invalid.
-	return &cloudinit.MachineConfig{Series: "quantal"}
+	return &instancecfg.InstanceConfig{Series: "quantal"}
 }
 
 func (*customDataSuite) TestMakeCustomDataPropagatesError(c *gc.C) {
-	_, err := makeCustomData(makeBadMachineConfig())
+	_, err := makeCustomData(makeBadInstanceConfig())
 	c.Assert(err, gc.NotNil)
 	c.Check(err, gc.ErrorMatches, "failure while generating custom data: invalid machine configuration: invalid machine id")
 }
 
 func (*customDataSuite) TestMakeCustomDataEncodesUserData(c *gc.C) {
-	cfg := makeMachineConfig(c)
+	cfg := makeInstanceConfig(c)
 
 	encodedData, err := makeCustomData(cfg)
 	c.Assert(err, jc.ErrorIsNil)
 
 	data, err := base64.StdEncoding.DecodeString(encodedData)
 	c.Assert(err, jc.ErrorIsNil)
-	reference, err := environs.ComposeUserData(cfg, nil)
+	reference, err := providerinit.ComposeUserData(cfg, nil)
 	c.Assert(err, jc.ErrorIsNil)
 	c.Check(data, gc.DeepEquals, reference)
 }
