@@ -17,6 +17,7 @@ import (
 	"github.com/juju/utils/set"
 	"launchpad.net/gwacl"
 
+	"github.com/juju/juju/cloudconfig/instancecfg"
 	"github.com/juju/juju/constraints"
 	"github.com/juju/juju/environs"
 	"github.com/juju/juju/environs/config"
@@ -534,16 +535,6 @@ func (env *azureEnviron) SupportedArchitectures() ([]string, error) {
 	return env.supportedArchitectures, err
 }
 
-// SupportNetworks is specified on the EnvironCapability interface.
-func (env *azureEnviron) SupportNetworks() bool {
-	return false
-}
-
-// SupportAddressAllocation is specified on the EnvironCapability interface.
-func (e *azureEnviron) SupportAddressAllocation(netId network.Id) (bool, error) {
-	return false, nil
-}
-
 // selectInstanceTypeAndImage returns the appropriate instances.InstanceType and
 // the OS image name for launching a virtual machine with the given parameters.
 func (env *azureEnviron) selectInstanceTypeAndImage(constraint *instances.InstanceConstraint) (*instances.InstanceType, string, error) {
@@ -711,24 +702,29 @@ func deploymentNameV2(serviceName string) string {
 	return serviceName + "-v2"
 }
 
+// MaintainInstance is specified in the InstanceBroker interface.
+func (*azureEnviron) MaintainInstance(args environs.StartInstanceParams) error {
+	return nil
+}
+
 // StartInstance is specified in the InstanceBroker interface.
 func (env *azureEnviron) StartInstance(args environs.StartInstanceParams) (*environs.StartInstanceResult, error) {
-	if args.MachineConfig.HasNetworks() {
+	if args.InstanceConfig.HasNetworks() {
 		return nil, errors.New("starting instances with networks is not supported yet")
 	}
 
-	err := environs.FinishMachineConfig(args.MachineConfig, env.Config())
+	err := instancecfg.FinishInstanceConfig(args.InstanceConfig, env.Config())
 	if err != nil {
 		return nil, err
 	}
 
 	// Pick envtools.  Needed for the custom data (which is what we normally
 	// call userdata).
-	args.MachineConfig.Tools = args.Tools[0]
-	logger.Infof("picked tools %q", args.MachineConfig.Tools)
+	args.InstanceConfig.Tools = args.Tools[0]
+	logger.Infof("picked tools %q", args.InstanceConfig.Tools)
 
 	// Compose userdata.
-	userData, err := makeCustomData(args.MachineConfig)
+	userData, err := makeCustomData(args.InstanceConfig)
 	if err != nil {
 		return nil, errors.Annotate(err, "cannot compose user data")
 	}
@@ -765,7 +761,7 @@ func (env *azureEnviron) StartInstance(args environs.StartInstanceParams) (*envi
 	vhd := env.newOSDisk(sourceImageName)
 	// If we're creating machine-0, we'll want to expose port 22.
 	// All other machines get an auto-generated public port for SSH.
-	stateServer := multiwatcher.AnyJobNeedsState(args.MachineConfig.Jobs...)
+	stateServer := multiwatcher.AnyJobNeedsState(args.InstanceConfig.Jobs...)
 	role := env.newRole(instanceType.Id, vhd, userData, stateServer)
 	inst, err := createInstance(env, snapshot.api, role, cloudServiceName, stateServer)
 	if err != nil {
@@ -1107,27 +1103,6 @@ func (env *azureEnviron) Instances(ids []instance.Id) ([]instance.Instance, erro
 	return instances, err
 }
 
-// AllocateAddress requests an address to be allocated for the
-// given instance on the given network. This is not implemented on the
-// Azure provider yet.
-func (*azureEnviron) AllocateAddress(_ instance.Id, _ network.Id, _ network.Address) error {
-	return errors.NotImplementedf("AllocateAddress")
-}
-
-// ReleaseAddress releases a specific address previously allocated with
-// AllocateAddress.
-func (*azureEnviron) ReleaseAddress(_ instance.Id, _ network.Id, _ network.Address) error {
-	return errors.NotImplementedf("ReleaseAddress")
-}
-
-// Subnets returns basic information about all subnets known
-// by the provider for the environment. They may be unknown to juju
-// yet (i.e. when called initially or when a new network was created).
-// This is not implemented by the Azure provider yet.
-func (*azureEnviron) Subnets(_ instance.Id) ([]network.SubnetInfo, error) {
-	return nil, errors.NotImplementedf("Subnets")
-}
-
 // AllInstances is specified in the InstanceBroker interface.
 func (env *azureEnviron) AllInstances() ([]instance.Instance, error) {
 	// The instance list is built using the list of all the Azure
