@@ -22,6 +22,7 @@ import (
 	"launchpad.net/gwacl"
 
 	"github.com/juju/juju/api"
+	"github.com/juju/juju/cloudconfig/instancecfg"
 	"github.com/juju/juju/constraints"
 	"github.com/juju/juju/environs"
 	"github.com/juju/juju/environs/bootstrap"
@@ -188,17 +189,6 @@ func (s *environSuite) TestSupportedArchitectures(c *gc.C) {
 	c.Assert(a, gc.DeepEquals, []string{"amd64"})
 }
 
-func (s *environSuite) TestSupportNetworks(c *gc.C) {
-	env := s.setupEnvWithDummyMetadata(c)
-	c.Assert(env.SupportNetworks(), jc.IsFalse)
-}
-func (s *environSuite) TestSupportAddressAllocation(c *gc.C) {
-	env := s.setupEnvWithDummyMetadata(c)
-	result, err := env.SupportAddressAllocation("")
-	c.Assert(result, jc.IsFalse)
-	c.Assert(err, jc.ErrorIsNil)
-}
-
 func (suite *environSuite) TestGetEnvPrefixContainsEnvName(c *gc.C) {
 	env := makeEnviron(c)
 	c.Check(strings.Contains(env.getEnvPrefix(), env.Config().Name()), jc.IsTrue)
@@ -1389,6 +1379,8 @@ func (s *baseEnvironSuite) setupEnvWithDummyMetadata(c *gc.C) *azureEnviron {
 	envAttrs := makeAzureConfigMap(c)
 	envAttrs["location"] = "North Europe"
 	env := makeEnvironWithConfig(c, envAttrs)
+	_, supported := environs.SupportsNetworking(env)
+	c.Assert(supported, jc.IsFalse)
 	s.setDummyStorage(c, env)
 	images := []*imagemetadata.ImageMetadata{
 		{
@@ -1492,18 +1484,19 @@ func (s *startInstanceSuite) SetUpTest(c *gc.C) {
 		Tag:      machineTag,
 	}
 	apiInfo := &api.Info{
-		Addrs:    []string{"localhost:124"},
-		CACert:   coretesting.CACert,
-		Password: "admin",
-		Tag:      machineTag,
+		Addrs:      []string{"localhost:124"},
+		CACert:     coretesting.CACert,
+		Password:   "admin",
+		Tag:        machineTag,
+		EnvironTag: coretesting.EnvironmentTag,
 	}
-	mcfg, err := environs.NewMachineConfig("1", "yanonce", imagemetadata.ReleasedStream, "quantal", true, nil, stateInfo, apiInfo)
+	icfg, err := instancecfg.NewInstanceConfig("1", "yanonce", imagemetadata.ReleasedStream, "quantal", true, nil, stateInfo, apiInfo)
 	c.Assert(err, jc.ErrorIsNil)
 	s.params = environs.StartInstanceParams{
 		Tools: envtesting.AssertUploadFakeToolsVersions(
 			c, s.env.storage, s.env.Config().AgentStream(), s.env.Config().AgentStream(), envtesting.V120p...,
 		),
-		MachineConfig: mcfg,
+		InstanceConfig: icfg,
 	}
 }
 
@@ -1579,13 +1572,13 @@ func (s *startInstanceSuite) TestStartInstanceDistributionGroup(c *gc.C) {
 func (s *startInstanceSuite) TestStartInstanceStateServerJobs(c *gc.C) {
 	// If the machine has the JobManagesEnviron job,
 	// we should see stateServer==true.
-	s.params.MachineConfig.Jobs = []multiwatcher.MachineJob{
+	s.params.InstanceConfig.Jobs = []multiwatcher.MachineJob{
 		multiwatcher.JobHostUnits,
 		multiwatcher.JobManageNetworking,
 	}
 	_, stateServer := s.startInstance(c)
 	c.Assert(stateServer, jc.IsFalse)
-	s.params.MachineConfig.Jobs = []multiwatcher.MachineJob{
+	s.params.InstanceConfig.Jobs = []multiwatcher.MachineJob{
 		multiwatcher.JobHostUnits,
 		multiwatcher.JobManageEnviron,
 		multiwatcher.JobManageNetworking,
