@@ -7,7 +7,7 @@ import (
 	"fmt"
 
 	"github.com/juju/schema"
-	"launchpad.net/goamz/aws"
+	"gopkg.in/amz.v3/aws"
 
 	"github.com/juju/juju/environs/config"
 )
@@ -67,9 +67,10 @@ var configFields = schema.Fields{
 }
 
 var configDefaults = schema.Defaults{
-	"access-key": "",
-	"secret-key": "",
-	"region":     "us-east-1",
+	"access-key":     "",
+	"secret-key":     "",
+	"region":         "us-east-1",
+	"control-bucket": "",
 }
 
 type environConfig struct {
@@ -110,6 +111,19 @@ func validateConfig(cfg, old *config.Config) (*environConfig, error) {
 	if err != nil {
 		return nil, err
 	}
+
+	// Add EC2 specific defaults.
+	providerDefaults := make(map[string]interface{})
+
+	// Storage.
+	if _, ok := cfg.StorageDefaultBlockSource(); !ok {
+		providerDefaults[config.StorageDefaultBlockSourceKey] = EBS_ProviderType
+	}
+	if len(providerDefaults) > 0 {
+		if cfg, err = cfg.Apply(providerDefaults); err != nil {
+			return nil, err
+		}
+	}
 	ecfg := &environConfig{cfg, validated}
 
 	if ecfg.accessKey() == "" || ecfg.secretKey() == "" {
