@@ -13,8 +13,8 @@ import (
 
 	jc "github.com/juju/testing/checkers"
 	"github.com/juju/utils"
+	"gopkg.in/amz.v3/aws"
 	gc "gopkg.in/check.v1"
-	"launchpad.net/goamz/aws"
 
 	"github.com/juju/juju/environs"
 	"github.com/juju/juju/environs/config"
@@ -43,17 +43,18 @@ var testAuth = aws.Auth{"gopher", "long teeth"}
 // when mutated by the mutate function, or that the parse matches the
 // given error.
 type configTest struct {
-	config        map[string]interface{}
-	change        map[string]interface{}
-	expect        map[string]interface{}
-	region        string
-	cbucket       string
-	pbucket       string
-	pbucketRegion string
-	accessKey     string
-	secretKey     string
-	firewallMode  string
-	err           string
+	config             map[string]interface{}
+	change             map[string]interface{}
+	expect             map[string]interface{}
+	region             string
+	cbucket            string
+	pbucket            string
+	pbucketRegion      string
+	accessKey          string
+	secretKey          string
+	firewallMode       string
+	blockStorageSource string
+	err                string
 }
 
 type attrs map[string]interface{}
@@ -202,6 +203,14 @@ var configTests = []configTest{
 	}, {
 		config:       attrs{},
 		firewallMode: config.FwInstance,
+	}, {
+		config:             attrs{},
+		blockStorageSource: "ebs",
+	}, {
+		config: attrs{
+			"default-block-storage-source": "ebs-fast",
+		},
+		blockStorageSource: "ebs-fast",
 	}, {
 		config: attrs{
 			"firewall-mode": "instance",
@@ -293,6 +302,28 @@ func (s *ConfigSuite) TestMissingAuth(c *gc.C) {
 	test.check(c)
 }
 
+func (s *ConfigSuite) TestPrepareForCreateInsertsUniqueControlBucket(c *gc.C) {
+	s.PatchValue(&verifyCredentials, func(*environ) error { return nil })
+	attrs := testing.FakeConfig().Merge(testing.Attrs{
+		"type": "ec2",
+	})
+	cfg, err := config.New(config.NoDefaults, attrs)
+	c.Assert(err, jc.ErrorIsNil)
+
+	cfg1, err := providerInstance.PrepareForCreateEnvironment(cfg)
+	c.Assert(err, jc.ErrorIsNil)
+
+	bucket1 := cfg1.UnknownAttrs()["control-bucket"]
+	c.Assert(bucket1, gc.Matches, "[a-f0-9]{32}")
+
+	cfg2, err := providerInstance.PrepareForCreateEnvironment(cfg)
+	c.Assert(err, jc.ErrorIsNil)
+	bucket2 := cfg2.UnknownAttrs()["control-bucket"]
+	c.Assert(bucket2, gc.Matches, "[a-f0-9]{32}")
+
+	c.Assert(bucket1, gc.Not(gc.Equals), bucket2)
+}
+
 func (s *ConfigSuite) TestPrepareInsertsUniqueControlBucket(c *gc.C) {
 	s.PatchValue(&verifyCredentials, func(*environ) error { return nil })
 	attrs := testing.FakeConfig().Merge(testing.Attrs{
@@ -302,12 +333,12 @@ func (s *ConfigSuite) TestPrepareInsertsUniqueControlBucket(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 
 	ctx := envtesting.BootstrapContext(c)
-	env0, err := providerInstance.Prepare(ctx, cfg)
+	env0, err := providerInstance.PrepareForBootstrap(ctx, cfg)
 	c.Assert(err, jc.ErrorIsNil)
 	bucket0 := env0.(*environ).ecfg().controlBucket()
 	c.Assert(bucket0, gc.Matches, "[a-f0-9]{32}")
 
-	env1, err := providerInstance.Prepare(ctx, cfg)
+	env1, err := providerInstance.PrepareForBootstrap(ctx, cfg)
 	c.Assert(err, jc.ErrorIsNil)
 	bucket1 := env1.(*environ).ecfg().controlBucket()
 	c.Assert(bucket1, gc.Matches, "[a-f0-9]{32}")
@@ -324,7 +355,7 @@ func (s *ConfigSuite) TestPrepareDoesNotTouchExistingControlBucket(c *gc.C) {
 	cfg, err := config.New(config.NoDefaults, attrs)
 	c.Assert(err, jc.ErrorIsNil)
 
-	env, err := providerInstance.Prepare(envtesting.BootstrapContext(c), cfg)
+	env, err := providerInstance.PrepareForBootstrap(envtesting.BootstrapContext(c), cfg)
 	c.Assert(err, jc.ErrorIsNil)
 	bucket := env.(*environ).ecfg().controlBucket()
 	c.Assert(bucket, gc.Equals, "burblefoo")
