@@ -11,18 +11,21 @@ import (
 	"time"
 
 	"github.com/juju/errors"
+	"github.com/juju/names"
 	"github.com/juju/utils"
-	"launchpad.net/goamz/aws"
-	"launchpad.net/goamz/ec2"
-	"launchpad.net/goamz/s3"
+	"gopkg.in/amz.v3/aws"
+	"gopkg.in/amz.v3/ec2"
+	"gopkg.in/amz.v3/s3"
 
+	"github.com/juju/juju/cloudconfig/instancecfg"
+	"github.com/juju/juju/cloudconfig/providerinit"
 	"github.com/juju/juju/constraints"
 	"github.com/juju/juju/environs"
 	"github.com/juju/juju/environs/config"
 	"github.com/juju/juju/environs/imagemetadata"
 	"github.com/juju/juju/environs/instances"
 	"github.com/juju/juju/environs/simplestreams"
-	"github.com/juju/juju/environs/storage"
+	envstorage "github.com/juju/juju/environs/storage"
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/juju/arch"
 	"github.com/juju/juju/network"
@@ -36,6 +39,10 @@ const (
 	none                        = "none"
 	invalidParameterValue       = "InvalidParameterValue"
 	privateAddressLimitExceeded = "PrivateIpAddressLimitExceeded"
+
+	// tagName is the AWS-specific tag key that populates resources'
+	// name columns in the console.
+	tagName = "Name"
 )
 
 // Use shortAttempt to poll for short-term events or for retrying API calls.
@@ -62,7 +69,7 @@ type environ struct {
 	ecfgUnlocked    *environConfig
 	ec2Unlocked     *ec2.EC2
 	s3Unlocked      *s3.S3
-	storageUnlocked storage.Storage
+	storageUnlocked envstorage.Storage
 
 	availabilityZonesMutex sync.Mutex
 	availabilityZones      []common.AvailabilityZone
@@ -71,7 +78,8 @@ type environ struct {
 	cachedDefaultVpc *defaultVpc
 }
 
-var _ environs.Environ = (*environ)(nil)
+// Ensure EC2 provider supports environs.NetworkingEnviron.
+var _ environs.NetworkingEnviron = (*environ)(nil)
 var _ simplestreams.HasRegion = (*environ)(nil)
 var _ state.Prechecker = (*environ)(nil)
 var _ state.InstanceDistributor = (*environ)(nil)
@@ -90,25 +98,37 @@ func (e *environ) Config() *config.Config {
 	return e.ecfg().Config
 }
 
-func (e *environ) SetConfig(cfg *config.Config) error {
+func awsClients(cfg *config.Config) (*ec2.EC2, *s3.S3, *environConfig, error) {
 	ecfg, err := providerInstance.newConfig(cfg)
+	if err != nil {
+		return nil, nil, nil, err
+	}
+
+	auth := aws.Auth{ecfg.accessKey(), ecfg.secretKey()}
+	region := aws.Regions[ecfg.region()]
+	signer := aws.SignV4Factory(region.Name, "ec2")
+	return ec2.New(auth, region, signer), s3.New(auth, region), ecfg, nil
+}
+
+func (e *environ) SetConfig(cfg *config.Config) error {
+	ec2Client, s3Client, ecfg, err := awsClients(cfg)
 	if err != nil {
 		return err
 	}
 	e.ecfgMutex.Lock()
 	defer e.ecfgMutex.Unlock()
 	e.ecfgUnlocked = ecfg
+	e.ec2Unlocked = ec2Client
+	e.s3Unlocked = s3Client
 
-	auth := aws.Auth{ecfg.accessKey(), ecfg.secretKey()}
-	region := aws.Regions[ecfg.region()]
-	e.ec2Unlocked = ec2.New(auth, region)
-	e.s3Unlocked = s3.New(auth, region)
+	bucket, err := e.s3Unlocked.Bucket(ecfg.controlBucket())
+	if err != nil {
+		return err
+	}
 
 	// create new storage instances, existing instances continue
 	// to reference their existing configuration.
-	e.storageUnlocked = &ec2storage{
-		bucket: e.s3Unlocked.Bucket(ecfg.controlBucket()),
-	}
+	e.storageUnlocked = &ec2storage{bucket: bucket}
 	return nil
 }
 
@@ -169,7 +189,7 @@ func (e *environ) Name() string {
 	return e.name
 }
 
-func (e *environ) Storage() storage.Storage {
+func (e *environ) Storage() envstorage.Storage {
 	e.ecfgMutex.Lock()
 	stor := e.storageUnlocked
 	e.ecfgMutex.Unlock()
@@ -204,15 +224,11 @@ func (e *environ) SupportedArchitectures() ([]string, error) {
 	return e.supportedArchitectures, err
 }
 
-// SupportNetworks is specified on the EnvironCapability interface.
-func (e *environ) SupportNetworks() bool {
-	// TODO(dimitern) Once we have support for VPCs and advanced
-	// networking, return true here.
-	return false
-}
-
-// SupportAddressAllocation is specified on the EnvironCapability interface.
-func (e *environ) SupportAddressAllocation(netId network.Id) (bool, error) {
+// SupportsAddressAllocation is specified on environs.Networking.
+func (e *environ) SupportsAddressAllocation(_ network.Id) (bool, error) {
+	if !environs.AddressAllocationEnabled() {
+		return false, errors.NotSupportedf("address allocation")
+	}
 	_, hasDefaultVpc, err := e.defaultVpc()
 	if err != nil {
 		return false, errors.Trace(err)
@@ -406,8 +422,29 @@ func (e *environ) DistributeInstances(candidates, distributionGroup []instance.I
 
 var availabilityZoneAllocations = common.AvailabilityZoneAllocations
 
+// MaintainInstance is specified in the InstanceBroker interface.
+func (*environ) MaintainInstance(args environs.StartInstanceParams) error {
+	return nil
+}
+
+// resourceName returns the string to use for a resource's Name tag,
+// to help users identify Juju-managed resources in the AWS console.
+func resourceName(tag names.Tag, envName string) string {
+	return fmt.Sprintf("juju-%s-%s", envName, tag)
+}
+
 // StartInstance is specified in the InstanceBroker interface.
-func (e *environ) StartInstance(args environs.StartInstanceParams) (*environs.StartInstanceResult, error) {
+func (e *environ) StartInstance(args environs.StartInstanceParams) (_ *environs.StartInstanceResult, resultErr error) {
+	var inst *ec2Instance
+	defer func() {
+		if resultErr == nil || inst == nil {
+			return
+		}
+		if err := e.StopInstances(inst.Id()); err != nil {
+			logger.Errorf("error stopping failed instance: %v", err)
+		}
+	}()
+
 	var availabilityZones []string
 	if args.Placement != "" {
 		placement, err := e.parsePlacement(args.Placement)
@@ -444,7 +481,7 @@ func (e *environ) StartInstance(args environs.StartInstanceParams) (*environs.St
 		}
 	}
 
-	if args.MachineConfig.HasNetworks() {
+	if args.InstanceConfig.HasNetworks() {
 		return nil, errors.New("starting instances with networks is not supported yet")
 	}
 	arches := args.Tools.Arches()
@@ -469,26 +506,24 @@ func (e *environ) StartInstance(args environs.StartInstanceParams) (*environs.St
 		return nil, errors.Errorf("chosen architecture %v not present in %v", spec.Image.Arch, arches)
 	}
 
-	args.MachineConfig.Tools = tools[0]
-	if err := environs.FinishMachineConfig(args.MachineConfig, e.Config()); err != nil {
+	args.InstanceConfig.Tools = tools[0]
+	if err := instancecfg.FinishInstanceConfig(args.InstanceConfig, e.Config()); err != nil {
 		return nil, err
 	}
 
-	userData, err := environs.ComposeUserData(args.MachineConfig, nil)
+	userData, err := providerinit.ComposeUserData(args.InstanceConfig, nil)
 	if err != nil {
 		return nil, errors.Annotate(err, "cannot make user data")
 	}
 	logger.Debugf("ec2 user data; %d bytes", len(userData))
 	cfg := e.Config()
-	groups, err := e.setUpGroups(args.MachineConfig.MachineId, cfg.APIPort())
+	groups, err := e.setUpGroups(args.InstanceConfig.MachineId, cfg.APIPort())
 	if err != nil {
 		return nil, errors.Annotate(err, "cannot set up groups")
 	}
 	var instResp *ec2.RunInstancesResp
 
-	blockDeviceMappings, disks, err := getBlockDeviceMappings(
-		*spec.InstanceType.VirtType, &args,
-	)
+	blockDeviceMappings, err := getBlockDeviceMappings(args.Constraints)
 	if err != nil {
 		return nil, errors.Annotate(err, "cannot create block device mappings")
 	}
@@ -518,19 +553,23 @@ func (e *environ) StartInstance(args environs.StartInstanceParams) (*environs.St
 		return nil, errors.Errorf("expected 1 started instance, got %d", len(instResp.Instances))
 	}
 
-	inst := &ec2Instance{
+	inst = &ec2Instance{
 		e:        e,
 		Instance: &instResp.Instances[0],
 	}
 	logger.Infof("started instance %q in %q", inst.Id(), inst.Instance.AvailZone)
 
-	// TODO(axw) extract volume ID, store in BlockDevice.ProviderId field,
-	// and tag all resources (instances and volumes). We can't do this until
-	// goamz's BlockDeviceMapping structure is updated to include VolumeId.
+	// Tag instance, for accounting and identification.
+	args.InstanceConfig.Tags[tagName] = resourceName(
+		names.NewMachineTag(args.InstanceConfig.MachineId), e.Config().Name(),
+	)
+	if err := tagResources(e.ec2(), args.InstanceConfig.Tags, string(inst.Id())); err != nil {
+		return nil, errors.Annotate(err, "tagging instance")
+	}
 
-	if multiwatcher.AnyJobNeedsState(args.MachineConfig.Jobs...) {
+	if multiwatcher.AnyJobNeedsState(args.InstanceConfig.Jobs...) {
 		if err := common.AddStateInstance(e.Storage(), inst.Id()); err != nil {
-			logger.Errorf("could not record instance in provider-state: %v", err)
+			return nil, errors.Annotate(err, "recording instance in provider-state")
 		}
 	}
 
@@ -546,10 +585,30 @@ func (e *environ) StartInstance(args environs.StartInstanceParams) (*environs.St
 	return &environs.StartInstanceResult{
 		Instance: inst,
 		Hardware: &hc,
-		Disks:    disks,
 	}, nil
 }
 
+// tagResources calls ec2.CreateTags, tagging each of the specified resources
+// with the given tags. tagResources will retry for a short period of time
+// if it receives a *.NotFound error response from EC2.
+func tagResources(e *ec2.EC2, tags map[string]string, resourceIds ...string) error {
+	if len(tags) == 0 {
+		return nil
+	}
+	ec2Tags := make([]ec2.Tag, 0, len(tags))
+	for k, v := range tags {
+		ec2Tags = append(ec2Tags, ec2.Tag{k, v})
+	}
+	var err error
+	for a := shortAttempt.Start(); a.Next(); {
+		_, err = e.CreateTags(resourceIds, ec2Tags)
+		if err == nil || !strings.HasSuffix(ec2ErrCode(err), ".NotFound") {
+			return err
+		}
+	}
+	return err
+}
+
 var runInstances = _runInstances
 
 // runInstances calls ec2.RunInstances for a fixed number of attempts until
@@ -705,6 +764,7 @@ func (e *environ) fetchNetworkInterfaceId(ec2Inst *ec2.EC2, instId instance.Id)
 		if err == nil {
 			break
 		}
+		logger.Tracef("Instances(%q) returned: %v", instId, err)
 	}
 	if err != nil {
 		// either the instance doesn't exist or we couldn't get through to
@@ -713,10 +773,10 @@ func (e *environ) fetchNetworkInterfaceId(ec2Inst *ec2.EC2, instId instance.Id)
 	}
 
 	if len(instancesResp.Reservations) == 0 {
-		return "", errors.New("unexpected AWS response: instance not found")
+		return "", errors.New("unexpected AWS response: reservation not found")
 	}
 	if len(instancesResp.Reservations[0].Instances) == 0 {
-		return "", errors.New("unexpected AWS response: reservation not found")
+		return "", errors.New("unexpected AWS response: instance not found")
 	}
 	if len(instancesResp.Reservations[0].Instances[0].NetworkInterfaces) == 0 {
 		return "", errors.New("unexpected AWS response: network interface not found")
@@ -725,71 +785,166 @@ func (e *environ) fetchNetworkInterfaceId(ec2Inst *ec2.EC2, instId instance.Id)
 	return networkInterfaceId, nil
 }
 
-// AllocateAddress requests an address to be allocated for the
-// given instance on the given network. This is not implemented by the
-// EC2 provider yet.
-func (e *environ) AllocateAddress(instId instance.Id, _ network.Id, addr network.Address) error {
+// AllocateAddress requests an address to be allocated for the given
+// instance on the given subnet. Implements NetworkingEnviron.AllocateAddress.
+func (e *environ) AllocateAddress(instId instance.Id, _ network.Id, addr network.Address) (err error) {
+	if !environs.AddressAllocationEnabled() {
+		return errors.NotSupportedf("address allocation")
+	}
+
+	defer errors.DeferredAnnotatef(&err, "failed to allocate address %q for instance %q", addr, instId)
+
+	var nicId string
 	ec2Inst := e.ec2()
-	networkInterfaceId, err := e.fetchNetworkInterfaceId(ec2Inst, instId)
+	nicId, err = e.fetchNetworkInterfaceId(ec2Inst, instId)
 	if err != nil {
-		return errors.Annotatef(err, "failed to assign IP address %q to instance %q", addr, instId)
+		return errors.Trace(err)
 	}
 	for a := shortAttempt.Start(); a.Next(); {
-		err = AssignPrivateIPAddress(ec2Inst, networkInterfaceId, addr)
+		err = AssignPrivateIPAddress(ec2Inst, nicId, addr)
+		logger.Tracef("AssignPrivateIPAddresses(%v, %v) returned: %v", nicId, addr, err)
 		if err == nil {
+			logger.Tracef("allocated address %v for instance %v, NIC %v", addr, instId, nicId)
 			break
 		}
 		if ec2Err, ok := err.(*ec2.Error); ok {
 			if ec2Err.Code == invalidParameterValue {
 				// Note: this Code is also used if we specify
 				// an IP address outside the subnet. Take care!
+				logger.Tracef("address %q not available for allocation", addr)
 				return environs.ErrIPAddressUnavailable
 			} else if ec2Err.Code == privateAddressLimitExceeded {
+				logger.Tracef("no more addresses available on the subnet")
 				return environs.ErrIPAddressesExhausted
 			}
 		}
 
 	}
-	if err != nil {
-		return errors.Annotatef(err, "failed to assign IP address %q to instance %q", addr, instId)
-	}
-	return nil
+	return err
 }
 
 // ReleaseAddress releases a specific address previously allocated with
-// AllocateAddress.
-func (e *environ) ReleaseAddress(instId instance.Id, _ network.Id, addr network.Address) error {
+// AllocateAddress. Implements NetworkingEnviron.ReleaseAddress.
+func (e *environ) ReleaseAddress(instId instance.Id, _ network.Id, addr network.Address) (err error) {
+	if !environs.AddressAllocationEnabled() {
+		return errors.NotSupportedf("address allocation")
+	}
+
+	defer errors.DeferredAnnotatef(&err, "failed to release address %q from instance %q", addr, instId)
+
+	// If the instance ID is unknown the address has already been released
+	// and we can ignore this request.
+	if instId == instance.UnknownId {
+		logger.Debugf("release address %q with an unknown instance ID is a no-op (ignoring)", addr.Value)
+		return nil
+	}
+
+	var nicId string
 	ec2Inst := e.ec2()
-	networkInterfaceId, err := e.fetchNetworkInterfaceId(ec2Inst, instId)
+	nicId, err = e.fetchNetworkInterfaceId(ec2Inst, instId)
 	if err != nil {
-		return errors.Annotatef(err, "failed to unassign IP address %q to instance %q", addr, instId)
+		return errors.Trace(err)
 	}
 	for a := shortAttempt.Start(); a.Next(); {
-		_, err = ec2Inst.UnassignPrivateIPAddresses(networkInterfaceId, []string{addr.Value})
+		_, err = ec2Inst.UnassignPrivateIPAddresses(nicId, []string{addr.Value})
+		logger.Tracef("UnassignPrivateIPAddresses(%q, %q) returned: %v", nicId, addr, err)
 		if err == nil {
+			logger.Tracef("released address %q from instance %q, NIC %q", addr, instId, nicId)
 			break
 		}
-
 	}
-	if err != nil {
-		return errors.Annotatef(err, "failed to unassign IP address %q for instance %q", addr, instId)
-	}
-	return nil
+	return err
 }
 
-// Subnets returns basic information about all subnets known
-// by the provider for the environment. They may be unknown to juju
-// yet (i.e. when called initially or when a new subnet was created).
-func (e *environ) Subnets(_ instance.Id) ([]network.SubnetInfo, error) {
+// NetworkInterfaces implements NetworkingEnviron.NetworkInterfaces.
+func (e *environ) NetworkInterfaces(instId instance.Id) ([]network.InterfaceInfo, error) {
+	ec2Client := e.ec2()
+	var err error
+	var networkInterfacesResp *ec2.NetworkInterfacesResp
+	for a := shortAttempt.Start(); a.Next(); {
+		logger.Tracef("retrieving NICs for instance %q", instId)
+		filter := ec2.NewFilter()
+		filter.Add("attachment.instance-id", string(instId))
+		networkInterfacesResp, err = ec2Client.NetworkInterfaces(nil, filter)
+		logger.Tracef("instance %q NICs: %#v (err: %v)", instId, networkInterfacesResp, err)
+		if err != nil {
+			logger.Warningf("failed to get instance %q interfaces: %v (retrying)", instId, err)
+			continue
+		}
+		if len(networkInterfacesResp.Interfaces) == 0 {
+			logger.Tracef("instance %q has no NIC attachment yet, retrying...", instId)
+			continue
+		}
+		logger.Tracef("found instance %q NICS: %#v", instId, networkInterfacesResp.Interfaces)
+		break
+	}
+	if err != nil {
+		// either the instance doesn't exist or we couldn't get through to
+		// the ec2 api
+		return nil, errors.Annotatef(err, "cannot get instance %q network interfaces", instId)
+	}
+	ec2Interfaces := networkInterfacesResp.Interfaces
+	result := make([]network.InterfaceInfo, len(ec2Interfaces))
+	for i, iface := range ec2Interfaces {
+		resp, err := ec2Client.Subnets([]string{iface.SubnetId}, nil)
+		if err != nil {
+			return nil, errors.Annotatef(err, "failed to retrieve subnet %q info", iface.SubnetId)
+		}
+		if len(resp.Subnets) != 1 {
+			return nil, errors.Errorf("expected 1 subnet, got %d", len(resp.Subnets))
+		}
+		subnet := resp.Subnets[0]
+		cidr := subnet.CIDRBlock
+
+		result[i] = network.InterfaceInfo{
+			DeviceIndex:      iface.Attachment.DeviceIndex,
+			MACAddress:       iface.MACAddress,
+			CIDR:             cidr,
+			NetworkName:      "", // Not needed for now.
+			ProviderId:       network.Id(iface.Id),
+			ProviderSubnetId: network.Id(iface.SubnetId),
+			VLANTag:          0, // Not supported on EC2.
+			// Getting the interface name is not supported on EC2, so fake it.
+			InterfaceName: fmt.Sprintf("unsupported%d", iface.Attachment.DeviceIndex),
+			Disabled:      false,
+			NoAutoStart:   false,
+			ConfigType:    network.ConfigDHCP,
+			Address:       network.NewScopedAddress(iface.PrivateIPAddress, network.ScopeCloudLocal),
+		}
+	}
+	return result, nil
+}
+
+// Subnets returns basic information about the specified subnets known
+// by the provider for the specified instance. subnetIds must not be
+// empty. Implements NetworkingEnviron.Subnets.
+func (e *environ) Subnets(_ instance.Id, subnetIds []network.Id) ([]network.SubnetInfo, error) {
+	// At some point in the future an empty netIds may mean "fetch all subnets"
+	// but until that functionality is needed it's an error.
+	if len(subnetIds) == 0 {
+		return nil, errors.Errorf("subnetIds must not be empty")
+	}
 	ec2Inst := e.ec2()
-	// TODO: (mfoord 2014-12-15) can we filter by instance ID here?
+	// We can't filter by instance id here, unfortunately.
 	resp, err := ec2Inst.Subnets(nil, nil)
 	if err != nil {
-		return nil, errors.Annotatef(err, "failed to retrieve subnet info")
+		return nil, errors.Annotatef(err, "failed to retrieve subnets")
+	}
+
+	subIdSet := make(map[string]bool)
+	for _, subId := range subnetIds {
+		subIdSet[string(subId)] = false
 	}
 
 	var results []network.SubnetInfo
 	for _, subnet := range resp.Subnets {
+		_, ok := subIdSet[subnet.Id]
+		if !ok {
+			logger.Tracef("subnet %q not in %v, skipping", subnet.Id, subnetIds)
+			continue
+		}
+		subIdSet[subnet.Id] = true
+
 		cidr := subnet.CIDRBlock
 		ip, ipnet, err := net.ParseCIDR(cidr)
 		if err != nil {
@@ -802,7 +957,7 @@ func (e *environ) Subnets(_ instance.Id) ([]network.SubnetInfo, error) {
 			logger.Warningf("skipping subnet %q, invalid IP: %v", cidr, err)
 			continue
 		}
-		// the first four addresses in a subnet are reserved, see
+		// First four addresses in a subnet are reserved, see
 		// http://goo.gl/rrWTIo
 		allocatableLow := network.DecimalToIPv4(start + 4)
 
@@ -810,19 +965,30 @@ func (e *environ) Subnets(_ instance.Id) ([]network.SubnetInfo, error) {
 		zeros := bits - ones
 		numIPs := uint32(1) << uint32(zeros)
 		highIP := start + numIPs - 1
-		// the last address in a subnet is reserved
+		// The last address in a subnet is also reserved (see same ref).
 		allocatableHigh := network.DecimalToIPv4(highIP - 1)
 
-		// No VLANTag available
 		info := network.SubnetInfo{
 			CIDR:              cidr,
 			ProviderId:        network.Id(subnet.Id),
+			VLANTag:           0, // Not supported on EC2
 			AllocatableIPLow:  allocatableLow,
 			AllocatableIPHigh: allocatableHigh,
 		}
+		logger.Tracef("found subnet with info %#v", info)
 		results = append(results, info)
 	}
 
+	notFound := []string{}
+	for subId, found := range subIdSet {
+		if !found {
+			notFound = append(notFound, subId)
+		}
+	}
+	if len(notFound) != 0 {
+		return nil, errors.Errorf("failed to find the following subnet ids: %v", notFound)
+	}
+
 	return results, nil
 }
 
@@ -1089,7 +1255,7 @@ var zeroGroup ec2.SecurityGroup
 // the named group only.
 func (e *environ) ensureGroup(name string, perms []ec2.IPPerm) (g ec2.SecurityGroup, err error) {
 	ec2inst := e.ec2()
-	resp, err := ec2inst.CreateSecurityGroup(name, "juju group")
+	resp, err := ec2inst.CreateSecurityGroup("", name, "juju group")
 	if err != nil && ec2ErrCode(err) != "InvalidGroup.Duplicate" {
 		return zeroGroup, err
 	}
@@ -1217,6 +1383,8 @@ func isZoneConstrainedError(err error) bool {
 			// if the AZ does not have a default subnet. Until we have proper
 			// support for networks, we'll skip over these.
 			return strings.HasPrefix(err.Message, "No default subnet for availability zone")
+		case "VolumeTypeNotAvailableInZone":
+			return true
 		}
 	}
 	return false
