@@ -6,17 +6,26 @@ package ec2
 import (
 	"io"
 
-	"launchpad.net/goamz/aws"
-	"launchpad.net/goamz/ec2"
-	"launchpad.net/goamz/s3"
+	"gopkg.in/amz.v3/aws"
+	"gopkg.in/amz.v3/ec2"
+	"gopkg.in/amz.v3/s3"
 
 	"github.com/juju/juju/environs"
 	"github.com/juju/juju/environs/imagemetadata"
 	"github.com/juju/juju/environs/jujutest"
 	"github.com/juju/juju/environs/storage"
 	"github.com/juju/juju/instance"
+	jujustorage "github.com/juju/juju/storage"
 )
 
+func EBSProvider() jujustorage.Provider {
+	return &ebsProvider{}
+}
+
+func StorageEC2(vs jujustorage.VolumeSource) *ec2.EC2 {
+	return vs.(*ebsVolumeSource).ec2
+}
+
 func ControlBucketName(e environs.Environ) string {
 	return e.(*environ).ecfg().controlBucket()
 }
@@ -46,6 +55,7 @@ var (
 	AvailabilityZoneAllocations = &availabilityZoneAllocations
 	RunInstances                = &runInstances
 	BlockDeviceNamer            = blockDeviceNamer
+	GetBlockDeviceMappings      = getBlockDeviceMappings
 )
 
 // BucketStorage returns a storage instance addressing
