@@ -10,8 +10,8 @@ import (
 	"strings"
 
 	jc "github.com/juju/testing/checkers"
+	amzec2 "gopkg.in/amz.v3/ec2"
 	gc "gopkg.in/check.v1"
-	amzec2 "launchpad.net/goamz/ec2"
 
 	"github.com/juju/juju/constraints"
 	"github.com/juju/juju/environs"
@@ -379,7 +379,8 @@ func (t *LiveTests) TestStopInstances(c *gc.C) {
 func (t *LiveTests) TestPutBucketOnlyOnce(c *gc.C) {
 	t.PrepareOnce(c)
 	s3inst := ec2.EnvironS3(t.Env)
-	b := s3inst.Bucket("test-once-" + uniqueName)
+	b, err := s3inst.Bucket("test-once-" + uniqueName)
+	c.Assert(err, jc.ErrorIsNil)
 	s := ec2.BucketStorage(b)
 
 	// Check that we don't do a PutBucket every time by
@@ -388,7 +389,7 @@ func (t *LiveTests) TestPutBucketOnlyOnce(c *gc.C) {
 	// which should fail because it doesn't try to do
 	// the PutBucket again.
 
-	err := s.Put("test-object", strings.NewReader("test"), 4)
+	err = s.Put("test-object", strings.NewReader("test"), 4)
 	c.Assert(err, jc.ErrorIsNil)
 
 	err = s.Remove("test-object")
@@ -404,7 +405,7 @@ func (t *LiveTests) TestPutBucketOnlyOnce(c *gc.C) {
 // createGroup creates a new EC2 group and returns it. If it already exists,
 // it revokes all its permissions and returns the existing group.
 func createGroup(c *gc.C, ec2conn *amzec2.EC2, name, descr string) amzec2.SecurityGroup {
-	resp, err := ec2conn.CreateSecurityGroup(name, descr)
+	resp, err := ec2conn.CreateSecurityGroup("", name, descr)
 	if err == nil {
 		return resp.SecurityGroup
 	}
