@@ -14,12 +14,12 @@ import (
 	jc "github.com/juju/testing/checkers"
 	"github.com/juju/utils"
 	"github.com/juju/utils/set"
+	"gopkg.in/amz.v3/aws"
+	amzec2 "gopkg.in/amz.v3/ec2"
+	"gopkg.in/amz.v3/ec2/ec2test"
+	"gopkg.in/amz.v3/s3/s3test"
 	gc "gopkg.in/check.v1"
 	goyaml "gopkg.in/yaml.v1"
-	"launchpad.net/goamz/aws"
-	amzec2 "launchpad.net/goamz/ec2"
-	"launchpad.net/goamz/ec2/ec2test"
-	"launchpad.net/goamz/s3/s3test"
 
 	"github.com/juju/juju/constraints"
 	"github.com/juju/juju/environs"
@@ -31,13 +31,13 @@ import (
 	"github.com/juju/juju/environs/simplestreams"
 	envtesting "github.com/juju/juju/environs/testing"
 	"github.com/juju/juju/environs/tools"
+	"github.com/juju/juju/feature"
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/juju/arch"
 	"github.com/juju/juju/juju/testing"
 	"github.com/juju/juju/network"
 	"github.com/juju/juju/provider/common"
 	"github.com/juju/juju/provider/ec2"
-	"github.com/juju/juju/storage"
 	coretesting "github.com/juju/juju/testing"
 	"github.com/juju/juju/utils/ssh"
 	"github.com/juju/juju/version"
@@ -64,7 +64,6 @@ func registerLocalTests() {
 	// has entries in the images/query txt files.
 	aws.Regions["test"] = aws.Region{
 		Name: "test",
-		Sign: aws.SignV2,
 	}
 
 	gc.Suite(&localServerSuite{})
@@ -119,7 +118,6 @@ func (srv *localServer) startServer(c *gc.C) {
 		EC2Endpoint:          srv.ec2srv.URL(),
 		S3Endpoint:           srv.s3srv.URL(),
 		S3LocationConstraint: true,
-		Sign:                 aws.SignV2,
 	}
 	srv.addSpice(c)
 
@@ -186,6 +184,7 @@ func (t *localServerSuite) TearDownSuite(c *gc.C) {
 
 func (t *localServerSuite) SetUpTest(c *gc.C) {
 	t.BaseSuite.SetUpTest(c)
+	t.SetFeatureFlags(feature.AddressAllocation)
 	t.srv.startServer(c)
 	t.Tests.SetUpTest(c)
 	t.PatchValue(&version.Current, version.Binary{
@@ -201,6 +200,13 @@ func (t *localServerSuite) TearDownTest(c *gc.C) {
 	t.BaseSuite.TearDownTest(c)
 }
 
+func (t *localServerSuite) prepareEnviron(c *gc.C) environs.NetworkingEnviron {
+	env := t.Prepare(c)
+	netenv, supported := environs.SupportsNetworking(env)
+	c.Assert(supported, jc.IsTrue)
+	return netenv
+}
+
 func (t *localServerSuite) TestBootstrapInstanceUserDataAndState(c *gc.C) {
 	env := t.Prepare(c)
 	err := bootstrap.Bootstrap(envtesting.BootstrapContext(c), env, bootstrap.BootstrapParams{})
@@ -239,6 +245,8 @@ func (t *localServerSuite) TestBootstrapInstanceUserDataAndState(c *gc.C) {
 		"ssh_authorized_keys": splitAuthKeys(env.Config().AuthorizedKeys()),
 		"runcmd": []interface{}{
 			"set -xe",
+			"install -D -m 644 /dev/null '/etc/init/juju-clean-shutdown.conf'",
+			"printf '%s\\n' '\nauthor \"Juju Team <juju@lists.ubuntu.com>\"\ndescription \"Stop all network interfaces on shutdown\"\nstart on runlevel [016]\ntask\nconsole output\n\nexec /sbin/ifdown -a -v --force\n' > '/etc/init/juju-clean-shutdown.conf'",
 			"install -D -m 644 /dev/null '/var/lib/juju/nonce.txt'",
 			"printf '%s\\n' 'user-admin:bootstrap' > '/var/lib/juju/nonce.txt'",
 		},
@@ -471,6 +479,11 @@ var azConstrainedErr = &amzec2.Error{
 	Message: "The requested Availability Zone is currently constrained etc.",
 }
 
+var azVolumeTypeNotAvailableInZoneErr = &amzec2.Error{
+	Code:    "VolumeTypeNotAvailableInZone",
+	Message: "blah blah",
+}
+
 var azInsufficientInstanceCapacityErr = &amzec2.Error{
 	Code: "InsufficientInstanceCapacity",
 	Message: "We currently do not have sufficient m1.small capacity in the " +
@@ -489,6 +502,10 @@ func (t *localServerSuite) TestStartInstanceAvailZoneAllConstrained(c *gc.C) {
 	t.testStartInstanceAvailZoneAllConstrained(c, azConstrainedErr)
 }
 
+func (t *localServerSuite) TestStartInstanceVolumeTypeNotAvailable(c *gc.C) {
+	t.testStartInstanceAvailZoneAllConstrained(c, azVolumeTypeNotAvailableInZoneErr)
+}
+
 func (t *localServerSuite) TestStartInstanceAvailZoneAllInsufficientInstanceCapacity(c *gc.C) {
 	t.testStartInstanceAvailZoneAllConstrained(c, azInsufficientInstanceCapacityErr)
 }
@@ -699,13 +716,14 @@ func (t *localServerSuite) TestSupportedArchitectures(c *gc.C) {
 	c.Assert(a, jc.SameContents, []string{"amd64", "i386"})
 }
 
-func (t *localServerSuite) TestSupportNetworks(c *gc.C) {
+func (t *localServerSuite) TestSupportsNetworking(c *gc.C) {
 	env := t.Prepare(c)
-	c.Assert(env.SupportNetworks(), jc.IsFalse)
+	_, supported := environs.SupportsNetworking(env)
+	c.Assert(supported, jc.IsTrue)
 }
 
 func (t *localServerSuite) TestAllocateAddressFailureToFindNetworkInterface(c *gc.C) {
-	env := t.Prepare(c)
+	env := t.prepareEnviron(c)
 	err := bootstrap.Bootstrap(envtesting.BootstrapContext(c), env, bootstrap.BootstrapParams{})
 	c.Assert(err, jc.ErrorIsNil)
 
@@ -724,12 +742,12 @@ func (t *localServerSuite) TestAllocateAddressFailureToFindNetworkInterface(c *g
 	c.Assert(errors.Cause(err), gc.ErrorMatches, "unexpected AWS response: network interface not found")
 }
 
-func (t *localServerSuite) setUpInstanceWithDefaultVpc(c *gc.C) (environs.Environ, instance.Id) {
+func (t *localServerSuite) setUpInstanceWithDefaultVpc(c *gc.C) (environs.NetworkingEnviron, instance.Id) {
 	// setting a default-vpc will create a network interface
 	t.srv.ec2srv.SetInitialAttributes(map[string][]string{
-		"default-vpc": []string{"vpc-xxxxxxx"},
+		"default-vpc": {"vpc-xxxxxxx"},
 	})
-	env := t.Prepare(c)
+	env := t.prepareEnviron(c)
 	err := bootstrap.Bootstrap(envtesting.BootstrapContext(c), env, bootstrap.BootstrapParams{})
 	c.Assert(err, jc.ErrorIsNil)
 
@@ -796,13 +814,44 @@ func (t *localServerSuite) TestReleaseAddress(c *gc.C) {
 	// Releasing a second time tests that the first call actually released
 	// it plus tests the error handling of ReleaseAddress
 	err = env.ReleaseAddress(instId, "", addr)
-	msg := fmt.Sprintf("failed to unassign IP address \"%v\" for instance \"%v\".*", addr.Value, instId)
+	msg := fmt.Sprintf(`failed to release address "8\.0\.0\.4" from instance %q.*`, instId)
 	c.Assert(err, gc.ErrorMatches, msg)
 }
 
+func (t *localServerSuite) TestReleaseAddressUnknownInstance(c *gc.C) {
+	env, _ := t.setUpInstanceWithDefaultVpc(c)
+
+	// We should be able to release an address with an unknown instance id
+	// without it being allocated.
+	addr := network.Address{Value: "8.0.0.4"}
+	err := env.ReleaseAddress(instance.UnknownId, "", addr)
+	c.Assert(err, jc.ErrorIsNil)
+}
+
+func (t *localServerSuite) TestNetworkInterfaces(c *gc.C) {
+	env, instId := t.setUpInstanceWithDefaultVpc(c)
+	interfaces, err := env.NetworkInterfaces(instId)
+	c.Assert(err, jc.ErrorIsNil)
+	expectedInterfaces := []network.InterfaceInfo{{
+		DeviceIndex:      0,
+		MACAddress:       "20:01:60:cb:27:37",
+		CIDR:             "10.10.0.0/20",
+		ProviderId:       "eni-0",
+		ProviderSubnetId: "subnet-0",
+		VLANTag:          0,
+		InterfaceName:    "unsupported0",
+		Disabled:         false,
+		NoAutoStart:      false,
+		ConfigType:       network.ConfigDHCP,
+		Address:          network.NewScopedAddress("10.10.0.5", network.ScopeCloudLocal),
+	}}
+	c.Assert(interfaces, jc.DeepEquals, expectedInterfaces)
+}
+
 func (t *localServerSuite) TestSubnets(c *gc.C) {
 	env, _ := t.setUpInstanceWithDefaultVpc(c)
-	subnets, err := env.Subnets("")
+
+	subnets, err := env.Subnets("", []network.Id{"subnet-0"})
 	c.Assert(err, jc.ErrorIsNil)
 
 	defaultSubnets := []network.SubnetInfo{{
@@ -816,65 +865,99 @@ func (t *localServerSuite) TestSubnets(c *gc.C) {
 	c.Assert(subnets, jc.DeepEquals, defaultSubnets)
 }
 
-func (t *localServerSuite) TestSupportAddressAllocationTrue(c *gc.C) {
+func (t *localServerSuite) TestSubnetsNoNetIds(c *gc.C) {
+	env, _ := t.setUpInstanceWithDefaultVpc(c)
+
+	_, err := env.Subnets("", []network.Id{})
+	c.Assert(err, gc.ErrorMatches, "subnetIds must not be empty")
+}
+
+func (t *localServerSuite) TestSubnetsMissingSubnet(c *gc.C) {
+	env, _ := t.setUpInstanceWithDefaultVpc(c)
+
+	_, err := env.Subnets("", []network.Id{"subnet-0", "Missing"})
+	c.Assert(err, gc.ErrorMatches, `failed to find the following subnet ids: \[Missing\]`)
+}
+
+func (t *localServerSuite) TestSupportsAddressAllocationTrue(c *gc.C) {
 	t.srv.ec2srv.SetInitialAttributes(map[string][]string{
-		"default-vpc": []string{"vpc-xxxxxxx"},
+		"default-vpc": {"vpc-xxxxxxx"},
 	})
-	env := t.Prepare(c)
-	result, err := env.SupportAddressAllocation("")
+	env := t.prepareEnviron(c)
+	result, err := env.SupportsAddressAllocation("")
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(result, jc.IsTrue)
 }
 
-func (t *localServerSuite) TestSupportAddressAllocationCaches(c *gc.C) {
+func (t *localServerSuite) TestSupportsAddressAllocationWithNoFeatureFlag(c *gc.C) {
+	t.SetFeatureFlags() // clear the flags.
+	env := t.prepareEnviron(c)
+	result, err := env.SupportsAddressAllocation("")
+	c.Assert(err, gc.ErrorMatches, "address allocation not supported")
+	c.Assert(err, jc.Satisfies, errors.IsNotSupported)
+	c.Assert(result, jc.IsFalse)
+}
+
+func (t *localServerSuite) TestAllocateAddressWithNoFeatureFlag(c *gc.C) {
+	t.SetFeatureFlags() // clear the flags.
+	env := t.prepareEnviron(c)
+	err := env.AllocateAddress("i-foo", "net1", network.NewAddresses("1.2.3.4")[0])
+	c.Assert(err, gc.ErrorMatches, "address allocation not supported")
+	c.Assert(err, jc.Satisfies, errors.IsNotSupported)
+}
+
+func (t *localServerSuite) TestReleaseAddressWithNoFeatureFlag(c *gc.C) {
+	t.SetFeatureFlags() // clear the flags.
+	env := t.prepareEnviron(c)
+	err := env.ReleaseAddress("i-foo", "net1", network.NewAddresses("1.2.3.4")[0])
+	c.Assert(err, gc.ErrorMatches, "address allocation not supported")
+	c.Assert(err, jc.Satisfies, errors.IsNotSupported)
+}
+
+func (t *localServerSuite) TestSupportsAddressAllocationCaches(c *gc.C) {
 	t.srv.ec2srv.SetInitialAttributes(map[string][]string{
-		"default-vpc": []string{"none"},
+		"default-vpc": {"none"},
 	})
-	env := t.Prepare(c)
-	result, err := env.SupportAddressAllocation("")
+	env := t.prepareEnviron(c)
+	result, err := env.SupportsAddressAllocation("")
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(result, jc.IsFalse)
 
 	// this value won't change normally, the change here is to
 	// ensure that subsequent calls use the cached value
 	t.srv.ec2srv.SetInitialAttributes(map[string][]string{
-		"default-vpc": []string{"vpc-xxxxxxx"},
+		"default-vpc": {"vpc-xxxxxxx"},
 	})
-	result, err = env.SupportAddressAllocation("")
+	result, err = env.SupportsAddressAllocation("")
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(result, jc.IsFalse)
 }
 
-func (t *localServerSuite) TestSupportAddressAllocationFalse(c *gc.C) {
+func (t *localServerSuite) TestSupportsAddressAllocationFalse(c *gc.C) {
 	t.srv.ec2srv.SetInitialAttributes(map[string][]string{
-		"default-vpc": []string{"none"},
+		"default-vpc": {"none"},
 	})
-	env := t.Prepare(c)
-	result, err := env.SupportAddressAllocation("")
+	env := t.prepareEnviron(c)
+	result, err := env.SupportsAddressAllocation("")
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(result, jc.IsFalse)
 }
 
-func (t *localServerSuite) TestStartInstanceDisks(c *gc.C) {
+func (t *localServerSuite) TestInstanceTags(c *gc.C) {
 	env := t.Prepare(c)
 	err := bootstrap.Bootstrap(envtesting.BootstrapContext(c), env, bootstrap.BootstrapParams{})
 	c.Assert(err, jc.ErrorIsNil)
 
-	params := environs.StartInstanceParams{
-		Disks: []storage.DiskParams{{
-			Size: 512, // round up to 1GiB
-		}, {
-			Size: 1024, // 1GiB exactly
-		}, {
-			Size: 1025, // round up to 2GiB
-		}},
-	}
-	result, err := testing.StartInstanceWithParams(env, "1", params, nil)
+	instances, err := env.AllInstances()
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(result.Disks, gc.HasLen, 3)
-	c.Assert(result.Disks[0].Size, gc.Equals, uint64(1024))
-	c.Assert(result.Disks[1].Size, gc.Equals, uint64(1024))
-	c.Assert(result.Disks[2].Size, gc.Equals, uint64(2048))
+	c.Assert(instances, gc.HasLen, 1)
+
+	ec2Inst := ec2.InstanceEC2(instances[0])
+	c.Assert(ec2Inst.Tags, jc.SameContents, []amzec2.Tag{
+		{"Name", "juju-sample-machine-0"},
+		{"juju-env-uuid", coretesting.EnvironmentTag.Id()},
+		{"juju-is-state", "true"},
+	})
 }
 
 // localNonUSEastSuite is similar to localServerSuite but the S3 mock server
