@@ -15,7 +15,9 @@ import (
 	"github.com/juju/names"
 	"github.com/juju/utils"
 
-	"github.com/juju/juju/cloudinit"
+	"github.com/juju/juju/cloudconfig/cloudinit"
+	"github.com/juju/juju/cloudconfig/instancecfg"
+	"github.com/juju/juju/cloudconfig/providerinit"
 	"github.com/juju/juju/constraints"
 	"github.com/juju/juju/environs"
 	"github.com/juju/juju/environs/imagemetadata"
@@ -23,7 +25,6 @@ import (
 	"github.com/juju/juju/environs/simplestreams"
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/juju/arch"
-	"github.com/juju/juju/network"
 	"github.com/juju/juju/provider/common"
 	"github.com/juju/juju/state/multiwatcher"
 	"github.com/juju/juju/tools"
@@ -33,6 +34,7 @@ var (
 	vTypeSmartmachine   = "smartmachine"
 	vTypeVirtualmachine = "kvm"
 	signedImageDataOnly = false
+	defaultCpuCores     = uint64(1)
 )
 
 type joyentCompute struct {
@@ -83,9 +85,14 @@ func (env *joyentEnviron) ConstraintsValidator() (constraints.Validator, error)
 	return validator, nil
 }
 
+// MaintainInstance is specified in the InstanceBroker interface.
+func (*joyentEnviron) MaintainInstance(args environs.StartInstanceParams) error {
+	return nil
+}
+
 func (env *joyentEnviron) StartInstance(args environs.StartInstanceParams) (*environs.StartInstanceResult, error) {
 
-	if args.MachineConfig.HasNetworks() {
+	if args.InstanceConfig.HasNetworks() {
 		return nil, errors.New("starting instances with networks is not supported yet")
 	}
 
@@ -105,9 +112,9 @@ func (env *joyentEnviron) StartInstance(args environs.StartInstanceParams) (*env
 		return nil, errors.Errorf("chosen architecture %v not present in %v", spec.Image.Arch, arches)
 	}
 
-	args.MachineConfig.Tools = tools[0]
+	args.InstanceConfig.Tools = tools[0]
 
-	if err := environs.FinishMachineConfig(args.MachineConfig, env.Config()); err != nil {
+	if err := instancecfg.FinishInstanceConfig(args.InstanceConfig, env.Config()); err != nil {
 		return nil, err
 	}
 
@@ -116,7 +123,10 @@ func (env *joyentEnviron) StartInstance(args environs.StartInstanceParams) (*env
 	// different 10.x.x.x/21 networks and adding this route allows
 	// them to talk despite this. See:
 	// https://bugs.launchpad.net/juju-core/+bug/1401130
-	cloudcfg := cloudinit.New()
+	cloudcfg, err := cloudinit.New(args.InstanceConfig.Series)
+	if err != nil {
+		return nil, errors.Annotate(err, "cannot create cloudinit template")
+	}
 	ifupScript := `
 #!/bin/bash
 
@@ -129,7 +139,7 @@ func (env *joyentEnviron) StartInstance(args environs.StartInstanceParams) (*env
 `[1:]
 	cloudcfg.AddBootTextFile("/etc/network/if-up.d/joyent", ifupScript, 0755)
 
-	userData, err := environs.ComposeUserData(args.MachineConfig, cloudcfg)
+	userData, err := providerinit.ComposeUserData(args.InstanceConfig, cloudcfg)
 	if err != nil {
 		return nil, errors.Annotate(err, "cannot make user data")
 	}
@@ -178,7 +188,7 @@ func (env *joyentEnviron) StartInstance(args environs.StartInstanceParams) (*env
 		env:     env,
 	}
 
-	if multiwatcher.AnyJobNeedsState(args.MachineConfig.Jobs...) {
+	if multiwatcher.AnyJobNeedsState(args.InstanceConfig.Jobs...) {
 		if err := common.AddStateInstance(env.Storage(), inst.Id()); err != nil {
 			logger.Errorf("could not record instance in provider-state: %v", err)
 		}
@@ -256,27 +266,6 @@ func (env *joyentEnviron) Instances(ids []instance.Id) ([]instance.Instance, err
 	return instances, nil
 }
 
-// AllocateAddress requests an address to be allocated for the
-// given instance on the given network. This is not implemented on the
-// Joyent provider yet.
-func (*joyentEnviron) AllocateAddress(_ instance.Id, _ network.Id, _ network.Address) error {
-	return errors.NotImplementedf("AllocateAddress")
-}
-
-// ReleaseAddress releases a specific address previously allocated with
-// AllocateAddress.
-func (*joyentEnviron) ReleaseAddress(_ instance.Id, _ network.Id, _ network.Address) error {
-	return errors.NotImplementedf("ReleaseAddress")
-}
-
-// Subnets returns basic information about all subnets known by
-// the provider for the environment. They may be unknown to juju yet
-// (i.e. when called initially or when a new network was created).
-// This is not implemented on the Joyent provider yet.
-func (*joyentEnviron) Subnets(_ instance.Id) ([]network.SubnetInfo, error) {
-	return nil, errors.NotImplementedf("Subnets")
-}
-
 func (env *joyentEnviron) StopInstances(ids ...instance.Id) error {
 	// Remove all the instances in parallel so that we incur less round-trips.
 	var wg sync.WaitGroup
@@ -327,11 +316,11 @@ func (env *joyentEnviron) stopInstance(id string) error {
 }
 
 func (env *joyentEnviron) pollMachineState(machineId, state string) bool {
-	machineConfig, err := env.compute.cloudapi.GetMachine(machineId)
+	instanceConfig, err := env.compute.cloudapi.GetMachine(machineId)
 	if err != nil {
 		return false
 	}
-	return strings.EqualFold(machineConfig.State, state)
+	return strings.EqualFold(instanceConfig.State, state)
 }
 
 func (env *joyentEnviron) listInstanceTypes() ([]instances.InstanceType, error) {
@@ -341,6 +330,14 @@ func (env *joyentEnviron) listInstanceTypes() ([]instances.InstanceType, error)
 	}
 	allInstanceTypes := []instances.InstanceType{}
 	for _, pkg := range packages {
+		// ListPackages does not include the virt type of the package.
+		// However, Joyent says the smart packages have zero VCPUs.
+		var virtType *string
+		if pkg.VCPUs > 0 {
+			virtType = &vTypeVirtualmachine
+		} else {
+			virtType = &vTypeSmartmachine
+		}
 		instanceType := instances.InstanceType{
 			Id:       pkg.Id,
 			Name:     pkg.Name,
@@ -348,7 +345,7 @@ func (env *joyentEnviron) listInstanceTypes() ([]instances.InstanceType, error)
 			Mem:      uint64(pkg.Memory),
 			CpuCores: uint64(pkg.VCPUs),
 			RootDisk: uint64(pkg.Disk * 1024),
-			VirtType: &vTypeVirtualmachine,
+			VirtType: virtType,
 		}
 		allInstanceTypes = append(allInstanceTypes, instanceType)
 	}
@@ -357,6 +354,10 @@ func (env *joyentEnviron) listInstanceTypes() ([]instances.InstanceType, error)
 
 // FindInstanceSpec returns an InstanceSpec satisfying the supplied instanceConstraint.
 func (env *joyentEnviron) FindInstanceSpec(ic *instances.InstanceConstraint) (*instances.InstanceSpec, error) {
+	// Require at least one VCPU so we get KVM rather than smart package.
+	if ic.Constraints.CpuCores == nil {
+		ic.Constraints.CpuCores = &defaultCpuCores
+	}
 	allInstanceTypes, err := env.listInstanceTypes()
 	if err != nil {
 		return nil, err
