@@ -20,13 +20,13 @@ import (
 	"github.com/juju/utils/symlink"
 
 	"github.com/juju/juju/agent"
-	coreCloudinit "github.com/juju/juju/cloudinit"
-	"github.com/juju/juju/cloudinit/sshinit"
+	"github.com/juju/juju/cloudconfig"
+	"github.com/juju/juju/cloudconfig/cloudinit"
+	"github.com/juju/juju/cloudconfig/instancecfg"
 	"github.com/juju/juju/constraints"
 	"github.com/juju/juju/container"
 	"github.com/juju/juju/container/factory"
 	"github.com/juju/juju/environs"
-	"github.com/juju/juju/environs/cloudinit"
 	"github.com/juju/juju/environs/config"
 	"github.com/juju/juju/environs/filestorage"
 	"github.com/juju/juju/environs/httpstorage"
@@ -37,8 +37,8 @@ import (
 	"github.com/juju/juju/mongo"
 	"github.com/juju/juju/network"
 	"github.com/juju/juju/provider/common"
+	"github.com/juju/juju/service"
 	servicecommon "github.com/juju/juju/service/common"
-	"github.com/juju/juju/service/upstart"
 	"github.com/juju/juju/state/multiwatcher"
 	"github.com/juju/juju/tools"
 	"github.com/juju/juju/version"
@@ -70,16 +70,6 @@ func (*localEnviron) SupportedArchitectures() ([]string, error) {
 	return []string{localArch}, nil
 }
 
-// SupportNetworks is specified on the EnvironCapability interface.
-func (*localEnviron) SupportNetworks() bool {
-	return false
-}
-
-// SupportAddressAllocation is specified on the EnvironCapability interface.
-func (e *localEnviron) SupportAddressAllocation(netId network.Id) (bool, error) {
-	return false, nil
-}
-
 func (*localEnviron) PrecheckInstance(series string, cons constraints.Value, placement string) error {
 	if placement != "" {
 		return fmt.Errorf("unknown placement directive: %s", placement)
@@ -132,18 +122,18 @@ func (env *localEnviron) Bootstrap(
 
 // finishBootstrap converts the machine config to cloud-config,
 // converts that to a script, and then executes it locally.
-func (env *localEnviron) finishBootstrap(ctx environs.BootstrapContext, mcfg *cloudinit.MachineConfig) error {
-	mcfg.InstanceId = bootstrapInstanceId
-	mcfg.DataDir = env.config.rootDir()
-	mcfg.LogDir = fmt.Sprintf("/var/log/juju-%s", env.config.namespace())
-	mcfg.CloudInitOutputLog = filepath.Join(mcfg.DataDir, "cloud-init-output.log")
+func (env *localEnviron) finishBootstrap(ctx environs.BootstrapContext, icfg *instancecfg.InstanceConfig) error {
+	icfg.InstanceId = bootstrapInstanceId
+	icfg.DataDir = env.config.rootDir()
+	icfg.LogDir = fmt.Sprintf("/var/log/juju-%s", env.config.namespace())
+	icfg.CloudInitOutputLog = filepath.Join(icfg.DataDir, "cloud-init-output.log")
 
 	// No JobManageNetworking added in order not to change the network
 	// configuration of the user's machine.
-	mcfg.Jobs = []multiwatcher.MachineJob{multiwatcher.JobManageEnviron}
+	icfg.Jobs = []multiwatcher.MachineJob{multiwatcher.JobManageEnviron}
 
-	mcfg.MachineAgentServiceName = env.machineAgentServiceName()
-	mcfg.AgentEnvironment = map[string]string{
+	icfg.MachineAgentServiceName = env.machineAgentServiceName()
+	icfg.AgentEnvironment = map[string]string{
 		agent.Namespace:   env.config.namespace(),
 		agent.StorageDir:  env.config.storageDir(),
 		agent.StorageAddr: env.config.storageAddr(),
@@ -155,8 +145,8 @@ func (env *localEnviron) finishBootstrap(ctx environs.BootstrapContext, mcfg *cl
 		agent.MongoOplogSize: "1", // 1MB
 	}
 
-	if err := environs.FinishMachineConfig(mcfg, env.Config()); err != nil {
-		return err
+	if err := instancecfg.FinishInstanceConfig(icfg, env.Config()); err != nil {
+		return errors.Trace(err)
 	}
 
 	// Since Juju's state machine is currently the host machine
@@ -164,62 +154,65 @@ func (env *localEnviron) finishBootstrap(ctx environs.BootstrapContext, mcfg *cl
 	cfgAttrs := env.config.AllAttrs()
 	if val, ok := cfgAttrs["enable-os-refresh-update"].(bool); !ok {
 		logger.Infof("local provider; disabling refreshing OS updates.")
-		mcfg.EnableOSRefreshUpdate = false
+		icfg.EnableOSRefreshUpdate = false
 	} else {
-		mcfg.EnableOSRefreshUpdate = val
+		icfg.EnableOSRefreshUpdate = val
 	}
 	if val, ok := cfgAttrs["enable-os-upgrade"].(bool); !ok {
 		logger.Infof("local provider; disabling OS upgrades.")
-		mcfg.EnableOSUpgrade = false
+		icfg.EnableOSUpgrade = false
 	} else {
-		mcfg.EnableOSUpgrade = val
+		icfg.EnableOSUpgrade = val
 	}
 
 	// don't write proxy or mirror settings for local machine
-	mcfg.AptProxySettings = proxy.Settings{}
-	mcfg.ProxySettings = proxy.Settings{}
-	mcfg.AptMirror = ""
+	icfg.AptProxySettings = proxy.Settings{}
+	icfg.ProxySettings = proxy.Settings{}
+	icfg.AptMirror = ""
 
-	cloudcfg := coreCloudinit.New()
-	cloudcfg.SetAptUpdate(mcfg.EnableOSRefreshUpdate)
-	cloudcfg.SetAptUpgrade(mcfg.EnableOSUpgrade)
+	cloudcfg, err := cloudinit.New(icfg.Series)
+	if err != nil {
+		return errors.Trace(err)
+	}
+	cloudcfg.SetSystemUpdate(icfg.EnableOSRefreshUpdate)
+	cloudcfg.SetSystemUpgrade(icfg.EnableOSUpgrade)
 
 	// Since rsyslogd is restricted by apparmor to only write to /var/log/**
 	// we now provide a symlink to the written file in the local log dir.
 	// Also, we leave the old all-machines.log file in
 	// /var/log/juju-{{namespace}} until we start the environment again. So
 	// potentially remove it at the start of the cloud-init.
-	localLogDir := filepath.Join(mcfg.DataDir, "log")
+	localLogDir := filepath.Join(icfg.DataDir, "log")
 	if err := os.RemoveAll(localLogDir); err != nil {
-		return err
+		return errors.Trace(err)
 	}
-	if err := symlink.New(mcfg.LogDir, localLogDir); err != nil {
-		return err
+	if err := symlink.New(icfg.LogDir, localLogDir); err != nil {
+		return errors.Trace(err)
 	}
-	if err := os.Remove(mcfg.CloudInitOutputLog); err != nil && !os.IsNotExist(err) {
-		return err
+	if err := os.Remove(icfg.CloudInitOutputLog); err != nil && !os.IsNotExist(err) {
+		return errors.Trace(err)
 	}
 	cloudcfg.AddScripts(
-		fmt.Sprintf("rm -fr %s", mcfg.LogDir),
+		fmt.Sprintf("rm -fr %s", icfg.LogDir),
 		fmt.Sprintf("rm -f /var/spool/rsyslog/machine-0-%s", env.config.namespace()),
 	)
-	udata, err := cloudinit.NewUserdataConfig(mcfg, cloudcfg)
+	udata, err := cloudconfig.NewUserdataConfig(icfg, cloudcfg)
 	if err != nil {
-		return err
+		return errors.Trace(err)
 	}
 	if err := udata.ConfigureJuju(); err != nil {
-		return err
+		return errors.Trace(err)
 	}
-	return executeCloudConfig(ctx, mcfg, cloudcfg)
+	return executeCloudConfig(ctx, icfg, cloudcfg)
 }
 
-var executeCloudConfig = func(ctx environs.BootstrapContext, mcfg *cloudinit.MachineConfig, cloudcfg *coreCloudinit.Config) error {
+var executeCloudConfig = func(ctx environs.BootstrapContext, icfg *instancecfg.InstanceConfig, cloudcfg cloudinit.CloudConfig) error {
 	// Finally, convert cloud-config to a script and execute it.
-	configScript, err := sshinit.ConfigureScript(cloudcfg)
+	configScript, err := cloudcfg.RenderScript()
 	if err != nil {
 		return nil
 	}
-	script := shell.DumpFileOnErrorScript(mcfg.CloudInitOutputLog) + configScript
+	script := shell.DumpFileOnErrorScript(icfg.CloudInitOutputLog) + configScript
 	cmd := exec.Command("sudo", "/bin/bash", "-s")
 	cmd.Stdin = strings.NewReader(script)
 	cmd.Stdout = ctx.GetStdout()
@@ -252,7 +245,7 @@ func (env *localEnviron) SetConfig(cfg *config.Config) error {
 	ecfg, err := providerInstance.newConfig(cfg)
 	if err != nil {
 		logger.Errorf("failed to create new environ config: %v", err)
-		return err
+		return errors.Trace(err)
 	}
 	env.localMutex.Lock()
 	defer env.localMutex.Unlock()
@@ -284,7 +277,7 @@ func (env *localEnviron) SetConfig(cfg *config.Config) error {
 	env.containerManager, err = factory.NewContainerManager(
 		containerType, managerConfig, imageURLGetter)
 	if err != nil {
-		return err
+		return errors.Trace(err)
 	}
 
 	// When the localEnviron value is created on the client
@@ -305,11 +298,11 @@ func (env *localEnviron) SetConfig(cfg *config.Config) error {
 	// from the command line, so it is ok to work on the assumption that we
 	// have direct access to the directories.
 	if err := env.config.createDirs(); err != nil {
-		return err
+		return errors.Trace(err)
 	}
 	// Record the network bridge address and create a filestorage.
 	if err := env.resolveBridgeAddress(cfg); err != nil {
-		return err
+		return errors.Trace(err)
 	}
 	return env.setLocalStorage()
 }
@@ -365,25 +358,30 @@ func (env *localEnviron) ConstraintsValidator() (constraints.Validator, error) {
 	return validator, nil
 }
 
+// MaintainInstance is specified in the InstanceBroker interface.
+func (*localEnviron) MaintainInstance(args environs.StartInstanceParams) error {
+	return nil
+}
+
 // StartInstance is specified in the InstanceBroker interface.
 func (env *localEnviron) StartInstance(args environs.StartInstanceParams) (*environs.StartInstanceResult, error) {
-	if args.MachineConfig.HasNetworks() {
+	if args.InstanceConfig.HasNetworks() {
 		return nil, fmt.Errorf("starting instances with networks is not supported yet.")
 	}
 	series := args.Tools.OneSeries()
-	logger.Debugf("StartInstance: %q, %s", args.MachineConfig.MachineId, series)
-	args.MachineConfig.Tools = args.Tools[0]
+	logger.Debugf("StartInstance: %q, %s", args.InstanceConfig.MachineId, series)
+	args.InstanceConfig.Tools = args.Tools[0]
 
-	args.MachineConfig.MachineContainerType = env.config.container()
-	logger.Debugf("tools: %#v", args.MachineConfig.Tools)
-	if err := environs.FinishMachineConfig(args.MachineConfig, env.config.Config); err != nil {
+	args.InstanceConfig.MachineContainerType = env.config.container()
+	logger.Debugf("tools: %#v", args.InstanceConfig.Tools)
+	if err := instancecfg.FinishInstanceConfig(args.InstanceConfig, env.config.Config); err != nil {
 		return nil, err
 	}
-	// TODO: evaluate the impact of setting the contstraints on the
-	// machineConfig for all machines rather than just state server nodes.
-	// This limiation is why the constraints are assigned directly here.
-	args.MachineConfig.Constraints = args.Constraints
-	args.MachineConfig.AgentEnvironment[agent.Namespace] = env.config.namespace()
+	// TODO: evaluate the impact of setting the constraints on the
+	// instanceConfig for all machines rather than just state server nodes.
+	// This limitation is why the constraints are assigned directly here.
+	args.InstanceConfig.Constraints = args.Constraints
+	args.InstanceConfig.AgentEnvironment[agent.Namespace] = env.config.namespace()
 	inst, hardware, err := createContainer(env, args)
 	if err != nil {
 		return nil, err
@@ -397,8 +395,13 @@ func (env *localEnviron) StartInstance(args environs.StartInstanceParams) (*envi
 // Override for testing.
 var createContainer = func(env *localEnviron, args environs.StartInstanceParams) (instance.Instance, *instance.HardwareCharacteristics, error) {
 	series := args.Tools.OneSeries()
-	network := container.BridgeNetworkConfig(env.config.networkBridge())
-	inst, hardware, err := env.containerManager.CreateContainer(args.MachineConfig, series, network)
+	network := container.BridgeNetworkConfig(env.config.networkBridge(), 0, args.NetworkInfo)
+	allowLoopMounts, _ := env.config.AllowLXCLoopMounts()
+	isLXC := env.config.container() == instance.LXC
+	storage := &container.StorageConfig{
+		AllowMount: !isLXC || allowLoopMounts,
+	}
+	inst, hardware, err := env.containerManager.CreateContainer(args.InstanceConfig, series, network, storage)
 	if err != nil {
 		return nil, nil, err
 	}
@@ -449,27 +452,6 @@ func (env *localEnviron) Instances(ids []instance.Id) ([]instance.Instance, erro
 	return insts, err
 }
 
-// AllocateAddress requests an address to be allocated for the
-// given instance on the given network. This is not supported on the
-// local provider.
-func (*localEnviron) AllocateAddress(_ instance.Id, _ network.Id, _ network.Address) error {
-	return errors.NotSupportedf("AllocateAddress")
-}
-
-// ReleaseAddress releases a specific address previously allocated with
-// AllocateAddress.
-func (*localEnviron) ReleaseAddress(_ instance.Id, _ network.Id, _ network.Address) error {
-	return errors.NotSupportedf("ReleaseAddress")
-}
-
-// Subnets returns basic information about all subnets known
-// by the provider for the environment. They may be unknown to juju
-// yet (i.e. when called initially or when a new network was created).
-// This is not implemented by the local provider yet.
-func (*localEnviron) Subnets(_ instance.Id) ([]network.SubnetInfo, error) {
-	return nil, errors.NotSupportedf("Subnets")
-}
-
 // AllInstances is specified in the InstanceBroker interface.
 func (env *localEnviron) AllInstances() (instances []instance.Instance, err error) {
 	instances = append(instances, &localInstance{bootstrapInstanceId, env})
@@ -546,10 +528,21 @@ func (env *localEnviron) Destroy() error {
 			}
 		}
 	}
-	// Stop the mongo database and machine agent. It's possible that the
-	// service doesn't exist or is not running, so don't check the error.
-	mongo.RemoveService(env.config.namespace())
-	upstart.NewService(env.machineAgentServiceName(), servicecommon.Conf{}).StopAndRemove()
+	// Stop the mongo database and machine agent. We log any errors but
+	// do not fail, so that remaining "destroy" steps will still happen.
+	err = mongoRemoveService(env.config.namespace())
+	if err != nil && !errors.IsNotFound(err) {
+		logger.Errorf("while stopping mongod: %v", err)
+	}
+	svc, err := discoverService(env.machineAgentServiceName())
+	if err == nil {
+		if err := svc.Stop(); err != nil {
+			logger.Errorf("while stopping machine agent: %v", err)
+		}
+		if err := svc.Remove(); err != nil {
+			logger.Errorf("while disabling machine agent: %v", err)
+		}
+	}
 
 	// Finally, remove the data-dir.
 	if err := os.RemoveAll(env.config.rootDir()); err != nil && !os.IsNotExist(err) {
@@ -566,6 +559,19 @@ func (env *localEnviron) Destroy() error {
 	return nil
 }
 
+type agentService interface {
+	Stop() error
+	Remove() error
+}
+
+var mongoRemoveService = func(namespace string) error {
+	return mongo.RemoveService(namespace)
+}
+
+var discoverService = func(name string) (agentService, error) {
+	return service.DiscoverService(name, servicecommon.Conf{})
+}
+
 // OpenPorts is specified in the Environ interface.
 func (env *localEnviron) OpenPorts(ports []network.PortRange) error {
 	return fmt.Errorf("open ports not implemented")
