@@ -7,25 +7,29 @@ import (
 	jc "github.com/juju/testing/checkers"
 	gc "gopkg.in/check.v1"
 
-	"github.com/juju/juju/environs"
 	"github.com/juju/juju/environs/config"
 	"github.com/juju/juju/instance"
+	"github.com/juju/juju/mongo"
+	"github.com/juju/juju/service/common"
+	svctesting "github.com/juju/juju/service/common/testing"
 )
 
 var (
-	CheckIfRoot        = &checkIfRoot
-	CheckLocalPort     = &checkLocalPort
-	DetectAptProxies   = &detectAptProxies
-	ExecuteCloudConfig = &executeCloudConfig
-	Provider           = providerInstance
-	UserCurrent        = &userCurrent
+	CheckIfRoot          = &checkIfRoot
+	CheckLocalPort       = &checkLocalPort
+	DetectPackageProxies = &detectPackageProxies
+	ExecuteCloudConfig   = &executeCloudConfig
+	Provider             = providerInstance
+	GetNamespace         = &getNamespace
 )
 
-// ConfigNamespace returns the result of the namespace call on the
+// CheckConfigNamespace checks the result of the namespace call on the
 // localConfig.
-func ConfigNamespace(cfg *config.Config) string {
-	env, _ := providerInstance.Open(cfg)
-	return env.(*localEnviron).config.namespace()
+func CheckConfigNamespace(c *gc.C, cfg *config.Config, expected string) {
+	env, err := providerInstance.Open(cfg)
+	c.Assert(err, jc.ErrorIsNil)
+	namespace := env.(*localEnviron).config.namespace()
+	c.Assert(namespace, gc.Equals, expected)
 }
 
 // CreateDirs calls createDirs on the localEnviron.
@@ -72,14 +76,19 @@ func (inst *mockInstance) Id() instance.Id {
 	return instance.Id(inst.id)
 }
 
-type startInstanceFunc func(*localEnviron, environs.StartInstanceParams) (instance.Instance, *instance.HardwareCharacteristics, error)
+func PatchServices(patchValue func(interface{}, interface{}), data *svctesting.FakeServiceData) {
+	patchValue(&mongoRemoveService, func(namespace string) error {
+		data.AddCall("RemoveService", namespace)
+		data.SetStatus(mongo.ServiceName(namespace), "")
+		return data.NextErr()
+	})
+	patchValue(&discoverService, func(name string) (agentService, error) {
+		return NewService(name, common.Conf{}, data), nil
+	})
+}
 
-func PatchCreateContainer(s *testing.CleanupSuite, c *gc.C, expectedURL string) startInstanceFunc {
-	mockFunc := func(_ *localEnviron, args environs.StartInstanceParams) (instance.Instance, *instance.HardwareCharacteristics, error) {
-		c.Assert(args.Tools, gc.HasLen, 1)
-		c.Assert(args.Tools[0].URL, gc.Equals, expectedURL)
-		return &mockInstance{id: "mock"}, nil, nil
-	}
-	s.PatchValue(&createContainer, mockFunc)
-	return mockFunc
+func NewService(name string, conf common.Conf, data *svctesting.FakeServiceData) *svctesting.FakeService {
+	svc := svctesting.NewFakeService(name, conf)
+	svc.FakeServiceData = data
+	return svc
 }
