@@ -24,7 +24,9 @@ import (
 	"launchpad.net/gomaasapi"
 
 	"github.com/juju/juju/agent"
-	"github.com/juju/juju/cloudinit"
+	"github.com/juju/juju/cloudconfig/cloudinit"
+	"github.com/juju/juju/cloudconfig/instancecfg"
+	"github.com/juju/juju/cloudconfig/providerinit"
 	"github.com/juju/juju/constraints"
 	"github.com/juju/juju/environs"
 	"github.com/juju/juju/environs/config"
@@ -35,6 +37,7 @@ import (
 	"github.com/juju/juju/state/multiwatcher"
 	"github.com/juju/juju/tools"
 	"github.com/juju/juju/version"
+	"github.com/juju/names"
 )
 
 const (
@@ -67,7 +70,7 @@ func releaseNodes(nodes gomaasapi.MAASObject, ids url.Values) error {
 func reserveIPAddress(ipaddresses gomaasapi.MAASObject, cidr string, addr network.Address) error {
 	params := url.Values{}
 	params.Add("network", cidr)
-	params.Add("ip", addr.Value)
+	params.Add("requested_address", addr.Value)
 	_, err := ipaddresses.CallPost("reserve", params)
 	return err
 }
@@ -116,10 +119,23 @@ func NewEnviron(cfg *config.Config) (*maasEnviron, error) {
 
 // Bootstrap is specified in the Environ interface.
 func (env *maasEnviron) Bootstrap(ctx environs.BootstrapContext, args environs.BootstrapParams) (arch, series string, _ environs.BootstrapFinalizer, _ error) {
-	// Override the network bridge device used for both LXC and KVM
-	// containers, because we'll be creating juju-br0 at bootstrap
-	// time.
-	args.ContainerBridgeName = environs.DefaultBridgeName
+	if !environs.AddressAllocationEnabled() {
+		// When address allocation is not enabled, we should use the
+		// default bridge for both LXC and KVM containers. The bridge
+		// is created as part of the userdata for every node during
+		// StartInstance.
+		logger.Infof(
+			"address allocation feature disabled; using %q bridge for all containers",
+			instancecfg.DefaultBridgeName,
+		)
+		args.ContainerBridgeName = instancecfg.DefaultBridgeName
+	} else {
+		logger.Debugf(
+			"address allocation feature enabled; using static IPs for containers: %q",
+			instancecfg.DefaultBridgeName,
+		)
+	}
+
 	result, series, finalizer, err := common.BootstrapInstance(ctx, env, args)
 	if err != nil {
 		return "", "", nil, err
@@ -216,8 +232,12 @@ func (env *maasEnviron) SupportedArchitectures() ([]string, error) {
 	return env.supportedArchitectures, nil
 }
 
-// SupportAddressAllocation is specified on the EnvironCapability interface.
-func (env *maasEnviron) SupportAddressAllocation(netId network.Id) (bool, error) {
+// SupportsAddressAllocation is specified on environs.Networking.
+func (env *maasEnviron) SupportsAddressAllocation(_ network.Id) (bool, error) {
+	if !environs.AddressAllocationEnabled() {
+		return false, errors.NotSupportedf("address allocation")
+	}
+
 	caps, err := env.getCapabilities()
 	if err != nil {
 		return false, errors.Annotatef(err, "getCapabilities failed")
@@ -286,39 +306,39 @@ func (env *maasEnviron) getNodegroupInterfaces(nodegroups []string) map[string][
 		interfacesObject := nodegroupsObject.GetSubObject(uuid).GetSubObject("interfaces")
 		interfacesResult, err := interfacesObject.CallGet("list", nil)
 		if err != nil {
-			logger.Warningf("could not fetch nodegroup-interfaces for nodegroup %v: %v", uuid, err)
+			logger.Debugf("cannot list interfaces for nodegroup %v: %v", uuid, err)
 			continue
 		}
 		interfaces, err := interfacesResult.GetArray()
 		if err != nil {
-			logger.Warningf("could not fetch nodegroup-interfaces for nodegroup %v: %v", uuid, err)
+			logger.Debugf("cannot get interfaces for nodegroup %v: %v", uuid, err)
 			continue
 		}
 		for _, interfaceResult := range interfaces {
 			nic, err := interfaceResult.GetMap()
 			if err != nil {
-				logger.Warningf("could not fetch interface %v for nodegroup %v: %v", nic, uuid, err)
+				logger.Debugf("cannot get interface %v for nodegroup %v: %v", nic, uuid, err)
 				continue
 			}
 			ip, err := nic["ip"].GetString()
 			if err != nil {
-				logger.Warningf("could not fetch interface %v for nodegroup %v: %v", nic, uuid, err)
+				logger.Debugf("cannot get interface IP %v for nodegroup %v: %v", nic, uuid, err)
 				continue
 			}
 			static_low, err := nic["static_ip_range_low"].GetString()
 			if err != nil {
-				logger.Warningf("could not fetch static IP range lower bound for interface %v on nodegroup %v: %v", nic, uuid, err)
+				logger.Debugf("cannot get static IP range lower bound for interface %v on nodegroup %v: %v", nic, uuid, err)
 				continue
 			}
 			static_high, err := nic["static_ip_range_high"].GetString()
 			if err != nil {
-				logger.Warningf("could not fetch static IP range higher bound for interface %v on nodegroup %v: %v", nic, uuid, err)
+				logger.Infof("cannot get static IP range higher bound for interface %v on nodegroup %v: %v", nic, uuid, err)
 				continue
 			}
 			static_low_ip := net.ParseIP(static_low)
 			static_high_ip := net.ParseIP(static_high)
 			if static_low_ip == nil || static_high_ip == nil {
-				logger.Warningf("invalid IP in static range for interface %v on nodegroup %v: %q %q", nic, uuid, static_low_ip, static_high_ip)
+				logger.Debugf("invalid IP in static range for interface %v on nodegroup %v: %q %q", nic, uuid, static_low_ip, static_high_ip)
 				continue
 			}
 			nodegroupsInterfacesMap[ip] = []net.IP{static_low_ip, static_high_ip}
@@ -462,16 +482,6 @@ func (e *maasEnviron) InstanceAvailabilityZoneNames(ids []instance.Id) ([]string
 	return zones, nil
 }
 
-// SupportNetworks is specified on the EnvironCapability interface.
-func (env *maasEnviron) SupportNetworks() bool {
-	caps, err := env.getCapabilities()
-	if err != nil {
-		logger.Debugf("getCapabilities failed: %v", err)
-		return false
-	}
-	return caps.Contains(capNetworksManagement)
-}
-
 type maasPlacement struct {
 	nodeName string
 	zoneName string
@@ -527,7 +537,8 @@ func (env *maasEnviron) getCapabilities() (set.Strings, error) {
 			if err, ok := err.(gomaasapi.ServerError); ok && err.StatusCode == 404 {
 				return caps, fmt.Errorf("MAAS does not support version info")
 			}
-			return caps, err
+		} else {
+			break
 		}
 	}
 	if err != nil {
@@ -591,10 +602,6 @@ func convertConstraints(cons constraints.Value) url.Values {
 			params.Add("not_tags", strings.Join(notTags, ","))
 		}
 	}
-	// TODO(bug 1212689): ignore root-disk constraint for now.
-	if cons.RootDisk != nil {
-		logger.Warningf("ignoring unsupported constraint 'root-disk'")
-	}
 	if cons.CpuPower != nil {
 		logger.Warningf("ignoring unsupported constraint 'cpu-power'")
 	}
@@ -636,10 +643,45 @@ func addNetworks(params url.Values, includeNetworks, excludeNetworks []string) {
 	}
 }
 
+// addVolumes converts volume information into
+// url.Values object suitable to pass to MAAS when acquiring a node.
+func addVolumes(params url.Values, volumes []volumeInfo) {
+	if len(volumes) == 0 {
+		return
+	}
+	// Requests for specific values are passed to the acquire URL
+	// as a storage URL parameter of the form:
+	// [volume-name:]sizeinGB[tag,...]
+	// See http://maas.ubuntu.com/docs/api.html#nodes
+
+	// eg storage=root:0(ssd),data:20(magnetic,5400rpm),45
+	makeVolumeParams := func(v volumeInfo) string {
+		var params string
+		if v.name != "" {
+			params = v.name + ":"
+		}
+		params += fmt.Sprintf("%d", v.sizeInGB)
+		if len(v.tags) > 0 {
+			params += fmt.Sprintf("(%s)", strings.Join(v.tags, ","))
+		}
+		return params
+	}
+	var volParms []string
+	for _, v := range volumes {
+		params := makeVolumeParams(v)
+		volParms = append(volParms, params)
+	}
+	params.Add("storage", strings.Join(volParms, ","))
+}
+
 // acquireNode allocates a node from the MAAS.
-func (environ *maasEnviron) acquireNode(nodeName, zoneName string, cons constraints.Value, includeNetworks, excludeNetworks []string) (gomaasapi.MAASObject, error) {
+func (environ *maasEnviron) acquireNode(
+	nodeName, zoneName string, cons constraints.Value, includeNetworks, excludeNetworks []string, volumes []volumeInfo,
+) (gomaasapi.MAASObject, error) {
+
 	acquireParams := convertConstraints(cons)
 	addNetworks(acquireParams, includeNetworks, excludeNetworks)
+	addVolumes(acquireParams, volumes)
 	acquireParams.Add("agent_name", environ.ecfg().maasAgentName())
 	if zoneName != "" {
 		acquireParams.Add("zone", zoneName)
@@ -700,35 +742,6 @@ func (environ *maasEnviron) startNode(node gomaasapi.MAASObject, series string,
 	return err
 }
 
-const bridgeConfigTemplate = `cat >> {{.Config}} << EOF
-
-iface {{.PrimaryNIC}} inet manual
-
-auto {{.Bridge}}
-iface {{.Bridge}} inet dhcp
-    bridge_ports {{.PrimaryNIC}}
-EOF
-grep -q 'iface {{.PrimaryNIC}} inet dhcp' {{.Config}} && \
-sed -i 's/iface {{.PrimaryNIC}} inet dhcp//' {{.Config}}`
-
-// setupJujuNetworking returns a string representing the script to run
-// in order to prepare the Juju-specific networking config on a node.
-func setupJujuNetworking(primaryIface string) (string, error) {
-	parsedTemplate := template.Must(
-		template.New("BridgeConfig").Parse(bridgeConfigTemplate),
-	)
-	var buf bytes.Buffer
-	err := parsedTemplate.Execute(&buf, map[string]interface{}{
-		"Config":     "/etc/network/interfaces",
-		"Bridge":     environs.DefaultBridgeName,
-		"PrimaryNIC": primaryIface,
-	})
-	if err != nil {
-		return "", errors.Annotate(err, "bridge config template error")
-	}
-	return buf.String(), nil
-}
-
 var unsupportedConstraints = []string{
 	constraints.CpuPower,
 	constraints.InstanceType,
@@ -746,12 +759,12 @@ func (environ *maasEnviron) ConstraintsValidator() (constraints.Validator, error
 	return validator, nil
 }
 
-// setupNetworks prepares a []network.Info for the given instance. Any
-// networks in networksToDisable will be configured as disabled on the
-// machine. Any disabled network interfaces (as discovered from the
-// lshw output for the node) will stay disabled. The interface name
-// discovered as primary is also returned.
-func (environ *maasEnviron) setupNetworks(inst instance.Instance, networksToDisable set.Strings) ([]network.Info, string, error) {
+// setupNetworks prepares a []network.InterfaceInfo for the given
+// instance. Any networks in networksToDisable will be configured as
+// disabled on the machine. Any disabled network interfaces (as
+// discovered from the lshw output for the node) will stay disabled.
+// The interface name discovered as primary is also returned.
+func (environ *maasEnviron) setupNetworks(inst instance.Instance, networksToDisable set.Strings) ([]network.InterfaceInfo, string, error) {
 	// Get the instance network interfaces first.
 	interfaces, primaryIface, err := environ.getInstanceNetworkInterfaces(inst)
 	if err != nil {
@@ -763,7 +776,7 @@ func (environ *maasEnviron) setupNetworks(inst instance.Instance, networksToDisa
 		return nil, "", errors.Annotatef(err, "getInstanceNetworks failed")
 	}
 	logger.Debugf("node %q has networks %v", inst.Id(), networks)
-	var tempNetworkInfo []network.Info
+	var tempInterfaceInfo []network.InterfaceInfo
 	for _, netw := range networks {
 		disabled := networksToDisable.Contains(netw.Name)
 		netCIDR := &net.IPNet{
@@ -777,7 +790,7 @@ func (environ *maasEnviron) setupNetworks(inst instance.Instance, networksToDisa
 		logger.Debugf("network %q has MACs: %v", netw.Name, macs)
 		for _, mac := range macs {
 			if ifinfo, ok := interfaces[mac]; ok {
-				tempNetworkInfo = append(tempNetworkInfo, network.Info{
+				tempInterfaceInfo = append(tempInterfaceInfo, network.InterfaceInfo{
 					MACAddress:    mac,
 					InterfaceName: ifinfo.InterfaceName,
 					DeviceIndex:   ifinfo.DeviceIndex,
@@ -792,20 +805,20 @@ func (environ *maasEnviron) setupNetworks(inst instance.Instance, networksToDisa
 	}
 	// Verify we filled-in everything for all networks/interfaces
 	// and drop incomplete records.
-	var networkInfo []network.Info
-	for _, info := range tempNetworkInfo {
+	var interfaceInfo []network.InterfaceInfo
+	for _, info := range tempInterfaceInfo {
 		if info.ProviderId == "" || info.NetworkName == "" || info.CIDR == "" {
-			logger.Warningf("ignoring network interface %q: missing network information", info.InterfaceName)
+			logger.Infof("ignoring interface %q: missing subnet info", info.InterfaceName)
 			continue
 		}
 		if info.MACAddress == "" || info.InterfaceName == "" {
-			logger.Warningf("ignoring network %q: missing network interface information", info.ProviderId)
+			logger.Infof("ignoring subnet %q: missing interface info", info.ProviderId)
 			continue
 		}
-		networkInfo = append(networkInfo, info)
+		interfaceInfo = append(interfaceInfo, info)
 	}
-	logger.Debugf("node %q network information: %#v", inst.Id(), networkInfo)
-	return networkInfo, primaryIface, nil
+	logger.Debugf("node %q network information: %#v", inst.Id(), interfaceInfo)
+	return interfaceInfo, primaryIface, nil
 }
 
 // DistributeInstances implements the state.InstanceDistributor policy.
@@ -815,6 +828,11 @@ func (e *maasEnviron) DistributeInstances(candidates, distributionGroup []instan
 
 var availabilityZoneAllocations = common.AvailabilityZoneAllocations
 
+// MaintainInstance is specified in the InstanceBroker interface.
+func (*maasEnviron) MaintainInstance(args environs.StartInstanceParams) error {
+	return nil
+}
+
 // StartInstance is specified in the InstanceBroker interface.
 func (environ *maasEnviron) StartInstance(args environs.StartInstanceParams) (
 	*environs.StartInstanceResult, error,
@@ -862,16 +880,24 @@ func (environ *maasEnviron) StartInstance(args environs.StartInstanceParams) (
 		availabilityZones = []string{""}
 	}
 
-	requestedNetworks := args.MachineConfig.Networks
+	// Networking.
+	requestedNetworks := args.InstanceConfig.Networks
 	includeNetworks := append(args.Constraints.IncludeNetworks(), requestedNetworks...)
 	excludeNetworks := args.Constraints.ExcludeNetworks()
 
+	// Storage.
+	volumes, err := buildMAASVolumeParameters(args.Volumes, args.Constraints)
+	if err != nil {
+		return nil, errors.Annotate(err, "invalid volume parameters")
+	}
+
 	snArgs := selectNodeArgs{
 		Constraints:       args.Constraints,
 		AvailabilityZones: availabilityZones,
 		NodeName:          nodeName,
 		IncludeNetworks:   includeNetworks,
 		ExcludeNetworks:   excludeNetworks,
+		Volumes:           volumes,
 	}
 	node, err := environ.selectNode(snArgs)
 	if err != nil {
@@ -898,9 +924,9 @@ func (environ *maasEnviron) StartInstance(args environs.StartInstanceParams) (
 	if err != nil {
 		return nil, err
 	}
-	args.MachineConfig.Tools = selectedTools[0]
+	args.InstanceConfig.Tools = selectedTools[0]
 
-	var networkInfo []network.Info
+	var networkInfo []network.InterfaceInfo
 	networkInfo, primaryIface, err := environ.setupNetworks(inst, set.NewStrings(excludeNetworks...))
 	if err != nil {
 		return nil, err
@@ -911,21 +937,24 @@ func (environ *maasEnviron) StartInstance(args environs.StartInstanceParams) (
 		return nil, err
 	}
 	// Override the network bridge to use for both LXC and KVM
-	// containers on the new instance.
-	if args.MachineConfig.AgentEnvironment == nil {
-		args.MachineConfig.AgentEnvironment = make(map[string]string)
+	// containers on the new instance, if address allocation feature
+	// flag is not enabled.
+	if !environs.AddressAllocationEnabled() {
+		if args.InstanceConfig.AgentEnvironment == nil {
+			args.InstanceConfig.AgentEnvironment = make(map[string]string)
+		}
+		args.InstanceConfig.AgentEnvironment[agent.LxcBridge] = instancecfg.DefaultBridgeName
 	}
-	args.MachineConfig.AgentEnvironment[agent.LxcBridge] = environs.DefaultBridgeName
-	if err := environs.FinishMachineConfig(args.MachineConfig, environ.Config()); err != nil {
+	if err := instancecfg.FinishInstanceConfig(args.InstanceConfig, environ.Config()); err != nil {
 		return nil, err
 	}
-	series := args.MachineConfig.Tools.Version.Series
+	series := args.InstanceConfig.Tools.Version.Series
 
 	cloudcfg, err := environ.newCloudinitConfig(hostname, primaryIface, series)
 	if err != nil {
 		return nil, err
 	}
-	userdata, err := environs.ComposeUserData(args.MachineConfig, cloudcfg)
+	userdata, err := providerinit.ComposeUserData(args.InstanceConfig, cloudcfg)
 	if err != nil {
 		msg := fmt.Errorf("could not compose userdata for bootstrap node: %v", err)
 		return nil, msg
@@ -937,16 +966,34 @@ func (environ *maasEnviron) StartInstance(args environs.StartInstanceParams) (
 	}
 	logger.Debugf("started instance %q", inst.Id())
 
-	if multiwatcher.AnyJobNeedsState(args.MachineConfig.Jobs...) {
+	if multiwatcher.AnyJobNeedsState(args.InstanceConfig.Jobs...) {
 		if err := common.AddStateInstance(environ.Storage(), inst.Id()); err != nil {
 			logger.Errorf("could not record instance in provider-state: %v", err)
 		}
 	}
 
+	requestedVolumes := make([]names.VolumeTag, len(args.Volumes))
+	for i, v := range args.Volumes {
+		requestedVolumes[i] = v.Tag
+	}
+	resultVolumes, resultAttachments, err := inst.volumes(
+		names.NewMachineTag(args.InstanceConfig.MachineId),
+		requestedVolumes,
+	)
+	if err != nil {
+		return nil, err
+	}
+	if len(resultVolumes) != len(requestedVolumes) {
+		err = errors.New("the version of MAAS being used does not support Juju storage")
+		return nil, err
+	}
+
 	return &environs.StartInstanceResult{
-		Instance:    inst,
-		Hardware:    hc,
-		NetworkInfo: networkInfo,
+		Instance:          inst,
+		Hardware:          hc,
+		NetworkInfo:       networkInfo,
+		Volumes:           resultVolumes,
+		VolumeAttachments: resultAttachments,
 	}, nil
 }
 
@@ -974,7 +1021,9 @@ func (environ *maasEnviron) waitForNodeDeployment(id instance.Id) error {
 		if statusValues[systemId] == "Deployed" {
 			return nil
 		}
-		logger.Debugf("maas instance %v has status %q", id, statusValues[systemId])
+		if statusValues[systemId] == "Failed deployment" {
+			return errors.Errorf("instance %q failed to deploy", id)
+		}
 	}
 	return errors.Errorf("instance %q is started but not deployed", id)
 }
@@ -1017,6 +1066,7 @@ type selectNodeArgs struct {
 	Constraints       constraints.Value
 	IncludeNetworks   []string
 	ExcludeNetworks   []string
+	Volumes           []volumeInfo
 }
 
 func (environ *maasEnviron) selectNode(args selectNodeArgs) (*gomaasapi.MAASObject, error) {
@@ -1030,6 +1080,7 @@ func (environ *maasEnviron) selectNode(args selectNodeArgs) (*gomaasapi.MAASObje
 			args.Constraints,
 			args.IncludeNetworks,
 			args.ExcludeNetworks,
+			args.Volumes,
 		)
 
 		if err, ok := err.(gomaasapi.ServerError); ok && err.StatusCode == http.StatusConflict {
@@ -1048,45 +1099,104 @@ func (environ *maasEnviron) selectNode(args selectNodeArgs) (*gomaasapi.MAASObje
 	return &node, nil
 }
 
+const bridgeConfigTemplate = `
+# In case we already created the bridge, don't do it again.
+grep -q "iface {{.Bridge}} inet dhcp" && exit 0
+
+# Discover primary interface at run-time using the default route (if set)
+PRIMARY_IFACE=$(ip route list exact 0/0 | egrep -o 'dev [^ ]+' | cut -b5-)
+
+# If $PRIMARY_IFACE is empty, there's nothing to do.
+[ -z "$PRIMARY_IFACE" ] && exit 0
+
+# Change the config to make $PRIMARY_IFACE manual instead of DHCP,
+# then create the bridge and enslave $PRIMARY_IFACE into it.
+grep -q "iface ${PRIMARY_IFACE} inet dhcp" {{.Config}} && \
+sed -i "s/iface ${PRIMARY_IFACE} inet dhcp//" {{.Config}} && \
+cat >> {{.Config}} << EOF
+
+# Primary interface (defining the default route)
+iface ${PRIMARY_IFACE} inet manual
+
+# Bridge to use for LXC/KVM containers
+auto {{.Bridge}}
+iface {{.Bridge}} inet dhcp
+    bridge_ports ${PRIMARY_IFACE}
+EOF
+
+# Make the primary interface not auto-starting.
+grep -q "auto ${PRIMARY_IFACE}" {{.Config}} && \
+sed -i "s/auto ${PRIMARY_IFACE}//" {{.Config}}
+
+# Stop $PRIMARY_IFACE and start the bridge instead.
+ifdown -v ${PRIMARY_IFACE} ; ifup -v {{.Bridge}}
+
+# Finally, remove the route using $PRIMARY_IFACE (if any) so it won't
+# clash with the same automatically added route for juju-br0 (except
+# for the device name).
+ip route flush dev $PRIMARY_IFACE scope link proto kernel || true
+`
+
+// setupJujuNetworking returns a string representing the script to run
+// in order to prepare the Juju-specific networking config on a node.
+func setupJujuNetworking() (string, error) {
+	parsedTemplate := template.Must(
+		template.New("BridgeConfig").Parse(bridgeConfigTemplate),
+	)
+	var buf bytes.Buffer
+	err := parsedTemplate.Execute(&buf, map[string]interface{}{
+		"Config": "/etc/network/interfaces",
+		"Bridge": instancecfg.DefaultBridgeName,
+	})
+	if err != nil {
+		return "", errors.Annotate(err, "bridge config template error")
+	}
+	return buf.String(), nil
+}
+
 // newCloudinitConfig creates a cloudinit.Config structure
 // suitable as a base for initialising a MAAS node.
-func (environ *maasEnviron) newCloudinitConfig(hostname, primaryIface, series string) (*cloudinit.Config, error) {
-	info := machineInfo{hostname}
-	runCmd, err := info.cloudinitRunCmd(series)
-
+func (environ *maasEnviron) newCloudinitConfig(hostname, primaryIface, series string) (cloudinit.CloudConfig, error) {
+	cloudcfg, err := cloudinit.New(series)
 	if err != nil {
 		return nil, err
 	}
 
-	cloudcfg := cloudinit.New()
+	info := machineInfo{hostname}
+	runCmd, err := info.cloudinitRunCmd(cloudcfg)
+	if err != nil {
+		return nil, errors.Trace(err)
+	}
+
 	operatingSystem, err := version.GetOSFromSeries(series)
 	if err != nil {
-		return nil, err
+		return nil, errors.Trace(err)
 	}
-
 	switch operatingSystem {
 	case version.Windows:
-		cloudcfg.AddScripts(
-			runCmd,
-		)
+		cloudcfg.AddScripts(runCmd)
 	case version.Ubuntu:
-		cloudcfg.SetAptUpdate(true)
-		if on, set := environ.Config().DisableNetworkManagement(); on && set {
-			logger.Infof("network management disabled - setting up br0, eth0 disabled")
-			cloudcfg.AddScripts("set -xe", runCmd)
-		} else {
-			bridgeScript, err := setupJujuNetworking(primaryIface)
+		cloudcfg.SetSystemUpdate(true)
+		cloudcfg.AddScripts("set -xe", runCmd)
+		// Only create the default bridge if we're not using static
+		// address allocation for containers.
+		if !environs.AddressAllocationEnabled() {
+			// Address allocated feature flag might be disabled, but
+			// DisableNetworkManagement can still disable the bridge
+			// creation.
+			if on, set := environ.Config().DisableNetworkManagement(); on && set {
+				logger.Infof(
+					"network management disabled - not using %q bridge for containers",
+					instancecfg.DefaultBridgeName,
+				)
+				break
+			}
+			bridgeScript, err := setupJujuNetworking()
 			if err != nil {
 				return nil, errors.Trace(err)
 			}
 			cloudcfg.AddPackage("bridge-utils")
-			cloudcfg.AddScripts(
-				"set -xe",
-				runCmd,
-				"ifdown "+primaryIface,
-				bridgeScript,
-				"ifup "+environs.DefaultBridgeName,
-			)
+			cloudcfg.AddRunCmd(bridgeScript)
 		}
 	}
 	return cloudcfg, nil
@@ -1228,90 +1338,216 @@ func (environ *maasEnviron) Instances(ids []instance.Id) ([]instance.Instance, e
 
 // AllocateAddress requests an address to be allocated for the
 // given instance on the given network.
-func (environ *maasEnviron) AllocateAddress(instId instance.Id, netId network.Id, addr network.Address) error {
-	subnets, err := environ.Subnets(instId)
+func (environ *maasEnviron) AllocateAddress(instId instance.Id, subnetId network.Id, addr network.Address) (err error) {
+	if !environs.AddressAllocationEnabled() {
+		return errors.NotSupportedf("address allocation")
+	}
+
+	defer errors.DeferredAnnotatef(&err, "failed to allocate address %q for instance %q", addr, instId)
+	var subnets []network.SubnetInfo
+
+	subnets, err = environ.Subnets(instId, []network.Id{subnetId})
+	logger.Tracef("Subnets(%q, %q, %q) returned: %v (%v)", instId, subnetId, addr, subnets, err)
 	if err != nil {
 		return errors.Trace(err)
 	}
-	var foundSub *network.SubnetInfo
-	for i, sub := range subnets {
-		if sub.ProviderId == netId {
-			foundSub = &subnets[i]
-			break
-		}
-	}
-	if foundSub == nil {
-		return errors.Errorf("could not find network matching %v", netId)
+	if len(subnets) != 1 {
+		return errors.Errorf("could not find subnet matching %q", subnetId)
 	}
+	foundSub := subnets[0]
+	logger.Tracef("found subnet %#v", foundSub)
+
 	cidr := foundSub.CIDR
 	ipaddresses := environ.getMAASClient().GetSubObject("ipaddresses")
 	err = ReserveIPAddress(ipaddresses, cidr, addr)
-	if err != nil {
-		maasErr, ok := err.(gomaasapi.ServerError)
-		if !ok {
-			return errors.Trace(err)
-		}
-		// For an "out of range" IP address, maas raises
-		// StaticIPAddressOutOfRange - an error 403
-		// If there are no more addresses we get
-		// StaticIPAddressExhaustion - an error 503
-		// For an address already in use we get
-		// StaticIPAddressUnavailable - an error 404
-		if maasErr.StatusCode == 404 {
-			return environs.ErrIPAddressUnavailable
-		} else if maasErr.StatusCode == 503 {
-			return environs.ErrIPAddressesExhausted
-		}
-		// any error other than a 404 or 503 is "unexpected" and should
-		// be returned directly.
-		return errors.Trace(err)
+	if err == nil {
+		logger.Infof("allocated address %q for instance %q on subnet %q", addr, instId, cidr)
+		return nil
 	}
 
-	return nil
+	maasErr, ok := err.(gomaasapi.ServerError)
+	if !ok {
+		return errors.Trace(err)
+	}
+	// For an "out of range" IP address, maas raises
+	// StaticIPAddressOutOfRange - an error 403
+	// If there are no more addresses we get
+	// StaticIPAddressExhaustion - an error 503
+	// For an address already in use we get
+	// StaticIPAddressUnavailable - an error 404
+	if maasErr.StatusCode == 404 {
+		logger.Tracef("address %q not available for allocation", addr)
+		return environs.ErrIPAddressUnavailable
+	} else if maasErr.StatusCode == 503 {
+		logger.Tracef("no more addresses available on the subnet")
+		return environs.ErrIPAddressesExhausted
+	}
+	// any error other than a 404 or 503 is "unexpected" and should
+	// be returned directly.
+	return errors.Trace(err)
 }
 
 // ReleaseAddress releases a specific address previously allocated with
 // AllocateAddress.
-func (environ *maasEnviron) ReleaseAddress(_ instance.Id, _ network.Id, addr network.Address) error {
+func (environ *maasEnviron) ReleaseAddress(instId instance.Id, _ network.Id, addr network.Address) (err error) {
+	if !environs.AddressAllocationEnabled() {
+		return errors.NotSupportedf("address allocation")
+	}
+
+	defer errors.DeferredAnnotatef(&err, "failed to release IP address %q from instance %q", addr, instId)
 	ipaddresses := environ.getMAASClient().GetSubObject("ipaddresses")
 	// This can return a 404 error if the address has already been released
 	// or is unknown by maas. However this, like any other error, would be
 	// unexpected - so we don't treat it specially and just return it to
 	// the caller.
-	err := ReleaseIPAddress(ipaddresses, addr)
-	if err != nil {
-		return errors.Annotatef(err, "failed to release IP address %v", addr.Value)
-	}
-	return nil
+	return ReleaseIPAddress(ipaddresses, addr)
 }
 
-// Subnets returns basic information about all subnets known
-// by the provider for the environment, for a specific instance.
-func (environ *maasEnviron) Subnets(instId instance.Id) ([]network.SubnetInfo, error) {
+// NetworkInterfaces implements Environ.NetworkInterfaces.
+func (environ *maasEnviron) NetworkInterfaces(instId instance.Id) ([]network.InterfaceInfo, error) {
 	instances, err := environ.acquiredInstances([]instance.Id{instId})
 	if err != nil {
-		return nil, errors.Annotatef(err, "could not find instance %v", instId)
+		return nil, errors.Annotatef(err, "could not find instance %q", instId)
+	}
+	if len(instances) == 0 {
+		return nil, errors.NotFoundf("instance %q", instId)
+	}
+	inst := instances[0]
+	interfaces, _, err := environ.getInstanceNetworkInterfaces(inst)
+	if err != nil {
+		return nil, errors.Annotatef(err, "failed to get instance %q network interfaces", instId)
+	}
+
+	networks, err := environ.getInstanceNetworks(inst)
+	if err != nil {
+		return nil, errors.Annotatef(err, "failed to get instance %q subnets", instId)
+	}
+
+	macToNetworkMap := make(map[string]networkDetails)
+	for _, network := range networks {
+		macs, err := environ.listConnectedMacs(network)
+		if err != nil {
+			return nil, errors.Trace(err)
+		}
+		for _, mac := range macs {
+			macToNetworkMap[mac] = network
+		}
+	}
+
+	result := []network.InterfaceInfo{}
+	for serial, iface := range interfaces {
+		deviceIndex := iface.DeviceIndex
+		interfaceName := iface.InterfaceName
+		disabled := iface.Disabled
+
+		ifaceInfo := network.InterfaceInfo{
+			DeviceIndex:   deviceIndex,
+			InterfaceName: interfaceName,
+			Disabled:      disabled,
+			NoAutoStart:   disabled,
+			MACAddress:    serial,
+			ConfigType:    network.ConfigDHCP,
+		}
+		details, ok := macToNetworkMap[serial]
+		if ok {
+			ifaceInfo.VLANTag = details.VLANTag
+			ifaceInfo.ProviderSubnetId = network.Id(details.Name)
+			mask := net.IPMask(net.ParseIP(details.Mask))
+			cidr := net.IPNet{net.ParseIP(details.IP), mask}
+			ifaceInfo.CIDR = cidr.String()
+			ifaceInfo.Address = network.NewAddress(cidr.IP.String())
+		} else {
+			logger.Debugf("no subnet information for MAC address %q, instance %q", serial, instId)
+		}
+		result = append(result, ifaceInfo)
+	}
+	return result, nil
+}
+
+// listConnectedMacs calls the MAAS list_connected_macs API to fetch all the
+// the MAC addresses attached to a specific network.
+func (environ *maasEnviron) listConnectedMacs(network networkDetails) ([]string, error) {
+	client := environ.getMAASClient().GetSubObject("networks").GetSubObject(network.Name)
+	json, err := client.CallGet("list_connected_macs", nil)
+	if err != nil {
+		return nil, err
+	}
+
+	macs, err := json.GetArray()
+	if err != nil {
+		return nil, err
+	}
+	result := []string{}
+	for _, macObj := range macs {
+		macMap, err := macObj.GetMap()
+		if err != nil {
+			return nil, err
+		}
+		mac, err := macMap["mac_address"].GetString()
+		if err != nil {
+			return nil, err
+		}
+
+		result = append(result, mac)
+	}
+	return result, nil
+}
+
+// Subnets returns basic information about the specified subnets known
+// by the provider for the specified instance. subnetIds must not be
+// empty. Implements NetworkingEnviron.Subnets.
+func (environ *maasEnviron) Subnets(instId instance.Id, subnetIds []network.Id) ([]network.SubnetInfo, error) {
+	// At some point in the future an empty netIds may mean "fetch all subnets"
+	// but until that functionality is needed it's an error.
+	if len(subnetIds) == 0 {
+		return nil, errors.Errorf("subnetIds must not be empty")
+	}
+	instances, err := environ.acquiredInstances([]instance.Id{instId})
+	if err != nil {
+		return nil, errors.Annotatef(err, "could not find instance %q", instId)
 	}
 	if len(instances) == 0 {
 		return nil, errors.NotFoundf("instance %v", instId)
 	}
 	inst := instances[0]
-	networks, err := environ.getInstanceNetworks(inst)
+	// The MAAS API get networks call returns named subnets, not physical networks,
+	// so we save the data from this call into a variable called subnets.
+	// http://maas.ubuntu.com/docs/api.html#networks
+	subnets, err := environ.getInstanceNetworks(inst)
 	if err != nil {
-		return nil, errors.Annotatef(err, "getInstanceNetworks failed")
+		return nil, errors.Annotatef(err, "cannot get instance %q subnets", instId)
 	}
-	logger.Debugf("node %q has networks %v", instId, networks)
+	logger.Debugf("instance %q has subnets %v", instId, subnets)
 
 	nodegroups, err := environ.getNodegroups()
 	if err != nil {
-		return nil, errors.Annotatef(err, "getNodegroups failed")
+		return nil, errors.Annotatef(err, "cannot get instance %q node groups", instId)
 	}
 	nodegroupInterfaces := environ.getNodegroupInterfaces(nodegroups)
+
+	subnetIdSet := make(map[network.Id]bool)
+	for _, netId := range subnetIds {
+		subnetIdSet[netId] = false
+	}
+	processedIds := make(map[network.Id]bool)
+
 	var networkInfo []network.SubnetInfo
-	for _, netw := range networks {
+	for _, subnet := range subnets {
+		_, ok := subnetIdSet[network.Id(subnet.Name)]
+		if !ok {
+			// This id is not what we're looking for.
+			continue
+		}
+		if _, ok := processedIds[network.Id(subnet.Name)]; ok {
+			// Don't add the same subnet twice.
+			continue
+		}
+		// mark that we've found this subnet
+		processedIds[network.Id(subnet.Name)] = true
+		subnetIdSet[network.Id(subnet.Name)] = true
 		netCIDR := &net.IPNet{
-			IP:   net.ParseIP(netw.IP),
-			Mask: net.IPMask(net.ParseIP(netw.Mask)),
+			IP:   net.ParseIP(subnet.IP),
+			Mask: net.IPMask(net.ParseIP(subnet.Mask)),
 		}
 		var allocatableHigh, allocatableLow net.IP
 		for ip, bounds := range nodegroupInterfaces {
@@ -1322,24 +1558,36 @@ func (environ *maasEnviron) Subnets(instId instance.Id) ([]network.SubnetInfo, e
 				break
 			}
 		}
-		netInfo := network.SubnetInfo{
+		subnetInfo := network.SubnetInfo{
 			CIDR:              netCIDR.String(),
-			VLANTag:           netw.VLANTag,
-			ProviderId:        network.Id(netw.Name),
+			VLANTag:           subnet.VLANTag,
+			ProviderId:        network.Id(subnet.Name),
 			AllocatableIPLow:  allocatableLow,
 			AllocatableIPHigh: allocatableHigh,
 		}
 
 		// Verify we filled-in everything for all networks
 		// and drop incomplete records.
-		if netInfo.ProviderId == "" || netInfo.CIDR == "" {
-			logger.Warningf("ignoring network  %q: missing information (%#v)", netw.Name, netInfo)
+		if subnetInfo.ProviderId == "" || subnetInfo.CIDR == "" {
+			logger.Infof("ignoring subnet  %q: missing information (%#v)", subnet.Name, subnetInfo)
 			continue
 		}
 
-		networkInfo = append(networkInfo, netInfo)
+		logger.Tracef("found subnet with info %#v", subnetInfo)
+		networkInfo = append(networkInfo, subnetInfo)
 	}
-	logger.Debugf("available networks for instance %v: %#v", inst.Id(), networkInfo)
+	logger.Debugf("available subnets for instance %v: %#v", inst.Id(), networkInfo)
+
+	notFound := []network.Id{}
+	for subnetId, found := range subnetIdSet {
+		if !found {
+			notFound = append(notFound, subnetId)
+		}
+	}
+	if len(notFound) != 0 {
+		return nil, errors.Errorf("failed to find the following subnets: %v", notFound)
+	}
+
 	return networkInfo, nil
 }
 
@@ -1542,12 +1790,11 @@ func extractInterfaces(inst instance.Instance, lshwXML []byte) (map[string]iface
 	primaryIface := ""
 	interfaces := make(map[string]ifaceInfo)
 	var processNodes func(nodes []Node) error
+	var baseIndex int
 	processNodes = func(nodes []Node) error {
 		for _, node := range nodes {
 			if strings.HasPrefix(node.Id, "network") {
-				// If there's a single interface, the ID won't have an
-				// index suffix.
-				index := 0
+				index := baseIndex
 				if strings.HasPrefix(node.Id, "network:") {
 					// There is an index suffix, parse it.
 					var err error
@@ -1555,7 +1802,10 @@ func extractInterfaces(inst instance.Instance, lshwXML []byte) (map[string]iface
 					if err != nil {
 						return errors.Annotatef(err, "lshw output for node %q has invalid ID suffix for %q", inst.Id(), node.Id)
 					}
+				} else {
+					baseIndex++
 				}
+
 				if primaryIface == "" && !node.Disabled {
 					primaryIface = node.LogicalName
 					logger.Debugf("node %q primary network interface is %q", inst.Id(), primaryIface)
