@@ -10,14 +10,15 @@ import (
 	gc "gopkg.in/check.v1"
 	"launchpad.net/gomaasapi"
 
-	"github.com/juju/juju/cloudinit"
+	"github.com/juju/juju/cloudconfig/cloudinit"
 	"github.com/juju/juju/environs"
 	"github.com/juju/juju/environs/storage"
 )
 
 var (
-	ShortAttempt = &shortAttempt
-	APIVersion   = apiVersion
+	ShortAttempt            = &shortAttempt
+	APIVersion              = apiVersion
+	MaasStorageProviderType = maasStorageProviderType
 )
 
 func MAASAgentName(env environs.Environ) string {
@@ -28,7 +29,7 @@ func GetMAASClient(env environs.Environ) *gomaasapi.MAASObject {
 	return env.(*maasEnviron).getMAASClient()
 }
 
-func NewCloudinitConfig(env environs.Environ, hostname, iface, series string) (*cloudinit.Config, error) {
+func NewCloudinitConfig(env environs.Environ, hostname, iface, series string) (cloudinit.CloudConfig, error) {
 	return env.(*maasEnviron).newCloudinitConfig(hostname, iface, series)
 }
 
