@@ -8,10 +8,11 @@ import (
 	"path"
 	"strings"
 
+	"github.com/juju/errors"
 	"github.com/juju/utils"
 	goyaml "gopkg.in/yaml.v1"
 
-	"github.com/juju/juju/cloudinit"
+	"github.com/juju/juju/cloudconfig/cloudinit"
 	"github.com/juju/juju/environs/config"
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/juju/paths"
@@ -53,24 +54,21 @@ var _MAASInstanceFilename = path.Join(maasDataDir, "MAASmachine.txt")
 // cloudinitRunCmd returns the shell command that, when run, will create the
 // "machine info" file containing the hostname of a machine.
 // That command is destined to be used by cloudinit.
-func (info *machineInfo) cloudinitRunCmd(series string) (string, error) {
-	dataDir, err := paths.DataDir(series)
+func (info *machineInfo) cloudinitRunCmd(cloudcfg cloudinit.CloudConfig) (string, error) {
+	dataDir, err := paths.DataDir(cloudcfg.GetSeries())
 	if err != nil {
-		return "", err
+		return "", errors.Trace(err)
 	}
-	renderer, err := cloudinit.NewRenderer(series)
-	if err != nil {
-		return "", err
-	}
-
 	yaml, err := goyaml.Marshal(info)
 	if err != nil {
-		return "", err
+		return "", errors.Trace(err)
 	}
-	fileName := renderer.PathJoin(renderer.FromSlash(dataDir), "MAASmachine.txt")
-	script := renderer.Mkdir(dataDir)
-	contents := utils.ShQuote(string(yaml))
-	script = append(script, renderer.WriteFile(fileName, contents, 0755)...)
+	renderer := cloudcfg.ShellRenderer()
+	fileName := renderer.Join(renderer.FromSlash(dataDir), "MAASmachine.txt")
+	script := renderer.MkdirAll(dataDir)
+	contents := renderer.Quote(string(yaml))
+	script = append(script, renderer.WriteFile(fileName, []byte(contents))...)
+	script = append(script, renderer.Chmod(fileName, 0755)...)
 	return strings.Join(script, "\n"), nil
 }
 
