@@ -16,9 +16,9 @@ import (
 	"github.com/juju/utils"
 
 	"github.com/juju/juju/agent"
+	"github.com/juju/juju/cloudconfig/instancecfg"
 	"github.com/juju/juju/constraints"
 	"github.com/juju/juju/environs"
-	"github.com/juju/juju/environs/cloudinit"
 	"github.com/juju/juju/environs/config"
 	"github.com/juju/juju/environs/httpstorage"
 	"github.com/juju/juju/environs/manual"
@@ -68,6 +68,11 @@ type manualEnviron struct {
 var errNoStartInstance = errors.New("manual provider cannot start instances")
 var errNoStopInstance = errors.New("manual provider cannot stop instances")
 
+// MaintainInstance is specified in the InstanceBroker interface.
+func (*manualEnviron) MaintainInstance(args environs.StartInstanceParams) error {
+	return nil
+}
+
 func (*manualEnviron) StartInstance(args environs.StartInstanceParams) (*environs.StartInstanceResult, error) {
 	return nil, errNoStartInstance
 }
@@ -96,16 +101,6 @@ func (e *manualEnviron) SupportedArchitectures() ([]string, error) {
 	return arch.AllSupportedArches, nil
 }
 
-// SupportNetworks is specified on the EnvironCapability interface.
-func (e *manualEnviron) SupportNetworks() bool {
-	return false
-}
-
-// SupportAddressAllocation is specified on the EnvironCapability interface.
-func (e *manualEnviron) SupportAddressAllocation(netId network.Id) (bool, error) {
-	return false, nil
-}
-
 func (e *manualEnviron) Bootstrap(ctx environs.BootstrapContext, args environs.BootstrapParams) (arch, series string, _ environs.BootstrapFinalizer, _ error) {
 	// Set "use-sshstorage" to false, so agents know not to use sshstorage.
 	cfg, err := e.Config().Apply(map[string]interface{}{"use-sshstorage": false})
@@ -133,16 +128,16 @@ func (e *manualEnviron) Bootstrap(ctx environs.BootstrapContext, args environs.B
 	if err != nil {
 		return "", "", nil, err
 	}
-	finalize := func(ctx environs.BootstrapContext, mcfg *cloudinit.MachineConfig) error {
-		mcfg.InstanceId = BootstrapInstanceId
-		mcfg.HardwareCharacteristics = &hc
-		if err := environs.FinishMachineConfig(mcfg, e.Config()); err != nil {
+	finalize := func(ctx environs.BootstrapContext, icfg *instancecfg.InstanceConfig) error {
+		icfg.InstanceId = BootstrapInstanceId
+		icfg.HardwareCharacteristics = &hc
+		if err := instancecfg.FinishInstanceConfig(icfg, e.Config()); err != nil {
 			return err
 		}
 		for k, v := range agentEnv {
-			mcfg.AgentEnvironment[k] = v
+			icfg.AgentEnvironment[k] = v
 		}
-		return common.ConfigureMachine(ctx, ssh.DefaultClient, host, mcfg)
+		return common.ConfigureMachine(ctx, ssh.DefaultClient, host, icfg)
 	}
 	return *hc.Arch, series, finalize, nil
 }
@@ -254,27 +249,6 @@ func (e *manualEnviron) Instances(ids []instance.Id) (instances []instance.Insta
 	return instances, err
 }
 
-// AllocateAddress requests an address to be allocated for the
-// given instance on the given network. This is not supported on the
-// manual provider.
-func (*manualEnviron) AllocateAddress(_ instance.Id, _ network.Id, _ network.Address) error {
-	return errors.NotSupportedf("AllocateAddress")
-}
-
-// ReleaseAddress releases a specific address previously allocated with
-// AllocateAddress.
-func (*manualEnviron) ReleaseAddress(_ instance.Id, _ network.Id, _ network.Address) error {
-	return errors.NotSupportedf("ReleaseAddress")
-}
-
-// Subnets returns basic information about all subnets known
-// by the provider for the environment. They may be unknown to juju
-// yet (i.e. when called initially or when a new network was created).
-// This is not implemented by the manual provider yet.
-func (*manualEnviron) Subnets(_ instance.Id) ([]network.SubnetInfo, error) {
-	return nil, errors.NotSupportedf("Subnets")
-}
-
 var newSSHStorage = func(sshHost, storageDir, storageTmpdir string) (storage.Storage, error) {
 	logger.Debugf("using ssh storage at host %q dir %q", sshHost, storageDir)
 	return sshstorage.NewSSHStorage(sshstorage.NewSSHStorageParams{
