@@ -32,13 +32,13 @@ func (s *providerSuite) SetUpTest(c *gc.C) {
 	})
 }
 
-func (s *providerSuite) TestPrepare(c *gc.C) {
+func (s *providerSuite) TestPrepareForBootstrap(c *gc.C) {
 	minimal := manual.MinimalConfigValues()
 	minimal["use-sshstorage"] = true
 	delete(minimal, "storage-auth-key")
 	testConfig, err := config.New(config.UseDefaults, minimal)
 	c.Assert(err, jc.ErrorIsNil)
-	env, err := manual.ProviderInstance.Prepare(envtesting.BootstrapContext(c), testConfig)
+	env, err := manual.ProviderInstance.PrepareForBootstrap(envtesting.BootstrapContext(c), testConfig)
 	c.Assert(err, jc.ErrorIsNil)
 	cfg := env.Config()
 	key, _ := cfg.UnknownAttrs()["storage-auth-key"].(string)
@@ -50,7 +50,7 @@ func (s *providerSuite) TestPrepareUseSSHStorage(c *gc.C) {
 	minimal["use-sshstorage"] = false
 	testConfig, err := config.New(config.UseDefaults, minimal)
 	c.Assert(err, jc.ErrorIsNil)
-	_, err = manual.ProviderInstance.Prepare(envtesting.BootstrapContext(c), testConfig)
+	_, err = manual.ProviderInstance.PrepareForBootstrap(envtesting.BootstrapContext(c), testConfig)
 	c.Assert(err, gc.ErrorMatches, "use-sshstorage must not be specified")
 
 	s.PatchValue(manual.NewSSHStorage, func(sshHost, storageDir, storageTmpdir string) (storage.Storage, error) {
@@ -59,7 +59,7 @@ func (s *providerSuite) TestPrepareUseSSHStorage(c *gc.C) {
 	minimal["use-sshstorage"] = true
 	testConfig, err = config.New(config.UseDefaults, minimal)
 	c.Assert(err, jc.ErrorIsNil)
-	_, err = manual.ProviderInstance.Prepare(envtesting.BootstrapContext(c), testConfig)
+	_, err = manual.ProviderInstance.PrepareForBootstrap(envtesting.BootstrapContext(c), testConfig)
 	c.Assert(err, gc.ErrorMatches, "initialising SSH storage failed: newSSHStorage failed")
 }
 
@@ -69,7 +69,7 @@ func (s *providerSuite) TestPrepareSetsUseSSHStorage(c *gc.C) {
 	testConfig, err := config.New(config.UseDefaults, attrs)
 	c.Assert(err, jc.ErrorIsNil)
 
-	env, err := manual.ProviderInstance.Prepare(envtesting.BootstrapContext(c), testConfig)
+	env, err := manual.ProviderInstance.PrepareForBootstrap(envtesting.BootstrapContext(c), testConfig)
 	c.Assert(err, jc.ErrorIsNil)
 	cfg := env.Config()
 	value := cfg.AllAttrs()["use-sshstorage"]
