@@ -9,19 +9,20 @@ import (
 	"strings"
 	"text/template"
 
-	"launchpad.net/goose/errors"
-	"launchpad.net/goose/identity"
-	"launchpad.net/goose/nova"
-	"launchpad.net/goose/swift"
+	"gopkg.in/goose.v1/errors"
+	"gopkg.in/goose.v1/identity"
+	"gopkg.in/goose.v1/nova"
+	"gopkg.in/goose.v1/swift"
 
 	"github.com/juju/juju/constraints"
 	"github.com/juju/juju/environs"
 	"github.com/juju/juju/environs/instances"
 	"github.com/juju/juju/environs/jujutest"
 	"github.com/juju/juju/environs/simplestreams"
-	"github.com/juju/juju/environs/storage"
+	envstorage "github.com/juju/juju/environs/storage"
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/network"
+	"github.com/juju/juju/storage"
 )
 
 // This provides the content for code accessing test:///... URLs. This allows
@@ -36,16 +37,16 @@ func init() {
 var (
 	ShortAttempt   = &shortAttempt
 	StorageAttempt = &storageAttempt
+	CinderAttempt  = &cinderAttempt
 )
 
 // MetadataStorage returns a Storage instance which is used to store simplestreams metadata for tests.
-func MetadataStorage(e environs.Environ) storage.Storage {
+func MetadataStorage(e environs.Environ) envstorage.Storage {
 	ecfg := e.(*environ).ecfg()
-	authModeCfg := AuthMode(ecfg.authMode())
 	container := "juju-dist-test"
 	metadataStorage := &openstackstorage{
 		containerName: container,
-		swift:         swift.New(e.(*environ).authClient(ecfg, authModeCfg)),
+		swift:         swift.New(authClient(ecfg)),
 	}
 
 	// Ensure the container exists.
@@ -73,6 +74,13 @@ var (
 	AvailabilityZoneAllocations = &availabilityZoneAllocations
 )
 
+type OpenstackStorage openstackStorage
+
+func NewCinderVolumeSource(s OpenstackStorage) storage.VolumeSource {
+	const envName = "testenv"
+	return &cinderVolumeSource{openstackStorage(s), envName}
+}
+
 var indexData = `
 		{
 		 "index": {
@@ -238,7 +246,7 @@ var imagesData = `
 
 const productMetadatafile = "image-metadata/products.json"
 
-func UseTestImageData(stor storage.Storage, cred *identity.Credentials) {
+func UseTestImageData(stor envstorage.Storage, cred *identity.Credentials) {
 	// Put some image metadata files into the public storage.
 	t := template.Must(template.New("").Parse(indexData))
 	var metadata bytes.Buffer
@@ -251,7 +259,7 @@ func UseTestImageData(stor storage.Storage, cred *identity.Credentials) {
 		productMetadatafile, strings.NewReader(imagesData), int64(len(imagesData)))
 }
 
-func RemoveTestImageData(stor storage.Storage) {
+func RemoveTestImageData(stor envstorage.Storage) {
 	stor.Remove(simplestreams.UnsignedIndex("v1", 1))
 	stor.Remove(productMetadatafile)
 }
@@ -308,13 +316,9 @@ func EnsureGroup(e environs.Environ, name string, rules []nova.RuleInfo) (nova.S
 	return e.(*environ).ensureGroup(name, rules)
 }
 
-func CollectInstances(e environs.Environ, ids []instance.Id, out map[string]instance.Instance) []instance.Id {
-	return e.(*environ).collectInstances(ids, out)
-}
-
 // ImageMetadataStorage returns a Storage object pointing where the goose
 // infrastructure sets up its keystone entry for image metadata
-func ImageMetadataStorage(e environs.Environ) storage.Storage {
+func ImageMetadataStorage(e environs.Environ) envstorage.Storage {
 	env := e.(*environ)
 	return &openstackstorage{
 		containerName: "imagemetadata",
@@ -324,7 +328,7 @@ func ImageMetadataStorage(e environs.Environ) storage.Storage {
 
 // CreateCustomStorage creates a swift container and returns the Storage object
 // so you can put data into it.
-func CreateCustomStorage(e environs.Environ, containerName string) storage.Storage {
+func CreateCustomStorage(e environs.Environ, containerName string) envstorage.Storage {
 	env := e.(*environ)
 	swiftClient := swift.New(env.client)
 	if err := swiftClient.CreateContainer(containerName, swift.PublicRead); err != nil {
@@ -337,7 +341,7 @@ func CreateCustomStorage(e environs.Environ, containerName string) storage.Stora
 }
 
 // BlankContainerStorage creates a Storage object with blank container name.
-func BlankContainerStorage() storage.Storage {
+func BlankContainerStorage() envstorage.Storage {
 	return &openstackstorage{}
 }
 
@@ -354,3 +358,4 @@ var PortsToRuleInfo = portsToRuleInfo
 var RuleMatchesPortRange = ruleMatchesPortRange
 
 var MakeServiceURL = &makeServiceURL
+var ProviderInstance = providerInstance
