@@ -14,17 +14,18 @@ import (
 	"sync"
 	"time"
 
-	jujuerrors "github.com/juju/errors"
+	"github.com/juju/errors"
 	"github.com/juju/loggo"
 	"github.com/juju/names"
 	"github.com/juju/utils"
-	"launchpad.net/goose/client"
-	gooseerrors "launchpad.net/goose/errors"
-	"launchpad.net/goose/identity"
-	"launchpad.net/goose/nova"
-	"launchpad.net/goose/swift"
+	"gopkg.in/goose.v1/client"
+	gooseerrors "gopkg.in/goose.v1/errors"
+	"gopkg.in/goose.v1/identity"
+	"gopkg.in/goose.v1/nova"
+	"gopkg.in/goose.v1/swift"
 
-	"github.com/juju/errors"
+	"github.com/juju/juju/cloudconfig/instancecfg"
+	"github.com/juju/juju/cloudconfig/providerinit"
 	"github.com/juju/juju/constraints"
 	"github.com/juju/juju/environs"
 	"github.com/juju/juju/environs/config"
@@ -32,7 +33,6 @@ import (
 	"github.com/juju/juju/environs/instances"
 	"github.com/juju/juju/environs/simplestreams"
 	"github.com/juju/juju/environs/storage"
-	envtools "github.com/juju/juju/environs/tools"
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/juju/arch"
 	"github.com/juju/juju/network"
@@ -62,12 +62,6 @@ var shortAttempt = utils.AttemptStrategy{
 	Delay: 200 * time.Millisecond,
 }
 
-func init() {
-	environs.RegisterProvider("openstack", environProvider{})
-	environs.RegisterImageDataSourceFunc("keystone catalog", getKeystoneImageSource)
-	envtools.RegisterToolsDataSourceFunc("keystone catalog", getKeystoneToolsSource)
-}
-
 func (p environProvider) BoilerplateConfig() string {
 	return `
 # https://juju.ubuntu.com/docs/config-openstack.html
@@ -256,16 +250,26 @@ func (p environProvider) Open(cfg *config.Config) (environs.Environ, error) {
 	return e, nil
 }
 
-func (p environProvider) Prepare(ctx environs.BootstrapContext, cfg *config.Config) (environs.Environ, error) {
+// RestrictedConfigAttributes is specified in the EnvironProvider interface.
+func (p environProvider) RestrictedConfigAttributes() []string {
+	return []string{"region", "auth-url", "auth-mode"}
+}
+
+// PrepareForCreateEnvironment is specified in the EnvironProvider interface.
+func (p environProvider) PrepareForCreateEnvironment(cfg *config.Config) (*config.Config, error) {
 	attrs := cfg.UnknownAttrs()
 	if _, ok := attrs["control-bucket"]; !ok {
 		uuid, err := utils.NewUUID()
 		if err != nil {
-			return nil, err
+			return nil, errors.Trace(err)
 		}
 		attrs["control-bucket"] = fmt.Sprintf("%x", uuid.Raw())
 	}
-	cfg, err := cfg.Apply(attrs)
+	return cfg.Apply(attrs)
+}
+
+func (p environProvider) PrepareForBootstrap(ctx environs.BootstrapContext, cfg *config.Config) (environs.Environ, error) {
+	cfg, err := p.PrepareForCreateEnvironment(cfg)
 	if err != nil {
 		return nil, err
 	}
@@ -460,7 +464,7 @@ func (inst *openstackInstance) Addresses() ([]network.Address, error) {
 func convertNovaAddresses(publicIP string, addresses map[string][]nova.IPAddress) []network.Address {
 	var machineAddresses []network.Address
 	if publicIP != "" {
-		publicAddr := network.NewAddress(publicIP, network.ScopePublic)
+		publicAddr := network.NewScopedAddress(publicIP, network.ScopePublic)
 		publicAddr.NetworkName = "public"
 		machineAddresses = append(machineAddresses, publicAddr)
 	}
@@ -482,7 +486,7 @@ func convertNovaAddresses(publicIP string, addresses map[string][]nova.IPAddress
 			if address.Version == 6 {
 				addrtype = network.IPv6Address
 			}
-			machineAddr := network.NewAddress(address.Address, networkScope)
+			machineAddr := network.NewScopedAddress(address.Address, networkScope)
 			machineAddr.NetworkName = netName
 			if machineAddr.Type != addrtype {
 				logger.Warningf("derived address type %v, nova reports %v", machineAddr.Type, addrtype)
@@ -568,18 +572,6 @@ func (e *environ) SupportedArchitectures() ([]string, error) {
 	return e.supportedArchitectures, err
 }
 
-// SupportNetworks is specified on the EnvironCapability interface.
-func (e *environ) SupportNetworks() bool {
-	// TODO(dimitern) Once we have support for networking, inquire
-	// about capabilities and return true if supported.
-	return false
-}
-
-// SupportAddressAllocation is specified on the EnvironCapability interface.
-func (e *environ) SupportAddressAllocation(netId network.Id) (bool, error) {
-	return false, nil
-}
-
 var unsupportedConstraints = []string{
 	constraints.Tags,
 	constraints.CpuPower,
@@ -631,7 +623,7 @@ func (e *environ) AvailabilityZones() ([]common.AvailabilityZone, error) {
 	if e.availabilityZones == nil {
 		zones, err := novaListAvailabilityZones(e.nova())
 		if gooseerrors.IsNotImplemented(err) {
-			return nil, jujuerrors.NotImplementedf("availability zones")
+			return nil, errors.NotImplementedf("availability zones")
 		}
 		if err != nil {
 			return nil, err
@@ -738,7 +730,7 @@ func (e *environ) Config() *config.Config {
 	return e.ecfg().Config
 }
 
-func (e *environ) authClient(ecfg *environConfig, authModeCfg AuthMode) client.AuthenticatingClient {
+func authClient(ecfg *environConfig) client.AuthenticatingClient {
 	cred := &identity.Credentials{
 		User:       ecfg.username(),
 		Secrets:    ecfg.password(),
@@ -748,7 +740,7 @@ func (e *environ) authClient(ecfg *environConfig, authModeCfg AuthMode) client.A
 	}
 	// authModeCfg has already been validated so we know it's one of the values below.
 	var authMode identity.AuthMode
-	switch authModeCfg {
+	switch AuthMode(ecfg.authMode()) {
 	case AuthLegacy:
 		authMode = identity.AuthLegacy
 	case AuthUserPass:
@@ -772,7 +764,7 @@ var authenticateClient = func(e *environ) error {
 		// but provide a readable and helpful error message
 		// to the user.
 		logger.Debugf("authentication failed: %v", err)
-		return jujuerrors.New(`authentication failed.
+		return errors.New(`authentication failed.
 
 Please ensure the credentials are correct. A common mistake is
 to specify the wrong tenant. Use the OpenStack "project" name
@@ -788,13 +780,11 @@ func (e *environ) SetConfig(cfg *config.Config) error {
 	}
 	// At this point, the authentication method config value has been validated so we extract it's value here
 	// to avoid having to validate again each time when creating the OpenStack client.
-	var authModeCfg AuthMode
 	e.ecfgMutex.Lock()
 	defer e.ecfgMutex.Unlock()
-	authModeCfg = AuthMode(ecfg.authMode())
 	e.ecfgUnlocked = ecfg
 
-	e.client = e.authClient(ecfg, authModeCfg)
+	e.client = authClient(ecfg)
 
 	e.novaUnlocked = nova.New(e.client)
 
@@ -816,7 +806,7 @@ func (e *environ) SetConfig(cfg *config.Config) error {
 func getKeystoneImageSource(env environs.Environ) (simplestreams.DataSource, error) {
 	e, ok := env.(*environ)
 	if !ok {
-		return nil, jujuerrors.NotSupportedf("non-openstack environment")
+		return nil, errors.NotSupportedf("non-openstack environment")
 	}
 	return e.getKeystoneDataSource(&e.keystoneImageDataSourceMutex, &e.keystoneImageDataSource, "product-streams")
 }
@@ -826,7 +816,7 @@ func getKeystoneImageSource(env environs.Environ) (simplestreams.DataSource, err
 func getKeystoneToolsSource(env environs.Environ) (simplestreams.DataSource, error) {
 	e, ok := env.(*environ)
 	if !ok {
-		return nil, jujuerrors.NotSupportedf("non-openstack environment")
+		return nil, errors.NotSupportedf("non-openstack environment")
 	}
 	return e.getKeystoneDataSource(&e.keystoneToolsDataSourceMutex, &e.keystoneToolsDataSource, "juju-tools")
 }
@@ -845,7 +835,7 @@ func (e *environ) getKeystoneDataSource(mu *sync.Mutex, datasource *simplestream
 
 	url, err := makeServiceURL(e.client, keystoneName, nil)
 	if err != nil {
-		return nil, jujuerrors.NewNotSupported(err, fmt.Sprintf("cannot make service URL: %v", err))
+		return nil, errors.NewNotSupported(err, fmt.Sprintf("cannot make service URL: %v", err))
 	}
 	verify := utils.VerifySSLHostnames
 	if !e.Config().SSLHostnameVerification() {
@@ -945,6 +935,11 @@ func (e *environ) DistributeInstances(candidates, distributionGroup []instance.I
 
 var availabilityZoneAllocations = common.AvailabilityZoneAllocations
 
+// MaintainInstance is specified in the InstanceBroker interface.
+func (*environ) MaintainInstance(args environs.StartInstanceParams) error {
+	return nil
+}
+
 // StartInstance is specified in the InstanceBroker interface.
 func (e *environ) StartInstance(args environs.StartInstanceParams) (*environs.StartInstanceResult, error) {
 	var availabilityZones []string
@@ -972,7 +967,7 @@ func (e *environ) StartInstance(args environs.StartInstanceParams) (*environs.St
 			}
 		}
 		zoneInstances, err := availabilityZoneAllocations(e, group)
-		if jujuerrors.IsNotImplemented(err) {
+		if errors.IsNotImplemented(err) {
 			// Availability zones are an extension, so we may get a
 			// not implemented error; ignore these.
 		} else if err != nil {
@@ -988,7 +983,7 @@ func (e *environ) StartInstance(args environs.StartInstanceParams) (*environs.St
 		}
 	}
 
-	if args.MachineConfig.HasNetworks() {
+	if args.InstanceConfig.HasNetworks() {
 		return nil, fmt.Errorf("starting instances with networks is not supported yet.")
 	}
 
@@ -1008,16 +1003,17 @@ func (e *environ) StartInstance(args environs.StartInstanceParams) (*environs.St
 		return nil, fmt.Errorf("chosen architecture %v not present in %v", spec.Image.Arch, arches)
 	}
 
-	args.MachineConfig.Tools = tools[0]
+	args.InstanceConfig.Tools = tools[0]
 
-	if err := environs.FinishMachineConfig(args.MachineConfig, e.Config()); err != nil {
+	if err := instancecfg.FinishInstanceConfig(args.InstanceConfig, e.Config()); err != nil {
 		return nil, err
 	}
-	userData, err := environs.ComposeUserData(args.MachineConfig, nil)
+	userData, err := providerinit.ComposeUserData(args.InstanceConfig, nil)
 	if err != nil {
 		return nil, fmt.Errorf("cannot make user data: %v", err)
 	}
 	logger.Debugf("openstack user data; %d bytes", len(userData))
+
 	var networks = []nova.ServerNetworks{}
 	usingNetwork := e.ecfg().network()
 	if usingNetwork != "" {
@@ -1039,8 +1035,9 @@ func (e *environ) StartInstance(args environs.StartInstanceParams) (*environs.St
 			logger.Infof("allocated public IP %s", publicIP.IP)
 		}
 	}
+
 	cfg := e.Config()
-	groups, err := e.setUpGroups(args.MachineConfig.MachineId, cfg.APIPort())
+	groups, err := e.setUpGroups(args.InstanceConfig.MachineId, cfg.APIPort())
 	if err != nil {
 		return nil, fmt.Errorf("cannot set up groups: %v", err)
 	}
@@ -1048,16 +1045,23 @@ func (e *environ) StartInstance(args environs.StartInstanceParams) (*environs.St
 	for i, g := range groups {
 		groupNames[i] = nova.SecurityGroupName{g.Name}
 	}
+
+	machineName := resourceName(
+		names.NewMachineTag(args.InstanceConfig.MachineId),
+		e.Config().Name(),
+	)
+
 	var server *nova.Entity
 	for _, availZone := range availabilityZones {
 		var opts = nova.RunServerOpts{
-			Name:               e.machineFullName(args.MachineConfig.MachineId),
+			Name:               machineName,
 			FlavorId:           spec.InstanceType.Id,
 			ImageId:            spec.Image.Id,
 			UserData:           userData,
 			SecurityGroupNames: groupNames,
 			Networks:           networks,
 			AvailabilityZone:   availZone,
+			Metadata:           args.InstanceConfig.Tags,
 		}
 		for a := shortAttempt.Start(); a.Next(); {
 			server, err = e.nova().RunServer(opts)
@@ -1096,7 +1100,7 @@ func (e *environ) StartInstance(args environs.StartInstanceParams) (*environs.St
 		inst.floatingIP = publicIP
 		logger.Infof("assigned public IP %s to %q", publicIP.IP, inst.Id())
 	}
-	if multiwatcher.AnyJobNeedsState(args.MachineConfig.Jobs...) {
+	if multiwatcher.AnyJobNeedsState(args.InstanceConfig.Jobs...) {
 		if err := common.AddStateInstance(e.Storage(), inst.Id()); err != nil {
 			logger.Errorf("could not record instance in provider-state: %v", err)
 		}
@@ -1144,43 +1148,48 @@ func (e *environ) StopInstances(ids ...instance.Id) error {
 	return common.RemoveStateInstances(e.Storage(), ids...)
 }
 
-// collectInstances tries to get information on each instance id in ids.
-// It fills the slots in the given map for known servers with status
-// either ACTIVE or BUILD. Returns a list of missing ids.
-func (e *environ) collectInstances(ids []instance.Id, out map[string]instance.Instance) []instance.Id {
-	var err error
-	serversById := make(map[string]nova.ServerDetail)
+func (e *environ) isAliveServer(server nova.ServerDetail) bool {
+	switch server.Status {
+	// HPCloud uses "BUILD(spawning)" as an intermediate BUILD state
+	// once networking is available.
+	case nova.StatusActive, nova.StatusBuild, nova.StatusBuildSpawning, nova.StatusShutoff, nova.StatusSuspended:
+		return true
+	}
+	return false
+}
+
+func (e *environ) listServers(ids []instance.Id) ([]nova.ServerDetail, error) {
+	wantedServers := make([]nova.ServerDetail, 0, len(ids))
 	if len(ids) == 1 {
-		// most common case - single instance
-		var server *nova.ServerDetail
-		server, err = e.nova().GetServer(string(ids[0]))
-		if server != nil {
-			serversById[server.Id] = *server
+		// Common case, single instance, may return NotFound
+		var maybeServer *nova.ServerDetail
+		maybeServer, err := e.nova().GetServer(string(ids[0]))
+		if err != nil {
+			return nil, err
 		}
-	} else {
-		var servers []nova.ServerDetail
-		servers, err = e.nova().ListServersDetail(e.machinesFilter())
-		for _, server := range servers {
-			serversById[server.Id] = server
+		// Only return server details if it is currently alive
+		if maybeServer != nil && e.isAliveServer(*maybeServer) {
+			wantedServers = append(wantedServers, *maybeServer)
 		}
+		return wantedServers, nil
 	}
+	// List all servers that may be in the environment
+	servers, err := e.nova().ListServersDetail(e.machinesFilter())
 	if err != nil {
-		return ids
+		return nil, err
 	}
-	var missing []instance.Id
+	// Create a set of the ids of servers that are wanted
+	idSet := make(map[string]struct{}, len(ids))
 	for _, id := range ids {
-		if server, found := serversById[string(id)]; found {
-			// HPCloud uses "BUILD(spawning)" as an intermediate BUILD states once networking is available.
-			switch server.Status {
-			case nova.StatusActive, nova.StatusBuild, nova.StatusBuildSpawning, nova.StatusShutoff, nova.StatusSuspended:
-				// TODO(wallyworld): lookup the flavor details to fill in the instance type data
-				out[string(id)] = &openstackInstance{e: e, serverDetail: &server}
-				continue
-			}
-		}
-		missing = append(missing, id)
+		idSet[string(id)] = struct{}{}
 	}
-	return missing
+	// Return only servers with the wanted ids that are currently alive
+	for _, server := range servers {
+		if _, ok := idSet[server.Id]; ok && e.isAliveServer(server) {
+			wantedServers = append(wantedServers, server)
+		}
+	}
+	return wantedServers, nil
 }
 
 // updateFloatingIPAddresses updates the instances with any floating IP address
@@ -1206,23 +1215,41 @@ func (e *environ) Instances(ids []instance.Id) ([]instance.Instance, error) {
 	if len(ids) == 0 {
 		return nil, nil
 	}
-	missing := ids
-	found := make(map[string]instance.Instance)
 	// Make a series of requests to cope with eventual consistency.
 	// Each request will attempt to add more instances to the requested
 	// set.
+	var foundServers []nova.ServerDetail
 	for a := shortAttempt.Start(); a.Next(); {
-		if missing = e.collectInstances(missing, found); len(missing) == 0 {
+		var err error
+		foundServers, err = e.listServers(ids)
+		if err != nil {
+			logger.Debugf("error listing servers: %v", err)
+			if !gooseerrors.IsNotFound(err) {
+				return nil, err
+			}
+		}
+		if len(foundServers) == len(ids) {
 			break
 		}
 	}
-	if len(found) == 0 {
+	logger.Tracef("%d/%d live servers found", len(foundServers), len(ids))
+	if len(foundServers) == 0 {
 		return nil, environs.ErrNoInstances
 	}
 
+	instsById := make(map[string]instance.Instance, len(foundServers))
+	for i, server := range foundServers {
+		// TODO(wallyworld): lookup the flavor details to fill in the
+		// instance type data
+		instsById[server.Id] = &openstackInstance{
+			e:            e,
+			serverDetail: &foundServers[i],
+		}
+	}
+
 	// Update the instance structs with any floating IP address that has been assigned to the instance.
 	if e.ecfg().useFloatingIP() {
-		if err := e.updateFloatingIPAddresses(found); err != nil {
+		if err := e.updateFloatingIPAddresses(instsById); err != nil {
 			return nil, err
 		}
 	}
@@ -1230,7 +1257,7 @@ func (e *environ) Instances(ids []instance.Id) ([]instance.Instance, error) {
 	insts := make([]instance.Instance, len(ids))
 	var err error
 	for i, id := range ids {
-		if inst := found[string(id)]; inst != nil {
+		if inst := instsById[string(id)]; inst != nil {
 			insts[i] = inst
 		} else {
 			err = environs.ErrPartialInstances
@@ -1239,27 +1266,6 @@ func (e *environ) Instances(ids []instance.Id) ([]instance.Instance, error) {
 	return insts, err
 }
 
-// AllocateAddress requests an address to be allocated for the
-// given instance on the given network. This is not implemented on the
-// OpenStack provider yet.
-func (*environ) AllocateAddress(_ instance.Id, _ network.Id, _ network.Address) error {
-	return jujuerrors.NotImplementedf("AllocateAddress")
-}
-
-// ReleaseAddress releases a specific address previously allocated with
-// AllocateAddress.
-func (*environ) ReleaseAddress(_ instance.Id, _ network.Id, _ network.Address) error {
-	return errors.NotImplementedf("ReleaseAddress")
-}
-
-// Subnets returns basic information about all subnets known
-// by the provider for the environment. They may be unknown to juju
-// yet (i.e. when called initially or when a new network was created).
-// This is not implemented by the OpenStack provider yet.
-func (*environ) Subnets(_ instance.Id) ([]network.SubnetInfo, error) {
-	return nil, jujuerrors.NotImplementedf("Subnets")
-}
-
 func (e *environ) AllInstances() (insts []instance.Instance, err error) {
 	servers, err := e.nova().ListServersDetail(e.machinesFilter())
 	if err != nil {
@@ -1267,7 +1273,7 @@ func (e *environ) AllInstances() (insts []instance.Instance, err error) {
 	}
 	instsById := make(map[string]instance.Instance)
 	for _, server := range servers {
-		if server.Status == nova.StatusActive || server.Status == nova.StatusBuild {
+		if e.isAliveServer(server) {
 			var s = server
 			// TODO(wallyworld): lookup the flavor details to fill in the instance type data
 			instsById[s.Id] = &openstackInstance{e: e, serverDetail: &s}
@@ -1289,19 +1295,19 @@ func (e *environ) AllInstances() (insts []instance.Instance, err error) {
 func (e *environ) Destroy() error {
 	err := common.Destroy(e)
 	if err != nil {
-		return jujuerrors.Trace(err)
+		return errors.Trace(err)
 	}
 	if err := e.Storage().RemoveAll(); err != nil {
-		return jujuerrors.Trace(err)
+		return errors.Trace(err)
 	}
 	novaClient := e.nova()
 	securityGroups, err := novaClient.ListSecurityGroups()
 	if err != nil {
-		return jujuerrors.Annotate(err, "cannot list security groups")
+		return errors.Annotate(err, "cannot list security groups")
 	}
 	re, err := regexp.Compile(fmt.Sprintf("^%s(-\\d+)?$", e.jujuGroupName()))
 	if err != nil {
-		return jujuerrors.Trace(err)
+		return errors.Trace(err)
 	}
 	globalGroupName := e.globalGroupName()
 	for _, group := range securityGroups {
@@ -1327,8 +1333,8 @@ func (e *environ) jujuGroupName() string {
 	return fmt.Sprintf("juju-%s", e.name)
 }
 
-func (e *environ) machineFullName(machineId string) string {
-	return fmt.Sprintf("juju-%s-%s", e.Config().Name(), names.NewMachineTag(machineId))
+func resourceName(tag names.Tag, envName string) string {
+	return fmt.Sprintf("juju-%s-%s", envName, tag)
 }
 
 // machinesFilter returns a nova.Filter matching all machines in the environment.
