@@ -6,9 +6,11 @@ package state
 import (
 	"fmt"
 	"strconv"
+	"time"
 
 	"github.com/juju/errors"
 	"github.com/juju/names"
+	"github.com/juju/replicaset"
 	jujutxn "github.com/juju/txn"
 	"gopkg.in/mgo.v2/bson"
 	"gopkg.in/mgo.v2/txn"
@@ -16,7 +18,6 @@ import (
 	"github.com/juju/juju/constraints"
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/network"
-	"github.com/juju/juju/replicaset"
 )
 
 // MachineTemplate holds attributes that are to be associated
@@ -58,6 +59,22 @@ type MachineTemplate struct {
 	// should be part of.
 	RequestedNetworks []string
 
+	// Volumes holds the parameters for volumes that are to be created
+	// and attached to the machine.
+	Volumes []MachineVolumeParams
+
+	// VolumeAttachments holds the parameters for attaching existing
+	// volumes to the machine.
+	VolumeAttachments map[names.VolumeTag]VolumeAttachmentParams
+
+	// Filesystems holds the parameters for filesystems that are to be
+	// created and attached to the machine.
+	Filesystems []MachineFilesystemParams
+
+	// FilesystemAttachments holds the parameters for attaching existing
+	// filesystems to the machine.
+	FilesystemAttachments map[names.FilesystemTag]FilesystemAttachmentParams
+
 	// Nonce holds a unique value that can be used to check
 	// if a new instance was really started for this machine.
 	// See Machine.SetProvisioned. This must be set if InstanceId is set.
@@ -76,13 +93,27 @@ type MachineTemplate struct {
 	principals []string
 }
 
+// MachineVolumeParams holds the parameters for creating a volume and
+// attaching it to a new machine.
+type MachineVolumeParams struct {
+	Volume     VolumeParams
+	Attachment VolumeAttachmentParams
+}
+
+// MachineFilesystemParams holds the parameters for creating a filesystem
+// and attaching it to a new machine.
+type MachineFilesystemParams struct {
+	Filesystem FilesystemParams
+	Attachment FilesystemAttachmentParams
+}
+
 // AddMachineInsideNewMachine creates a new machine within a container
 // of the given type inside another new machine. The two given templates
 // specify the form of the child and parent respectively.
 func (st *State) AddMachineInsideNewMachine(template, parentTemplate MachineTemplate, containerType instance.ContainerType) (*Machine, error) {
 	mdoc, ops, err := st.addMachineInsideNewMachineOps(template, parentTemplate, containerType)
 	if err != nil {
-		return nil, fmt.Errorf("cannot add a new machine: %v", err)
+		return nil, errors.Annotate(err, "cannot add a new machine")
 	}
 	return st.addMachine(mdoc, ops)
 }
@@ -92,7 +123,7 @@ func (st *State) AddMachineInsideNewMachine(template, parentTemplate MachineTemp
 func (st *State) AddMachineInsideMachine(template MachineTemplate, parentId string, containerType instance.ContainerType) (*Machine, error) {
 	mdoc, ops, err := st.addMachineInsideMachineOps(template, parentId, containerType)
 	if err != nil {
-		return nil, fmt.Errorf("cannot add a new machine: %v", err)
+		return nil, errors.Annotate(err, "cannot add a new machine")
 	}
 	return st.addMachine(mdoc, ops)
 }
@@ -127,9 +158,9 @@ func (st *State) AddMachines(templates ...MachineTemplate) (_ []*Machine, err er
 	var ms []*Machine
 	env, err := st.Environment()
 	if err != nil {
-		return nil, err
+		return nil, errors.Trace(err)
 	} else if env.Life() != Alive {
-		return nil, fmt.Errorf("environment is no longer alive")
+		return nil, errors.New("environment is no longer alive")
 	}
 	var ops []txn.Op
 	var mdocs []*machineDoc
@@ -138,12 +169,12 @@ func (st *State) AddMachines(templates ...MachineTemplate) (_ []*Machine, err er
 		// only permitted if unit placement is supported.
 		if len(template.principals) == 0 && template.InstanceId == "" {
 			if err := st.supportsUnitPlacement(); err != nil {
-				return nil, err
+				return nil, errors.Trace(err)
 			}
 		}
 		mdoc, addOps, err := st.addMachineOps(template)
 		if err != nil {
-			return nil, err
+			return nil, errors.Trace(err)
 		}
 		mdocs = append(mdocs, mdoc)
 		ms = append(ms, newMachine(st, mdoc))
@@ -151,12 +182,12 @@ func (st *State) AddMachines(templates ...MachineTemplate) (_ []*Machine, err er
 	}
 	ssOps, err := st.maintainStateServersOps(mdocs, nil)
 	if err != nil {
-		return nil, err
+		return nil, errors.Trace(err)
 	}
 	ops = append(ops, ssOps...)
 	ops = append(ops, env.assertAliveOp())
 	if err := st.runTransaction(ops); err != nil {
-		return nil, onAbort(err, fmt.Errorf("environment is no longer alive"))
+		return nil, onAbort(err, errors.New("environment is no longer alive"))
 	}
 	return ms, nil
 }
@@ -166,13 +197,13 @@ func (st *State) addMachine(mdoc *machineDoc, ops []txn.Op) (*Machine, error) {
 	if err != nil {
 		return nil, err
 	} else if env.Life() != Alive {
-		return nil, fmt.Errorf("environment is no longer alive")
+		return nil, errors.New("environment is no longer alive")
 	}
 	ops = append([]txn.Op{env.assertAliveOp()}, ops...)
 	if err := st.runTransaction(ops); err != nil {
 		enverr := env.Refresh()
 		if (enverr == nil && env.Life() != Alive) || errors.IsNotFound(enverr) {
-			return nil, fmt.Errorf("environment is no longer alive")
+			return nil, errors.New("environment is no longer alive")
 		} else if enverr != nil {
 			err = enverr
 		}
@@ -188,14 +219,14 @@ func (st *State) addMachine(mdoc *machineDoc, ops []txn.Op) (*Machine, error) {
 func (st *State) effectiveMachineTemplate(p MachineTemplate, allowStateServer bool) (tmpl MachineTemplate, err error) {
 	// First check for obvious errors.
 	if p.Series == "" {
-		return tmpl, fmt.Errorf("no series specified")
+		return tmpl, errors.New("no series specified")
 	}
 	if p.InstanceId != "" {
 		if p.Nonce == "" {
-			return tmpl, fmt.Errorf("cannot add a machine with an instance id and no nonce")
+			return tmpl, errors.New("cannot add a machine with an instance id and no nonce")
 		}
 	} else if p.Nonce != "" {
-		return tmpl, fmt.Errorf("cannot specify a nonce without an instance id")
+		return tmpl, errors.New("cannot specify a nonce without an instance id")
 	}
 
 	p.Constraints, err = st.resolveConstraints(p.Constraints)
@@ -210,12 +241,12 @@ func (st *State) effectiveMachineTemplate(p MachineTemplate, allowStateServer bo
 	p.Constraints.Container = nil
 
 	if len(p.Jobs) == 0 {
-		return tmpl, fmt.Errorf("no jobs specified")
+		return tmpl, errors.New("no jobs specified")
 	}
 	jset := make(map[MachineJob]bool)
 	for _, j := range p.Jobs {
 		if jset[j] {
-			return MachineTemplate{}, fmt.Errorf("duplicate job: %s", j)
+			return MachineTemplate{}, errors.Errorf("duplicate job: %s", j)
 		}
 		jset[j] = true
 	}
@@ -231,7 +262,7 @@ func (st *State) effectiveMachineTemplate(p MachineTemplate, allowStateServer bo
 // based on the given template. It also returns the machine document
 // that will be inserted.
 func (st *State) addMachineOps(template MachineTemplate) (*machineDoc, []txn.Op, error) {
-	template, err := st.effectiveMachineTemplate(template, true)
+	template, err := st.effectiveMachineTemplate(template, st.IsStateServer())
 	if err != nil {
 		return nil, nil, err
 	}
@@ -245,7 +276,11 @@ func (st *State) addMachineOps(template MachineTemplate) (*machineDoc, []txn.Op,
 		return nil, nil, err
 	}
 	mdoc := st.machineDocForTemplate(template, strconv.Itoa(seq))
-	prereqOps, machineOp := st.insertNewMachineOps(mdoc, template)
+	prereqOps, machineOp, err := st.insertNewMachineOps(mdoc, template)
+	if err != nil {
+		return nil, nil, errors.Trace(err)
+	}
+	prereqOps = append(prereqOps, assertEnvAliveOp(st.EnvironUUID()))
 	prereqOps = append(prereqOps, st.insertNewContainerRefOp(mdoc.Id))
 	if template.InstanceId != "" {
 		prereqOps = append(prereqOps, txn.Op{
@@ -267,6 +302,7 @@ func (st *State) addMachineOps(template MachineTemplate) (*machineDoc, []txn.Op,
 			},
 		})
 	}
+
 	return mdoc, append(prereqOps, machineOp), nil
 }
 
@@ -293,14 +329,14 @@ func (m *Machine) supportsContainerType(ctype instance.ContainerType) bool {
 // a container of the given type on an existing machine.
 func (st *State) addMachineInsideMachineOps(template MachineTemplate, parentId string, containerType instance.ContainerType) (*machineDoc, []txn.Op, error) {
 	if template.InstanceId != "" {
-		return nil, nil, fmt.Errorf("cannot specify instance id for a new container")
+		return nil, nil, errors.New("cannot specify instance id for a new container")
 	}
 	template, err := st.effectiveMachineTemplate(template, false)
 	if err != nil {
 		return nil, nil, err
 	}
 	if containerType == "" {
-		return nil, nil, fmt.Errorf("no container type specified")
+		return nil, nil, errors.New("no container type specified")
 	}
 	// Adding a machine within a machine implies add-machine or placement.
 	if err := st.supportsUnitPlacement(); err != nil {
@@ -314,7 +350,7 @@ func (st *State) addMachineInsideMachineOps(template MachineTemplate, parentId s
 		return nil, nil, err
 	}
 	if !parent.supportsContainerType(containerType) {
-		return nil, nil, fmt.Errorf("machine %s cannot host %s containers", parentId, containerType)
+		return nil, nil, errors.Errorf("machine %s cannot host %s containers", parentId, containerType)
 	}
 	newId, err := st.newContainerId(parentId, containerType)
 	if err != nil {
@@ -322,7 +358,10 @@ func (st *State) addMachineInsideMachineOps(template MachineTemplate, parentId s
 	}
 	mdoc := st.machineDocForTemplate(template, newId)
 	mdoc.ContainerType = string(containerType)
-	prereqOps, machineOp := st.insertNewMachineOps(mdoc, template)
+	prereqOps, machineOp, err := st.insertNewMachineOps(mdoc, template)
+	if err != nil {
+		return nil, nil, errors.Trace(err)
+	}
 	prereqOps = append(prereqOps,
 		// Update containers record for host machine.
 		st.addChildToContainerRefOp(parentId, mdoc.Id),
@@ -348,7 +387,7 @@ func (st *State) newContainerId(parentId string, containerType instance.Containe
 // of the child and parent respectively.
 func (st *State) addMachineInsideNewMachineOps(template, parentTemplate MachineTemplate, containerType instance.ContainerType) (*machineDoc, []txn.Op, error) {
 	if template.InstanceId != "" || parentTemplate.InstanceId != "" {
-		return nil, nil, fmt.Errorf("cannot specify instance id for a new container")
+		return nil, nil, errors.New("cannot specify instance id for a new container")
 	}
 	seq, err := st.sequence("machine")
 	if err != nil {
@@ -359,7 +398,7 @@ func (st *State) addMachineInsideNewMachineOps(template, parentTemplate MachineT
 		return nil, nil, err
 	}
 	if containerType == "" {
-		return nil, nil, fmt.Errorf("no container type specified")
+		return nil, nil, errors.New("no container type specified")
 	}
 	if parentTemplate.InstanceId == "" {
 		// Adding a machine within a machine implies add-machine or placement.
@@ -382,8 +421,14 @@ func (st *State) addMachineInsideNewMachineOps(template, parentTemplate MachineT
 	}
 	mdoc := st.machineDocForTemplate(template, newId)
 	mdoc.ContainerType = string(containerType)
-	parentPrereqOps, parentOp := st.insertNewMachineOps(parentDoc, parentTemplate)
-	prereqOps, machineOp := st.insertNewMachineOps(mdoc, template)
+	parentPrereqOps, parentOp, err := st.insertNewMachineOps(parentDoc, parentTemplate)
+	if err != nil {
+		return nil, nil, errors.Trace(err)
+	}
+	prereqOps, machineOp, err := st.insertNewMachineOps(mdoc, template)
+	if err != nil {
+		return nil, nil, errors.Trace(err)
+	}
 	prereqOps = append(prereqOps, parentPrereqOps...)
 	prereqOps = append(prereqOps,
 		// The host machine doesn't exist yet, create a new containers record.
@@ -405,7 +450,7 @@ func (st *State) machineDocForTemplate(template MachineTemplate, id string) *mac
 		Principals: template.principals,
 		Life:       Alive,
 		Nonce:      template.Nonce,
-		Addresses:  instanceAddressesToAddresses(template.Addresses),
+		Addresses:  fromNetworkAddresses(template.Addresses),
 		NoVote:     template.NoVote,
 		Placement:  template.Placement,
 	}
@@ -414,7 +459,7 @@ func (st *State) machineDocForTemplate(template MachineTemplate, id string) *mac
 // insertNewMachineOps returns operations to insert the given machine
 // document into the database, based on the given template. Only the
 // constraints and networks are used from the template.
-func (st *State) insertNewMachineOps(mdoc *machineDoc, template MachineTemplate) (prereqOps []txn.Op, machineOp txn.Op) {
+func (st *State) insertNewMachineOps(mdoc *machineDoc, template MachineTemplate) (prereqOps []txn.Op, machineOp txn.Op, err error) {
 	machineOp = txn.Op{
 		C:      machinesC,
 		Id:     mdoc.DocID,
@@ -422,18 +467,99 @@ func (st *State) insertNewMachineOps(mdoc *machineDoc, template MachineTemplate)
 		Insert: mdoc,
 	}
 
-	return []txn.Op{
-		createConstraintsOp(st, machineGlobalKey(mdoc.Id), template.Constraints),
-		createStatusOp(st, machineGlobalKey(mdoc.Id), statusDoc{
-			Status:  StatusPending,
-			EnvUUID: st.EnvironUUID(),
-		}),
+	statusDoc := statusDoc{
+		Status:  StatusPending,
+		EnvUUID: st.EnvironUUID(),
+		Updated: time.Now().UnixNano(),
+	}
+	globalKey := machineGlobalKey(mdoc.Id)
+	prereqOps = []txn.Op{
+		createConstraintsOp(st, globalKey, template.Constraints),
+		createStatusOp(st, globalKey, statusDoc),
 		// TODO(dimitern) 2014-04-04 bug #1302498
 		// Once we can add networks independently of machine
 		// provisioning, we should check the given networks are valid
 		// and known before setting them.
-		createRequestedNetworksOp(st, machineGlobalKey(mdoc.Id), template.RequestedNetworks),
-	}, machineOp
+		createRequestedNetworksOp(st, globalKey, template.RequestedNetworks),
+		createMachineBlockDevicesOp(mdoc.Id),
+	}
+
+	storageOps, err := st.machineStorageOps(mdoc, &machineStorageParams{
+		filesystems:           template.Filesystems,
+		filesystemAttachments: template.FilesystemAttachments,
+		volumes:               template.Volumes,
+		volumeAttachments:     template.VolumeAttachments,
+	})
+	if err != nil {
+		return nil, txn.Op{}, errors.Trace(err)
+	}
+	prereqOps = append(prereqOps, storageOps...)
+
+	// At the last moment we still have statusDoc in scope, set the initial
+	// history entry. This is risky, and may lead to extra entries, but that's
+	// an intrinsic problem with mixing txn and non-txn ops -- we can't sync
+	// them cleanly.
+	probablyUpdateStatusHistory(st, globalKey, statusDoc)
+	return prereqOps, machineOp, nil
+}
+
+type machineStorageParams struct {
+	volumes               []MachineVolumeParams
+	volumeAttachments     map[names.VolumeTag]VolumeAttachmentParams
+	filesystems           []MachineFilesystemParams
+	filesystemAttachments map[names.FilesystemTag]FilesystemAttachmentParams
+}
+
+func (st *State) machineStorageOps(mdoc *machineDoc, args *machineStorageParams) ([]txn.Op, error) {
+	var filesystemOps, volumeOps []txn.Op
+	var fsAttachments []filesystemAttachmentTemplate
+	var volumeAttachments []volumeAttachmentTemplate
+
+	// Create filesystems and filesystem attachments.
+	for _, f := range args.filesystems {
+		ops, filesystemTag, volumeTag, err := st.addFilesystemOps(f.Filesystem, mdoc.Id)
+		if err != nil {
+			return nil, errors.Trace(err)
+		}
+		filesystemOps = append(filesystemOps, ops...)
+		fsAttachments = append(fsAttachments, filesystemAttachmentTemplate{
+			filesystemTag, f.Attachment,
+		})
+		if volumeTag != (names.VolumeTag{}) {
+			// The filesystem requires a volume, so create a volume attachment too.
+			volumeAttachments = append(volumeAttachments, volumeAttachmentTemplate{
+				volumeTag, VolumeAttachmentParams{},
+			})
+		}
+	}
+
+	// Create volumes and volume attachments.
+	for _, v := range args.volumes {
+		op, tag, err := st.addVolumeOp(v.Volume, mdoc.Id)
+		if err != nil {
+			return nil, errors.Trace(err)
+		}
+		volumeOps = append(volumeOps, op)
+		volumeAttachments = append(volumeAttachments, volumeAttachmentTemplate{
+			tag, v.Attachment,
+		})
+	}
+
+	// TODO(axw) handle args.filesystemAttachments, args.volumeAttachments
+	// when we handle attaching to existing (e.g. shared) storage.
+
+	ops := make([]txn.Op, 0, len(filesystemOps)+len(volumeOps)+len(fsAttachments)+len(volumeAttachments))
+	if len(fsAttachments) > 0 {
+		attachmentOps := createMachineFilesystemAttachmentsOps(mdoc.Id, fsAttachments)
+		ops = append(ops, filesystemOps...)
+		ops = append(ops, attachmentOps...)
+	}
+	if len(volumeAttachments) > 0 {
+		attachmentOps := createMachineVolumeAttachmentsOps(mdoc.Id, volumeAttachments)
+		ops = append(ops, volumeOps...)
+		ops = append(ops, attachmentOps...)
+	}
+	return ops, nil
 }
 
 func hasJob(jobs []MachineJob, job MachineJob) bool {
@@ -445,7 +571,7 @@ func hasJob(jobs []MachineJob, job MachineJob) bool {
 	return false
 }
 
-var errStateServerNotAllowed = fmt.Errorf("state server jobs specified without calling EnsureAvailability")
+var errStateServerNotAllowed = errors.New("state server jobs specified but not allowed")
 
 // maintainStateServersOps returns a set of operations that will maintain
 // the state server information when the given machine documents
@@ -474,10 +600,10 @@ func (st *State) maintainStateServersOps(mdocs []*machineDoc, currentInfo *State
 		var err error
 		currentInfo, err = st.StateServerInfo()
 		if err != nil {
-			return nil, fmt.Errorf("cannot get state server info: %v", err)
+			return nil, errors.Annotate(err, "cannot get state server info")
 		}
 		if len(currentInfo.MachineIds) > 0 || len(currentInfo.VotingMachineIds) > 0 {
-			return nil, fmt.Errorf("state servers already exist")
+			return nil, errors.New("state servers already exist")
 		}
 	}
 	ops := []txn.Op{{
@@ -508,10 +634,10 @@ func (st *State) EnsureAvailability(
 ) (StateServersChanges, error) {
 
 	if numStateServers < 0 || (numStateServers != 0 && numStateServers%2 != 1) {
-		return StateServersChanges{}, fmt.Errorf("number of state servers must be odd and non-negative")
+		return StateServersChanges{}, errors.New("number of state servers must be odd and non-negative")
 	}
 	if numStateServers > replicaset.MaxPeers {
-		return StateServersChanges{}, fmt.Errorf("state server count is too large (allowed %d)", replicaset.MaxPeers)
+		return StateServersChanges{}, errors.Errorf("state server count is too large (allowed %d)", replicaset.MaxPeers)
 	}
 	var change StateServersChanges
 	buildTxn := func(attempt int) ([]txn.Op, error) {
@@ -527,10 +653,10 @@ func (st *State) EnsureAvailability(
 			}
 		}
 		if len(currentInfo.VotingMachineIds) > desiredStateServerCount {
-			return nil, fmt.Errorf("cannot reduce state server count")
+			return nil, errors.New("cannot reduce state server count")
 		}
 
-		intent, err := st.ensureAvailabilityIntentions(currentInfo)
+		intent, err := st.ensureAvailabilityIntentions(currentInfo, placement)
 		if err != nil {
 			return nil, err
 		}
@@ -548,11 +674,18 @@ func (st *State) EnsureAvailability(
 			intent.promote = intent.promote[:n]
 		}
 		voteCount += len(intent.promote)
+
+		if n := desiredStateServerCount - voteCount; n < len(intent.convert) {
+			intent.convert = intent.convert[:n]
+		}
+		voteCount += len(intent.convert)
+
 		intent.newCount = desiredStateServerCount - voteCount
-		logger.Infof("%d new machines; promoting %v", intent.newCount, intent.promote)
+
+		logger.Infof("%d new machines; promoting %v; converting %v", intent.newCount, intent.promote, intent.convert)
 
 		var ops []txn.Op
-		ops, change, err = st.ensureAvailabilityIntentionOps(intent, currentInfo, cons, series, placement)
+		ops, change, err = st.ensureAvailabilityIntentionOps(intent, currentInfo, cons, series)
 		return ops, err
 	}
 	if err := st.run(buildTxn); err != nil {
@@ -569,6 +702,7 @@ type StateServersChanges struct {
 	Maintained []string
 	Promoted   []string
 	Demoted    []string
+	Converted  []string
 }
 
 // ensureAvailabilityIntentionOps returns operations to fulfil the desired intent.
@@ -577,7 +711,6 @@ func (st *State) ensureAvailabilityIntentionOps(
 	currentInfo *StateServerInfo,
 	cons constraints.Value,
 	series string,
-	placement []string,
 ) ([]txn.Op, StateServersChanges, error) {
 	var ops []txn.Op
 	var change StateServersChanges
@@ -589,16 +722,20 @@ func (st *State) ensureAvailabilityIntentionOps(
 		ops = append(ops, demoteStateServerOps(m)...)
 		change.Demoted = append(change.Demoted, m.doc.Id)
 	}
+	for _, m := range intent.convert {
+		ops = append(ops, convertStateServerOps(m)...)
+		change.Converted = append(change.Converted, m.doc.Id)
+	}
 	// Use any placement directives that have been provided
 	// when adding new machines, until the directives have
 	// been all used up. Set up a helper function to do the
 	// work required.
 	placementCount := 0
 	getPlacement := func() string {
-		if placementCount >= len(placement) {
+		if placementCount >= len(intent.placement) {
 			return ""
 		}
-		result := placement[placementCount]
+		result := intent.placement[placementCount]
 		placementCount++
 		return result
 	}
@@ -631,16 +768,16 @@ func (st *State) ensureAvailabilityIntentionOps(
 	for _, m := range intent.maintain {
 		tag, err := names.ParseTag(m.Tag().String())
 		if err != nil {
-			return nil, StateServersChanges{}, fmt.Errorf("could not parse machine tag: %v", err)
+			return nil, StateServersChanges{}, errors.Annotate(err, "could not parse machine tag")
 		}
 		if tag.Kind() != names.MachineTagKind {
-			return nil, StateServersChanges{}, fmt.Errorf("expected machine tag kind, got %s", tag.Kind())
+			return nil, StateServersChanges{}, errors.Errorf("expected machine tag kind, got %s", tag.Kind())
 		}
 		change.Maintained = append(change.Maintained, tag.Id())
 	}
 	ssOps, err := st.maintainStateServersOps(mdocs, currentInfo)
 	if err != nil {
-		return nil, StateServersChanges{}, fmt.Errorf("cannot prepare machine add operations: %v", err)
+		return nil, StateServersChanges{}, errors.Annotate(err, "cannot prepare machine add operations")
 	}
 	ops = append(ops, ssOps...)
 	return ops, change, nil
@@ -656,8 +793,10 @@ var stateServerAvailable = func(m *Machine) (bool, error) {
 }
 
 type ensureAvailabilityIntent struct {
-	newCount                          int
-	promote, maintain, demote, remove []*Machine
+	newCount  int
+	placement []string
+
+	promote, maintain, demote, remove, convert []*Machine
 }
 
 // ensureAvailabilityIntentions returns what we would like
@@ -666,8 +805,40 @@ type ensureAvailabilityIntent struct {
 //   demoting unavailable, voting machines;
 //   removing unavailable, non-voting, non-vote-holding machines;
 //   gathering available, non-voting machines that may be promoted;
-func (st *State) ensureAvailabilityIntentions(info *StateServerInfo) (*ensureAvailabilityIntent, error) {
+func (st *State) ensureAvailabilityIntentions(info *StateServerInfo, placement []string) (*ensureAvailabilityIntent, error) {
 	var intent ensureAvailabilityIntent
+	for _, s := range placement {
+		// TODO(natefinch): unscoped placements shouldn't ever get here (though
+		// they do currently).  We should fix up the CLI to always add a scope
+		// to placements and then we can remove the need to deal with unscoped
+		// placements.
+		p, err := instance.ParsePlacement(s)
+		if err == instance.ErrPlacementScopeMissing {
+			intent.placement = append(intent.placement, s)
+			continue
+		}
+		if err == nil && p.Scope == instance.MachineScope {
+			// TODO(natefinch) add env provider policy to check if conversion is
+			// possible (e.g. cannot be supported by Azure in HA mode).
+
+			if names.IsContainerMachine(p.Directive) {
+				return nil, errors.New("container placement directives not supported")
+			}
+
+			m, err := st.Machine(p.Directive)
+			if err != nil {
+				return nil, errors.Annotatef(err, "can't find machine for placement directive %q", s)
+			}
+			if m.IsManager() {
+				return nil, errors.Errorf("machine for placement directive %q is already a state server", s)
+			}
+			intent.convert = append(intent.convert, m)
+			intent.placement = append(intent.placement, s)
+			continue
+		}
+		return nil, errors.Errorf("unsupported placement directive %q", s)
+	}
+
 	for _, mid := range info.MachineIds {
 		m, err := st.Machine(mid)
 		if err != nil {
@@ -702,10 +873,30 @@ func (st *State) ensureAvailabilityIntentions(info *StateServerInfo) (*ensureAva
 			intent.remove = append(intent.remove, m)
 		}
 	}
-	logger.Infof("initial intentions: promote %v; maintain %v; demote %v; remove %v", intent.promote, intent.maintain, intent.demote, intent.remove)
+	logger.Infof("initial intentions: promote %v; maintain %v; demote %v; remove %v; convert: %v",
+		intent.promote, intent.maintain, intent.demote, intent.remove, intent.convert)
 	return &intent, nil
 }
 
+func convertStateServerOps(m *Machine) []txn.Op {
+	return []txn.Op{{
+		C:  machinesC,
+		Id: m.doc.DocID,
+		Update: bson.D{
+			{"$addToSet", bson.D{{"jobs", JobManageEnviron}}},
+			{"$set", bson.D{{"novote", false}}},
+		},
+		Assert: bson.D{{"jobs", bson.D{{"$nin", []MachineJob{JobManageEnviron}}}}},
+	}, {
+		C:  stateServersC,
+		Id: environGlobalKey,
+		Update: bson.D{
+			{"$addToSet", bson.D{{"votingmachineids", m.doc.Id}}},
+			{"$addToSet", bson.D{{"machineids", m.doc.Id}}},
+		},
+	}}
+}
+
 func promoteStateServerOps(m *Machine) []txn.Op {
 	return []txn.Op{{
 		C:      machinesC,
