@@ -16,19 +16,26 @@ import (
 	"github.com/juju/juju/constraints"
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/state"
+	"github.com/juju/juju/storage/poolmanager"
+	"github.com/juju/juju/storage/provider"
+	"github.com/juju/juju/storage/provider/registry"
 )
 
 type AssignSuite struct {
 	ConnSuite
-	wordpress *state.Service
+	wordpress  *state.Service
+	storageSvc *state.Service
 }
 
 var _ = gc.Suite(&AssignSuite{})
-var _ = gc.Suite(&assignCleanSuite{ConnSuite{}, state.AssignCleanEmpty, nil})
-var _ = gc.Suite(&assignCleanSuite{ConnSuite{}, state.AssignClean, nil})
 
 func (s *AssignSuite) SetUpTest(c *gc.C) {
 	s.ConnSuite.SetUpTest(c)
+	pm := poolmanager.New(state.NewStateSettings(s.State))
+	_, err := pm.Create("loop-pool", provider.LoopProviderType, map[string]interface{}{})
+	c.Assert(err, jc.ErrorIsNil)
+	registry.RegisterEnvironStorageProviders("someprovider", provider.LoopProviderType)
+
 	wordpress := s.AddTestingServiceWithNetworks(
 		c,
 		"wordpress",
@@ -37,6 +44,16 @@ func (s *AssignSuite) SetUpTest(c *gc.C) {
 	)
 	wordpress.SetConstraints(constraints.MustParse("networks=net3,^net4,^net5"))
 	s.wordpress = wordpress
+	s.storageSvc = s.AddTestingServiceWithStorage(
+		c, "storage-block", s.AddTestingCharm(c, "storage-block"),
+		map[string]state.StorageConstraints{
+			"data": {
+				Pool:  "loop-pool",
+				Count: 1,
+				Size:  1024,
+			},
+		},
+	)
 }
 
 func (s *AssignSuite) addSubordinate(c *gc.C, principal *state.Unit) *state.Unit {
@@ -669,6 +686,9 @@ type assignCleanSuite struct {
 	wordpress *state.Service
 }
 
+var _ = gc.Suite(&assignCleanSuite{ConnSuite{}, state.AssignCleanEmpty, nil})
+var _ = gc.Suite(&assignCleanSuite{ConnSuite{}, state.AssignClean, nil})
+
 func (s *assignCleanSuite) SetUpTest(c *gc.C) {
 	c.Logf("assignment policy for this test: %q", s.policy)
 	s.ConnSuite.SetUpTest(c)
@@ -988,6 +1008,47 @@ func (s *assignCleanSuite) TestAssignUnitToMachineWorksWithMachine0(c *gc.C) {
 	c.Assert(assignedTo.Id(), gc.Equals, "0")
 }
 
+func (s *AssignSuite) TestAssignUnitWithStorageCleanAvailable(c *gc.C) {
+	cons, err := s.storageSvc.StorageConstraints()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(cons, gc.HasLen, 2)
+
+	unit, err := s.storageSvc.AddUnit()
+	c.Assert(err, jc.ErrorIsNil)
+	storageAttachments, err := s.State.UnitStorageAttachments(unit.UnitTag())
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(storageAttachments, gc.HasLen, 1)
+
+	// Add a clean machine.
+	clean, err := s.State.AddMachine("quantal", state.JobHostUnits)
+	c.Assert(err, jc.ErrorIsNil)
+
+	// assign the unit to a machine, requesting clean/empty. Since
+	// the unit has storage instances associated, it will be forced
+	// onto a new machine.
+	err = s.State.AssignUnit(unit, state.AssignCleanEmpty)
+	c.Assert(err, jc.ErrorIsNil)
+
+	// Check the machine on the unit is set.
+	machineId, err := unit.AssignedMachineId()
+	c.Assert(err, jc.ErrorIsNil)
+	// Check that the machine isn't our clean one.
+	c.Assert(machineId, gc.Not(gc.Equals), clean.Id())
+
+	// Check that a volume attachments were added to the machine.
+	machine, err := s.State.Machine(machineId)
+	c.Assert(err, jc.ErrorIsNil)
+	volumeAttachments, err := s.State.MachineVolumeAttachments(machine.MachineTag())
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(volumeAttachments, gc.HasLen, 1)
+
+	volume, err := s.State.Volume(volumeAttachments[0].Volume())
+	c.Assert(err, jc.ErrorIsNil)
+	volumeStorageInstance, err := volume.StorageInstance()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(volumeStorageInstance, gc.Equals, storageAttachments[0].StorageInstance())
+}
+
 func (s *assignCleanSuite) TestAssignUnitPolicy(c *gc.C) {
 	_, err := s.State.AddMachine("quantal", state.JobManageEnviron) // bootstrap machine
 	c.Assert(err, jc.ErrorIsNil)
