@@ -8,6 +8,7 @@ import (
 	"strings"
 
 	"github.com/juju/errors"
+	"github.com/juju/names"
 	"gopkg.in/mgo.v2/bson"
 	"gopkg.in/mgo.v2/txn"
 )
@@ -22,6 +23,7 @@ const (
 	cleanupRemovedUnit                 cleanupKind = "removedUnit"
 	cleanupServicesForDyingEnvironment cleanupKind = "services"
 	cleanupForceDestroyedMachine       cleanupKind = "machine"
+	cleanupAttachmentsForDyingStorage  cleanupKind = "storageAttachments"
 )
 
 // cleanupDoc represents a potentially large set of documents that should be
@@ -84,6 +86,8 @@ func (st *State) Cleanup() error {
 			err = st.cleanupServicesForDyingEnvironment()
 		case cleanupForceDestroyedMachine:
 			err = st.cleanupForceDestroyedMachine(doc.Prefix)
+		case cleanupAttachmentsForDyingStorage:
+			err = st.cleanupAttachmentsForDyingStorage(doc.Prefix)
 		default:
 			err = fmt.Errorf("unknown cleanup kind %q", doc.Kind)
 		}
@@ -107,16 +111,18 @@ func (st *State) Cleanup() error {
 }
 
 func (st *State) cleanupRelationSettings(prefix string) error {
+	settings, closer := st.getCollection(settingsC)
+	defer closer()
 	// Documents marked for cleanup are not otherwise referenced in the
 	// system, and will not be under watch, and are therefore safe to
 	// delete directly.
-	settings, closer := st.getCollection(settingsC)
-	defer closer()
+	settingsW := settings.Writeable()
+
 	sel := bson.D{{"_id", bson.D{{"$regex", "^" + st.docID(prefix)}}}}
-	if count, err := settings.Find(sel).Count(); err != nil {
+	if count, err := settingsW.Find(sel).Count(); err != nil {
 		return fmt.Errorf("cannot detect cleanup targets: %v", err)
 	} else if count != 0 {
-		if _, err := settings.RemoveAll(sel); err != nil {
+		if _, err := settingsW.RemoveAll(sel); err != nil {
 			return fmt.Errorf("cannot remove documents marked for cleanup: %v", err)
 		}
 	}
@@ -176,9 +182,8 @@ func (st *State) cleanupUnitsForDyingService(serviceName string) error {
 	return nil
 }
 
-// cleanupDyingUnit marks the unit as departing from all its joined relations,
-// allowing related units to start converging to a state in which that unit is
-// gone as quickly as possible.
+// cleanupDyingUnit marks resources owned by the unit as dying, to ensure
+// they are cleaned up as well.
 func (st *State) cleanupDyingUnit(name string) error {
 	unit, err := st.Unit(name)
 	if errors.IsNotFound(err) {
@@ -186,6 +191,9 @@ func (st *State) cleanupDyingUnit(name string) error {
 	} else if err != nil {
 		return err
 	}
+	// Mark the unit as departing from its joined relations, allowing
+	// related units to start converging to a state in which that unit
+	// is gone as quickly as possible.
 	relations, err := unit.RelationsJoined()
 	if err != nil {
 		return err
@@ -201,6 +209,22 @@ func (st *State) cleanupDyingUnit(name string) error {
 			return err
 		}
 	}
+	// Mark storage attachments as dying, so that they are detached
+	// and removed from state, allowing the unit to terminate.
+	storageAttachments, err := st.UnitStorageAttachments(unit.UnitTag())
+	if err != nil {
+		return err
+	}
+	for _, storageAttachment := range storageAttachments {
+		err := st.DestroyStorageAttachment(
+			storageAttachment.StorageInstance(), unit.UnitTag(),
+		)
+		if errors.IsNotFound(err) {
+			continue
+		} else if err != nil {
+			return err
+		}
+	}
 	return nil
 }
 
@@ -313,7 +337,7 @@ func (st *State) obliterateUnit(unitName string) error {
 	// prevent further dependencies being added. If we're really lucky, the
 	// unit will be removed immediately.
 	if err := unit.Destroy(); err != nil {
-		return err
+		return errors.Annotatef(err, "cannot destroy unit %q", unitName)
 	}
 	if err := unit.Refresh(); errors.IsNotFound(err) {
 		return nil
@@ -330,3 +354,31 @@ func (st *State) obliterateUnit(unitName string) error {
 	}
 	return unit.Remove()
 }
+
+// cleanupAttachmentsForDyingStorage sets all storage attachments related
+// to the specified storage instance to Dying, if they are not already Dying
+// or Dead. It's expected to be used when a storage instance is destroyed.
+func (st *State) cleanupAttachmentsForDyingStorage(storageId string) error {
+	storageTag := names.NewStorageTag(storageId)
+
+	// This won't miss attachments, because a Dying service cannot have
+	// attachments added to it. But we do have to remove the attachments
+	// themselves via individual transactions, because they could be in
+	// any state at all.
+	coll, closer := st.getCollection(storageAttachmentsC)
+	defer closer()
+
+	var doc storageAttachmentDoc
+	fields := bson.D{{"unitid", 1}}
+	iter := coll.Find(bson.D{{"storageid", storageId}}).Select(fields).Iter()
+	for iter.Next(&doc) {
+		unitTag := names.NewUnitTag(doc.Unit)
+		if err := st.DestroyStorageAttachment(storageTag, unitTag); err != nil {
+			return errors.Annotate(err, "destroying storage attachment")
+		}
+	}
+	if err := iter.Close(); err != nil {
+		return errors.Annotate(err, "cannot read storage attachment document")
+	}
+	return nil
+}
