@@ -7,8 +7,6 @@ import (
 	stdtesting "testing"
 
 	"github.com/juju/names"
-	gitjujutesting "github.com/juju/testing"
-	jc "github.com/juju/testing/checkers"
 	gc "gopkg.in/check.v1"
 	"gopkg.in/mgo.v2"
 
@@ -26,8 +24,7 @@ func TestPackage(t *stdtesting.T) {
 // ConnSuite provides the infrastructure for all other
 // test suites (StateSuite, CharmSuite, MachineSuite, etc).
 type ConnSuite struct {
-	gitjujutesting.MgoSuite
-	testing.BaseSuite
+	statetesting.StateSuite
 	annotations  *mgo.Collection
 	charms       *mgo.Collection
 	machines     *mgo.Collection
@@ -36,34 +33,20 @@ type ConnSuite struct {
 	services     *mgo.Collection
 	units        *mgo.Collection
 	stateServers *mgo.Collection
-	State        *state.State
 	policy       statetesting.MockPolicy
 	factory      *factory.Factory
 	envTag       names.EnvironTag
-	owner        names.UserTag
-}
-
-func (cs *ConnSuite) SetUpSuite(c *gc.C) {
-	cs.BaseSuite.SetUpSuite(c)
-	cs.MgoSuite.SetUpSuite(c)
-}
-
-func (cs *ConnSuite) TearDownSuite(c *gc.C) {
-	cs.MgoSuite.TearDownSuite(c)
-	cs.BaseSuite.TearDownSuite(c)
 }
 
 func (cs *ConnSuite) SetUpTest(c *gc.C) {
 	c.Log("SetUpTest")
-	cs.BaseSuite.SetUpTest(c)
-	cs.MgoSuite.SetUpTest(c)
+
 	cs.policy = statetesting.MockPolicy{}
-	cfg := testing.EnvironConfig(c)
-	cs.owner = names.NewLocalUserTag("test-admin")
-	cs.State = TestingInitialize(c, cs.owner, cfg, &cs.policy)
-	uuid, ok := cfg.UUID()
-	c.Assert(ok, jc.IsTrue)
-	cs.envTag = names.NewEnvironTag(uuid)
+	cs.StateSuite.Policy = &cs.policy
+
+	cs.StateSuite.SetUpTest(c)
+
+	cs.envTag = cs.State.EnvironTag()
 	cs.factory = factory.NewFactory(cs.State)
 
 	jujuDB := cs.MgoSuite.Session.DB("juju")
@@ -79,25 +62,20 @@ func (cs *ConnSuite) SetUpTest(c *gc.C) {
 	c.Log("SetUpTest done")
 }
 
-func (cs *ConnSuite) TearDownTest(c *gc.C) {
-	if cs.State != nil {
-		// If setup fails, we don't have a State yet
-		cs.State.Close()
-	}
-	cs.MgoSuite.TearDownTest(c)
-	cs.BaseSuite.TearDownTest(c)
-}
-
 func (s *ConnSuite) AddTestingCharm(c *gc.C, name string) *state.Charm {
 	return state.AddTestingCharm(c, s.State, name)
 }
 
 func (s *ConnSuite) AddTestingService(c *gc.C, name string, ch *state.Charm) *state.Service {
-	return state.AddTestingService(c, s.State, name, ch, s.owner)
+	return state.AddTestingService(c, s.State, name, ch, s.Owner)
 }
 
 func (s *ConnSuite) AddTestingServiceWithNetworks(c *gc.C, name string, ch *state.Charm, networks []string) *state.Service {
-	return state.AddTestingServiceWithNetworks(c, s.State, name, ch, s.owner, networks)
+	return state.AddTestingServiceWithNetworks(c, s.State, name, ch, s.Owner, networks)
+}
+
+func (s *ConnSuite) AddTestingServiceWithStorage(c *gc.C, name string, ch *state.Charm, storage map[string]state.StorageConstraints) *state.Service {
+	return state.AddTestingServiceWithStorage(c, s.State, name, ch, s.Owner, storage)
 }
 
 func (s *ConnSuite) AddSeriesCharm(c *gc.C, name, series string) *state.Charm {
