@@ -32,7 +32,6 @@ import (
 type Machine struct {
 	st  *State
 	doc machineDoc
-	annotator
 	presence.Presencer
 }
 
@@ -130,11 +129,6 @@ func newMachine(st *State, doc *machineDoc) *Machine {
 		st:  st,
 		doc: *doc,
 	}
-	machine.annotator = annotator{
-		globalKey: machine.globalKey(),
-		tag:       machine.Tag(),
-		st:        st,
-	}
 	return machine
 }
 
@@ -224,6 +218,12 @@ func getInstanceData(st *State, id string) (instanceData, error) {
 // will be different from other Tag values returned by any other entities
 // from the same state.
 func (m *Machine) Tag() names.Tag {
+	return m.MachineTag()
+}
+
+// MachineTag returns the more specific MachineTag type as opposed
+// to the more generic Tag type.
+func (m *Machine) MachineTag() names.MachineTag {
 	return names.NewMachineTag(m.Id())
 }
 
@@ -344,7 +344,7 @@ func (m *Machine) SetMongoPassword(password string) error {
 	if !m.IsManager() {
 		return errors.NotSupportedf("setting mongo password for non-state server machine %v", m)
 	}
-	return mongo.SetAdminMongoPassword(m.st.db.Session, m.Tag().String(), password)
+	return mongo.SetAdminMongoPassword(m.st.session, m.Tag().String(), password)
 }
 
 // SetPassword sets the password for the machine's agent.
@@ -365,7 +365,11 @@ func (m *Machine) setPasswordHash(passwordHash string) error {
 		Assert: notDeadDoc,
 		Update: bson.D{{"$set", bson.D{{"passwordhash", passwordHash}}}},
 	}}
-	if err := m.st.runTransaction(ops); err != nil {
+	// A "raw" transaction is used here because this code has to work
+	// before the machine env UUID DB migration has run. In this case
+	// we don't want the automatic env UUID prefixing to the doc _id
+	// to occur.
+	if err := m.st.runRawTransaction(ops); err != nil {
 		return fmt.Errorf("cannot set password of machine %v: %v", m, onAbort(err, ErrDead))
 	}
 	m.doc.PasswordHash = passwordHash
@@ -471,6 +475,12 @@ func (m *Machine) ParentId() (string, bool) {
 	return parentId, parentId != ""
 }
 
+// IsContainer returns true if the machine is a container.
+func (m *Machine) IsContainer() bool {
+	_, isContainer := m.ParentId()
+	return isContainer
+}
+
 type HasContainersError struct {
 	MachineId    string
 	ContainerIds []string
@@ -521,29 +531,30 @@ func (original *Machine) advanceLifecycle(life Life) (err error) {
 		Id:     m.doc.DocID,
 		Update: bson.D{{"$set", bson.D{{"life", life}}}},
 	}
-	advanceAsserts := bson.D{
-		{"jobs", bson.D{{"$nin", []MachineJob{JobManageEnviron}}}},
-		{"$or", []bson.D{
+	// noUnits asserts that the machine has no principal units.
+	noUnits := bson.DocElem{
+		"$or", []bson.D{
 			{{"principals", bson.D{{"$size", 0}}}},
 			{{"principals", bson.D{{"$exists", false}}}},
-		}},
-		{"hasvote", bson.D{{"$ne", true}}},
+		},
 	}
 	// multiple attempts: one with original data, one with refreshed data, and a final
 	// one intended to determine the cause of failure of the preceding attempt.
 	buildTxn := func(attempt int) ([]txn.Op, error) {
-		// If the transaction was aborted, grab a fresh copy of the machine data.
+		advanceAsserts := bson.D{
+			{"jobs", bson.D{{"$nin", []MachineJob{JobManageEnviron}}}},
+			{"hasvote", bson.D{{"$ne", true}}},
+		}
+		// Grab a fresh copy of the machine data.
 		// We don't write to original, because the expectation is that state-
 		// changing methods only set the requested change on the receiver; a case
 		// could perhaps be made that this is not a helpful convention in the
 		// context of the new state API, but we maintain consistency in the
 		// face of uncertainty.
-		if attempt != 0 {
-			if m, err = m.st.Machine(m.doc.Id); errors.IsNotFound(err) {
-				return nil, jujutxn.ErrNoOperations
-			} else if err != nil {
-				return nil, err
-			}
+		if m, err = m.st.Machine(m.doc.Id); errors.IsNotFound(err) {
+			return nil, jujutxn.ErrNoOperations
+		} else if err != nil {
+			return nil, err
 		}
 		// Check that the life change is sane, and collect the assertions
 		// necessary to determine that it remains so.
@@ -552,12 +563,12 @@ func (original *Machine) advanceLifecycle(life Life) (err error) {
 			if m.doc.Life != Alive {
 				return nil, jujutxn.ErrNoOperations
 			}
-			op.Assert = append(advanceAsserts, isAliveDoc...)
+			advanceAsserts = append(advanceAsserts, isAliveDoc...)
 		case Dead:
 			if m.doc.Life == Dead {
 				return nil, jujutxn.ErrNoOperations
 			}
-			op.Assert = append(advanceAsserts, notDeadDoc...)
+			advanceAsserts = append(advanceAsserts, notDeadDoc...)
 		default:
 			panic(fmt.Errorf("cannot advance lifecycle to %v", life))
 		}
@@ -572,12 +583,63 @@ func (original *Machine) advanceLifecycle(life Life) (err error) {
 		if m.doc.HasVote {
 			return nil, fmt.Errorf("machine %s is a voting replica set member", m.doc.Id)
 		}
-		if len(m.doc.Principals) != 0 {
+		// If there are no alive units left on the machine, or all the services are dying,
+		// then the machine may be soon destroyed by a cleanup worker.
+		// In that case, we don't want to return any error about not being able to
+		// destroy a machine with units as it will be a lie.
+		if life == Dying {
+			canDie := true
+			var principalUnitnames []string
+			for _, principalUnit := range m.doc.Principals {
+				principalUnitnames = append(principalUnitnames, principalUnit)
+				u, err := m.st.Unit(principalUnit)
+				if err != nil {
+					return nil, errors.Annotatef(err, "reading machine %s principal unit %v", m, m.doc.Principals[0])
+				}
+				svc, err := u.Service()
+				if err != nil {
+					return nil, errors.Annotatef(err, "reading machine %s principal unit service %v", m, u.doc.Service)
+				}
+				if u.Life() == Alive && svc.Life() == Alive {
+					canDie = false
+					break
+				}
+			}
+			if canDie {
+				containers, err := m.Containers()
+				if err != nil {
+					return nil, errors.Annotatef(err, "reading machine %s containers", m)
+				}
+				canDie = len(containers) == 0
+			}
+			if canDie {
+				checkUnits := bson.DocElem{
+					"$or", []bson.D{
+						{{"principals", principalUnitnames}},
+						{{"principals", bson.D{{"$size", 0}}}},
+						{{"principals", bson.D{{"$exists", false}}}},
+					},
+				}
+				op.Assert = append(advanceAsserts, checkUnits)
+				containerCheck := txn.Op{
+					C:  containerRefsC,
+					Id: m.doc.DocID,
+					Assert: bson.D{{"$or", []bson.D{
+						{{"children", bson.D{{"$size", 0}}}},
+						{{"children", bson.D{{"$exists", false}}}},
+					}}},
+				}
+				return []txn.Op{op, containerCheck}, nil
+			}
+		}
+		if len(m.doc.Principals) > 0 {
 			return nil, &HasAssignedUnitsError{
 				MachineId: m.doc.Id,
 				UnitNames: m.doc.Principals,
 			}
 		}
+		// Add the additional asserts needed for this transaction.
+		op.Assert = append(advanceAsserts, noUnits)
 		return []txn.Op{op}, nil
 	}
 	if err = m.st.run(buildTxn); err == jujutxn.ErrExcessiveContention {
@@ -655,6 +717,7 @@ func (m *Machine) Remove() (err error) {
 		removeRequestedNetworksOp(m.st, m.globalKey()),
 		annotationRemoveOp(m.st, m.globalKey()),
 		removeRebootDocOp(m.st, m.globalKey()),
+		removeMachineBlockDevicesOp(m.Id()),
 	}
 	ifacesOps, err := m.removeNetworkInterfacesOps()
 	if err != nil {
@@ -664,14 +727,18 @@ func (m *Machine) Remove() (err error) {
 	if err != nil {
 		return err
 	}
-	blockDeviceOps, err := removeMachineBlockDevicesOps(m.st, m.Id())
-	if err != nil {
-		return err
-	}
 	ops = append(ops, ifacesOps...)
 	ops = append(ops, portsOps...)
-	ops = append(ops, blockDeviceOps...)
 	ops = append(ops, removeContainerRefOps(m.st, m.Id())...)
+	ipAddresses, err := m.st.AllocatedIPAddresses(m.Id())
+	if err != nil {
+		return errors.Trace(err)
+	}
+	for _, address := range ipAddresses {
+		logger.Tracef("creating op to set IP addr %q to Dead", address.Value())
+		ops = append(ops, ensureIPAddressDeadOp(address))
+	}
+	logger.Tracef("removing machine %q", m.Id())
 	// The only abort conditions in play indicate that the machine has already
 	// been removed.
 	return onAbort(m.st.runTransaction(ops), nil)
@@ -681,18 +748,14 @@ func (m *Machine) Remove() (err error) {
 // state. It returns an error that satisfies errors.IsNotFound if the
 // machine has been removed.
 func (m *Machine) Refresh() error {
-	machines, closer := m.st.getCollection(machinesC)
-	defer closer()
-
-	var doc machineDoc
-	err := machines.FindId(m.doc.DocID).One(&doc)
-	if err == mgo.ErrNotFound {
-		return errors.NotFoundf("machine %v", m)
-	}
+	mdoc, err := m.st.getMachineDoc(m.Id())
 	if err != nil {
+		if errors.IsNotFound(err) {
+			return err
+		}
 		return errors.Annotatef(err, "cannot refresh machine %v", m)
 	}
-	m.doc = doc
+	m.doc = *mdoc
 	return nil
 }
 
@@ -727,7 +790,7 @@ func (m *Machine) WaitAgentPresence(timeout time.Duration) (err error) {
 // It returns the started pinger.
 func (m *Machine) SetAgentPresence() (*presence.Pinger, error) {
 	presenceCollection := m.st.getPresence()
-	p := presence.NewPinger(presenceCollection, m.globalKey())
+	p := presence.NewPinger(presenceCollection, m.st.environTag, m.globalKey())
 	err := p.Start()
 	if err != nil {
 		return nil, err
@@ -904,7 +967,10 @@ func (m *Machine) SetProvisioned(id instance.Id, nonce string, characteristics *
 // Merge SetProvisioned() in here or drop it at that point.
 func (m *Machine) SetInstanceInfo(
 	id instance.Id, nonce string, characteristics *instance.HardwareCharacteristics,
-	networks []NetworkInfo, interfaces []NetworkInterfaceInfo) error {
+	networks []NetworkInfo, interfaces []NetworkInterfaceInfo,
+	volumes map[names.VolumeTag]VolumeInfo,
+	volumeAttachments map[names.VolumeTag]VolumeAttachmentInfo,
+) error {
 
 	// Add the networks and interfaces first.
 	for _, network := range networks {
@@ -925,6 +991,12 @@ func (m *Machine) SetInstanceInfo(
 			return errors.Trace(err)
 		}
 	}
+	if err := setProvisionedVolumeInfo(m.st, volumes); err != nil {
+		return errors.Trace(err)
+	}
+	if err := setMachineVolumeAttachmentInfo(m.st, m.Id(), volumeAttachments); err != nil {
+		return errors.Trace(err)
+	}
 	return m.SetProvisioned(id, nonce, characteristics)
 }
 
@@ -937,11 +1009,11 @@ func mergedAddresses(machineAddresses, providerAddresses []address) []network.Ad
 			continue
 		}
 		providerValues.Add(address.Value)
-		merged = append(merged, address.InstanceAddress())
+		merged = append(merged, address.networkAddress())
 	}
 	for _, address := range machineAddresses {
 		if !providerValues.Contains(address.Value) {
-			merged = append(merged, address.InstanceAddress())
+			merged = append(merged, address.networkAddress())
 		}
 	}
 	return merged
@@ -958,20 +1030,34 @@ func (m *Machine) Addresses() (addresses []network.Address) {
 	return mergedAddresses(m.doc.MachineAddresses, m.doc.Addresses)
 }
 
-// SetAddresses records any addresses related to the machine, sourced
+// SetProviderAddresses records any addresses related to the machine, sourced
 // by asking the provider.
-func (m *Machine) SetAddresses(addresses ...network.Address) (err error) {
-	if err = m.setAddresses(addresses, &m.doc.Addresses, "addresses"); err != nil {
+func (m *Machine) SetProviderAddresses(addresses ...network.Address) (err error) {
+	mdoc, err := m.st.getMachineDoc(m.Id())
+	if err != nil {
+		return errors.Annotatef(err, "cannot refresh provider addresses for machine %s", m)
+	}
+	if err = m.setAddresses(addresses, &mdoc.Addresses, "addresses"); err != nil {
 		return fmt.Errorf("cannot set addresses of machine %v: %v", m, err)
 	}
+	m.doc.Addresses = mdoc.Addresses
 	return nil
 }
 
+// ProviderAddresses returns any hostnames and ips associated with a machine,
+// as determined by asking the provider.
+func (m *Machine) ProviderAddresses() (addresses []network.Address) {
+	for _, address := range m.doc.Addresses {
+		addresses = append(addresses, address.networkAddress())
+	}
+	return
+}
+
 // MachineAddresses returns any hostnames and ips associated with a machine,
 // determined by asking the machine itself.
 func (m *Machine) MachineAddresses() (addresses []network.Address) {
 	for _, address := range m.doc.MachineAddresses {
-		addresses = append(addresses, address.InstanceAddress())
+		addresses = append(addresses, address.networkAddress())
 	}
 	return
 }
@@ -979,53 +1065,76 @@ func (m *Machine) MachineAddresses() (addresses []network.Address) {
 // SetMachineAddresses records any addresses related to the machine, sourced
 // by asking the machine.
 func (m *Machine) SetMachineAddresses(addresses ...network.Address) (err error) {
-	if err = m.setAddresses(addresses, &m.doc.MachineAddresses, "machineaddresses"); err != nil {
+	mdoc, err := m.st.getMachineDoc(m.Id())
+	if err != nil {
+		return errors.Annotatef(err, "cannot refresh machine addresses for machine %s", m)
+	}
+	if err = m.setAddresses(addresses, &mdoc.MachineAddresses, "machineaddresses"); err != nil {
 		return fmt.Errorf("cannot set machine addresses of machine %v: %v", m, err)
 	}
+	m.doc.MachineAddresses = mdoc.MachineAddresses
 	return nil
 }
 
 // setAddresses updates the machine's addresses (either Addresses or
-// MachineAddresses, depending on the field argument).
+// MachineAddresses, depending on the field argument). Changes are
+// only predicated on the machine not being Dead; concurrent address
+// changes are ignored.
 func (m *Machine) setAddresses(addresses []network.Address, field *[]address, fieldName string) error {
-	var changed bool
+	var addressesToSet []network.Address
+	if !m.IsContainer() {
+		// Check addresses first. We'll only add those addresses
+		// which are not in the IP address collection.
+		ipAddresses, closer := m.st.getCollection(ipaddressesC)
+		defer closer()
+
+		addressValues := make([]string, len(addresses))
+		for i, address := range addresses {
+			addressValues[i] = address.Value
+		}
+		ipDocs := []ipaddressDoc{}
+		sel := bson.D{{"value", bson.D{{"$in", addressValues}}}, {"state", AddressStateAllocated}}
+		err := ipAddresses.Find(sel).All(&ipDocs)
+		if err != nil {
+			return err
+		}
+		ipDocValues := set.NewStrings()
+		for _, ipDoc := range ipDocs {
+			ipDocValues.Add(ipDoc.Value)
+		}
+		for _, address := range addresses {
+			if !ipDocValues.Contains(address.Value) {
+				addressesToSet = append(addressesToSet, address)
+			}
+		}
+	} else {
+		// Containers will set all addresses.
+		addressesToSet = make([]network.Address, len(addresses))
+		copy(addressesToSet, addresses)
+	}
+
+	// Update addresses now.
 	envConfig, err := m.st.EnvironConfig()
 	if err != nil {
 		return err
 	}
-	network.SortAddresses(addresses, envConfig.PreferIPv6())
-	stateAddresses := instanceAddressesToAddresses(addresses)
-	buildTxn := func(attempt int) ([]txn.Op, error) {
-		changed = false
-		if attempt > 0 {
-			if err := m.Refresh(); err != nil {
-				return nil, err
-			}
-		}
-		if m.doc.Life == Dead {
-			return nil, ErrDead
-		}
-		op := txn.Op{
-			C:      machinesC,
-			Id:     m.doc.DocID,
-			Assert: append(bson.D{{fieldName, *field}}, notDeadDoc...),
-		}
-		if !addressesEqual(addresses, addressesToInstanceAddresses(*field)) {
-			op.Update = bson.D{{"$set", bson.D{{fieldName, stateAddresses}}}}
-			changed = true
-		}
-		return []txn.Op{op}, nil
-	}
-	switch err := m.st.run(buildTxn); err {
-	case nil:
-	case jujutxn.ErrExcessiveContention:
-		return errors.Annotatef(err, "cannot set %s for machine %s", fieldName, m)
-	default:
-		return err
-	}
-	if !changed {
+	network.SortAddresses(addressesToSet, envConfig.PreferIPv6())
+	stateAddresses := fromNetworkAddresses(addressesToSet)
+
+	if addressesEqual(addressesToSet, networkAddresses(*field)) {
 		return nil
 	}
+	if err := m.st.runTransaction([]txn.Op{{
+		C:      machinesC,
+		Id:     m.doc.DocID,
+		Assert: notDeadDoc,
+		Update: bson.D{{"$set", bson.D{{fieldName, stateAddresses}}}},
+	}}); err != nil {
+		if err == txn.ErrAborted {
+			return ErrDead
+		}
+		return errors.Trace(err)
+	}
 	*field = stateAddresses
 	return nil
 }
@@ -1098,20 +1207,23 @@ func (m *Machine) AddNetworkInterface(args NetworkInterfaceInfo) (iface *Network
 		return nil, fmt.Errorf("interface name must be not empty")
 	}
 	doc := newNetworkInterfaceDoc(m.doc.Id, m.st.EnvironUUID(), args)
-	ops := []txn.Op{{
-		C:      networksC,
-		Id:     m.st.docID(args.NetworkName),
-		Assert: txn.DocExists,
-	}, {
-		C:      machinesC,
-		Id:     m.doc.DocID,
-		Assert: isAliveDoc,
-	}, {
-		C:      networkInterfacesC,
-		Id:     doc.Id,
-		Assert: txn.DocMissing,
-		Insert: doc,
-	}}
+	ops := []txn.Op{
+		assertEnvAliveOp(m.st.EnvironUUID()),
+		{
+			C:      networksC,
+			Id:     m.st.docID(args.NetworkName),
+			Assert: txn.DocExists,
+		}, {
+			C:      machinesC,
+			Id:     m.doc.DocID,
+			Assert: isAliveDoc,
+		}, {
+			C:      networkInterfacesC,
+			Id:     doc.Id,
+			Assert: txn.DocMissing,
+			Insert: doc,
+		},
+	}
 
 	err = m.st.runTransaction(ops)
 	switch err {
@@ -1223,38 +1335,39 @@ func (m *Machine) SetConstraints(cons constraints.Value) (err error) {
 }
 
 // Status returns the status of the machine.
-func (m *Machine) Status() (status Status, info string, data map[string]interface{}, err error) {
-	doc, err := getStatus(m.st, m.globalKey())
-	if err != nil {
-		return "", "", nil, err
-	}
-	status = doc.Status
-	info = doc.StatusInfo
-	data = doc.StatusData
-	return
+func (m *Machine) Status() (StatusInfo, error) {
+	return getStatus(m.st, m.globalKey(), "machine")
 }
 
 // SetStatus sets the status of the machine.
 func (m *Machine) SetStatus(status Status, info string, data map[string]interface{}) error {
-	// If a machine is not yet provisioned, we allow its status
-	// to be set back to pending (when a retry is to occur).
-	_, err := m.InstanceId()
-	allowPending := errors.IsNotProvisioned(err)
-	doc, err := newMachineStatusDoc(status, info, data, allowPending)
-	if err != nil {
-		return err
+	switch status {
+	case StatusStarted, StatusStopped:
+	case StatusError:
+		if info == "" {
+			return errors.Errorf("cannot set status %q without info", status)
+		}
+	case StatusPending:
+		// If a machine is not yet provisioned, we allow its status
+		// to be set back to pending (when a retry is to occur).
+		_, err := m.InstanceId()
+		allowPending := errors.IsNotProvisioned(err)
+		if allowPending {
+			break
+		}
+		fallthrough
+	case StatusDown:
+		return errors.Errorf("cannot set status %q", status)
+	default:
+		return errors.Errorf("cannot set invalid status %q", status)
 	}
-	ops := []txn.Op{{
-		C:      machinesC,
-		Id:     m.doc.DocID,
-		Assert: notDeadDoc,
-	},
-		updateStatusOp(m.st, m.globalKey(), doc.statusDoc),
-	}
-	if err = m.st.runTransaction(ops); err != nil {
-		return fmt.Errorf("cannot set status of machine %q: %v", m, onAbort(err, errNotAlive))
-	}
-	return nil
+	return setStatus(m.st, setStatusParams{
+		badge:     "machine",
+		globalKey: m.globalKey(),
+		status:    status,
+		message:   info,
+		rawData:   data,
+	})
 }
 
 // Clean returns true if the machine does not have any deployed units or containers.
@@ -1341,17 +1454,17 @@ func (m *Machine) markInvalidContainers() error {
 			}
 			// There should never be a circumstance where an unsupported container is started.
 			// Nonetheless, we check and log an error if such a situation arises.
-			status, _, _, err := container.Status()
+			statusInfo, err := container.Status()
 			if err != nil {
 				logger.Errorf("finding status of container %v to mark as invalid: %v", containerId, err)
 				continue
 			}
-			if status == StatusPending {
+			if statusInfo.Status == StatusPending {
 				containerType := ContainerTypeFromId(containerId)
 				container.SetStatus(
 					StatusError, "unsupported container", map[string]interface{}{"type": containerType})
 			} else {
-				logger.Errorf("unsupported container %v has unexpected status %v", containerId, status)
+				logger.Errorf("unsupported container %v has unexpected status %v", containerId, statusInfo.Status)
 			}
 		}
 	}
@@ -1363,16 +1476,7 @@ func (m *Machine) SetMachineBlockDevices(info ...BlockDeviceInfo) error {
 	return setMachineBlockDevices(m.st, m.Id(), info)
 }
 
-// BlockDevices gets the aggregated list of block devices attached to the
-// machine.
-func (m *Machine) BlockDevices() ([]BlockDevice, error) {
-	devices, err := getMachineBlockDevices(m.st, m.Id())
-	if err != nil {
-		return nil, err
-	}
-	result := make([]BlockDevice, len(devices))
-	for i, dev := range devices {
-		result[i] = dev
-	}
-	return result, nil
+// VolumeAttachments returns the machine's volume attachments.
+func (m *Machine) VolumeAttachments() ([]VolumeAttachment, error) {
+	return m.st.MachineVolumeAttachments(m.MachineTag())
 }
