@@ -9,11 +9,13 @@ import (
 
 	"github.com/juju/errors"
 	"github.com/juju/loggo"
+	"github.com/juju/names"
 	jc "github.com/juju/testing/checkers"
-	"github.com/juju/txn"
+	jujutxn "github.com/juju/txn"
 	gc "gopkg.in/check.v1"
 	"gopkg.in/mgo.v2"
 	"gopkg.in/mgo.v2/bson"
+	"gopkg.in/mgo.v2/txn"
 
 	"github.com/juju/juju/constraints"
 	"github.com/juju/juju/environs/config"
@@ -21,6 +23,9 @@ import (
 	"github.com/juju/juju/network"
 	"github.com/juju/juju/state"
 	"github.com/juju/juju/state/testing"
+	"github.com/juju/juju/storage/poolmanager"
+	"github.com/juju/juju/storage/provider"
+	"github.com/juju/juju/storage/provider/registry"
 	coretesting "github.com/juju/juju/testing"
 	"github.com/juju/juju/version"
 )
@@ -68,7 +73,7 @@ func (s *MachineSuite) TestSetRebootFlagDeadMachine(c *gc.C) {
 }
 
 func (s *MachineSuite) TestSetRebootFlagDeadMachineRace(c *gc.C) {
-	setFlag := txn.TestHook{
+	setFlag := jujutxn.TestHook{
 		Before: func() {
 			err := s.machine.EnsureDead()
 			c.Assert(err, jc.ErrorIsNil)
@@ -222,6 +227,21 @@ func (s *MachineSuite) TestMachineIsManual(c *gc.C) {
 	}
 }
 
+func (s *MachineSuite) TestMachineIsContainer(c *gc.C) {
+	machine, err := s.State.AddMachine("quantal", state.JobHostUnits)
+	c.Assert(err, jc.ErrorIsNil)
+
+	template := state.MachineTemplate{
+		Series: "quantal",
+		Jobs:   []state.MachineJob{state.JobHostUnits},
+	}
+	container, err := s.State.AddMachineInsideMachine(template, machine.Id(), instance.LXC)
+	c.Assert(err, jc.ErrorIsNil)
+
+	c.Assert(machine.IsContainer(), jc.IsFalse)
+	c.Assert(container.IsContainer(), jc.IsTrue)
+}
+
 func (s *MachineSuite) TestLifeJobManageEnviron(c *gc.C) {
 	// A JobManageEnviron machine must never advance lifecycle.
 	m := s.machine0
@@ -256,7 +276,7 @@ func (s *MachineSuite) TestLifeJobHostUnits(c *gc.C) {
 	err = unit.AssignToMachine(s.machine)
 	c.Assert(err, jc.ErrorIsNil)
 	err = s.machine.Destroy()
-	c.Assert(err, gc.FitsTypeOf, &state.HasAssignedUnitsError{})
+	c.Assert(err, jc.Satisfies, state.IsHasAssignedUnitsError)
 	c.Assert(err, gc.ErrorMatches, `machine 1 has unit "wordpress/0" assigned`)
 	err1 := s.machine.EnsureDead()
 	c.Assert(err1, gc.DeepEquals, err)
@@ -325,7 +345,7 @@ func (s *MachineSuite) TestDestroyCancel(c *gc.C) {
 		c.Assert(unit.AssignToMachine(s.machine), gc.IsNil)
 	}).Check()
 	err = s.machine.Destroy()
-	c.Assert(err, gc.FitsTypeOf, &state.HasAssignedUnitsError{})
+	c.Assert(err, jc.Satisfies, state.IsHasAssignedUnitsError)
 }
 
 func (s *MachineSuite) TestDestroyContention(c *gc.C) {
@@ -333,7 +353,7 @@ func (s *MachineSuite) TestDestroyContention(c *gc.C) {
 	unit, err := svc.AddUnit()
 	c.Assert(err, jc.ErrorIsNil)
 
-	perturb := txn.TestHook{
+	perturb := jujutxn.TestHook{
 		Before: func() { c.Assert(unit.AssignToMachine(s.machine), gc.IsNil) },
 		After:  func() { c.Assert(unit.UnassignFromMachine(), gc.IsNil) },
 	}
@@ -343,6 +363,86 @@ func (s *MachineSuite) TestDestroyContention(c *gc.C) {
 	c.Assert(err, gc.ErrorMatches, "machine 1 cannot advance lifecycle: state changing too quickly; try again soon")
 }
 
+func (s *MachineSuite) TestDestroyWithServiceDestroyPending(c *gc.C) {
+	svc := s.AddTestingService(c, "wordpress", s.AddTestingCharm(c, "wordpress"))
+	unit, err := svc.AddUnit()
+	c.Assert(err, jc.ErrorIsNil)
+	err = unit.AssignToMachine(s.machine)
+	c.Assert(err, jc.ErrorIsNil)
+
+	err = svc.Destroy()
+	c.Assert(err, jc.ErrorIsNil)
+	err = s.machine.Destroy()
+	c.Assert(err, jc.ErrorIsNil)
+	// Machine is still advanced to Dying.
+	life := s.machine.Life()
+	c.Assert(life, gc.Equals, state.Dying)
+}
+
+func (s *MachineSuite) TestDestroyFailsWhenNewUnitAdded(c *gc.C) {
+	svc := s.AddTestingService(c, "wordpress", s.AddTestingCharm(c, "wordpress"))
+	unit, err := svc.AddUnit()
+	c.Assert(err, jc.ErrorIsNil)
+	err = unit.AssignToMachine(s.machine)
+	c.Assert(err, jc.ErrorIsNil)
+
+	err = svc.Destroy()
+	c.Assert(err, jc.ErrorIsNil)
+
+	defer state.SetBeforeHooks(c, s.State, func() {
+		anotherSvc := s.AddTestingService(c, "mysql", s.AddTestingCharm(c, "mysql"))
+		anotherUnit, err := anotherSvc.AddUnit()
+		c.Assert(err, jc.ErrorIsNil)
+		err = anotherUnit.AssignToMachine(s.machine)
+		c.Assert(err, jc.ErrorIsNil)
+	}).Check()
+
+	err = s.machine.Destroy()
+	c.Assert(err, jc.Satisfies, state.IsHasAssignedUnitsError)
+	life := s.machine.Life()
+	c.Assert(life, gc.Equals, state.Alive)
+}
+
+func (s *MachineSuite) TestDestroyWithUnitDestroyPending(c *gc.C) {
+	svc := s.AddTestingService(c, "wordpress", s.AddTestingCharm(c, "wordpress"))
+	unit, err := svc.AddUnit()
+	c.Assert(err, jc.ErrorIsNil)
+	err = unit.AssignToMachine(s.machine)
+	c.Assert(err, jc.ErrorIsNil)
+
+	err = unit.Destroy()
+	c.Assert(err, jc.ErrorIsNil)
+	err = s.machine.Destroy()
+	c.Assert(err, jc.ErrorIsNil)
+	// Machine is still advanced to Dying.
+	life := s.machine.Life()
+	c.Assert(life, gc.Equals, state.Dying)
+}
+
+func (s *MachineSuite) TestDestroyFailsWhenNewContainerAdded(c *gc.C) {
+	svc := s.AddTestingService(c, "wordpress", s.AddTestingCharm(c, "wordpress"))
+	unit, err := svc.AddUnit()
+	c.Assert(err, jc.ErrorIsNil)
+	err = unit.AssignToMachine(s.machine)
+	c.Assert(err, jc.ErrorIsNil)
+
+	err = svc.Destroy()
+	c.Assert(err, jc.ErrorIsNil)
+
+	defer state.SetBeforeHooks(c, s.State, func() {
+		_, err := s.State.AddMachineInsideMachine(state.MachineTemplate{
+			Series: "quantal",
+			Jobs:   []state.MachineJob{state.JobHostUnits},
+		}, s.machine.Id(), instance.LXC)
+		c.Assert(err, jc.ErrorIsNil)
+	}).Check()
+
+	err = s.machine.Destroy()
+	c.Assert(err, jc.Satisfies, state.IsHasAssignedUnitsError)
+	life := s.machine.Life()
+	c.Assert(life, gc.Equals, state.Alive)
+}
+
 func (s *MachineSuite) TestRemove(c *gc.C) {
 	err := s.machine.Remove()
 	c.Assert(err, gc.ErrorMatches, "cannot remove machine 1: machine is not dead")
@@ -366,6 +466,51 @@ func (s *MachineSuite) TestRemove(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 }
 
+func (s *MachineSuite) TestRemoveMarksAddressesAsDead(c *gc.C) {
+	err := s.machine.SetProvisioned("fake", "totally-fake", nil)
+	c.Assert(err, jc.ErrorIsNil)
+
+	addr1, err := s.State.AddIPAddress(network.NewAddress("10.0.0.1"), "foo")
+	c.Assert(err, jc.ErrorIsNil)
+	err = addr1.AllocateTo(s.machine.Id(), "bar")
+	c.Assert(err, jc.ErrorIsNil)
+
+	addr2, err := s.State.AddIPAddress(network.NewAddress("10.0.0.2"), "foo")
+	c.Assert(err, jc.ErrorIsNil)
+	err = addr2.AllocateTo(s.machine.Id(), "bar")
+	c.Assert(err, jc.ErrorIsNil)
+
+	addr3, err := s.State.AddIPAddress(network.NewAddress("10.0.0.3"), "bar")
+	c.Assert(err, jc.ErrorIsNil)
+	err = addr3.AllocateTo(s.machine0.Id(), "bar")
+	c.Assert(err, jc.ErrorIsNil)
+
+	addr4, err := s.State.AddIPAddress(network.NewAddress("10.0.0.4"), "foo")
+	c.Assert(err, jc.ErrorIsNil)
+	err = addr4.AllocateTo(s.machine.Id(), "bar")
+	c.Assert(err, jc.ErrorIsNil)
+	err = addr4.EnsureDead()
+	c.Assert(err, jc.ErrorIsNil)
+
+	err = s.machine.EnsureDead()
+	c.Assert(err, jc.ErrorIsNil)
+	err = s.machine.Remove()
+	c.Assert(err, jc.ErrorIsNil)
+
+	err = addr1.Refresh()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(addr1.Life(), gc.Equals, state.Dead)
+	err = addr2.Refresh()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(addr2.Life(), gc.Equals, state.Dead)
+	err = addr3.Refresh()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(addr3.Life(), gc.Equals, state.Alive)
+	err = addr4.Refresh()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(addr4.Life(), gc.Equals, state.Dead)
+}
+
 func (s *MachineSuite) TestHasVote(c *gc.C) {
 	c.Assert(s.machine.HasVote(), jc.IsFalse)
 
@@ -437,14 +582,27 @@ func (s *MachineSuite) TestMachineSetAgentPresence(c *gc.C) {
 }
 
 func (s *MachineSuite) TestTag(c *gc.C) {
-	c.Assert(s.machine.Tag().String(), gc.Equals, "machine-1")
+	tag := s.machine.MachineTag()
+	c.Assert(tag.Kind(), gc.Equals, names.MachineTagKind)
+	c.Assert(tag.Id(), gc.Equals, "1")
+
+	// To keep gccgo happy, don't compare an interface with a struct.
+	var asTag names.Tag = tag
+	c.Assert(s.machine.Tag(), gc.Equals, asTag)
 }
 
 func (s *MachineSuite) TestSetMongoPassword(c *gc.C) {
-	info := state.TestingMongoInfo()
-	st, err := state.Open(info, state.TestingDialOpts(), state.Policy(nil))
+	info := testing.NewMongoInfo()
+	st, err := state.Open(s.envTag, info, testing.NewDialOpts(), state.Policy(nil))
 	c.Assert(err, jc.ErrorIsNil)
-	defer st.Close()
+	defer func() {
+		// Remove the admin password so that the test harness can reset the state.
+		err := st.SetAdminMongoPassword("")
+		c.Check(err, jc.ErrorIsNil)
+		err = st.Close()
+		c.Check(err, jc.ErrorIsNil)
+	}()
+
 	// Turn on fully-authenticated mode.
 	err = st.SetAdminMongoPassword("admin-secret")
 	c.Assert(err, jc.ErrorIsNil)
@@ -460,12 +618,13 @@ func (s *MachineSuite) TestSetMongoPassword(c *gc.C) {
 	// Check that we cannot log in with the wrong password.
 	info.Tag = ent.Tag()
 	info.Password = "bar"
-	err = tryOpenState(info)
-	c.Assert(err, jc.Satisfies, errors.IsUnauthorized)
+	err = tryOpenState(s.envTag, info)
+	c.Check(errors.Cause(err), jc.Satisfies, errors.IsUnauthorized)
+	c.Check(err, gc.ErrorMatches, `cannot log in to admin database as "machine-0": unauthorized mongo access: .*`)
 
 	// Check that we can log in with the correct password.
 	info.Password = "foo"
-	st1, err := state.Open(info, state.TestingDialOpts(), state.Policy(nil))
+	st1, err := state.Open(s.envTag, info, testing.NewDialOpts(), state.Policy(nil))
 	c.Assert(err, jc.ErrorIsNil)
 	defer st1.Close()
 
@@ -478,21 +637,18 @@ func (s *MachineSuite) TestSetMongoPassword(c *gc.C) {
 
 	// Check that we cannot log in with the old password.
 	info.Password = "foo"
-	err = tryOpenState(info)
-	c.Assert(err, jc.Satisfies, errors.IsUnauthorized)
+	err = tryOpenState(s.envTag, info)
+	c.Check(errors.Cause(err), jc.Satisfies, errors.IsUnauthorized)
+	c.Check(err, gc.ErrorMatches, `cannot log in to admin database as "machine-0": unauthorized mongo access: .*`)
 
 	// Check that we can log in with the correct password.
 	info.Password = "bar"
-	err = tryOpenState(info)
+	err = tryOpenState(s.envTag, info)
 	c.Assert(err, jc.ErrorIsNil)
 
 	// Check that the administrator can still log in.
 	info.Tag, info.Password = nil, "admin-secret"
-	err = tryOpenState(info)
-	c.Assert(err, jc.ErrorIsNil)
-
-	// Remove the admin password so that the test harness can reset the state.
-	err = st.SetAdminMongoPassword("")
+	err = tryOpenState(s.envTag, info)
 	c.Assert(err, jc.ErrorIsNil)
 }
 
@@ -502,6 +658,30 @@ func (s *MachineSuite) TestSetPassword(c *gc.C) {
 	})
 }
 
+func (s *MachineSuite) TestSetPasswordPreEnvUUID(c *gc.C) {
+	// Ensure that SetPassword works for machines even when the env
+	// UUID upgrade migrations haven't run yet.
+	type oldMachineDoc struct {
+		Id     string `bson:"_id"`
+		Series string
+	}
+	s.machines.Insert(&oldMachineDoc{"99", "quantal"})
+
+	m, err := s.State.Machine("99")
+	c.Assert(err, jc.ErrorIsNil)
+
+	// Set the password and make sure it sticks.
+	c.Assert(m.PasswordValid(goodPassword), jc.IsFalse)
+	err = m.SetPassword(goodPassword)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(m.PasswordValid(goodPassword), jc.IsTrue)
+
+	// Check a newly-fetched entity has the same password.
+	err = m.Refresh()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(m.PasswordValid(goodPassword), jc.IsTrue)
+}
+
 func (s *MachineSuite) TestSetAgentCompatPassword(c *gc.C) {
 	e, err := s.State.Machine(s.machine.Id())
 	c.Assert(err, jc.ErrorIsNil)
@@ -875,18 +1055,47 @@ func (s *MachineSuite) TestMachineSetCheckProvisioned(c *gc.C) {
 }
 
 func (s *MachineSuite) TestMachineSetInstanceInfoFailureDoesNotProvision(c *gc.C) {
-	c.Assert(s.machine.CheckProvisioned("fake_nonce"), jc.IsFalse)
+	assertNotProvisioned := func() {
+		c.Assert(s.machine.CheckProvisioned("fake_nonce"), jc.IsFalse)
+	}
+
+	assertNotProvisioned()
 	invalidNetworks := []state.NetworkInfo{{Name: ""}}
-	invalidInterfaces := []state.NetworkInterfaceInfo{{MACAddress: ""}}
-	err := s.machine.SetInstanceInfo("umbrella/0", "fake_nonce", nil, invalidNetworks, nil)
+	err := s.machine.SetInstanceInfo("umbrella/0", "fake_nonce", nil, invalidNetworks, nil, nil, nil)
 	c.Assert(err, gc.ErrorMatches, `cannot add network "": name must be not empty`)
-	c.Assert(s.machine.CheckProvisioned("fake_nonce"), jc.IsFalse)
-	err = s.machine.SetInstanceInfo("umbrella/0", "fake_nonce", nil, nil, invalidInterfaces)
+	assertNotProvisioned()
+
+	invalidInterfaces := []state.NetworkInterfaceInfo{{MACAddress: ""}}
+	err = s.machine.SetInstanceInfo("umbrella/0", "fake_nonce", nil, nil, invalidInterfaces, nil, nil)
 	c.Assert(err, gc.ErrorMatches, `cannot add network interface "" to machine "1": MAC address must be not empty`)
-	c.Assert(s.machine.CheckProvisioned("fake_nonce"), jc.IsFalse)
+	assertNotProvisioned()
+
+	invalidVolumes := map[names.VolumeTag]state.VolumeInfo{names.NewVolumeTag("1065"): state.VolumeInfo{}}
+	err = s.machine.SetInstanceInfo("umbrella/0", "fake_nonce", nil, nil, nil, invalidVolumes, nil)
+	c.Assert(err, gc.ErrorMatches, `cannot set info for volume \"1065\": volume \"1065\" not found`)
+	assertNotProvisioned()
+
+	// TODO(axw) test invalid volume attachment
+}
+
+func (s *MachineSuite) addVolume(c *gc.C, params state.VolumeParams, machineId string) names.VolumeTag {
+	op, tag, err := state.AddVolumeOp(s.State, params, machineId)
+	c.Assert(err, jc.ErrorIsNil)
+	err = state.RunTransaction(s.State, []txn.Op{op})
+	c.Assert(err, jc.ErrorIsNil)
+	return tag
 }
 
 func (s *MachineSuite) TestMachineSetInstanceInfoSuccess(c *gc.C) {
+	pm := poolmanager.New(state.NewStateSettings(s.State))
+	_, err := pm.Create("loop-pool", provider.LoopProviderType, map[string]interface{}{})
+	c.Assert(err, jc.ErrorIsNil)
+	registry.RegisterEnvironStorageProviders("someprovider", provider.LoopProviderType)
+
+	// Must create the requested block device prior to SetInstanceInfo.
+	volumeTag := s.addVolume(c, state.VolumeParams{Size: 1000, Pool: "loop-pool"}, "123")
+	c.Assert(volumeTag, gc.Equals, names.NewVolumeTag("123/0"))
+
 	c.Assert(s.machine.CheckProvisioned("fake_nonce"), jc.IsFalse)
 	networks := []state.NetworkInfo{
 		{Name: "net1", ProviderId: "net1", CIDR: "0.1.2.0/24", VLANTag: 0},
@@ -894,7 +1103,12 @@ func (s *MachineSuite) TestMachineSetInstanceInfoSuccess(c *gc.C) {
 	interfaces := []state.NetworkInterfaceInfo{
 		{MACAddress: "aa:bb:cc:dd:ee:ff", NetworkName: "net1", InterfaceName: "eth0", IsVirtual: false},
 	}
-	err := s.machine.SetInstanceInfo("umbrella/0", "fake_nonce", nil, networks, interfaces)
+	volumeInfo := state.VolumeInfo{
+		VolumeId: "storage-123",
+		Size:     1234,
+	}
+	volumes := map[names.VolumeTag]state.VolumeInfo{volumeTag: volumeInfo}
+	err = s.machine.SetInstanceInfo("umbrella/0", "fake_nonce", nil, networks, interfaces, volumes, nil)
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(s.machine.CheckProvisioned("fake_nonce"), jc.IsTrue)
 	network, err := s.State.Network(networks[0].Name)
@@ -911,6 +1125,13 @@ func (s *MachineSuite) TestMachineSetInstanceInfoSuccess(c *gc.C) {
 	c.Check(ifaces[0].MACAddress(), gc.Equals, interfaces[0].MACAddress)
 	c.Check(ifaces[0].MachineTag(), gc.Equals, s.machine.Tag())
 	c.Check(ifaces[0].IsVirtual(), gc.Equals, interfaces[0].IsVirtual)
+
+	volume, err := s.State.Volume(volumeTag)
+	c.Assert(err, jc.ErrorIsNil)
+	info, err := volume.Info()
+	c.Assert(err, jc.ErrorIsNil)
+	volumeInfo.Pool = "loop-pool" // taken from params
+	c.Assert(info, gc.Equals, volumeInfo)
 }
 
 func (s *MachineSuite) TestMachineSetProvisionedWhenNotAlive(c *gc.C) {
@@ -1142,7 +1363,7 @@ func (s *MachineSuite) TestWatchDiesOnStateClose(c *gc.C) {
 	//  Unit.Watch
 	//  State.WatchForEnvironConfigChanges
 	//  Unit.WatchConfigSettings
-	testWatcherDiesWhenStateCloses(c, func(c *gc.C, st *state.State) waiter {
+	testWatcherDiesWhenStateCloses(c, s.envTag, func(c *gc.C, st *state.State) waiter {
 		m, err := st.Machine(s.machine.Id())
 		c.Assert(err, jc.ErrorIsNil)
 		w := m.Watch()
@@ -1180,7 +1401,7 @@ func (s *MachineSuite) TestWatchPrincipalUnits(c *gc.C) {
 	wc.AssertNoChange()
 
 	// Change the unit; no change.
-	err = mysql0.SetStatus(state.StatusActive, "", nil)
+	err = mysql0.SetAgentStatus(state.StatusIdle, "", nil)
 	c.Assert(err, jc.ErrorIsNil)
 	wc.AssertNoChange()
 
@@ -1209,7 +1430,7 @@ func (s *MachineSuite) TestWatchPrincipalUnits(c *gc.C) {
 	wc.AssertNoChange()
 
 	// Change the subordinate; no change.
-	err = logging0.SetStatus(state.StatusActive, "", nil)
+	err = logging0.SetAgentStatus(state.StatusIdle, "", nil)
 	c.Assert(err, jc.ErrorIsNil)
 	wc.AssertNoChange()
 
@@ -1250,7 +1471,7 @@ func (s *MachineSuite) TestWatchPrincipalUnits(c *gc.C) {
 func (s *MachineSuite) TestWatchPrincipalUnitsDiesOnStateClose(c *gc.C) {
 	// This test is testing logic in watcher.unitsWatcher, which
 	// is also used by Unit.WatchSubordinateUnits.
-	testWatcherDiesWhenStateCloses(c, func(c *gc.C, st *state.State) waiter {
+	testWatcherDiesWhenStateCloses(c, s.envTag, func(c *gc.C, st *state.State) waiter {
 		m, err := st.Machine(s.machine.Id())
 		c.Assert(err, jc.ErrorIsNil)
 		w := m.WatchPrincipalUnits()
@@ -1284,7 +1505,7 @@ func (s *MachineSuite) TestWatchUnits(c *gc.C) {
 	wc.AssertNoChange()
 
 	// Change the unit; no change.
-	err = mysql0.SetStatus(state.StatusActive, "", nil)
+	err = mysql0.SetAgentStatus(state.StatusIdle, "", nil)
 	c.Assert(err, jc.ErrorIsNil)
 	wc.AssertNoChange()
 
@@ -1314,7 +1535,7 @@ func (s *MachineSuite) TestWatchUnits(c *gc.C) {
 	wc.AssertNoChange()
 
 	// Change the subordinate; no change.
-	err = logging0.SetStatus(state.StatusActive, "", nil)
+	err = logging0.SetAgentStatus(state.StatusIdle, "", nil)
 	c.Assert(err, jc.ErrorIsNil)
 	wc.AssertNoChange()
 
@@ -1354,7 +1575,7 @@ func (s *MachineSuite) TestWatchUnits(c *gc.C) {
 }
 
 func (s *MachineSuite) TestWatchUnitsDiesOnStateClose(c *gc.C) {
-	testWatcherDiesWhenStateCloses(c, func(c *gc.C, st *state.State) waiter {
+	testWatcherDiesWhenStateCloses(c, s.envTag, func(c *gc.C, st *state.State) waiter {
 		m, err := st.Machine(s.machine.Id())
 		c.Assert(err, jc.ErrorIsNil)
 		w := m.WatchUnits()
@@ -1363,25 +1584,6 @@ func (s *MachineSuite) TestWatchUnitsDiesOnStateClose(c *gc.C) {
 	})
 }
 
-func (s *MachineSuite) TestAnnotatorForMachine(c *gc.C) {
-	testAnnotator(c, func() (state.Annotator, error) {
-		return s.State.Machine(s.machine.Id())
-	})
-}
-
-func (s *MachineSuite) TestAnnotationRemovalForMachine(c *gc.C) {
-	annotations := map[string]string{"mykey": "myvalue"}
-	err := s.machine.SetAnnotations(annotations)
-	c.Assert(err, jc.ErrorIsNil)
-	err = s.machine.EnsureDead()
-	c.Assert(err, jc.ErrorIsNil)
-	err = s.machine.Remove()
-	c.Assert(err, jc.ErrorIsNil)
-	ann, err := s.machine.Annotations()
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(ann, gc.DeepEquals, make(map[string]string))
-}
-
 func (s *MachineSuite) TestConstraintsFromEnvironment(c *gc.C) {
 	econs1 := constraints.MustParse("mem=1G")
 	econs2 := constraints.MustParse("mem=2G")
@@ -1483,183 +1685,97 @@ func (s *MachineSuite) TestConstraintsLifecycle(c *gc.C) {
 	c.Assert(err, gc.ErrorMatches, `constraints not found`)
 }
 
-func (s *MachineSuite) TestGetSetStatusWhileAlive(c *gc.C) {
-	err := s.machine.SetStatus(state.StatusError, "", nil)
-	c.Assert(err, gc.ErrorMatches, `cannot set status "error" without info`)
-	err = s.machine.SetStatus(state.StatusDown, "", nil)
-	c.Assert(err, gc.ErrorMatches, `cannot set status "down"`)
-	err = s.machine.SetStatus(state.Status("vliegkat"), "orville", nil)
-	c.Assert(err, gc.ErrorMatches, `cannot set invalid status "vliegkat"`)
-
-	status, info, data, err := s.machine.Status()
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(status, gc.Equals, state.StatusPending)
-	c.Assert(info, gc.Equals, "")
-	c.Assert(data, gc.HasLen, 0)
-
-	err = s.machine.SetStatus(state.StatusStarted, "", nil)
-	c.Assert(err, jc.ErrorIsNil)
-	status, info, data, err = s.machine.Status()
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(status, gc.Equals, state.StatusStarted)
-	c.Assert(info, gc.Equals, "")
-	c.Assert(data, gc.HasLen, 0)
-
-	err = s.machine.SetStatus(state.StatusError, "provisioning failed", map[string]interface{}{
-		"foo": "bar",
-	})
-	c.Assert(err, jc.ErrorIsNil)
-	status, info, data, err = s.machine.Status()
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(status, gc.Equals, state.StatusError)
-	c.Assert(info, gc.Equals, "provisioning failed")
-	c.Assert(data, gc.DeepEquals, map[string]interface{}{
-		"foo": "bar",
-	})
-}
-
-func (s *MachineSuite) TestSetStatusPending(c *gc.C) {
-	err := s.machine.SetStatus(state.StatusPending, "", nil)
-	c.Assert(err, jc.ErrorIsNil)
-	// Cannot set status to pending once a machine is provisioned.
-	err = s.machine.SetProvisioned("umbrella/0", "fake_nonce", nil)
-	c.Assert(err, jc.ErrorIsNil)
-	err = s.machine.SetStatus(state.StatusPending, "", nil)
-	c.Assert(err, gc.ErrorMatches, `cannot set status "pending"`)
-}
-
-func (s *MachineSuite) TestGetSetStatusWhileNotAlive(c *gc.C) {
-	// When Dying set/get should work.
-	err := s.machine.Destroy()
-	c.Assert(err, jc.ErrorIsNil)
-	err = s.machine.SetStatus(state.StatusStopped, "", nil)
-	c.Assert(err, jc.ErrorIsNil)
-	status, info, data, err := s.machine.Status()
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(status, gc.Equals, state.StatusStopped)
-	c.Assert(info, gc.Equals, "")
-	c.Assert(data, gc.HasLen, 0)
-
-	// When Dead set should fail, but get will work.
-	err = s.machine.EnsureDead()
-	c.Assert(err, jc.ErrorIsNil)
-	err = s.machine.SetStatus(state.StatusStarted, "not really", nil)
-	c.Assert(err, gc.ErrorMatches, `cannot set status of machine "1": not found or not alive`)
-	status, info, data, err = s.machine.Status()
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(status, gc.Equals, state.StatusStopped)
-	c.Assert(info, gc.Equals, "")
-	c.Assert(data, gc.HasLen, 0)
-
-	err = s.machine.Remove()
-	c.Assert(err, jc.ErrorIsNil)
-	err = s.machine.SetStatus(state.StatusStarted, "not really", nil)
-	c.Assert(err, gc.ErrorMatches, `cannot set status of machine "1": not found or not alive`)
-	_, _, _, err = s.machine.Status()
-	c.Assert(err, gc.ErrorMatches, "status not found")
-}
-
-func (s *MachineSuite) TestGetSetStatusDataStandard(c *gc.C) {
-	err := s.machine.SetStatus(state.StatusStarted, "", nil)
-	c.Assert(err, jc.ErrorIsNil)
-	_, _, _, err = s.machine.Status()
-	c.Assert(err, jc.ErrorIsNil)
-
-	// Regular status setting with data.
-	err = s.machine.SetStatus(state.StatusError, "provisioning failed", map[string]interface{}{
-		"1st-key": "one",
-		"2nd-key": 2,
-		"3rd-key": true,
-	})
-	c.Assert(err, jc.ErrorIsNil)
-	status, info, data, err := s.machine.Status()
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(status, gc.Equals, state.StatusError)
-	c.Assert(info, gc.Equals, "provisioning failed")
-	c.Assert(data, gc.DeepEquals, map[string]interface{}{
-		"1st-key": "one",
-		"2nd-key": 2,
-		"3rd-key": true,
-	})
-}
-
-func (s *MachineSuite) TestGetSetStatusDataMongo(c *gc.C) {
-	err := s.machine.SetStatus(state.StatusStarted, "", nil)
-	c.Assert(err, jc.ErrorIsNil)
-	_, _, _, err = s.machine.Status()
-	c.Assert(err, jc.ErrorIsNil)
-
-	// Status setting with MongoDB special values.
-	err = s.machine.SetStatus(state.StatusError, "mongo", map[string]interface{}{
-		`{name: "Joe"}`: "$where",
-		"eval":          `eval(function(foo) { return foo; }, "bar")`,
-		"mapReduce":     "mapReduce",
-		"group":         "group",
-	})
-	c.Assert(err, jc.ErrorIsNil)
-	status, info, data, err := s.machine.Status()
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(status, gc.Equals, state.StatusError)
-	c.Assert(info, gc.Equals, "mongo")
-	c.Assert(data, gc.DeepEquals, map[string]interface{}{
-		`{name: "Joe"}`: "$where",
-		"eval":          `eval(function(foo) { return foo; }, "bar")`,
-		"mapReduce":     "mapReduce",
-		"group":         "group",
-	})
-}
-
-func (s *MachineSuite) TestGetSetStatusDataChange(c *gc.C) {
-	err := s.machine.SetStatus(state.StatusStarted, "", nil)
-	c.Assert(err, jc.ErrorIsNil)
-	_, _, _, err = s.machine.Status()
-	c.Assert(err, jc.ErrorIsNil)
-
-	// Status setting and changing data afterwards.
-	data := map[string]interface{}{
-		"1st-key": "one",
-		"2nd-key": 2,
-		"3rd-key": true,
-	}
-	err = s.machine.SetStatus(state.StatusError, "provisioning failed", data)
-	c.Assert(err, jc.ErrorIsNil)
-	data["4th-key"] = 4.0
-
-	status, info, data, err := s.machine.Status()
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(status, gc.Equals, state.StatusError)
-	c.Assert(info, gc.Equals, "provisioning failed")
-	c.Assert(data, gc.DeepEquals, map[string]interface{}{
-		"1st-key": "one",
-		"2nd-key": 2,
-		"3rd-key": true,
-	})
-
-	// Set status data to nil, so an empty map will be returned.
-	err = s.machine.SetStatus(state.StatusStarted, "", nil)
-	c.Assert(err, jc.ErrorIsNil)
-
-	status, info, data, err = s.machine.Status()
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(status, gc.Equals, state.StatusStarted)
-	c.Assert(info, gc.Equals, "")
-	c.Assert(data, gc.HasLen, 0)
-}
-
-func (s *MachineSuite) TestSetAddresses(c *gc.C) {
+func (s *MachineSuite) TestSetProviderAddresses(c *gc.C) {
 	machine, err := s.State.AddMachine("quantal", state.JobHostUnits)
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(machine.Addresses(), gc.HasLen, 0)
 
-	addresses := []network.Address{
-		network.NewAddress("127.0.0.1", network.ScopeUnknown),
-		network.NewAddress("8.8.8.8", network.ScopeUnknown),
-	}
-	err = machine.SetAddresses(addresses...)
+	addresses := network.NewAddresses("127.0.0.1", "8.8.8.8")
+	err = machine.SetProviderAddresses(addresses...)
 	c.Assert(err, jc.ErrorIsNil)
 	err = machine.Refresh()
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(machine.Addresses(), jc.SameContents, addresses)
+
+	expectedAddresses := network.NewAddresses("8.8.8.8", "127.0.0.1")
+	c.Assert(machine.Addresses(), jc.DeepEquals, expectedAddresses)
+}
+
+func (s *MachineSuite) TestSetProviderAddressesWithContainers(c *gc.C) {
+	machine, err := s.State.AddMachine("quantal", state.JobHostUnits)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(machine.Addresses(), gc.HasLen, 0)
+
+	// Create subnet and pick two addresses.
+	subnetInfo := state.SubnetInfo{
+		CIDR:              "192.168.1.0/24",
+		AllocatableIPLow:  "192.168.1.0",
+		AllocatableIPHigh: "192.168.1.10",
+	}
+	subnet, err := s.State.AddSubnet(subnetInfo)
+	c.Assert(err, jc.ErrorIsNil)
+
+	ipAddr1, err := subnet.PickNewAddress()
+	c.Assert(err, jc.ErrorIsNil)
+	err = ipAddr1.SetState(state.AddressStateAllocated)
+	c.Assert(err, jc.ErrorIsNil)
+	ipAddr2, err := subnet.PickNewAddress()
+	c.Assert(err, jc.ErrorIsNil)
+	err = ipAddr2.SetState(state.AddressStateAllocated)
+	c.Assert(err, jc.ErrorIsNil)
+
+	// When setting all addresses the subnet addresses have to be
+	// filtered out.
+	addresses := network.NewAddresses(
+		"127.0.0.1",
+		"8.8.8.8",
+		ipAddr1.Value(),
+		ipAddr2.Value(),
+	)
+	err = machine.SetProviderAddresses(addresses...)
+	c.Assert(err, jc.ErrorIsNil)
+	err = machine.Refresh()
+	c.Assert(err, jc.ErrorIsNil)
+
+	expectedAddresses := network.NewAddresses("8.8.8.8", "127.0.0.1")
+	c.Assert(machine.Addresses(), jc.DeepEquals, expectedAddresses)
+}
+
+func (s *MachineSuite) TestSetProviderAddressesOnContainer(c *gc.C) {
+	machine, err := s.State.AddMachine("quantal", state.JobHostUnits)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(machine.Addresses(), gc.HasLen, 0)
+
+	// Create subnet and pick two addresses.
+	subnetInfo := state.SubnetInfo{
+		CIDR:              "192.168.1.0/24",
+		AllocatableIPLow:  "192.168.1.0",
+		AllocatableIPHigh: "192.168.1.10",
+	}
+	subnet, err := s.State.AddSubnet(subnetInfo)
+	c.Assert(err, jc.ErrorIsNil)
+
+	ipAddr, err := subnet.PickNewAddress()
+	c.Assert(err, jc.ErrorIsNil)
+	err = ipAddr.SetState(state.AddressStateAllocated)
+	c.Assert(err, jc.ErrorIsNil)
+
+	// Create an LXC container inside the machine.
+	template := state.MachineTemplate{
+		Series: "quantal",
+		Jobs:   []state.MachineJob{state.JobHostUnits},
+	}
+	container, err := s.State.AddMachineInsideMachine(template, machine.Id(), instance.LXC)
+	c.Assert(err, jc.ErrorIsNil)
+
+	// When setting all addresses the subnet address has to accepted.
+	addresses := network.NewAddresses("127.0.0.1", ipAddr.Value())
+	err = container.SetProviderAddresses(addresses...)
+	c.Assert(err, jc.ErrorIsNil)
+	err = container.Refresh()
+	c.Assert(err, jc.ErrorIsNil)
+
+	expectedAddresses := network.NewAddresses(ipAddr.Value(), "127.0.0.1")
+	c.Assert(container.Addresses(), jc.DeepEquals, expectedAddresses)
 }
 
 func (s *MachineSuite) TestSetMachineAddresses(c *gc.C) {
@@ -1667,15 +1783,36 @@ func (s *MachineSuite) TestSetMachineAddresses(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(machine.Addresses(), gc.HasLen, 0)
 
-	addresses := []network.Address{
-		network.NewAddress("127.0.0.1", network.ScopeUnknown),
-		network.NewAddress("8.8.8.8", network.ScopeUnknown),
-	}
+	addresses := network.NewAddresses("127.0.0.1", "8.8.8.8")
 	err = machine.SetMachineAddresses(addresses...)
 	c.Assert(err, jc.ErrorIsNil)
 	err = machine.Refresh()
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(machine.MachineAddresses(), gc.DeepEquals, addresses)
+
+	expectedAddresses := network.NewAddresses("8.8.8.8", "127.0.0.1")
+	c.Assert(machine.MachineAddresses(), jc.DeepEquals, expectedAddresses)
+}
+
+func (s *MachineSuite) TestSetEmptyMachineAddresses(c *gc.C) {
+	machine, err := s.State.AddMachine("quantal", state.JobHostUnits)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(machine.Addresses(), gc.HasLen, 0)
+
+	// Add some machine addresses initially to make sure they're removed.
+	addresses := network.NewAddresses("127.0.0.1", "8.8.8.8")
+	err = machine.SetMachineAddresses(addresses...)
+	c.Assert(err, jc.ErrorIsNil)
+	err = machine.Refresh()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(machine.MachineAddresses(), gc.HasLen, 2)
+
+	// Make call with empty address list.
+	err = machine.SetMachineAddresses()
+	c.Assert(err, jc.ErrorIsNil)
+	err = machine.Refresh()
+	c.Assert(err, jc.ErrorIsNil)
+
+	c.Assert(machine.MachineAddresses(), gc.HasLen, 0)
 }
 
 func (s *MachineSuite) TestMergedAddresses(c *gc.C) {
@@ -1693,7 +1830,7 @@ func (s *MachineSuite) TestMergedAddresses(c *gc.C) {
 		"127.0.0.2",
 		"example.org",
 	)
-	err = machine.SetAddresses(providerAddresses...)
+	err = machine.SetProviderAddresses(providerAddresses...)
 	c.Assert(err, jc.ErrorIsNil)
 
 	machineAddresses := network.NewAddresses(
@@ -1739,7 +1876,7 @@ func (s *MachineSuite) TestMergedAddresses(c *gc.C) {
 		gc.IsNil,
 	)
 
-	err = machine.SetAddresses(providerAddresses...)
+	err = machine.SetProviderAddresses(providerAddresses...)
 	c.Assert(err, jc.ErrorIsNil)
 	err = machine.SetMachineAddresses(machineAddresses...)
 	c.Assert(err, jc.ErrorIsNil)
@@ -1760,27 +1897,27 @@ func (s *MachineSuite) TestMergedAddresses(c *gc.C) {
 	))
 }
 
-func (s *MachineSuite) TestSetAddressesConcurrentChangeDifferent(c *gc.C) {
+func (s *MachineSuite) TestSetProviderAddressesConcurrentChangeDifferent(c *gc.C) {
 	machine, err := s.State.AddMachine("quantal", state.JobHostUnits)
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(machine.Addresses(), gc.HasLen, 0)
 
-	addr0 := network.NewAddress("127.0.0.1", network.ScopeUnknown)
-	addr1 := network.NewAddress("8.8.8.8", network.ScopeUnknown)
+	addr0 := network.NewAddress("127.0.0.1")
+	addr1 := network.NewAddress("8.8.8.8")
 
 	defer state.SetBeforeHooks(c, s.State, func() {
 		machine, err := s.State.Machine(machine.Id())
 		c.Assert(err, jc.ErrorIsNil)
-		err = machine.SetAddresses(addr1, addr0)
+		err = machine.SetProviderAddresses(addr1, addr0)
 		c.Assert(err, jc.ErrorIsNil)
 	}).Check()
 
-	err = machine.SetAddresses(addr0, addr1)
+	err = machine.SetProviderAddresses(addr0, addr1)
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(machine.Addresses(), jc.SameContents, []network.Address{addr0, addr1})
 }
 
-func (s *MachineSuite) TestSetAddressesConcurrentChangeEqual(c *gc.C) {
+func (s *MachineSuite) TestSetProviderAddressesConcurrentChangeEqual(c *gc.C) {
 	machine, err := s.State.AddMachine("quantal", state.JobHostUnits)
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(machine.Addresses(), gc.HasLen, 0)
@@ -1788,30 +1925,66 @@ func (s *MachineSuite) TestSetAddressesConcurrentChangeEqual(c *gc.C) {
 	revno0, err := state.TxnRevno(s.State, "machines", machineDocID)
 	c.Assert(err, jc.ErrorIsNil)
 
-	addr0 := network.NewAddress("127.0.0.1", network.ScopeUnknown)
-	addr1 := network.NewAddress("8.8.8.8", network.ScopeUnknown)
+	addr0 := network.NewAddress("127.0.0.1")
+	addr1 := network.NewAddress("8.8.8.8")
 
 	var revno1 int64
 	defer state.SetBeforeHooks(c, s.State, func() {
 		machine, err := s.State.Machine(machine.Id())
 		c.Assert(err, jc.ErrorIsNil)
-		err = machine.SetAddresses(addr0, addr1)
+		err = machine.SetProviderAddresses(addr0, addr1)
 		c.Assert(err, jc.ErrorIsNil)
 		revno1, err = state.TxnRevno(s.State, "machines", machineDocID)
 		c.Assert(err, jc.ErrorIsNil)
-		c.Assert(revno1, gc.Equals, revno0+1)
+		c.Assert(revno1, jc.GreaterThan, revno0)
 	}).Check()
 
-	err = machine.SetAddresses(addr0, addr1)
+	err = machine.SetProviderAddresses(addr0, addr1)
 	c.Assert(err, jc.ErrorIsNil)
 
-	// Doc should not have been updated, but Machine object's view should be.
+	// Doc will be updated; concurrent changes are explicitly ignored.
 	revno2, err := state.TxnRevno(s.State, "machines", machineDocID)
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(revno2, gc.Equals, revno1)
+	c.Assert(revno2, jc.GreaterThan, revno1)
 	c.Assert(machine.Addresses(), jc.SameContents, []network.Address{addr0, addr1})
 }
 
+func (s *MachineSuite) TestSetProviderAddressesInvalidateMemory(c *gc.C) {
+	machine, err := s.State.AddMachine("quantal", state.JobHostUnits)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(machine.Addresses(), gc.HasLen, 0)
+	machineDocID := state.DocID(s.State, machine.Id())
+
+	addr0 := network.NewAddress("127.0.0.1")
+	addr1 := network.NewAddress("8.8.8.8")
+
+	// Set addresses to [addr0] initially. We'll get a separate Machine
+	// object to update addresses, to ensure that the in-memory cache of
+	// addresses does not prevent the initial Machine from updating
+	// addresses back to the original value.
+	err = machine.SetProviderAddresses(addr0)
+	c.Assert(err, jc.ErrorIsNil)
+	revno0, err := state.TxnRevno(s.State, "machines", machineDocID)
+	c.Assert(err, jc.ErrorIsNil)
+
+	machine2, err := s.State.Machine(machine.Id())
+	c.Assert(err, jc.ErrorIsNil)
+	err = machine2.SetProviderAddresses(addr1)
+	c.Assert(err, jc.ErrorIsNil)
+	revno1, err := state.TxnRevno(s.State, "machines", machineDocID)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(revno1, jc.GreaterThan, revno0)
+	c.Assert(machine.Addresses(), jc.SameContents, []network.Address{addr0})
+	c.Assert(machine2.Addresses(), jc.SameContents, []network.Address{addr1})
+
+	err = machine.SetProviderAddresses(addr0)
+	c.Assert(err, jc.ErrorIsNil)
+	revno2, err := state.TxnRevno(s.State, "machines", machineDocID)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(revno2, jc.GreaterThan, revno1)
+	c.Assert(machine.Addresses(), jc.SameContents, []network.Address{addr0})
+}
+
 func (s *MachineSuite) addMachineWithSupportedContainer(c *gc.C, container instance.ContainerType) *state.Machine {
 	machine, err := s.State.AddMachine("quantal", state.JobHostUnits)
 	c.Assert(err, jc.ErrorIsNil)
@@ -1937,18 +2110,18 @@ func (s *MachineSuite) TestSetSupportedContainersSetsUnknownToError(c *gc.C) {
 	// A supported (kvm) container will have a pending status.
 	err = supportedContainer.Refresh()
 	c.Assert(err, jc.ErrorIsNil)
-	status, info, data, err := supportedContainer.Status()
+	statusInfo, err := supportedContainer.Status()
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(status, gc.Equals, state.StatusPending)
+	c.Assert(statusInfo.Status, gc.Equals, state.StatusPending)
 
 	// An unsupported (lxc) container will have an error status.
 	err = container.Refresh()
 	c.Assert(err, jc.ErrorIsNil)
-	status, info, data, err = container.Status()
+	statusInfo, err = container.Status()
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(status, gc.Equals, state.StatusError)
-	c.Assert(info, gc.Equals, "unsupported container")
-	c.Assert(data, gc.DeepEquals, map[string]interface{}{"type": "lxc"})
+	c.Assert(statusInfo.Status, gc.Equals, state.StatusError)
+	c.Assert(statusInfo.Message, gc.Equals, "unsupported container")
+	c.Assert(statusInfo.Data, gc.DeepEquals, map[string]interface{}{"type": "lxc"})
 }
 
 func (s *MachineSuite) TestSupportsNoContainersSetsAllToError(c *gc.C) {
@@ -1972,12 +2145,12 @@ func (s *MachineSuite) TestSupportsNoContainersSetsAllToError(c *gc.C) {
 	for _, container := range containers {
 		err = container.Refresh()
 		c.Assert(err, jc.ErrorIsNil)
-		status, info, data, err := container.Status()
+		statusInfo, err := container.Status()
 		c.Assert(err, jc.ErrorIsNil)
-		c.Assert(status, gc.Equals, state.StatusError)
-		c.Assert(info, gc.Equals, "unsupported container")
+		c.Assert(statusInfo.Status, gc.Equals, state.StatusError)
+		c.Assert(statusInfo.Message, gc.Equals, "unsupported container")
 		containerType := state.ContainerTypeFromId(container.Id())
-		c.Assert(data, gc.DeepEquals, map[string]interface{}{"type": string(containerType)})
+		c.Assert(statusInfo.Data, gc.DeepEquals, map[string]interface{}{"type": string(containerType)})
 	}
 }
 
@@ -2011,7 +2184,7 @@ func (s *MachineSuite) TestWatchInterfaces(c *gc.C) {
 		NetworkName:   "vlan42",
 		IsVirtual:     true,
 	}}
-	err := s.machine.SetInstanceInfo("umbrella/0", "fake_nonce", nil, networks, interfaces)
+	err := s.machine.SetInstanceInfo("umbrella/0", "fake_nonce", nil, networks, interfaces, nil, nil)
 	c.Assert(err, jc.ErrorIsNil)
 
 	// Read dynamically generated document Ids.
@@ -2088,7 +2261,7 @@ func (s *MachineSuite) TestWatchInterfaces(c *gc.C) {
 		NetworkName:   "vlan42",
 		IsVirtual:     true,
 	}}
-	err = machine2.SetInstanceInfo("m-too", "fake_nonce", nil, networks, interfaces2)
+	err = machine2.SetInstanceInfo("m-too", "fake_nonce", nil, networks, interfaces2, nil, nil)
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(ifaces, gc.HasLen, 3)
 	wc.AssertNoChange()
@@ -2114,7 +2287,7 @@ func (s *MachineSuite) TestWatchInterfaces(c *gc.C) {
 }
 
 func (s *MachineSuite) TestWatchInterfacesDiesOnStateClose(c *gc.C) {
-	testWatcherDiesWhenStateCloses(c, func(c *gc.C, st *state.State) waiter {
+	testWatcherDiesWhenStateCloses(c, s.envTag, func(c *gc.C, st *state.State) waiter {
 		m, err := st.Machine(s.machine.Id())
 		c.Assert(err, jc.ErrorIsNil)
 		w := m.WatchInterfaces()
