@@ -14,6 +14,7 @@ import (
 	"github.com/juju/juju/network"
 	"github.com/juju/juju/state/multiwatcher"
 	"github.com/juju/juju/state/watcher"
+	"time"
 )
 
 // allWatcherStateBacking implements allWatcherBacking by
@@ -43,12 +44,12 @@ func (m *backingMachine) updated(st *State, store *multiwatcherStore, id interfa
 	if oldInfo == nil {
 		// We're adding the entry for the first time,
 		// so fetch the associated machine status.
-		sdoc, err := getStatus(st, machineGlobalKey(m.Id))
+		statusInfo, err := getStatus(st, machineGlobalKey(m.Id), "machine")
 		if err != nil {
 			return err
 		}
-		info.Status = multiwatcher.Status(sdoc.Status)
-		info.StatusInfo = sdoc.StatusInfo
+		info.Status = multiwatcher.Status(statusInfo.Status)
+		info.StatusInfo = statusInfo.Message
 	} else {
 		// The entry already exists, so preserve the current status and
 		// instance data.
@@ -74,10 +75,6 @@ func (m *backingMachine) updated(st *State, store *multiwatcherStore, id interfa
 }
 
 func (m *backingMachine) removed(st *State, store *multiwatcherStore, id interface{}) {
-	// TODO(mjs) This isn't correct - the store should be using
-	// environment UUID prefixed ids but we can't fix it properly
-	// until davecheney smashes the allwatcher to apiserver/params
-	// dependency.
 	store.Remove(multiwatcher.EntityId{
 		Kind: "machine",
 		Id:   st.localID(id.(string)),
@@ -90,20 +87,6 @@ func (m *backingMachine) mongoId() interface{} {
 
 type backingUnit unitDoc
 
-// translateLegacyUnitAgentStatus returns the status value the current GUI expects to see.
-// This is a short term requirement until the GUI is updated to be able to handle
-// the new values.
-// We use string literals here to avoid import loops and lots of messy refactoring.
-func translateLegacyUnitAgentStatus(in multiwatcher.Status) multiwatcher.Status {
-	switch in {
-	case multiwatcher.Status("failed"):
-		return multiwatcher.Status("down")
-	case multiwatcher.Status("active"):
-		return multiwatcher.Status("started")
-	}
-	return in
-}
-
 func getUnitPortRangesAndPorts(st *State, unitName string) ([]network.PortRange, []network.Port, error) {
 	// Get opened port ranges for the unit and convert them to ports,
 	// as older clients/servers do not know about ranges). See bug
@@ -119,7 +102,7 @@ func getUnitPortRangesAndPorts(st *State, unitName string) ([]network.PortRange,
 	portRanges, err := unit.OpenedPorts()
 	// Since the port ranges are associated with the unit's machine,
 	// we need to check for NotAssignedError.
-	if IsNotAssigned(err) {
+	if errors.IsNotAssigned(err) {
 		// Not assigned, so there won't be any ports opened.
 		// Empty slices ensure backwards compatibility with older clients.
 		// See Bug #1425435.
@@ -143,6 +126,22 @@ func getUnitPortRangesAndPorts(st *State, unitName string) ([]network.PortRange,
 	return portRanges, compatiblePorts, nil
 }
 
+func unitAndAgentStatus(st *State, name string) (unitStatus, agentStatus *StatusInfo, err error) {
+	unit, err := st.Unit(name)
+	if err != nil {
+		return nil, nil, errors.Trace(err)
+	}
+	unitStatusResult, err := unit.Status()
+	if err != nil {
+		return nil, nil, errors.Trace(err)
+	}
+	agentStatusResult, err := unit.AgentStatus()
+	if err != nil {
+		return nil, nil, errors.Trace(err)
+	}
+	return &unitStatusResult, &agentStatusResult, nil
+}
+
 func (u *backingUnit) updated(st *State, store *multiwatcherStore, id interface{}) error {
 	info := &multiwatcher.UnitInfo{
 		Name:        u.Name,
@@ -156,15 +155,49 @@ func (u *backingUnit) updated(st *State, store *multiwatcherStore, id interface{
 	}
 	oldInfo := store.Get(info.EntityId())
 	if oldInfo == nil {
+		logger.Debugf("new unit %q added to backing state", u.Name)
 		// We're adding the entry for the first time,
 		// so fetch the associated unit status and opened ports.
-		sdoc, err := getStatus(st, unitGlobalKey(u.Name))
+		unitStatus, agentStatus, err := unitAndAgentStatus(st, u.Name)
 		if err != nil {
-			return err
+			return errors.Annotatef(err, "reading unit and agent status for %q", u.Name)
 		}
-		info.Status = multiwatcher.Status(sdoc.Status)
-		info.Status = translateLegacyUnitAgentStatus(info.Status)
-		info.StatusInfo = sdoc.StatusInfo
+		// Unit and workload status.
+		info.WorkloadStatus = multiwatcher.StatusInfo{
+			Current: multiwatcher.Status(unitStatus.Status),
+			Message: unitStatus.Message,
+			Data:    unitStatus.Data,
+			Since:   unitStatus.Since,
+		}
+		if u.Tools != nil {
+			info.AgentStatus.Version = u.Tools.Version.Number.String()
+		}
+		info.AgentStatus = multiwatcher.StatusInfo{
+			Current: multiwatcher.Status(agentStatus.Status),
+			Message: agentStatus.Message,
+			Data:    agentStatus.Data,
+			Since:   agentStatus.Since,
+		}
+		// Legacy status info.
+		if unitStatus.Status == StatusError {
+			info.Status = multiwatcher.Status(unitStatus.Status)
+			info.StatusInfo = unitStatus.Message
+			info.StatusData = unitStatus.Data
+		} else {
+			legacyStatus, ok := TranslateToLegacyAgentState(agentStatus.Status, unitStatus.Status, unitStatus.Message)
+			if !ok {
+				logger.Warningf(
+					"translate to legacy status encounted unexpected workload status %q and agent status %q",
+					unitStatus.Status, agentStatus.Status)
+			}
+			info.Status = multiwatcher.Status(legacyStatus)
+			info.StatusInfo = agentStatus.Message
+			info.StatusData = agentStatus.Data
+		}
+		if len(info.StatusData) == 0 {
+			info.StatusData = nil
+		}
+
 		portRanges, compatiblePorts, err := getUnitPortRangesAndPorts(st, u.Name)
 		if err != nil {
 			return errors.Trace(err)
@@ -175,8 +208,12 @@ func (u *backingUnit) updated(st *State, store *multiwatcherStore, id interface{
 	} else {
 		// The entry already exists, so preserve the current status and ports.
 		oldInfo := oldInfo.(*multiwatcher.UnitInfo)
+		// Legacy status.
 		info.Status = oldInfo.Status
 		info.StatusInfo = oldInfo.StatusInfo
+		// Unit and workload status.
+		info.AgentStatus = oldInfo.AgentStatus
+		info.WorkloadStatus = oldInfo.WorkloadStatus
 		info.Ports = oldInfo.Ports
 		info.PortRanges = oldInfo.PortRanges
 	}
@@ -207,7 +244,6 @@ func getUnitAddresses(st *State, unitName string) (publicAddress, privateAddress
 }
 
 func (u *backingUnit) removed(st *State, store *multiwatcherStore, id interface{}) {
-	// TODO(mjs) as per backingMachine.removed()
 	store.Remove(multiwatcher.EntityId{
 		Kind: "unit",
 		Id:   st.localID(id.(string)),
@@ -240,14 +276,47 @@ func (svc *backingService) updated(st *State, store *multiwatcherStore, id inter
 	oldInfo := store.Get(info.EntityId())
 	needConfig := false
 	if oldInfo == nil {
+		logger.Debugf("new service %q added to backing state", svc.Name)
+		key := serviceGlobalKey(svc.Name)
 		// We're adding the entry for the first time,
 		// so fetch the associated child documents.
-		c, err := readConstraints(st, serviceGlobalKey(svc.Name))
+		c, err := readConstraints(st, key)
 		if err != nil {
-			return err
+			return errors.Trace(err)
 		}
 		info.Constraints = c
 		needConfig = true
+		// Fetch the status.
+		service, err := st.Service(svc.Name)
+		if err != nil {
+			return errors.Trace(err)
+		}
+		serviceStatus, err := service.Status()
+		if err != nil {
+			logger.Warningf("reading service status for key %s: %v", key, err)
+		}
+		if err != nil && !errors.IsNotFound(err) {
+			return errors.Annotatef(err, "reading service status for key %s", key)
+		}
+		if err == nil {
+			info.Status = multiwatcher.StatusInfo{
+				Current: multiwatcher.Status(serviceStatus.Status),
+				Message: serviceStatus.Message,
+				Data:    serviceStatus.Data,
+				Since:   serviceStatus.Since,
+			}
+		} else {
+			// TODO(wallyworld) - bug http://pad.lv/1451283
+			// return an error here once we figure out what's happening
+			// Not sure how status can even return NotFound as it is created
+			// with the service initially. For now, we'll log the error as per
+			// the above and return Unknown.
+			now := time.Now()
+			info.Status = multiwatcher.StatusInfo{
+				Current: multiwatcher.Status(StatusUnknown),
+				Since:   &now,
+			}
+		}
 	} else {
 		// The entry already exists, so preserve the current status.
 		oldInfo := oldInfo.(*multiwatcher.ServiceInfo)
@@ -266,7 +335,7 @@ func (svc *backingService) updated(st *State, store *multiwatcherStore, id inter
 		var err error
 		info.Config, _, err = readSettingsDoc(st, serviceSettingsKey(svc.Name, svc.CharmURL))
 		if err != nil {
-			return err
+			return errors.Trace(err)
 		}
 	}
 	store.Update(info)
@@ -274,7 +343,6 @@ func (svc *backingService) updated(st *State, store *multiwatcherStore, id inter
 }
 
 func (svc *backingService) removed(st *State, store *multiwatcherStore, id interface{}) {
-	// TODO(mjs) as per backingMachine.removed()
 	store.Remove(multiwatcher.EntityId{
 		Kind: "service",
 		Id:   st.localID(id.(string)),
@@ -294,6 +362,36 @@ func (svc *backingService) mongoId() interface{} {
 	return svc.DocID
 }
 
+type backingAction actionDoc
+
+func (a *backingAction) mongoId() interface{} {
+	return a.DocId
+}
+
+func (a *backingAction) removed(st *State, store *multiwatcherStore, id interface{}) {
+	store.Remove(multiwatcher.EntityId{
+		Kind: "action",
+		Id:   st.localID(id.(string)),
+	})
+}
+
+func (a *backingAction) updated(st *State, store *multiwatcherStore, id interface{}) error {
+	info := &multiwatcher.ActionInfo{
+		Id:         st.localID(a.DocId),
+		Receiver:   a.Receiver,
+		Name:       a.Name,
+		Parameters: a.Parameters,
+		Status:     string(a.Status),
+		Message:    a.Message,
+		Results:    a.Results,
+		Enqueued:   a.Enqueued,
+		Started:    a.Started,
+		Completed:  a.Completed,
+	}
+	store.Update(info)
+	return nil
+}
+
 type backingRelation relationDoc
 
 func (r *backingRelation) updated(st *State, store *multiwatcherStore, id interface{}) error {
@@ -314,7 +412,6 @@ func (r *backingRelation) updated(st *State, store *multiwatcherStore, id interf
 }
 
 func (r *backingRelation) removed(st *State, store *multiwatcherStore, id interface{}) {
-	// TODO(mjs) as per backingMachine.removed()
 	store.Remove(multiwatcher.EntityId{
 		Kind: "relation",
 		Id:   st.localID(id.(string)),
@@ -352,24 +449,60 @@ func (a *backingAnnotation) mongoId() interface{} {
 	return a.GlobalKey
 }
 
+type backingBlock blockDoc
+
+func (a *backingBlock) updated(st *State, store *multiwatcherStore, id interface{}) error {
+	info := &multiwatcher.BlockInfo{
+		Id:      st.localID(a.DocID),
+		Tag:     a.Tag,
+		Type:    a.Type.ToParams(),
+		Message: a.Message,
+	}
+	store.Update(info)
+	return nil
+}
+
+func (a *backingBlock) removed(st *State, store *multiwatcherStore, id interface{}) {
+	store.Remove(multiwatcher.EntityId{
+		Kind: "block",
+		Id:   st.localID(id.(string)),
+	})
+}
+
+func (a *backingBlock) mongoId() interface{} {
+	return a.DocID
+}
+
 type backingStatus statusDoc
 
 func (s *backingStatus) updated(st *State, store *multiwatcherStore, id interface{}) error {
-	parentId, ok := backingEntityIdForGlobalKey(st.localID(id.(string)))
+	parentID, ok := backingEntityIdForGlobalKey(st.localID(id.(string)))
 	if !ok {
 		return nil
 	}
-	info0 := store.Get(parentId)
+	info0 := store.Get(parentID)
 	switch info := info0.(type) {
 	case nil:
 		// The parent info doesn't exist. Ignore the status until it does.
 		return nil
 	case *multiwatcher.UnitInfo:
 		newInfo := *info
-		newInfo.Status = multiwatcher.Status(s.Status)
-		newInfo.Status = translateLegacyUnitAgentStatus(newInfo.Status)
-		newInfo.StatusInfo = s.StatusInfo
-		newInfo.StatusData = s.StatusData
+		// Get the unit's current recorded status from state.
+		// It's needed to reset the unit status when a unit comes off error.
+		statusInfo, err := getStatus(st, unitGlobalKey(newInfo.Name), "unit")
+		if err != nil {
+			return err
+		}
+		if err := s.updatedUnitStatus(st, store, id.(string), statusInfo, &newInfo); err != nil {
+			return err
+		}
+		info0 = &newInfo
+	case *multiwatcher.ServiceInfo:
+		newInfo := *info
+		newInfo.Status.Current = multiwatcher.Status(s.Status)
+		newInfo.Status.Message = s.StatusInfo
+		newInfo.Status.Data = s.StatusData
+		newInfo.Status.Since = unixNanoToTime(s.Updated)
 		info0 = &newInfo
 	case *multiwatcher.MachineInfo:
 		newInfo := *info
@@ -384,6 +517,74 @@ func (s *backingStatus) updated(st *State, store *multiwatcherStore, id interfac
 	return nil
 }
 
+func (s *backingStatus) updatedUnitStatus(st *State, store *multiwatcherStore, id string, unitStatus StatusInfo, newInfo *multiwatcher.UnitInfo) error {
+	// Unit or workload status - display the agent status or any error.
+	if strings.HasSuffix(id, "#charm") || s.Status == StatusError {
+		newInfo.WorkloadStatus.Current = multiwatcher.Status(s.Status)
+		newInfo.WorkloadStatus.Message = s.StatusInfo
+		newInfo.WorkloadStatus.Data = s.StatusData
+		newInfo.WorkloadStatus.Since = unixNanoToTime(s.Updated)
+	} else {
+		newInfo.AgentStatus.Current = multiwatcher.Status(s.Status)
+		newInfo.AgentStatus.Message = s.StatusInfo
+		newInfo.AgentStatus.Data = s.StatusData
+		newInfo.AgentStatus.Since = unixNanoToTime(s.Updated)
+		// If the unit was in error and now it's not, we need to reset its
+		// status back to what was previously recorded.
+		if newInfo.WorkloadStatus.Current == multiwatcher.Status(StatusError) {
+			newInfo.WorkloadStatus.Current = multiwatcher.Status(unitStatus.Status)
+			newInfo.WorkloadStatus.Message = unitStatus.Message
+			newInfo.WorkloadStatus.Data = unitStatus.Data
+			newInfo.WorkloadStatus.Since = unixNanoToTime(s.Updated)
+		}
+	}
+
+	// Legacy status info - it is an aggregated value between workload and agent statuses.
+	legacyStatus, ok := TranslateToLegacyAgentState(
+		Status(newInfo.AgentStatus.Current),
+		Status(newInfo.WorkloadStatus.Current),
+		newInfo.WorkloadStatus.Message,
+	)
+	if !ok {
+		logger.Warningf(
+			"translate to legacy status encounted unexpected workload status %q and agent status %q",
+			newInfo.WorkloadStatus.Current, newInfo.AgentStatus.Current)
+	}
+	newInfo.Status = multiwatcher.Status(legacyStatus)
+	if newInfo.Status == multiwatcher.Status(StatusError) {
+		newInfo.StatusInfo = newInfo.WorkloadStatus.Message
+		newInfo.StatusData = newInfo.WorkloadStatus.Data
+	} else {
+		newInfo.StatusInfo = newInfo.AgentStatus.Message
+		newInfo.StatusData = newInfo.AgentStatus.Data
+	}
+
+	// A change in a unit's status might also affect it's service.
+	service, err := st.Service(newInfo.Service)
+	if err != nil {
+		return errors.Trace(err)
+	}
+	serviceId, ok := backingEntityIdForGlobalKey(service.globalKey())
+	if !ok {
+		return nil
+	}
+	serviceInfo := store.Get(serviceId)
+	if serviceInfo == nil {
+		return nil
+	}
+	status, err := service.Status()
+	if err != nil {
+		return errors.Trace(err)
+	}
+	newServiceInfo := *serviceInfo.(*multiwatcher.ServiceInfo)
+	newServiceInfo.Status.Current = multiwatcher.Status(status.Status)
+	newServiceInfo.Status.Message = status.Message
+	newServiceInfo.Status.Data = status.Data
+	newServiceInfo.Status.Since = status.Since
+	store.Update(&newServiceInfo)
+	return nil
+}
+
 func (s *backingStatus) removed(st *State, store *multiwatcherStore, id interface{}) {
 	// If the status is removed, the parent will follow not long after,
 	// so do nothing.
@@ -397,11 +598,11 @@ type backingConstraints constraintsDoc
 
 func (c *backingConstraints) updated(st *State, store *multiwatcherStore, id interface{}) error {
 	localID := st.localID(id.(string))
-	parentId, ok := backingEntityIdForGlobalKey(localID)
+	parentID, ok := backingEntityIdForGlobalKey(localID)
 	if !ok {
 		return nil
 	}
-	info0 := store.Get(parentId)
+	info0 := store.Get(parentID)
 	switch info := info0.(type) {
 	case nil:
 		// The parent info doesn't exist. Ignore the status until it does.
@@ -430,11 +631,11 @@ type backingSettings map[string]interface{}
 
 func (s *backingSettings) updated(st *State, store *multiwatcherStore, id interface{}) error {
 	localID := st.localID(id.(string))
-	parentId, url, ok := backingEntityIdForSettingsKey(localID)
+	parentID, url, ok := backingEntityIdForSettingsKey(localID)
 	if !ok {
 		return nil
 	}
-	info0 := store.Get(parentId)
+	info0 := store.Get(parentID)
 	switch info := info0.(type) {
 	case nil:
 		// The parent info doesn't exist. Ignore the status until it does.
@@ -459,7 +660,25 @@ func (s *backingSettings) updated(st *State, store *multiwatcherStore, id interf
 	return nil
 }
 
-func (s *backingSettings) removed(st *State, store *multiwatcherStore, id interface{}) {}
+func (s *backingSettings) removed(st *State, store *multiwatcherStore, id interface{}) {
+	localID := st.localID(id.(string))
+	parentID, url, ok := backingEntityIdForSettingsKey(localID)
+	if !ok {
+		// Service is already gone along with its settings.
+		return
+	}
+	parent := store.Get(parentID)
+	if info, ok := parent.(*multiwatcher.ServiceInfo); ok {
+		if info.CharmURL != url {
+			return
+		}
+		newInfo := *info
+		cleanSettingsMap(*s)
+		newInfo.Config = *s
+		parent = &newInfo
+		store.Update(parent)
+	}
+}
 
 func (s *backingSettings) mongoId() interface{} {
 	panic("cannot find mongo id from settings document")
@@ -488,11 +707,11 @@ type backingOpenedPorts map[string]interface{}
 
 func (p *backingOpenedPorts) updated(st *State, store *multiwatcherStore, id interface{}) error {
 	localID := st.localID(id.(string))
-	parentId, ok := backingEntityIdForOpenedPortsKey(localID)
+	parentID, ok := backingEntityIdForOpenedPortsKey(localID)
 	if !ok {
 		return nil
 	}
-	switch info := store.Get(parentId).(type) {
+	switch info := store.Get(parentID).(type) {
 	case nil:
 		// The parent info doesn't exist. This is unexpected because the port
 		// always refers to a machine. Anyway, ignore the ports for now.
@@ -513,7 +732,32 @@ func (p *backingOpenedPorts) updated(st *State, store *multiwatcherStore, id int
 	return nil
 }
 
-func (p *backingOpenedPorts) removed(st *State, store *multiwatcherStore, id interface{}) {}
+func (p *backingOpenedPorts) removed(st *State, store *multiwatcherStore, id interface{}) {
+	localID := st.localID(id.(string))
+	parentID, ok := backingEntityIdForOpenedPortsKey(localID)
+	if !ok {
+		return
+	}
+	switch info := store.Get(parentID).(type) {
+	case nil:
+		// The parent info doesn't exist. This is unexpected because the port
+		// always refers to a machine. Anyway, ignore the ports for now.
+		return
+	case *multiwatcher.MachineInfo:
+		// Retrieve the units placed in the machine.
+		units, err := st.UnitsFor(info.Id)
+		if err != nil {
+			logger.Errorf("cannot retrieve units for %q: %v", info.Id, err)
+			return
+		}
+		// Update the ports on all units assigned to the machine.
+		for _, u := range units {
+			if err := updateUnitPorts(st, store, u); err != nil {
+				logger.Errorf("cannot update unit ports for %q: %v", u.Name(), err)
+			}
+		}
+	}
+}
 
 func (p *backingOpenedPorts) mongoId() interface{} {
 	panic("cannot find mongo id from openedPorts document")
@@ -568,6 +812,7 @@ func backingEntityIdForGlobalKey(key string) (multiwatcher.EntityId, bool) {
 	case 'm':
 		return (&multiwatcher.MachineInfo{Id: id}).EntityId(), true
 	case 'u':
+		id = strings.TrimSuffix(id, "#charm")
 		return (&multiwatcher.UnitInfo{Name: id}).EntityId(), true
 	case 's':
 		return (&multiwatcher.ServiceInfo{Name: id}).EntityId(), true
@@ -598,8 +843,8 @@ type backingEntityDoc interface {
 // type of value we use to store entity information
 // for that collection.
 type allWatcherStateCollection struct {
-	*mgo.Collection
-
+	// name stores the name of the collection.
+	name string
 	// infoType stores the type of the info type
 	// that we use for this collection.
 	infoType reflect.Type
@@ -616,34 +861,40 @@ func newAllWatcherStateBacking(st *State) Backing {
 	}
 
 	collections := []allWatcherStateCollection{{
-		Collection: st.db.C(machinesC),
-		infoType:   reflect.TypeOf(backingMachine{}),
+		name:     machinesC,
+		infoType: reflect.TypeOf(backingMachine{}),
 	}, {
-		Collection: st.db.C(unitsC),
-		infoType:   reflect.TypeOf(backingUnit{}),
+		name:     unitsC,
+		infoType: reflect.TypeOf(backingUnit{}),
 	}, {
-		Collection: st.db.C(servicesC),
-		infoType:   reflect.TypeOf(backingService{}),
+		name:     servicesC,
+		infoType: reflect.TypeOf(backingService{}),
 	}, {
-		Collection: st.db.C(relationsC),
-		infoType:   reflect.TypeOf(backingRelation{}),
+		name:     actionsC,
+		infoType: reflect.TypeOf(backingAction{}),
 	}, {
-		Collection: st.db.C(annotationsC),
-		infoType:   reflect.TypeOf(backingAnnotation{}),
+		name:     relationsC,
+		infoType: reflect.TypeOf(backingRelation{}),
 	}, {
-		Collection: st.db.C(statusesC),
+		name:     annotationsC,
+		infoType: reflect.TypeOf(backingAnnotation{}),
+	}, {
+		name:     blocksC,
+		infoType: reflect.TypeOf(backingBlock{}),
+	}, {
+		name:       statusesC,
 		infoType:   reflect.TypeOf(backingStatus{}),
 		subsidiary: true,
 	}, {
-		Collection: st.db.C(constraintsC),
+		name:       constraintsC,
 		infoType:   reflect.TypeOf(backingConstraints{}),
 		subsidiary: true,
 	}, {
-		Collection: st.db.C(settingsC),
+		name:       settingsC,
 		infoType:   reflect.TypeOf(backingSettings{}),
 		subsidiary: true,
 	}, {
-		Collection: st.db.C(openedPortsC),
+		name:       openedPortsC,
 		infoType:   reflect.TypeOf(backingOpenedPorts{}),
 		subsidiary: true,
 	}}
@@ -654,25 +905,30 @@ func newAllWatcherStateBacking(st *State) Backing {
 			panic(fmt.Errorf("duplicate collection type %s", docType))
 		}
 		collectionByType[docType] = c
-		if _, ok := b.collectionByName[c.Name]; ok {
-			panic(fmt.Errorf("duplicate collection name %q", c.Name))
+		if _, ok := b.collectionByName[c.name]; ok {
+			panic(fmt.Errorf("duplicate collection name %q", c.name))
 		}
-		b.collectionByName[c.Name] = c
+		b.collectionByName[c.name] = c
 	}
 	return b
 }
 
+func (b *allWatcherStateBacking) filterEnv(docID interface{}) bool {
+	_, err := b.st.strictLocalID(docID.(string))
+	return err == nil
+}
+
 // Watch watches all the collections.
 func (b *allWatcherStateBacking) Watch(in chan<- watcher.Change) {
 	for _, c := range b.collectionByName {
-		b.st.watcher.WatchCollection(c.Name, in)
+		b.st.watcher.WatchCollectionWithFilter(c.name, in, b.filterEnv)
 	}
 }
 
 // Unwatch unwatches all the collections.
 func (b *allWatcherStateBacking) Unwatch(in chan<- watcher.Change) {
 	for _, c := range b.collectionByName {
-		b.st.watcher.UnwatchCollection(c.Name, in)
+		b.st.watcher.UnwatchCollection(c.name, in)
 	}
 }
 
@@ -686,15 +942,20 @@ func (b *allWatcherStateBacking) GetAll(all *multiwatcherStore) error {
 		if c.subsidiary {
 			continue
 		}
-		col := db.C(c.Name)
+		col, closer := db.GetCollection(c.name)
+		defer closer()
 		infoSlicePtr := reflect.New(reflect.SliceOf(c.infoType))
 		if err := col.Find(nil).All(infoSlicePtr.Interface()); err != nil {
-			return fmt.Errorf("cannot get all %s: %v", c.Name, err)
+			return fmt.Errorf("cannot get all %s: %v", c.name, err)
 		}
 		infos := infoSlicePtr.Elem()
 		for i := 0; i < infos.Len(); i++ {
 			info := infos.Index(i).Addr().Interface().(backingEntityDoc)
-			info.updated(b.st, all, info.mongoId())
+			id := info.mongoId()
+			err := info.updated(b.st, all, id)
+			if err != nil {
+				return errors.Annotatef(err, "failed to initialise backing for %s:%v", c.name, id)
+			}
 		}
 	}
 	return nil
@@ -703,14 +964,12 @@ func (b *allWatcherStateBacking) GetAll(all *multiwatcherStore) error {
 // Changed updates the allWatcher's idea of the current state
 // in response to the given change.
 func (b *allWatcherStateBacking) Changed(all *multiwatcherStore, change watcher.Change) error {
-	db, closer := b.st.newDB()
-	defer closer()
-
 	c, ok := b.collectionByName[change.C]
 	if !ok {
 		panic(fmt.Errorf("unknown collection %q in fetch request", change.C))
 	}
-	col := db.C(c.Name)
+	col, closer := b.st.getCollection(c.name)
+	defer closer()
 	doc := reflect.New(c.infoType).Interface().(backingEntityDoc)
 
 	// TODO(rog) investigate ways that this can be made more efficient
