@@ -7,8 +7,9 @@ import (
 	"bytes"
 	"encoding/json"
 	"fmt"
+	"time"
 
-	"gopkg.in/juju/charm.v4"
+	"gopkg.in/juju/charm.v5"
 
 	"github.com/juju/juju/constraints"
 	"github.com/juju/juju/instance"
@@ -96,6 +97,8 @@ func (d *Delta) UnmarshalJSON(data []byte) error {
 		d.Entity = new(RelationInfo)
 	case "annotation":
 		d.Entity = new(AnnotationInfo)
+	case "block":
+		d.Entity = new(BlockInfo)
 	default:
 		return fmt.Errorf("Unexpected entity name %q", entityKind)
 	}
@@ -141,6 +144,15 @@ func (i *MachineInfo) EntityId() EntityId {
 	}
 }
 
+type StatusInfo struct {
+	Err     error
+	Current Status
+	Message string
+	Since   *time.Time
+	Version string
+	Data    map[string]interface{}
+}
+
 type ServiceInfo struct {
 	Name        string `bson:"_id"`
 	Exposed     bool
@@ -151,6 +163,7 @@ type ServiceInfo struct {
 	Constraints constraints.Value
 	Config      map[string]interface{}
 	Subordinate bool
+	Status      StatusInfo
 }
 
 func (i *ServiceInfo) EntityId() EntityId {
@@ -170,10 +183,14 @@ type UnitInfo struct {
 	MachineId      string
 	Ports          []network.Port
 	PortRanges     []network.PortRange
-	Status         Status
-	StatusInfo     string
-	StatusData     map[string]interface{}
 	Subordinate    bool
+	// The following 3 status values are deprecated.
+	Status     Status
+	StatusInfo string
+	StatusData map[string]interface{}
+	// Workload and agent state are modelled separately.
+	WorkloadStatus StatusInfo
+	AgentStatus    StatusInfo
 }
 
 func (i *UnitInfo) EntityId() EntityId {
@@ -183,6 +200,26 @@ func (i *UnitInfo) EntityId() EntityId {
 	}
 }
 
+type ActionInfo struct {
+	Id         string                 `bson:"_id"`
+	Receiver   string                 `bson:"receiver"`
+	Name       string                 `bson:"name"`
+	Parameters map[string]interface{} `bson:"parameters"`
+	Status     string                 `bson:"status"`
+	Message    string                 `bson:"message"`
+	Results    map[string]interface{} `bson:"results"`
+	Enqueued   time.Time              `bson:"enqueued"`
+	Started    time.Time              `bson:"started"`
+	Completed  time.Time              `bson:"completed"`
+}
+
+func (i *ActionInfo) EntityId() EntityId {
+	return EntityId{
+		Kind: "action",
+		Id:   i.Id,
+	}
+}
+
 type RelationInfo struct {
 	Key       string `bson:"_id"`
 	Id        int
@@ -241,3 +278,34 @@ func AnyJobNeedsState(jobs ...MachineJob) bool {
 	}
 	return false
 }
+
+// BlockInfo holds the information about blocks
+// in this environment that are watched.
+type BlockInfo struct {
+	Id      string    `bson:"_id"`
+	Type    BlockType `bson:"type"`
+	Message string    `bson:"message,omitempty"`
+	Tag     string    `bson:"tag"`
+}
+
+// EntityId returns block id.
+func (i *BlockInfo) EntityId() EntityId {
+	return EntityId{
+		Kind: "block",
+		Id:   i.Id,
+	}
+}
+
+// BlockType values define environment block type.
+type BlockType string
+
+const (
+	// BlockDestroy type identifies destroy blocks.
+	BlockDestroy BlockType = "BlockDestroy"
+
+	// BlockRemove type identifies remove blocks.
+	BlockRemove BlockType = "BlockRemove"
+
+	// BlockChange type identifies change blocks.
+	BlockChange BlockType = "BlockChange"
+)
