@@ -9,27 +9,28 @@ import (
 	"sort"
 	"strconv"
 	"strings"
+	"time"
 
 	"github.com/juju/errors"
 	"github.com/juju/names"
 	jujutxn "github.com/juju/txn"
-	"gopkg.in/juju/charm.v4"
+	"gopkg.in/juju/charm.v5"
 	"gopkg.in/mgo.v2"
 	"gopkg.in/mgo.v2/bson"
 	"gopkg.in/mgo.v2/txn"
 
 	"github.com/juju/juju/constraints"
+	"github.com/juju/juju/leadership"
 )
 
 // Service represents the state of a service.
 type Service struct {
 	st  *State
 	doc serviceDoc
-	annotator
 }
 
 // serviceDoc represents the internal state of a service in MongoDB.
-// Note the correspondence with ServiceInfo in apiserver/params.
+// Note the correspondence with ServiceInfo in apiserver.
 type serviceDoc struct {
 	DocID             string     `bson:"_id"`
 	Name              string     `bson:"name"`
@@ -54,11 +55,6 @@ func newService(st *State, doc *serviceDoc) *Service {
 		st:  st,
 		doc: *doc,
 	}
-	svc.annotator = annotator{
-		globalKey: svc.globalKey(),
-		tag:       svc.Tag(),
-		st:        st,
-	}
 	return svc
 }
 
@@ -215,23 +211,29 @@ func (s *Service) destroyOps() ([]txn.Op, error) {
 // asserts will be included in the operation on the service document.
 func (s *Service) removeOps(asserts bson.D) []txn.Op {
 	settingsDocID := s.st.docID(s.settingsKey())
-	ops := []txn.Op{{
-		C:      servicesC,
-		Id:     s.doc.DocID,
-		Assert: asserts,
-		Remove: true,
-	}, {
-		C:      settingsrefsC,
-		Id:     settingsDocID,
-		Remove: true,
-	}, {
-		C:      settingsC,
-		Id:     settingsDocID,
-		Remove: true,
-	}}
-	ops = append(ops, removeRequestedNetworksOp(s.st, s.globalKey()))
-	ops = append(ops, removeConstraintsOp(s.st, s.globalKey()))
-	return append(ops, annotationRemoveOp(s.st, s.globalKey()))
+	ops := []txn.Op{
+		{
+			C:      servicesC,
+			Id:     s.doc.DocID,
+			Assert: asserts,
+			Remove: true,
+		}, {
+			C:      settingsrefsC,
+			Id:     settingsDocID,
+			Remove: true,
+		}, {
+			C:      settingsC,
+			Id:     settingsDocID,
+			Remove: true,
+		},
+		removeRequestedNetworksOp(s.st, s.globalKey()),
+		removeStorageConstraintsOp(s.globalKey()),
+		removeConstraintsOp(s.st, s.globalKey()),
+		annotationRemoveOp(s.st, s.globalKey()),
+		removeLeadershipSettingsOp(s.Tag().Id()),
+		removeStatusOp(s.st, s.globalKey()),
+	}
+	return ops
 }
 
 // IsExposed returns whether this service is exposed. The explicitly open
@@ -373,6 +375,79 @@ func (s *Service) checkRelationsOps(ch *Charm, relations []*Relation) ([]txn.Op,
 	return asserts, nil
 }
 
+func (s *Service) checkStorageUpgrade(newMeta *charm.Meta) (err error) {
+	defer errors.DeferredAnnotatef(&err, "cannot upgrade service %q to charm %q", s, newMeta.Name)
+	ch, _, err := s.Charm()
+	if err != nil {
+		return errors.Trace(err)
+	}
+	oldMeta := ch.Meta()
+	for name := range oldMeta.Storage {
+		if _, ok := newMeta.Storage[name]; !ok {
+			return errors.Errorf("storage %q removed", name)
+		}
+	}
+	less := func(a, b int) bool {
+		return a != -1 && (b == -1 || a < b)
+	}
+	for name, newStorageMeta := range newMeta.Storage {
+		oldStorageMeta, ok := oldMeta.Storage[name]
+		if !ok {
+			if newStorageMeta.CountMin > 0 {
+				return errors.Errorf("required storage %q added", name)
+			}
+			// New storage is fine as long as it is not required.
+			//
+			// TODO(axw) introduce a way of adding storage at
+			// upgrade time. We should also look at supplying
+			// a way of adding/changing other things during
+			// upgrade, e.g. changing service config.
+			continue
+		}
+		if newStorageMeta.Type != oldStorageMeta.Type {
+			return errors.Errorf(
+				"existing storage %q type changed from %q to %q",
+				name, oldStorageMeta.Type, newStorageMeta.Type,
+			)
+		}
+		if newStorageMeta.Shared != oldStorageMeta.Shared {
+			return errors.Errorf(
+				"existing storage %q shared changed from %v to %v",
+				name, oldStorageMeta.Shared, newStorageMeta.Shared,
+			)
+		}
+		if newStorageMeta.ReadOnly != oldStorageMeta.ReadOnly {
+			return errors.Errorf(
+				"existing storage %q read-only changed from %v to %v",
+				name, oldStorageMeta.ReadOnly, newStorageMeta.ReadOnly,
+			)
+		}
+		if newStorageMeta.Location != oldStorageMeta.Location {
+			return errors.Errorf(
+				"existing storage %q location changed from %q to %q",
+				name, oldStorageMeta.Location, newStorageMeta.Location,
+			)
+		}
+		if newStorageMeta.CountMin > oldStorageMeta.CountMin {
+			return errors.Errorf(
+				"existing storage %q range contracted: min increased from %d to %d",
+				name, oldStorageMeta.CountMin, newStorageMeta.CountMin,
+			)
+		}
+		if less(newStorageMeta.CountMax, oldStorageMeta.CountMax) {
+			var oldCountMax interface{} = oldStorageMeta.CountMax
+			if oldStorageMeta.CountMax == -1 {
+				oldCountMax = "<unbounded>"
+			}
+			return errors.Errorf(
+				"existing storage %q range contracted: max decreased from %v to %d",
+				name, oldCountMax, newStorageMeta.CountMax,
+			)
+		}
+	}
+	return nil
+}
+
 // changeCharmOps returns the operations necessary to set a service's
 // charm URL to a new value.
 func (s *Service) changeCharmOps(ch *Charm, force bool) ([]txn.Op, error) {
@@ -469,6 +544,12 @@ func (s *Service) changeCharmOps(ch *Charm, force bool) ([]txn.Op, error) {
 	}
 	ops = append(ops, relOps...)
 
+	// Check storage to ensure no storage is removed, and no required
+	// storage is added for which there are no constraints.
+	if err := s.checkStorageUpgrade(ch.Meta()); err != nil {
+		return nil, errors.Trace(err)
+	}
+
 	// And finally, decrement the old settings.
 	return append(ops, decOps...), nil
 }
@@ -569,6 +650,8 @@ func (s *Service) newUnitName() (string, error) {
 	return name, nil
 }
 
+const MessageWaitForAgentInit = "Waiting for agent initialization to finish"
+
 // addUnitOps returns a unique name for a new unit, and a list of txn operations
 // necessary to create that unit. The principalName param must be non-empty if
 // and only if s is a subordinate service. Only one subordinate of a given
@@ -584,36 +667,59 @@ func (s *Service) addUnitOps(principalName string, asserts bson.D) (string, []tx
 	if err != nil {
 		return "", nil, err
 	}
+
+	// Create instances of the charm's declared stores.
+	storageOps, numStorageAttachments, err := s.unitStorageOps(name)
+	if err != nil {
+		return "", nil, errors.Trace(err)
+	}
+
 	docID := s.st.docID(name)
 	globalKey := unitGlobalKey(name)
+	agentGlobalKey := unitAgentGlobalKey(name)
+	meterStatusGlobalKey := unitAgentGlobalKey(name)
 	udoc := &unitDoc{
-		DocID:     docID,
-		Name:      name,
-		EnvUUID:   s.doc.EnvUUID,
-		Service:   s.doc.Name,
-		Series:    s.doc.Series,
-		Life:      Alive,
-		Principal: principalName,
+		DocID:                  docID,
+		Name:                   name,
+		EnvUUID:                s.doc.EnvUUID,
+		Service:                s.doc.Name,
+		Series:                 s.doc.Series,
+		Life:                   Alive,
+		Principal:              principalName,
+		StorageAttachmentCount: numStorageAttachments,
 	}
-	sdoc := statusDoc{
+	now := time.Now()
+	agentStatusDoc := statusDoc{
 		Status:  StatusAllocating,
+		Updated: now.UnixNano(),
 		EnvUUID: s.st.EnvironUUID(),
 	}
+	unitStatusDoc := statusDoc{
+		// TODO(fwereade): this violates the spec. Should be "waiting".
+		Status:     StatusUnknown,
+		StatusInfo: MessageWaitForAgentInit,
+		Updated:    now.UnixNano(),
+		EnvUUID:    s.st.EnvironUUID(),
+	}
 	ops := []txn.Op{
+		createStatusOp(s.st, globalKey, unitStatusDoc),
+		createStatusOp(s.st, agentGlobalKey, agentStatusDoc),
+		createMeterStatusOp(s.st, meterStatusGlobalKey, &meterStatusDoc{Code: MeterNotSet.String()}),
 		{
 			C:      unitsC,
 			Id:     docID,
 			Assert: txn.DocMissing,
 			Insert: udoc,
 		},
-		createStatusOp(s.st, globalKey, sdoc),
-		createMeterStatusOp(s.st, globalKey, &meterStatusDoc{Code: MeterNotSet}),
 		{
 			C:      servicesC,
 			Id:     s.doc.DocID,
 			Assert: append(isAliveDoc, asserts...),
 			Update: bson.D{{"$inc", bson.D{{"unitcount", 1}}}},
-		}}
+		},
+	}
+	ops = append(ops, storageOps...)
+
 	if s.doc.Subordinate {
 		ops = append(ops, txn.Op{
 			C:  unitsC,
@@ -632,11 +738,45 @@ func (s *Service) addUnitOps(principalName string, asserts bson.D) (string, []tx
 		if err != nil {
 			return "", nil, err
 		}
-		ops = append(ops, createConstraintsOp(s.st, globalKey, cons))
+		ops = append(ops, createConstraintsOp(s.st, agentGlobalKey, cons))
 	}
+
+	// At the last moment we still have the statusDocs in scope, set the initial
+	// history entries. This is risky, and may lead to extra entries, but that's
+	// an intrinsic problem with mixing txn and non-txn ops -- we can't sync
+	// them cleanly.
+	probablyUpdateStatusHistory(s.st, globalKey, unitStatusDoc)
+	probablyUpdateStatusHistory(s.st, agentGlobalKey, agentStatusDoc)
 	return name, ops, nil
 }
 
+// unitStorageOps returns operations for creating storage
+// instances and attachments for a new unit. unitStorageOps
+// returns the number of initial storage attachments, to
+// initialise the unit's storage attachment refcount.
+func (s *Service) unitStorageOps(unitName string) (ops []txn.Op, numStorageAttachments int, err error) {
+	cons, err := s.StorageConstraints()
+	if err != nil {
+		return nil, -1, err
+	}
+	charm, _, err := s.Charm()
+	if err != nil {
+		return nil, -1, err
+	}
+	meta := charm.Meta()
+	url := charm.URL()
+	tag := names.NewUnitTag(unitName)
+	// TODO(wallyworld) - record constraints info in data model - size and pool name
+	ops, numStorageAttachments, err = createStorageOps(
+		s.st, tag, meta, url, cons,
+		false, // unit is not assigned yet; don't create machine storage
+	)
+	if err != nil {
+		return nil, -1, errors.Trace(err)
+	}
+	return ops, numStorageAttachments, nil
+}
+
 // SCHEMACHANGE
 // TODO(mattyw) remove when schema upgrades are possible
 func (s *Service) GetOwnerTag() string {
@@ -680,24 +820,31 @@ func (s *Service) removeUnitOps(u *Unit, asserts bson.D) ([]txn.Op, error) {
 	if err != nil {
 		return nil, err
 	}
+	storageInstanceOps, err := removeStorageInstancesOps(s.st, u.Tag())
+	if err != nil {
+		return nil, err
+	}
 
 	observedFieldsMatch := bson.D{
 		{"charmurl", u.doc.CharmURL},
 		{"machineid", u.doc.MachineId},
 	}
-	ops = append(ops, txn.Op{
-		C:      unitsC,
-		Id:     u.doc.DocID,
-		Assert: append(observedFieldsMatch, asserts...),
-		Remove: true,
-	},
-		removeConstraintsOp(s.st, u.globalKey()),
+	ops = append(ops,
+		txn.Op{
+			C:      unitsC,
+			Id:     u.doc.DocID,
+			Assert: append(observedFieldsMatch, asserts...),
+			Remove: true,
+		},
+		removeMeterStatusOp(s.st, u.globalMeterStatusKey()),
+		removeStatusOp(s.st, u.globalAgentKey()),
 		removeStatusOp(s.st, u.globalKey()),
-		removeMeterStatusOp(s.st, u.globalKey()),
+		removeConstraintsOp(s.st, u.globalAgentKey()),
 		annotationRemoveOp(s.st, u.globalKey()),
 		s.st.newCleanupOp(cleanupRemovedUnit, u.doc.Name),
 	)
 	ops = append(ops, portsOps...)
+	ops = append(ops, storageInstanceOps...)
 	if u.doc.CharmURL != nil {
 		decOps, err := settingsDecRefOps(s.st, s.doc.Name, u.doc.CharmURL)
 		if errors.IsNotFound(err) {
@@ -802,17 +949,100 @@ func (s *Service) UpdateConfigSettings(changes charm.Settings) error {
 	if err != nil {
 		return err
 	}
-	for name, value := range changes {
-		if value == nil {
-			node.Delete(name)
-		} else {
-			node.Set(name, value)
-		}
-	}
+	node.apply(changes)
 	_, err = node.Write()
 	return err
 }
 
+// LeaderSettings returns a service's leader settings. If nothing has been set
+// yet, it will return an empty map; this is not an error.
+func (s *Service) LeaderSettings() (map[string]string, error) {
+	// There's no compelling reason to have these methods on Service -- and
+	// thus require an extra db read to access them -- but it stops the State
+	// type getting even more cluttered.
+
+	docId := leadershipSettingsDocId(s.doc.Name)
+	rawMap, _, err := readSettingsDoc(s.st, docId)
+	if cause := errors.Cause(err); cause == mgo.ErrNotFound {
+		return nil, errors.NotFoundf("service")
+	} else if err != nil {
+		return nil, errors.Trace(err)
+	}
+	result := make(map[string]string)
+	for escapedKey, interfaceValue := range rawMap {
+		key := unescapeReplacer.Replace(escapedKey)
+		if value, _ := interfaceValue.(string); value != "" {
+			// Empty strings are technically bad data -- when set, they clear.
+			result[key] = value
+		} else {
+			// Some bad data isn't reason enough to obscure the good data.
+			logger.Warningf("unexpected leader settings value for %s: %#v", key, interfaceValue)
+		}
+	}
+	return result, nil
+}
+
+// UpdateLeaderSettings updates the service's leader settings with the supplied
+// values, but will fail (with a suitable error) if the supplied Token loses
+// validity. Empty values in the supplied map will be cleared in the database.
+func (s *Service) UpdateLeaderSettings(token leadership.Token, updates map[string]string) error {
+	// There's no compelling reason to have these methods on Service -- and
+	// thus require an extra db read to access them -- but it stops the State
+	// type getting even more cluttered.
+
+	// We can calculate the actual update ahead of time; it's not dependent
+	// upon the current state of the document. (*Writing* it should depend
+	// on document state, but that's handled below.)
+	docId := leadershipSettingsDocId(s.doc.Name)
+	sets := bson.M{}
+	unsets := bson.M{}
+	for unescapedKey, value := range updates {
+		key := escapeReplacer.Replace(unescapedKey)
+		if value == "" {
+			unsets[key] = 1
+		} else {
+			sets[key] = value
+		}
+	}
+	update := setUnsetUpdate(sets, unsets)
+
+	isNullChange := func(rawMap map[string]interface{}) bool {
+		for key := range unsets {
+			if _, found := rawMap[key]; found {
+				return false
+			}
+		}
+		for key, value := range sets {
+			if current := rawMap[key]; current != value {
+				return false
+			}
+		}
+		return true
+	}
+
+	buildTxn := func(_ int) ([]txn.Op, error) {
+		// Read the current document state so we can abort if there's
+		// no actual change; and the txn-revno so we can assert on it
+		// and prevent these settings from landing late.
+		rawMap, txnRevno, err := readSettingsDoc(s.st, docId)
+		if cause := errors.Cause(err); cause == mgo.ErrNotFound {
+			return nil, errors.NotFoundf("service")
+		} else if err != nil {
+			return nil, errors.Trace(err)
+		}
+		if isNullChange(rawMap) {
+			return nil, jujutxn.ErrNoOperations
+		}
+		return []txn.Op{{
+			C:      settingsC,
+			Id:     docId,
+			Assert: bson.D{{"txn-revno", txnRevno}},
+			Update: update,
+		}}, nil
+	}
+	return s.st.run(buildTxnWithLeadership(buildTxn, token))
+}
+
 var ErrSubordinateConstraints = stderrors.New("constraints do not apply to subordinate services")
 
 // Constraints returns the current service constraints.
@@ -893,6 +1123,10 @@ func (s *Service) SetMetricCredentials(b []byte) error {
 	return nil
 }
 
+func (s *Service) StorageConstraints() (map[string]StorageConstraints, error) {
+	return readStorageConstraints(s.st, s.globalKey())
+}
+
 // settingsIncRefOp returns an operation that increments the ref count
 // of the service settings identified by serviceName and curl. If
 // canCreate is false, a missing document will be treated as an error;
@@ -980,3 +1214,103 @@ type settingsRefsDoc struct {
 	RefCount int
 	EnvUUID  string `bson:"env-uuid"`
 }
+
+// Status returns the status of the service.
+// Only unit leaders are allowed to set the status of the service.
+// If no status is recorded, then there are no unit leaders and the
+// status is derived from the unit status values.
+func (s *Service) Status() (StatusInfo, error) {
+	statuses, closer := s.st.getCollection(statusesC)
+	defer closer()
+	query := statuses.Find(bson.D{{"_id", s.globalKey()}, {"neverset", true}})
+	if count, err := query.Count(); err != nil {
+		return StatusInfo{}, errors.Trace(err)
+	} else if count != 0 {
+		// This indicates that SetStatus has never been called on this service.
+		// This in turn implies the service status document is likely to be
+		// inaccurate, so we return aggregated unit statuses instead.
+		//
+		// TODO(fwereade): this is completely wrong and will produce bad results
+		// in not-very-challenging scenarios. The leader unit remains responsible
+		// for setting the service status in a timely way, *whether or not the
+		// charm's hooks exists or sets a service status*. This logic should be
+		// removed as soon as possible, and the responsibilities implemented in
+		// the right places rather than being applied at seeming random.
+		units, err := s.AllUnits()
+		if err != nil {
+			return StatusInfo{}, err
+		}
+		logger.Tracef("service %q has %d units", s.Name(), len(units))
+		if len(units) > 0 {
+			return s.deriveStatus(units)
+		}
+	}
+	return getStatus(s.st, s.globalKey(), "service")
+}
+
+// SetStatus sets the status for the service.
+func (s *Service) SetStatus(status Status, info string, data map[string]interface{}) error {
+	if !ValidWorkloadStatus(status) {
+		return errors.Errorf("cannot set invalid status %q", status)
+	}
+	return setStatus(s.st, setStatusParams{
+		badge:     "service",
+		globalKey: s.globalKey(),
+		status:    status,
+		message:   info,
+		rawData:   data,
+	})
+}
+
+// ServiceAndUnitsStatus returns the status for this service and all its units.
+func (s *Service) ServiceAndUnitsStatus() (StatusInfo, map[string]StatusInfo, error) {
+	serviceStatus, err := s.Status()
+	if err != nil {
+		return StatusInfo{}, nil, errors.Trace(err)
+	}
+	units, err := s.AllUnits()
+	if err != nil {
+		return StatusInfo{}, nil, err
+	}
+	results := make(map[string]StatusInfo, len(units))
+	for _, unit := range units {
+		unitStatus, err := unit.Status()
+		if err != nil {
+			return StatusInfo{}, nil, err
+		}
+		results[unit.Name()] = unitStatus
+	}
+	return serviceStatus, results, nil
+
+}
+
+func (s *Service) deriveStatus(units []*Unit) (StatusInfo, error) {
+	var result StatusInfo
+	for _, unit := range units {
+		currentSeverity := statusServerities[result.Status]
+		unitStatus, err := unit.Status()
+		if err != nil {
+			return StatusInfo{}, errors.Annotatef(err, "deriving service status from %q", unit.Name())
+		}
+		unitSeverity := statusServerities[unitStatus.Status]
+		if unitSeverity > currentSeverity {
+			result.Status = unitStatus.Status
+			result.Message = unitStatus.Message
+			result.Data = unitStatus.Data
+			result.Since = unitStatus.Since
+		}
+	}
+	return result, nil
+}
+
+// statusSeverities holds status values with a severity measure.
+// Status values with higher severity are used in preference to others.
+var statusServerities = map[Status]int{
+	StatusError:       100,
+	StatusBlocked:     90,
+	StatusWaiting:     80,
+	StatusMaintenance: 70,
+	StatusTerminated:  60,
+	StatusActive:      50,
+	StatusUnknown:     40,
+}
