@@ -118,6 +118,18 @@ func (c *Settings) Delete(key string) {
 	delete(c.core, key)
 }
 
+// apply updates the Settings with the provided changes. Any requested
+// setting with a nil value will be removed.
+func (c *Settings) apply(changes map[string]interface{}) {
+	for name, value := range changes {
+		if value == nil {
+			c.Delete(name)
+		} else {
+			c.Set(name, value)
+		}
+	}
+}
+
 // cacheKeys returns the keys of all caches as a key=>true map.
 func cacheKeys(caches ...map[string]interface{}) map[string]bool {
 	keys := make(map[string]bool)
@@ -266,6 +278,11 @@ func readSettingsDoc(st *State, key string) (map[string]interface{}, int64, erro
 	return config, txnRevno, nil
 }
 
+// ReadSettings returns the settings for the given key.
+func (st *State) ReadSettings(key string) (*Settings, error) {
+	return readSettings(st, key)
+}
+
 // readSettings returns the Settings for key.
 func readSettings(st *State, key string) (*Settings, error) {
 	s := newSettings(st, key)
@@ -305,16 +322,40 @@ func createSettings(st *State, key string, values map[string]interface{}) (*Sett
 
 // removeSettings removes the Settings for key.
 func removeSettings(st *State, key string) error {
-	settings, closer := st.getCollection(settingsC)
-	defer closer()
-
-	err := settings.RemoveId(key)
-	if err == mgo.ErrNotFound {
+	ops := []txn.Op{{
+		C:      settingsC,
+		Id:     st.docID(key),
+		Assert: txn.DocExists,
+		Remove: true,
+	}}
+	err := st.runTransaction(ops)
+	if err == txn.ErrAborted {
 		return errors.NotFoundf("settings")
+	} else if err != nil {
+		return errors.Trace(err)
 	}
 	return nil
 }
 
+// listSettings returns all the settings with the specified key prefix.
+func listSettings(st *State, keyPrefix string) (map[string]map[string]interface{}, error) {
+	settings, closer := st.getRawCollection(settingsC)
+	defer closer()
+
+	var matchingSettings []map[string]interface{}
+	findExpr := fmt.Sprintf("^%s.*$", st.docID(keyPrefix))
+	if err := settings.Find(bson.D{{"_id", bson.D{{"$regex", findExpr}}}}).All(&matchingSettings); err != nil {
+		return nil, err
+	}
+	result := make(map[string]map[string]interface{})
+	for i := range matchingSettings {
+		id := matchingSettings[i]["_id"].(string)
+		cleanSettingsMap(matchingSettings[i])
+		result[st.localID(id)] = matchingSettings[i]
+	}
+	return result, nil
+}
+
 // replaceSettingsOp returns a txn.Op that deletes the document's contents and
 // replaces it with the supplied values, and a function that should be called on
 // txn failure to determine whether this operation failed (due to a concurrent
@@ -365,3 +406,38 @@ func setUnsetUpdate(set, unset bson.M) bson.D {
 	}
 	return update
 }
+
+// StateSettings is used to expose various settings APIs outside of the state package.
+type StateSettings struct {
+	st *State
+}
+
+// NewStateSettings creates a StateSettings from state.
+func NewStateSettings(st *State) *StateSettings {
+	return &StateSettings{st}
+}
+
+// CreateSettings exposes createSettings on state for use outside the state package.
+func (s *StateSettings) CreateSettings(key string, settings map[string]interface{}) error {
+	_, err := createSettings(s.st, key, settings)
+	return err
+}
+
+// ReadSettings exposes readSettings on state for use outside the state package.
+func (s *StateSettings) ReadSettings(key string) (map[string]interface{}, error) {
+	if settings, err := readSettings(s.st, key); err != nil {
+		return nil, err
+	} else {
+		return settings.Map(), nil
+	}
+}
+
+// RemoveSettings exposes removeSettings on state for use outside the state package.
+func (s *StateSettings) RemoveSettings(key string) error {
+	return removeSettings(s.st, key)
+}
+
+// ListSettings exposes listSettings on state for use outside the state package.
+func (s *StateSettings) ListSettings(keyPrefix string) (map[string]map[string]interface{}, error) {
+	return listSettings(s.st, keyPrefix)
+}
