@@ -14,13 +14,14 @@ import (
 	"strconv"
 	"strings"
 	"sync"
+	"time"
 
 	"github.com/juju/errors"
 	"github.com/juju/loggo"
 	"github.com/juju/names"
 	jujutxn "github.com/juju/txn"
 	"github.com/juju/utils"
-	"gopkg.in/juju/charm.v4"
+	"gopkg.in/juju/charm.v5"
 	"gopkg.in/mgo.v2"
 	"gopkg.in/mgo.v2/bson"
 	"gopkg.in/mgo.v2/txn"
@@ -29,6 +30,8 @@ import (
 	"github.com/juju/juju/environs/config"
 	"github.com/juju/juju/mongo"
 	"github.com/juju/juju/network"
+	"github.com/juju/juju/state/leadership"
+	"github.com/juju/juju/state/lease"
 	"github.com/juju/juju/state/presence"
 	"github.com/juju/juju/state/watcher"
 	"github.com/juju/juju/version"
@@ -37,92 +40,40 @@ import (
 var logger = loggo.GetLogger("juju.state")
 
 const (
-	// The following define the mongo collections used to record the Juju environment state.
-	environmentsC      = "environments"
-	charmsC            = "charms"
-	machinesC          = "machines"
-	containerRefsC     = "containerRefs"
-	instanceDataC      = "instanceData"
-	relationsC         = "relations"
-	relationScopesC    = "relationscopes"
-	servicesC          = "services"
-	requestedNetworksC = "requestednetworks"
-	networksC          = "networks"
-	networkInterfacesC = "networkinterfaces"
-	minUnitsC          = "minunits"
-	settingsC          = "settings"
-	settingsrefsC      = "settingsrefs"
-	constraintsC       = "constraints"
-	unitsC             = "units"
-	subnetsC           = "subnets"
-	ipaddressesC       = "ipaddresses"
+	// jujuDB is the name of the main juju database.
+	jujuDB = "juju"
 
-	// actionsC and related collections store state of Actions that
-	// have been enqueued.
-	actionsC = "actions"
-	// actionNotificationsC are only used for notification of newly
-	// enqueued Actions.
-	actionNotificationsC = "actionnotifications"
-	// actionResultsC is deprecated and will soon be folded into
-	// actionsC.
-	actionresultsC = "actionresults"
-
-	usersC        = "users"
-	envUsersC     = "envusers"
-	presenceC     = "presence"
-	cleanupsC     = "cleanups"
-	annotationsC  = "annotations"
-	statusesC     = "statuses"
-	stateServersC = "stateServers"
-	openedPortsC  = "openedPorts"
-	metricsC      = "metrics"
-	upgradeInfoC  = "upgradeInfo"
-	rebootC       = "reboot"
-	blockDevicesC = "blockdevices"
-	datastoresC   = "datastores"
-
-	// leaseC is used to store lease tokens
-	leaseC = "lease"
-
-	// sequenceC is used to generate unique identifiers.
-	sequenceC = "sequence"
-
-	// meterStatusC is the collection used to store meter status information.
-	meterStatusC = "meterStatus"
-
-	// toolsmetadataC is the collection used to store tools metadata.
-	toolsmetadataC = "toolsmetadata"
-
-	// These collections are used by the mgo transaction runner.
-	txnLogC = "txns.log"
-	txnsC   = "txns"
+	// presenceDB is the name of the database used to hold presence pinger data.
+	presenceDB = "presence"
+	presenceC  = "presence"
 
 	// blobstoreDB is the name of the blobstore GridFS database.
 	blobstoreDB = "blobstore"
 
-	// restoreInfoC is used to track restore progress
-	restoreInfoC = "restoreInfo"
+	// serviceLeadershipNamespace is the name of the lease.Client namespace
+	// used by the leadership manager.
+	serviceLeadershipNamespace = "service-leadership"
 )
 
 // State represents the state of an environment
 // managed by juju.
 type State struct {
-	// TODO(katco-): As state gets broken up, remove this.
-	*LeasePersistor
-	// transactionRunner is normally nil, which means that a new one
-	// will be created for each operation, ensuring a fresh mgo.Session
-	// is used. However, for tests, a value may be assigned and this will
-	// be used instead of creating a new runnner each time.
-	transactionRunner jujutxn.Runner
-	mongoInfo         *mongo.MongoInfo
-	policy            Policy
-	db                *mgo.Database
+	environTag names.EnvironTag
+	serverTag  names.EnvironTag
+	mongoInfo  *mongo.MongoInfo
+	session    *mgo.Session
+	database   Database
+	policy     Policy
+
+	// TODO(fwereade): move these out of state and make them independent
+	// workers on which state depends.
 	watcher           *watcher.Watcher
 	pwatcher          *presence.Watcher
+	leadershipManager leadership.ManagerWorker
+
 	// mu guards allManager.
 	mu         sync.Mutex
 	allManager *storeManager
-	environTag names.EnvironTag
 }
 
 // StateServingInfo holds information needed by a state server.
@@ -139,17 +90,122 @@ type StateServingInfo struct {
 	SystemIdentity string
 }
 
+// IsStateServer returns true if this state instance has the bootstrap
+// environment UUID.
+func (st *State) IsStateServer() bool {
+	return st.environTag == st.serverTag
+}
+
+// RemoveAllEnvironDocs removes all documents from multi-environment
+// collections. The environment should be put into a dying state before call
+// this method. Otherwise, there is a race condition in which collections
+// could be added to during or after the running of this method.
+func (st *State) RemoveAllEnvironDocs() error {
+	env, err := st.Environment()
+	if err != nil {
+		return errors.Trace(err)
+	}
+	id := userEnvNameIndex(env.Owner().Username(), env.Name())
+	ops := []txn.Op{{
+		// Cleanup the owner:envName unique key.
+		C:      userenvnameC,
+		Id:     id,
+		Remove: true,
+	}, {
+		C:      environmentsC,
+		Id:     st.EnvironUUID(),
+		Remove: true,
+	}}
+
+	// Add all per-environment docs to the txn.
+	for name, info := range st.database.Schema() {
+		if info.global {
+			continue
+		}
+		coll, closer := st.getCollection(name)
+		defer closer()
+
+		var ids []bson.M
+		err := coll.Find(nil).Select(bson.D{{"_id", 1}}).All(&ids)
+		if err != nil {
+			return errors.Trace(err)
+		}
+		for _, id := range ids {
+			ops = append(ops, txn.Op{
+				C:      name,
+				Id:     id["_id"],
+				Remove: true,
+			})
+		}
+	}
+
+	return st.runTransaction(ops)
+}
+
 // ForEnviron returns a connection to mongo for the specified environment. The
-// connection uses the same credentails and policy as the existing connection.
+// connection uses the same credentials and policy as the existing connection.
 func (st *State) ForEnviron(env names.EnvironTag) (*State, error) {
-	newState, err := open(st.mongoInfo, mongo.DialOpts{}, st.policy)
+	newState, err := open(env, st.mongoInfo, mongo.DefaultDialOpts(), st.policy)
 	if err != nil {
 		return nil, errors.Trace(err)
 	}
-	newState.environTag = env
+	if err := newState.start(st.serverTag); err != nil {
+		return nil, errors.Trace(err)
+	}
 	return newState, nil
 }
 
+// start starts the presence watcher and leadership manager, and fills in the
+// serverTag field with the supplied value.
+func (st *State) start(serverTag names.EnvironTag) error {
+	st.serverTag = serverTag
+
+	var clientId string
+	if identity := st.mongoInfo.Tag; identity != nil {
+		// TODO(fwereade): it feels a bit wrong to take this from MongoInfo -- I
+		// think it's just coincidental that the mongodb user happens to map to
+		// the machine that's executing the code -- but there doesn't seem to be
+		// an accessible alternative.
+		clientId = identity.String()
+	} else {
+		// If we're running state anonymously, we can still use the lease
+		// manager; but we need to make sure we use a unique client ID, and
+		// will thus not be very performant.
+		logger.Infof("running state anonymously; using unique client id")
+		uuid, err := utils.NewUUID()
+		if err != nil {
+			return errors.Trace(err)
+		}
+		clientId = fmt.Sprintf("anon-%s", uuid.String())
+	}
+
+	logger.Infof("creating lease client as %s", clientId)
+	clock := GetClock()
+	leaseClient, err := lease.NewClient(lease.ClientConfig{
+		Id:         clientId,
+		Namespace:  serviceLeadershipNamespace,
+		Collection: leasesC,
+		Mongo:      &environMongo{st},
+		Clock:      clock,
+	})
+	if err != nil {
+		return errors.Annotatef(err, "cannot create lease client")
+	}
+	logger.Infof("starting leadership manager")
+	leadershipManager, err := leadership.NewManager(leadership.ManagerConfig{
+		Client: leaseClient,
+		Clock:  clock,
+	})
+	if err != nil {
+		return errors.Annotatef(err, "cannot create leadership manager")
+	}
+	st.leadershipManager = leadershipManager
+
+	logger.Infof("starting presence watcher")
+	st.pwatcher = presence.NewWatcher(st.getPresence(), st.environTag)
+	return nil
+}
+
 // EnvironTag() returns the environment tag for the environment controlled by
 // this state instance.
 func (st *State) EnvironTag() names.EnvironTag {
@@ -162,24 +218,64 @@ func (st *State) EnvironUUID() string {
 	return st.environTag.Id()
 }
 
-// getPresence returns the presence collection.
+// userEnvNameIndex returns a string to be used as a userenvnameC unique index.
+func userEnvNameIndex(username, envName string) string {
+	return strings.ToLower(username) + ":" + envName
+}
+
+// EnsureEnvironmentRemoved returns an error if any multi-environment
+// documents for this environment are found. It is intended only to be used in
+// tests and exported so it can be used in the tests of other packages.
+func (st *State) EnsureEnvironmentRemoved() error {
+	found := map[string]int{}
+	var foundOrdered []string
+	for name, info := range st.database.Schema() {
+		if info.global {
+			continue
+		}
+		coll, closer := st.getCollection(name)
+		defer closer()
+		n, err := coll.Find(nil).Count()
+		if err != nil {
+			return errors.Trace(err)
+		}
+		if n != 0 {
+			found[name] = n
+			foundOrdered = append(foundOrdered, name)
+		}
+	}
+
+	if len(found) != 0 {
+		errMessage := fmt.Sprintf("found documents for environment with uuid %s:", st.EnvironUUID())
+		sort.Strings(foundOrdered)
+		for _, name := range foundOrdered {
+			number := found[name]
+			errMessage += fmt.Sprintf(" %d %s doc,", number, name)
+		}
+		// Remove trailing comma.
+		errMessage = errMessage[:len(errMessage)-1]
+		return errors.New(errMessage)
+	}
+	return nil
+}
+
+// getPresence returns the presence m.
 func (st *State) getPresence() *mgo.Collection {
-	return st.db.Session.DB("presence").C(presenceC)
+	return st.session.DB(presenceDB).C(presenceC)
 }
 
 // newDB returns a database connection using a new session, along with
 // a closer function for the session. This is useful where you need to work
 // with various collections in a single session, so don't want to call
 // getCollection multiple times.
-func (st *State) newDB() (*mgo.Database, func()) {
-	session := st.db.Session.Copy()
-	return st.db.With(session), session.Close
+func (st *State) newDB() (Database, func()) {
+	return st.database.CopySession()
 }
 
 // Ping probes the state's database connection to ensure
 // that it is still alive.
 func (st *State) Ping() error {
-	return st.db.Session.Ping()
+	return st.session.Ping()
 }
 
 // MongoSession returns the underlying mongodb session
@@ -187,7 +283,7 @@ func (st *State) Ping() error {
 // can maintain the mongo replica set and should not
 // otherwise be used.
 func (st *State) MongoSession() *mgo.Session {
-	return st.db.Session
+	return st.session
 }
 
 type closeFunc func()
@@ -269,7 +365,8 @@ func (st *State) checkCanUpgrade(currentVersion, newVersion string) error {
 	}}}
 	var agentTags []string
 	for _, name := range []string{machinesC, unitsC} {
-		collection := db.C(name)
+		collection, closer := db.GetCollection(name)
+		defer closer()
 		var doc struct {
 			DocID string `bson:"_id"`
 		}
@@ -441,17 +538,6 @@ func (st *State) SetEnvironConstraints(cons constraints.Value) error {
 	return writeConstraints(st, environGlobalKey, cons)
 }
 
-var ErrDead = fmt.Errorf("not found or dead")
-var errNotAlive = fmt.Errorf("not found or not alive")
-
-func onAbort(txnErr, err error) error {
-	if txnErr == txn.ErrAborted ||
-		errors.Cause(txnErr) == txn.ErrAborted {
-		return errors.Trace(err)
-	}
-	return errors.Trace(txnErr)
-}
-
 // AllMachines returns all machines in the environment
 // ordered by id.
 func (st *State) AllMachines() (machines []*Machine, err error) {
@@ -522,6 +608,14 @@ func machineIdLessThan(id1, id2 string) bool {
 
 // Machine returns the machine with the given id.
 func (st *State) Machine(id string) (*Machine, error) {
+	mdoc, err := st.getMachineDoc(id)
+	if err != nil {
+		return nil, err
+	}
+	return newMachine(st, mdoc), nil
+}
+
+func (st *State) getMachineDoc(id string) (*machineDoc, error) {
 	machinesCollection, closer := st.getRawCollection(machinesC)
 	defer closer()
 
@@ -542,7 +636,7 @@ func (st *State) Machine(id string) (*Machine, error) {
 		if mdoc.Id == "" {
 			mdoc.Id = mdoc.DocID
 		}
-		return newMachine(st, mdoc), nil
+		return mdoc, nil
 	case mgo.ErrNotFound:
 		return nil, errors.NotFoundf("machine %s", id)
 	default:
@@ -603,6 +697,10 @@ func (st *State) FindEntity(tag names.Tag) (Entity, error) {
 		} else {
 			return st.Charm(url)
 		}
+	case names.VolumeTag:
+		return st.Volume(tag)
+	case names.FilesystemTag:
+		return st.Filesystem(tag)
 	default:
 		return nil, errors.Errorf("unsupported tag %T", tag)
 	}
@@ -655,35 +753,28 @@ func (st *State) tagToCollectionAndId(tag names.Tag) (string, interface{}, error
 // AddCharm adds the ch charm with curl to the state.
 // On success the newly added charm state is returned.
 func (st *State) AddCharm(ch charm.Charm, curl *charm.URL, storagePath, bundleSha256 string) (stch *Charm, err error) {
-	// The charm may already exist in state as a placeholder, so we
-	// check for that situation and update the existing charm record
-	// if necessary, otherwise add a new record.
-	var existing charmDoc
 	charms, closer := st.getCollection(charmsC)
 	defer closer()
 
-	err = charms.Find(bson.D{{"_id", curl.String()}, {"placeholder", true}}).One(&existing)
-	if err == mgo.ErrNotFound {
-		cdoc := &charmDoc{
-			DocID:        st.docID(curl.String()),
-			URL:          curl,
-			EnvUUID:      st.EnvironTag().Id(),
-			Meta:         ch.Meta(),
-			Config:       ch.Config(),
-			Metrics:      ch.Metrics(),
-			Actions:      ch.Actions(),
-			BundleSha256: bundleSha256,
-			StoragePath:  storagePath,
+	query := charms.FindId(curl.String()).Select(bson.D{{"placeholder", 1}})
+
+	buildTxn := func(attempt int) ([]txn.Op, error) {
+		var placeholderDoc struct {
+			Placeholder bool `bson:"placeholder"`
 		}
-		err = charms.Insert(cdoc)
-		if err != nil {
-			return nil, errors.Annotatef(err, "cannot add charm %q", curl)
+		if err := query.One(&placeholderDoc); err == mgo.ErrNotFound {
+			return insertCharmOps(st, ch, curl, storagePath, bundleSha256)
+		} else if err != nil {
+			return nil, errors.Trace(err)
+		} else if placeholderDoc.Placeholder {
+			return updateCharmOps(st, ch, curl, storagePath, bundleSha256, stillPlaceholder)
 		}
-		return newCharm(st, cdoc), nil
-	} else if err != nil {
-		return nil, errors.Trace(err)
+		return nil, errors.AlreadyExistsf("charm %q", curl)
 	}
-	return st.updateCharmDoc(ch, curl, storagePath, bundleSha256, stillPlaceholder)
+	if err = st.run(buildTxn); err == nil {
+		return st.Charm(curl)
+	}
+	return nil, errors.Trace(err)
 }
 
 // AllCharms returns all charms in state.
@@ -794,20 +885,7 @@ func (st *State) PrepareLocalCharmUpload(curl *charm.URL) (chosenUrl *charm.URL,
 			chosenRevision = maxRevision + 1
 		}
 		chosenUrl = curl.WithRevision(chosenRevision)
-
-		uploadedCharm := &charmDoc{
-			DocID:         st.docID(chosenUrl.String()),
-			EnvUUID:       st.EnvironTag().Id(),
-			URL:           chosenUrl,
-			PendingUpload: true,
-		}
-		ops := []txn.Op{{
-			C:      charmsC,
-			Id:     uploadedCharm.DocID,
-			Assert: txn.DocMissing,
-			Insert: uploadedCharm,
-		}}
-		return ops, nil
+		return insertPendingCharmOps(st, chosenUrl)
 	}
 	if err = st.run(buildTxn); err == nil {
 		return chosenUrl, nil
@@ -818,7 +896,7 @@ func (st *State) PrepareLocalCharmUpload(curl *charm.URL) (chosenUrl *charm.URL,
 // PrepareStoreCharmUpload must be called before a charm store charm
 // is uploaded to the provider storage in order to create a charm
 // document in state. If a charm with the same URL is already in
-// state, it will be returned as a *state.Charm (is can be still
+// state, it will be returned as a *state.Charm (it can be still
 // pending or already uploaded). Otherwise, a new charm document is
 // added in state with just the given charm URL and
 // PendingUpload=true, which is then returned as a *state.Charm.
@@ -843,55 +921,28 @@ func (st *State) PrepareStoreCharmUpload(curl *charm.URL) (*Charm, error) {
 	buildTxn := func(attempt int) ([]txn.Op, error) {
 		// Find an uploaded or pending charm with the given exact curl.
 		err := charms.FindId(curl.String()).One(&uploadedCharm)
-		if err != nil && err != mgo.ErrNotFound {
-			return nil, errors.Trace(err)
-		} else if err == nil && !uploadedCharm.Placeholder {
-			// The charm exists and it's either uploaded or still
-			// pending, but it's not a placeholder. In any case,
-			// there's nothing to do.
-			return nil, jujutxn.ErrNoOperations
-		} else if err == mgo.ErrNotFound {
-			// Prepare the pending charm document for insertion.
+		switch {
+		case err == mgo.ErrNotFound:
 			uploadedCharm = charmDoc{
 				DocID:         st.docID(curl.String()),
 				EnvUUID:       st.EnvironTag().Id(),
 				URL:           curl,
 				PendingUpload: true,
-				Placeholder:   false,
 			}
-		}
-
-		var ops []txn.Op
-		if uploadedCharm.Placeholder {
-			// Convert the placeholder to a pending charm, while
-			// asserting the fields updated after an upload have not
-			// changed yet.
-			ops = []txn.Op{{
-				C:  charmsC,
-				Id: uploadedCharm.DocID,
-				Assert: bson.D{
-					{"bundlesha256", ""},
-					{"pendingupload", false},
-					{"placeholder", true},
-				},
-				Update: bson.D{{"$set", bson.D{
-					{"pendingupload", true},
-					{"placeholder", false},
-				}}},
-			}}
+			return insertAnyCharmOps(&uploadedCharm)
+		case err != nil:
+			return nil, errors.Trace(err)
+		case uploadedCharm.Placeholder:
 			// Update the fields of the document we're returning.
 			uploadedCharm.PendingUpload = true
 			uploadedCharm.Placeholder = false
-		} else {
-			// No charm document with this curl yet, insert it.
-			ops = []txn.Op{{
-				C:      charmsC,
-				Id:     uploadedCharm.DocID,
-				Assert: txn.DocMissing,
-				Insert: uploadedCharm,
-			}}
+			return convertPlaceholderCharmOps(uploadedCharm.DocID)
+		default:
+			// The charm exists and it's either uploaded or still
+			// pending, but it's not a placeholder. In any case,
+			// there's nothing to do.
+			return nil, jujutxn.ErrNoOperations
 		}
-		return ops, nil
 	}
 	if err = st.run(buildTxn); err == nil {
 		return newCharm(st, &uploadedCharm), nil
@@ -930,90 +981,20 @@ func (st *State) AddStoreCharmPlaceholder(curl *charm.URL) (err error) {
 		}
 
 		// Delete all previous placeholders so we don't fill up the database with unused data.
-		ops, err := st.deleteOldPlaceholderCharmsOps(curl)
+		deleteOps, err := deleteOldPlaceholderCharmsOps(st, charms, curl)
 		if err != nil {
 			return nil, errors.Trace(err)
 		}
-		// Add the new charm doc.
-		placeholderCharm := &charmDoc{
-			DocID:       st.docID(curl.String()),
-			EnvUUID:     st.EnvironTag().Id(),
-			URL:         curl,
-			Placeholder: true,
+		insertOps, err := insertPlaceholderCharmOps(st, curl)
+		if err != nil {
+			return nil, errors.Trace(err)
 		}
-		ops = append(ops, txn.Op{
-			C:      charmsC,
-			Id:     placeholderCharm.DocID,
-			Assert: txn.DocMissing,
-			Insert: placeholderCharm,
-		})
+		ops := append(deleteOps, insertOps...)
 		return ops, nil
 	}
 	return errors.Trace(st.run(buildTxn))
 }
 
-// deleteOldPlaceholderCharmsOps returns the txn ops required to delete all placeholder charm
-// records older than the specified charm URL.
-func (st *State) deleteOldPlaceholderCharmsOps(curl *charm.URL) ([]txn.Op, error) {
-	// Get a regex with the charm URL and no revision.
-	noRevURL := curl.WithRevision(-1)
-	curlRegex := "^" + regexp.QuoteMeta(st.docID(noRevURL.String()))
-	charms, closer := st.getCollection(charmsC)
-	defer closer()
-
-	var docs []charmDoc
-	query := bson.D{{"_id", bson.D{{"$regex", curlRegex}}}, {"placeholder", true}}
-	err := charms.Find(query).Select(bson.D{{"_id", 1}, {"url", 1}}).All(&docs)
-	if err != nil {
-		return nil, errors.Trace(err)
-	}
-	var ops []txn.Op
-	for _, doc := range docs {
-		if doc.URL.Revision >= curl.Revision {
-			continue
-		}
-		ops = append(ops, txn.Op{
-			C:      charmsC,
-			Id:     doc.DocID,
-			Assert: stillPlaceholder,
-			Remove: true,
-		})
-	}
-	return ops, nil
-}
-
-// ErrCharmAlreadyUploaded is returned by UpdateUploadedCharm() when
-// the given charm is already uploaded and marked as not pending in
-// state.
-type ErrCharmAlreadyUploaded struct {
-	curl *charm.URL
-}
-
-func (e *ErrCharmAlreadyUploaded) Error() string {
-	return fmt.Sprintf("charm %q already uploaded", e.curl)
-}
-
-// IsCharmAlreadyUploadedError returns if the given error is
-// ErrCharmAlreadyUploaded.
-func IsCharmAlreadyUploadedError(err interface{}) bool {
-	if err == nil {
-		return false
-	}
-	// In case of a wrapped error, check the cause first.
-	value := err
-	cause := errors.Cause(err.(error))
-	if cause != nil {
-		value = cause
-	}
-	_, ok := value.(*ErrCharmAlreadyUploaded)
-	return ok
-}
-
-// ErrCharmRevisionAlreadyModified is returned when a pending or
-// placeholder charm is no longer pending or a placeholder, signaling
-// the charm is available in state with its full information.
-var ErrCharmRevisionAlreadyModified = fmt.Errorf("charm revision already modified")
-
 // UpdateUploadedCharm marks the given charm URL as uploaded and
 // updates the rest of its data, returning it as *state.Charm.
 func (st *State) UpdateUploadedCharm(ch charm.Charm, curl *charm.URL, storagePath, bundleSha256 string) (*Charm, error) {
@@ -1032,40 +1013,10 @@ func (st *State) UpdateUploadedCharm(ch charm.Charm, curl *charm.URL, storagePat
 		return nil, errors.Trace(&ErrCharmAlreadyUploaded{curl})
 	}
 
-	return st.updateCharmDoc(ch, curl, storagePath, bundleSha256, stillPending)
-}
-
-// updateCharmDoc updates the charm with specified URL with the given
-// data, and resets the placeholder and pendingupdate flags.  If the
-// charm is no longer a placeholder or pending (depending on preReq),
-// it returns ErrCharmRevisionAlreadyModified.
-func (st *State) updateCharmDoc(
-	ch charm.Charm, curl *charm.URL, storagePath, bundleSha256 string, preReq interface{}) (*Charm, error) {
-
-	// Make sure we escape any "$" and "." in config option names
-	// first. See http://pad.lv/1308146.
-	cfg := ch.Config()
-	escapedConfig := charm.NewConfig()
-	for optionName, option := range cfg.Options {
-		escapedName := escapeReplacer.Replace(optionName)
-		escapedConfig.Options[escapedName] = option
+	ops, err := updateCharmOps(st, ch, curl, storagePath, bundleSha256, stillPending)
+	if err != nil {
+		return nil, errors.Trace(err)
 	}
-	updateFields := bson.D{{"$set", bson.D{
-		{"meta", ch.Meta()},
-		{"config", escapedConfig},
-		{"actions", ch.Actions()},
-		{"metrics", ch.Metrics()},
-		{"storagepath", storagePath},
-		{"bundlesha256", bundleSha256},
-		{"pendingupload", false},
-		{"placeholder", false},
-	}}}
-	ops := []txn.Op{{
-		C:      charmsC,
-		Id:     st.docID(curl.String()),
-		Assert: preReq,
-		Update: updateFields,
-	}}
 	if err := st.runTransaction(ops); err != nil {
 		return nil, onAbort(err, ErrCharmRevisionAlreadyModified)
 	}
@@ -1107,12 +1058,15 @@ func (st *State) addPeerRelationsOps(serviceName string, peers map[string]charm.
 // AddService creates a new service, running the supplied charm, with the
 // supplied name (which must be unique). If the charm defines peer relations,
 // they will be created automatically.
-func (st *State) AddService(name, owner string, ch *Charm, networks []string) (service *Service, err error) {
+func (st *State) AddService(
+	name, owner string, ch *Charm, networks []string, storage map[string]StorageConstraints, settingValues charm.Settings,
+) (service *Service, err error) {
 	defer errors.DeferredAnnotatef(&err, "cannot add service %q", name)
 	ownerTag, err := names.ParseUserTag(owner)
 	if err != nil {
 		return nil, errors.Annotatef(err, "Invalid ownertag %s", owner)
 	}
+
 	// Sanity checks.
 	if !names.IsValidService(name) {
 		return nil, errors.Errorf("invalid name")
@@ -1134,6 +1088,20 @@ func (st *State) AddService(name, owner string, ch *Charm, networks []string) (s
 	if _, err := st.EnvironmentUser(ownerTag); err != nil {
 		return nil, errors.Trace(err)
 	}
+
+	if storage == nil {
+		storage = make(map[string]StorageConstraints)
+	}
+	if err := addDefaultStorageConstraints(st, storage, ch.Meta()); err != nil {
+		return nil, errors.Trace(err)
+	}
+	if err := validateStorageConstraints(st, storage, ch.Meta()); err != nil {
+		return nil, errors.Trace(err)
+	}
+
+	settings := newSettings(nil, "")
+	settings.apply(settingValues)
+
 	serviceID := st.docID(name)
 	// Create the service addition operations.
 	peers := ch.Meta().Peers
@@ -1149,6 +1117,21 @@ func (st *State) AddService(name, owner string, ch *Charm, networks []string) (s
 		OwnerTag:      owner,
 	}
 	svc := newService(st, svcDoc)
+
+	statusDoc := statusDoc{
+		EnvUUID: st.EnvironUUID(),
+		// TODO(fwereade): this violates the spec. Should be "waiting".
+		// Implemented like this to be consistent with incorrect add-unit
+		// behaviour.
+		Status:     StatusUnknown,
+		StatusInfo: MessageWaitForAgentInit,
+		Updated:    time.Now().UnixNano(),
+		// This exists to preserve questionable unit-aggregation behaviour
+		// while we work out how to switch to an implementation that makes
+		// sense. It is also set in AddMissingServiceStatuses.
+		NeverSet: true,
+	}
+
 	ops := []txn.Op{
 		env.assertAliveOp(),
 		createConstraintsOp(st, svc.globalKey(), constraints.Value{}),
@@ -1157,7 +1140,10 @@ func (st *State) AddService(name, owner string, ch *Charm, networks []string) (s
 		// provisioning, we should check the given networks are valid
 		// and known before setting them.
 		createRequestedNetworksOp(st, svc.globalKey(), networks),
-		createSettingsOp(st, svc.settingsKey(), nil),
+		createStorageConstraintsOp(svc.globalKey(), storage),
+		createSettingsOp(st, svc.settingsKey(), settings.Map()),
+		addLeadershipSettingsOp(svc.Tag().Id()),
+		createStatusOp(st, svc.globalKey(), statusDoc),
 		{
 			C:      settingsrefsC,
 			Id:     st.docID(svc.settingsKey()),
@@ -1165,13 +1151,14 @@ func (st *State) AddService(name, owner string, ch *Charm, networks []string) (s
 			Insert: settingsRefsDoc{
 				RefCount: 1,
 				EnvUUID:  st.EnvironUUID()},
-		},
-		{
+		}, {
 			C:      servicesC,
 			Id:     serviceID,
 			Assert: txn.DocMissing,
 			Insert: svcDoc,
-		}}
+		},
+	}
+
 	// Collect peer relation addition operations.
 	peerOps, err := st.addPeerRelationsOps(name, peers)
 	if err != nil {
@@ -1179,11 +1166,11 @@ func (st *State) AddService(name, owner string, ch *Charm, networks []string) (s
 	}
 	ops = append(ops, peerOps...)
 
+	// At the last moment before inserting the service, prime status history.
+	probablyUpdateStatusHistory(st, svc.globalKey(), statusDoc)
+
 	if err := st.runTransaction(ops); err == txn.ErrAborted {
-		err := env.Refresh()
-		if (err == nil && env.Life() != Alive) || errors.IsNotFound(err) {
-			return nil, errors.Errorf("environment is no longer alive")
-		} else if err != nil {
+		if err := checkEnvLife(st); err != nil {
 			return nil, errors.Trace(err)
 		}
 		return nil, errors.Errorf("service already exists")
@@ -1197,43 +1184,50 @@ func (st *State) AddService(name, owner string, ch *Charm, networks []string) (s
 	return svc, nil
 }
 
-// AddIPAddress creates and returns a new IP address
+// AddIPAddress creates and returns a new IP address. It can return an
+// error satisfying IsNotValid() or IsAlreadyExists() when the addr
+// does not contain a valid IP, or when addr is already added.
 func (st *State) AddIPAddress(addr network.Address, subnetid string) (ipaddress *IPAddress, err error) {
-	defer errors.DeferredAnnotatef(&err, "cannot add IP address %v", addr.Value)
+	defer errors.DeferredAnnotatef(&err, "cannot add IP address %q", addr)
 
 	// This checks for a missing value as well as invalid values
 	ip := net.ParseIP(addr.Value)
 	if ip == nil {
-		return nil, errors.Errorf("invalid IP address %q", addr.Value)
+		return nil, errors.NotValidf("address")
 	}
 
 	addressID := st.docID(addr.Value)
 	ipDoc := ipaddressDoc{
 		DocID:    addressID,
 		EnvUUID:  st.EnvironUUID(),
+		Life:     Alive,
 		State:    AddressStateUnknown,
 		SubnetId: subnetid,
 		Value:    addr.Value,
-		Type:     addr.Type,
-		Scope:    addr.Scope,
+		Type:     string(addr.Type),
+		Scope:    string(addr.Scope),
 	}
 
 	ipaddress = &IPAddress{doc: ipDoc, st: st}
-	ops := []txn.Op{{
-		C:      ipaddressesC,
-		Id:     addressID,
-		Assert: txn.DocMissing,
-		Insert: ipDoc,
-	}}
+	ops := []txn.Op{
+		assertEnvAliveOp(st.EnvironUUID()),
+		{
+			C:      ipaddressesC,
+			Id:     addressID,
+			Assert: txn.DocMissing,
+			Insert: ipDoc,
+		},
+	}
 
 	err = st.runTransaction(ops)
 	switch err {
 	case txn.ErrAborted:
-		if _, err = st.IPAddress(addr.Value); err == nil {
-			return nil, errors.AlreadyExistsf("IP address %q", addr.Value)
-		} else if err != nil {
+		if err := checkEnvLife(st); err != nil {
 			return nil, errors.Trace(err)
 		}
+		if _, err = st.IPAddress(addr.Value); err == nil {
+			return nil, errors.AlreadyExistsf("address")
+		}
 	case nil:
 		return ipaddress, nil
 	}
@@ -1256,9 +1250,43 @@ func (st *State) IPAddress(value string) (*IPAddress, error) {
 	return &IPAddress{st, *doc}, nil
 }
 
+// AllocatedIPAddresses returns all the allocated addresses for a machine
+func (st *State) AllocatedIPAddresses(machineId string) ([]*IPAddress, error) {
+	return st.fetchIPAddresses(bson.D{{"machineid", machineId}})
+}
+
+// DeadIPAddresses returns all IP addresses with a Life of Dead
+func (st *State) DeadIPAddresses() ([]*IPAddress, error) {
+	return st.fetchIPAddresses(bson.D{{"life", Dead}})
+}
+
+// fetchIPAddresses is a helper function for finding IP addresses
+func (st *State) fetchIPAddresses(query bson.D) ([]*IPAddress, error) {
+	addresses, closer := st.getCollection(ipaddressesC)
+	result := []*IPAddress{}
+	defer closer()
+	var doc struct {
+		Value string
+	}
+	iter := addresses.Find(query).Iter()
+	for iter.Next(&doc) {
+		addr, err := st.IPAddress(doc.Value)
+		if err != nil {
+			// shouldn't happen as we're only fetching
+			// addresses we know exist.
+			continue
+		}
+		result = append(result, addr)
+	}
+	if err := iter.Close(); err != nil {
+		return result, err
+	}
+	return result, nil
+}
+
 // AddSubnet creates and returns a new subnet
 func (st *State) AddSubnet(args SubnetInfo) (subnet *Subnet, err error) {
-	defer errors.DeferredAnnotatef(&err, "cannot add subnet %v", args.CIDR)
+	defer errors.DeferredAnnotatef(&err, "cannot add subnet %q", args.CIDR)
 
 	subnetID := st.docID(args.CIDR)
 	subDoc := subnetDoc{
@@ -1277,16 +1305,22 @@ func (st *State) AddSubnet(args SubnetInfo) (subnet *Subnet, err error) {
 	if err != nil {
 		return nil, err
 	}
-	ops := []txn.Op{{
-		C:      subnetsC,
-		Id:     subnetID,
-		Assert: txn.DocMissing,
-		Insert: subDoc,
-	}}
+	ops := []txn.Op{
+		assertEnvAliveOp(st.EnvironUUID()),
+		{
+			C:      subnetsC,
+			Id:     subnetID,
+			Assert: txn.DocMissing,
+			Insert: subDoc,
+		},
+	}
 
 	err = st.runTransaction(ops)
 	switch err {
 	case txn.ErrAborted:
+		if err := checkEnvLife(st); err != nil {
+			return nil, errors.Trace(err)
+		}
 		if _, err = st.Subnet(args.CIDR); err == nil {
 			return nil, errors.AlreadyExistsf("subnet %q", args.CIDR)
 		} else if err != nil {
@@ -1299,7 +1333,7 @@ func (st *State) AddSubnet(args SubnetInfo) (subnet *Subnet, err error) {
 		if err == nil {
 			return subnet, nil
 		}
-		return nil, errors.Annotatef(err, "ProviderId not unique %q", args.ProviderId)
+		return nil, errors.Errorf("ProviderId %q not unique", args.ProviderId)
 	}
 	return nil, errors.Trace(err)
 }
@@ -1343,15 +1377,21 @@ func (st *State) AddNetwork(args NetworkInfo) (n *Network, err error) {
 		return nil, errors.Errorf("invalid VLAN tag %d: must be between 0 and 4094", args.VLANTag)
 	}
 	doc := st.newNetworkDoc(args)
-	ops := []txn.Op{{
-		C:      networksC,
-		Id:     doc.DocID,
-		Assert: txn.DocMissing,
-		Insert: doc,
-	}}
+	ops := []txn.Op{
+		assertEnvAliveOp(st.EnvironUUID()),
+		{
+			C:      networksC,
+			Id:     doc.DocID,
+			Assert: txn.DocMissing,
+			Insert: doc,
+		},
+	}
 	err = st.runTransaction(ops)
 	switch err {
 	case txn.ErrAborted:
+		if err := checkEnvLife(st); err != nil {
+			return nil, errors.Trace(err)
+		}
 		if _, err = st.Network(args.Name); err == nil {
 			return nil, errors.AlreadyExistsf("network %q", args.Name)
 		} else if err != nil {
@@ -1443,7 +1483,11 @@ func (st *State) AllServices() (services []*Service, err error) {
 // where the environment uuid is prefixed to the
 // localID.
 func (st *State) docID(localID string) string {
-	return st.EnvironUUID() + ":" + localID
+	prefix := st.EnvironUUID() + ":"
+	if strings.HasPrefix(localID, prefix) {
+		return localID
+	}
+	return prefix + localID
 }
 
 // localID returns the local id value by stripping
@@ -1844,7 +1888,7 @@ func (st *State) StartSync() {
 // all subsequent attempts to access the state must
 // be authorized; otherwise no authorization is required.
 func (st *State) SetAdminMongoPassword(password string) error {
-	err := mongo.SetAdminMongoPassword(st.db.Session, mongo.AdminUser, password)
+	err := mongo.SetAdminMongoPassword(st.session, mongo.AdminUser, password)
 	return errors.Trace(err)
 }
 
@@ -1877,8 +1921,17 @@ type StateServerInfo struct {
 // StateServerInfo returns information about
 // the currently configured state server machines.
 func (st *State) StateServerInfo() (*StateServerInfo, error) {
-	stateServers, closer := st.getCollection(stateServersC)
-	defer closer()
+	session := st.session.Copy()
+	defer session.Close()
+	return readRawStateServerInfo(st.session)
+}
+
+// readRawStateServerInfo reads StateServerInfo direct from the supplied session,
+// falling back to the bootstrap environment document to extract the UUID when
+// required.
+func readRawStateServerInfo(session *mgo.Session) (*StateServerInfo, error) {
+	db := session.DB(jujuDB)
+	stateServers := db.C(stateServersC)
 
 	var doc stateServersDoc
 	err := stateServers.Find(bson.D{{"_id", environGlobalKey}}).One(&doc)
@@ -1893,8 +1946,7 @@ func (st *State) StateServerInfo() (*StateServerInfo, error) {
 		// upgrade steps have been run. Without this hack environTag
 		// on State ends up empty, breaking basic functionality needed
 		// to run upgrade steps (a chicken-and-egg scenario).
-		environments, closer := st.getCollection(environmentsC)
-		defer closer()
+		environments := db.C(environmentsC)
 
 		var envDoc environmentDoc
 		query := environments.Find(nil)
