@@ -13,7 +13,8 @@ import (
 	"github.com/juju/names"
 	jujutxn "github.com/juju/txn"
 	"github.com/juju/utils"
-	"gopkg.in/juju/charm.v4"
+	"github.com/juju/utils/set"
+	"gopkg.in/juju/charm.v5"
 	"gopkg.in/mgo.v2"
 	"gopkg.in/mgo.v2/bson"
 	"gopkg.in/mgo.v2/txn"
@@ -62,35 +63,54 @@ const (
 	ResolvedNoHooks    ResolvedMode = "no-hooks"
 )
 
+// port identifies a network port number for a particular protocol.
+// TODO(mue) Not really used anymore, se bellow. Can be removed when
+// cleaning unitDoc.
+type port struct {
+	Protocol string `bson:"protocol"`
+	Number   int    `bson:"number"`
+}
+
+// networkPorts is a convenience helper to return the state type
+// as network type, here for a slice of Port.
+func networkPorts(ports []port) []network.Port {
+	netPorts := make([]network.Port, len(ports))
+	for i, port := range ports {
+		netPorts[i] = network.Port{port.Protocol, port.Number}
+	}
+	return netPorts
+}
+
 // unitDoc represents the internal state of a unit in MongoDB.
 // Note the correspondence with UnitInfo in apiserver/params.
 type unitDoc struct {
-	DocID        string `bson:"_id"`
-	Name         string `bson:"name"`
-	EnvUUID      string `bson:"env-uuid"`
-	Service      string
-	Series       string
-	CharmURL     *charm.URL
-	Principal    string
-	Subordinates []string
-	MachineId    string
-	Resolved     ResolvedMode
-	Tools        *tools.Tools `bson:",omitempty"`
-	Life         Life
-	TxnRevno     int64 `bson:"txn-revno"`
-	PasswordHash string
+	DocID                  string `bson:"_id"`
+	Name                   string `bson:"name"`
+	EnvUUID                string `bson:"env-uuid"`
+	Service                string
+	Series                 string
+	CharmURL               *charm.URL
+	Principal              string
+	Subordinates           []string
+	StorageAttachmentCount int `bson:"storageattachmentcount"`
+	MachineId              string
+	Resolved               ResolvedMode
+	Tools                  *tools.Tools `bson:",omitempty"`
+	Life                   Life
+	TxnRevno               int64 `bson:"txn-revno"`
+	PasswordHash           string
 
-	// No longer used - to be removed.
-	Ports          []network.Port
-	PublicAddress  string
-	PrivateAddress string
+	// TODO(mue) No longer actively used, only in upgrades.go.
+	// To be removed later.
+	Ports          []port `bson:"ports"`
+	PublicAddress  string `bson:"publicaddress"`
+	PrivateAddress string `bson:"privateaddress"`
 }
 
 // Unit represents the state of a service unit.
 type Unit struct {
 	st  *State
 	doc unitDoc
-	annotator
 	presence.Presencer
 }
 
@@ -99,11 +119,6 @@ func newUnit(st *State, udoc *unitDoc) *Unit {
 		st:  st,
 		doc: *udoc,
 	}
-	unit.annotator = annotator{
-		globalKey: unit.globalKey(),
-		tag:       unit.Tag(),
-		st:        st,
-	}
 	return unit
 }
 
@@ -157,7 +172,17 @@ func (u *Unit) Name() string {
 
 // unitGlobalKey returns the global database key for the named unit.
 func unitGlobalKey(name string) string {
-	return "u#" + name
+	return "u#" + name + "#charm"
+}
+
+// globalAgentKey returns the global database key for the unit.
+func (u *Unit) globalAgentKey() string {
+	return unitAgentGlobalKey(u.doc.Name)
+}
+
+// globalMeterStatusKey returns the global database key for the meter status of the unit.
+func (u *Unit) globalMeterStatusKey() string {
+	return unitAgentGlobalKey(u.doc.Name)
 }
 
 // globalKey returns the global database key for the unit.
@@ -289,6 +314,9 @@ func (u *Unit) Destroy() (err error) {
 		return nil, jujutxn.ErrNoOperations
 	}
 	if err = unit.st.run(buildTxn); err == nil {
+		if historyErr := unit.eraseHistory(); historyErr != nil {
+			logger.Errorf("cannot delete history for unit %q: %v", unit.globalKey(), err)
+		}
 		if err = unit.Refresh(); errors.IsNotFound(err) {
 			return nil
 		}
@@ -296,12 +324,22 @@ func (u *Unit) Destroy() (err error) {
 	return err
 }
 
-var unitNotInstalled = bson.D{
-	{"$or", []bson.D{
-		{{"status", StatusPending}},
-		{{"status", StatusAllocating}},
-		{{"status", StatusInstalling}},
-	}}}
+func (u *Unit) eraseHistory() error {
+	history, closer := u.st.getCollection(statusesHistoryC)
+	defer closer()
+	// XXX(fwereade): 2015-06-19 this is anything but safe: we must not mix
+	// txn and non-txn operations in the same collection without clear and
+	// detailed reasoning for so doing.
+	historyW := history.Writeable()
+
+	if _, err := historyW.RemoveAll(bson.D{{"statusid", u.globalKey()}}); err != nil {
+		return err
+	}
+	if _, err := historyW.RemoveAll(bson.D{{"statusid", u.globalAgentKey()}}); err != nil {
+		return err
+	}
+	return nil
+}
 
 // destroyOps returns the operations required to destroy the unit. If it
 // returns errRefresh, the unit should be refreshed and the destruction
@@ -342,26 +380,34 @@ func (u *Unit) destroyOps() ([]txn.Op, error) {
 	}, cleanupOp, minUnitsOp}
 	if u.doc.Principal != "" {
 		return setDyingOps, nil
-	} else if len(u.doc.Subordinates) != 0 {
+	} else if len(u.doc.Subordinates)+u.doc.StorageAttachmentCount != 0 {
 		return setDyingOps, nil
 	}
 
-	sdocId := u.globalKey()
-	sdoc, err := getStatus(u.st, sdocId)
-	if errors.IsNotFound(err) {
+	// See if the unit agent has started running.
+	// If so then we can't set directly to dead.
+	agentStatusDocId := u.globalAgentKey()
+	agentStatusInfo, agentErr := getStatus(u.st, agentStatusDocId, "agent")
+	if errors.IsNotFound(agentErr) {
 		return nil, errAlreadyDying
-	} else if err != nil {
-		return nil, err
+	} else if agentErr != nil {
+		return nil, errors.Trace(agentErr)
 	}
-	if sdoc.Status != StatusPending && sdoc.Status != StatusAllocating && sdoc.Status != StatusInstalling {
+	if agentStatusInfo.Status != StatusAllocating {
 		return setDyingOps, nil
 	}
+
 	ops := []txn.Op{{
 		C:      statusesC,
-		Id:     u.st.docID(sdocId),
-		Assert: unitNotInstalled,
+		Id:     u.st.docID(agentStatusDocId),
+		Assert: bson.D{{"status", StatusAllocating}},
 	}, minUnitsOp}
-	removeAsserts := append(isAliveDoc, unitHasNoSubordinates...)
+	removeAsserts := append(isAliveDoc, bson.DocElem{
+		"$and", []bson.D{
+			unitHasNoSubordinates,
+			unitHasNoStorageAttachments,
+		},
+	})
 	removeOps, err := u.removeOps(removeAsserts)
 	if err == errAlreadyRemoved {
 		return nil, errAlreadyDying
@@ -435,15 +481,15 @@ func (u *Unit) destroyHostOps(s *Service) (ops []txn.Op, err error) {
 	var machineAssert bson.D
 	if machineCheck {
 		machineAssert = bson.D{{"$and", []bson.D{
-			bson.D{{"principals", []string{u.doc.Name}}},
-			bson.D{{"jobs", bson.D{{"$nin", []MachineJob{JobManageEnviron}}}}},
-			bson.D{{"hasvote", bson.D{{"$ne", true}}}},
+			{{"principals", []string{u.doc.Name}}},
+			{{"jobs", bson.D{{"$nin", []MachineJob{JobManageEnviron}}}}},
+			{{"hasvote", bson.D{{"$ne", true}}}},
 		}}}
 	} else {
 		machineAssert = bson.D{{"$or", []bson.D{
-			bson.D{{"principals", bson.D{{"$ne", []string{u.doc.Name}}}}},
-			bson.D{{"jobs", bson.D{{"$in", []MachineJob{JobManageEnviron}}}}},
-			bson.D{{"hasvote", true}},
+			{{"principals", bson.D{{"$ne", []string{u.doc.Name}}}}},
+			{{"jobs", bson.D{{"$in", []MachineJob{JobManageEnviron}}}}},
+			{{"hasvote", true}},
 		}}}
 	}
 
@@ -477,8 +523,8 @@ func (u *Unit) removeOps(asserts bson.D) ([]txn.Op, error) {
 	return svc.removeUnitOps(u, asserts)
 }
 
-// ErrUnitHasSubordinates is a standard error to indicate that an a Unit
-// cannot complete an operation to end it's life because it still has
+// ErrUnitHasSubordinates is a standard error to indicate that a Unit
+// cannot complete an operation to end its life because it still has
 // subordinate services
 var ErrUnitHasSubordinates = stderrors.New("unit has subordinates")
 
@@ -489,9 +535,22 @@ var unitHasNoSubordinates = bson.D{{
 	},
 }}
 
+// ErrUnitHasStorageAttachments is a standard error to indicate that
+// a Unit cannot complete an operation to end its life because it still
+// has storage attachments.
+var ErrUnitHasStorageAttachments = stderrors.New("unit has storage attachments")
+
+var unitHasNoStorageAttachments = bson.D{{
+	"$or", []bson.D{
+		{{"storageattachmentcount", 0}},
+		{{"storageattachmentcount", bson.D{{"$exists", false}}}},
+	},
+}}
+
 // EnsureDead sets the unit lifecycle to Dead if it is Alive or Dying.
 // It does nothing otherwise. If the unit has subordinates, it will
-// return ErrUnitHasSubordinates.
+// return ErrUnitHasSubordinates; otherwise, if it has storage instances,
+// it will return ErrUnitHasStorageInstances.
 func (u *Unit) EnsureDead() (err error) {
 	if u.doc.Life == Dead {
 		return nil
@@ -501,10 +560,16 @@ func (u *Unit) EnsureDead() (err error) {
 			u.doc.Life = Dead
 		}
 	}()
+	assert := append(notDeadDoc, bson.DocElem{
+		"$and", []bson.D{
+			unitHasNoSubordinates,
+			unitHasNoStorageAttachments,
+		},
+	})
 	ops := []txn.Op{{
 		C:      unitsC,
 		Id:     u.doc.DocID,
-		Assert: append(notDeadDoc, unitHasNoSubordinates...),
+		Assert: assert,
 		Update: bson.D{{"$set", bson.D{{"life", Dead}}}},
 	}}
 	if err := u.st.runTransaction(ops); err != txn.ErrAborted {
@@ -515,7 +580,15 @@ func (u *Unit) EnsureDead() (err error) {
 	} else if !notDead {
 		return nil
 	}
-	return ErrUnitHasSubordinates
+	if err := u.Refresh(); errors.IsNotFound(err) {
+		return nil
+	} else if err != nil {
+		return err
+	}
+	if len(u.doc.Subordinates) > 0 {
+		return ErrUnitHasSubordinates
+	}
+	return ErrUnitHasStorageAttachments
 }
 
 // Remove removes the unit from state, and may remove its service as well, if
@@ -715,37 +788,72 @@ func (u *Unit) Refresh() error {
 	return nil
 }
 
+// Agent Returns an agent by its unit's name.
+func (u *Unit) Agent() *UnitAgent {
+	return newUnitAgent(u.st, u.Tag(), u.Name())
+}
+
+// SetAgentStatus calls SetStatus for this unit's agent, this call
+// is equivalent to the former call to SetStatus when Agent and Unit
+// were not separate entities.
+func (u *Unit) SetAgentStatus(status Status, info string, data map[string]interface{}) error {
+	agent := newUnitAgent(u.st, u.Tag(), u.Name())
+	return agent.SetStatus(status, info, data)
+}
+
+// AgentStatus calls Status for this unit's agent, this call
+// is equivalent to the former call to Status when Agent and Unit
+// were not separate entities.
+func (u *Unit) AgentStatus() (StatusInfo, error) {
+	agent := newUnitAgent(u.st, u.Tag(), u.Name())
+	return agent.Status()
+}
+
+// StatusHistory returns a slice of at most <size> StatusInfo items
+// representing past statuses for this unit.
+func (u *Unit) StatusHistory(size int) ([]StatusInfo, error) {
+	return statusHistory(u.st, u.globalKey(), size)
+}
+
 // Status returns the status of the unit.
-func (u *Unit) Status() (status Status, info string, data map[string]interface{}, err error) {
-	doc, err := getStatus(u.st, u.globalKey())
+// This method relies on globalKey instead of globalAgentKey since it is part of
+// the effort to separate Unit from UnitAgent. Now the Status for UnitAgent is in
+// the UnitAgent struct.
+func (u *Unit) Status() (StatusInfo, error) {
+	// The current health spec says when a hook error occurs, the workload should
+	// be in error state, but the state model more correctly records the agent
+	// itself as being in error. So we'll do that model translation here.
+	// TODO(fwereade) as on unitagent, this transformation does not belong here.
+	// For now, pretend we're always reading the unit status.
+	info, err := getStatus(u.st, u.globalAgentKey(), "unit")
 	if err != nil {
-		return "", "", nil, err
+		return StatusInfo{}, err
 	}
-	status = doc.Status
-	info = doc.StatusInfo
-	data = doc.StatusData
-	return
+	if info.Status != StatusError {
+		info, err = getStatus(u.st, u.globalKey(), "unit")
+		if err != nil {
+			return StatusInfo{}, err
+		}
+	}
+	return info, nil
 }
 
 // SetStatus sets the status of the unit agent. The optional values
 // allow to pass additional helpful status data.
+// This method relies on globalKey instead of globalAgentKey since it is part of
+// the effort to separate Unit from UnitAgent. Now the SetStatus for UnitAgent is in
+// the UnitAgent struct.
 func (u *Unit) SetStatus(status Status, info string, data map[string]interface{}) error {
-	doc, err := newUnitAgentStatusDoc(status, info, data)
-	if err != nil {
-		return err
+	if !ValidWorkloadStatus(status) {
+		return errors.Errorf("cannot set invalid status %q", status)
 	}
-	ops := []txn.Op{{
-		C:      unitsC,
-		Id:     u.doc.DocID,
-		Assert: notDeadDoc,
-	},
-		updateStatusOp(u.st, u.globalKey(), doc.statusDoc),
-	}
-	err = u.st.runTransaction(ops)
-	if err != nil {
-		return fmt.Errorf("cannot set status of unit %q: %v", u, onAbort(err, ErrDead))
-	}
-	return nil
+	return setStatus(u.st, setStatusParams{
+		badge:     "unit",
+		globalKey: u.globalKey(),
+		status:    status,
+		message:   info,
+		rawData:   data,
+	})
 }
 
 // OpenPorts opens the given port range and protocol for the unit, if
@@ -853,9 +961,10 @@ func (u *Unit) SetCharmURL(curl *charm.URL) error {
 
 	db, closer := u.st.newDB()
 	defer closer()
-	envUUID := u.st.EnvironUUID()
-	units := getCollectionFromDB(db, unitsC, envUUID)
-	charms := getCollectionFromDB(db, charmsC, envUUID)
+	units, closer := db.GetCollection(unitsC)
+	defer closer()
+	charms, closer := db.GetCollection(charmsC)
+	defer closer()
 
 	buildTxn := func(attempt int) ([]txn.Op, error) {
 		if attempt > 0 {
@@ -918,7 +1027,7 @@ func (u *Unit) SetCharmURL(curl *charm.URL) error {
 
 // AgentPresence returns whether the respective remote agent is alive.
 func (u *Unit) AgentPresence() (bool, error) {
-	return u.st.pwatcher.Alive(u.globalKey())
+	return u.st.pwatcher.Alive(u.globalAgentKey())
 }
 
 // Tag returns a name identifying the unit.
@@ -938,8 +1047,8 @@ func (u *Unit) UnitTag() names.UnitTag {
 func (u *Unit) WaitAgentPresence(timeout time.Duration) (err error) {
 	defer errors.DeferredAnnotatef(&err, "waiting for agent of unit %q", u)
 	ch := make(chan presence.Change)
-	u.st.pwatcher.Watch(u.globalKey(), ch)
-	defer u.st.pwatcher.Unwatch(u.globalKey(), ch)
+	u.st.pwatcher.Watch(u.globalAgentKey(), ch)
+	defer u.st.pwatcher.Unwatch(u.globalAgentKey(), ch)
 	for i := 0; i < 2; i++ {
 		select {
 		case change := <-ch:
@@ -959,7 +1068,7 @@ func (u *Unit) WaitAgentPresence(timeout time.Duration) (err error) {
 // It returns the started pinger.
 func (u *Unit) SetAgentPresence() (*presence.Pinger, error) {
 	presenceCollection := u.st.getPresence()
-	p := presence.NewPinger(presenceCollection, u.globalKey())
+	p := presence.NewPinger(presenceCollection, u.st.EnvironTag(), u.globalAgentKey())
 	err := p.Start()
 	if err != nil {
 		return nil, err
@@ -967,25 +1076,16 @@ func (u *Unit) SetAgentPresence() (*presence.Pinger, error) {
 	return p, nil
 }
 
-// NotAssignedError indicates that a unit is not assigned to a machine (and, in
-// the case of subordinate units, that the unit's principal is not assigned).
-type NotAssignedError struct{ Unit *Unit }
-
-func (e *NotAssignedError) Error() string {
-	return fmt.Sprintf("unit %q is not assigned to a machine", e.Unit)
-}
-
-// IsNotAssigned verifies that err is an instance of NotAssignedError
-func IsNotAssigned(err error) bool {
-	_, ok := errors.Cause(err).(*NotAssignedError)
-	return ok
+func unitNotAssignedError(u *Unit) error {
+	msg := fmt.Sprintf("unit %q is not assigned to a machine", u)
+	return errors.NewNotAssigned(nil, msg)
 }
 
 // AssignedMachineId returns the id of the assigned machine.
 func (u *Unit) AssignedMachineId() (id string, err error) {
 	if u.IsPrincipal() {
 		if u.doc.MachineId == "" {
-			return "", &NotAssignedError{u}
+			return "", unitNotAssignedError(u)
 		}
 		return u.doc.MachineId, nil
 	}
@@ -1001,7 +1101,7 @@ func (u *Unit) AssignedMachineId() (id string, err error) {
 		return "", err
 	}
 	if pudoc.MachineId == "" {
-		return "", &NotAssignedError{u}
+		return "", unitNotAssignedError(u)
 	}
 	return pudoc.MachineId, nil
 }
@@ -1049,6 +1149,18 @@ func (u *Unit) assignToMachine(m *Machine, unused bool) (err error) {
 	if err := u.st.supportsUnitPlacement(); err != nil {
 		return err
 	}
+	storageParams, err := u.machineStorageParams()
+	if err != nil {
+		return errors.Trace(err)
+	}
+	if err := validateDynamicMachineStorageParams(m, storageParams); err != nil {
+		return errors.Trace(err)
+	}
+	storageOps, err := u.st.machineStorageOps(&m.doc, storageParams)
+	if err != nil {
+		return errors.Trace(err)
+	}
+
 	assert := append(isAliveDoc, bson.D{
 		{"$or", []bson.D{
 			{{"machineid", ""}},
@@ -1070,6 +1182,7 @@ func (u *Unit) assignToMachine(m *Machine, unused bool) (err error) {
 		Assert: massert,
 		Update: bson.D{{"$addToSet", bson.D{{"principals", u.doc.Name}}}, {"$set", bson.D{{"clean", false}}}},
 	}}
+	ops = append(ops, storageOps...)
 	err = u.st.runTransaction(ops)
 	if err == nil {
 		u.doc.MachineId = m.doc.Id
@@ -1098,6 +1211,83 @@ func (u *Unit) assignToMachine(m *Machine, unused bool) (err error) {
 	return inUseErr
 }
 
+// validateDynamicMachineStorageParams validates that the provided machine
+// storage parameters are compatible with the specified machine.
+func validateDynamicMachineStorageParams(m *Machine, params *machineStorageParams) error {
+	if len(params.volumes)+len(params.filesystems)+len(params.volumeAttachments)+len(params.filesystemAttachments) == 0 {
+		return nil
+	}
+	if m.ContainerType() != "" {
+		// TODO(axw) consult storage providers to check if they
+		// support adding storage to containers. Loop is fine,
+		// for example.
+		//
+		// TODO(axw) later we might allow *any* storage, and
+		// passthrough/bindmount storage. That would imply either
+		// container creation time only, or requiring containers
+		// to be restarted to pick up new configuration.
+		return errors.NotSupportedf("adding storage to %s container", m.ContainerType())
+	}
+	pools := make(set.Strings)
+	for _, v := range params.volumes {
+		v, err := m.st.volumeParamsWithDefaults(v.Volume)
+		if err != nil {
+			return errors.Trace(err)
+		}
+		pools.Add(v.Pool)
+	}
+	for _, f := range params.filesystems {
+		f, err := m.st.filesystemParamsWithDefaults(f.Filesystem)
+		if err != nil {
+			return errors.Trace(err)
+		}
+		pools.Add(f.Pool)
+	}
+	for volumeTag := range params.volumeAttachments {
+		volume, err := m.st.Volume(volumeTag)
+		if err != nil {
+			return errors.Trace(err)
+		}
+		if params, ok := volume.Params(); ok {
+			pools.Add(params.Pool)
+		} else {
+			info, err := volume.Info()
+			if err != nil {
+				return errors.Trace(err)
+			}
+			pools.Add(info.Pool)
+		}
+	}
+	for filesystemTag := range params.filesystemAttachments {
+		filesystem, err := m.st.Filesystem(filesystemTag)
+		if err != nil {
+			return errors.Trace(err)
+		}
+		if params, ok := filesystem.Params(); ok {
+			pools.Add(params.Pool)
+		} else {
+			info, err := filesystem.Info()
+			if err != nil {
+				return errors.Trace(err)
+			}
+			pools.Add(info.Pool)
+		}
+	}
+	for pool := range pools {
+		providerType, provider, err := poolStorageProvider(m.st, pool)
+		if err != nil {
+			return errors.Trace(err)
+		}
+		if !provider.Dynamic() {
+			return errors.Errorf(
+				"%s storage provider does not support dynamic storage",
+				providerType,
+			)
+		}
+	}
+	return nil
+}
+
 func assignContextf(err *error, unit *Unit, target string) {
 	if *err != nil {
 		*err = fmt.Errorf("cannot assign unit %q to %s: %v", unit, target, *err)
@@ -1210,7 +1400,7 @@ func (u *Unit) assignToNewMachine(template MachineTemplate, parentId string, con
 
 // Constraints returns the unit's deployment constraints.
 func (u *Unit) Constraints() (*constraints.Value, error) {
-	cons, err := readConstraints(u.st, u.globalKey())
+	cons, err := readConstraints(u.st, u.globalAgentKey())
 	if errors.IsNotFound(err) {
 		// Lack of constraints indicates lack of unit.
 		return nil, errors.NotFoundf("unit")
@@ -1240,23 +1430,24 @@ func (u *Unit) AssignToNewMachineOrContainer() (err error) {
 	}
 
 	// Find a clean, empty machine on which to create a container.
-	var host machineDoc
 	hostCons := *cons
 	noContainer := instance.NONE
 	hostCons.Container = &noContainer
-	query, closer, err := u.findCleanMachineQuery(true, &hostCons)
+	query, err := u.findCleanMachineQuery(true, &hostCons)
 	if err != nil {
 		return err
 	}
+	machinesCollection, closer := u.st.getCollection(machinesC)
 	defer closer()
-	err = query.One(&host)
-	if err == mgo.ErrNotFound {
+	var host machineDoc
+	if err := machinesCollection.Find(query).One(&host); err == mgo.ErrNotFound {
 		// No existing clean, empty machine so create a new one.
 		// The container constraint will be used by AssignToNewMachine to create the required container.
 		return u.AssignToNewMachine()
 	} else if err != nil {
 		return err
 	}
+
 	svc, err := u.Service()
 	if err != nil {
 		return err
@@ -1307,15 +1498,170 @@ func (u *Unit) AssignToNewMachine() (err error) {
 	if err != nil {
 		return err
 	}
+	storageParams, err := u.machineStorageParams()
+	if err != nil {
+		return errors.Trace(err)
+	}
 	template := MachineTemplate{
-		Series:            u.doc.Series,
-		Constraints:       *cons,
-		Jobs:              []MachineJob{JobHostUnits},
-		RequestedNetworks: requestedNetworks,
+		Series:                u.doc.Series,
+		Constraints:           *cons,
+		Jobs:                  []MachineJob{JobHostUnits},
+		RequestedNetworks:     requestedNetworks,
+		Volumes:               storageParams.volumes,
+		VolumeAttachments:     storageParams.volumeAttachments,
+		Filesystems:           storageParams.filesystems,
+		FilesystemAttachments: storageParams.filesystemAttachments,
 	}
 	return u.assignToNewMachine(template, "", containerType)
 }
 
+// machineStorageParams returns parameters for creating volumes/filesystems
+// and volume/filesystem attachments for a machine that the unit will be
+// assigned to.
+func (u *Unit) machineStorageParams() (*machineStorageParams, error) {
+	storageAttachments, err := u.st.UnitStorageAttachments(u.UnitTag())
+	if err != nil {
+		return nil, errors.Annotate(err, "getting storage attachments")
+	}
+	svc, err := u.Service()
+	if err != nil {
+		return nil, errors.Trace(err)
+	}
+	curl, _ := svc.CharmURL()
+	if curl == nil {
+		return nil, errors.Errorf("no URL set for service %q", svc.Name())
+	}
+	ch, err := u.st.Charm(curl)
+	if err != nil {
+		return nil, errors.Annotate(err, "getting charm")
+	}
+	allCons, err := u.StorageConstraints()
+	if err != nil {
+		return nil, errors.Annotatef(err, "getting storage constraints")
+	}
+
+	chMeta := ch.Meta()
+
+	var volumes []MachineVolumeParams
+	var filesystems []MachineFilesystemParams
+	volumeAttachments := make(map[names.VolumeTag]VolumeAttachmentParams)
+	filesystemAttachments := make(map[names.FilesystemTag]FilesystemAttachmentParams)
+	for _, storageAttachment := range storageAttachments {
+		storage, err := u.st.StorageInstance(storageAttachment.StorageInstance())
+		if err != nil {
+			return nil, errors.Annotatef(err, "getting storage instance")
+		}
+		machineParams, err := machineStorageParamsForStorageInstance(
+			u.st, chMeta, u.UnitTag(), allCons, storage,
+		)
+		if err != nil {
+			return nil, errors.Trace(err)
+		}
+
+		volumes = append(volumes, machineParams.volumes...)
+		for k, v := range machineParams.volumeAttachments {
+			volumeAttachments[k] = v
+		}
+
+		filesystems = append(filesystems, machineParams.filesystems...)
+		for k, v := range machineParams.filesystemAttachments {
+			filesystemAttachments[k] = v
+		}
+	}
+	result := &machineStorageParams{
+		volumes,
+		volumeAttachments,
+		filesystems,
+		filesystemAttachments,
+	}
+	return result, nil
+}
+
+// machineStorageParamsForStorageInstance returns parameters for creating
+// volumes/filesystems and volume/filesystem attachments for a machine that
+// the unit will be assigned to. These parameters are based on a given storage
+// instance.
+func machineStorageParamsForStorageInstance(
+	st *State,
+	charmMeta *charm.Meta,
+	unit names.UnitTag,
+	allCons map[string]StorageConstraints,
+	storage StorageInstance,
+) (*machineStorageParams, error) {
+
+	charmStorage := charmMeta.Storage[storage.StorageName()]
+
+	var volumes []MachineVolumeParams
+	var filesystems []MachineFilesystemParams
+	volumeAttachments := make(map[names.VolumeTag]VolumeAttachmentParams)
+	filesystemAttachments := make(map[names.FilesystemTag]FilesystemAttachmentParams)
+
+	switch storage.Kind() {
+	case StorageKindBlock:
+		volumeAttachmentParams := VolumeAttachmentParams{
+			charmStorage.ReadOnly,
+		}
+		if unit == storage.Owner() {
+			// The storage instance is owned by the unit, so we'll need
+			// to create a volume.
+			cons := allCons[storage.StorageName()]
+			volumeParams := VolumeParams{
+				storage: storage.StorageTag(),
+				Pool:    cons.Pool,
+				Size:    cons.Size,
+			}
+			volumes = append(volumes, MachineVolumeParams{
+				volumeParams, volumeAttachmentParams,
+			})
+		} else {
+			// The storage instance is owned by the service, so there
+			// should be a (shared) volume already, for which we will
+			// just add an attachment.
+			volume, err := st.StorageInstanceVolume(storage.StorageTag())
+			if err != nil {
+				return nil, errors.Annotatef(err, "getting volume for storage %q", storage.Tag().Id())
+			}
+			volumeAttachments[volume.VolumeTag()] = volumeAttachmentParams
+		}
+	case StorageKindFilesystem:
+		filesystemAttachmentParams := FilesystemAttachmentParams{
+			charmStorage.Location,
+			charmStorage.ReadOnly,
+		}
+		if unit == storage.Owner() {
+			// The storage instance is owned by the unit, so we'll need
+			// to create a filesystem.
+			cons := allCons[storage.StorageName()]
+			filesystemParams := FilesystemParams{
+				storage: storage.StorageTag(),
+				Pool:    cons.Pool,
+				Size:    cons.Size,
+			}
+			filesystems = append(filesystems, MachineFilesystemParams{
+				filesystemParams, filesystemAttachmentParams,
+			})
+		} else {
+			// The storage instance is owned by the service, so there
+			// should be a (shared) filesystem already, for which we will
+			// just add an attachment.
+			filesystem, err := st.StorageInstanceFilesystem(storage.StorageTag())
+			if err != nil {
+				return nil, errors.Annotatef(err, "getting filesystem for storage %q", storage.Tag().Id())
+			}
+			filesystemAttachments[filesystem.FilesystemTag()] = filesystemAttachmentParams
+		}
+	default:
+		return nil, errors.Errorf("invalid storage kind %v", storage.Kind())
+	}
+	result := &machineStorageParams{
+		volumes,
+		volumeAttachments,
+		filesystems,
+		filesystemAttachments,
+	}
+	return result, nil
+}
+
 var noCleanMachines = stderrors.New("all eligible machines in use")
 
 // AssignToCleanMachine assigns u to a machine which is marked as clean. A machine
@@ -1352,23 +1698,20 @@ var hasNoContainersTerm = bson.DocElem{
 
 // findCleanMachineQuery returns a Mongo query to find clean (and possibly empty) machines with
 // characteristics matching the specified constraints.
-func (u *Unit) findCleanMachineQuery(requireEmpty bool, cons *constraints.Value) (_ *mgo.Query, _ func(), err error) {
+func (u *Unit) findCleanMachineQuery(requireEmpty bool, cons *constraints.Value) (bson.D, error) {
 	db, closer := u.st.newDB()
-	defer func() {
-		if err != nil {
-			closer()
-		}
-	}()
-	containerRefsCollection := db.C(containerRefsC)
+	defer closer()
+	containerRefsCollection, closer := db.GetCollection(containerRefsC)
+	defer closer()
 
 	// Select all machines that can accept principal units and are clean.
 	var containerRefs []machineContainers
 	// If we need empty machines, first build up a list of machine ids which have containers
 	// so we can exclude those.
 	if requireEmpty {
-		err = containerRefsCollection.Find(bson.D{hasContainerTerm}).All(&containerRefs)
+		err := containerRefsCollection.Find(bson.D{hasContainerTerm}).All(&containerRefs)
 		if err != nil {
-			return nil, closer, err
+			return nil, err
 		}
 	}
 	var machinesWithContainers = make([]string, len(containerRefs))
@@ -1421,10 +1764,11 @@ func (u *Unit) findCleanMachineQuery(requireEmpty bool, cons *constraints.Value)
 		suitableTerms = append(suitableTerms, bson.DocElem{"tags", bson.D{{"$all", *cons.Tags}}})
 	}
 	if len(suitableTerms) > 0 {
-		instanceData := db.C(instanceDataC)
-		err := instanceData.Find(suitableTerms).Select(bson.M{"_id": 1}).All(&suitableInstanceData)
+		instanceDataCollection, closer := db.GetCollection(instanceDataC)
+		defer closer()
+		err := instanceDataCollection.Find(suitableTerms).Select(bson.M{"_id": 1}).All(&suitableInstanceData)
 		if err != nil {
-			return nil, closer, err
+			return nil, err
 		}
 		var suitableIds = make([]string, len(suitableInstanceData))
 		for i, m := range suitableInstanceData {
@@ -1432,8 +1776,7 @@ func (u *Unit) findCleanMachineQuery(requireEmpty bool, cons *constraints.Value)
 		}
 		terms = append(terms, bson.DocElem{"_id", bson.D{{"$in", suitableIds}}})
 	}
-	machines := db.C(machinesC)
-	return machines.Find(terms), closer, nil
+	return terms, nil
 }
 
 // assignToCleanMaybeEmptyMachine implements AssignToCleanMachine and AssignToCleanEmptyMachine.
@@ -1451,25 +1794,38 @@ func (u *Unit) assignToCleanMaybeEmptyMachine(requireEmpty bool) (m *Machine, er
 		return nil, err
 	}
 
+	// TODO(axw) once we support dynamic storage provisioning, we
+	// should check whether all of the storage constraints can be
+	// fulfilled dynamically (by querying a policy).
+	storageCons, err := u.StorageConstraints()
+	if err != nil {
+		assignContextf(&err, u, context)
+		return nil, err
+	}
+	if len(storageCons) > 0 {
+		return nil, noCleanMachines
+	}
+
 	// Get the unit constraints to see what deployment requirements we have to adhere to.
 	cons, err := u.Constraints()
 	if err != nil {
 		assignContextf(&err, u, context)
 		return nil, err
 	}
-	query, closer, err := u.findCleanMachineQuery(requireEmpty, cons)
+	query, err := u.findCleanMachineQuery(requireEmpty, cons)
 	if err != nil {
 		assignContextf(&err, u, context)
 		return nil, err
 	}
-	defer closer()
 
 	// Find all of the candidate machines, and associated
 	// instances for those that are provisioned. Instances
 	// will be distributed across in preference to
 	// unprovisioned machines.
+	machinesCollection, closer := u.st.getCollection(machinesC)
+	defer closer()
 	var mdocs []*machineDoc
-	if err := query.All(&mdocs); err != nil {
+	if err := machinesCollection.Find(query).All(&mdocs); err != nil {
 		assignContextf(&err, u, context)
 		return nil, err
 	}
@@ -1564,7 +1920,8 @@ func (u *Unit) UnassignFromMachine() (err error) {
 type ActionSpecsByName map[string]charm.ActionSpec
 
 // AddAction adds a new Action of type name and using arguments payload to
-// this Unit, and returns its ID.
+// this Unit, and returns its ID.  Note that the use of spec.InsertDefaults
+// mutates payload.
 func (u *Unit) AddAction(name string, payload map[string]interface{}) (*Action, error) {
 	if len(name) == 0 {
 		return nil, errors.New("no action name given")
@@ -1577,11 +1934,16 @@ func (u *Unit) AddAction(name string, payload map[string]interface{}) (*Action,
 	if !ok {
 		return nil, errors.Errorf("action %q not defined on unit %q", name, u.Name())
 	}
-	_, err = spec.ValidateParams(payload)
+	// Reject bad payloads before attempting to insert defaults.
+	err = spec.ValidateParams(payload)
 	if err != nil {
 		return nil, err
 	}
-	return u.st.EnqueueAction(u.Tag(), name, payload)
+	payloadWithDefaults, err := spec.InsertDefaults(payload)
+	if err != nil {
+		return nil, err
+	}
+	return u.st.EnqueueAction(u.Tag(), name, payloadWithDefaults)
 }
 
 // ActionSpecs gets the ActionSpec map for the Unit's charm.
@@ -1638,17 +2000,25 @@ func (u *Unit) PendingActions() ([]*Action, error) {
 	return u.st.matchingActionsPending(u)
 }
 
+// RunningActions returns a list of actions running on this unit.
+func (u *Unit) RunningActions() ([]*Action, error) {
+	return u.st.matchingActionsRunning(u)
+}
+
 // Resolve marks the unit as having had any previous state transition
 // problems resolved, and informs the unit that it may attempt to
 // reestablish normal workflow. The retryHooks parameter informs
 // whether to attempt to reexecute previous failed hooks or to continue
 // as if they had succeeded before.
 func (u *Unit) Resolve(retryHooks bool) error {
-	status, _, _, err := u.Status()
+	// We currently check agent status to see if a unit is
+	// in error state. As the new Juju Health work is completed,
+	// this will change to checking the unit status.
+	statusInfo, err := u.Status()
 	if err != nil {
 		return err
 	}
-	if status != StatusError {
+	if statusInfo.Status != StatusError {
 		return errors.Errorf("unit %q is not in an error state", u)
 	}
 	mode := ResolvedNoHooks
@@ -1709,16 +2079,39 @@ func (u *Unit) ClearResolved() error {
 	return nil
 }
 
-// AddMetric adds a new batch of metrics to the database.
-// A UUID for the metric will be generated and the new MetricBatch will be returned
-func (u *Unit) AddMetrics(created time.Time, metrics []Metric) (*MetricBatch, error) {
-	charmUrl, ok := u.CharmURL()
-	if !ok {
-		return nil, stderrors.New("failed to add metrics, couldn't find charm url")
+// AddMetrics adds a new batch of metrics to the database.
+func (u *Unit) AddMetrics(batchUUID string, created time.Time, charmURLRaw string, metrics []Metric) (*MetricBatch, error) {
+	var charmURL *charm.URL
+	if charmURLRaw == "" {
+		var ok bool
+		charmURL, ok = u.CharmURL()
+		if !ok {
+			return nil, stderrors.New("failed to add metrics, couldn't find charm url")
+		}
+	} else {
+		var err error
+		charmURL, err = charm.ParseURL(charmURLRaw)
+		if err != nil {
+			return nil, errors.NewNotValid(err, "could not parse charm URL")
+		}
 	}
 	service, err := u.Service()
 	if err != nil {
 		return nil, errors.Annotatef(err, "couldn't retrieve service whilst adding metrics")
 	}
-	return u.st.addMetrics(u.UnitTag(), charmUrl, created, metrics, service.MetricCredentials())
+	batch := BatchParam{
+		UUID:        batchUUID,
+		CharmURL:    charmURL,
+		Created:     created,
+		Metrics:     metrics,
+		Credentials: service.MetricCredentials(),
+	}
+	return u.st.addMetrics(u.UnitTag(), batch)
+}
+
+// StorageConstraints returns the unit's storage constraints.
+func (u *Unit) StorageConstraints() (map[string]StorageConstraints, error) {
+	// TODO(axw) eventually we should be able to override service
+	// storage constraints at the unit level.
+	return readStorageConstraints(u.st, serviceGlobalKey(u.doc.Service))
 }
