@@ -1,4 +1,4 @@
-// Copyright 2012-2015 Canonical Ltd.
+// Copyright 2015 Canonical Ltd.
 // Licensed under the AGPLv3, see LICENCE file for details.
 
 package state_test
@@ -10,7 +10,7 @@ import (
 	jc "github.com/juju/testing/checkers"
 	jujutxn "github.com/juju/txn"
 	gc "gopkg.in/check.v1"
-	"gopkg.in/juju/charm.v4"
+	"gopkg.in/juju/charm.v5"
 
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/network"
@@ -182,7 +182,7 @@ func (s *UnitSuite) setAssignedMachineAddresses(c *gc.C, u *state.Unit) {
 	c.Assert(err, jc.ErrorIsNil)
 	err = machine.SetProvisioned("i-exist", "fake_nonce", nil)
 	c.Assert(err, jc.ErrorIsNil)
-	err = machine.SetAddresses(network.Address{
+	err = machine.SetProviderAddresses(network.Address{
 		Type:  network.IPv4Address,
 		Scope: network.ScopeCloudLocal,
 		Value: "private.address.example.com",
@@ -215,10 +215,10 @@ func (s *UnitSuite) TestPublicAddress(c *gc.C) {
 	c.Check(address, gc.Equals, "")
 	c.Assert(ok, jc.IsFalse)
 
-	public := network.NewAddress("8.8.8.8", network.ScopePublic)
-	private := network.NewAddress("127.0.0.1", network.ScopeCloudLocal)
+	public := network.NewScopedAddress("8.8.8.8", network.ScopePublic)
+	private := network.NewScopedAddress("127.0.0.1", network.ScopeCloudLocal)
 
-	err = machine.SetAddresses(public, private)
+	err = machine.SetProviderAddresses(public, private)
 	c.Assert(err, jc.ErrorIsNil)
 
 	address, ok = s.unit.PublicAddress()
@@ -232,11 +232,11 @@ func (s *UnitSuite) TestPublicAddressMachineAddresses(c *gc.C) {
 	err = s.unit.AssignToMachine(machine)
 	c.Assert(err, jc.ErrorIsNil)
 
-	publicProvider := network.NewAddress("8.8.8.8", network.ScopePublic)
-	privateProvider := network.NewAddress("127.0.0.1", network.ScopeCloudLocal)
-	privateMachine := network.NewAddress("127.0.0.2", network.ScopeUnknown)
+	publicProvider := network.NewScopedAddress("8.8.8.8", network.ScopePublic)
+	privateProvider := network.NewScopedAddress("127.0.0.1", network.ScopeCloudLocal)
+	privateMachine := network.NewAddress("127.0.0.2")
 
-	err = machine.SetAddresses(privateProvider)
+	err = machine.SetProviderAddresses(privateProvider)
 	c.Assert(err, jc.ErrorIsNil)
 	err = machine.SetMachineAddresses(privateMachine)
 	c.Assert(err, jc.ErrorIsNil)
@@ -244,7 +244,7 @@ func (s *UnitSuite) TestPublicAddressMachineAddresses(c *gc.C) {
 	c.Check(address, gc.Equals, "127.0.0.1")
 	c.Assert(ok, jc.IsTrue)
 
-	err = machine.SetAddresses(publicProvider, privateProvider)
+	err = machine.SetProviderAddresses(publicProvider, privateProvider)
 	c.Assert(err, jc.ErrorIsNil)
 	address, ok = s.unit.PublicAddress()
 	c.Check(address, gc.Equals, "8.8.8.8")
@@ -272,10 +272,10 @@ func (s *UnitSuite) TestPrivateAddress(c *gc.C) {
 	c.Check(address, gc.Equals, "")
 	c.Assert(ok, jc.IsFalse)
 
-	public := network.NewAddress("8.8.8.8", network.ScopePublic)
-	private := network.NewAddress("127.0.0.1", network.ScopeCloudLocal)
+	public := network.NewScopedAddress("8.8.8.8", network.ScopePublic)
+	private := network.NewScopedAddress("127.0.0.1", network.ScopeCloudLocal)
 
-	err = machine.SetAddresses(public, private)
+	err = machine.SetProviderAddresses(public, private)
 	c.Assert(err, jc.ErrorIsNil)
 
 	address, ok = s.unit.PrivateAddress()
@@ -526,148 +526,6 @@ func (s *UnitSuite) TestRefresh(c *gc.C) {
 	c.Assert(err, jc.Satisfies, errors.IsNotFound)
 }
 
-func (s *UnitSuite) TestGetSetStatusWhileAlive(c *gc.C) {
-	err := s.unit.SetStatus(state.StatusError, "", nil)
-	c.Assert(err, gc.ErrorMatches, `cannot set status "error" without info`)
-	err = s.unit.SetStatus(state.StatusAllocating, "", nil)
-	c.Assert(err, gc.ErrorMatches, `cannot set status "allocating"`)
-	err = s.unit.SetStatus(state.StatusFailed, "", nil)
-	c.Assert(err, gc.ErrorMatches, `cannot set status "failed"`)
-	err = s.unit.SetStatus(state.Status("vliegkat"), "orville", nil)
-	c.Assert(err, gc.ErrorMatches, `cannot set invalid status "vliegkat"`)
-
-	status, info, data, err := s.unit.Status()
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(status, gc.Equals, state.StatusAllocating)
-	c.Assert(info, gc.Equals, "")
-	c.Assert(data, gc.HasLen, 0)
-
-	err = s.unit.SetStatus(state.StatusActive, "", nil)
-	c.Assert(err, jc.ErrorIsNil)
-	status, info, data, err = s.unit.Status()
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(status, gc.Equals, state.StatusActive)
-	c.Assert(info, gc.Equals, "")
-	c.Assert(data, gc.HasLen, 0)
-
-	err = s.unit.SetStatus(state.StatusError, "test-hook failed", map[string]interface{}{
-		"foo": "bar",
-	})
-	c.Assert(err, jc.ErrorIsNil)
-	status, info, data, err = s.unit.Status()
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(status, gc.Equals, state.StatusError)
-	c.Assert(info, gc.Equals, "test-hook failed")
-	c.Assert(data, gc.DeepEquals, map[string]interface{}{
-		"foo": "bar",
-	})
-}
-
-func (s *UnitSuite) TestGetSetStatusWhileNotAlive(c *gc.C) {
-	err := s.unit.Destroy()
-	c.Assert(err, jc.ErrorIsNil)
-	err = s.unit.SetStatus(state.StatusActive, "not really", nil)
-	c.Assert(err, gc.ErrorMatches, `cannot set status of unit "wordpress/0": not found or dead`)
-	_, _, _, err = s.unit.Status()
-	c.Assert(err, gc.ErrorMatches, "status not found")
-
-	err = s.unit.EnsureDead()
-	c.Assert(err, jc.ErrorIsNil)
-	err = s.unit.SetStatus(state.StatusActive, "not really", nil)
-	c.Assert(err, gc.ErrorMatches, `cannot set status of unit "wordpress/0": not found or dead`)
-	_, _, _, err = s.unit.Status()
-	c.Assert(err, gc.ErrorMatches, "status not found")
-}
-
-func (s *UnitSuite) TestGetSetStatusDataStandard(c *gc.C) {
-	err := s.unit.SetStatus(state.StatusActive, "", nil)
-	c.Assert(err, jc.ErrorIsNil)
-	_, _, _, err = s.unit.Status()
-	c.Assert(err, jc.ErrorIsNil)
-
-	// Regular status setting with data.
-	err = s.unit.SetStatus(state.StatusError, "test-hook failed", map[string]interface{}{
-		"1st-key": "one",
-		"2nd-key": 2,
-		"3rd-key": true,
-	})
-	c.Assert(err, jc.ErrorIsNil)
-
-	status, info, data, err := s.unit.Status()
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(status, gc.Equals, state.StatusError)
-	c.Assert(info, gc.Equals, "test-hook failed")
-	c.Assert(data, gc.DeepEquals, map[string]interface{}{
-		"1st-key": "one",
-		"2nd-key": 2,
-		"3rd-key": true,
-	})
-}
-
-func (s *UnitSuite) TestGetSetStatusDataMongo(c *gc.C) {
-	err := s.unit.SetStatus(state.StatusActive, "", nil)
-	c.Assert(err, jc.ErrorIsNil)
-	_, _, _, err = s.unit.Status()
-	c.Assert(err, jc.ErrorIsNil)
-
-	// Status setting with MongoDB special values.
-	err = s.unit.SetStatus(state.StatusError, "mongo", map[string]interface{}{
-		`{name: "Joe"}`: "$where",
-		"eval":          `eval(function(foo) { return foo; }, "bar")`,
-		"mapReduce":     "mapReduce",
-		"group":         "group",
-	})
-	c.Assert(err, jc.ErrorIsNil)
-
-	status, info, data, err := s.unit.Status()
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(status, gc.Equals, state.StatusError)
-	c.Assert(info, gc.Equals, "mongo")
-	c.Assert(data, gc.DeepEquals, map[string]interface{}{
-		`{name: "Joe"}`: "$where",
-		"eval":          `eval(function(foo) { return foo; }, "bar")`,
-		"mapReduce":     "mapReduce",
-		"group":         "group",
-	})
-}
-
-func (s *UnitSuite) TestGetSetStatusDataChange(c *gc.C) {
-	err := s.unit.SetStatus(state.StatusActive, "", nil)
-	c.Assert(err, jc.ErrorIsNil)
-	_, _, _, err = s.unit.Status()
-	c.Assert(err, jc.ErrorIsNil)
-
-	// Status setting and changing data afterwards.
-	data := map[string]interface{}{
-		"1st-key": "one",
-		"2nd-key": 2,
-		"3rd-key": true,
-	}
-	err = s.unit.SetStatus(state.StatusError, "test-hook failed", data)
-	c.Assert(err, jc.ErrorIsNil)
-	data["4th-key"] = 4.0
-
-	status, info, data, err := s.unit.Status()
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(status, gc.Equals, state.StatusError)
-	c.Assert(info, gc.Equals, "test-hook failed")
-	c.Assert(data, gc.DeepEquals, map[string]interface{}{
-		"1st-key": "one",
-		"2nd-key": 2,
-		"3rd-key": true,
-	})
-
-	// Set status data to nil, so an empty map will be returned.
-	err = s.unit.SetStatus(state.StatusActive, "", nil)
-	c.Assert(err, jc.ErrorIsNil)
-
-	status, info, data, err = s.unit.Status()
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(status, gc.Equals, state.StatusActive)
-	c.Assert(info, gc.Equals, "")
-	c.Assert(data, gc.HasLen, 0)
-}
-
 func (s *UnitSuite) TestSetCharmURLSuccess(c *gc.C) {
 	preventUnitDestroyRemove(c, s.unit)
 	curl, ok := s.unit.CharmURL()
@@ -778,7 +636,7 @@ func (s *UnitSuite) TestSetCharmURLRetriesWithDifferentURL(c *gc.C) {
 
 func (s *UnitSuite) TestDestroySetStatusRetry(c *gc.C) {
 	defer state.SetRetryHooks(c, s.State, func() {
-		err := s.unit.SetStatus(state.StatusActive, "", nil)
+		err := s.unit.SetAgentStatus(state.StatusIdle, "", nil)
 		c.Assert(err, jc.ErrorIsNil)
 	}, func() {
 		assertLife(c, s.unit, state.Dying)
@@ -879,25 +737,26 @@ func (s *UnitSuite) TestCannotShortCircuitDestroyWithSubordinates(c *gc.C) {
 	assertLife(c, s.unit, state.Dying)
 }
 
-func (s *UnitSuite) TestCannotShortCircuitDestroyWithStatus(c *gc.C) {
+func (s *UnitSuite) TestCannotShortCircuitDestroyWithAgentStatus(c *gc.C) {
 	for i, test := range []struct {
 		status state.Status
 		info   string
 	}{{
-		state.StatusActive, "",
+		state.StatusExecuting, "blah",
 	}, {
-		state.StatusError, "blah",
+		state.StatusIdle, "blah",
 	}, {
-		state.StatusStopping, "",
+		state.StatusFailed, "blah",
+	}, {
+		state.StatusRebooting, "blah",
 	}} {
 		c.Logf("test %d: %s", i, test.status)
 		unit, err := s.service.AddUnit()
 		c.Assert(err, jc.ErrorIsNil)
-		err = unit.SetStatus(test.status, test.info, nil)
+		err = unit.SetAgentStatus(test.status, test.info, nil)
 		c.Assert(err, jc.ErrorIsNil)
 		err = unit.Destroy()
 		c.Assert(err, jc.ErrorIsNil)
-		c.Assert(unit.Life(), gc.Equals, state.Dying)
 		assertLife(c, unit, state.Dying)
 	}
 }
@@ -1005,7 +864,7 @@ func (s *UnitSuite) TestResolve(c *gc.C) {
 	err = s.unit.Resolve(true)
 	c.Assert(err, gc.ErrorMatches, `unit "wordpress/0" is not in an error state`)
 
-	err = s.unit.SetStatus(state.StatusError, "gaaah", nil)
+	err = s.unit.SetAgentStatus(state.StatusError, "gaaah", nil)
 	c.Assert(err, jc.ErrorIsNil)
 	err = s.unit.Resolve(false)
 	c.Assert(err, jc.ErrorIsNil)
@@ -1059,15 +918,15 @@ func (s *UnitSuite) TestOpenedPorts(c *gc.C) {
 	// Verify ports can be opened and closed only when the unit has
 	// assigned machine.
 	err := s.unit.OpenPort("tcp", 10)
-	c.Assert(err, jc.Satisfies, state.IsNotAssigned)
+	c.Assert(errors.Cause(err), jc.Satisfies, errors.IsNotAssigned)
 	err = s.unit.OpenPorts("tcp", 10, 20)
-	c.Assert(err, jc.Satisfies, state.IsNotAssigned)
+	c.Assert(errors.Cause(err), jc.Satisfies, errors.IsNotAssigned)
 	err = s.unit.ClosePort("tcp", 10)
-	c.Assert(err, jc.Satisfies, state.IsNotAssigned)
+	c.Assert(errors.Cause(err), jc.Satisfies, errors.IsNotAssigned)
 	err = s.unit.ClosePorts("tcp", 10, 20)
-	c.Assert(err, jc.Satisfies, state.IsNotAssigned)
+	c.Assert(errors.Cause(err), jc.Satisfies, errors.IsNotAssigned)
 	open, err := s.unit.OpenedPorts()
-	c.Assert(err, jc.Satisfies, state.IsNotAssigned)
+	c.Assert(errors.Cause(err), jc.Satisfies, errors.IsNotAssigned)
 	c.Assert(open, gc.HasLen, 0)
 
 	machine, err := s.State.AddMachine("quantal", state.JobHostUnits)
@@ -1659,25 +1518,6 @@ func (s *UnitSuite) TestWatchUnit(c *gc.C) {
 	testing.NewNotifyWatcherC(c, s.State, w).AssertOneChange()
 }
 
-func (s *UnitSuite) TestAnnotatorForUnit(c *gc.C) {
-	testAnnotator(c, func() (state.Annotator, error) {
-		return s.State.Unit("wordpress/0")
-	})
-}
-
-func (s *UnitSuite) TestAnnotationRemovalForUnit(c *gc.C) {
-	annotations := map[string]string{"mykey": "myvalue"}
-	err := s.unit.SetAnnotations(annotations)
-	c.Assert(err, jc.ErrorIsNil)
-	err = s.unit.EnsureDead()
-	c.Assert(err, jc.ErrorIsNil)
-	err = s.unit.Remove()
-	c.Assert(err, jc.ErrorIsNil)
-	ann, err := s.unit.Annotations()
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(ann, gc.DeepEquals, make(map[string]string))
-}
-
 func (s *UnitSuite) TestUnitAgentTools(c *gc.C) {
 	preventUnitDestroyRemove(c, s.unit)
 	testAgentTools(c, s.unit, `unit "wordpress/0"`)
