@@ -7,15 +7,18 @@ import (
 	"fmt"
 	"math/rand"
 	"strconv"
+	"sync/atomic"
 	"time"
 
 	"github.com/juju/names"
 	jc "github.com/juju/testing/checkers"
 	"github.com/juju/utils"
 	gc "gopkg.in/check.v1"
-	"gopkg.in/juju/charm.v4"
+	"gopkg.in/juju/charm.v5"
 
+	"github.com/juju/juju/environs"
 	"github.com/juju/juju/instance"
+	"github.com/juju/juju/network"
 	"github.com/juju/juju/state"
 	"github.com/juju/juju/testcharms"
 	"github.com/juju/juju/testing"
@@ -26,10 +29,11 @@ const (
 )
 
 type Factory struct {
-	st    *state.State
-	index int
+	st *state.State
 }
 
+var index uint32
+
 func NewFactory(st *state.State) *Factory {
 	return &Factory{st: st}
 }
@@ -66,6 +70,9 @@ type MachineParams struct {
 	Nonce           string
 	InstanceId      instance.Id
 	Characteristics *instance.HardwareCharacteristics
+	Addresses       []network.Address
+	Volumes         []state.MachineVolumeParams
+	Filesystems     []state.MachineFilesystemParams
 }
 
 // ServiceParams is used when specifying parameters for a new service.
@@ -73,13 +80,16 @@ type ServiceParams struct {
 	Name    string
 	Charm   *state.Charm
 	Creator names.Tag
+	Status  *state.StatusInfo
 }
 
 // UnitParams are used to create units.
 type UnitParams struct {
 	Service     *state.Service
 	Machine     *state.Machine
+	Password    string
 	SetCharmURL bool
+	Status      *state.StatusInfo
 }
 
 // RelationParams are used to create relations.
@@ -98,6 +108,7 @@ type EnvParams struct {
 	Name        string
 	Owner       names.Tag
 	ConfigAttrs testing.Attrs
+	Prepare     bool
 }
 
 // RandomSuffix adds a random 5 character suffix to the presented string.
@@ -109,16 +120,16 @@ func (*Factory) RandomSuffix(prefix string) string {
 	return result
 }
 
-func (factory *Factory) UniqueInteger() int {
-	factory.index++
-	return factory.index
+func uniqueInteger() int {
+	index = atomic.AddUint32(&index, 1)
+	return int(index)
 }
 
-func (factory *Factory) UniqueString(prefix string) string {
+func uniqueString(prefix string) string {
 	if prefix == "" {
 		prefix = "no-prefix"
 	}
-	return fmt.Sprintf("%s-%d", prefix, factory.UniqueInteger())
+	return fmt.Sprintf("%s-%d", prefix, uniqueInteger())
 }
 
 // MakeUser will create a user with values defined by the params.
@@ -130,10 +141,10 @@ func (factory *Factory) MakeUser(c *gc.C, params *UserParams) *state.User {
 		params = &UserParams{}
 	}
 	if params.Name == "" {
-		params.Name = factory.UniqueString("username")
+		params.Name = uniqueString("username")
 	}
 	if params.DisplayName == "" {
-		params.DisplayName = factory.UniqueString("display name")
+		params.DisplayName = uniqueString("display name")
 	}
 	if params.Password == "" {
 		params.Password = "password"
@@ -148,7 +159,7 @@ func (factory *Factory) MakeUser(c *gc.C, params *UserParams) *state.User {
 		params.Name, params.DisplayName, params.Password, creatorUserTag.Name())
 	c.Assert(err, jc.ErrorIsNil)
 	if !params.NoEnvUser {
-		_, err := factory.st.AddEnvironmentUser(user.UserTag(), names.NewUserTag(user.CreatedBy()))
+		_, err := factory.st.AddEnvironmentUser(user.UserTag(), names.NewUserTag(user.CreatedBy()), params.DisplayName)
 		c.Assert(err, jc.ErrorIsNil)
 	}
 	if params.Disabled {
@@ -170,12 +181,16 @@ func (factory *Factory) MakeEnvUser(c *gc.C, params *EnvUserParams) *state.Envir
 		user := factory.MakeUser(c, &UserParams{NoEnvUser: true})
 		params.User = user.UserTag().Username()
 	}
+	if params.DisplayName == "" {
+		params.DisplayName = uniqueString("display name")
+	}
 	if params.CreatedBy == nil {
-		user := factory.MakeUser(c, nil)
-		params.CreatedBy = user.UserTag()
+		env, err := factory.st.Environment()
+		c.Assert(err, jc.ErrorIsNil)
+		params.CreatedBy = env.Owner()
 	}
 	createdByUserTag := params.CreatedBy.(names.UserTag)
-	envUser, err := factory.st.AddEnvironmentUser(names.NewUserTag(params.User), createdByUserTag)
+	envUser, err := factory.st.AddEnvironmentUser(names.NewUserTag(params.User), createdByUserTag, params.DisplayName)
 	c.Assert(err, jc.ErrorIsNil)
 	return envUser
 }
@@ -194,7 +209,7 @@ func (factory *Factory) paramsFillDefaults(c *gc.C, params *MachineParams) *Mach
 		params.Jobs = []state.MachineJob{state.JobHostUnits}
 	}
 	if params.InstanceId == "" {
-		params.InstanceId = instance.Id(factory.UniqueString("id"))
+		params.InstanceId = instance.Id(uniqueString("id"))
 	}
 	if params.Password == "" {
 		var err error
@@ -236,14 +251,33 @@ func (factory *Factory) MakeMachineNested(c *gc.C, parentId string, params *Mach
 // set.
 // If params is not specified, defaults are used.
 func (factory *Factory) MakeMachine(c *gc.C, params *MachineParams) *state.Machine {
+	machine, _ := factory.MakeMachineReturningPassword(c, params)
+	return machine
+}
+
+// MakeMachineReturningPassword will add a machine with values defined in
+// params. For some values in params, if they are missing, some meaningful
+// empty values will be set. If params is not specified, defaults are used.
+// The machine and its password are returned.
+func (factory *Factory) MakeMachineReturningPassword(c *gc.C, params *MachineParams) (*state.Machine, string) {
 	params = factory.paramsFillDefaults(c, params)
-	machine, err := factory.st.AddMachine(params.Series, params.Jobs...)
+	machineTemplate := state.MachineTemplate{
+		Series:      params.Series,
+		Jobs:        params.Jobs,
+		Volumes:     params.Volumes,
+		Filesystems: params.Filesystems,
+	}
+	machine, err := factory.st.AddOneMachine(machineTemplate)
 	c.Assert(err, jc.ErrorIsNil)
 	err = machine.SetProvisioned(params.InstanceId, params.Nonce, params.Characteristics)
 	c.Assert(err, jc.ErrorIsNil)
 	err = machine.SetPassword(params.Password)
 	c.Assert(err, jc.ErrorIsNil)
-	return machine
+	if len(params.Addresses) > 0 {
+		err := machine.SetProviderAddresses(params.Addresses...)
+		c.Assert(err, jc.ErrorIsNil)
+	}
+	return machine, params.Password
 }
 
 // MakeCharm creates a charm with the values specified in params.
@@ -265,7 +299,7 @@ func (factory *Factory) MakeCharm(c *gc.C, params *CharmParams) *state.Charm {
 		params.Series = "quantal"
 	}
 	if params.Revision == "" {
-		params.Revision = fmt.Sprintf("%d", factory.UniqueInteger())
+		params.Revision = fmt.Sprintf("%d", uniqueInteger())
 	}
 	if params.URL == "" {
 		params.URL = fmt.Sprintf("cs:%s/%s-%s", params.Series, params.Name, params.Revision)
@@ -274,7 +308,7 @@ func (factory *Factory) MakeCharm(c *gc.C, params *CharmParams) *state.Charm {
 	ch := testcharms.Repo.CharmDir(params.Name)
 
 	curl := charm.MustParseURL(params.URL)
-	bundleSHA256 := factory.UniqueString("bundlesha")
+	bundleSHA256 := uniqueString("bundlesha")
 	charm, err := factory.st.AddCharm(ch, curl, "fake-storage-path", bundleSHA256)
 	c.Assert(err, jc.ErrorIsNil)
 	return charm
@@ -298,8 +332,14 @@ func (factory *Factory) MakeService(c *gc.C, params *ServiceParams) *state.Servi
 		params.Creator = creator.Tag()
 	}
 	_ = params.Creator.(names.UserTag)
-	service, err := factory.st.AddService(params.Name, params.Creator.String(), params.Charm, nil)
+	service, err := factory.st.AddService(params.Name, params.Creator.String(), params.Charm, nil, nil, nil)
 	c.Assert(err, jc.ErrorIsNil)
+
+	if params.Status != nil {
+		err = service.SetStatus(params.Status.Status, params.Status.Message, params.Status.Data)
+		c.Assert(err, jc.ErrorIsNil)
+	}
+
 	return service
 }
 
@@ -307,6 +347,14 @@ func (factory *Factory) MakeService(c *gc.C, params *ServiceParams) *state.Servi
 // sane defaults for missing values.
 // If params is not specified, defaults are used.
 func (factory *Factory) MakeUnit(c *gc.C, params *UnitParams) *state.Unit {
+	unit, _ := factory.MakeUnitReturningPassword(c, params)
+	return unit
+}
+
+// MakeUnit creates a service unit with specified params, filling in sane
+// defaults for missing values. If params is not specified, defaults are used.
+// The unit and its password are returned.
+func (factory *Factory) MakeUnitReturningPassword(c *gc.C, params *UnitParams) (*state.Unit, string) {
 	if params == nil {
 		params = &UnitParams{}
 	}
@@ -316,6 +364,11 @@ func (factory *Factory) MakeUnit(c *gc.C, params *UnitParams) *state.Unit {
 	if params.Service == nil {
 		params.Service = factory.MakeService(c, nil)
 	}
+	if params.Password == "" {
+		var err error
+		params.Password, err = utils.RandomPassword()
+		c.Assert(err, jc.ErrorIsNil)
+	}
 	unit, err := params.Service.AddUnit()
 	c.Assert(err, jc.ErrorIsNil)
 	err = unit.AssignToMachine(params.Machine)
@@ -325,7 +378,15 @@ func (factory *Factory) MakeUnit(c *gc.C, params *UnitParams) *state.Unit {
 		err = unit.SetCharmURL(serviceCharmURL)
 		c.Assert(err, jc.ErrorIsNil)
 	}
-	return unit
+	err = unit.SetPassword(params.Password)
+	c.Assert(err, jc.ErrorIsNil)
+
+	if params.Status != nil {
+		err = unit.SetStatus(params.Status.Status, params.Status.Message, params.Status.Data)
+		c.Assert(err, jc.ErrorIsNil)
+	}
+
+	return unit, params.Password
 }
 
 // MakeMetric makes a metric with specified params, filling in
@@ -345,10 +406,13 @@ func (factory *Factory) MakeMetric(c *gc.C, params *MetricParams) *state.MetricB
 		params.Time = &now
 	}
 	if params.Metrics == nil {
-		params.Metrics = []state.Metric{{"pings", strconv.Itoa(factory.UniqueInteger()), *params.Time}}
+		params.Metrics = []state.Metric{{"pings", strconv.Itoa(uniqueInteger()), *params.Time}}
 	}
 
-	metric, err := params.Unit.AddMetrics(*params.Time, params.Metrics)
+	chURL, ok := params.Unit.CharmURL()
+	c.Assert(ok, gc.Equals, true)
+
+	metric, err := params.Unit.AddMetrics(utils.MustNewUUID().String(), *params.Time, chURL.String(), params.Metrics)
 	c.Assert(err, jc.ErrorIsNil)
 	if params.Sent {
 		err := metric.SetSent()
@@ -401,21 +465,38 @@ func (factory *Factory) MakeEnvironment(c *gc.C, params *EnvParams) *state.State
 		params = new(EnvParams)
 	}
 	if params.Name == "" {
-		params.Name = factory.UniqueString("testenv")
+		params.Name = uniqueString("testenv")
 	}
 	if params.Owner == nil {
 		origEnv, err := factory.st.Environment()
 		c.Assert(err, jc.ErrorIsNil)
 		params.Owner = origEnv.Owner()
 	}
+	// It only makes sense to make an environment with the same provider
+	// as the initial environment, or things will break elsewhere.
+	currentCfg, err := factory.st.EnvironConfig()
+	c.Assert(err, jc.ErrorIsNil)
 
 	uuid, err := utils.NewUUID()
 	c.Assert(err, jc.ErrorIsNil)
 	cfg := testing.CustomEnvironConfig(c, testing.Attrs{
-		"name": params.Name,
-		"uuid": uuid.String(),
+		"name":       params.Name,
+		"uuid":       uuid.String(),
+		"type":       currentCfg.Type(),
+		"state-port": currentCfg.StatePort(),
+		"api-port":   currentCfg.APIPort(),
 	}.Merge(params.ConfigAttrs))
 	_, st, err := factory.st.NewEnvironment(cfg, params.Owner.(names.UserTag))
 	c.Assert(err, jc.ErrorIsNil)
+	if params.Prepare {
+		// Prepare the environment.
+		provider, err := environs.Provider(cfg.Type())
+		c.Assert(err, jc.ErrorIsNil)
+		cfg, err = provider.PrepareForCreateEnvironment(cfg)
+		c.Assert(err, jc.ErrorIsNil)
+		// Now save the config back.
+		err = st.UpdateEnvironConfig(cfg.AllAttrs(), nil, nil)
+		c.Assert(err, jc.ErrorIsNil)
+	}
 	return st
 }
