@@ -5,73 +5,38 @@ package factory_test
 
 import (
 	"fmt"
+	"regexp"
 	"time"
 
 	"github.com/juju/errors"
 	"github.com/juju/names"
-	jtesting "github.com/juju/testing"
 	jc "github.com/juju/testing/checkers"
 	"github.com/juju/utils"
 	gc "gopkg.in/check.v1"
-	"gopkg.in/juju/charm.v4"
+	"gopkg.in/juju/charm.v5"
 
 	"github.com/juju/juju/instance"
-	"github.com/juju/juju/mongo"
 	"github.com/juju/juju/state"
 	statetesting "github.com/juju/juju/state/testing"
+	"github.com/juju/juju/storage/provider"
+	"github.com/juju/juju/storage/provider/registry"
 	"github.com/juju/juju/testing"
 	"github.com/juju/juju/testing/factory"
 )
 
 type factorySuite struct {
-	testing.BaseSuite
-	jtesting.MgoSuite
-	State   *state.State
+	statetesting.StateSuite
 	Factory *factory.Factory
 }
 
 var _ = gc.Suite(&factorySuite{})
 
-func (s *factorySuite) SetUpSuite(c *gc.C) {
-	s.BaseSuite.SetUpSuite(c)
-	s.MgoSuite.SetUpSuite(c)
-}
-
-func (s *factorySuite) TearDownSuite(c *gc.C) {
-	s.MgoSuite.TearDownSuite(c)
-	s.BaseSuite.TearDownSuite(c)
-}
-
 func (s *factorySuite) SetUpTest(c *gc.C) {
-	s.BaseSuite.SetUpTest(c)
-	s.MgoSuite.SetUpTest(c)
-	policy := statetesting.MockPolicy{}
-
-	info := &mongo.MongoInfo{
-		Info: mongo.Info{
-			Addrs:  []string{jtesting.MgoServer.Addr()},
-			CACert: testing.CACert,
-		},
-	}
-	opts := mongo.DialOpts{
-		Timeout: testing.LongWait,
-	}
-	cfg := testing.EnvironConfig(c)
-	owner := names.NewLocalUserTag("factory-admin")
-	st, err := state.Initialize(owner, info, cfg, opts, &policy)
-	c.Assert(err, jc.ErrorIsNil)
-	s.State = st
+	s.Policy = new(statetesting.MockPolicy)
+	s.StateSuite.SetUpTest(c)
 	s.Factory = factory.NewFactory(s.State)
 }
 
-func (s *factorySuite) TearDownTest(c *gc.C) {
-	if s.State != nil {
-		s.State.Close()
-	}
-	s.MgoSuite.TearDownTest(c)
-	s.BaseSuite.TearDownTest(c)
-}
-
 func (s *factorySuite) TestMakeUserNil(c *gc.C) {
 	user := s.Factory.MakeUser(c, nil)
 	c.Assert(user.IsDisabled(), jc.IsFalse)
@@ -183,9 +148,11 @@ func (s *factorySuite) TestMakeEnvUserParams(c *gc.C) {
 		Creator:   names.NewUserTag("createdby"),
 		NoEnvUser: true,
 	})
+
 	envUser := s.Factory.MakeEnvUser(c, &factory.EnvUserParams{
-		User:      "foobar",
-		CreatedBy: names.NewUserTag("createdby"),
+		User:        "foobar",
+		CreatedBy:   names.NewUserTag("createdby"),
+		DisplayName: "Foo Bar",
 	})
 
 	saved, err := s.State.EnvironmentUser(envUser.UserTag())
@@ -193,6 +160,7 @@ func (s *factorySuite) TestMakeEnvUserParams(c *gc.C) {
 	c.Assert(saved.EnvironmentTag().Id(), gc.Equals, envUser.EnvironmentTag().Id())
 	c.Assert(saved.UserName(), gc.Equals, "foobar@local")
 	c.Assert(saved.CreatedBy(), gc.Equals, "createdby@local")
+	c.Assert(saved.DisplayName(), gc.Equals, "Foo Bar")
 }
 
 func (s *factorySuite) TestMakeEnvUserInvalidCreatedBy(c *gc.C) {
@@ -212,19 +180,21 @@ func (s *factorySuite) TestMakeEnvUserInvalidCreatedBy(c *gc.C) {
 func (s *factorySuite) TestMakeEnvUserNonLocalUser(c *gc.C) {
 	creator := s.Factory.MakeUser(c, &factory.UserParams{Name: "created-by"})
 	envUser := s.Factory.MakeEnvUser(c, &factory.EnvUserParams{
-		User:      "foobar@ubuntuone",
-		CreatedBy: creator.UserTag(),
+		User:        "foobar@ubuntuone",
+		DisplayName: "Foo Bar",
+		CreatedBy:   creator.UserTag(),
 	})
 
 	saved, err := s.State.EnvironmentUser(envUser.UserTag())
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(saved.EnvironmentTag().Id(), gc.Equals, envUser.EnvironmentTag().Id())
 	c.Assert(saved.UserName(), gc.Equals, "foobar@ubuntuone")
+	c.Assert(saved.DisplayName(), gc.Equals, "Foo Bar")
 	c.Assert(saved.CreatedBy(), gc.Equals, creator.UserTag().Username())
 }
 
 func (s *factorySuite) TestMakeMachineNil(c *gc.C) {
-	machine := s.Factory.MakeMachine(c, nil)
+	machine, password := s.Factory.MakeMachineReturningPassword(c, nil)
 	c.Assert(machine, gc.NotNil)
 
 	saved, err := s.State.Machine(machine.Id())
@@ -236,6 +206,7 @@ func (s *factorySuite) TestMakeMachineNil(c *gc.C) {
 	c.Assert(saved.Tag(), gc.Equals, machine.Tag())
 	c.Assert(saved.Life(), gc.Equals, machine.Life())
 	c.Assert(saved.Jobs(), gc.DeepEquals, machine.Jobs())
+	c.Assert(saved.PasswordValid(password), jc.IsTrue)
 	savedInstanceId, err := saved.InstanceId()
 	c.Assert(err, jc.ErrorIsNil)
 	machineInstanceId, err := machine.InstanceId()
@@ -245,21 +216,29 @@ func (s *factorySuite) TestMakeMachineNil(c *gc.C) {
 }
 
 func (s *factorySuite) TestMakeMachine(c *gc.C) {
+	registry.RegisterEnvironStorageProviders("someprovider", provider.LoopProviderType)
 	series := "quantal"
 	jobs := []state.MachineJob{state.JobManageEnviron}
 	password, err := utils.RandomPassword()
 	c.Assert(err, jc.ErrorIsNil)
 	nonce := "some-nonce"
 	id := instance.Id("some-id")
+	volumes := []state.MachineVolumeParams{{Volume: state.VolumeParams{Size: 1024}}}
+	filesystems := []state.MachineFilesystemParams{{
+		Filesystem: state.FilesystemParams{Pool: "loop", Size: 2048},
+	}}
 
-	machine := s.Factory.MakeMachine(c, &factory.MachineParams{
-		Series:     series,
-		Jobs:       jobs,
-		Password:   password,
-		Nonce:      nonce,
-		InstanceId: id,
+	machine, pwd := s.Factory.MakeMachineReturningPassword(c, &factory.MachineParams{
+		Series:      series,
+		Jobs:        jobs,
+		Password:    password,
+		Nonce:       nonce,
+		InstanceId:  id,
+		Volumes:     volumes,
+		Filesystems: filesystems,
 	})
 	c.Assert(machine, gc.NotNil)
+	c.Assert(pwd, gc.Equals, password)
 
 	c.Assert(machine.Series(), gc.Equals, series)
 	c.Assert(machine.Jobs(), gc.DeepEquals, jobs)
@@ -269,6 +248,30 @@ func (s *factorySuite) TestMakeMachine(c *gc.C) {
 	c.Assert(machine.CheckProvisioned(nonce), jc.IsTrue)
 	c.Assert(machine.PasswordValid(password), jc.IsTrue)
 
+	assertVolume := func(name string, size uint64) {
+		volume, err := s.State.Volume(names.NewVolumeTag(name))
+		c.Assert(err, jc.ErrorIsNil)
+		volParams, ok := volume.Params()
+		c.Assert(ok, jc.IsTrue)
+		c.Assert(volParams, jc.DeepEquals, state.VolumeParams{Pool: "loop", Size: size})
+		volAttachments, err := s.State.VolumeAttachments(volume.VolumeTag())
+		c.Assert(err, jc.ErrorIsNil)
+		c.Assert(volAttachments, gc.HasLen, 1)
+		c.Assert(volAttachments[0].Machine(), gc.Equals, machine.Tag())
+	}
+	assertVolume(machine.Id()+"/0", 2048) // backing the filesystem
+	assertVolume(machine.Id()+"/1", 1024)
+
+	filesystem, err := s.State.Filesystem(names.NewFilesystemTag(machine.Id() + "/0"))
+	c.Assert(err, jc.ErrorIsNil)
+	fsParams, ok := filesystem.Params()
+	c.Assert(ok, jc.IsTrue)
+	c.Assert(fsParams, jc.DeepEquals, state.FilesystemParams{Pool: "loop", Size: 2048})
+	fsAttachments, err := s.State.MachineFilesystemAttachments(machine.Tag().(names.MachineTag))
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(fsAttachments, gc.HasLen, 1)
+	c.Assert(fsAttachments[0].Machine(), gc.Equals, machine.Tag())
+
 	saved, err := s.State.Machine(machine.Id())
 	c.Assert(err, jc.ErrorIsNil)
 
@@ -476,7 +479,7 @@ func (s *factorySuite) TestMakeMetric(c *gc.C) {
 		Unit:    unit,
 		Time:    &now,
 		Sent:    true,
-		Metrics: []state.Metric{state.Metric{"pings", "1", now}},
+		Metrics: []state.Metric{{"pings", "1", now}},
 	})
 	c.Assert(metric, gc.NotNil)
 
@@ -500,7 +503,8 @@ func (s *factorySuite) TestMakeEnvironmentNil(c *gc.C) {
 
 	env, err := st.Environment()
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(env.Name(), gc.Equals, "testenv-1")
+	re := regexp.MustCompile(`^testenv-\d+$`)
+	c.Assert(re.MatchString(env.Name()), jc.IsTrue)
 	c.Assert(env.UUID() == s.State.EnvironUUID(), jc.IsFalse)
 	origEnv, err := s.State.Environment()
 	c.Assert(err, jc.ErrorIsNil)
