@@ -13,9 +13,9 @@ import (
 	"path/filepath"
 	"sync"
 
-	cryptossh "code.google.com/p/go.crypto/ssh"
 	"github.com/juju/testing"
 	jc "github.com/juju/testing/checkers"
+	cryptossh "golang.org/x/crypto/ssh"
 	gc "gopkg.in/check.v1"
 
 	"github.com/juju/juju/utils/ssh"
