@@ -4,6 +4,7 @@
 package version
 
 import (
+	"fmt"
 	"io/ioutil"
 	"strconv"
 	"strings"
@@ -82,7 +83,8 @@ func readSeries() (string, error) {
 	case strings.ToLower(Ubuntu.String()):
 		return getValue(ubuntuSeries, values["VERSION_ID"])
 	case strings.ToLower(CentOS.String()):
-		return getValue(centosSeries, values["VERSION_ID"])
+		codename := fmt.Sprintf("%s%s", values["ID"], values["VERSION_ID"])
+		return getValue(centosSeries, codename)
 	default:
 		return "unknown", nil
 	}
