@@ -6,26 +6,22 @@ package deployer
 import (
 	"fmt"
 	"os"
-	"path"
 	"path/filepath"
 	"regexp"
-	"strings"
 
+	"github.com/juju/errors"
 	"github.com/juju/names"
+	"github.com/juju/utils/shell"
 
 	"github.com/juju/juju/agent"
 	"github.com/juju/juju/agent/tools"
 	"github.com/juju/juju/apiserver/params"
-	jujunames "github.com/juju/juju/juju/names"
-	"github.com/juju/juju/juju/osenv"
 	"github.com/juju/juju/service"
 	"github.com/juju/juju/service/common"
 	"github.com/juju/juju/version"
 )
 
-// InitDir is the default upstart init directory.
-// This is a var so it can be overridden by tests.
-var InitDir = "/etc/init"
+// TODO(ericsnow) Use errors.Trace, etc. in this file.
 
 // APICalls defines the interface to the API that the simple context needs.
 type APICalls interface {
@@ -43,9 +39,11 @@ type SimpleContext struct {
 	// running the deployer.
 	agentConfig agent.Config
 
-	// initDir specifies the directory used by upstart on the local system.
-	// It is typically set to "/etc/init".
-	initDir string
+	// discoverService is a surrogate for service.DiscoverService.
+	discoverService func(string, common.Conf) (deployerService, error)
+
+	// listServices is a surrogate for service.ListServices.
+	listServices func() ([]string, error)
 }
 
 var _ Context = (*SimpleContext)(nil)
@@ -75,7 +73,12 @@ func NewSimpleContext(agentConfig agent.Config, api APICalls) *SimpleContext {
 	return &SimpleContext{
 		api:         api,
 		agentConfig: agentConfig,
-		initDir:     InitDir,
+		discoverService: func(name string, conf common.Conf) (deployerService, error) {
+			return service.DiscoverService(name, conf)
+		},
+		listServices: func() ([]string, error) {
+			return service.ListServices()
+		},
 	}
 }
 
@@ -85,8 +88,19 @@ func (ctx *SimpleContext) AgentConfig() agent.Config {
 
 func (ctx *SimpleContext) DeployUnit(unitName, initialPassword string) (err error) {
 	// Check sanity.
-	svc := ctx.service(unitName)
-	if svc.Installed() {
+	renderer, err := shell.NewRenderer("")
+	if err != nil {
+		return errors.Trace(err)
+	}
+	svc, err := ctx.service(unitName, renderer)
+	if err != nil {
+		return errors.Trace(err)
+	}
+	installed, err := svc.Installed()
+	if err != nil {
+		return errors.Trace(err)
+	}
+	if installed {
 		return fmt.Errorf("unit %q is already deployed", unitName)
 	}
 
@@ -116,6 +130,7 @@ func (ctx *SimpleContext) DeployUnit(unitName, initialPassword string) (err erro
 			Tag:               tag,
 			Password:          initialPassword,
 			Nonce:             "unused",
+			Environment:       ctx.agentConfig.Environment(),
 			// TODO: remove the state addresses here and test when api only.
 			StateAddresses: result.StateAddresses,
 			APIAddresses:   result.APIAddresses,
@@ -133,55 +148,52 @@ func (ctx *SimpleContext) DeployUnit(unitName, initialPassword string) (err erro
 	}
 	defer removeOnErr(&err, conf.Dir())
 
-	// Install an upstart job that runs the unit agent.
-	logPath := path.Join(logDir, tag.String()+".log")
-	cmd := strings.Join([]string{
-		filepath.FromSlash(path.Join(toolsDir, jujunames.Jujud)), "unit",
-		"--data-dir", dataDir,
-		"--unit-name", unitName,
-		"--debug", // TODO: propagate debug state sensibly
-	}, " ")
-	// TODO(thumper): 2013-09-02 bug 1219630
-	// As much as I'd like to remove JujuContainerType now, it is still
-	// needed as MAAS still needs it at this stage, and we can't fix
-	// everything at once.
-	envVars := map[string]string{
-		osenv.JujuContainerTypeEnvKey: containerType,
-	}
-	osenv.MergeEnvironment(envVars, osenv.FeatureFlags())
-	sconf := common.Conf{
-		Desc:    "juju unit agent for " + unitName,
-		Cmd:     cmd,
-		Out:     logPath,
-		Env:     envVars,
-		InitDir: ctx.initDir,
-	}
-	svc.UpdateConfig(sconf)
-	return svc.Install()
-}
-
-// findUpstartJob tries to find an upstart job matching the
-// given unit name in one of these formats:
-//   jujud-<deployer-tag>:<unit-tag>.conf (for compatibility)
-//   jujud-<unit-tag>.conf (default)
-func (ctx *SimpleContext) findUpstartJob(unitName string) service.Service {
-	unitsAndJobs, err := ctx.deployedUnitsUpstartJobs()
-	if err != nil {
-		return nil
-	}
-	if job, ok := unitsAndJobs[unitName]; ok {
-		svc := service.NewService(job, common.Conf{InitDir: ctx.initDir})
-		return svc
+	// Install an init service that runs the unit agent.
+	if err := service.InstallAndStart(svc); err != nil {
+		return errors.Trace(err)
 	}
 	return nil
 }
 
-func (ctx *SimpleContext) RecallUnit(unitName string) error {
-	svc := ctx.findUpstartJob(unitName)
-	if svc == nil || !svc.Installed() {
-		return fmt.Errorf("unit %q is not deployed", unitName)
+type deployerService interface {
+	Installed() (bool, error)
+	Install() error
+	Remove() error
+	Start() error
+	Stop() error
+}
+
+// findUpstartJob tries to find an init system job matching the
+// given unit name in one of these formats:
+//   jujud-<deployer-tag>:<unit-tag>.conf (for compatibility)
+//   jujud-<unit-tag>.conf (default)
+func (ctx *SimpleContext) findInitSystemJob(unitName string) (deployerService, error) {
+	unitsAndJobs, err := ctx.deployedUnitsInitSystemJobs()
+	if err != nil {
+		return nil, errors.Trace(err)
 	}
-	if err := svc.StopAndRemove(); err != nil {
+	if job, ok := unitsAndJobs[unitName]; ok {
+		return ctx.discoverService(job, common.Conf{})
+	}
+	return nil, errors.Errorf("unit %q is not deployed", unitName)
+}
+
+func (ctx *SimpleContext) RecallUnit(unitName string) error {
+	svc, err := ctx.findInitSystemJob(unitName)
+	if err != nil {
+		return errors.Trace(err)
+	}
+	installed, err := svc.Installed()
+	if err != nil {
+		return errors.Trace(err)
+	}
+	if !installed {
+		return errors.Errorf("unit %q is not deployed", unitName)
+	}
+	if err := svc.Stop(); err != nil {
+		return err
+	}
+	if err := svc.Remove(); err != nil {
 		return err
 	}
 	tag := names.NewUnitTag(unitName)
@@ -189,7 +201,7 @@ func (ctx *SimpleContext) RecallUnit(unitName string) error {
 	agentDir := agent.Dir(dataDir, tag)
 	// Recursivley change mode to 777 on windows to avoid
 	// Operation not permitted errors when deleting the agentDir
-	err := recursiveChmod(agentDir, os.FileMode(0777))
+	err = recursiveChmod(agentDir, os.FileMode(0777))
 	if err != nil {
 		return err
 	}
@@ -203,8 +215,8 @@ func (ctx *SimpleContext) RecallUnit(unitName string) error {
 
 var deployedRe = regexp.MustCompile("^(jujud-.*unit-([a-z0-9-]+)-([0-9]+))$")
 
-func (ctx *SimpleContext) deployedUnitsUpstartJobs() (map[string]string, error) {
-	fis, err := service.ListServices(ctx.initDir)
+func (ctx *SimpleContext) deployedUnitsInitSystemJobs() (map[string]string, error) {
+	fis, err := ctx.listServices()
 	if err != nil {
 		return nil, err
 	}
@@ -225,7 +237,7 @@ func (ctx *SimpleContext) deployedUnitsUpstartJobs() (map[string]string, error)
 }
 
 func (ctx *SimpleContext) DeployedUnits() ([]string, error) {
-	unitsAndJobs, err := ctx.deployedUnitsUpstartJobs()
+	unitsAndJobs, err := ctx.deployedUnitsInitSystemJobs()
 	if err != nil {
 		return nil, err
 	}
@@ -238,16 +250,30 @@ func (ctx *SimpleContext) DeployedUnits() ([]string, error) {
 
 // service returns a service.Service corresponding to the specified
 // unit.
-func (ctx *SimpleContext) service(unitName string) service.Service {
+func (ctx *SimpleContext) service(unitName string, renderer shell.Renderer) (deployerService, error) {
 	tag := names.NewUnitTag(unitName).String()
 	svcName := "jujud-" + tag
-	svc := service.NewService(svcName, common.Conf{InitDir: ctx.initDir})
-	return svc
+
+	info := service.NewAgentInfo(
+		service.AgentKindUnit,
+		unitName,
+		ctx.agentConfig.DataDir(),
+		ctx.agentConfig.LogDir(),
+	)
+
+	// TODO(thumper): 2013-09-02 bug 1219630
+	// As much as I'd like to remove JujuContainerType now, it is still
+	// needed as MAAS still needs it at this stage, and we can't fix
+	// everything at once.
+	containerType := ctx.agentConfig.Value(agent.ContainerType)
+
+	conf := service.ContainerAgentConf(info, renderer, containerType)
+	return ctx.discoverService(svcName, conf)
 }
 
 func removeOnErr(err *error, path string) {
 	if *err != nil {
-		if err := os.Remove(path); err != nil {
+		if err := os.RemoveAll(path); err != nil {
 			logger.Warningf("installer: cannot remove %q: %v", path, err)
 		}
 	}
