@@ -5,11 +5,11 @@ package deployer_test
 
 import (
 	"encoding/json"
-	"fmt"
 	"io/ioutil"
 	"os"
 	"path/filepath"
 	"regexp"
+	"runtime"
 	"sort"
 
 	"github.com/juju/names"
@@ -18,6 +18,9 @@ import (
 
 	"github.com/juju/juju/agent"
 	"github.com/juju/juju/agent/tools"
+	"github.com/juju/juju/service/common"
+	svctesting "github.com/juju/juju/service/common/testing"
+	"github.com/juju/juju/service/upstart"
 	"github.com/juju/juju/state/multiwatcher"
 	"github.com/juju/juju/testing"
 	coretools "github.com/juju/juju/tools"
@@ -25,6 +28,16 @@ import (
 	"github.com/juju/juju/worker/deployer"
 )
 
+var quote, cmdSuffix string
+
+func init() {
+	quote = "'"
+	if runtime.GOOS == "windows" {
+		cmdSuffix = ".exe"
+		quote = `"`
+	}
+}
+
 type SimpleContextSuite struct {
 	SimpleToolsFixture
 }
@@ -85,9 +98,9 @@ func (s *SimpleContextSuite) TestOldDeployedUnitsCanBeRecalled(c *gc.C) {
 
 	// Simulate some previously deployed units with the old
 	// upstart conf filename format (+deployer tags).
-	s.injectUnit(c, "jujud-machine-0:unit-mysql-0.conf", "unit-mysql-0")
+	s.injectUnit(c, "jujud-machine-0:unit-mysql-0", "unit-mysql-0")
 	s.assertUpstartCount(c, 1)
-	s.injectUnit(c, "jujud-unit-wordpress-0:unit-nrpe-0.conf", "unit-nrpe-0")
+	s.injectUnit(c, "jujud-unit-wordpress-0:unit-nrpe-0", "unit-nrpe-0")
 	s.assertUpstartCount(c, 2)
 
 	// Make sure we can discover them.
@@ -133,16 +146,16 @@ func (s *SimpleContextSuite) TestOldDeployedUnitsCanBeRecalled(c *gc.C) {
 type SimpleToolsFixture struct {
 	dataDir  string
 	logDir   string
-	initDir  string
 	origPath string
 	binDir   string
+
+	data *svctesting.FakeServiceData
 }
 
 var fakeJujud = "#!/bin/bash --norc\n# fake-jujud\nexit 0\n"
 
 func (fix *SimpleToolsFixture) SetUp(c *gc.C, dataDir string) {
 	fix.dataDir = dataDir
-	fix.initDir = c.MkDir()
 	fix.logDir = c.MkDir()
 	toolsDir := tools.SharedToolsDir(fix.dataDir, version.Current)
 	err := os.MkdirAll(toolsDir, 0755)
@@ -164,6 +177,8 @@ func (fix *SimpleToolsFixture) SetUp(c *gc.C, dataDir string) {
 	fix.makeBin(c, "started-status", `echo "blah start/running, process 666"`)
 	fix.makeBin(c, "start", "cp $(which started-status) $(which status)")
 	fix.makeBin(c, "stop", "cp $(which stopped-status) $(which status)")
+
+	fix.data = svctesting.NewFakeServiceData()
 }
 
 func (fix *SimpleToolsFixture) TearDown(c *gc.C) {
@@ -177,24 +192,20 @@ func (fix *SimpleToolsFixture) makeBin(c *gc.C, name, script string) {
 }
 
 func (fix *SimpleToolsFixture) assertUpstartCount(c *gc.C, count int) {
-	fis, err := ioutil.ReadDir(fix.initDir)
-	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(fis, gc.HasLen, count)
+	c.Assert(fix.data.InstalledNames, gc.HasLen, count)
 }
 
 func (fix *SimpleToolsFixture) getContext(c *gc.C) *deployer.SimpleContext {
 	config := agentConfig(names.NewMachineTag("99"), fix.dataDir, fix.logDir)
-	return deployer.NewTestSimpleContext(config, fix.initDir, fix.logDir)
+	return deployer.NewTestSimpleContext(config, fix.logDir, fix.data)
 }
 
 func (fix *SimpleToolsFixture) getContextForMachine(c *gc.C, machineTag names.Tag) *deployer.SimpleContext {
 	config := agentConfig(machineTag, fix.dataDir, fix.logDir)
-	return deployer.NewTestSimpleContext(config, fix.initDir, fix.logDir)
+	return deployer.NewTestSimpleContext(config, fix.logDir, fix.data)
 }
 
-func (fix *SimpleToolsFixture) paths(tag names.Tag) (confPath, agentDir, toolsDir string) {
-	confName := fmt.Sprintf("jujud-%s.conf", tag)
-	confPath = filepath.Join(fix.initDir, confName)
+func (fix *SimpleToolsFixture) paths(tag names.Tag) (agentDir, toolsDir string) {
 	agentDir = agent.Dir(fix.dataDir, tag)
 	toolsDir = tools.ToolsDir(fix.dataDir, tag.String())
 	return
@@ -202,8 +213,19 @@ func (fix *SimpleToolsFixture) paths(tag names.Tag) (confPath, agentDir, toolsDi
 
 func (fix *SimpleToolsFixture) checkUnitInstalled(c *gc.C, name, password string) {
 	tag := names.NewUnitTag(name)
-	uconfPath, _, toolsDir := fix.paths(tag)
-	uconfData, err := ioutil.ReadFile(uconfPath)
+
+	svcName := "jujud-" + tag.String()
+	c.Assert(svcName, jc.Satisfies, fix.data.InstalledNames.Contains)
+
+	var svcConf common.Conf
+	for _, svc := range fix.data.Installed {
+		if svc.Name() == svcName {
+			svcConf = svc.Conf()
+			break
+		}
+	}
+	// TODO(ericsnow) For now we just use upstart serialization.
+	uconfData, err := upstart.Serialize(svcName, svcConf)
 	c.Assert(err, jc.ErrorIsNil)
 	uconf := string(uconfData)
 
@@ -211,16 +233,18 @@ func (fix *SimpleToolsFixture) checkUnitInstalled(c *gc.C, name, password string
 	execs := regex.FindAllString(uconf, -1)
 
 	if nil == execs {
-		c.Fatalf("no command found in %s:\n%s", uconfPath, uconf)
+		c.Fatalf("no command found in conf:\n%s", uconf)
 	} else if 1 > len(execs) {
 		c.Fatalf("Test is not built to handle more than one exec line.")
 	}
 
+	_, toolsDir := fix.paths(tag)
+	jujudPath := filepath.Join(toolsDir, "jujud"+cmdSuffix)
+
 	logPath := filepath.Join(fix.logDir, tag.String()+".log")
-	jujudPath := filepath.Join(toolsDir, "jujud")
 
 	for _, pat := range []string{
-		"^exec " + jujudPath + " unit ",
+		"^exec " + quote + jujudPath + quote + " unit ",
 		" --unit-name " + name + " ",
 		" >> " + logPath + " 2>&1$",
 	} {
@@ -243,8 +267,11 @@ func (fix *SimpleToolsFixture) checkUnitInstalled(c *gc.C, name, password string
 
 func (fix *SimpleToolsFixture) checkUnitRemoved(c *gc.C, name string) {
 	tag := names.NewUnitTag(name)
-	confPath, agentDir, toolsDir := fix.paths(tag)
-	for _, path := range []string{confPath, agentDir, toolsDir} {
+
+	c.Assert(name, gc.Not(jc.Satisfies), fix.data.InstalledNames.Contains)
+
+	agentDir, toolsDir := fix.paths(tag)
+	for _, path := range []string{agentDir, toolsDir} {
 		_, err := ioutil.ReadFile(path)
 		if err == nil {
 			c.Logf("Warning: %q not removed as expected", path)
@@ -254,12 +281,11 @@ func (fix *SimpleToolsFixture) checkUnitRemoved(c *gc.C, name string) {
 	}
 }
 
-func (fix *SimpleToolsFixture) injectUnit(c *gc.C, upstartConf, unitTag string) {
-	confPath := filepath.Join(fix.initDir, upstartConf)
-	err := ioutil.WriteFile(confPath, []byte("#!/bin/bash --norc\necho $0"), 0644)
-	c.Assert(err, jc.ErrorIsNil)
+func (fix *SimpleToolsFixture) injectUnit(c *gc.C, name, unitTag string) {
+	fix.data.SetStatus(name, "installed")
+
 	toolsDir := filepath.Join(fix.dataDir, "tools", unitTag)
-	err = os.MkdirAll(toolsDir, 0755)
+	err := os.MkdirAll(toolsDir, 0755)
 	c.Assert(err, jc.ErrorIsNil)
 }
 
@@ -297,6 +323,10 @@ func (mock *mockConfig) WriteUpgradedToVersion(newVersion version.Number) error
 	return nil
 }
 
+func (mock *mockConfig) Environment() names.EnvironTag {
+	return testing.EnvironmentTag
+}
+
 func (mock *mockConfig) CACert() string {
 	return testing.CACert
 }
