@@ -342,7 +342,7 @@ func (m *testMachine) Id() string {
 	return m.id
 }
 
-func (m *testMachine) Addresses() []network.Address {
+func (m *testMachine) ProviderAddresses() []network.Address {
 	m.mu.Lock()
 	defer m.mu.Unlock()
 	return m.addresses
@@ -356,11 +356,11 @@ func (m *testMachine) InstanceId() (instance.Id, error) {
 }
 
 // This is stubbed out for testing.
-var MachineStatus = func(m *testMachine) (status state.Status, info string, data map[string]interface{}, err error) {
-	return m.status, "", nil, nil
+var MachineStatus = func(m *testMachine) (state.StatusInfo, error) {
+	return state.StatusInfo{m.status, "", nil, nil}, nil
 }
 
-func (m *testMachine) Status() (status state.Status, info string, data map[string]interface{}, err error) {
+func (m *testMachine) Status() (state.StatusInfo, error) {
 	return MachineStatus(m)
 }
 
@@ -381,7 +381,7 @@ func (m *testMachine) SetInstanceStatus(status string) error {
 	return nil
 }
 
-func (m *testMachine) SetAddresses(addrs ...network.Address) error {
+func (m *testMachine) SetProviderAddresses(addrs ...network.Address) error {
 	if m.setAddressesErr != nil {
 		return m.setAddressesErr
 	}
