@@ -13,12 +13,12 @@ import (
 	"sync"
 
 	"github.com/juju/errors"
+	"github.com/juju/replicaset"
 	"github.com/juju/utils/voyeur"
 	"launchpad.net/tomb"
 
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/network"
-	"github.com/juju/juju/replicaset"
 	"github.com/juju/juju/state"
 	"github.com/juju/juju/worker"
 )
