@@ -9,12 +9,12 @@ import (
 	"time"
 
 	"github.com/juju/errors"
+	"github.com/juju/replicaset"
 	"launchpad.net/tomb"
 
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/mongo"
 	"github.com/juju/juju/network"
-	"github.com/juju/juju/replicaset"
 	"github.com/juju/juju/state"
 	"github.com/juju/juju/worker"
 )
