@@ -6,19 +6,23 @@ package provisioner_test
 import (
 	"fmt"
 	"path/filepath"
+	"runtime"
 	"time"
 
 	"github.com/juju/errors"
 	"github.com/juju/names"
+	gitjujutesting "github.com/juju/testing"
 	jc "github.com/juju/testing/checkers"
 	gc "gopkg.in/check.v1"
 
 	"github.com/juju/juju/agent"
+	"github.com/juju/juju/cloudconfig/instancecfg"
 	"github.com/juju/juju/constraints"
 	"github.com/juju/juju/container"
 	"github.com/juju/juju/container/kvm/mock"
 	kvmtesting "github.com/juju/juju/container/kvm/testing"
 	"github.com/juju/juju/environs"
+	"github.com/juju/juju/feature"
 	"github.com/juju/juju/instance"
 	instancetest "github.com/juju/juju/instance/testing"
 	jujutesting "github.com/juju/juju/juju/testing"
@@ -39,11 +43,15 @@ type kvmBrokerSuite struct {
 	kvmSuite
 	broker      environs.InstanceBroker
 	agentConfig agent.Config
+	api         *fakeAPI
 }
 
 var _ = gc.Suite(&kvmBrokerSuite{})
 
 func (s *kvmSuite) SetUpTest(c *gc.C) {
+	if runtime.GOOS == "windows" {
+		c.Skip("Skipping kvm tests on windows")
+	}
 	s.TestSuite.SetUpTest(c)
 	s.events = make(chan mock.Event)
 	s.eventsDone = make(chan struct{})
@@ -63,6 +71,9 @@ func (s *kvmSuite) TearDownTest(c *gc.C) {
 }
 
 func (s *kvmBrokerSuite) SetUpTest(c *gc.C) {
+	if runtime.GOOS == "windows" {
+		c.Skip("Skipping kvm tests on windows")
+	}
 	s.kvmSuite.SetUpTest(c)
 	var err error
 	s.agentConfig, err = agent.NewAgentConfig(
@@ -74,10 +85,12 @@ func (s *kvmBrokerSuite) SetUpTest(c *gc.C) {
 			Nonce:             "nonce",
 			APIAddresses:      []string{"10.0.0.1:1234"},
 			CACert:            coretesting.CACert,
+			Environment:       coretesting.EnvironmentTag,
 		})
 	c.Assert(err, jc.ErrorIsNil)
+	s.api = NewFakeAPI()
 	managerConfig := container.ManagerConfig{container.ConfigName: "juju"}
-	s.broker, err = provisioner.NewKvmBroker(&fakeAPI{}, s.agentConfig, managerConfig)
+	s.broker, err = provisioner.NewKvmBroker(s.api, s.agentConfig, managerConfig, false)
 	c.Assert(err, jc.ErrorIsNil)
 }
 
@@ -85,7 +98,7 @@ func (s *kvmBrokerSuite) startInstance(c *gc.C, machineId string) instance.Insta
 	machineNonce := "fake-nonce"
 	stateInfo := jujutesting.FakeStateInfo(machineId)
 	apiInfo := jujutesting.FakeAPIInfo(machineId)
-	machineConfig, err := environs.NewMachineConfig(machineId, machineNonce, "released", "quantal", true, nil, stateInfo, apiInfo)
+	instanceConfig, err := instancecfg.NewInstanceConfig(machineId, machineNonce, "released", "quantal", true, nil, stateInfo, apiInfo)
 	c.Assert(err, jc.ErrorIsNil)
 	cons := constraints.Value{}
 	possibleTools := coretools.List{&coretools.Tools{
@@ -93,14 +106,83 @@ func (s *kvmBrokerSuite) startInstance(c *gc.C, machineId string) instance.Insta
 		URL:     "http://tools.testing.invalid/2.3.4-quantal-amd64.tgz",
 	}}
 	result, err := s.broker.StartInstance(environs.StartInstanceParams{
-		Constraints:   cons,
-		Tools:         possibleTools,
-		MachineConfig: machineConfig,
+		Constraints:    cons,
+		Tools:          possibleTools,
+		InstanceConfig: instanceConfig,
 	})
 	c.Assert(err, jc.ErrorIsNil)
 	return result.Instance
 }
 
+func (s *kvmBrokerSuite) maintainInstance(c *gc.C, machineId string) {
+	machineNonce := "fake-nonce"
+	stateInfo := jujutesting.FakeStateInfo(machineId)
+	apiInfo := jujutesting.FakeAPIInfo(machineId)
+	instanceConfig, err := instancecfg.NewInstanceConfig(machineId, machineNonce, "released", "quantal", true, nil, stateInfo, apiInfo)
+	c.Assert(err, jc.ErrorIsNil)
+	cons := constraints.Value{}
+	possibleTools := coretools.List{&coretools.Tools{
+		Version: version.MustParseBinary("2.3.4-quantal-amd64"),
+		URL:     "http://tools.testing.invalid/2.3.4-quantal-amd64.tgz",
+	}}
+	err = s.broker.MaintainInstance(environs.StartInstanceParams{
+		Constraints:    cons,
+		Tools:          possibleTools,
+		InstanceConfig: instanceConfig,
+	})
+	c.Assert(err, jc.ErrorIsNil)
+}
+
+func (s *kvmBrokerSuite) TestStartInstance(c *gc.C) {
+	machineId := "1/kvm/0"
+	s.SetFeatureFlags(feature.AddressAllocation)
+	kvm := s.startInstance(c, machineId)
+	s.api.CheckCalls(c, []gitjujutesting.StubCall{{
+		FuncName: "PrepareContainerInterfaceInfo",
+		Args:     []interface{}{names.NewMachineTag("1-kvm-0")},
+	}, {
+		FuncName: "ContainerConfig",
+	}})
+	c.Assert(kvm.Id(), gc.Equals, instance.Id("juju-machine-1-kvm-0"))
+	s.assertInstances(c, kvm)
+}
+
+func (s *kvmBrokerSuite) TestStartInstanceAddressAllocationDisabled(c *gc.C) {
+	machineId := "1/kvm/0"
+	kvm := s.startInstance(c, machineId)
+	s.api.CheckCalls(c, []gitjujutesting.StubCall{{
+		FuncName: "ContainerConfig",
+	}})
+	c.Assert(kvm.Id(), gc.Equals, instance.Id("juju-machine-1-kvm-0"))
+	s.assertInstances(c, kvm)
+}
+
+func (s *kvmBrokerSuite) TestMaintainInstance(c *gc.C) {
+	machineId := "1/kvm/0"
+	s.SetFeatureFlags(feature.AddressAllocation)
+	kvm := s.startInstance(c, machineId)
+	s.api.ResetCalls()
+
+	s.maintainInstance(c, machineId)
+	s.api.CheckCalls(c, []gitjujutesting.StubCall{{
+		FuncName: "GetContainerInterfaceInfo",
+		Args:     []interface{}{names.NewMachineTag("1-kvm-0")},
+	}})
+	c.Assert(kvm.Id(), gc.Equals, instance.Id("juju-machine-1-kvm-0"))
+	s.assertInstances(c, kvm)
+}
+
+func (s *kvmBrokerSuite) TestMaintainInstanceAddressAllocationDisabled(c *gc.C) {
+	machineId := "1/kvm/0"
+	kvm := s.startInstance(c, machineId)
+	s.api.ResetCalls()
+
+	s.maintainInstance(c, machineId)
+	s.api.CheckCalls(c, []gitjujutesting.StubCall{})
+	c.Assert(kvm.Id(), gc.Equals, instance.Id("juju-machine-1-kvm-0"))
+	s.assertInstances(c, kvm)
+}
+
 func (s *kvmBrokerSuite) TestStopInstance(c *gc.C) {
 	kvm0 := s.startInstance(c, "1/kvm/0")
 	kvm1 := s.startInstance(c, "1/kvm/1")
@@ -151,6 +233,9 @@ type kvmProvisionerSuite struct {
 var _ = gc.Suite(&kvmProvisionerSuite{})
 
 func (s *kvmProvisionerSuite) SetUpSuite(c *gc.C) {
+	if runtime.GOOS == "windows" {
+		c.Skip("Skipping kvm tests on windows")
+	}
 	s.CommonProvisionerSuite.SetUpSuite(c)
 	s.kvmSuite.SetUpSuite(c)
 }
@@ -168,13 +253,23 @@ func (s *kvmProvisionerSuite) SetUpTest(c *gc.C) {
 	s.ContainerFactory.AddListener(s.events)
 }
 
+func (s *kvmProvisionerSuite) nextEvent(c *gc.C) mock.Event {
+	select {
+	case event := <-s.events:
+		return event
+	case <-time.After(coretesting.LongWait):
+		c.Fatalf("no event arrived")
+	}
+	panic("not reachable")
+}
+
 func (s *kvmProvisionerSuite) expectStarted(c *gc.C, machine *state.Machine) string {
 	// This check in particular leads to tests just hanging
 	// indefinitely quite often on i386.
 	coretesting.SkipIfI386(c, "lp:1425569")
 
 	s.State.StartSync()
-	event := <-s.events
+	event := s.nextEvent(c)
 	c.Assert(event.Action, gc.Equals, mock.Started)
 	err := machine.Refresh()
 	c.Assert(err, jc.ErrorIsNil)
@@ -188,7 +283,7 @@ func (s *kvmProvisionerSuite) expectStopped(c *gc.C, instId string) {
 	coretesting.SkipIfI386(c, "lp:1425569")
 
 	s.State.StartSync()
-	event := <-s.events
+	event := s.nextEvent(c)
 	c.Assert(event.Action, gc.Equals, mock.Stopped)
 	c.Assert(event.InstanceId, gc.Equals, instId)
 }
@@ -212,7 +307,7 @@ func (s *kvmProvisionerSuite) newKvmProvisioner(c *gc.C) provisioner.Provisioner
 	machineTag := names.NewMachineTag("0")
 	agentConfig := s.AgentConfigForTag(c, machineTag)
 	managerConfig := container.ManagerConfig{container.ConfigName: "juju"}
-	broker, err := provisioner.NewKvmBroker(s.provisioner, agentConfig, managerConfig)
+	broker, err := provisioner.NewKvmBroker(s.provisioner, agentConfig, managerConfig, false)
 	c.Assert(err, jc.ErrorIsNil)
 	toolsFinder := (*provisioner.GetToolsFinder)(s.provisioner)
 	return provisioner.NewContainerProvisioner(instance.KVM, s.provisioner, agentConfig, broker, toolsFinder)
@@ -268,3 +363,9 @@ func (s *kvmProvisionerSuite) TestContainerStartedAndStopped(c *gc.C) {
 	s.expectStopped(c, instId)
 	s.waitRemoved(c, container)
 }
+
+func (s *kvmProvisionerSuite) TestKVMProvisionerObservesConfigChanges(c *gc.C) {
+	p := s.newKvmProvisioner(c)
+	defer stop(c, p)
+	s.assertProvisionerObservesConfigChanges(c, p)
+}
