@@ -5,6 +5,7 @@ package provisioner
 
 import (
 	"fmt"
+	"regexp"
 	"time"
 
 	"github.com/juju/errors"
@@ -16,14 +17,15 @@ import (
 	apiprovisioner "github.com/juju/juju/api/provisioner"
 	apiwatcher "github.com/juju/juju/api/watcher"
 	"github.com/juju/juju/apiserver/params"
+	"github.com/juju/juju/cloudconfig/instancecfg"
 	"github.com/juju/juju/constraints"
 	"github.com/juju/juju/environmentserver/authentication"
 	"github.com/juju/juju/environs"
-	"github.com/juju/juju/environs/cloudinit"
 	"github.com/juju/juju/environs/config"
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/network"
 	"github.com/juju/juju/state/watcher"
+	"github.com/juju/juju/storage"
 	coretools "github.com/juju/juju/tools"
 	"github.com/juju/juju/version"
 	"github.com/juju/juju/worker"
@@ -229,7 +231,7 @@ func (task *provisionerTask) processMachines(ids []string) error {
 	}
 
 	// Find machines without an instance id or that are dead
-	pending, dead, err := task.pendingOrDead(ids)
+	pending, dead, maintain, err := task.pendingOrDeadOrMaintain(ids)
 	if err != nil {
 		return err
 	}
@@ -284,6 +286,9 @@ func (task *provisionerTask) processMachines(ids []string) error {
 		delete(task.machines, machine.Id())
 	}
 
+	// Any machines that require maintenance get pinged
+	task.maintainMachines(maintain)
+
 	// Start an instance for the pending ones
 	return task.startMachines(pending)
 }
@@ -330,46 +335,25 @@ func (task *provisionerTask) populateMachineMaps(ids []string) error {
 
 // pendingOrDead looks up machines with ids and returns those that do not
 // have an instance id assigned yet, and also those that are dead.
-func (task *provisionerTask) pendingOrDead(ids []string) (pending, dead []*apiprovisioner.Machine, err error) {
+func (task *provisionerTask) pendingOrDeadOrMaintain(ids []string) (pending, dead, maintain []*apiprovisioner.Machine, err error) {
 	for _, id := range ids {
 		machine, found := task.machines[id]
 		if !found {
 			logger.Infof("machine %q not found", id)
 			continue
 		}
-		switch machine.Life() {
-		case params.Dying:
-			if _, err := machine.InstanceId(); err == nil {
-				continue
-			} else if !params.IsCodeNotProvisioned(err) {
-				return nil, nil, errors.Annotatef(err, "failed to load machine %q instance id: %v", machine)
-			}
-			logger.Infof("killing dying, unprovisioned machine %q", machine)
-			if err := machine.EnsureDead(); err != nil {
-				return nil, nil, errors.Annotatef(err, "failed to ensure machine dead %q: %v", machine)
-			}
-			fallthrough
-		case params.Dead:
-			dead = append(dead, machine)
-			continue
+		var classification MachineClassification
+		classification, err = classifyMachine(machine)
+		if err != nil {
+			return // return the error
 		}
-		if instId, err := machine.InstanceId(); err != nil {
-			if !params.IsCodeNotProvisioned(err) {
-				logger.Errorf("failed to load machine %q instance id: %v", machine, err)
-				continue
-			}
-			status, _, err := machine.Status()
-			if err != nil {
-				logger.Infof("cannot get machine %q status: %v", machine, err)
-				continue
-			}
-			if status == params.StatusPending {
-				pending = append(pending, machine)
-				logger.Infof("found machine %q pending provisioning", machine)
-				continue
-			}
-		} else {
-			logger.Infof("machine %v already started as instance %q", machine, instId)
+		switch classification {
+		case Pending:
+			pending = append(pending, machine)
+		case Dead:
+			dead = append(dead, machine)
+		case Maintain:
+			maintain = append(maintain, machine)
 		}
 	}
 	logger.Tracef("pending machines: %v", pending)
@@ -377,6 +361,68 @@ func (task *provisionerTask) pendingOrDead(ids []string) (pending, dead []*apipr
 	return
 }
 
+type ClassifiableMachine interface {
+	Life() params.Life
+	InstanceId() (instance.Id, error)
+	EnsureDead() error
+	Status() (params.Status, string, error)
+	Id() string
+}
+
+type MachineClassification string
+
+const (
+	None     MachineClassification = "none"
+	Pending  MachineClassification = "Pending"
+	Dead     MachineClassification = "Dead"
+	Maintain MachineClassification = "Maintain"
+)
+
+func classifyMachine(machine ClassifiableMachine) (
+	MachineClassification, error) {
+	switch machine.Life() {
+	case params.Dying:
+		if _, err := machine.InstanceId(); err == nil {
+			return None, nil
+		} else if !params.IsCodeNotProvisioned(err) {
+			return None, errors.Annotatef(err, "failed to load dying machine id:%s, details:%v", machine.Id(), machine)
+		}
+		logger.Infof("killing dying, unprovisioned machine %q", machine)
+		if err := machine.EnsureDead(); err != nil {
+			return None, errors.Annotatef(err, "failed to ensure machine dead id:%s, details:%v", machine.Id(), machine)
+		}
+		fallthrough
+	case params.Dead:
+		return Dead, nil
+	}
+	if instId, err := machine.InstanceId(); err != nil {
+		if !params.IsCodeNotProvisioned(err) {
+			return None, errors.Annotatef(err, "failed to load machine id:%s, details:%v", machine.Id(), machine)
+		}
+		status, _, err := machine.Status()
+		if err != nil {
+			logger.Infof("cannot get machine id:%s, details:%v, err:%v", machine.Id(), machine, err)
+			return None, nil
+		}
+		if status == params.StatusPending {
+			logger.Infof("found machine pending provisioning id:%s, details:%v", machine.Id(), machine)
+			return Pending, nil
+		}
+	} else {
+		logger.Infof("machine %s already started as instance %q", machine.Id(), instId)
+		if err != nil {
+			logger.Infof("Error fetching provisioning info")
+		} else {
+			isLxc := regexp.MustCompile(`\d+/lxc/\d+`)
+			isKvm := regexp.MustCompile(`\d+/kvm/\d+`)
+			if isLxc.MatchString(machine.Id()) || isKvm.MatchString(machine.Id()) {
+				return Maintain, nil
+			}
+		}
+	}
+	return None, nil
+}
+
 // findUnknownInstances finds instances which are not associated with a machine.
 func (task *provisionerTask) findUnknownInstances(stopping []instance.Instance) ([]instance.Instance, error) {
 	// Make a copy of the instances we know about.
@@ -442,11 +488,11 @@ func (task *provisionerTask) stopInstances(instances []instance.Instance) error
 	return nil
 }
 
-func (task *provisionerTask) constructMachineConfig(
+func (task *provisionerTask) constructInstanceConfig(
 	machine *apiprovisioner.Machine,
 	auth authentication.AuthenticationProvider,
 	pInfo *params.ProvisioningInfo,
-) (*cloudinit.MachineConfig, error) {
+) (*instancecfg.InstanceConfig, error) {
 
 	stateInfo, apiInfo, err := auth.SetupAuthentication(machine)
 	if err != nil {
@@ -462,7 +508,7 @@ func (task *provisionerTask) constructMachineConfig(
 	}
 
 	nonce := fmt.Sprintf("%s:%s", task.machineTag, uuid)
-	return environs.NewMachineConfig(
+	return instancecfg.NewInstanceConfig(
 		machine.Id(),
 		nonce,
 		task.imageStream,
@@ -476,17 +522,67 @@ func (task *provisionerTask) constructMachineConfig(
 
 func constructStartInstanceParams(
 	machine *apiprovisioner.Machine,
-	machineConfig *cloudinit.MachineConfig,
+	instanceConfig *instancecfg.InstanceConfig,
 	provisioningInfo *params.ProvisioningInfo,
 	possibleTools coretools.List,
-) environs.StartInstanceParams {
+) (environs.StartInstanceParams, error) {
+
+	volumes := make([]storage.VolumeParams, len(provisioningInfo.Volumes))
+	for i, v := range provisioningInfo.Volumes {
+		volumeTag, err := names.ParseVolumeTag(v.VolumeTag)
+		if err != nil {
+			return environs.StartInstanceParams{}, errors.Trace(err)
+		}
+		if v.Attachment == nil {
+			return environs.StartInstanceParams{}, errors.Errorf("volume params missing attachment")
+		}
+		machineTag, err := names.ParseMachineTag(v.Attachment.MachineTag)
+		if err != nil {
+			return environs.StartInstanceParams{}, errors.Trace(err)
+		}
+		if machineTag != machine.Tag() {
+			return environs.StartInstanceParams{}, errors.Errorf("volume attachment params has invalid machine tag")
+		}
+		if v.Attachment.InstanceId != "" {
+			return environs.StartInstanceParams{}, errors.Errorf("volume attachment params specifies instance ID")
+		}
+		volumes[i] = storage.VolumeParams{
+			volumeTag,
+			v.Size,
+			storage.ProviderType(v.Provider),
+			v.Attributes,
+			v.Tags,
+			&storage.VolumeAttachmentParams{
+				AttachmentParams: storage.AttachmentParams{
+					Machine:  machineTag,
+					ReadOnly: v.Attachment.ReadOnly,
+				},
+				Volume: volumeTag,
+			},
+		}
+	}
+
 	return environs.StartInstanceParams{
 		Constraints:       provisioningInfo.Constraints,
 		Tools:             possibleTools,
-		MachineConfig:     machineConfig,
+		InstanceConfig:    instanceConfig,
 		Placement:         provisioningInfo.Placement,
 		DistributionGroup: machine.DistributionGroup,
+		Volumes:           volumes,
+	}, nil
+}
+
+func (task *provisionerTask) maintainMachines(machines []*apiprovisioner.Machine) error {
+	for _, m := range machines {
+		logger.Infof("maintainMachines: %v", m)
+		startInstanceParams := environs.StartInstanceParams{}
+		startInstanceParams.InstanceConfig = &instancecfg.InstanceConfig{}
+		startInstanceParams.InstanceConfig.MachineId = m.Id()
+		if err := task.broker.MaintainInstance(startInstanceParams); err != nil {
+			return errors.Annotatef(err, "cannot maintain machine %v", m)
+		}
 	}
+	return nil
 }
 
 func (task *provisionerTask) startMachines(machines []*apiprovisioner.Machine) error {
@@ -497,12 +593,12 @@ func (task *provisionerTask) startMachines(machines []*apiprovisioner.Machine) e
 			return err
 		}
 
-		machineCfg, err := task.constructMachineConfig(m, task.auth, pInfo)
+		instanceCfg, err := task.constructInstanceConfig(m, task.auth, pInfo)
 		if err != nil {
 			return err
 		}
 
-		assocProvInfoAndMachCfg(pInfo, machineCfg)
+		assocProvInfoAndMachCfg(pInfo, instanceCfg)
 
 		possibleTools, err := task.toolsFinder.FindTools(
 			version.Current.Number,
@@ -513,12 +609,15 @@ func (task *provisionerTask) startMachines(machines []*apiprovisioner.Machine) e
 			return task.setErrorStatus("cannot find tools for machine %q: %v", m, err)
 		}
 
-		startInstanceParams := constructStartInstanceParams(
+		startInstanceParams, err := constructStartInstanceParams(
 			m,
-			machineCfg,
+			instanceCfg,
 			pInfo,
 			possibleTools,
 		)
+		if err != nil {
+			return task.setErrorStatus("cannot construct params for machine %q: %v", m, err)
+		}
 
 		if err := task.startMachine(m, pInfo, startInstanceParams); err != nil {
 			return errors.Annotatef(err, "cannot start machine %v", m)
@@ -536,18 +635,21 @@ func (task *provisionerTask) setErrorStatus(message string, machine *apiprovisio
 	return nil
 }
 
-func (task *provisionerTask) prepareNetworkAndInterfaces(networkInfo []network.Info) (
-	networks []params.Network, ifaces []params.NetworkInterface) {
+func (task *provisionerTask) prepareNetworkAndInterfaces(networkInfo []network.InterfaceInfo) (
+	networks []params.Network, ifaces []params.NetworkInterface, err error) {
 	if len(networkInfo) == 0 {
-		return nil, nil
+		return nil, nil, nil
 	}
 	visitedNetworks := set.NewStrings()
 	for _, info := range networkInfo {
+		if !names.IsValidNetwork(info.NetworkName) {
+			return nil, nil, errors.Errorf("invalid network name %q", info.NetworkName)
+		}
 		networkTag := names.NewNetworkTag(info.NetworkName).String()
 		if !visitedNetworks.Contains(networkTag) {
 			networks = append(networks, params.Network{
 				Tag:        networkTag,
-				ProviderId: info.ProviderId,
+				ProviderId: string(info.ProviderId),
 				CIDR:       info.CIDR,
 				VLANTag:    info.VLANTag,
 			})
@@ -561,7 +663,7 @@ func (task *provisionerTask) prepareNetworkAndInterfaces(networkInfo []network.I
 			Disabled:      info.Disabled,
 		})
 	}
-	return networks, ifaces
+	return networks, ifaces, nil
 }
 
 func (task *provisionerTask) startMachine(
@@ -589,14 +691,27 @@ func (task *provisionerTask) startMachine(
 
 	inst := result.Instance
 	hardware := result.Hardware
-	nonce := startInstanceParams.MachineConfig.MachineNonce
-	networks, ifaces := task.prepareNetworkAndInterfaces(result.NetworkInfo)
+	nonce := startInstanceParams.InstanceConfig.MachineNonce
+	networks, ifaces, err := task.prepareNetworkAndInterfaces(result.NetworkInfo)
+	if err != nil {
+		return task.setErrorStatus("cannot prepare network for machine %q: %v", machine, err)
+	}
+	volumes := volumesToApiserver(result.Volumes)
+	volumeAttachments := volumeAttachmentsToApiserver(result.VolumeAttachments)
 
-	err = machine.SetInstanceInfo(inst.Id(), nonce, hardware, networks, ifaces)
+	// TODO(dimitern) In a newer Provisioner API version, change
+	// SetInstanceInfo or add a new method that takes and saves in
+	// state all the information available on a network.InterfaceInfo
+	// for each interface, so we can later manage interfaces
+	// dynamically at run-time.
+	err = machine.SetInstanceInfo(inst.Id(), nonce, hardware, networks, ifaces, volumes, volumeAttachments)
 	if err != nil && params.IsCodeNotImplemented(err) {
 		return fmt.Errorf("cannot provision instance %v for machine %q with networks: not implemented", inst.Id(), machine)
 	} else if err == nil {
-		logger.Infof("started machine %s as instance %s with hardware %q, networks %v, interfaces %v", machine, inst.Id(), hardware, networks, ifaces)
+		logger.Infof(
+			"started machine %s as instance %s with hardware %q, networks %v, interfaces %v, volumes %v, volume attachments %v",
+			machine, inst.Id(), hardware, networks, ifaces, volumes, volumeAttachments,
+		)
 		return nil
 	}
 	// We need to stop the instance right away here, set error status and go on.
@@ -609,31 +724,60 @@ func (task *provisionerTask) startMachine(
 }
 
 type provisioningInfo struct {
-	Constraints   constraints.Value
-	Series        string
-	Placement     string
-	MachineConfig *cloudinit.MachineConfig
+	Constraints    constraints.Value
+	Series         string
+	Placement      string
+	InstanceConfig *instancecfg.InstanceConfig
 }
 
 func assocProvInfoAndMachCfg(
 	provInfo *params.ProvisioningInfo,
-	machineConfig *cloudinit.MachineConfig,
+	instanceConfig *instancecfg.InstanceConfig,
 ) *provisioningInfo {
 
-	machineConfig.Networks = provInfo.Networks
+	instanceConfig.Networks = provInfo.Networks
+	instanceConfig.Tags = provInfo.Tags
 
 	if len(provInfo.Jobs) > 0 {
-		machineConfig.Jobs = provInfo.Jobs
+		instanceConfig.Jobs = provInfo.Jobs
 	}
 
 	return &provisioningInfo{
-		Constraints:   provInfo.Constraints,
-		Series:        provInfo.Series,
-		Placement:     provInfo.Placement,
-		MachineConfig: machineConfig,
+		Constraints:    provInfo.Constraints,
+		Series:         provInfo.Series,
+		Placement:      provInfo.Placement,
+		InstanceConfig: instanceConfig,
 	}
 }
 
+func volumesToApiserver(volumes []storage.Volume) []params.Volume {
+	result := make([]params.Volume, len(volumes))
+	for i, v := range volumes {
+		result[i] = params.Volume{
+			v.Tag.String(),
+			params.VolumeInfo{
+				v.VolumeId,
+				v.HardwareId,
+				v.Size,
+				v.Persistent,
+			},
+		}
+	}
+	return result
+}
+
+func volumeAttachmentsToApiserver(attachments []storage.VolumeAttachment) map[string]params.VolumeAttachmentInfo {
+	result := make(map[string]params.VolumeAttachmentInfo)
+	for _, a := range attachments {
+		result[a.Volume.String()] = params.VolumeAttachmentInfo{
+			a.DeviceName,
+			a.BusAddress,
+			a.ReadOnly,
+		}
+	}
+	return result
+}
+
 // ProvisioningInfo is new in 1.20; wait for the API server to be
 // upgraded so we don't spew errors on upgrade.
 func (task *provisionerTask) blockUntilProvisioned(
