@@ -11,6 +11,7 @@ import (
 	"net"
 	"os"
 	"os/user"
+	"path/filepath"
 	"runtime"
 	"strconv"
 	"time"
@@ -78,15 +79,27 @@ type certPair struct {
 
 var _ worker.NotifyWatchHandler = (*RsyslogConfigHandler)(nil)
 
+func syslogUser() string {
+	var user string
+	switch version.Current.OS {
+	case version.CentOS:
+		user = "root"
+	default:
+		user = "syslog"
+	}
+
+	return user
+}
+
 // NewRsyslogConfigWorker returns a worker.Worker that uses
 // WatchForRsyslogChanges and updates rsyslog configuration based
 // on changes. The worker will remove the configuration file
 // on teardown.
-func NewRsyslogConfigWorker(st *apirsyslog.State, mode RsyslogMode, tag names.Tag, namespace string, stateServerAddrs []string) (worker.Worker, error) {
+func NewRsyslogConfigWorker(st *apirsyslog.State, mode RsyslogMode, tag names.Tag, namespace string, stateServerAddrs []string, jujuConfigDir string) (worker.Worker, error) {
 	if version.Current.OS == version.Windows && mode == RsyslogModeAccumulate {
 		return worker.NewNoOpWorker(), nil
 	}
-	handler, err := newRsyslogConfigHandler(st, mode, tag, namespace, stateServerAddrs)
+	handler, err := newRsyslogConfigHandler(st, mode, tag, namespace, stateServerAddrs, jujuConfigDir)
 	if err != nil {
 		return nil, err
 	}
@@ -94,12 +107,27 @@ func NewRsyslogConfigWorker(st *apirsyslog.State, mode RsyslogMode, tag names.Ta
 	return worker.NewNotifyWorker(handler), nil
 }
 
-func newRsyslogConfigHandler(st *apirsyslog.State, mode RsyslogMode, tag names.Tag, namespace string, stateServerAddrs []string) (*RsyslogConfigHandler, error) {
-	var syslogConfig *syslog.SyslogConfig
+func newRsyslogConfigHandler(st *apirsyslog.State, mode RsyslogMode, tag names.Tag, namespace string, stateServerAddrs []string, jujuConfigDir string) (*RsyslogConfigHandler, error) {
+	if namespace != "" {
+		jujuConfigDir += "-" + namespace
+	}
+	jujuConfigDir = filepath.Join(jujuConfigDir, "rsyslog")
+	if err := os.MkdirAll(jujuConfigDir, 0755); err != nil {
+		return nil, errors.Trace(err)
+	}
+
+	syslogConfig := &syslog.SyslogConfig{
+		LogFileName:          tag.String(),
+		LogDir:               logDir,
+		JujuConfigDir:        jujuConfigDir,
+		Port:                 0,
+		Namespace:            namespace,
+		StateServerAddresses: stateServerAddrs,
+	}
 	if mode == RsyslogModeAccumulate {
-		syslogConfig = syslog.NewAccumulateConfig(tag.String(), logDir, 0, namespace, stateServerAddrs)
+		syslog.NewAccumulateConfig(syslogConfig)
 	} else {
-		syslogConfig = syslog.NewForwardConfig(tag.String(), logDir, 0, namespace, stateServerAddrs)
+		syslog.NewForwardConfig(syslogConfig)
 	}
 
 	// Historically only machine-0 includes the namespace in the log
@@ -123,6 +151,7 @@ func newRsyslogConfigHandler(st *apirsyslog.State, mode RsyslogMode, tag names.T
 	if namespace != "" {
 		syslogConfig.LogDir += "-" + namespace
 	}
+
 	return &RsyslogConfigHandler{
 		st:           st,
 		mode:         mode,
@@ -133,7 +162,7 @@ func newRsyslogConfigHandler(st *apirsyslog.State, mode RsyslogMode, tag names.T
 
 func (h *RsyslogConfigHandler) SetUp() (watcher.NotifyWatcher, error) {
 	if h.mode == RsyslogModeAccumulate {
-		if err := h.ensureCertificates(); err != nil {
+		if err := h.ensureCA(); err != nil {
 			return nil, errors.Annotate(err, "failed to write rsyslog certificates")
 		}
 
@@ -186,7 +215,11 @@ func (h *RsyslogConfigHandler) replaceRemoteLogger(caCert string) error {
 			host = j
 		}
 		target := fmt.Sprintf("%s:%d", host, h.syslogConfig.Port)
-		logTag := "juju" + h.syslogConfig.Namespace + "-" + h.tag.String()
+		namespace := h.syslogConfig.Namespace
+		if namespace != "" {
+			namespace = "-" + namespace
+		}
+		logTag := "juju" + namespace + "-" + h.tag.String()
 		logger.Debugf("making syslog connection for %q to %s", logTag, target)
 		writer, err := dialSyslog("tcp", target, rsyslog.LOG_DEBUG, logTag, tlsConf)
 		if err != nil {
@@ -215,7 +248,7 @@ func (h *RsyslogConfigHandler) replaceRemoteLogger(caCert string) error {
 	return nil
 }
 
-func (h *RsyslogConfigHandler) Handle() error {
+func (h *RsyslogConfigHandler) Handle(_ <-chan struct{}) error {
 	cfg, err := h.st.GetRsyslogConfig(h.tag.String())
 	if err != nil {
 		return errors.Annotate(err, "cannot get environ config")
@@ -331,49 +364,30 @@ func (h *RsyslogConfigHandler) rsyslogHosts() ([]string, error) {
 	return hosts, nil
 }
 
-// ensureCertificates ensures that a CA certificate,
-// server certificate, and private key exist in the log
-// directory, and writes them if not. The CA certificate
-// is entered into the environment configuration to be
-// picked up by other agents.
-func (h *RsyslogConfigHandler) ensureCertificates() error {
-	// We write ca-cert.pem last, after propagating into state.
-	// If it's there, then there's nothing to do. Otherwise,
-	// start over.
-	caCertPEM := h.syslogConfig.CACertPath()
-	if _, err := os.Stat(caCertPEM); err == nil {
+// ensureCA ensures that a CA certificate and key exist in state,
+// to be picked up by all rsyslog workers in the environment.
+func (h *RsyslogConfigHandler) ensureCA() error {
+	// We never write the CA key to local disk, so
+	// we must check state to know whether or not
+	// we need to generate new certs and keys.
+	cfg, err := h.st.GetRsyslogConfig(h.tag.String())
+	if err != nil {
+		return errors.Annotate(err, "cannot get environ config")
+	}
+	if cfg.CACert != "" && cfg.CAKey != "" {
 		return nil
 	}
 
-	// Generate a new CA and server cert/key pairs.
-	// The CA key will be discarded after the server
-	// cert has been generated.
+	// Generate a new CA and server cert/key pairs, and
+	// publish to state. Rsyslog workers will observe
+	// this and generate certificates and keys for
+	// rsyslog in response.
 	expiry := time.Now().UTC().AddDate(10, 0, 0)
 	caCertPEM, caKeyPEM, err := cert.NewCA("rsyslog", expiry)
 	if err != nil {
 		return err
 	}
-
-	// Update the environment config with the CA cert,
-	// so clients can configure rsyslog.
-	if err := h.st.SetRsyslogCert(caCertPEM, caKeyPEM); err != nil {
-		return err
-	}
-
-	rsyslogCertPEM, rsyslogKeyPEM, err := h.rsyslogServerCerts(caCertPEM, caKeyPEM)
-	if err != nil {
-		return err
-	}
-
-	if err := writeCertificates([]certPair{
-		{h.syslogConfig.ServerCertPath(), rsyslogCertPEM},
-		{h.syslogConfig.ServerKeyPath(), rsyslogKeyPEM},
-		{h.syslogConfig.CACertPath(), caCertPEM},
-	}); err != nil {
-		return errors.Trace(err)
-	}
-
-	return nil
+	return h.st.SetRsyslogCert(caCertPEM, caKeyPEM)
 }
 
 // writeCertificates persists any certPair to disk. If any
@@ -382,7 +396,7 @@ func (h *RsyslogConfigHandler) ensureCertificates() error {
 // a suitable order in the case of failue.
 func writeCertificates(pairs []certPair) error {
 	// Files must be chowned to syslog:adm.
-	syslogUid, syslogGid, err := lookupUser("syslog")
+	syslogUid, syslogGid, err := lookupUser(syslogUser())
 	if err != nil {
 		return err
 	}
@@ -431,7 +445,7 @@ func (h *RsyslogConfigHandler) rsyslogServerCerts(caCert, caKey string) (string,
 // exist in the log directory and creates them if they do not.
 func (h *RsyslogConfigHandler) ensureLogrotate() error {
 	// Files must be chowned to syslog
-	syslogUid, syslogGid, err := lookupUser("syslog")
+	syslogUid, syslogGid, err := lookupUser(syslogUser())
 	if err != nil {
 		return err
 	}
