@@ -4,13 +4,11 @@
 package filter
 
 import (
-	"sort"
-
 	"github.com/juju/errors"
 	"github.com/juju/loggo"
 	"github.com/juju/names"
-	"gopkg.in/juju/charm.v4"
-	"gopkg.in/juju/charm.v4/hooks"
+	"github.com/juju/utils/set"
+	"gopkg.in/juju/charm.v5"
 	"launchpad.net/tomb"
 
 	"github.com/juju/juju/api/uniter"
@@ -18,7 +16,6 @@ import (
 	"github.com/juju/juju/apiserver/params"
 	"github.com/juju/juju/state/watcher"
 	"github.com/juju/juju/worker"
-	"github.com/juju/juju/worker/uniter/hook"
 )
 
 var filterLogger = loggo.GetLogger("juju.worker.uniter.filter")
@@ -37,27 +34,36 @@ type filter struct {
 	// The out* chans, when set to the corresponding out*On chan (rather than
 	// nil) indicate that an event of the appropriate type is ready to send
 	// to the client.
-	outConfig        chan struct{}
-	outConfigOn      chan struct{}
-	outAction        chan *hook.Info
-	outActionOn      chan *hook.Info
-	outUpgrade       chan *charm.URL
-	outUpgradeOn     chan *charm.URL
-	outResolved      chan params.ResolvedMode
-	outResolvedOn    chan params.ResolvedMode
-	outRelations     chan []int
-	outRelationsOn   chan []int
-	outMeterStatus   chan struct{}
-	outMeterStatusOn chan struct{}
+	outConfig           chan struct{}
+	outConfigOn         chan struct{}
+	outAction           chan string
+	outActionOn         chan string
+	outLeaderSettings   chan struct{}
+	outLeaderSettingsOn chan struct{}
+	outUpgrade          chan *charm.URL
+	outUpgradeOn        chan *charm.URL
+	outResolved         chan params.ResolvedMode
+	outResolvedOn       chan params.ResolvedMode
+	outRelations        chan []int
+	outRelationsOn      chan []int
+	outMeterStatus      chan struct{}
+	outMeterStatusOn    chan struct{}
+	outStorage          chan []names.StorageTag
+	outStorageOn        chan []names.StorageTag
 	// The want* chans are used to indicate that the filter should send
 	// events if it has them available.
-	wantForcedUpgrade chan bool
-	wantResolved      chan struct{}
+	wantForcedUpgrade  chan bool
+	wantResolved       chan struct{}
+	wantLeaderSettings chan bool
 
 	// discardConfig is used to indicate that any pending config event
 	// should be discarded.
 	discardConfig chan struct{}
 
+	// discardLeaderSettings is used to indicate any pending Leader
+	// Settings event should be discarded.
+	discardLeaderSettings chan struct{}
+
 	// setCharm is used to request that the unit's charm URL be set to
 	// a new value. This must be done in the filter's goroutine, so
 	// that config watches can be stopped and restarted pointing to
@@ -92,8 +98,9 @@ type filter struct {
 	upgradeAvailable serviceCharm
 	upgrade          *charm.URL
 	relations        []int
+	storage          []names.StorageTag
 	actionsPending   []string
-	nextAction       *hook.Info
+	nextAction       string
 
 	// meterStatusCode and meterStatusInfo reflect the meter status values of the unit.
 	meterStatusCode string
@@ -104,27 +111,25 @@ type filter struct {
 // supplied unit.
 func NewFilter(st *uniter.State, unitTag names.UnitTag) (Filter, error) {
 	f := &filter{
-		st:                st,
-		outUnitDying:      make(chan struct{}),
-		outConfig:         make(chan struct{}),
-		outConfigOn:       make(chan struct{}),
-		outAction:         make(chan *hook.Info),
-		outActionOn:       make(chan *hook.Info),
-		outUpgrade:        make(chan *charm.URL),
-		outUpgradeOn:      make(chan *charm.URL),
-		outResolved:       make(chan params.ResolvedMode),
-		outResolvedOn:     make(chan params.ResolvedMode),
-		outRelations:      make(chan []int),
-		outRelationsOn:    make(chan []int),
-		outMeterStatus:    make(chan struct{}),
-		outMeterStatusOn:  make(chan struct{}),
-		wantForcedUpgrade: make(chan bool),
-		wantResolved:      make(chan struct{}),
-		discardConfig:     make(chan struct{}),
-		setCharm:          make(chan *charm.URL),
-		didSetCharm:       make(chan struct{}),
-		clearResolved:     make(chan struct{}),
-		didClearResolved:  make(chan struct{}),
+		st:                    st,
+		outUnitDying:          make(chan struct{}),
+		outConfigOn:           make(chan struct{}),
+		outActionOn:           make(chan string),
+		outLeaderSettingsOn:   make(chan struct{}),
+		outUpgradeOn:          make(chan *charm.URL),
+		outResolvedOn:         make(chan params.ResolvedMode),
+		outRelationsOn:        make(chan []int),
+		outMeterStatusOn:      make(chan struct{}),
+		outStorageOn:          make(chan []names.StorageTag),
+		wantForcedUpgrade:     make(chan bool),
+		wantResolved:          make(chan struct{}),
+		wantLeaderSettings:    make(chan bool),
+		discardConfig:         make(chan struct{}),
+		discardLeaderSettings: make(chan struct{}),
+		setCharm:              make(chan *charm.URL),
+		didSetCharm:           make(chan struct{}),
+		clearResolved:         make(chan struct{}),
+		didClearResolved:      make(chan struct{}),
 	}
 	go func() {
 		defer f.tomb.Done()
@@ -182,7 +187,7 @@ func (f *filter) ConfigEvents() <-chan struct{} {
 
 // ActionEvents returns a channel that will receive a signal whenever the unit
 // receives new Actions.
-func (f *filter) ActionEvents() <-chan *hook.Info {
+func (f *filter) ActionEvents() <-chan string {
 	return f.outActionOn
 }
 
@@ -192,6 +197,12 @@ func (f *filter) RelationsEvents() <-chan []int {
 	return f.outRelationsOn
 }
 
+// StorageEvents returns a channel that will receive the tags of all the unit's
+// associated storage instances.
+func (f *filter) StorageEvents() <-chan []names.StorageTag {
+	return f.outStorageOn
+}
+
 // WantUpgradeEvent controls whether the filter will generate upgrade
 // events for unforced service charm changes.
 func (f *filter) WantUpgradeEvent(mustForce bool) {
@@ -255,6 +266,40 @@ func (f *filter) ClearResolved() error {
 	}
 }
 
+// LeaderSettingsEvents returns a channel that will receive an event whenever
+// there is a leader settings change. Events can be temporarily suspended by
+// calling WantLeaderSettingsEvents(false), and then reenabled by calling
+// WantLeaderSettingsEvents(true)
+func (f *filter) LeaderSettingsEvents() <-chan struct{} {
+	return f.outLeaderSettingsOn
+}
+
+// DiscardLeaderSettingsEvent can be called to discard any pending
+// LeaderSettingsEvents. This is used by code that saw a LeaderSettings change,
+// and has been prepping for a response. Just before they request the current
+// LeaderSettings, they can discard any other pending changes, since they know
+// they will be handling all changes that have occurred before right now.
+func (f *filter) DiscardLeaderSettingsEvent() {
+	select {
+	case <-f.tomb.Dying():
+	case f.discardLeaderSettings <- nothing:
+	}
+}
+
+// WantLeaderSettingsEvents can be used to enable/disable events being sent on
+// the LeaderSettingsEvents() channel. This is used when an agent notices that
+// it is the leader, it wants to disable getting events for changes that it is
+// generating. Calling this with sendEvents=false disables getting change
+// events. Calling this with sendEvents=true will enable future changes, and
+// queues up an immediate event so that the agent will refresh its information
+// for any events it might have missed while it thought it was the leader.
+func (f *filter) WantLeaderSettingsEvents(sendEvents bool) {
+	select {
+	case <-f.tomb.Dying():
+	case f.wantLeaderSettings <- sendEvents:
+	}
+}
+
 // DiscardConfigEvent indicates that the filter should discard any pending
 // config event.
 func (f *filter) DiscardConfigEvent() {
@@ -341,6 +386,22 @@ func (f *filter) loop(unitTag names.UnitTag) (err error) {
 		return err
 	}
 	defer watcher.Stop(addressesw, &f.tomb)
+	storagew, err := f.unit.WatchStorage()
+	if err != nil {
+		return err
+	}
+	defer watcher.Stop(storagew, &f.tomb)
+	leaderSettingsw, err := f.st.LeadershipSettings.WatchLeadershipSettings(f.service.Tag().Id())
+	if err != nil {
+		return err
+	}
+	defer watcher.Stop(leaderSettingsw, &f.tomb)
+
+	// Ignore external requests for leader settings behaviour until we see the first change.
+	var discardLeaderSettings <-chan struct{}
+	var wantLeaderSettings <-chan bool
+	// By default we send all leaderSettings onwards.
+	sendLeaderSettings := true
 
 	// Config events cannot be meaningfully discarded until one is available;
 	// once we receive the initial config and address changes, we unblock
@@ -433,6 +494,31 @@ func (f *filter) loop(unitTag names.UnitTag) (err error) {
 				}
 			}
 			f.relationsChanged(ids)
+		case ids, ok := <-storagew.Changes():
+			filterLogger.Debugf("got storage change")
+			if !ok {
+				return watcher.EnsureErr(storagew)
+			}
+			tags := make([]names.StorageTag, len(ids))
+			for i, id := range ids {
+				tag := names.NewStorageTag(id)
+				tags[i] = tag
+			}
+			f.storageChanged(tags)
+		case _, ok = <-leaderSettingsw.Changes():
+			filterLogger.Debugf("got leader settings change: ok=%t", ok)
+			if !ok {
+				return watcher.EnsureErr(leaderSettingsw)
+			}
+			if sendLeaderSettings {
+				// only send the leader settings changed event
+				// if it hasn't been explicitly disabled
+				f.outLeaderSettings = f.outLeaderSettingsOn
+			} else {
+				filterLogger.Debugf("not sending leader settings change (want=false)")
+			}
+			discardLeaderSettings = f.discardLeaderSettings
+			wantLeaderSettings = f.wantLeaderSettings
 
 		// Send events on active out chans.
 		case f.outUpgrade <- f.upgrade:
@@ -444,6 +530,9 @@ func (f *filter) loop(unitTag names.UnitTag) (err error) {
 		case f.outConfig <- nothing:
 			filterLogger.Debugf("sent config event")
 			f.outConfig = nil
+		case f.outLeaderSettings <- nothing:
+			filterLogger.Debugf("sent leader settings event")
+			f.outLeaderSettings = nil
 		case f.outAction <- f.nextAction:
 			f.nextAction = f.getNextAction()
 			filterLogger.Debugf("sent action event")
@@ -454,6 +543,10 @@ func (f *filter) loop(unitTag names.UnitTag) (err error) {
 		case f.outMeterStatus <- nothing:
 			filterLogger.Debugf("sent meter status change event")
 			f.outMeterStatus = nil
+		case f.outStorage <- f.storage:
+			filterLogger.Debugf("sent storage event")
+			f.outStorage = nil
+			f.storage = nil
 
 		// Handle explicit requests.
 		case curl := <-f.setCharm:
@@ -505,6 +598,17 @@ func (f *filter) loop(unitTag names.UnitTag) (err error) {
 			if f.resolved != params.ResolvedNone {
 				f.outResolved = f.outResolvedOn
 			}
+		case sendEvents := <-wantLeaderSettings:
+			filterLogger.Debugf("want leader settings event: %t", sendEvents)
+			sendLeaderSettings = sendEvents
+			if sendEvents {
+				// go ahead and send an event right now,
+				// they're waiting for us
+				f.outLeaderSettings = f.outLeaderSettingsOn
+			} else {
+				// Make sure we don't have a pending event
+				f.outLeaderSettings = nil
+			}
 		case <-f.clearResolved:
 			filterLogger.Debugf("resolved event handled")
 			f.outResolved = nil
@@ -522,6 +626,9 @@ func (f *filter) loop(unitTag names.UnitTag) (err error) {
 		case <-discardConfig:
 			filterLogger.Debugf("discarded config event")
 			f.outConfig = nil
+		case <-discardLeaderSettings:
+			filterLogger.Debugf("discarded leader settings event")
+			f.outLeaderSettings = nil
 		}
 	}
 }
@@ -623,38 +730,49 @@ func (f *filter) upgradeChanged() (err error) {
 }
 
 // relationsChanged responds to service relation changes.
-func (f *filter) relationsChanged(ids []int) {
-outer:
-	for _, id := range ids {
-		for _, existing := range f.relations {
-			if id == existing {
-				continue outer
-			}
-		}
-		f.relations = append(f.relations, id)
+func (f *filter) relationsChanged(changed []int) {
+	ids := set.NewInts(f.relations...)
+	for _, id := range changed {
+		ids.Add(id)
 	}
-	if len(f.relations) != 0 {
-		sort.Ints(f.relations)
+	if len(f.relations) != len(ids) {
+		f.relations = ids.SortedValues()
 		f.outRelations = f.outRelationsOn
 	}
 }
 
-func (f *filter) getNextAction() *hook.Info {
-	if len(f.actionsPending) > 0 {
-		nextAction := hook.Info{
-			Kind:     hooks.Action,
-			ActionId: f.actionsPending[0],
+// storageChanged responds to unit storage changes.
+func (f *filter) storageChanged(changed []names.StorageTag) {
+	tags := set.NewTags() // f.storage is []StorageTag, not []Tag
+	for _, tag := range f.storage {
+		tags.Add(tag)
+	}
+	for _, tag := range changed {
+		tags.Add(tag)
+	}
+	if len(f.storage) != len(tags) {
+		storage := make([]names.StorageTag, len(tags))
+		for i, tag := range tags.SortedValues() {
+			storage[i] = tag.(names.StorageTag)
 		}
+		f.storage = storage
+		f.outStorage = f.outStorageOn
+	}
+}
+
+func (f *filter) getNextAction() string {
+	if len(f.actionsPending) > 0 {
+		actionId := f.actionsPending[0]
 
 		f.outAction = f.outActionOn
 		f.actionsPending = f.actionsPending[1:]
 
-		return &nextAction
+		return actionId
 	} else {
 		f.outAction = nil
 	}
 
-	return nil
+	return ""
 }
 
 // serviceCharm holds information about a charm.
