@@ -8,7 +8,14 @@ import (
 	"fmt"
 
 	"github.com/juju/names"
-	"gopkg.in/juju/charm.v4/hooks"
+	"gopkg.in/juju/charm.v5/hooks"
+)
+
+// TODO(fwereade): move these definitions to juju/charm/hooks.
+const (
+	LeaderElected         hooks.Kind = "leader-elected"
+	LeaderDeposed         hooks.Kind = "leader-deposed"
+	LeaderSettingsChanged hooks.Kind = "leader-settings-changed"
 )
 
 // Info holds details required to execute a hook. Not all fields are
@@ -28,9 +35,8 @@ type Info struct {
 	// associated with RemoteUnit. It is only set when RemoteUnit is set.
 	ChangeVersion int64 `yaml:"change-version,omitempty"`
 
-	// ActionId is the state State.actions ID of the Action document to
-	// be retrieved by RunHook.
-	ActionId string `yaml:"action-id,omitempty"`
+	// StorageId is the ID of the storage instance relevant to the hook.
+	StorageId string `yaml:"storage-id,omitempty"`
 }
 
 // Validate returns an error if the info is not valid.
@@ -44,10 +50,27 @@ func (hi Info) Validate() error {
 	case hooks.Install, hooks.Start, hooks.ConfigChanged, hooks.UpgradeCharm, hooks.Stop, hooks.RelationBroken, hooks.CollectMetrics, hooks.MeterStatusChanged:
 		return nil
 	case hooks.Action:
-		if !names.IsValidAction(hi.ActionId) {
-			return fmt.Errorf("action id %q cannot be parsed as an action tag", hi.ActionId)
+		return fmt.Errorf("hooks.Kind Action is deprecated")
+	case hooks.StorageAttached, hooks.StorageDetaching:
+		if !names.IsValidStorage(hi.StorageId) {
+			return fmt.Errorf("invalid storage ID %q", hi.StorageId)
 		}
 		return nil
+	// TODO(fwereade): define these in charm/hooks...
+	case LeaderElected, LeaderDeposed, LeaderSettingsChanged:
+		return nil
 	}
 	return fmt.Errorf("unknown hook kind %q", hi.Kind)
 }
+
+// Committer is an interface that may be used to convey the fact that the
+// specified hook has been successfully executed, and committed.
+type Committer interface {
+	CommitHook(Info) error
+}
+
+// Validator is an interface that may be used to validate a hook execution
+// request prior to executing it.
+type Validator interface {
+	ValidateHook(Info) error
+}
