@@ -5,17 +5,23 @@ package runner_test
 
 import (
 	"os"
+	"strings"
 	"time"
 
 	"github.com/juju/errors"
 	"github.com/juju/names"
+	"github.com/juju/testing"
 	jc "github.com/juju/testing/checkers"
+	"github.com/juju/utils"
 	"github.com/juju/utils/fs"
 	gc "gopkg.in/check.v1"
-	"gopkg.in/juju/charm.v4/hooks"
+	"gopkg.in/juju/charm.v5/hooks"
 
 	"github.com/juju/juju/apiserver/params"
+	"github.com/juju/juju/state"
+	"github.com/juju/juju/storage"
 	"github.com/juju/juju/testcharms"
+	"github.com/juju/juju/worker/leadership"
 	"github.com/juju/juju/worker/uniter/hook"
 	"github.com/juju/juju/worker/uniter/runner"
 )
@@ -29,6 +35,14 @@ type FactorySuite struct {
 
 var _ = gc.Suite(&FactorySuite{})
 
+type fakeTracker struct {
+	leadership.Tracker
+}
+
+func (fakeTracker) ServiceName() string {
+	return "service-name"
+}
+
 func (s *FactorySuite) SetUpTest(c *gc.C) {
 	s.HookContextSuite.SetUpTest(c)
 	s.paths = NewRealPaths(c)
@@ -36,7 +50,9 @@ func (s *FactorySuite) SetUpTest(c *gc.C) {
 	factory, err := runner.NewFactory(
 		s.uniter,
 		s.unit.Tag().(names.UnitTag),
+		fakeTracker{},
 		s.getRelationInfos,
+		s.storage,
 		s.paths,
 	)
 	c.Assert(err, jc.ErrorIsNil)
@@ -70,11 +86,11 @@ func (s *FactorySuite) setUpCacheMethods(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 }
 
-func (s *FactorySuite) updateCache(relId int, unitName string, settings params.RelationSettings) {
+func (s *FactorySuite) updateCache(relId int, unitName string, settings params.Settings) {
 	runner.UpdateCachedSettings(s.factory, relId, unitName, settings)
 }
 
-func (s *FactorySuite) getCache(relId int, unitName string) (params.RelationSettings, bool) {
+func (s *FactorySuite) getCache(relId int, unitName string) (params.Settings, bool) {
 	return runner.CachedSettings(s.factory, relId, unitName)
 }
 
@@ -122,6 +138,21 @@ func (s *FactorySuite) AssertNotActionContext(c *gc.C, ctx runner.Context) {
 	c.Assert(err, gc.ErrorMatches, "not running an action")
 }
 
+func (s *FactorySuite) AssertNotStorageContext(c *gc.C, ctx runner.Context) {
+	storageAttachment, ok := ctx.HookStorage()
+	c.Assert(storageAttachment, gc.IsNil)
+	c.Assert(ok, jc.IsFalse)
+}
+
+func (s *FactorySuite) AssertStorageContext(c *gc.C, ctx runner.Context, id string, attachment storage.StorageAttachmentInfo) {
+	fromCache, ok := ctx.HookStorage()
+	c.Assert(ok, jc.IsTrue)
+	c.Assert(fromCache, gc.NotNil)
+	c.Assert(fromCache.Tag().Id(), gc.Equals, id)
+	c.Assert(fromCache.Kind(), gc.Equals, attachment.Kind)
+	c.Assert(fromCache.Location(), gc.Equals, attachment.Location)
+}
+
 func (s *FactorySuite) AssertRelationContext(c *gc.C, ctx runner.Context, relId int, remoteUnit string) *runner.ContextRelation {
 	actualRemoteUnit, _ := ctx.RemoteUnitName()
 	c.Assert(actualRemoteUnit, gc.Equals, remoteUnit)
@@ -145,6 +176,7 @@ func (s *FactorySuite) TestNewCommandRunnerNoRelation(c *gc.C) {
 	s.AssertCoreContext(c, ctx)
 	s.AssertNotActionContext(c, ctx)
 	s.AssertNotRelationContext(c, ctx)
+	s.AssertNotStorageContext(c, ctx)
 }
 
 func (s *FactorySuite) TestNewCommandRunnerRelationIdDoesNotExist(c *gc.C) {
@@ -203,6 +235,7 @@ func (s *FactorySuite) TestNewCommandRunnerForceNoRemoteUnit(c *gc.C) {
 	s.AssertCoreContext(c, ctx)
 	s.AssertNotActionContext(c, ctx)
 	s.AssertRelationContext(c, ctx, 0, "")
+	s.AssertNotStorageContext(c, ctx)
 }
 
 func (s *FactorySuite) TestNewCommandRunnerForceRemoteUnitMissing(c *gc.C) {
@@ -214,6 +247,7 @@ func (s *FactorySuite) TestNewCommandRunnerForceRemoteUnitMissing(c *gc.C) {
 	s.AssertCoreContext(c, ctx)
 	s.AssertNotActionContext(c, ctx)
 	s.AssertRelationContext(c, ctx, 0, "blah/123")
+	s.AssertNotStorageContext(c, ctx)
 }
 
 func (s *FactorySuite) TestNewCommandRunnerInferRemoteUnit(c *gc.C) {
@@ -225,6 +259,7 @@ func (s *FactorySuite) TestNewCommandRunnerInferRemoteUnit(c *gc.C) {
 	s.AssertCoreContext(c, ctx)
 	s.AssertNotActionContext(c, ctx)
 	s.AssertRelationContext(c, ctx, 0, "foo/2")
+	s.AssertNotStorageContext(c, ctx)
 }
 
 func (s *FactorySuite) TestNewHookRunner(c *gc.C) {
@@ -235,6 +270,7 @@ func (s *FactorySuite) TestNewHookRunner(c *gc.C) {
 	s.AssertCoreContext(c, ctx)
 	s.AssertNotActionContext(c, ctx)
 	s.AssertNotRelationContext(c, ctx)
+	s.AssertNotStorageContext(c, ctx)
 }
 
 func (s *FactorySuite) TestNewHookRunnerWithBadHook(c *gc.C) {
@@ -243,6 +279,73 @@ func (s *FactorySuite) TestNewHookRunnerWithBadHook(c *gc.C) {
 	c.Assert(err, gc.ErrorMatches, `unknown hook kind ""`)
 }
 
+func (s *FactorySuite) TestNewHookRunnerWithStorage(c *gc.C) {
+	// We need to set up a unit that has storage metadata defined.
+	ch := s.AddTestingCharm(c, "storage-block")
+	sCons := map[string]state.StorageConstraints{
+		"data": {Pool: "", Size: 1024, Count: 1},
+	}
+	service := s.AddTestingServiceWithStorage(c, "storage-block", ch, sCons)
+	s.machine = nil // allocate a new machine
+	unit := s.AddUnit(c, service)
+
+	storageAttachments, err := s.State.UnitStorageAttachments(unit.UnitTag())
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(storageAttachments, gc.HasLen, 1)
+	storageTag := storageAttachments[0].StorageInstance()
+
+	volume, err := s.State.StorageInstanceVolume(storageTag)
+	c.Assert(err, jc.ErrorIsNil)
+	volumeTag := volume.VolumeTag()
+	machineTag := s.machine.MachineTag()
+
+	err = s.State.SetVolumeInfo(
+		volumeTag, state.VolumeInfo{
+			VolumeId: "vol-123",
+			Size:     456,
+		},
+	)
+	c.Assert(err, jc.ErrorIsNil)
+	err = s.State.SetVolumeAttachmentInfo(
+		machineTag, volumeTag, state.VolumeAttachmentInfo{
+			DeviceName: "sdb",
+		},
+	)
+	c.Assert(err, jc.ErrorIsNil)
+
+	password, err := utils.RandomPassword()
+	err = unit.SetPassword(password)
+	c.Assert(err, jc.ErrorIsNil)
+	st := s.OpenAPIAs(c, unit.Tag(), password)
+	uniter, err := st.Uniter()
+	c.Assert(err, jc.ErrorIsNil)
+
+	factory, err := runner.NewFactory(
+		uniter,
+		unit.Tag().(names.UnitTag),
+		fakeTracker{},
+		s.getRelationInfos,
+		s.storage,
+		s.paths,
+	)
+	c.Assert(err, jc.ErrorIsNil)
+
+	rnr, err := factory.NewHookRunner(hook.Info{
+		Kind:      hooks.StorageAttached,
+		StorageId: "data/0",
+	})
+	c.Assert(err, jc.ErrorIsNil)
+	s.AssertPaths(c, rnr)
+	ctx := rnr.Context()
+	c.Assert(ctx.UnitName(), gc.Equals, "storage-block/0")
+	s.AssertStorageContext(c, ctx, "data/0", storage.StorageAttachmentInfo{
+		Kind:     storage.StorageKindBlock,
+		Location: "/dev/sdb",
+	})
+	s.AssertNotActionContext(c, ctx)
+	s.AssertNotRelationContext(c, ctx)
+}
+
 func (s *FactorySuite) TestNewHookRunnerWithRelation(c *gc.C) {
 	rnr, err := s.factory.NewHookRunner(hook.Info{
 		Kind:       hooks.RelationBroken,
@@ -254,6 +357,7 @@ func (s *FactorySuite) TestNewHookRunnerWithRelation(c *gc.C) {
 	s.AssertCoreContext(c, ctx)
 	s.AssertNotActionContext(c, ctx)
 	s.AssertRelationContext(c, ctx, 1, "")
+	s.AssertNotStorageContext(c, ctx)
 }
 
 func (s *FactorySuite) TestNewHookRunnerPrunesNonMemberCaches(c *gc.C) {
@@ -261,8 +365,8 @@ func (s *FactorySuite) TestNewHookRunnerPrunesNonMemberCaches(c *gc.C) {
 	// Write cached member settings for a member and a non-member.
 	s.setUpCacheMethods(c)
 	s.membership[0] = []string{"rel0/0"}
-	s.updateCache(0, "rel0/0", params.RelationSettings{"keep": "me"})
-	s.updateCache(0, "rel0/1", params.RelationSettings{"drop": "me"})
+	s.updateCache(0, "rel0/0", params.Settings{"keep": "me"})
+	s.updateCache(0, "rel0/1", params.Settings{"drop": "me"})
 
 	rnr, err := s.factory.NewHookRunner(hook.Info{Kind: hooks.Install})
 	c.Assert(err, jc.ErrorIsNil)
@@ -271,7 +375,7 @@ func (s *FactorySuite) TestNewHookRunnerPrunesNonMemberCaches(c *gc.C) {
 
 	settings0, found := s.getCache(0, "rel0/0")
 	c.Assert(found, jc.IsTrue)
-	c.Assert(settings0, jc.DeepEquals, params.RelationSettings{"keep": "me"})
+	c.Assert(settings0, jc.DeepEquals, params.Settings{"keep": "me"})
 
 	settings1, found := s.getCache(0, "rel0/1")
 	c.Assert(found, jc.IsFalse)
@@ -285,7 +389,7 @@ func (s *FactorySuite) TestNewHookRunnerPrunesNonMemberCaches(c *gc.C) {
 	// Nothing's really in scope, so the call would fail if they weren't.
 	settings0, err = relCtx.ReadSettings("rel0/0")
 	c.Assert(err, jc.ErrorIsNil)
-	c.Assert(settings0, jc.DeepEquals, params.RelationSettings{"keep": "me"})
+	c.Assert(settings0, jc.DeepEquals, params.Settings{"keep": "me"})
 
 	// Verify that the non-member settings were purged by looking them up and
 	// checking for the expected error.
@@ -298,7 +402,7 @@ func (s *FactorySuite) TestNewHookRunnerRelationJoinedUpdatesRelationContextAndC
 	// Write some cached settings for r/0, so we can verify the cache gets cleared.
 	s.setUpCacheMethods(c)
 	s.membership[1] = []string{"r/0"}
-	s.updateCache(1, "r/0", params.RelationSettings{"foo": "bar"})
+	s.updateCache(1, "r/0", params.Settings{"foo": "bar"})
 
 	rnr, err := s.factory.NewHookRunner(hook.Info{
 		Kind:       hooks.RelationJoined,
@@ -310,6 +414,7 @@ func (s *FactorySuite) TestNewHookRunnerRelationJoinedUpdatesRelationContextAndC
 	ctx := rnr.Context()
 	s.AssertCoreContext(c, ctx)
 	s.AssertNotActionContext(c, ctx)
+	s.AssertNotStorageContext(c, ctx)
 	rel := s.AssertRelationContext(c, ctx, 1, "r/0")
 	c.Assert(rel.UnitNames(), jc.DeepEquals, []string{"r/0"})
 	cached0, member := s.getCache(1, "r/0")
@@ -322,8 +427,8 @@ func (s *FactorySuite) TestNewHookRunnerRelationChangedUpdatesRelationContextAnd
 	// the change for r/4 clears its cache but leaves r/0's alone.
 	s.setUpCacheMethods(c)
 	s.membership[1] = []string{"r/0", "r/4"}
-	s.updateCache(1, "r/0", params.RelationSettings{"foo": "bar"})
-	s.updateCache(1, "r/4", params.RelationSettings{"baz": "qux"})
+	s.updateCache(1, "r/0", params.Settings{"foo": "bar"})
+	s.updateCache(1, "r/4", params.Settings{"baz": "qux"})
 
 	rnr, err := s.factory.NewHookRunner(hook.Info{
 		Kind:       hooks.RelationChanged,
@@ -335,10 +440,11 @@ func (s *FactorySuite) TestNewHookRunnerRelationChangedUpdatesRelationContextAnd
 	ctx := rnr.Context()
 	s.AssertCoreContext(c, ctx)
 	s.AssertNotActionContext(c, ctx)
+	s.AssertNotStorageContext(c, ctx)
 	rel := s.AssertRelationContext(c, ctx, 1, "r/4")
 	c.Assert(rel.UnitNames(), jc.DeepEquals, []string{"r/0", "r/4"})
 	cached0, member := s.getCache(1, "r/0")
-	c.Assert(cached0, jc.DeepEquals, params.RelationSettings{"foo": "bar"})
+	c.Assert(cached0, jc.DeepEquals, params.Settings{"foo": "bar"})
 	c.Assert(member, jc.IsTrue)
 	cached4, member := s.getCache(1, "r/4")
 	c.Assert(cached4, gc.IsNil)
@@ -350,8 +456,8 @@ func (s *FactorySuite) TestNewHookRunnerRelationDepartedUpdatesRelationContextAn
 	// the depart for r/0 leaves r/4's cache alone (while discarding r/0's).
 	s.setUpCacheMethods(c)
 	s.membership[1] = []string{"r/0", "r/4"}
-	s.updateCache(1, "r/0", params.RelationSettings{"foo": "bar"})
-	s.updateCache(1, "r/4", params.RelationSettings{"baz": "qux"})
+	s.updateCache(1, "r/0", params.Settings{"foo": "bar"})
+	s.updateCache(1, "r/4", params.Settings{"baz": "qux"})
 
 	rnr, err := s.factory.NewHookRunner(hook.Info{
 		Kind:       hooks.RelationDeparted,
@@ -363,13 +469,14 @@ func (s *FactorySuite) TestNewHookRunnerRelationDepartedUpdatesRelationContextAn
 	ctx := rnr.Context()
 	s.AssertCoreContext(c, ctx)
 	s.AssertNotActionContext(c, ctx)
+	s.AssertNotStorageContext(c, ctx)
 	rel := s.AssertRelationContext(c, ctx, 1, "r/0")
 	c.Assert(rel.UnitNames(), jc.DeepEquals, []string{"r/4"})
 	cached0, member := s.getCache(1, "r/0")
 	c.Assert(cached0, gc.IsNil)
 	c.Assert(member, jc.IsFalse)
 	cached4, member := s.getCache(1, "r/4")
-	c.Assert(cached4, jc.DeepEquals, params.RelationSettings{"baz": "qux"})
+	c.Assert(cached4, jc.DeepEquals, params.Settings{"baz": "qux"})
 	c.Assert(member, jc.IsTrue)
 }
 
@@ -382,8 +489,8 @@ func (s *FactorySuite) TestNewHookRunnerRelationBrokenRetainsCaches(c *gc.C) {
 	// relations and ignore everything else.
 	s.setUpCacheMethods(c)
 	s.membership[1] = []string{"r/0", "r/4"}
-	s.updateCache(1, "r/0", params.RelationSettings{"foo": "bar"})
-	s.updateCache(1, "r/4", params.RelationSettings{"baz": "qux"})
+	s.updateCache(1, "r/0", params.Settings{"foo": "bar"})
+	s.updateCache(1, "r/4", params.Settings{"baz": "qux"})
 
 	rnr, err := s.factory.NewHookRunner(hook.Info{
 		Kind:       hooks.RelationBroken,
@@ -395,10 +502,10 @@ func (s *FactorySuite) TestNewHookRunnerRelationBrokenRetainsCaches(c *gc.C) {
 	rel := s.AssertRelationContext(c, ctx, 1, "")
 	c.Assert(rel.UnitNames(), jc.DeepEquals, []string{"r/0", "r/4"})
 	cached0, member := s.getCache(1, "r/0")
-	c.Assert(cached0, jc.DeepEquals, params.RelationSettings{"foo": "bar"})
+	c.Assert(cached0, jc.DeepEquals, params.Settings{"foo": "bar"})
 	c.Assert(member, jc.IsTrue)
 	cached4, member := s.getCache(1, "r/4")
-	c.Assert(cached4, jc.DeepEquals, params.RelationSettings{"baz": "qux"})
+	c.Assert(cached4, jc.DeepEquals, params.Settings{"baz": "qux"})
 	c.Assert(member, jc.IsTrue)
 }
 
@@ -448,14 +555,7 @@ func (s *FactorySuite) TestNewHookRunnerMetricsEnabled(c *gc.C) {
 	c.Assert(err, jc.ErrorIsNil)
 }
 
-func (s *FactorySuite) TestNewActionRunnerBadCharm(c *gc.C) {
-	rnr, err := s.factory.NewActionRunner("irrelevant")
-	c.Assert(rnr, gc.IsNil)
-	c.Assert(errors.Cause(err), jc.Satisfies, os.IsNotExist)
-	c.Assert(err, gc.Not(jc.Satisfies), runner.IsBadActionError)
-}
-
-func (s *FactorySuite) TestNewActionRunner(c *gc.C) {
+func (s *FactorySuite) TestNewActionRunnerGood(c *gc.C) {
 	s.SetCharm(c, "dummy")
 	action, err := s.State.EnqueueAction(s.unit.Tag(), "snapshot", map[string]interface{}{
 		"outfile": "/some/file.bz2",
@@ -475,6 +575,19 @@ func (s *FactorySuite) TestNewActionRunner(c *gc.C) {
 		},
 		ResultsMap: map[string]interface{}{},
 	})
+	vars := ctx.HookVars(s.paths)
+	c.Assert(len(vars) > 0, jc.IsTrue, gc.Commentf("expected HookVars but found none"))
+	combined := strings.Join(vars, "|")
+	c.Assert(combined, gc.Matches, `(^|.*\|)JUJU_ACTION_NAME=snapshot(\|.*|$)`)
+	c.Assert(combined, gc.Matches, `(^|.*\|)JUJU_ACTION_UUID=`+action.Id()+`(\|.*|$)`)
+	c.Assert(combined, gc.Matches, `(^|.*\|)JUJU_ACTION_TAG=`+action.Tag().String()+`(\|.*|$)`)
+}
+
+func (s *FactorySuite) TestNewActionRunnerBadCharm(c *gc.C) {
+	rnr, err := s.factory.NewActionRunner("irrelevant")
+	c.Assert(rnr, gc.IsNil)
+	c.Assert(errors.Cause(err), jc.Satisfies, os.IsNotExist)
+	c.Assert(err, gc.Not(jc.Satisfies), runner.IsBadActionError)
 }
 
 func (s *FactorySuite) TestNewActionRunnerBadName(c *gc.C) {
@@ -522,3 +635,65 @@ func (s *FactorySuite) TestNewActionRunnerUnauthAction(c *gc.C) {
 	c.Check(err, gc.ErrorMatches, "action no longer available")
 	c.Check(err, gc.Equals, runner.ErrActionNotAvailable)
 }
+
+func (s *FactorySuite) testLeadershipContextWiring(c *gc.C, createRunner func() runner.Runner) {
+	var stub testing.Stub
+	stub.SetErrors(errors.New("bam"))
+	restore := runner.PatchNewLeadershipContext(
+		func(accessor runner.LeadershipSettingsAccessor, tracker leadership.Tracker) runner.LeadershipContext {
+			stub.AddCall("NewLeadershipContext", accessor, tracker)
+			return &StubLeadershipContext{Stub: &stub}
+		},
+	)
+	defer restore()
+
+	rnr := createRunner()
+	isLeader, err := rnr.Context().IsLeader()
+	c.Check(err, gc.ErrorMatches, "bam")
+	c.Check(isLeader, jc.IsFalse)
+
+	stub.CheckCalls(c, []testing.StubCall{{
+		FuncName: "NewLeadershipContext",
+		Args:     []interface{}{s.uniter.LeadershipSettings, fakeTracker{}},
+	}, {
+		FuncName: "IsLeader",
+	}})
+
+}
+
+func (s *FactorySuite) TestNewHookRunnerLeadershipContext(c *gc.C) {
+	s.testLeadershipContextWiring(c, func() runner.Runner {
+		rnr, err := s.factory.NewHookRunner(hook.Info{Kind: hooks.ConfigChanged})
+		c.Assert(err, jc.ErrorIsNil)
+		return rnr
+	})
+}
+
+func (s *FactorySuite) TestNewActionRunnerLeadershipContext(c *gc.C) {
+	s.testLeadershipContextWiring(c, func() runner.Runner {
+		s.SetCharm(c, "dummy")
+		action, err := s.State.EnqueueAction(s.unit.Tag(), "snapshot", nil)
+		c.Assert(err, jc.ErrorIsNil)
+		rnr, err := s.factory.NewActionRunner(action.Id())
+		c.Assert(err, jc.ErrorIsNil)
+		return rnr
+	})
+}
+
+func (s *FactorySuite) TestNewCommandRunnerLeadershipContext(c *gc.C) {
+	s.testLeadershipContextWiring(c, func() runner.Runner {
+		rnr, err := s.factory.NewCommandRunner(runner.CommandInfo{RelationId: -1})
+		c.Assert(err, jc.ErrorIsNil)
+		return rnr
+	})
+}
+
+type StubLeadershipContext struct {
+	runner.LeadershipContext
+	*testing.Stub
+}
+
+func (stub *StubLeadershipContext) IsLeader() (bool, error) {
+	stub.MethodCall(stub, "IsLeader")
+	return false, stub.NextErr()
+}
