@@ -1,4 +1,4 @@
-// Copyright 2012-2014 Canonical Ltd.
+// Copyright 2012-2015 Canonical Ltd.
 // Licensed under the AGPLv3, see LICENCE file for details.
 
 package uniter
@@ -7,31 +7,37 @@ import (
 	"fmt"
 	"os"
 	"strings"
+	"sync"
+	"time"
 
 	"github.com/juju/errors"
 	"github.com/juju/loggo"
 	"github.com/juju/names"
 	"github.com/juju/utils/exec"
 	"github.com/juju/utils/fslock"
-	corecharm "gopkg.in/juju/charm.v4"
-	"gopkg.in/juju/charm.v4/hooks"
+	corecharm "gopkg.in/juju/charm.v5"
 	"launchpad.net/tomb"
 
 	"github.com/juju/juju/api/uniter"
 	"github.com/juju/juju/apiserver/params"
-	"github.com/juju/juju/state/watcher"
+	coreleadership "github.com/juju/juju/leadership"
 	"github.com/juju/juju/version"
 	"github.com/juju/juju/worker"
+	"github.com/juju/juju/worker/leadership"
 	"github.com/juju/juju/worker/uniter/charm"
 	"github.com/juju/juju/worker/uniter/filter"
-	"github.com/juju/juju/worker/uniter/hook"
 	"github.com/juju/juju/worker/uniter/operation"
 	"github.com/juju/juju/worker/uniter/runner"
 	"github.com/juju/juju/worker/uniter/runner/jujuc"
+	"github.com/juju/juju/worker/uniter/storage"
 )
 
 var logger = loggo.GetLogger("juju.worker.uniter")
 
+// leadershipGuarantee defines the period of time for which a successful call
+// to the is-leader hook tool guarantees continued leadership.
+var leadershipGuarantee = 30 * time.Second
+
 // A UniterExecutionObserver gets the appropriate methods called when a hook
 // is executed and either succeeds or fails.  Missing hooks don't get reported
 // in this way.
@@ -40,27 +46,6 @@ type UniterExecutionObserver interface {
 	HookFailed(hookName string)
 }
 
-// deployerProxy exists because we're not yet comfortable that we can safely
-// drop support for charm.gitDeployer. If we can, then the uniter doesn't
-// need a deployer reference at all: and we can drop fixDeployer, and even
-// the Notify* methods on the Deployer interface, and simply hand the
-// deployer we create over to the operationFactory at creation and forget
-// about it.
-//
-// We will never be *completely* certain that gitDeployer can be dropped,
-// because it's not done as an upgrade step (because we can't replace the
-// deployer while conflicted, and upgrades are not gated on no-conflicts);
-// and so long as there's a reasonable possibility that someone *might* have
-// been running a pre-1.19.1 environment, and have either upgraded directly
-// in a conflict state *or* have upgraded stepwise without fixing a conflict
-// state, we should keep this complexity.
-//
-// In practice, that possibility is growing ever more remote, but we're not
-// ready to pull the trigger yet.
-type deployerProxy struct {
-	charm.Deployer
-}
-
 // Uniter implements the capabilities of the unit agent. It is not intended to
 // implement the actual *behaviour* of the unit agent; that responsibility is
 // delegated to Mode values, which are expected to react to events and direct
@@ -72,15 +57,28 @@ type Uniter struct {
 	f         filter.Filter
 	unit      *uniter.Unit
 	relations Relations
+	cleanups  []cleanup
+	storage   *storage.Attachments
 
-	deployer          *deployerProxy
-	operationFactory  operation.Factory
-	operationExecutor operation.Executor
+	// Cache the last reported status information
+	// so we don't make unnecessary api calls.
+	setStatusMutex      sync.Mutex
+	lastReportedStatus  params.Status
+	lastReportedMessage string
 
-	hookLock    *fslock.Lock
+	deployer              *deployerProxy
+	operationFactory      operation.Factory
+	operationExecutor     operation.Executor
+	operationExecutorFunc executorFunc
+
+	leadershipClaimer coreleadership.Claimer
+	leadershipTracker leadership.Tracker
+
+	machineLock *fslock.Lock
 	runListener *RunListener
 
-	ranConfigChanged bool
+	ranLeaderSettingsChanged bool
+	ranConfigChanged         bool
 
 	// The execution observer is only used in tests at this stage. Should this
 	// need to be extended, perhaps a list of observers would be needed.
@@ -91,31 +89,70 @@ type Uniter struct {
 	collectMetricsAt CollectMetricsSignal
 }
 
+type executorFunc func(*Uniter) (operation.Executor, error)
+
 // NewUniter creates a new Uniter which will install, run, and upgrade
 // a charm on behalf of the unit with the given unitTag, by executing
 // hooks and operations provoked by changes in st.
-func NewUniter(st *uniter.State, unitTag names.UnitTag, dataDir string, hookLock *fslock.Lock) *Uniter {
+func NewUniter(
+	st *uniter.State,
+	unitTag names.UnitTag,
+	leadershipClaimer coreleadership.Claimer,
+	dataDir string,
+	machineLock *fslock.Lock,
+	operationExecutorFunc executorFunc,
+) *Uniter {
 	u := &Uniter{
-		st:               st,
-		paths:            NewPaths(dataDir, unitTag),
-		hookLock:         hookLock,
-		collectMetricsAt: inactiveMetricsTimer,
+		st:                    st,
+		paths:                 NewPaths(dataDir, unitTag),
+		machineLock:           machineLock,
+		leadershipClaimer:     leadershipClaimer,
+		collectMetricsAt:      inactiveMetricsTimer,
+		operationExecutorFunc: operationExecutorFunc,
+	}
+	if u.operationExecutorFunc == nil {
+		u.operationExecutorFunc = newOperationExecutor
 	}
 	go func() {
 		defer u.tomb.Done()
+		defer u.runCleanups()
 		u.tomb.Kill(u.loop(unitTag))
 	}()
 	return u
 }
 
+type cleanup func() error
+
+func (u *Uniter) addCleanup(cleanup cleanup) {
+	u.cleanups = append(u.cleanups, cleanup)
+}
+
+func (u *Uniter) runCleanups() {
+	for _, cleanup := range u.cleanups {
+		u.tomb.Kill(cleanup())
+	}
+}
+
 func (u *Uniter) loop(unitTag names.UnitTag) (err error) {
+	// Start tracking leadership state.
+	// TODO(fwereade): ideally, this wouldn't be created here; as a worker it's
+	// clearly better off being managed by a Runner. However, we haven't come up
+	// with a clean way to reference one (lineage of a...) worker from another,
+	// so for now the tracker is accessible only to its unit.
+	leadershipTracker := leadership.NewTrackerWorker(
+		unitTag, u.leadershipClaimer, leadershipGuarantee,
+	)
+	u.addCleanup(func() error {
+		return worker.Stop(leadershipTracker)
+	})
+	u.leadershipTracker = leadershipTracker
+
 	if err := u.init(unitTag); err != nil {
 		if err == worker.ErrTerminateAgent {
 			return err
 		}
 		return fmt.Errorf("failed to initialize uniter for %q: %v", unitTag, err)
 	}
-	defer u.runListener.Close()
 	logger.Infof("unit %q started", u.unit)
 
 	// Start filtering state change events for consumption by modes.
@@ -123,10 +160,14 @@ func (u *Uniter) loop(unitTag names.UnitTag) (err error) {
 	if err != nil {
 		return err
 	}
-	defer watcher.Stop(u.f, &u.tomb)
-	go func() {
-		u.tomb.Kill(u.f.Wait())
-	}()
+	u.addCleanup(u.f.Stop)
+
+	// Stop the uniter if either of these components fails.
+	go func() { u.tomb.Kill(leadershipTracker.Wait()) }()
+	go func() { u.tomb.Kill(u.f.Wait()) }()
+
+	// Start handling leader settings events, or not, as appropriate.
+	u.f.WantLeaderSettingsEvents(!u.operationState().Leader)
 
 	// Run modes until we encounter an error.
 	mode := ModeContinue
@@ -137,12 +178,17 @@ func (u *Uniter) loop(unitTag names.UnitTag) (err error) {
 		default:
 			mode, err = mode(u)
 			switch cause := errors.Cause(err); cause {
-			case operation.ErrHookFailed:
-				mode, err = ModeHookError, nil
 			case operation.ErrNeedsReboot:
 				err = worker.ErrRebootMachine
 			case tomb.ErrDying, worker.ErrTerminateAgent:
 				err = cause
+			case operation.ErrHookFailed:
+				mode, err = ModeHookError, nil
+			default:
+				charmURL, ok := operation.DeployConflictCharmURL(cause)
+				if ok {
+					mode, err = ModeConflicted(charmURL), nil
+				}
 			}
 		}
 	}
@@ -151,13 +197,13 @@ func (u *Uniter) loop(unitTag names.UnitTag) (err error) {
 }
 
 func (u *Uniter) setupLocks() (err error) {
-	if message := u.hookLock.Message(); u.hookLock.IsLocked() && message != "" {
+	if message := u.machineLock.Message(); u.machineLock.IsLocked() && message != "" {
 		// Look to see if it was us that held the lock before.  If it was, we
 		// should be safe enough to break it, as it is likely that we died
-		// before unlocking, and have been restarted by upstart.
+		// before unlocking, and have been restarted by the init system.
 		parts := strings.SplitN(message, ":", 2)
 		if len(parts) > 1 && parts[0] == u.unit.Name() {
-			if err := u.hookLock.BreakLock(); err != nil {
+			if err := u.machineLock.BreakLock(); err != nil {
 				return err
 			}
 		}
@@ -165,6 +211,12 @@ func (u *Uniter) setupLocks() (err error) {
 	return nil
 }
 
+func newOperationExecutor(u *Uniter) (operation.Executor, error) {
+	return operation.NewExecutor(
+		u.paths.State.OperationsFile, u.getServiceCharmURL, u.acquireExecutionLock,
+	)
+}
+
 func (u *Uniter) init(unitTag names.UnitTag) (err error) {
 	u.unit, err = u.st.Unit(unitTag)
 	if err != nil {
@@ -191,6 +243,14 @@ func (u *Uniter) init(unitTag names.UnitTag) (err error) {
 		return errors.Annotatef(err, "cannot create relations")
 	}
 	u.relations = relations
+	storageAttachments, err := storage.NewAttachments(
+		u.st, unitTag, u.paths.State.StorageDir, u.tomb.Dying(),
+	)
+	if err != nil {
+		return errors.Annotatef(err, "cannot create storage hook source")
+	}
+	u.storage = storageAttachments
+	u.addCleanup(storageAttachments.Stop)
 
 	deployer, err := charm.NewDeployer(
 		u.paths.State.CharmDir,
@@ -202,7 +262,7 @@ func (u *Uniter) init(unitTag names.UnitTag) (err error) {
 	}
 	u.deployer = &deployerProxy{deployer}
 	runnerFactory, err := runner.NewFactory(
-		u.st, unitTag, u.relations.GetInfo, u.paths,
+		u.st, unitTag, u.leadershipTracker, u.relations.GetInfo, u.storage, u.paths,
 	)
 	if err != nil {
 		return err
@@ -211,12 +271,11 @@ func (u *Uniter) init(unitTag names.UnitTag) (err error) {
 		u.deployer,
 		runnerFactory,
 		&operationCallbacks{u},
+		u.storage,
 		u.tomb.Dying(),
 	)
 
-	operationExecutor, err := operation.NewExecutor(
-		u.paths.State.OperationsFile, u.getServiceCharmURL,
-	)
+	operationExecutor, err := u.operationExecutorFunc(u)
 	if err != nil {
 		return err
 	}
@@ -227,6 +286,11 @@ func (u *Uniter) init(unitTag names.UnitTag) (err error) {
 	if err != nil {
 		return err
 	}
+	u.addCleanup(func() error {
+		// TODO(fwereade): RunListener returns no error on Close. This seems wrong.
+		u.runListener.Close()
+		return nil
+	})
 	// The socket needs to have permissions 777 in order for other users to use it.
 	if version.Current.OS != version.Windows {
 		return os.Chmod(u.paths.Runtime.JujuRunSocket, 0777)
@@ -265,25 +329,6 @@ func (u *Uniter) operationState() operation.State {
 	return u.operationExecutor.State()
 }
 
-// deploy deploys the supplied charm URL, and sets follow-up hook operation state
-// as indicated by reason.
-func (u *Uniter) deploy(curl *corecharm.URL, reason operation.Kind) error {
-	op, err := u.operationFactory.NewDeploy(curl, reason)
-	if err != nil {
-		return err
-	}
-	err = u.operationExecutor.Run(op)
-	if err != nil {
-		return err
-	}
-
-	// The new charm may have declared metrics where the old one had none
-	// (or vice versa), so reset the metrics collection policy according
-	// to current state.
-	// TODO(fwereade): maybe this should be in operation.deploy.Commit()?
-	return u.initializeMetricsCollector()
-}
-
 // initializeMetricsCollector enables the periodic collect-metrics hook
 // for charms that declare metrics.
 func (u *Uniter) initializeMetricsCollector() error {
@@ -298,9 +343,13 @@ func (u *Uniter) initializeMetricsCollector() error {
 // RunCommands executes the supplied commands in a hook context.
 func (u *Uniter) RunCommands(args RunCommandsArgs) (results *exec.ExecResponse, err error) {
 	// TODO(fwereade): this is *still* all sorts of messed-up and not especially
-	// goroutine-safe, but that's not what I'm fixing at the moment. We'll deal
-	// with that when we get a sane ops queue and are no longer depending on the
-	// uniter mode funcs for all the rest of our scheduling.
+	// goroutine-safe, but that's not what I'm fixing at the moment. We could
+	// address this by:
+	//  1) implementing an operation to encapsulate the relations.Update call
+	//  2) (quick+dirty) mutex runOperation until we can
+	//  3) (correct) feed RunCommands requests into the mode funcs (or any queue
+	//     that replaces them) such that they're handled and prioritised like
+	//     every other operation.
 	logger.Tracef("run commands: %s", args.Commands)
 
 	type responseInfo struct {
@@ -318,11 +367,7 @@ func (u *Uniter) RunCommands(args RunCommandsArgs) (results *exec.ExecResponse,
 		RemoteUnitName:  args.RemoteUnitName,
 		ForceRemoteUnit: args.ForceRemoteUnit,
 	}
-	op, err := u.operationFactory.NewCommands(commandArgs, sendResponse)
-	if err != nil {
-		return nil, err
-	}
-	err = u.operationExecutor.Run(op)
+	err = u.runOperation(newCommandsOp(commandArgs, sendResponse))
 	if err == nil {
 		select {
 		case response := <-responseChan:
@@ -341,42 +386,62 @@ func (u *Uniter) RunCommands(args RunCommandsArgs) (results *exec.ExecResponse,
 	return results, err
 }
 
-// runAction executes the supplied hook.Info as an Action.
-func (u *Uniter) runAction(actionId string) (err error) {
-	op, err := u.operationFactory.NewAction(actionId)
+// runOperation uses the uniter's operation factory to run the supplied creation
+// func, and then runs the resulting operation.
+//
+// This has a number of advantages over having mode funcs use the factory and
+// executor directly:
+//   * it cuts down on duplicated code in the mode funcs, making the logic easier
+//     to parse
+//   * it narrows the (conceptual) interface exposed to the mode funcs -- one day
+//     we might even be able to use a (real) interface and maybe even approach a
+//     point where we can run direct unit tests(!) on the modes themselves.
+//   * it opens a path to fixing RunCommands -- all operation creation and
+//     execution is done in a single place, and it's much easier to force those
+//     onto a single thread.
+//       * this can't be done quite yet, though, because relation changes are
+//         not yet encapsulated in operations, and that needs to happen before
+//         RunCommands will *actually* be goroutine-safe.
+func (u *Uniter) runOperation(creator creator) (err error) {
+	errorMessage := "creating operation to run"
+	defer func() {
+		reportAgentError(u, errorMessage, err)
+	}()
+	op, err := creator(u.operationFactory)
 	if err != nil {
-		return err
+		return errors.Annotatef(err, "cannot create operation")
 	}
+	errorMessage = op.String()
+	before := u.operationState()
+	defer func() {
+		// Check that if we lose leadership as a result of this
+		// operation, we want to start getting leader settings events,
+		// or if we gain leadership we want to stop receiving those
+		// events.
+		if after := u.operationState(); before.Leader != after.Leader {
+			u.f.WantLeaderSettingsEvents(before.Leader)
+		}
+	}()
 	return u.operationExecutor.Run(op)
 }
 
-// runHook executes the supplied hook.Info in an appropriate hook context. If
-// the hook itself fails to execute, it returns errHookFailed.
-func (u *Uniter) runHook(hi hook.Info) (err error) {
-	if hi.Kind == hooks.Action {
-		return u.runAction(hi.ActionId)
+// acquireExecutionLock acquires the machine-level execution lock, and
+// returns a func that must be called to unlock it. It's used by operation.Executor
+// when running operations that execute external code.
+func (u *Uniter) acquireExecutionLock(message string) (func() error, error) {
+	// We want to make sure we don't block forever when locking, but take the
+	// Uniter's tomb into account.
+	checkTomb := func() error {
+		select {
+		case <-u.tomb.Dying():
+			return tomb.ErrDying
+		default:
+			return nil
+		}
 	}
-	op, err := u.operationFactory.NewHook(hi)
-	if err != nil {
-		return err
+	message = fmt.Sprintf("%s: %s", u.unit.Name(), message)
+	if err := u.machineLock.LockWithFunc(message, checkTomb); err != nil {
+		return nil, err
 	}
-	return u.operationExecutor.Run(op)
-}
-
-func (u *Uniter) skipHook(hi hook.Info) (err error) {
-	op, err := u.operationFactory.NewHook(hi)
-	if err != nil {
-		return err
-	}
-	return u.operationExecutor.Skip(op)
-}
-
-// fixDeployer replaces the uniter's git-based charm deployer with a manifest-
-// based one, if necessary. It should not be called unless the existing charm
-// deployment is known to be in a stable state.
-func (u *Uniter) fixDeployer() error {
-	if err := charm.FixDeployer(&u.deployer.Deployer); err != nil {
-		return fmt.Errorf("cannot convert git deployment to manifest deployment: %v", err)
-	}
-	return nil
+	return func() error { return u.machineLock.Unlock() }, nil
 }
