@@ -7,11 +7,11 @@ import (
 	"bytes"
 	"fmt"
 	"io/ioutil"
-	"net/rpc"
 	"os"
 	"os/exec"
 	"path/filepath"
 	"reflect"
+	"runtime"
 	"strconv"
 	"strings"
 	"sync"
@@ -27,12 +27,14 @@ import (
 	"github.com/juju/utils/fslock"
 	"github.com/juju/utils/proxy"
 	gc "gopkg.in/check.v1"
-	corecharm "gopkg.in/juju/charm.v4"
+	corecharm "gopkg.in/juju/charm.v5"
 	goyaml "gopkg.in/yaml.v1"
 
 	apiuniter "github.com/juju/juju/api/uniter"
 	"github.com/juju/juju/apiserver/params"
+	"github.com/juju/juju/juju/sockets"
 	"github.com/juju/juju/juju/testing"
+	"github.com/juju/juju/leadership"
 	"github.com/juju/juju/network"
 	"github.com/juju/juju/state"
 	"github.com/juju/juju/state/storage"
@@ -41,6 +43,7 @@ import (
 	"github.com/juju/juju/worker"
 	"github.com/juju/juju/worker/uniter"
 	"github.com/juju/juju/worker/uniter/charm"
+	"github.com/juju/juju/worker/uniter/operation"
 )
 
 // worstCase is used for timeouts when timing out
@@ -59,7 +62,7 @@ func assertAssignUnit(c *gc.C, st *state.State, u *state.Unit) {
 	c.Assert(err, jc.ErrorIsNil)
 	err = machine.SetProvisioned("i-exist", "fake_nonce", nil)
 	c.Assert(err, jc.ErrorIsNil)
-	err = machine.SetAddresses(network.Address{
+	err = machine.SetProviderAddresses(network.Address{
 		Type:  network.IPv4Address,
 		Scope: network.ScopeCloudLocal,
 		Value: "private.address.example.com",
@@ -78,6 +81,7 @@ type context struct {
 	s             *UniterSuite
 	st            *state.State
 	api           *apiuniter.State
+	leaderClaimer leadership.Claimer
 	charms        map[string][]byte
 	hooks         []string
 	sch           *state.Charm
@@ -89,7 +93,9 @@ type context struct {
 	relationUnits map[string]*state.RelationUnit
 	subordinate   *state.Unit
 	ticker        *uniter.ManualTicker
+	err           string
 
+	wg             sync.WaitGroup
 	mu             sync.Mutex
 	hooksCompleted []string
 }
@@ -108,11 +114,21 @@ func (ctx *context) HookFailed(hookName string) {
 	ctx.mu.Unlock()
 }
 
+func (ctx *context) setExpectedError(err string) {
+	ctx.mu.Lock()
+	ctx.err = err
+	ctx.mu.Unlock()
+}
+
 func (ctx *context) run(c *gc.C, steps []stepper) {
 	defer func() {
 		if ctx.uniter != nil {
 			err := ctx.uniter.Stop()
-			c.Assert(err, jc.ErrorIsNil)
+			if ctx.err == "" {
+				c.Assert(err, jc.ErrorIsNil)
+			} else {
+				c.Assert(err, gc.ErrorMatches, ctx.err)
+			}
 		}
 	}()
 	for i, s := range steps {
@@ -132,43 +148,11 @@ func (ctx *context) apiLogin(c *gc.C) {
 	ctx.api, err = st.Uniter()
 	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(ctx.api, gc.NotNil)
+	ctx.leaderClaimer = ctx.st.LeadershipClaimer()
 }
 
-var goodHook = `
-#!/bin/bash --norc
-juju-log $JUJU_ENV_UUID %s $JUJU_REMOTE_UNIT
-`[1:]
-
-var badHook = `
-#!/bin/bash --norc
-juju-log $JUJU_ENV_UUID fail-%s $JUJU_REMOTE_UNIT
-exit 1
-`[1:]
-
-var rebootHook = `
-#!/bin/bash --norc
-juju-reboot
-`[1:]
-
-var badRebootHook = `
-#!/bin/bash --norc
-juju-reboot
-exit 1
-`[1:]
-
-var rebootNowHook = `
-#!/bin/bash --norc
-
-if [ -f "i_have_risen" ]
-then
-    exit 0
-fi
-touch i_have_risen
-juju-reboot --now
-`[1:]
-
 func (ctx *context) writeExplicitHook(c *gc.C, path string, contents string) {
-	err := ioutil.WriteFile(path, []byte(contents), 0755)
+	err := ioutil.WriteFile(path+cmdSuffix, []byte(contents), 0755)
 	c.Assert(err, jc.ErrorIsNil)
 }
 
@@ -200,39 +184,9 @@ metrics:
 }
 
 func (ctx *context) writeAction(c *gc.C, path, name string) {
-	var actions = map[string]string{
-		"action-log": `
-#!/bin/bash --norc
-juju-log $JUJU_ENV_UUID action-log
-`[1:],
-		"snapshot": `
-#!/bin/bash --norc
-action-set outfile.name="snapshot-01.tar" outfile.size="10.3GB"
-action-set outfile.size.magnitude="10.3" outfile.size.units="GB"
-action-set completion.status="yes" completion.time="5m"
-action-set completion="yes"
-`[1:],
-		"action-log-fail": `
-#!/bin/bash --norc
-action-fail "I'm afraid I can't let you do that, Dave."
-action-set foo="still works"
-`[1:],
-		"action-log-fail-error": `
-#!/bin/bash --norc
-action-fail too many arguments
-action-set foo="still works"
-action-fail "A real message"
-`[1:],
-		"action-reboot": `
-#!/bin/bash --norc
-juju-reboot || action-set reboot-delayed="good"
-juju-reboot --now || action-set reboot-now="good"
-`[1:],
-	}
-
 	actionPath := filepath.Join(path, "actions", name)
 	action := actions[name]
-	err := ioutil.WriteFile(actionPath, []byte(action), 0755)
+	err := ioutil.WriteFile(actionPath+cmdSuffix, []byte(action), 0755)
 	c.Assert(err, jc.ErrorIsNil)
 }
 
@@ -337,15 +291,36 @@ type createCharm struct {
 	customize func(*gc.C, *context, string)
 }
 
-var charmHooks = []string{
-	"install", "start", "config-changed", "upgrade-charm", "stop",
-	"db-relation-joined", "db-relation-changed", "db-relation-departed",
-	"db-relation-broken", "meter-status-changed", "collect-metrics",
+var (
+	baseCharmHooks = []string{
+		"install", "start", "config-changed", "upgrade-charm", "stop",
+		"db-relation-joined", "db-relation-changed", "db-relation-departed",
+		"db-relation-broken", "meter-status-changed", "collect-metrics",
+	}
+	leaderCharmHooks = []string{
+		"leader-elected", "leader-deposed", "leader-settings-changed",
+	}
+	storageCharmHooks = []string{
+		"wp-content-storage-attached", "wp-content-storage-detaching",
+	}
+)
+
+func startupHooks(minion bool) []string {
+	leaderHook := "leader-elected"
+	if minion {
+		leaderHook = "leader-settings-changed"
+	}
+	return []string{"install", leaderHook, "config-changed", "start"}
 }
 
 func (s createCharm) step(c *gc.C, ctx *context) {
 	base := testcharms.Repo.ClonedDirPath(c.MkDir(), "wordpress")
-	for _, name := range charmHooks {
+
+	allCharmHooks := baseCharmHooks
+	allCharmHooks = append(allCharmHooks, leaderCharmHooks...)
+	allCharmHooks = append(allCharmHooks, storageCharmHooks...)
+
+	for _, name := range allCharmHooks {
 		path := filepath.Join(base, "hooks", name)
 		good := true
 		for _, bad := range s.badHooks {
@@ -417,12 +392,18 @@ func (csau createServiceAndUnit) step(c *gc.C, ctx *context) {
 	ctx.apiLogin(c)
 }
 
-type createUniter struct{}
+type createUniter struct {
+	minion       bool
+	executorFunc func(*uniter.Uniter) (operation.Executor, error)
+}
 
-func (createUniter) step(c *gc.C, ctx *context) {
+func (s createUniter) step(c *gc.C, ctx *context) {
 	step(c, ctx, ensureStateWorker{})
 	step(c, ctx, createServiceAndUnit{})
-	step(c, ctx, startUniter{})
+	if s.minion {
+		step(c, ctx, forceMinion{})
+	}
+	step(c, ctx, startUniter{executorFunc: s.executorFunc})
 	step(c, ctx, waitAddresses{})
 }
 
@@ -455,7 +436,8 @@ func (waitAddresses) step(c *gc.C, ctx *context) {
 }
 
 type startUniter struct {
-	unitTag string
+	unitTag      string
+	executorFunc func(*uniter.Uniter) (operation.Executor, error)
 }
 
 func (s startUniter) step(c *gc.C, ctx *context) {
@@ -475,7 +457,7 @@ func (s startUniter) step(c *gc.C, ctx *context) {
 	locksDir := filepath.Join(ctx.dataDir, "locks")
 	lock, err := fslock.NewLock(locksDir, "uniter-hook-execution")
 	c.Assert(err, jc.ErrorIsNil)
-	ctx.uniter = uniter.NewUniter(ctx.api, tag, ctx.dataDir, lock)
+	ctx.uniter = uniter.NewUniter(ctx.api, tag, ctx.leaderClaimer, ctx.dataDir, lock, s.executorFunc)
 	uniter.SetUniterObserver(ctx.uniter, ctx)
 }
 
@@ -537,6 +519,10 @@ type stopUniter struct {
 
 func (s stopUniter) step(c *gc.C, ctx *context) {
 	u := ctx.uniter
+	if u == nil {
+		c.Logf("uniter not started, skipping stopUniter{}")
+		return
+	}
 	ctx.uniter = nil
 	err := u.Stop()
 	if s.err == "" {
@@ -554,12 +540,19 @@ func (s verifyWaiting) step(c *gc.C, ctx *context) {
 	step(c, ctx, waitHooks{})
 }
 
-type verifyRunning struct{}
+type verifyRunning struct {
+	minion bool
+}
 
 func (s verifyRunning) step(c *gc.C, ctx *context) {
 	step(c, ctx, stopUniter{})
 	step(c, ctx, startUniter{})
-	step(c, ctx, waitHooks{"config-changed"})
+	var hooks []string
+	if s.minion {
+		hooks = append(hooks, "leader-settings-changed")
+	}
+	hooks = append(hooks, "config-changed")
+	step(c, ctx, waitHooks(hooks))
 }
 
 type startupErrorWithCustomCharm struct {
@@ -574,11 +567,12 @@ func (s startupErrorWithCustomCharm) step(c *gc.C, ctx *context) {
 	})
 	step(c, ctx, serveCharm{})
 	step(c, ctx, createUniter{})
-	step(c, ctx, waitUnit{
-		status: params.StatusError,
-		info:   fmt.Sprintf(`hook failed: %q`, s.badHook),
+	step(c, ctx, waitUnitAgent{
+		statusGetter: unitStatusGetter,
+		status:       params.StatusError,
+		info:         fmt.Sprintf(`hook failed: %q`, s.badHook),
 	})
-	for _, hook := range []string{"install", "config-changed", "start"} {
+	for _, hook := range startupHooks(false) {
 		if hook == s.badHook {
 			step(c, ctx, waitHooks{"fail-" + hook})
 			break
@@ -596,11 +590,12 @@ func (s startupError) step(c *gc.C, ctx *context) {
 	step(c, ctx, createCharm{badHooks: []string{s.badHook}})
 	step(c, ctx, serveCharm{})
 	step(c, ctx, createUniter{})
-	step(c, ctx, waitUnit{
-		status: params.StatusError,
-		info:   fmt.Sprintf(`hook failed: %q`, s.badHook),
+	step(c, ctx, waitUnitAgent{
+		statusGetter: unitStatusGetter,
+		status:       params.StatusError,
+		info:         fmt.Sprintf(`hook failed: %q`, s.badHook),
 	})
-	for _, hook := range []string{"install", "config-changed", "start"} {
+	for _, hook := range startupHooks(false) {
 		if hook == s.badHook {
 			step(c, ctx, waitHooks{"fail-" + hook})
 			break
@@ -610,14 +605,16 @@ func (s startupError) step(c *gc.C, ctx *context) {
 	step(c, ctx, verifyCharm{})
 }
 
-type quickStart struct{}
+type quickStart struct {
+	minion bool
+}
 
 func (s quickStart) step(c *gc.C, ctx *context) {
 	step(c, ctx, createCharm{})
 	step(c, ctx, serveCharm{})
-	step(c, ctx, createUniter{})
-	step(c, ctx, waitUnit{status: params.StatusActive})
-	step(c, ctx, waitHooks{"install", "config-changed", "start"})
+	step(c, ctx, createUniter{minion: s.minion})
+	step(c, ctx, waitUnitAgent{status: params.StatusIdle})
+	step(c, ctx, waitHooks(startupHooks(s.minion)))
 	step(c, ctx, verifyCharm{})
 }
 
@@ -639,8 +636,8 @@ func (s startupRelationError) step(c *gc.C, ctx *context) {
 	step(c, ctx, createCharm{badHooks: []string{s.badHook}})
 	step(c, ctx, serveCharm{})
 	step(c, ctx, createUniter{})
-	step(c, ctx, waitUnit{status: params.StatusActive})
-	step(c, ctx, waitHooks{"install", "config-changed", "start"})
+	step(c, ctx, waitUnitAgent{status: params.StatusIdle})
+	step(c, ctx, waitHooks(startupHooks(false)))
 	step(c, ctx, verifyCharm{})
 	step(c, ctx, addRelation{})
 	step(c, ctx, addRelationUnit{})
@@ -655,15 +652,35 @@ func (s resolveError) step(c *gc.C, ctx *context) {
 	c.Assert(err, jc.ErrorIsNil)
 }
 
-type waitUnit struct {
-	status   params.Status
-	info     string
-	data     map[string]interface{}
-	charm    int
-	resolved state.ResolvedMode
+type statusfunc func() (state.StatusInfo, error)
+
+type statusfuncGetter func(ctx *context) statusfunc
+
+var unitStatusGetter = func(ctx *context) statusfunc {
+	return func() (state.StatusInfo, error) {
+		return ctx.unit.Status()
+	}
 }
 
-func (s waitUnit) step(c *gc.C, ctx *context) {
+var agentStatusGetter = func(ctx *context) statusfunc {
+	return func() (state.StatusInfo, error) {
+		return ctx.unit.AgentStatus()
+	}
+}
+
+type waitUnitAgent struct {
+	statusGetter func(ctx *context) statusfunc
+	status       params.Status
+	info         string
+	data         map[string]interface{}
+	charm        int
+	resolved     state.ResolvedMode
+}
+
+func (s waitUnitAgent) step(c *gc.C, ctx *context) {
+	if s.statusGetter == nil {
+		s.statusGetter = agentStatusGetter
+	}
 	timeout := time.After(worstCase)
 	for {
 		ctx.s.BackingState.StartSync()
@@ -687,25 +704,25 @@ func (s waitUnit) step(c *gc.C, ctx *context) {
 				c.Logf("want unit charm %q, got %q; still waiting", curl(s.charm), got)
 				continue
 			}
-			status, info, data, err := ctx.unit.Status()
+			statusInfo, err := s.statusGetter(ctx)()
 			c.Assert(err, jc.ErrorIsNil)
-			if string(status) != string(s.status) {
-				c.Logf("want unit status %q, got %q; still waiting", s.status, status)
+			if string(statusInfo.Status) != string(s.status) {
+				c.Logf("want unit status %q, got %q; still waiting", s.status, statusInfo.Status)
 				continue
 			}
-			if info != s.info {
-				c.Logf("want unit status info %q, got %q; still waiting", s.info, info)
+			if statusInfo.Message != s.info {
+				c.Logf("want unit status info %q, got %q; still waiting", s.info, statusInfo.Message)
 				continue
 			}
 			if s.data != nil {
-				if len(data) != len(s.data) {
-					c.Logf("want %d unit status data value(s), got %d; still waiting", len(s.data), len(data))
+				if len(statusInfo.Data) != len(s.data) {
+					c.Logf("want %d status data value(s), got %d; still waiting", len(s.data), len(statusInfo.Data))
 					continue
 				}
 				for key, value := range s.data {
-					if data[key] != value {
-						c.Logf("want unit status data value %q for key %q, got %q; still waiting",
-							value, key, data[key])
+					if statusInfo.Data[key] != value {
+						c.Logf("want status data value %q for key %q, got %q; still waiting",
+							value, key, statusInfo.Data[key])
 						continue
 					}
 				}
@@ -920,19 +937,20 @@ func (s startUpgradeError) step(c *gc.C, ctx *context) {
 		},
 		serveCharm{},
 		createUniter{},
-		waitUnit{
-			status: params.StatusActive,
+		waitUnitAgent{
+			status: params.StatusIdle,
 		},
-		waitHooks{"install", "config-changed", "start"},
+		waitHooks(startupHooks(false)),
 		verifyCharm{},
 
 		createCharm{revision: 1},
 		serveCharm{},
 		upgradeCharm{revision: 1},
-		waitUnit{
-			status: params.StatusError,
-			info:   "upgrade failed",
-			charm:  1,
+		waitUnitAgent{
+			statusGetter: unitStatusGetter,
+			status:       params.StatusError,
+			info:         "upgrade failed",
+			charm:        1,
 		},
 		verifyWaiting{},
 		verifyCharm{attemptedRevision: 1},
@@ -948,10 +966,11 @@ type verifyWaitingUpgradeError struct {
 
 func (s verifyWaitingUpgradeError) step(c *gc.C, ctx *context) {
 	verifyCharmSteps := []stepper{
-		waitUnit{
-			status: params.StatusError,
-			info:   "upgrade failed",
-			charm:  s.revision,
+		waitUnitAgent{
+			statusGetter: unitStatusGetter,
+			status:       params.StatusError,
+			info:         "upgrade failed",
+			charm:        s.revision,
 		},
 		verifyCharm{attemptedRevision: s.revision},
 	}
@@ -962,7 +981,7 @@ func (s verifyWaitingUpgradeError) step(c *gc.C, ctx *context) {
 			// to reset the error status, we can avoid a race in which a subsequent
 			// fixUpgradeError lands just before the restarting uniter retries the
 			// upgrade; and thus puts us in an unexpected state for future steps.
-			ctx.unit.SetStatus(state.StatusActive, "", nil)
+			ctx.unit.SetAgentStatus(state.StatusActive, "", nil)
 		}},
 		startUniter{},
 	}
@@ -1230,7 +1249,7 @@ func curl(revision int) *corecharm.URL {
 }
 
 func appendHook(c *gc.C, charm, name, data string) {
-	path := filepath.Join(charm, "hooks", name)
+	path := filepath.Join(charm, "hooks", name+cmdSuffix)
 	f, err := os.OpenFile(path, os.O_WRONLY|os.O_APPEND, 0755)
 	c.Assert(err, jc.ErrorIsNil)
 	defer f.Close()
@@ -1270,7 +1289,7 @@ func renameRelation(c *gc.C, charmPath, oldName, newName string) {
 	c.Assert(err, jc.ErrorIsNil)
 }
 
-func createHookLock(c *gc.C, dataDir string) *fslock.Lock {
+func createMachineLock(c *gc.C, dataDir string) *fslock.Lock {
 	lockDir := filepath.Join(dataDir, "locks")
 	lock, err := fslock.NewLock(lockDir, "uniter-hook-execution")
 	c.Assert(err, jc.ErrorIsNil)
@@ -1282,26 +1301,26 @@ type acquireHookSyncLock struct {
 }
 
 func (s acquireHookSyncLock) step(c *gc.C, ctx *context) {
-	lock := createHookLock(c, ctx.dataDir)
+	lock := createMachineLock(c, ctx.dataDir)
 	c.Assert(lock.IsLocked(), jc.IsFalse)
 	err := lock.Lock(s.message)
 	c.Assert(err, jc.ErrorIsNil)
 }
 
 var releaseHookSyncLock = custom{func(c *gc.C, ctx *context) {
-	lock := createHookLock(c, ctx.dataDir)
+	lock := createMachineLock(c, ctx.dataDir)
 	// Force the release.
 	err := lock.BreakLock()
 	c.Assert(err, jc.ErrorIsNil)
 }}
 
 var verifyHookSyncLockUnlocked = custom{func(c *gc.C, ctx *context) {
-	lock := createHookLock(c, ctx.dataDir)
+	lock := createMachineLock(c, ctx.dataDir)
 	c.Assert(lock.IsLocked(), jc.IsFalse)
 }}
 
 var verifyHookSyncLockLocked = custom{func(c *gc.C, ctx *context) {
-	lock := createHookLock(c, ctx.dataDir)
+	lock := createMachineLock(c, ctx.dataDir)
 	c.Assert(lock.IsLocked(), jc.IsTrue)
 }}
 
@@ -1360,11 +1379,18 @@ func (cmds asyncRunCommands) step(c *gc.C, ctx *context) {
 		RemoteUnitName: "",
 	}
 
-	socketPath := filepath.Join(ctx.path, "run.socket")
+	var socketPath string
+	if runtime.GOOS == "windows" {
+		socketPath = `\\.\pipe\unit-u-0-run`
+	} else {
+		socketPath = filepath.Join(ctx.path, "run.socket")
+	}
 
+	ctx.wg.Add(1)
 	go func() {
+		defer ctx.wg.Done()
 		// make sure the socket exists
-		client, err := rpc.Dial("unix", socketPath)
+		client, err := sockets.Dial(socketPath)
 		c.Assert(err, jc.ErrorIsNil)
 		defer client.Close()
 
@@ -1377,6 +1403,76 @@ func (cmds asyncRunCommands) step(c *gc.C, ctx *context) {
 	}()
 }
 
+type waitContextWaitGroup struct{}
+
+func (waitContextWaitGroup) step(c *gc.C, ctx *context) {
+	ctx.wg.Wait()
+}
+
+const otherLeader = "some-other-unit/123"
+
+type forceMinion struct{}
+
+func (forceMinion) step(c *gc.C, ctx *context) {
+	// TODO(fwereade): this is designed to work when the uniter is still running,
+	// which is... unexpected, because the uniter's running a tracker that will
+	// do its best to maintain leadership. But... it's possible for us to make it
+	// resign by going via the lease manager directly, and we can be confident
+	// that the uniter's tracker *will* see the problem when it fails to renew.
+	// This lets us test the uniter's behaviour under bizarre/adverse conditions
+	// (in addition to working just fine when the uniter's not running).
+	for i := 0; i < 3; i++ {
+		c.Logf("deposing local unit (attempt %d)", i)
+		leaseClock.Advance(61 * time.Second)
+		time.Sleep(coretesting.ShortWait)
+		c.Logf("promoting other unit (attempt %d)", i)
+		err := ctx.leaderClaimer.ClaimLeadership(ctx.svc.Name(), otherLeader, time.Minute)
+		if err == nil {
+			ctx.s.BackingState.StartSync()
+			return
+		} else if errors.Cause(err) != leadership.ErrClaimDenied {
+			c.Assert(err, jc.ErrorIsNil)
+		}
+	}
+	c.Fatalf("failed to promote a different leader")
+}
+
+type forceLeader struct{}
+
+func (forceLeader) step(c *gc.C, ctx *context) {
+	c.Logf("deposing other unit")
+	leaseClock.Advance(61 * time.Second)
+	time.Sleep(coretesting.ShortWait)
+	c.Logf("promoting local unit")
+	err := ctx.leaderClaimer.ClaimLeadership(ctx.svc.Name(), ctx.unit.Name(), time.Minute)
+	c.Assert(err, jc.ErrorIsNil)
+	ctx.s.BackingState.StartSync()
+}
+
+type setLeaderSettings map[string]string
+
+func (s setLeaderSettings) step(c *gc.C, ctx *context) {
+	// We do this directly on State, not the API, so we don't have to worry
+	// about getting an API conn for whatever unit's meant to be leader.
+	err := ctx.svc.UpdateLeaderSettings(successToken{}, s)
+	c.Assert(err, jc.ErrorIsNil)
+	ctx.s.BackingState.StartSync()
+}
+
+type successToken struct{}
+
+func (successToken) Check(interface{}) error {
+	return nil
+}
+
+type verifyLeaderSettings map[string]string
+
+func (verify verifyLeaderSettings) step(c *gc.C, ctx *context) {
+	actual, err := ctx.svc.LeaderSettings()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(actual, jc.DeepEquals, map[string]string(verify))
+}
+
 type verifyFile struct {
 	filename string
 	content  string
@@ -1507,10 +1603,10 @@ func (s startGitUpgradeError) step(c *gc.C, ctx *context) {
 		},
 		serveCharm{},
 		createUniter{},
-		waitUnit{
-			status: params.StatusActive,
+		waitUnitAgent{
+			status: params.StatusIdle,
 		},
-		waitHooks{"install", "config-changed", "start"},
+		waitHooks(startupHooks(false)),
 		verifyGitCharm{dirty: true},
 
 		createCharm{
@@ -1522,10 +1618,11 @@ func (s startGitUpgradeError) step(c *gc.C, ctx *context) {
 		},
 		serveCharm{},
 		upgradeCharm{revision: 1},
-		waitUnit{
-			status: params.StatusError,
-			info:   "upgrade failed",
-			charm:  1,
+		waitUnitAgent{
+			statusGetter: unitStatusGetter,
+			status:       params.StatusError,
+			info:         "upgrade failed",
+			charm:        1,
 		},
 		verifyWaiting{},
 		verifyGitCharm{dirty: true},
@@ -1534,3 +1631,63 @@ func (s startGitUpgradeError) step(c *gc.C, ctx *context) {
 		step(c, ctx, s_)
 	}
 }
+
+type provisionStorage struct{}
+
+func (s provisionStorage) step(c *gc.C, ctx *context) {
+	storageAttachments, err := ctx.st.UnitStorageAttachments(ctx.unit.UnitTag())
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(storageAttachments, gc.HasLen, 1)
+
+	filesystem, err := ctx.st.StorageInstanceFilesystem(storageAttachments[0].StorageInstance())
+	c.Assert(err, jc.ErrorIsNil)
+
+	filesystemInfo := state.FilesystemInfo{
+		Size:         1024,
+		FilesystemId: "fs-id",
+	}
+	err = ctx.st.SetFilesystemInfo(filesystem.FilesystemTag(), filesystemInfo)
+	c.Assert(err, jc.ErrorIsNil)
+
+	machineId, err := ctx.unit.AssignedMachineId()
+	c.Assert(err, jc.ErrorIsNil)
+
+	filesystemAttachmentInfo := state.FilesystemAttachmentInfo{
+		MountPoint: "/srv/wordpress/content",
+	}
+	err = ctx.st.SetFilesystemAttachmentInfo(
+		names.NewMachineTag(machineId),
+		filesystem.FilesystemTag(),
+		filesystemAttachmentInfo,
+	)
+	c.Assert(err, jc.ErrorIsNil)
+}
+
+type destroyStorageAttachment struct{}
+
+func (s destroyStorageAttachment) step(c *gc.C, ctx *context) {
+	storageAttachments, err := ctx.st.UnitStorageAttachments(ctx.unit.UnitTag())
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(storageAttachments, gc.HasLen, 1)
+	err = ctx.st.DestroyStorageAttachment(
+		storageAttachments[0].StorageInstance(),
+		ctx.unit.UnitTag(),
+	)
+	c.Assert(err, jc.ErrorIsNil)
+}
+
+type verifyStorageDetached struct{}
+
+func (s verifyStorageDetached) step(c *gc.C, ctx *context) {
+	storageAttachments, err := ctx.st.UnitStorageAttachments(ctx.unit.UnitTag())
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(storageAttachments, gc.HasLen, 0)
+}
+
+type expectError struct {
+	err string
+}
+
+func (s expectError) step(c *gc.C, ctx *context) {
+	ctx.setExpectedError(s.err)
+}
