@@ -4,7 +4,8 @@
 package names
 
 import (
-	gc "launchpad.net/gocheck"
+	"github.com/juju/utils"
+	gc "gopkg.in/check.v1"
 )
 
 var tagEqualityTests = []struct {
@@ -14,12 +15,14 @@ var tagEqualityTests = []struct {
 	{NewMachineTag("0"), MachineTag{id: "0"}},
 	{NewMachineTag("10/lxc/1"), MachineTag{id: "10-lxc-1"}},
 	{NewUnitTag("mysql/1"), UnitTag{name: "mysql-1"}},
-	{NewServiceTag("ceph"), ServiceTag{name: "ceph"}},
+	{NewServiceTag("ceph"), ServiceTag{Name: "ceph"}},
 	{NewRelationTag("wordpress:haproxy"), RelationTag{key: "wordpress.haproxy"}},
-	{NewEnvironTag("local"), EnvironTag{uuid: "local"}},
+	{NewEnvironTag("deadbeef-0123-4567-89ab-feedfacebeef"), EnvironTag{uuid: "deadbeef-0123-4567-89ab-feedfacebeef"}},
 	{NewUserTag("admin"), UserTag{name: "admin"}},
+	{NewUserTag("admin@local"), UserTag{name: "admin", provider: "local"}},
+	{NewUserTag("admin@foobar"), UserTag{name: "admin", provider: "foobar"}},
 	{NewNetworkTag("eth0"), NetworkTag{name: "eth0"}},
-	{NewActionTag("foo" + ActionMarker + "321"), ActionTag{id: "foo" + ActionMarker + "321"}},
+	{NewActionTag("01234567-aaaa-4bbb-8ccc-012345678901"), ActionTag{ID: stringToUUID("01234567-aaaa-4bbb-8ccc-012345678901")}},
 }
 
 type equalitySuite struct{}
@@ -31,3 +34,11 @@ func (s *equalitySuite) TestTagEquality(c *gc.C) {
 		c.Check(tt.want, gc.Equals, tt.expected)
 	}
 }
+
+func stringToUUID(id string) utils.UUID {
+	uuid, err := utils.UUIDFromString(id)
+	if err != nil {
+		panic(err)
+	}
+	return uuid
+}
